package cn.redcdn.buteldataadapter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import cn.redcdn.log.CustomLog;

public class DataAdapter extends BaseAdapter {
	protected final String TAG = DataAdapter.class.getName();
	protected LayoutInflater mInflater;

	protected Map<String, DataExpand> mExpandMap;

	private List<DataSet> mDataSetList;

	public DataAdapter(Context context) {
		mInflater = LayoutInflater.from(context);
		mDataSetList = new ArrayList<DataSet>();
		mExpandMap = new HashMap<String, DataExpand>();
	}

	/**
	 * 添加数据集合，不会触发UI刷新。 先添加的数据集合在显示顺序上靠前
	 * 
	 * @param set
	 * @return 添加后的index
	 */
	public int addDataSet(DataSet set) {
		return addDataSet(mDataSetList.size(), set);
	}

	/**
	 * 添加数据集合，不会触发UI刷新。 先添加的数据集合在显示顺序上靠前
	 * 
	 * @param index
	 *            插入位置
	 * @param set
	 * @return 添加后的index -1: 添加出错
	 */
	public int addDataSet(int index, DataSet set) {
		if (index < 0 || index > mDataSetList.size()) {
			CustomLog.e(TAG, "addDataSet: Illegal params, return -1 !");
			return -1;
		}
		if (!mDataSetList.contains(set)) {
			mDataSetList.add(index, set);
		}
		return index;
	}

	/**
	 * 移除数据集合，不触发UI刷新
	 * 
	 * @param index
	 * @return -1： 移除出错 0： 移除成功
	 */
	public int removeDataSet(int index) {
		if (index < 0 || index > mDataSetList.size()) {
			CustomLog.e(TAG, "addDataSet: Illegal params, return -1 !");
			return -1;
		}
		mDataSetList.get(index).release();
		mDataSetList.remove(index);
		return 0;
	}

	/**
	 * 添加扩展属性集合，用于配合 getView 显示做差异化
	 * 
	 * @param expandMap
	 */
	public void addExpandMap(Map<String, DataExpand> expandMap) {
		mExpandMap = expandMap;
	}

	/**
	 * 更新数据集合，将触发UI刷新
	 * 
	 * @param index
	 *            待更新的数据集合处于数据集合列表中的索引位置，该位置同时对应于显示顺序
	 * @param set
	 */
	public void updateDataSet(int index, DataSet set) {
		if (index < 0 || index >= mDataSetList.size()) {
			CustomLog.e(TAG, "updateDataSet: Illegal params, return null !");
			return;
		}
		mDataSetList.get(index).release();

		mDataSetList.remove(index);
		mDataSetList.add(index, set);
		notifyDataSetChanged();
	}

	/**
	 * 更新扩展属性集合，用于配合 getView 显示做差异化，将触发UI刷新
	 * 
	 * @param expandMap
	 */
	public void updateExpandMap(Map<String, DataExpand> expandMap) {
		mExpandMap = expandMap;
		notifyDataSetChanged();
	}
public void updateExpandMap(int position)
{
  notifyDataSetChanged();
  }
	/**
	 * 销毁Adapter 内部数据
	 */
	public void release() {
		for (int i = 0; i < mDataSetList.size(); i++) {
			mDataSetList.get(i).release();
		}
	}

	public int getDataSetSize() {
		return mDataSetList.size();
	}

	@Override
	public void notifyDataSetChanged() {
		super.notifyDataSetChanged();
	}

	@Override
	public int getCount() {
		int count = 0;

		for (int i = 0; i < mDataSetList.size(); i++) {
			count += mDataSetList.get(i).getCount();
		}

		return count;
	}

	@Override
	public Object getItem(int index) {
		if (index < 0 || index >= getCount()) {
			CustomLog.e(TAG, "getItem(): Illegal params, return null !");
			return null;
		}
		int count = 0;
		for (int i = 0; i < mDataSetList.size(); i++) {
			if (index < mDataSetList.get(i).getCount() + count) {
				return mDataSetList.get(i).getItem(index - count);
			} else {
				count += mDataSetList.get(i).getCount();
				continue;
			}
		}
		return null;
	}

	@Override
	public long getItemId(int arg0) {
		return arg0;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {

		return convertView;
	}
}
