package cn.redcdn.butelopensdk;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKOperationCode.CurrentRoleStatus;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKOperationCode.LiveStatus;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKOperationCode.MeetingStyleStatus;
import cn.redcdn.butelopensdk.constconfig.MediaType;
import cn.redcdn.butelopensdk.constconfig.SpeakerInfo;
import cn.redcdn.butelopensdk.constconfig.StreamInfo;
import cn.redcdn.log.CustomLog;

public class MeetingInfo {
	private final String TAG = MeetingInfo.class.getName();

	private boolean speakStatus = false;

	/** 会议id */
	public int meetingId = 0;

	/** 会议主持人id */
	public String adminId = "";

	/** 自己的视讯号id */
	public String accountId = null;

	/** 自己的参会名称 */
	public String accountName = null;

	/** 会议加锁信息：1代表加锁，0代表无锁 */
	public int intentLockInfo = 0;

	/** 主讲状态：1代表主讲，2代表普通模式 */
	public int isSpeaker = SpeakerType.NORMAL;// 是否是主讲，默认是普通模式

	/** 加入会议是否自动发言: true 表示自动发言；false表示不自动发言 */
	public boolean isAutoSpeak = true; // 加入会议是否自动发言

	/*** 保存SpeakerInfo 列表 ***/
	private List<SpeakerInfo> mSpeakerInfoList = null;

	/****** 主持人功能相关信息 *****/
	public int mMeetingStyle = MeetingStyleStatus.FREE_MODE;// 0是指定发言模式，1是自由发言模式，默认是自由发言模式

	public String mMeetingMasterName = "";

	public String mMeetingMasterAccountId = "";

	public int mCurrentRole = CurrentRoleStatus.COMMON_ROLE;// 2表示主持人，1表示普通参会方

	public int mLiveStatus = LiveStatus.NO_START_LIVING;// 1代表直播会议，2代表普通会议

	/******** 当前会议发言者信息 ********/
	public String shareDocName = "";

	/******** 是否在处理981 ********/
	public boolean isDealWith981 = false;

	public int exceptionRetryTime = 0;

	// public int lockInfo = 0;
	public boolean isIniting = true;
	private int mLoundspeakerStatus = LOUNDSPEAKER_OPEN; // 扬声器状态

	public final static int LOUNDSPEAKER_OPEN = 1;
	public final static int LOUNDSPEAKER_CLOSE = 2;

	public int speakersNum = 4;

	public int meetingChatState = 1; // 1代表开启，0代表关闭

	public class MemberStatus {
		public static final int IS_SPEAKING = 1;

		public static final int IS_LISTENING = 4;
	}

	public class CamStatus {
		public static final int OPEN = 1;

		public static final int CLOSE = 2;
	}

	public class SpeakerType {
		public static final int SPEAKER = 1;

		public static final int NORMAL = 2;
	}

	public class ScreenShareInfo {
		public static final int SHAREING = 1;

		public static final int NOT_SHAREING = 2;
	}

	public MeetingInfo() {
		mSpeakerInfoList = new ArrayList<SpeakerInfo>();
	}

	public int addSpeaker(SpeakerInfo speakerInfo) {
		if (mSpeakerInfoList.contains(speakerInfo)) {
			CustomLog.e(TAG, "MeetingInfo::addSpeaker() 已经包含改speaker");
			mSpeakerInfoList.remove(speakerInfo);
		}
		mSpeakerInfoList.add(speakerInfo);
		return 0;
	}

	public int removeSpeaker(String accountId) {
		SpeakerInfo info = getSpeakerByAccountId(accountId);
		if (info != null) {
			mSpeakerInfoList.remove(info);
			return 0;
		}
		return -1;
	}

	public SpeakerInfo getSpeakerByAccountId(String accountId) {
		for (int i = 0; i < mSpeakerInfoList.size(); i++) {
			if (mSpeakerInfoList.get(i).getAccountId().equals(accountId)) {
				return mSpeakerInfoList.get(i);
			}
		}
		return null;
	}

	public List<SpeakerInfo> getSpeakers() {
		return mSpeakerInfoList;
	}

	public List<String> getSpeakersAccountIdList() {
		List<String> list = new ArrayList<String>();
		for (int i = 0; i < mSpeakerInfoList.size(); i++) {
			list.add(mSpeakerInfoList.get(i).getAccountId());
		}
		return list;
	}

	/**
	 * 转换成STP需要的流描述信息：用一个字节表示，高4位置0，低4位表示四种流类型是否打开的状态，1为打开，0为关闭 </p> 如果未发言，该信息为
	 * 0 </p> 1. 表示视频流 2. 表示音频流 3. 表示数据视频流 4. 表示数据音频流 该顺序表示从低位到高位的排列顺序
	 * 
	 * @param accountId
	 *            发言者视讯号
	 * @return
	 */
	public int getStreamDesp(String accountId) {
		int result = 0;
		SpeakerInfo speaker = getSpeakerByAccountId(accountId);
		if (speaker != null) {
			Map<Integer, StreamInfo> streamsInfo = speaker.getmStreamsInfo();
			if (streamsInfo != null) {
				if (streamsInfo.containsKey(MediaType.TYPE_VIDEO)) {
					result += 1; // 视频流：0b1
				}

				if (streamsInfo.containsKey(MediaType.TYPE_AUDIO)) {
					result += 2; // 音频流：0b10
				}

				if (streamsInfo.containsKey(MediaType.TYPE_DOC_VIDEO)) {
					result += 4; // 文档视频流：0b100
				}

				if (streamsInfo.containsKey(MediaType.TYPE_DOC_AUDIO)) {
					result += 8; // 文档音频流：0b1000
				}
			}
		}
		return result;
	}

	public void LogSpeakersInfo() {
		CustomLog.d(TAG, "Speaks count: " + mSpeakerInfoList.size());
		for (int i = 0; i < mSpeakerInfoList.size(); i++) {
			SpeakerInfo speaker = mSpeakerInfoList.get(i);
			CustomLog.d(TAG,
					"index :" + i + " |accountId: " + speaker.getAccountId()
							+ " |accountName: " + speaker.getAccountName());
		}
	}

	public void setLoudspeakerStatus(int loudspeakerStatus) {
		CustomLog.d(TAG, "MeetingInfo::setLoudspeakerStatus()"
				+ loudspeakerStatus);
		this.mLoundspeakerStatus = loudspeakerStatus;
	}

	public int getLoudspeakerStatus() {
		return mLoundspeakerStatus;
	}

	public void setSpeakStatus(boolean speakStatus) {
		this.speakStatus = speakStatus;
	}

	public boolean getSpeakStatus() {
		return this.speakStatus;
	}

}
