package cn.redcdn.butelopensdk.constconfig;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import android.annotation.SuppressLint;
import cn.redcdn.butelopensdk.vo.VideoParameter;
import cn.redcdn.log.CustomLog;

public class SpeakerInfo {
	private final static String TAG = SpeakerInfo.class.getName().toString();

	private String accountId;// 视讯号

	private String accountName;// 名称

	private int audioResourceId = -1;// 音频resourceId

	private int mVolumeLevel = 0;// 音量等级

	private int camStatus = CAM_STATUS_CLOSE;// 摄像头打开状态，1打开，2关闭. 默认是关闭状态
	private int micStatus = MIC_STATUS_OFF;// 音频设备打开状态，1打开，2关闭. 默认是关闭状态
	private int speakerType = SPEAKER_TYPE_NORMAL;// 主讲状态1，普通状态2. 默认是普通状态
	private int mSyncId = -1; // 理论上一个发言者所有的流,其对应的syncId 都是一样的。
	private int screenShareStatus = SCREEN_NORMAL;// 屏幕分享状态，1屏幕分享者，2普通发言者
	private int mVideoStatus = VIDEO_OPEN; // 视频画面状态
	private int mVideoOffReason = VIDEO_OFF_REASON_NULL;// 视频画面关闭原因
	private int mDocVideoStatus = DOC_VIDEO_OPEN; // 文档画面状态
	private int mLoudspeakerStatus = LOUDSPEAKER_OPEN; // 扬声器状态

	private VideoParameter mVideoParam; // 视频采集参数
	private VideoParameter mDocVideoParam; // 文档视频采集参数

	// private int shareType;// 共享类型

	public final static int CAM_STATUS_OPEN = 1;
	public final static int CAM_STATUS_CLOSE = 2;

	public final static int MIC_STATUS_ON = 1;
	public final static int MIC_STATUS_OFF = 2;

	public final static int SCREEN_SHARING = 1;
	public final static int SCREEN_NORMAL = 2;

	public final static int SPEAKER_TYPE_MAIN = 1;
	public final static int SPEAKER_TYPE_NORMAL = 2;

	public final static int LOUDSPEAKER_OPEN = 1;
	public final static int LOUDSPEAKER_CLOSE = 2;

	/**
	 * 视频画面打开
	 */
	public final static int VIDEO_OPEN = 1;
	/**
	 * 视频画面关闭
	 */
	public final static int VIDEO_OFF = 2;

	/**
	 * 视频画面关闭原因：默认状态
	 */
	public final static int VIDEO_OFF_REASON_NULL = -1;

	/**
	 * 视频画面关闭原因：自动关闭；当系统判断视频画面分辨率过高时，默认进行关闭
	 */
	public final static int VIDEO_OFF_REASON_AUTO = 1;

	/**
	 * 视频画面关闭原因：手动关闭；用户手动操作关闭
	 */
	public final static int VIDEO_OFF_REASON_MANUAL = 2;

	/**
	 * 文档画面打开
	 */
	public final static int DOC_VIDEO_OPEN = 1;
	/**
	 * 文档画面关闭
	 */
	public final static int DOC_VIDEO_OFF = 2;

	// 使用resourceId作为key，值是一个对象包含resourcesId、streamId、syncId、mediaType
	private Map<Integer, StreamInfo> mStreamsInfo;

	@Override
	public String toString() {
		return "SpeakerInfo [accountId=" + accountId + ", accountName="
				+ accountName + ", camStatus=" + camStatus + ", micStatus="
				+ micStatus + ", speakerType=" + speakerType
				+ ", screenShareStatus=" + screenShareStatus
				+ ", mStreamsInfo=" + mStreamsInfo + "]";
	}

	public SpeakerInfo() {
		this("", "");

	}

	@SuppressLint("UseSparseArrays")
	public SpeakerInfo(String accountId, String accountName) {
//		CustomLog.d(TAG, "SpeakerInfo::SpeakerInfo() 构造SpeakerInfo");
		this.accountId = accountId;
		this.accountName = accountName;
		mStreamsInfo = new HashMap<Integer, StreamInfo>();
		mVideoParam = new VideoParameter();
		mDocVideoParam = new VideoParameter();
	}

	public String getAccountId() {
		return accountId;
	}

	public void setAccountId(String accountId) {
		CustomLog.d(TAG, "SpeakerInfo::setAccountId() " + accountId);
		this.accountId = accountId;
	}

	public String getAccountName() {
		return accountName;
	}

	public void setAccountName(String accountName) {
		CustomLog.d(TAG, "SpeakerInfo::setAccountName() " + accountName);
		this.accountName = accountName;
	}

	public void setCamStatus(int camStatus) {
		CustomLog.d(TAG, "SpeakerInfo::setCamStatus() " + camStatus);
		this.camStatus = camStatus;
	}

	public void setVideoParam(VideoParameter param) {
		CustomLog.d(TAG, "SpeakerInfo::setVideoParam() param");
		if (param == null) {
			CustomLog.e(TAG,
					"SpeakerInfo::setVideoParam() param error! 不能为null");
			return;
		}
		mVideoParam = param;
		mVideoParam.logSelf();
	}

	public VideoParameter getVideoParam() {
		return mVideoParam;
	}

	public int setDocVideoParam(VideoParameter param) {
		CustomLog.d(TAG, "SpeakerInfo::setDocVideoParam() param");
		if (param == null) {
			CustomLog.e(TAG,
					"SpeakerInfo::setDocVideoParam() param error! 不能为null");
			return -1;
		}

		if (mDocVideoParam.getCapWidth() == param.getCapWidth()
				&& mDocVideoParam.getCapHeight() == param.getCapHeight()) {
			mDocVideoParam = param;
			mDocVideoParam.logSelf();
			return 0;
		} else {
			mDocVideoParam = param;
			mDocVideoParam.logSelf();
			return 1;
		}

	}

	public VideoParameter getDocVideoParam() {
		return mDocVideoParam;
	}

	/**
	 * 获取摄像头状态
	 * 
	 * @return CAM_STATUS_OPEN：打开
	 *         <p>
	 *         CAM_STATUS_CLOSE: 关闭
	 */
	public int getCamStatus() {
		return camStatus;
	}

	public void setMICStatus(int micStatus) {
		CustomLog.d(TAG, "SpeakerInfo::setMICStatus() " + micStatus);
		this.micStatus = micStatus;
	}

	/**
	 * 获取Mic状态
	 * 
	 * @return MIC_STATUS_ON: 打开
	 *         <p>
	 *         MIC_STATUS_OFF: 关闭
	 */
	public int getMICStatus() {
		return micStatus;
	}

	public void setVideoStatus(int status, int reason) {
		CustomLog.d(TAG, "SpeakerInfo::setVideoStatus() status: " + status
				+ " |reason: " + reason);
		mVideoStatus = status;

		if (mVideoStatus == VIDEO_OPEN) {
			mVideoOffReason = VIDEO_OFF_REASON_NULL;
		} else if (mVideoStatus == VIDEO_OFF) {
			mVideoOffReason = reason;
		}
	}

	/**
	 * 获取视频画面关闭原因
	 * 
	 * @return VIDEO_OFF_REASON_NULL = -1 默认状态;
	 *         <p>
	 *         VIDEO_OFF_REASON_AUTO = 1 自动关闭;
	 *         <p>
	 *         VIDEO_OFF_REASON_MANUAL = 2 手动关闭
	 */
	public int getVideoOffReason() {
		CustomLog.i(TAG, "SpeakerInfo::getVideoOffReason() 获取视频画面关闭原因： "
				+ mVideoOffReason);
		return mVideoOffReason;
	}

	/**
	 * 获取视频画面状态
	 * 
	 * @return VIDEO_OPEN: 打开
	 *         <p>
	 *         VIDEO_OFF: 关闭
	 */
	public int getVideoStatus() {
		return mVideoStatus;
	}

	public void setDocVideoStatus(int status) {
		CustomLog.d(TAG, "SpeakerInfo::setDocVideoStatus() status: " + status);
		mDocVideoStatus = status;
	}

	/**
	 * 获取文档画面状态
	 * 
	 * @return DOC_VIDEO_OPEN: 打开
	 *         <p>
	 *         DOC_VIDEO_OFF: 关闭
	 */
	public int getDocVideoStatus() {
		return mDocVideoStatus;
	}

	/**
	 * 获取扬声器状态
	 * 
	 * @return
	 */
	public int getLoudspeakerStatus() {
		return mLoudspeakerStatus;
	}

	/**
	 * 设置扬声器状态
	 * 
	 * @param status
	 */
	public void setLoudspeakerStatus(int status) {
		CustomLog.d(TAG, "SpeakerInfo::setLoudspeakerStatus() status: "
				+ status);
		mLoudspeakerStatus = status;
	}

	/**
	 * 获取视频画面分辨率
	 * 
	 * @return {width,height}
	 */
	public int[] getVideoResolution() {
		int[] videoResolution = new int[2];
		videoResolution[0] = mVideoParam.getCapWidth();
		videoResolution[1] = mVideoParam.getCapHeight();
		CustomLog.d(TAG, "SpeakerInfo::getVideoResolution() width: "
				+ videoResolution[0] + " | height: " + videoResolution[1]);
		return videoResolution;
	}

	/**
	 * 获取文档画面分辨率
	 * 
	 * @return {width,height}
	 */
	public int[] getDocVideoResolution() {
		int[] docVideoResolution = new int[2];
		docVideoResolution[0] = mDocVideoParam.getCapWidth();
		docVideoResolution[1] = mDocVideoParam.getCapHeight();
		CustomLog
				.d(TAG, "SpeakerInfo::getDocVideoResolution() width: "
						+ docVideoResolution[0] + " | height: "
						+ docVideoResolution[1]);
		return docVideoResolution;
	}

	public int getSpeakerType() {
		return speakerType;
	}

	public void setSpeakerType(int speakerType) {
		CustomLog.d(TAG, "SpeakerInfo::setSpeakerType() " + speakerType);
		this.speakerType = speakerType;
	}

	public Map<Integer, StreamInfo> getmStreamsInfo() {
		return mStreamsInfo;
	}

	public StreamInfo getStreamInfo(int resourceId) {
		return mStreamsInfo.get(resourceId);
	}

	public void addStreamInfo(int resourceId, StreamInfo streamInfo) {
		CustomLog.d(TAG, "addStreamInfo() resourceId: " + resourceId
				+ " | streamInfo : " + streamInfo.toString());
		mStreamsInfo.put(resourceId, streamInfo);
	}

	public StreamInfo removeStreamInfo(int resourceId) {
		CustomLog.d(TAG, "removeStreamInfo() resourceId: " + resourceId);
		return mStreamsInfo.remove(resourceId);
	}

	/**
	 * 清除指定媒体类型的流信息（一种类型可能对应好几条流信息）
	 * 
	 * @param mediaType
	 * @return
	 */
	public int removeStreamInfoByMediaType(int mediaType) {
		CustomLog.d(TAG, "removeStreamInfoByMediaType() mediaType: "
				+ mediaType);
		List<Integer> removeResList = new ArrayList<Integer>();
		Iterator<Entry<Integer, StreamInfo>> it = getmStreamsInfo().entrySet()
				.iterator();
		while (it.hasNext()) {
			Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
					.next();
			int key = (Integer) entry.getKey();
			StreamInfo val = (StreamInfo) entry.getValue();
			CustomLog.i(TAG, "key=" + key + "val=" + val.toString());
			if (val.getMediaType() == mediaType) {
				removeResList.add(key);
			}
		}

		for (int res : removeResList) {
			removeStreamInfo(res);
		}

		return 0;
	}

	public int getStreamIdByResource(int resourceId) {
		StreamInfo stream = mStreamsInfo.get(resourceId);
		if (stream != null) {
			int streamId = stream.getStreamId();
			CustomLog.d(TAG, "getStreamIdByResource() streamId: " + streamId
					+ " | resourceId: " + resourceId);
			return streamId;
		}
		CustomLog.e(TAG, "getStreamIdByResource() 无对应streamId, resourceId: "
				+ resourceId);
		return -1;
	}

	public synchronized int getSreamIdByMediaType(int mediaType) {
		Iterator<Entry<Integer, StreamInfo>> it = getmStreamsInfo().entrySet()
				.iterator();
		while (it.hasNext()) {
			Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
					.next();
			StreamInfo val = (StreamInfo) entry.getValue();
			if (val.getMediaType() == mediaType) {
				int streamId = val.getStreamId();
//				CustomLog.d(TAG, "getSreamIdByMediaType() streamId: "
//						+ streamId + " | mediaType: " + mediaType);
				return streamId;
			}
		}

//		CustomLog.e(TAG, "getSreamIdByMediaType() 无对应streamId, mediaType: "
//				+ mediaType);
		return -1;
	}

	public synchronized StreamInfo getStreamInfoByMediaType(int mediaType) {
		Iterator<Entry<Integer, StreamInfo>> it = getmStreamsInfo().entrySet()
				.iterator();
		while (it.hasNext()) {
			Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
					.next();
			StreamInfo val = (StreamInfo) entry.getValue();
			if (val.getMediaType() == mediaType) {
				return val;
			}
		}

		CustomLog.e(TAG,
				"getStreamInfoByMediaType() 无对应StreamInfo, mediaType: "
						+ mediaType);
		return null;
	}

	public int getScreenShareStatus() {
		return screenShareStatus;
	}

	public int getSyncId() {
		if (mSyncId == -1) {
			mSyncId = productSyncId();
		}

		return mSyncId;
	}

	public void setScreenShareInfo(int status) {
		CustomLog.d(TAG, "setScreenShareInfo currentStatus: "
				+ screenShareStatus + " | status: " + status);
		if (screenShareStatus != status) {
			screenShareStatus = status;
		}

		if (screenShareStatus == SCREEN_NORMAL) {
			CustomLog.d(TAG,
					"SpeakerInfo::setScreenShareInfo() 停止屏幕分享，重置分享画面视频状态");
			setDocVideoStatus(DOC_VIDEO_OPEN);
		}
	}

	public int getMediaTypeFormResourceId(int resourceId) {
		Iterator<Entry<Integer, StreamInfo>> it = getmStreamsInfo().entrySet()
				.iterator();
		while (it.hasNext()) {
			Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
					.next();
			StreamInfo val = (StreamInfo) entry.getValue();
			if (val.getResourcesId() == resourceId) {
				return val.getMediaType();
			}
		}
		return -1;
	}

	/**
	 * 解析STP 携带的mediaType信息，由于STP使用的mediaType取值和媒体SM取值不一致，需要做对应解析
	 * 
	 * @param mediaType
	 *            STP所使用的mediaType值
	 * @return {@link MediaType}
	 */
	public static int ParseStpMediaTypeToSM(int mediaType) {
		switch (mediaType) {
		case 1: // 视频流
			return MediaType.TYPE_VIDEO;
		case 2: // 音频流
			return MediaType.TYPE_AUDIO;
		case 3: // 数据视频流
			return MediaType.TYPE_DOC_VIDEO;
		case 4: // 数据音频流
			return MediaType.TYPE_DOC_AUDIO;
		}
		return -1;
	}

	/**
	 * 解析SM_SDK 处mediaType信息转换为STP需要使用的信息，由于STP使用的mediaType取值和媒体SM取值不一致，需要做对应解析
	 * 
	 * @param mediaType
	 *            SM_SDK所使用的mediaType值
	 * @return {@link MediaType}
	 */
	public static int ParseSmMediaTypeToStp(int mediaType) {
		switch (mediaType) {
		case MediaType.TYPE_VIDEO: // 视频流
			return 1;
		case MediaType.TYPE_AUDIO: // 音频流
			return 2;
		case MediaType.TYPE_DOC_VIDEO: // 数据视频流
			return 3;
		case MediaType.TYPE_DOC_AUDIO: // 数据音频流
			return 4;
		}
		return -1;
	}

	/**
	 * 生成一个syncId，保证唯一性
	 * 
	 * @return syncId
	 */
	private int productSyncId() {
//		return (int) System.currentTimeMillis();
		return Integer.parseInt(accountId);
	}

	public void setAudioResourceId(int AudioResourceId) {
		this.audioResourceId = AudioResourceId;
	}

	public int getAudioResourceId() {
		return this.audioResourceId;
	}

	public void setVolumeLevel(int volumeLevel) {
		this.mVolumeLevel = volumeLevel;
	}

	public int getVolumeLevel() {
		return this.mVolumeLevel;
	}	
}
