package cn.redcdn.butelopensdk.hostagent;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.butelopensdk.constconfig.SendResult;
import cn.redcdn.butelopensdk.constconfig.ShareState;
import cn.redcdn.dep.MeetingHostAgentJNI;
import cn.redcdn.dep.P2PConnCBInterface;
import cn.redcdn.log.CustomLog;

public class HostAgentControl {
	private final String TAG = HostAgentControl.this.getClass().getName();

	public interface HostAgentListener {
		// 点对点邀请需要关注
		public void onReceiveData(String fromAccountId, String Data);

		// 一级指挥中心需要关注
		public void onOperationSuc(Object object);

		// 一级指挥中心、点对点邀请都需要关注
		public void onOperationFail(Object object);
	}

	public static HostAgentControl mHostAgentControl;
	private P2PConnCBInterface mP2PConnCBInterface;
	private String mAccountId; // 当前的视讯号
	private HostAgentListener mQueryListener; // 一级指挥中心查询listener
	private HostAgentListener mInviteListener; // 一级指挥中心邀请listener
	private HostAgentListener mQuitMeetingListener; // 通知一级指挥中心退会listener
	private HostAgentListener mP2PConnectListener; // 点对点listener
	private Map<Long, Object> mSendIdMap; // 发送消息ID对应的listenerMap
	private static boolean mInitFlag = false;

	private HostAgentControl() {

	}

	/**
	 * @brief 获取实例
	 * @param HostAgentControl
	 */
	public synchronized static HostAgentControl getInstance() {
		if (mHostAgentControl == null) {
			mHostAgentControl = new HostAgentControl();
			mInitFlag = false;
		}

		return mHostAgentControl;
	}

	/**
	 * @brief 初始化接口,需要传视讯号
	 * @param accountId
	 *            视讯号 必须在调用相关接口前设置
	 */
	public void init(String accountId) {
		if (!mInitFlag) {
			CustomLog.d(TAG, "init accountId=" + accountId);
			mInitFlag = true;
			mAccountId = accountId;
			mSendIdMap = new HashMap<Long, Object>();
			initHostAgent();
		} else {
			CustomLog.d(TAG, "alreay init");
		}
	}
	
	/**
	 * @brief release接口
	 */
	public void release() {
		CustomLog.d(TAG, "release");
		mInitFlag = false;
		mAccountId = "";
		if (mSendIdMap != null) {
			mSendIdMap.clear();
		}
	}

	private void initHostAgent() {
		CustomLog.d(TAG, "initHostAgent()");
		initP2PConnInterface();
		MeetingHostAgentJNI.initP2PConn(mAccountId, mP2PConnCBInterface);
	}

	// /**
	// * @brief 设置一级指挥中心共享listener
	// * @param shareListener
	// * 一级指挥中心共享listener
	// */
	// public void setShareListener(HostAgentListener shareListener) {
	// CustomLog.d(TAG, "setShareListener()");
	// mshareListener = shareListener;
	// }

	/**
	 * @brief 设置点对点透传消息listener
	 * @param P2PConnectListener
	 *            点对点透传消息listener
	 */
	public void setP2PConnectListener(HostAgentListener P2PConnectListener) {
		CustomLog.d(TAG, "setP2PConnectListener()");
		mP2PConnectListener = P2PConnectListener;
	}

	/**
	 * @brief 查询PC状态
	 * @param nubes
	 *            查询的视讯号list
	 * @param queryListener
	 *            查询pc状态监听回调
	 */
	public int queryPCState(List<String> nubes, HostAgentListener queryListener) {
		CustomLog.d(TAG, "queryPCState");
		mQueryListener = queryListener;
		for (int i = 0; i < nubes.size(); i++) {
			long operationCode = getOperationCode();
			mSendIdMap.put(operationCode, mQueryListener);
			int ret = MeetingHostAgentJNI.sendP2PConnData(mAccountId,
					nubes.get(i), getQueryJsonString(nubes.get(i)),
					operationCode);
			if (ret != 0) {
				callBackQueryPCState(
						false,
						createQueryResponseJson(nubes.get(i),
								ShareState.STATE_OTHER));
				CustomLog.d(TAG, "queryPCState 发送失败，从mSendIdMap中删除记录");
			}
		}

		return 0;
	}

	/**
	 * @brief 邀请PC入会
	 * @param dstAccountId
	 *            邀请的视讯号
	 * @param invitePCListener
	 *            邀请pc回调
	 */
	public int invitePC(String dstAccountId, int meetingId,
			HostAgentListener inviteListener) {
		CustomLog.d(TAG, "invitePC dstAccountId=" + dstAccountId
				+ " meetingId=" + meetingId);
		mInviteListener = inviteListener;
		long operationCode = getOperationCode();
		mSendIdMap.put(operationCode, mInviteListener);
		int ret = MeetingHostAgentJNI.sendP2PConnData(mAccountId, dstAccountId,
				getInviteJsonString(dstAccountId, meetingId), operationCode);
		if (ret != 0) {
			callBackInvitePC(false, null);
			CustomLog.d(TAG, "invitePC 发送失败，从mSendIdMap中删除记录");
		}
		return 0;
	}

	/**
	 * @brief 通知数据分享器退出会议
	 * @param dstAccountId
	 *            PC的视讯号
	 * @param invitePCListener
	 *            邀请pc回调
	 */
	public int notifyPCQuitMeeting(String dstAccountId,
			HostAgentListener quitMeetingListener) {
		CustomLog.d(TAG, "notifyPCQuitMeeting dstAccountId=" + dstAccountId);
		mQuitMeetingListener = quitMeetingListener;
		long operationCode = getOperationCode();
		mSendIdMap.put(operationCode, mQuitMeetingListener);
		int ret = MeetingHostAgentJNI.sendP2PConnData(mAccountId, dstAccountId,
				getQuitMeetingJsonString(dstAccountId), operationCode);
		if (ret != 0) {
			callBackQuitMeeting(false, null);
			CustomLog.d(TAG, "notifyPCQuitMeeting 发送失败，从mSendIdMap中删除记录");
		}
		return 0;
	}

	/**
	 * @brief 发送点对点消息
	 * @param dstAccountId
	 *            目标视讯号
	 * @param data
	 *            发送的内容
	 */
	public int sendP2PMsg(String dstAccountId, String data) {
		CustomLog.d(TAG, "sendP2PMsg dstAccountId=" + dstAccountId + " data="
				+ data);
		long operationCode = getOperationCode();
		mSendIdMap.put(operationCode, mP2PConnectListener);
		int ret = MeetingHostAgentJNI.sendP2PConnData(mAccountId, dstAccountId,
				data, operationCode);
		if (ret != 0) {
			mP2PConnectListener.onOperationFail(SendResult.SEND_FAIL);
			CustomLog.d(TAG, "sendP2PMsg 发送失败，从mSendIdMap中删除记录");
			mSendIdMap.remove(operationCode);
		}
		return 0;
	}

	private void callBackQueryPCState(boolean success, Object obj) {
		CustomLog.i(TAG, "ButelOpenSDK::callBackQueryPCState() 查询PC状态回调。"
				+ " | status: " + success + " obj=" + obj.toString());
		if (success) {
			mQueryListener.onOperationSuc(obj);
		} else {
			mQueryListener.onOperationFail(obj);
		}
	}

	private void callBackInvitePC(boolean success, Object obj) {
		CustomLog.i(TAG, "ButelOpenSDK::callBackInvitePC() 邀请PC回调"
				+ " | status: " + success);
		if (success) {
			mInviteListener.onOperationSuc(obj);
		} else {
			mInviteListener.onOperationFail(obj);
		}
	}

	private void callBackQuitMeeting(boolean success, Object obj) {
		CustomLog.i(TAG, "ButelOpenSDK::callBackQuitMeeting() 通知PC退会回调"
				+ " | status: " + success);
		if (success) {
			mQuitMeetingListener.onOperationSuc(obj);
		} else {
			mQuitMeetingListener.onOperationFail(obj);
		}
	}

	private String getQueryJsonString(String accountId) {
		JSONObject json = new JSONObject();
		try {
			json.put("cmdID", 40003);
			json.put("accountId", accountId);
		} catch (JSONException e) {
			CustomLog.e(TAG, e.getMessage());
		}
		return json.toString();
	}

	private String getInviteJsonString(String dstAccountId, int meetingId) {
		JSONObject json = new JSONObject();
		try {
			json.put("cmdID", 40001);
			json.put("accountId", dstAccountId);
			json.put("meetingId", meetingId);
			json.put("monitorId", mAccountId);
		} catch (JSONException e) {
			CustomLog.e(TAG, e.getMessage());
		}
		return json.toString();
	}

	private String getQuitMeetingJsonString(String dstAccountId) {
		JSONObject json = new JSONObject();
		try {
			json.put("cmdID", 40005);
			json.put("accountId", dstAccountId);
		} catch (JSONException e) {
			CustomLog.e(TAG, e.getMessage());
		}
		return json.toString();
	}

	private String createQueryResponseJson(String accountId, int state) {
		JSONObject json = new JSONObject();
		try {
			json.put("accountId", accountId);
			json.put("state", state);
		} catch (JSONException e) {
			CustomLog.e(TAG, e.getMessage());
		}
		return json.toString();
	}

	private long getOperationCode() {
		long operationCode = System.currentTimeMillis()
				+ (int) (Math.random() * 900 + 100);
		CustomLog.i(TAG, "getOperationCode operationCode=" + operationCode);
		return operationCode;
	}

	// add host p2p connect
	private void initP2PConnInterface() {
		mP2PConnCBInterface = new P2PConnCBInterface() {
			@Override
			public void OnRecvData(String accountId, String data) {
				CustomLog.d(TAG, "OnRecvData accountId=" + accountId + " data="
						+ data);
				int cmdId = -1;
				String pcAccountId = "";
				String meetingId = "";
				int rc = -1;
				int shareStatus = -1;
				try {
					JSONObject json = new JSONObject(data);
					if (json.has("cmdID")) {
						cmdId = json.getInt("cmdID");
					}
					if (json.has("accountId")) {
						pcAccountId = json.getString("accountId");
					}
					if (json.has("meetingId")) {
						meetingId = String.valueOf(json.getInt("meetingId"));
					}
					if (json.has("rc")) {
						rc = json.getInt("rc");
					}
					if (json.has("shareStatus")) {
						shareStatus = json.getInt("shareStatus");
					}
				} catch (JSONException e) {
					CustomLog.e(TAG, "OnRecvData e=" + e.getMessage());
				}
				if (cmdId == 40002) {
					if (rc == 0) {
						callBackInvitePC(true, null);
					} else {
						callBackInvitePC(false, null);
					}
				} else if (cmdId == 40006) {
					if (rc == 0) {
						callBackQuitMeeting(true, null);
					} else {
						callBackQuitMeeting(false, null);
					}
				} else if (cmdId == 40004) {
					if (shareStatus == 0) {
						callBackQueryPCState(
								true,
								createQueryResponseJson(pcAccountId,
										ShareState.STATE_IDLE));
					} else if (shareStatus == 1) {
						callBackQueryPCState(
								true,
								createQueryResponseJson(pcAccountId,
										ShareState.STATE_BUSY));
					}
				} else {
					CustomLog.d(TAG, "OnRecvData 透传消息");
					mP2PConnectListener.onReceiveData(accountId, data);
				}
			}

			@Override
			public void OnSendDataResult(String accountId, int rc, long context) {
				CustomLog.d(TAG, "OnSendDataResult accountId=" + accountId
						+ " rc=" + rc + " context=" + context);
				HostAgentListener hostAgentListener = (HostAgentListener) mSendIdMap
						.get(context);
				if (hostAgentListener == null) {
					CustomLog.d(TAG, "OnSendDataResult 没有对应的listener 不做处理");
					return;
				}
				if (rc != 0) {
					if (hostAgentListener == mInviteListener) {
						CustomLog.d(TAG, "OnSendDataResult callBackInvitePC");
						callBackInvitePC(false, null);
					} else if (hostAgentListener == mQuitMeetingListener) {
						CustomLog
								.d(TAG, "OnSendDataResult callBackQuitMeeting");
						callBackQuitMeeting(false, null);
					} else if (hostAgentListener == mQueryListener) {
						CustomLog.d(TAG,
								"OnSendDataResult callBackQueryPCState");
						if (rc == -1) {
							callBackQueryPCState(
									false,
									createQueryResponseJson(accountId,
											ShareState.STATE_OTHER));
						} else if (rc == -2) {
							callBackQueryPCState(
									false,
									createQueryResponseJson(accountId,
											ShareState.STATE_TIME_OUT));
						}
					} else if (hostAgentListener == mP2PConnectListener) {
						CustomLog
								.d(TAG,
										"OnSendDataResult mP2PConnectListener onOperationFail");
						if (rc == -1) {
							mP2PConnectListener
									.onOperationFail(SendResult.SEND_FAIL);
						} else if (rc == -2) {
							mP2PConnectListener
									.onOperationFail(SendResult.SEND_TIME_OUT);
						}
					} else {
						CustomLog.w(TAG,
								"OnSendDataResult 没有对应的Listener与之匹配，不做处理!");
					}
				} else {
					if (hostAgentListener == mP2PConnectListener) {
						CustomLog
								.d(TAG,
										"OnSendDataResult mP2PConnectListener onOperationSuc");
						mP2PConnectListener
								.onOperationSuc(SendResult.SEND_SUCCESS);
					}
				}
				CustomLog.d(TAG, "OnSendDataResult 收到发送结果回调，从mSendIdMap中删除记录");
				mSendIdMap.remove(context);
			}
		};
	}

}
