package cn.redcdn.butelopensdk.media;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.ImageFormat;
import android.graphics.PixelFormat;
import android.graphics.SurfaceTexture;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.hardware.Camera;
import android.hardware.Camera.CameraInfo;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.PreviewCallback;
import android.hardware.Camera.Size;
import android.hardware.usb.UsbDevice;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.Gravity;
import android.view.Surface;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TabHost;
import android.widget.Toast;

import cn.redcdn.butelopensdk.ButelOpenSDK;
import cn.redcdn.butelopensdk.MeetingInfo;
import cn.redcdn.butelopensdk.constconfig.DeviceRunEnvironment;
import cn.redcdn.butelopensdk.constconfig.DeviceType;
import cn.redcdn.butelopensdk.constconfig.ExtendSurfaceHolderCallBack;
import cn.redcdn.butelopensdk.constconfig.ExtendSurfaceHolderCallBack.SurfaceViewCallBack;
import cn.redcdn.butelopensdk.constconfig.MediaAdaptiveParam;
import cn.redcdn.butelopensdk.constconfig.MediaType;
import cn.redcdn.butelopensdk.constconfig.ShareType;
import cn.redcdn.butelopensdk.constconfig.SpeakerInfo;
import cn.redcdn.butelopensdk.constconfig.StreamInfo;
import cn.redcdn.butelopensdk.util.CpuUsageUtil;
import cn.redcdn.butelopensdk.vo.VideoParameter;
import cn.redcdn.log.CustomLog;

import com.channelsoft.sipsdk.SmSdkCBInterface;
import com.channelsoft.sipsdk.SmSdkJNI;
import com.channelsoft.sipsdk.mediaServerAgentJni;
import com.redcdn.keyeventwrite.KeyEventConfig;
import com.redcdn.keyeventwrite.KeyEventWrite;
import com.serenegiant.usb.DeviceFilter;
import com.serenegiant.usb.IFrameCallback;
import com.serenegiant.usb.USBMonitor;
import com.serenegiant.usb.USBMonitor.OnDeviceConnectListener;
import com.serenegiant.usb.USBMonitor.UsbControlBlock;
import com.serenegiant.usb.UVCCamera;

/**
 * 处理 SM_SDK 的命令封装
 * <p>
 * 其中对于本地摄像头的处理调整为真正需要使用摄像头时再申请打开摄像头、开启预览，包括：
 * <p>
 * 1. 发言：打开摄像头、开启摄像头预览
 * <p>
 * 2. 停止发言：关闭摄像头预览、关闭摄像头
 * <p>
 * 3. 收到vga、gvga切换：关闭摄像头预览、重新设置采集参数、打开摄像头预览
 * <p>
 * 4. 关闭摄像头：关闭摄像头预览、关闭摄像头、设置本地surfaceview 背景、停止向sm_sdk push数据
 * <p>
 * 5. 打开摄像头：打开摄像头、打开摄像头预览、清除本地surfaceview 背景、恢复向sm_sdk push数据
 * <p>
 * 6. 切换到语音模式：调用sm_sdk audioModeOn、如果在发言（关闭摄像头预览、关闭摄像头、设置本地surfaceview 背景）
 * <p>
 * 7. 切换到视频模式：调用sm_sdk audioModeOff、如果在发言（打开摄像头、打开摄像头预览、清除本地surfaceview 背景）
 * <p>
 * 8. 切换摄像头：关闭摄像头预览、关闭摄像头、切换索引、打开摄像头、打开摄像头预览
 */
public class SMSDKControl {
    private final String TAG = SMSDKControl.this.getClass().getName();

    public interface SMSDKListener {
        public void onStatus(int status, Object obj);
    }

    private static final int ON_ATTACH = 1;
    private static final int ON_DETTACH = 2;
    private static final int ON_CONNECT = 3;
    private static final int ON_DISCONNECT = 4;

    private int cameraTypeBoth = 0;
    private int cameraTypeUvcCamera = 1;
    private int cameraTypeSystemCamera = 2;

    private final Object mSync = new Object();

    private Context mContext;

    private int mMeetingId;

    private String mAccountId;

    private String mRcAddr;

    private String mLogConfigPath;

    private String mLogOutPath;

    private SMSDKListener mSMSDKListener;

    private String mDeviceType = DeviceType.DEVICE_M1S;

    private boolean mSelectUvcCamera = false;// 是否设置使用uvc摄像头

    private ButelOpenSDK mButelopensdk;

    private static final int RECOVER_CAMERA_DELAY_TIME = 3000; // 自预览异常时，重试间隔时间

    private int mDoRecoverCount = 0; // 自预览重试次数

    private int mRecoverTime = 3; // 自预览数据异常时，重试次数

    private boolean isWatchCamera = false; // 是否处于监测自预览状态

    private int mStatus = StatusCode.NOT_INIT;

    private Camera mCamera;

    private int mCameraIndex = -1; // 当前摄像头索引

    private boolean mCameraIsOpen = false;

    // 记录视频流相关信息，如果会议过程中切换摄像头，需要取消发布再重新发布
    private int mVideoStremId = -1;

    private int mVideoResourceId = -1;

    private int mVideoSyncId = -1;

    // 数据视图采用保持比例留黑边策略；人像视图采用保持比例、不留黑边、裁剪策略
    private final static int FILL_MODE_RAWRATIO_WITHBLACK = 0; // 保持原始比例留黑边

    @SuppressWarnings("unused")
    private final static int FILL_MODE_STRETCH = -1; // 拉伸铺满全屏

    private final static int FILL_MODE_RAWRATIO_ZOOM_NOBLACK = -2;// 保持原始比例，不留黑边，进行缩放裁剪

    // 标清: 640 x 360、300Kbps、15帧
    private final static int STANDARD_CAPTURE_RATE_WEIGHT = 640;

    private final static int STANDARD_CAPTURE_RATE_HEIGHT = 360;

    private final static int DEFAULT_UVC_WEIGHT = 640;

    private final static int DEFAULT_UVC_HEIGHT = 480;

    private final static int STANDARD_EncBitrate = 300;

    private HashMap<Integer, VideoParameter> mVideoParamMap;

    private boolean changingCamera = false; // 标记是否处于切换前后置摄像头状态

    // 记录标志为发布、取消发布视频流
    private boolean isSpeaking = false; // 是否处于发言状态

    private boolean isCameraOpen = true; // 是否开启了摄像头

    private SurfaceTexture mSurfTex;

    // private SurfaceHolder mSurfaceHolder; // 本地渲染surfaceView需要保存
    private SurfaceView mSurfaceView; // 本地渲染surfaceView需要保存，用于操作打开、关闭摄像头，以及设置关闭摄像头之后的背景

    private SurfaceHolder LocalHolder;

    private UVCCamera mUVCCamera;

    private USBMonitor mUSBMonitor;

    private boolean useUVCCamera = false;

    private List<DeviceFilter> mFilter;

    private int mCloseCameraGroundId;// 保存关闭摄像头默认图片

    private int mCameraOrientation = 0; // 摄像头旋转角度

    private int mPushFrameOrientation = 0; // pushframe旋转角度

    public final static int DEVICE_AUDIO_INIT_FAILED = -9; // 由媒体库底层定义，当音频采集失败时返回

    // 自适应参数，外部可设置，如果没有设置，使用默认值
    private int upBandwidth = 8000;

    private int downBandwidth = 8000;

    private int upRetransBwRatio = 20;

    @SuppressWarnings("unused")
    private int downRetransBwRatio = 20;

    private int videoUpFecDataCount = 6;

    private int videoUpFecCheckCount = /* 3 */0;

    private int videoUpDataSendCount = 0;

    private int videoUpCheckSendCount = 0;

    private int videoUpDataRetransSendCount = /* 3 */0;

    private int videoUpCheckRetransSendCount = 0;

    private int videoUpDataResendCount = /* 5 */10;

    private int DocVideoUpFecDataCount = 6;

    private int DocVideoUpFecCheckCount = /* 3 */0;

    private int DocVideoUpDataSendCount = 0;

    private int DocVideoUpCheckSendCount = 0;

    private int DocVideoUpDataRetransSendCount = /* 3 */0;

    private int DocVideoUpCheckRetransSendCount = 0;

    private int DocVideoUpDataResendCount = /* 5 */10;

    private int audioUpFecDataCount = 4;

    private int audioUpFecCheckCount = 6;

    private int audioUpDataSendCount = 1;

    private int audioUpCheckSendCount = 0;

    private int audioUpDataRetransSendCount = 3;

    private int audioUpCheckRetransSendCount = 0;

    private int audioUpDataResendCount = /* 5 */10;

    private int videoDownFecDataCount = 6;

    private int videoDownFecCheckCount = /* 3 */0;

    private int videoDownDataSendCount = 0;

    private int videoDownCheckSendCount = 0;

    private int videoDownDataRetransSendCount = /* 3 */0;

    private int videoDownCheckRetransSendCount = 0;

    private int videoDownDataResendCount = /* 5 */10;

    private int videoDelayTime = 300;

    private int DocVideoDownFecDataCount = 6;

    private int DocVideoDownFecCheckCount = /* 3 */0;

    private int DocVideoDownDataSendCount = 0;

    private int DocVideoDownCheckSendCount = 0;

    private int DocVideoDownDataRetransSendCount = /* 3 */0;

    private int DocVideoDownCheckRetransSendCount = 0;

    private int DocVideoDownDataResendCount = /* 5 */10;

    private int DocVideoDelayTime = 300;

    private int audioDownFecDataCount = 4;

    private int audioDownFecCheckCount = 6;

    private int audioDownDataSendCount = 1;

    private int audioDownCheckSendCount = 0;

    private int audioDownDataRetransSendCount = 3;

    private int audioDownCheckRetransSendCount = 0;

    private int audioDownDataResendCount = /* 5 */10;

    private int audioDelayTime = 300;

    private Timer setCpuTimer;

    private TimerTask setCpuTimerTask;

    private boolean mIsBigStream = false;

    private SmSdkJNI mSmSdkJNI;

    private ExtendSurfaceHolderCallBack mLocalSurfaceCallBack;

    private int mCodecType; // 编码类型

    private int mBigImage; // 大背景
    private int mSmallImage; // 小背景
    private static CameraInfo cameraInfo = new CameraInfo();

    private int mPollVideoStreamId = -1;
    private VideoParameter mPollVideoParameter;

    private List<String> mBigStreamList;
    private MeetingInfo mMeetingInfo;

    private SmSdkCBInterface smSDKCbInterface = new SmSdkCBInterface() {

        // @Override
        // public void SMSDK_CallBack(int statusCode, String description) {
        // CustomLog.d(TAG, "SMSDKControl::SMSDK_CallBack() statusCode: "
        // + statusCode + " | des: " + description);
        // Message msg = new Message();
        // msg.what = statusCode;
        // msg.obj = description;
        // mNotifyHandler.sendMessage(msg);
        // }

        @Override
        public void SMSDK_CallBack(int statusCode, byte[] description) {
            String desc = new String(description, 0, description.length);
            // CustomLog.d(TAG, "SMSDKControl::SMSDK_CallBack() statusCode: "
            // + statusCode + " | des: " + desc);
            Message msg = new Message();
            msg.what = statusCode;
            msg.obj = desc;
            mNotifyHandler.sendMessage(msg);
        }
    };

    public class MediaPlaySDKReturnCode {
        public static final int INIT_SUC = 10;

        public static final int INIT_FAILURE = 11;

        public static final int MIC_1_SPEAK_SUC = 12;

        public static final int MIC_1_SPEAK_FAILURE = 13;

        public static final int MIC_1_STOP_SPEAK_SUC = 14;

        public static final int MIC_1_STOP_SPEAK_FAILURE = 15;

        public static final int MIC_2_SPEAK_SUC = 16;

        public static final int MIC_2_SPEAK_FAILURE = 17;

        public static final int MIC_2_STOP_SPEAK_SUC = 18;

        public static final int MIC_2_STOP_SPEAK_FAILURE = 19;

        public static final int RELEASE_SUC = 20;

        public static final int RELEASE_FAILURE = 21;

        public static final int CHANGE_TO_VGA = 22;

        public static final int CHANGE_TO_QVGA = 23;

        public static final int MEDIAPLAY_REMOTE_NETWORK_POOR = 24;

        public static final int MEDIAPLAY_LOCAL_NETWORK_POOR = 25;

        public static final int MEDIAPLAY_ALL_NETWORK_POOR = 26;

        public static final int MEDIAPLAY_GET_NET_INFO = 27;

        public static final int MEDIAPLAY_NOTIFY_AUDIO_TYPE = 28;

        public static final int MEDIAPLAY_AUDIO_IN_DEVICE_INIT_FAILED = 29;

        public static final int MEDIAPLAY_AUDIO_OUT_DEVICE_INIT_FAILED = 30;

        public static final int CAMERA_OPEN_FAILED = 31;

        public static final int START_SHARE_DOC = 32;

        public static final int SHARE_NAME_CHANGE = 33;

        public static final int STOP_SHARE_DOC = 34;

        public static final int START_LOCAL_VIDEO_CALLBACK = 35;

        public static final int START_REMOTE_VIDEO_CALLBACK = 36;

        public static final int START_LOCAL_DOC_CALLBACK = 37;

        public static final int START_REMOTE_DOC_CALLBACK = 38;

        public static final int VIDEO_PARAM_UPDATE = 39; // 采集分辨率发生更新

        public static final int VIDEO_STREAMID_UPDATE = 40; // 视频streamId发生变化

        public static final int SIGNAL_STRENGTH_UPDATE = 41; // 网络信号强度更新通知

        public static final int DEVICES_LOST = 42;// 出现设备丢失

        public static final int CPU_OVERLOAD = 43; // 占用cpu过载

        public static final int NET_STATUS = 44; // 网络信号

        public static final int VIDEO_CLOSE_ADAPT = 45; // 自适应关闭视频

        public static final int VOLUME_UPDATE = 46; // 声音强度变化

        public static final int VIDEO_START_RENDER = 47; // 底层收到视频数据开始渲染

        public static final int SURFACEVIEW_STATE_CHANGE = 48; // surfaceview状态改变

        public static final int CAMERA_OPEN_SUCCESS = 49; // 摄像头打开成功
    }

    private class StatusCode {
        private static final int NOT_INIT = 0;

        private static final int INIT = 1;

        private static final int RELEASE = 6;
    }

    private Camera.PreviewCallback mPreviewCallBack = new PreviewCallback() {
        @Override
        public void onPreviewFrame(byte[] data, Camera camera) {
            // CustomLog.d(TAG, "向李阳push数据中:" + data.length);
            mDoRecoverCount = 0;
            recoverCameraHandler.removeMessages(0);
            if (isSpeaking || mPollVideoStreamId > 0) {
                if (isWatchCamera) {
                    recoverCameraHandler.sendEmptyMessageDelayed(0,
                            RECOVER_CAMERA_DELAY_TIME);
                }

                if (mCodecType == 0) {
                    // getCameraOrientation();

                    SmSdkJNI.SM_SDK_PushFrame(mVideoStremId,
                            SmSdkJNI.SMSDK_FRAME_TYPE_YUV, data, data.length,
                            mPushFrameOrientation);
                }

                if (mCamera != null) {
                    mCamera.addCallbackBuffer(data); // 需要调用，触发继续onPreviewFrame
                }
            }
        }
    };

    private Handler recoverCameraHandler = new Handler() {
        @Override
        public void handleMessage(android.os.Message msg) {
            if (mCodecType == 0) {
                CustomLog.d(TAG, "检测到发言状态不pushFrame,重启预览");
                String str = KeyEventConfig.PUSHFRAME_EXCEPTION_STATE + "_ok_"
                        + mAccountId + "_pushFrame异常";
                KeyEventWrite.write(str);
                if (mDoRecoverCount == mRecoverTime) {
                    CustomLog.e(TAG, "error 摄像头打开异常！尝试重启预览" + mRecoverTime
                            + "次，不再重启");
                    recoverCameraHandler.removeMessages(0);
                    mSMSDKListener.onStatus(
                            MediaPlaySDKReturnCode.CAMERA_OPEN_FAILED, null);
                    return;
                }
                stopCameraPreview();
                if (isSpeaking) {
                    mDoRecoverCount = mDoRecoverCount + 1;
                    CustomLog.d(TAG, "尝试重启预览,第 " + mDoRecoverCount + "次");
                    CustomLog.d(TAG, "recoverCameraHandler startCameraPreview");
                    startCameraPreview();
                } else {
                    mDoRecoverCount = 0;
                    CustomLog.d(TAG, "状态不合法，不需要恢复摄像头");
                }
            }
        }
    };

    private Handler mNotifyHandler = new Handler() {
        @Override
        public void handleMessage(android.os.Message msg) {
            int what = msg.what;
            String des = String.valueOf(msg.obj);
            // CustomLog.d(TAG, "SMSDKControl::handleMessage 收到smsdk通知： " + what
            // + " |des: " + des);
            switch (what) {
                case mediaServerAgentJni.msagent_MEDIAPLAY_REMOTE_NETWORK_POOR:
                    handleRemoteNetworkPoor();
                    break;
                case mediaServerAgentJni.msagent_MEDIAPLAY_LOCAL_NETWORK_POOR:
                    handleLocalNetworkPoor();
                    break;
                case mediaServerAgentJni.msagent_MEDIAPLAY_ALL_NETWORK_POOR:
                    handleAllNetworkPoor();
                    break;
                case mediaServerAgentJni.msagent_netInfo_getNetInfo:
                    handleGetNetInfo();
                    break;
                case mediaServerAgentJni.msagent_netInfo_notifyAudioMode:
                    handleNotifyAudioMode();
                    break;
                case SmSdkJNI.SM_SDK_lossrateLevel:
                    handleSignalStrengthUpdate(des);
                    break;
                case SmSdkJNI.SM_SDK_devicelost:
                    handleDeviceLost(des);
                    break;
                case SmSdkJNI.SM_SDK_CB_CPUStat:// 占用cpu过载
                    handleNotifyCpuOverLoad(des);
                    break;
                case SmSdkJNI.SM_SDK_CB_netStatusNotify:// 网络信号
                    handleNotifyNetStatus(des);
                    break;
                case SmSdkJNI.SM_SDK_CB_audioDBNotify:// 音量变化
                    // CustomLog.d(TAG, "SmSdkJNI.SM_SDK_CB_audioDBNotify");
                    handleNotifyVolumeUpdate(des);
                    break;
                case SmSdkJNI.SM_SDK_CB_videoCloseNotify:// 自适应关闭视频
                    handleNotifyVideoCloseAdapt(des);
                    break;

                case SmSdkJNI.SM_SDK_CB_frameWork_msgCB:
                    handleNotifyFrameWorkMsg(des);
                    break;
                default:
                    break;
            }
        }
    };

    private Handler cameraHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);
            switch (msg.what) {
                case ON_ATTACH:
                    CustomLog.d(TAG, "ON_ATTACH");
                    CustomLog.d(TAG,
                            "mCameraIsOpen:" + String.valueOf(mCameraIsOpen));
                    if (mCamera != null) {
                        CustomLog.d(TAG, "mCamera!=null");
                    } else {
                        CustomLog.d(TAG, "mCamera==null");
                    }
                    if (mCamera != null) {
                        closeCamera();
                        releaseCamera(cameraTypeSystemCamera);
                    }
                    if (null != mUSBMonitor && null != mFilter
                            && null != mUSBMonitor.getDeviceList(mFilter.get(0))
                            && mUSBMonitor.getDeviceList(mFilter.get(0)).size() > 0) {
                        mUSBMonitor.requestPermission((UsbDevice) mUSBMonitor
                                .getDeviceList(mFilter.get(0)).get(0));
                        useUVCCamera = true;
                    }
                    break;
                case ON_DETTACH:
                    CustomLog.d(TAG, "ON_DETTACH");
                    CustomLog.d(TAG,
                            "mCameraIsOpen:" + String.valueOf(mCameraIsOpen));
                    if (mUVCCamera != null) {
                        CustomLog.d(TAG, "mUVCCamera!=null");
                    } else {
                        CustomLog.d(TAG, "mUVCCamera==null");
                    }
                    if (mUVCCamera != null) {
                        closeCamera();
                        releaseCamera(cameraTypeUvcCamera);
                    }
                    break;
                case ON_CONNECT:
                    CustomLog.d(TAG, "ON_CONNECT");
                    UsbControlBlock ctrlBlock = (UsbControlBlock) msg.obj;
                    handleOnConnect(ctrlBlock);
                    break;
                case ON_DISCONNECT:
                    CustomLog.d(TAG, "ON_DISCONNECT");
                    CustomLog.d(TAG,
                            "mCameraIsOpen:" + String.valueOf(mCameraIsOpen));
                    if (mUVCCamera != null) {
                        CustomLog.d(TAG, "mUVCCamera!=null");
                    } else {
                        CustomLog.d(TAG, "mUVCCamera==null");
                    }
                    handleOnDisconnect();
                    break;
                default:
                    break;
            }
        }
    };

    public SMSDKControl(Context context) {
        CustomLog.d(TAG, "SMSDKControl 构造");
        mContext = context;
    }

    public void setCloseCameraBackGroundId(int closeCameraGroundId) {
        mCloseCameraGroundId = closeCameraGroundId;
    }

    public void setCloseCameraBackGroudBitmap(Bitmap bitmap) {
        CustomLog.d(TAG, "setCloseCameraBackGroudBitmap() ：" + bitmap);
        if (mSurfaceView != null) {
            try {
                Drawable drawable = new BitmapDrawable(bitmap);
                mSurfaceView.setBackground(drawable);
            } catch (Exception ex) {
                CustomLog.e(
                        TAG,
                        "setCloseCameraBackGroudBitmap error! "
                                + ex.getMessage());
            }

        }
    }

    public int initSMSDK(int meetingId, String accountId, String rcAddr,
                         String logConfigPath, String logOutPath, String deviceType,
                         boolean selectUvcCamera, ButelOpenSDK butelopensdk,
                         SMSDKListener msSDKListener, int codecType, MeetingInfo meetingInfo) {
        CustomLog.d(TAG, "SMSDKControl::initSMSDK() meetingId: " + meetingId
                + " | accountId: " + accountId + " | rcAddr: " + rcAddr
                + " | logConfigPath: " + logConfigPath + " | logOutPath: "
                + logOutPath + "|deviceType=" + deviceType + "|codecType="
                + codecType);
        if (mStatus == StatusCode.INIT) {
            CustomLog.e(TAG, "已经初始化，不需要再进行初始话。 return -1");
            return -1;
        }
        mMeetingId = meetingId;
        mAccountId = accountId;
        mRcAddr = rcAddr;
        mLogConfigPath = logConfigPath;
        mLogOutPath = logOutPath;
        mSMSDKListener = msSDKListener;
        mDeviceType = deviceType;
        mSelectUvcCamera = selectUvcCamera;
        mButelopensdk = butelopensdk;
        mCodecType = codecType;
        mVideoParamMap = new HashMap<Integer, VideoParameter>();
        mBigStreamList = new ArrayList<String>();
        mMeetingInfo = meetingInfo;
        if (mSelectUvcCamera) {
            mCameraIndex = 2;
        } else {
            mCameraIndex = getCameraIndexList(deviceType).get(0);
        }

        // todo
        mSmSdkJNI = new SmSdkJNI();
        mSmSdkJNI.setUserCallBack(smSDKCbInterface);
        int ret = SmSdkJNI.SM_SDK_Init(mAccountId, mRcAddr, mLogConfigPath,
                mLogOutPath, adjustLibPath());
        if (ret == 0) {
            mStatus = StatusCode.INIT;
            CustomLog.d(TAG, "SM_SDK_SetAdaptionUpBandWidthDetectMaxBW:5000");
            SmSdkJNI.SM_SDK_SetAdaptionUpBandWidthDetectMaxBW(5000);
            CustomLog.d(TAG, "SM_SDK_SetAdaptionDownBandWidthDetectMaxBW:5000");
            SmSdkJNI.SM_SDK_SetAdaptionDownBandWidthDetectMaxBW(5000);
            CustomLog.d(
                    TAG,
                    "SM_SDK_JoinMeeting:mMeetingId:"
                            + String.valueOf(mMeetingId) + " upBandwidth:"
                            + String.valueOf(upBandwidth) + " downBandwidth:"
                            + String.valueOf(downBandwidth)
                            + " upRetransBwRatio:"
                            + String.valueOf(upRetransBwRatio));
            SmSdkJNI.SM_SDK_JoinMeeting(String.valueOf(mMeetingId),
                    upBandwidth, downBandwidth, upRetransBwRatio);
            setCpuInfo();

            setCodecType(mCodecType);
        } else {
            CustomLog.e(TAG, "SmSdkJNI.SM_SDK_start error,ret: " + ret);
        }
        return ret;
    }

    private String adjustLibPath() {
        String path = mContext.getApplicationContext().getPackageResourcePath();
        String libRDCodecPath = "/data/data/" + mContext.getPackageName()
                + "/lib";
        CustomLog.d(TAG, "tttttttttt  libRDCodecPath = " + libRDCodecPath
                + "path=" + path);
        Log.e(TAG, "tttttttttt  libRDCodecPath = " + libRDCodecPath + "path="
                + path);
        return libRDCodecPath;
    }

    public void setNetInfo(String activeIp, int netType) {
        CustomLog.d(TAG, "SMSDKControl::setNetInfo() " + "activeIp = "
                + activeIp + " netType = " + netType);
    }

    public int setNetInfo(String localIP, int netType, int wifiLinkSpeed,
                          int wiffRssi, String wifiSsid) {
        if (mStatus == StatusCode.NOT_INIT) {
            CustomLog.e(TAG, "SMSDKControl::setNetInfo() not init! return -1");
            return -1;
        }
        CustomLog.d(TAG, "SMSDKControl::setNetInfo() " + "mSpeed = "
                + wifiLinkSpeed + " mRssi" + wiffRssi);
        SmSdkJNI.SM_SDK_SetNetInfo(localIP, netType, wifiLinkSpeed, wiffRssi,
                wifiSsid);
        return 0;
    }

    /**
     * 切换摄像头
     *
     * @return </p> =0 成功 </p> !=0 失败
     */
    public int changeCamera() {

        CustomLog.d(TAG, "SMSDKControl::changeCamera() 切换摄像头,当前发言状态： "
                + isSpeaking + " |当前摄像头索引: " + mCameraIndex);
        if (!isSpeaking) {
            CustomLog.d(TAG,
                    "error! SMSDKControl::changeCamera() 发言状态下才支持切换摄像头,当前并为发言");
            return -1;
        }
        if (useUVCCamera || mSelectUvcCamera) {
            if (mContext != null) {
                Toast.makeText(mContext, "当前是UVC摄像头，不支持切换", Toast.LENGTH_SHORT)
                        .show();
            }
            return -3;
        }
        changingCamera = true;
        VideoParameter preVideoParam = getVideoParam();

        mCameraIndex = 1 - mCameraIndex; // 一般情况下，最多存在2个摄像头设备做切换，也就是 index只介于
        // 0~1 之间

        if (mCameraIndex < 0) {
            CustomLog.e(TAG, "SMSDKControl::changeCamera 失败. camera index："
                    + mCameraIndex);
            changingCamera = false;
            return -2;
        }

        VideoParameter curVideoParam = getVideoParam();
        releaseCamera();

//        int result = initCamera();
//        if (result != 0) {
//            CustomLog.e(TAG, "SMSDKControl::changeCamera 异常 openCamera："
//                    + result);
//            changingCamera = false;
//            return -2;
//        }
        CustomLog.d(TAG, "changeCamera startCameraPreview");
        int result = startCameraPreview();

        if (result != 0) {
            CustomLog.e(TAG,
                    "SMSDKControl::changeCamera 异常 startCameraPreview："
                            + result);
            changingCamera = false;
            return -2;
        }

        if (preVideoParam.getCapWidth() != curVideoParam.getCapWidth()
                || preVideoParam.getCapHeight() != curVideoParam.getCapHeight()
                || preVideoParam.getCapFps() != curVideoParam.getCapFps()
                || preVideoParam.getEncBitrate() != curVideoParam
                .getEncBitrate()) {
            CustomLog
                    .d(TAG,
                            "SMSDKControl::changeCamera() 切换摄像头：视频采集参数有改动，需要重新发布视频流,并通知上层streamId变化");
            unPublish(mVideoStremId);
            mVideoStremId = publishCameraVideo(mVideoResourceId, mVideoSyncId);
            mLocalSurfaceCallBack.setValues(mVideoStremId);
            mSMSDKListener
                    .onStatus(MediaPlaySDKReturnCode.VIDEO_STREAMID_UPDATE,
                            mVideoStremId);
        }

        if (preVideoParam.getCapWidth() != curVideoParam.getCapWidth()
                || preVideoParam.getCapHeight() != curVideoParam.getCapHeight()) {
            CustomLog.d(TAG,
                    "SMSDKControl::changeCamera() 切换摄像头：采集分辨率有改动，需要通知上层");
            mSMSDKListener.onStatus(MediaPlaySDKReturnCode.VIDEO_PARAM_UPDATE,
                    getVideoParam());
        }
        changingCamera = false;
        return 0;
    }

    public void closeCamera() {
        CustomLog.d(TAG, "SMSDKControl::closeCamera() 关闭摄像头");
        setCameraOpenStatus(false);
    }

    public void release() {
        CustomLog.d(TAG, "卸载MediaPlaySDK");

        if (mUSBMonitor != null) {
            mUSBMonitor.destroy();
            mUSBMonitor = null;
        }

        if (setCpuTimer != null) {
            setCpuTimer.cancel();
            setCpuTimer = null;
        }
        if (setCpuTimerTask != null) {
            setCpuTimerTask.cancel();
            setCpuTimerTask = null;
        }
        if (mBigStreamList != null) {
            mBigStreamList.clear();
            mBigStreamList = null;
        }
        mMeetingInfo = null;
        CpuUsageUtil.getInstance().relase();

        mSurfaceView = null;
        mContext = null;
        mCameraIndex = -1;
        releaseCamera();
        SmSdkJNI.SM_SDK_Release();
        if (mStatus == StatusCode.INIT) {
            SmSdkJNI.SM_SDK_LeaveMeeting();

        }

        mStatus = StatusCode.RELEASE;

        String str = KeyEventConfig.UNINSTALL_MEDIA_MODEL + "_ok_" + mAccountId
                + "_卸载视频模块成功";
        KeyEventWrite.write(str);
    }

    public int publishCameraVideo(int resources, int syncId) {
        CustomLog.d(TAG, "SMSDKControl::publishCameraVideo() resources: "
                + resources + " | syncId: " + syncId + " | mCodecType: "
                + mCodecType);
        if (mCodecType == 0) {
            return publishCameraVideoSoft(resources, syncId);
        } else {
            return publishCameraVideoHard(resources, syncId);
        }
    }

    public int publishCameraVideoSoft(int resources, int syncId) {
        CustomLog.d(TAG, "SMSDKControl::publishCameraVideoSoft() resources: "
                + resources + " | syncId: " + syncId);
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::publishCameraVideoSoft() 未进行初始化，error. return -1");
            return -1;
        }

        int preViewWidth = getVideoParam().getCapWidth();
        int preViewHeight = getVideoParam().getCapHeight();
        int fps = getVideoParam().getCapFps();
        int encBitrate = getVideoParam().getEncBitrate();

        mVideoResourceId = resources;
        mVideoSyncId = syncId;

        mVideoStremId = SmSdkJNI.SM_SDK_PublishCameraVideo(
                SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SEND, resources,
                SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_CAMRA,
                SmSdkJNI.SMSDK_SOURCE_TYPE_DEVICE, null, null,
                SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK, preViewWidth,
                preViewHeight, fps, preViewWidth, preViewHeight, fps,
                encBitrate, SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
                SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264,
                SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_SOFT, videoUpFecDataCount,
                videoUpFecCheckCount, videoUpDataSendCount,
                videoUpCheckSendCount, videoUpDataRetransSendCount,
                videoUpCheckRetransSendCount, videoUpDataResendCount);
        CustomLog.d(TAG, "SMSDKControl::publishCameraVideoSoft() 返回streamId: "
                + mVideoStremId + " | " + SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SEND
                + " | " + resources + " | "
                + SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_CAMRA + " | "
                + SmSdkJNI.SMSDK_SOURCE_TYPE_DEVICE + " | " + "null | null | "
                + SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK + " | "
                + preViewWidth + " | " + preViewHeight + " | " + fps + " | "
                + preViewWidth + " | " + preViewHeight + " | " + fps + " | "
                + encBitrate + SmSdkJNI.SMSDK_CODEC_LEVEL_LOW + " | "
                + SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264 + " | "
                + SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_SOFT);
        return mVideoStremId;
    }

    public int publishCameraVideoHard(int resources, int syncId) {
        CustomLog.d(TAG, "SMSDKControl::publishCameraVideoHard() resources: "
                + resources + " | syncId: " + syncId);
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::publishCameraVideoHard() 未进行初始化，error. return -1");
            return -1;
        }

        int preViewWidth = getVideoParam().getCapWidth();
        int preViewHeight = getVideoParam().getCapHeight();
        int fps = getVideoParam().getCapFps();
        int encBitrate = getVideoParam().getEncBitrate();

//        int pollWidth = getVideoParam(999).getCapWidth();
//        int pollHeight = getVideoParam(999).getCapHeight();
//        int pollFps = getVideoParam(999).getCapFps();
//        int pollBitrate = getVideoParam(999).getEncBitrate();

        mVideoResourceId = resources;
        mVideoSyncId = syncId;

        mVideoStremId = SmSdkJNI.SM_SDK_PublishCameraVideo(
                SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SEND, resources,
                SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_CAMRA,
                SmSdkJNI.SMSDK_SOURCE_TYPE_DEVICE, null, null,
                SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK, preViewWidth,
                preViewHeight, fps, preViewWidth, preViewHeight, fps,
                encBitrate, SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
                SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264,
                SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_HARD, videoUpFecDataCount,
                videoUpFecCheckCount, videoUpDataSendCount,
                videoUpCheckSendCount, videoUpDataRetransSendCount,
                videoUpCheckRetransSendCount, videoUpDataResendCount);

//        CustomLog.d(TAG, "手机端硬编发布流,小流idle");
//        mVideoStremId = SmSdkJNI.SM_SDK_publishMultiStreamCameraVideo(
//                SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SENDMR, resources,
//                SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_CAMRA,
//                SmSdkJNI.SMSDK_SOURCE_TYPE_DEVICE, null, null,
//                SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK,
//                preViewWidth, preViewHeight, fps, preViewWidth,
//                preViewHeight, fps, encBitrate,
//                SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
//                SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264,
//                SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_HARD,
//                SmSdkJNI.SMSDK_STREAM_RUNNING, pollWidth, pollHeight,
//                pollFps, pollBitrate,
//                SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
//                SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264,
//                SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_HARD,
//                SmSdkJNI.SMSDK_STREAM_IDLE, videoUpFecDataCount,
//                videoUpFecCheckCount, videoUpDataSendCount,
//                videoUpCheckSendCount, videoUpDataRetransSendCount,
//                videoUpCheckRetransSendCount, videoUpDataResendCount);

        CustomLog.d(TAG, "SMSDKControl::publishCameraVideoHard() 返回streamId: " + mVideoStremId);

        if (mLocalSurfaceCallBack != null) {
            mLocalSurfaceCallBack.setValues(mVideoStremId);
        }
        // gexy add in 2018-11-19 16:20:18
        if (mSurfaceView != null) {
            CustomLog.d(TAG, "publishCameraVideoHard,mSurfaceView!=null");
            SurfaceHolder holder = mSurfaceView.getHolder();

            if (holder.getSurface().isValid()) {
                CustomLog.d(TAG, "publishCameraVideoHard:SM_SDK_SetDisplayWindow");
                int result = SmSdkJNI.SM_SDK_SetDisplayWindow(mVideoStremId,
                        (Object) holder.getSurface(), FILL_MODE_RAWRATIO_WITHBLACK,
                        FILL_MODE_RAWRATIO_WITHBLACK);
                if (result == 0) {
                    CustomLog
                            .d(TAG,
                                    "publishCameraVideoHard:TYPE_VIDEO,SM_SDK_SetDisplayWindow success");

                    mSurfTex = (SurfaceTexture) SmSdkJNI
                            .SM_SDK_GetDisplayWindow(mVideoStremId);
                    CustomLog.d(TAG, "publishCameraVideoHard startCameraPreview");
                    startCameraPreview(holder);
                } else {
                    CustomLog.e(TAG,
                            "publishCameraVideoHard:TYPE_VIDEO,SM_SDK_SetDisplayWindow fail, result:"
                                    + result);
                }
            }
        } else {
            CustomLog.d(TAG, "publishCameraVideoHard,mSurfaceView==null");
        }

        return mVideoStremId;
    }

    public int publishDocVideo(int resourceID, int width, int height,
                               int vedioRate, int syncId, int filmFormat) {
        CustomLog.d(TAG, "SMSDKControl::publishDocVideo() resourceID: "
                + resourceID + " | width: " + width + " | height: " + height
                + " | vedioRate:" + vedioRate + " | syncId: " + syncId);
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::publishDocVideo() 未进行初始化，error. return -1");
            return -1;
        }
        int fps = getVideoParam().getCapFps();
        int streamId = SmSdkJNI.SM_SDK_PublishRemoteWinCaptureVedio(
                SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SEND, resourceID,
                SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_DOC,
                SmSdkJNI.SMSDK_SOURCE_TYPE_CALLBACK, null, null,
                SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK, width, height,
                fps, width, height, fps, vedioRate,
                SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
                SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264, filmFormat,
                DocVideoUpFecDataCount, DocVideoUpFecCheckCount,
                DocVideoUpDataSendCount, DocVideoUpCheckSendCount,
                DocVideoUpDataRetransSendCount,
                DocVideoUpCheckRetransSendCount, DocVideoUpDataResendCount);

        CustomLog.d(TAG, "SMSDKControl::publishDocVideo() 返回streamId: "
                + streamId + SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SEND + " | "
                + resourceID + " | " + SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_DOC
                + " | " + SmSdkJNI.SMSDK_SOURCE_TYPE_CALLBACK + " | "
                + "null,null," + SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK
                + " | " + width + " | " + height + " | " + fps + " | " + width
                + " | " + height + "," + fps + " |  " + vedioRate
                + SmSdkJNI.SMSDK_CODEC_LEVEL_LOW + " | "
                + SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264 + " | "
                + SmSdkJNI.SMSDK_CODEC_LEVEL_LOW + " | " + filmFormat);

        return streamId;
    }

    public int publishMicAudio(int resourceID, int syncId) {
        CustomLog.d(TAG, "SMSDKControl::publishMicAudio() syncId: " + syncId);
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::publishMicAudio() 未进行初始化，error. return -1");
            return -1;
        }
        int streamId = SmSdkJNI.SM_SDK_PublishMicAudio(
                SmSdkJNI.SMSDK_STREAM_TYPE_AUDIO_SEND, resourceID,
                SmSdkJNI.SMSDK_MEDIA_TYPE_AUDIO_MICPHONE,
                SmSdkJNI.SMSDK_SOURCE_TYPE_DEVICE, null, 1, 8000, 1, 16, 8000,
                1, 16, 64, SmSdkJNI.SMSDK_AUDIO_CODEC_SPEEX,
                audioUpFecDataCount, audioUpFecCheckCount,
                audioUpDataSendCount, audioUpCheckSendCount,
                audioUpDataRetransSendCount, audioUpCheckRetransSendCount,
                audioUpDataResendCount);

        CustomLog.d(TAG, "SMSDKControl::publishMicAudio() 返回streamId: "
                + streamId);
        return streamId;
    }

    // 还未开始渲染的时候调用，媒体模块需要保证没问题，并且是有效的
    public int remoteSpeakerCloseVideo(int streamId) {
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::remoteSpeakerCloseVideo() 未进行初始化，error. return -1");
            return -1;
        }

        SmSdkJNI.SM_SDK_RemoteSpeakerCloseVideo(streamId);
        return 0;
    }

    public int remoteSpeakerOpenVideo(int streamId) {
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::remoteSpeakerOpenVideo() 未进行初始化，error. return -1");
            return -1;
        }
        SmSdkJNI.SM_SDK_RemoteSpeakerOpenVideo(streamId);
        return 0;
    }

    /**
     * @param mode SM_SDK_SPEAKMODE_MASTER = 0; //主讲模式,默认：640*480 15帧 300码率
     *             SM_SDK_SPEAKMODE_NORMAL = 1; //普通模式 默认：320*240 10帧 150码率
     */
    public int setSpeakMode(int mode) {
        return 0;
    }

    public int startRenderLocalVedio(final int streamId, final int mediaType,
                                     final SurfaceView surfaceView) {
        CustomLog.d(TAG, "startRenderLocalVedio mCodecType=" + mCodecType);
        if (mCodecType == 0) {
            return startRenderLocalVedioSoft(streamId, mediaType, surfaceView);
        } else {
            return startRenderLocalVedioHard(streamId, mediaType, surfaceView);
        }
    }

    /**
     * @param streamId
     * @param surfaceView
     */
    public int startRenderLocalVedioSoft(final int streamId,
                                         final int mediaType, final SurfaceView surfaceView) {
        // 需要区分mediaType 视频还是doc类型，区别对待,返回结果通过回调
        CustomLog.d(TAG, "SMSDKControl::startRenderLocalVedioSoft() streamId: "
                + streamId);
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::startRenderLocalVedioSoft() 未进行初始化，error. return -1");
            return -1;
        }

        // 如果是视频窗口，需要保存streamId 和 surfaceView 对象
        if (mediaType == MediaType.TYPE_VIDEO) {
            mSurfaceView = surfaceView;
            mVideoStremId = streamId;
        }

        SurfaceHolder holder = surfaceView.getHolder();

        LocalHolder = holder;

        if (mLocalSurfaceCallBack != null) {
            CustomLog.d(TAG, "startRenderLocalVedioSoft remove previous callback");
            holder.removeCallback(mLocalSurfaceCallBack);
        }
        mLocalSurfaceCallBack = new ExtendSurfaceHolderCallBack(
                new SurfaceViewCallBack() {
                    @Override
                    public void onSurfaceViewCreated(SurfaceHolder holder,
                                                     int streamId) {
                        CustomLog.d(TAG,
                                "SMSDKControl startRenderLocalVedioSoft::onSurfaceViewCreated() streamId: "
                                        + streamId + " | mediaType: "
                                        + mediaType);
                        if (mediaType == MediaType.TYPE_VIDEO) {
                            CustomLog.d(TAG,
                                    " startRenderLocalVedioSoft onSurfaceViewCreated,isCameraOpen:"
                                            + String.valueOf(isCameraOpen));
                            if (isCameraOpen || mPollVideoStreamId > 0) {
                                startCameraPreview(holder);
                            } else {
                                CustomLog
                                        .d(TAG,
                                                "SMSDKControl startRenderLocalVedioSoft::onSurfaceViewCreated()摄像头关闭状态，不做打开自预览操作");
                            }
                            changingCamera = false;
                        } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
                            SmSdkJNI.SM_SDK_StartLocalVideoRender(streamId,
                                    (Object) holder.getSurface(),
                                    FILL_MODE_RAWRATIO_WITHBLACK,
                                    FILL_MODE_RAWRATIO_WITHBLACK);
                        }
                    }

                    @Override
                    public void onSurfaceChanged(SurfaceHolder holder,
                                                 int format, int width, int height, int streamId) {
                        CustomLog.d(TAG,
                                "SMSDKControl startRenderLocalVedioSoft::onSurfaceChanged() format: "
                                        + format + " | width: " + width
                                        + " | height:" + height);
                    }

                    @Override
                    public void onSurfaceDestroyed(SurfaceHolder holder,
                                                   int streamId) {
                        CustomLog.d(TAG,
                                "SMSDKControl startRenderLocalVedioSoft::onSurfaceViewDestroyed() streamId: "
                                        + streamId + " | mediaType: "
                                        + mediaType);
                        SmSdkJNI.SM_SDK_StopLocalVideoRender(streamId);
                        if (mediaType == MediaType.TYPE_VIDEO) {
                            if (mButelopensdk.getSpeakerInfoById(mAccountId) != null) {
                                changingCamera = true;
                            }
                            releaseCamera();
                        }
                    }
                });
        mLocalSurfaceCallBack.setValues(streamId);
        holder.addCallback(mLocalSurfaceCallBack);
        holder.setFormat(PixelFormat.TRANSLUCENT);
        CustomLog
                .d(TAG,
                        "startRenderLocalVedioSoft holder.setFormat(PixelFormat.TRANSLUCENT)");

        return 0;
    }

    /**
     * @param streamId
     * @param surfaceView
     */
    public int startRenderLocalVedioHard(final int streamId,
                                         final int mediaType, final SurfaceView surfaceView) {
        // 需要区分mediaType 视频还是doc类型，区别对待,返回结果通过回调
        CustomLog.d(TAG, "SMSDKControl::startRenderLocalVedioHard() streamId: "
                + streamId);
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::startRenderLocalVedioHard() 未进行初始化，error. return -1");
            return -1;
        }

        // 如果是视频窗口，需要保存streamId 和 surfaceView 对象
        if (mediaType == MediaType.TYPE_VIDEO) {
            mSurfaceView = surfaceView;
            mVideoStremId = streamId;
        }

        SurfaceHolder holder = surfaceView.getHolder();

        LocalHolder = holder;

        if (mLocalSurfaceCallBack != null) {
            CustomLog.d(TAG, "startRenderLocalVedioHard remove previous callback");
            holder.removeCallback(mLocalSurfaceCallBack);
            mLocalSurfaceCallBack = null;
        }
        if (mLocalSurfaceCallBack == null) {
            mLocalSurfaceCallBack = new ExtendSurfaceHolderCallBack(
                    new SurfaceViewCallBack() {
                        @Override
                        public void onSurfaceViewCreated(SurfaceHolder holder,
                                                         int streamId) {
                            CustomLog.d(TAG,
                                    "SMSDKControl startRenderLocalVedioHard::onSurfaceViewCreated() streamId: "
                                            + streamId + " | mediaType: "
                                            + mediaType);
                            if (mediaType == MediaType.TYPE_VIDEO) {
                                CustomLog.d(TAG,
                                        "startRenderLocalVedioHard onSurfaceViewCreated,isCameraOpen:"
                                                + String.valueOf(isCameraOpen));
                                if (isCameraOpen || mPollVideoStreamId > 0) {

                                    // gexy add in 2018-11-19 16:21:41
                                    CustomLog
                                            .d(TAG,
                                                    "startRenderLocalVedioHard:SM_SDK_SetDisplayWindow");
                                    int result = SmSdkJNI.SM_SDK_SetDisplayWindow(
                                            streamId, (Object) holder.getSurface(),
                                            FILL_MODE_RAWRATIO_WITHBLACK,
                                            FILL_MODE_RAWRATIO_WITHBLACK);
                                    if (result == 0) {
                                        CustomLog
                                                .d(TAG,
                                                        "startRenderLocalVedioHard:TYPE_VIDEO,SM_SDK_SetDisplayWindow success");
                                    } else {
                                        CustomLog.e(TAG,
                                                "startRenderLocalVedioHard:TYPE_VIDEO,SM_SDK_SetDisplayWindow fail, result:"
                                                        + result);
                                    }
                                    mSurfTex = (SurfaceTexture) SmSdkJNI
                                            .SM_SDK_GetDisplayWindow(streamId);

                                    CustomLog
                                            .d(TAG,
                                                    "SMSDKControl startRenderLocalVedioHard::onSurfaceViewCreated() startCameraPreview");
                                    startCameraPreview(holder);

                                } else {
                                    CustomLog
                                            .d(TAG,
                                                    "SMSDKControl startRenderLocalVedioHard::onSurfaceViewCreated()摄像头关闭状态，不做打开自预览操作");
                                }
                                changingCamera = false;
                            } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
                                SmSdkJNI.SM_SDK_StartLocalVideoRender(streamId,
                                        (Object) holder.getSurface(),
                                        FILL_MODE_RAWRATIO_WITHBLACK,
                                        FILL_MODE_RAWRATIO_WITHBLACK);
                            }
                        }

                        @Override
                        public void onSurfaceChanged(SurfaceHolder holder,
                                                     int format, int width, int height, int streamId) {
                            CustomLog.d(TAG,
                                    "SMSDKControl startRenderLocalVedioHard::onSurfaceChanged() format: "
                                            + format + " | width: " + width
                                            + " | height:" + height);
                        }

                        @Override
                        public void onSurfaceDestroyed(SurfaceHolder holder,
                                                       int streamId) {
                            CustomLog.d(TAG,
                                    "SMSDKControl startRenderLocalVedioHard::onSurfaceViewDestroyed() streamId: "
                                            + streamId + " | mediaType: "
                                            + mediaType);
                            SmSdkJNI.SM_SDK_StopLocalVideoRender(streamId);
                            if (mediaType == MediaType.TYPE_VIDEO) {
                                if (mButelopensdk.getSpeakerInfoById(mAccountId) != null) {
                                    changingCamera = true;
                                }
                                releaseCamera();
                            }
                        }
                    });

        }

        if (mLocalSurfaceCallBack != null) {
            CustomLog.d(TAG, "startRenderLocalVedioHard change mVideoStremId=" + mVideoStremId);
            mLocalSurfaceCallBack.setValues(mVideoStremId);
        }
        holder.addCallback(mLocalSurfaceCallBack);
//            holder.setFormat(PixelFormat.TRANSLUCENT);

        if (mCodecType != 0) {
            CustomLog
                    .d(TAG,
                            "startRenderLocalVedioHard setRotation hardCodec setVideoOrientation mPushFrameOrientation="
                                    + mPushFrameOrientation);
            SmSdkJNI.SM_SDK_SetVideoOrientation(mVideoStremId,
                    mPushFrameOrientation);
        }

        if (holder.getSurface().isValid()) {
            CustomLog.d(TAG, "startRenderLocalVedioHard Surface.isValid() true: "
                    + holder.getSurface().toString());
            CustomLog.d(TAG, "startRenderLocalVedioHard SM_SDK_SetDisplayWindow");
            int result = SmSdkJNI.SM_SDK_SetDisplayWindow(mVideoStremId,
                    (Object) holder.getSurface(), FILL_MODE_RAWRATIO_WITHBLACK,
                    FILL_MODE_RAWRATIO_WITHBLACK);
            if (result == 0) {
                CustomLog
                        .d(TAG,
                                "startRenderLocalVedioHar TYPE_VIDEO,SM_SDK_SetDisplayWindow success");
            } else {
                CustomLog.e(TAG,
                        "startRenderLocalVedioHard TYPE_VIDEO,SM_SDK_SetDisplayWindow fail, result:"
                                + result);
            }
            if (mediaType == MediaType.TYPE_VIDEO) {
                mSurfTex = (SurfaceTexture) SmSdkJNI
                        .SM_SDK_GetDisplayWindow(streamId);
                startCameraPreview(holder);
            } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
                SmSdkJNI.SM_SDK_StartLocalVideoRender(streamId,
                        (Object) holder.getSurface(),
                        FILL_MODE_RAWRATIO_WITHBLACK,
                        FILL_MODE_RAWRATIO_WITHBLACK);
            }
        } else {
            CustomLog.d(TAG, "startRenderLocalVedioHard Surface.isValid() false");
        }
        return 0;
    }

    /**
     * @param streamId
     * @param surfaceView
     */
    public int startRenderRemoteVedio(final int streamId, final int mediaType,
                                      SurfaceView surfaceView) {
        CustomLog.d(TAG, "SMSDKControl::startRenderRemoteVedio() streamId: "
                + streamId);
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::startRenderRemoteVedio() 未进行初始化，error. return -1");
            return -1;
        }
        SurfaceHolder holder = surfaceView.getHolder();

        ExtendSurfaceHolderCallBack mExtendSurfaceHolderCallBack = new ExtendSurfaceHolderCallBack(
                new SurfaceViewCallBack() {

                    @Override
                    public void onSurfaceViewCreated(SurfaceHolder holder,
                                                     int streamId) {
                        CustomLog.d(TAG,
                                "startRenderRemoteVedio SMSDKControl::onSurfaceViewCreated() streamId: "
                                        + streamId + " surface: "
                                        + holder.getSurface().toString());
                        if (mediaType == MediaType.TYPE_VIDEO) {
                            SmSdkJNI.SM_SDK_StartRemoteVideoRender(streamId,
                                    (Object) holder.getSurface(),
                                    FILL_MODE_RAWRATIO_ZOOM_NOBLACK,
                                    FILL_MODE_RAWRATIO_ZOOM_NOBLACK);
                        } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
                            SmSdkJNI.SM_SDK_StartRemoteVideoRender(streamId,
                                    (Object) holder.getSurface(),
                                    FILL_MODE_RAWRATIO_WITHBLACK,
                                    FILL_MODE_RAWRATIO_WITHBLACK);
                        }

                        int isDocFlag = 0;
                        String accountId = mButelopensdk
                                .getAccountIdByVideoStreamId(streamId);
                        if (accountId == null) {
                            isDocFlag = 1;
                            accountId = mButelopensdk
                                    .getAccountIdByDocVideoStreamId(streamId);
                        }
                        try {
                            JSONObject jsonObject = new JSONObject();

                            jsonObject.put("accountId", accountId);
                            jsonObject.put("isDocFlag", isDocFlag);
                            jsonObject.put("state", 1); // 0：destroy 1：create

                            if (mSMSDKListener != null && accountId != null) {
                                mSMSDKListener
                                        .onStatus(
                                                MediaPlaySDKReturnCode.SURFACEVIEW_STATE_CHANGE,
                                                jsonObject.toString());
                            }
                        } catch (JSONException e) {
                            CustomLog.d(TAG,
                                    "startRenderRemoteVedio e=" + e.toString());
                        }
                    }

                    @Override
                    public void onSurfaceChanged(SurfaceHolder holder,
                                                 int format, int width, int height, int streamId) {
                    }

                    @Override
                    public void onSurfaceDestroyed(SurfaceHolder holder,
                                                   int streamId) {
                        CustomLog.d(TAG,
                                "SMSDKControl::onSurfaceDestroyed() streamId: "
                                        + streamId);
                        int isDocFlag = 0;
                        String accountId = mButelopensdk
                                .getAccountIdByVideoStreamId(streamId);
                        if (accountId == null) {
                            isDocFlag = 1;
                            accountId = mButelopensdk
                                    .getAccountIdByDocVideoStreamId(streamId);
                        }
                        try {
                            JSONObject jsonObject = new JSONObject();

                            jsonObject.put("accountId", accountId);
                            jsonObject.put("isDocFlag", isDocFlag);
                            jsonObject.put("state", 0); // 0：destroy 1：create

                            if (mSMSDKListener != null && accountId != null) {
                                mSMSDKListener
                                        .onStatus(
                                                MediaPlaySDKReturnCode.SURFACEVIEW_STATE_CHANGE,
                                                jsonObject.toString());
                            }
                        } catch (JSONException e) {
                            CustomLog.d(TAG,
                                    "startRenderRemoteVedio e=" + e.toString());
                        }

                        SmSdkJNI.SM_SDK_StopRemoteVideoRender(streamId);
                    }
                });
        mExtendSurfaceHolderCallBack.setValues(streamId);
        surfaceView.getHolder().addCallback(mExtendSurfaceHolderCallBack);
        holder.setFormat(PixelFormat.TRANSLUCENT);
        CustomLog.d(TAG, "holder.setFormat(PixelFormat.TRANSLUCENT)");
        CustomLog
                .d(TAG,
                        "startRenderRemoteVedio SMSDKControl::onSurfaceViewCreated() laoding.........");
        if (surfaceView.getHolder().getSurface().isValid()) {
            CustomLog.d(TAG,
                    "surfaceView.getHolder().getSurface().isValid() true: "
                            + holder.getSurface().toString());
            if (mediaType == MediaType.TYPE_VIDEO) {
                return SmSdkJNI.SM_SDK_StartRemoteVideoRender(streamId,
                        (Object) holder.getSurface(),
                        FILL_MODE_RAWRATIO_ZOOM_NOBLACK,
                        FILL_MODE_RAWRATIO_ZOOM_NOBLACK);
            } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
                return SmSdkJNI.SM_SDK_StartRemoteVideoRender(streamId,
                        (Object) holder.getSurface(),
                        FILL_MODE_RAWRATIO_WITHBLACK,
                        FILL_MODE_RAWRATIO_WITHBLACK);
            }
        }

        return 0;
    }

    /**
     * @param streamId
     */
    public int stopRenderLocalVedio(int streamId, int mediaType) {
        CustomLog.d(TAG, "SMSDKControl::stopRenderLocalVedio() streamId: "
                + streamId);
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::stopRenderLocalVedio() 未进行初始化，error. return -1");
            return -1;
        }

        if (mediaType == MediaType.TYPE_VIDEO) {
//            if (mCodecType == 0) {
            mSurfaceView = null;
            stopCameraPreview();
            releaseCamera();
//            }
        } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
            return SmSdkJNI.SM_SDK_StopLocalVideoRender(streamId);
        }

        return 0;
    }

    /**
     * @param streamId
     */
    public int stopRenderRemoteVideo(int streamId) {
        CustomLog.d(TAG, "SMSDKControl::stopRenderRemoteVideo() streamId: "
                + streamId);
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::stopRenderRemoteVideo() 未进行初始化，error. return -1");
            return -1;
        }
        return SmSdkJNI.SM_SDK_StopRemoteVideoRender(streamId);
    }

    public int subscribe(String userID, int mediaType, int resourceID,
                         int syncId) {
        CustomLog.d(TAG, "SMSDKControl::subscribe() userID: " + userID
                + " | meidaType: " + mediaType + " | resourceID: " + resourceID
                + " | syncId: " + syncId);
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::subscribe() 未进行初始化，error. return -1");
            return -1;
        }
        switch (mediaType) {
            case MediaType.TYPE_VIDEO:
                int videoStreamId = SmSdkJNI.SM_SDK_SubscribeVideo(userID,
                        SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_RECV, resourceID,
                        SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_CAMRA, syncId, null,
                        SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK,
                        videoDownFecDataCount, videoDownFecCheckCount,
                        videoDownDataSendCount, videoDownCheckSendCount,
                        videoDownDataRetransSendCount,
                        videoDownCheckRetransSendCount, videoDownDataResendCount,
                        videoDelayTime);

                if (mIsBigStream) {
                    CustomLog
                            .d(TAG,
                                    "SM_SDK_SubscribeVideo，设置大流，accountId:"
                                            + userID + " streamId:"
                                            + String.valueOf(videoStreamId));
                    int result = SmSdkJNI.SM_SDK_SetMRDisplayMode(videoStreamId,
                            SmSdkJNI.SM_SDK_displayWin_adaption_mode,
                            SmSdkJNI.SM_SDK_displayWin_big_wintype);
                    CustomLog.d(
                            TAG,
                            "SM_SDK_SetMRDisplayMode，result:"
                                    + String.valueOf(result));
                }

                return videoStreamId;
            case MediaType.TYPE_DOC_VIDEO:
                int docVideoStreamId = SmSdkJNI.SM_SDK_SubscribeVideo(userID,
                        SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_RECV, resourceID,
                        SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_DOC, syncId, null,
                        SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK,
                        DocVideoDownFecDataCount, DocVideoDownFecCheckCount,
                        DocVideoDownDataSendCount, DocVideoDownCheckSendCount,
                        DocVideoDownDataRetransSendCount,
                        DocVideoDownCheckRetransSendCount,
                        DocVideoDownDataResendCount, DocVideoDelayTime);

                if (mIsBigStream) {
                    int result = SmSdkJNI.SM_SDK_SetMRDisplayMode(docVideoStreamId,
                            SmSdkJNI.SM_SDK_displayWin_adaption_mode,
                            SmSdkJNI.SM_SDK_displayWin_big_wintype);
                    CustomLog.d(
                            TAG,
                            "SM_SDK_SetMRDisplayMode，result:"
                                    + String.valueOf(result));
                }

                return docVideoStreamId;
            case MediaType.TYPE_AUDIO:
                return SmSdkJNI.SM_SDK_SubscribeAudio(userID,
                        SmSdkJNI.SMSDK_STREAM_TYPE_AUDIO_RECV, resourceID,
                        SmSdkJNI.SMSDK_MEDIA_TYPE_AUDIO_MICPHONE, syncId, 1, null,
                        audioDownFecDataCount, audioDownFecCheckCount,
                        audioDownDataSendCount, audioDownCheckSendCount,
                        audioDownDataRetransSendCount,
                        audioDownCheckRetransSendCount, audioDownDataResendCount,
                        audioDelayTime);
            default:
                CustomLog.e(TAG, "not support mediaType format!");
                return -1;
        }

    }

    public int subscribe2(String userID, int mediaType, int resourceID,
                          int syncId, int type) {
        CustomLog.d(TAG, "SMSDKControl::subscribe2() userID: " + userID
                + " | meidaType: " + mediaType + " | resourceID: " + resourceID
                + " | syncId: " + syncId);
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::subscribe() 未进行初始化，error. return -1");
            return -1;
        }

        if (type == ShareType.SHARETYPE_WHITE_BOARD) {
            return SmSdkJNI.SM_SDK_SubscribeVideo2(userID,
                    SmSdkJNI.SMSDK_STREAM_TYPE_WHITEBOARD_RECV, resourceID,
                    SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_DOC, syncId, null,
                    SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK);
        } else {
            return SmSdkJNI.SM_SDK_SubscribeVideo2(userID,
                    SmSdkJNI.SMSDK_STREAM_TYPE_VIRTUALLAYER_RECV, resourceID,
                    SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_DOC, syncId, null,
                    SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK);
        }

    }

    /**
     * @param srtreamid sdk流id
     * @return 大于0 – StreamID；其他 – 错误
     * @brief 获取framework层的StreamID
     */
    public int getFrameWorkStreamID(int streamid) {
        CustomLog.d(TAG, "SMSDKControl::getFrameWorkStreamID() srtreamid: "
                + streamid);
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::subscribe() 未进行初始化，error. return -1");
            return -1;
        }
        return SmSdkJNI.SM_SDK_getFrameWorkStreamID(streamid);
    }

    /**
     * @param streamId
     */
    public int unSucscribe(int streamId) {
        CustomLog.d(TAG, "SMSDKControl::unSucscribe() streamId: " + streamId);
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::unSucscribe() 未进行初始化，error. return -1");
            return -1;
        }
        return SmSdkJNI.SM_SDK_UnSubscribe(streamId);
    }

    /**
     * @param streamId
     */
    public int continueSubscribe(int streamId, int mediaType) {
        CustomLog.d(TAG, "SMSDKControl::continueSubscribe() streamId: "
                + streamId + " mediaType=" + mediaType);
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::continueSubscribe() 未进行初始化，error. return -1");
            return -1;
        }

        int ret = SmSdkJNI.SM_SDK_ContinueRecv(streamId);

        if (mIsBigStream) {
            if (mediaType == MediaType.TYPE_VIDEO
                    || mediaType == MediaType.TYPE_DOC_VIDEO) {
                CustomLog.d(
                        TAG,
                        "continueSubscribe，设置大流 streamId:"
                                + String.valueOf(streamId));
                int result = SmSdkJNI.SM_SDK_SetMRDisplayMode(streamId,
                        SmSdkJNI.SM_SDK_displayWin_adaption_mode,
                        SmSdkJNI.SM_SDK_displayWin_big_wintype);
                CustomLog.d(
                        TAG,
                        "SM_SDK_SetMRDisplayMode，result:"
                                + String.valueOf(result));
            }
        }
        return ret;
    }

    /**
     * @param streamId
     */
    public int pauseSubscribe(int streamId) {
        CustomLog
                .d(TAG, "SMSDKControl::pauseSubscribe() streamId: " + streamId);
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::pauseSubscribe() 未进行初始化，error. return -1");
            return -1;
        }
        return SmSdkJNI.SM_SDK_PauseRecv(streamId);
    }

    public int unPublish(int streamId) {
        CustomLog.d(TAG, "SMSDKControl::unPublish() streamId: " + streamId);
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::unPublish() 未进行初始化，error. return -1");
            return -1;
        }

        int ret = SmSdkJNI.SM_SDK_UnPublish(streamId);
        CustomLog.d(TAG, "SMSDKControl::unPublish () result:" + ret);

        if (mPollVideoStreamId == streamId) {
            CustomLog.d(TAG, "取消发布poll stream");
            mPollVideoStreamId = -1;
        }
        return ret;
    }

    public void setSpeakStatus(boolean isSpeaking) {
        CustomLog
                .d(TAG, "SMSDKControl::setSpeakStatus() 设置是否发言： " + isSpeaking);
        if (isSpeaking) {
            setCameraOpenStatus(true);// 发言时摄像头默认开启
        }

        this.isSpeaking = isSpeaking;
    }

    public void setSpeakStatus(boolean isSpeaking, boolean isOpenCamera) {
        CustomLog.d(TAG, "SMSDKControl::setSpeakStatus() 设置是否发言： " + isSpeaking
                + " isOpenCamera:" + isOpenCamera);
        if (isSpeaking) {
            if (isOpenCamera) {
                setCameraOpenStatus(true);// 发言时摄像头默认开启
            } else {
                setCameraOpenStatus(false);
            }
        }
        this.isSpeaking = isSpeaking;
    }

    // 本地打开语音模式
    public int audioModeOn() {
        return 0;

    }

    // 本地关闭语音模式
    public int audioModeOff() {
        return 0;
    }

    /**
     * 获取所有的qos信息
     *
     * @return 返回所有流qos信息
     */
    public String getStreamQosInfo() {
        CustomLog.d(TAG, "SMSDKControl::getStreamQosInfo() ");
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::getStreamQosInfo() 未进行初始化，error. return -1");
            return "";
        }
        return SmSdkJNI.GetStreamQosInfo();
    }

    /**
     * 获取渲染视频信息
     *
     * @return VideoParameter
     */
    public VideoParameter getRenderVideoInfo(int streamId) {
        // {"frameRate":1,"vedioRate":2,"vedioWidth":3,"vedioHeight":4}
        JSONObject resultJson;
        VideoParameter videoparam = new VideoParameter();
        try {
            String renderQos = SmSdkJNI.SM_SDK_GetRenderQosInfo(streamId);
            resultJson = new JSONObject(renderQos);

            videoparam.setCapFps(resultJson.optInt("frameRate"));
            videoparam.setEncBitrate(resultJson.optInt("vedioRate"));
            videoparam.setCapWidth(resultJson.optInt("vedioWidth"));
            videoparam.setCapHeight(resultJson.optInt("vedioHeight"));
        } catch (JSONException e) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::getRenderVideoInfo() error! resultJson == null");
        }

        return videoparam;
    }

    /****************** Camera Operation ***************************/
    // 初始化摄像头，open操作. 根据设备类型选择需要优先打开的摄像头索引
    public int initCamera() {
        CustomLog.d(TAG, "SMSDKControl::initCamera()");

        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::initCamera() 未进行初始化，error. return -1");
            return -1;
        }
        if (mCameraIsOpen) {
            CustomLog.w(TAG, "SMSDKControl::initCamera() 摄像头已经初始化，不用进行初始化");
            return 0;
        }

        if (mSelectUvcCamera) {// 设置使用的是uvc摄像头
            recoverCameraHandler.removeMessages(0);
            if (mUSBMonitor == null) {
                // 注册USBMonitor
                mUSBMonitor = new USBMonitor(mContext, mOnDeviceConnectListener);
                synchronized (mSync) {
                    if (mUSBMonitor != null) {
                        mUSBMonitor.register();
                    }
                }
            } else {
                // 已经有USBMonitor，做onAttach中相同的处理
                if (mCamera != null) {
                    closeCamera();
                    releaseCamera(cameraTypeSystemCamera);
                }

                if (null != mUSBMonitor && null != mFilter
                        && null != mUSBMonitor.getDeviceList(mFilter.get(0))
                        && mUSBMonitor.getDeviceList(mFilter.get(0)).size() > 0) {
                    mUSBMonitor.requestPermission((UsbDevice) mUSBMonitor
                            .getDeviceList(mFilter.get(0)).get(0));
                    useUVCCamera = true;
                } else {
                    CustomLog.d(TAG, "getDeviceList.size==0");
                    if (mContext != null) {
                        Toast.makeText(mContext, "外接摄像头未连接", Toast.LENGTH_SHORT)
                                .show();
                    }
                }
            }
        }

        if (useUVCCamera || mSelectUvcCamera) {// 使用的是uvc摄像头，等待onConnect后打开uvc摄像头
            return 0;
        } else if (mCameraIndex == -1) { // 查找摄像头索引
            List<Integer> indexList = getCameraIndexList(mDeviceType);
            for (int i = 0; i < indexList.size(); i++) {
                int index = indexList.get(i);
                CustomLog.d(TAG, "尝试打开摄像头，index: " + index);
                try {
                    mCamera = Camera.open(index);
                } catch (RuntimeException ex) {
                    CustomLog.e(TAG, "开启摄像头失败! 0" + ex.getMessage());
                    continue;
                }
                mCameraIndex = index;
                break;
            }
        } else {
            try {
                if (!changingCamera && isCameraOpen == true) {
                    if (mDeviceType.equalsIgnoreCase(DeviceType.DEVICE_T8)) {
                        mCameraIndex = getCameraIndexList(mDeviceType).get(0);
                    } else {
                        mCameraIndex = getCameraIndexList(
                                DeviceType.DEVICE_PHONE).get(0);
                    }
                }
                mCamera = Camera.open(mCameraIndex);
            } catch (RuntimeException ex) {
                CustomLog.e(TAG, "开启摄像头失败! 1" + ex.getMessage());
                return -1;
            }
        }
        if (mCameraIndex == -1) {
            CustomLog.e(TAG, "打开摄像头失败！ ");
            String str = KeyEventConfig.START_CAMERA + "_fail_" + mAccountId
                    + "_开启摄像头失败";
            KeyEventWrite.write(str);
            return -1;
        }

        CustomLog.d(TAG, "打开摄像头成功,使用摄像头index: " + mCameraIndex);

        mCameraIsOpen = true;
        String str = KeyEventConfig.START_CAMERA + "_ok_" + mAccountId
                + "_开启摄像头成功";
        KeyEventWrite.write(str);

        return 0;
    }

    public int releaseCamera() {
        return releaseCamera(cameraTypeBoth);
    }

    /**
     * 释放摄像头
     *
     * @return
     */
    public int releaseCamera(int camreaType) {
        CustomLog.d(TAG, "SMSDKControl::releaseCamera() 关闭摄像头");
        CustomLog.d(TAG, "camreaType:" + String.valueOf(camreaType));
        if (mStatus != StatusCode.INIT) {
            CustomLog.e(TAG,
                    "SMSDKControl::releaseCamera() 未进行初始化，error. return -1");
            return -1;
        }

        recoverCameraHandler.removeMessages(0);
        stopCameraPreview();
        try {

            if (mUVCCamera != null && camreaType != cameraTypeSystemCamera) {
                mUVCCamera.destroy();
                mUVCCamera = null;
                mCameraIsOpen = false;
            }

            if (mCamera != null && camreaType != cameraTypeUvcCamera) {
                mCamera.release();
                mCamera = null;
                mCameraIsOpen = false;
                String str = KeyEventConfig.STOP_PREVIEW + "_ok_" + mAccountId
                        + "_关闭摄像头成功";
                KeyEventWrite.write(str);
            }
        } catch (Exception e) {
            CustomLog.e(TAG, "关闭摄像头异常：" + e.toString());

            String str = KeyEventConfig.STOP_PREVIEW + "_fail_" + mAccountId
                    + "_关闭摄像头失败";
            KeyEventWrite.write(str);
        }
        // 摄像头资源已释放，如果mSurfaceView还在使用，说明现在是关闭摄像头状态，需要设置默认背景
        if (mSurfaceView != null && !changingCamera) {
            // mSurfaceView.setBackgroundResource(mCloseCameraGroundId);
            lockCanvas(mSurfaceView);
            // 利用surfaceView 生命周期,修复最后一帧闪现问题。但会引起在某些手机上采集摄像头预览失败问题？
            mSurfaceView.setVisibility(View.GONE);
            // 延迟开启，避免某些机型上立即操作会导致摄像头预览异常
            mSurfaceView.postDelayed(new Runnable() {
                @Override
                public void run() {
                    if (mSurfaceView != null) {
                        mSurfaceView.setVisibility(View.VISIBLE);
                    }
                }
            }, 100);
        }
        return 0;
    }

    public int startCameraPreview() {
        if (mSurfaceView != null) {
            CustomLog.d(TAG, "startCameraPreview,mSurfaceView != null");
            return startCameraPreview(mSurfaceView.getHolder());
        }
        CustomLog.d(TAG, "startCameraPreview fail,mSurfaceView == null");
        return -100;
    }

    @SuppressWarnings({"deprecation", "unchecked"})
    public int startCameraPreview(SurfaceHolder holder) {

        isWatchCamera = true;
        boolean mFirstInitCamera = false;

        recoverCameraHandler.removeMessages(0);
        recoverCameraHandler.sendEmptyMessageDelayed(0,
                RECOVER_CAMERA_DELAY_TIME);

        if (mCamera == null) {
            int ret = initCamera();
            if (ret == 0) {
                mFirstInitCamera = true;
            }
        }
        setCameraOpenStatus(true);

        CustomLog.d(TAG, "SMSDKControl::startCameraPreview() 打開摄像头");
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::startCameraPreview() 未进行初始化，error. return -1");
            return -1;
        }
        // 摄像头资源已使用，如果mSurfaceView还在使用，说明现在是打開摄像头状态，需要把背景清除
        if (mSurfaceView != null) {
            mSurfaceView.setBackgroundDrawable(null);// 为了4.0版本兼容性使用该api
        }
        if (mCamera != null || mSelectUvcCamera && holder != null && mFirstInitCamera) {
            try {
                if (useUVCCamera || mSelectUvcCamera) {
                    if (mUVCCamera != null) {
                        if (mCodecType == 0) {
                            mUVCCamera.setPreviewDisplay(holder);
                        } else {
                            // gexy modified in 2018-11-19 16:24:18
                            mUVCCamera.setPreviewTexture(mSurfTex);
                        }
                    } else {
                        if (mSurfaceView != null) {
                            CustomLog.d(TAG,
                                    "mUVCCamera==null,setBackgroundResource");
                            // mSurfaceView.setBackgroundResource(mCloseCameraGroundId);
                        }
                    }
                } else if (!mSelectUvcCamera) {
                    if (mCodecType == 0) {
                        mCamera.setPreviewDisplay(holder);
                    } else {
                        // gexy modified in 2018-11-19 16:24:32
                        mCamera.setPreviewTexture(mSurfTex);
                    }
                }
            } catch (IOException e) {
                CustomLog.e(TAG,
                        "SMSDKControl::startCameraPreview() setPreviewDisplay 设置预览异常！ "
                                + e.getMessage());
                String str = KeyEventConfig.START_PREVIEW + "_fail_"
                        + mAccountId + "_开启预览失败1";
                KeyEventWrite.write(str);
                return -1;
            }
        }

        if (mFirstInitCamera) {
            Parameters parameter = null;
            int preViewWidth = getVideoParam().getCapWidth();
            int preViewHeight = getVideoParam().getCapHeight();

            try {
                if (useUVCCamera) {
                    CustomLog.d(TAG, "startCameraPreview mUVCCamera");
                    if (null != getVideoParam()) {
                        CustomLog
                                .d(TAG,
                                        "startCameraPreview mUVCCamera null!=getVideoParam()");
                        preViewWidth = getVideoParam().getCapWidth();
                        preViewHeight = getVideoParam().getCapHeight();
                    } else {
                        CustomLog
                                .d(TAG,
                                        "startCameraPreview mUVCCamera null==getVideoParam()");
                    }

                } else if (!mSelectUvcCamera) {
                    CustomLog.d(TAG, "startCameraPreview Camera");
                    if (null != getVideoParam()) {
                        preViewWidth = getVideoParam().getCapWidth();
                        preViewHeight = getVideoParam().getCapHeight();
                        if (null != mCamera.getParameters()) {
                            parameter = mCamera.getParameters();
                            CustomLog.d(TAG, "parameter=" + parameter.toString());
                            for (Size size : parameter.getSupportedPreviewSizes()) {
                                CustomLog.d(TAG, "SupportedPreviewSizes: "
                                        + size.width + "x" + size.height);
                            }
                            parameter.setPreviewSize(preViewWidth, preViewHeight); // 设置采集宽、高

                            parameter.setPictureFormat(PixelFormat.JPEG);
                            List<String> focusModes = parameter
                                    .getSupportedFocusModes();

                            for (int i = 0; i < focusModes.size(); i++) {
                                CustomLog
                                        .d(TAG,
                                                "focusModes:"
                                                        + String.valueOf(focusModes
                                                        .get(i)));
                            }

                            if (focusModes
                                    .contains(Camera.Parameters.FOCUS_MODE_CONTINUOUS_VIDEO)) {
                                // 自动对焦功能
                                parameter
                                        .setFocusMode(Camera.Parameters.FOCUS_MODE_CONTINUOUS_VIDEO);
                            }

                            mCamera.setParameters(parameter);
                        } else {
                            CustomLog.d(TAG, "null==mCamera.getParameters()");
                        }
                    } else {
                        CustomLog.d(TAG, "null==getVideoParam()");
                    }
                }

            } catch (Exception e) {
                CustomLog.e(TAG, e.toString());
                String str = KeyEventConfig.START_PREVIEW + "_fail_" + mAccountId
                        + "_开启预览失败2";
                KeyEventWrite.write(str);
                return -2;
            }

            if (useUVCCamera || mSelectUvcCamera) {
                return 0;
            }

            int imageFormat = 0;
            if (null != parameter) {
                imageFormat = parameter.getPreviewFormat();
            }
            if (ImageFormat.NV21 != imageFormat) {
                CustomLog.i(TAG,
                        "start camera preview fail, it's image format not NV21");
                String str = KeyEventConfig.START_PREVIEW + "_fail_" + mAccountId
                        + "_开启预览失败";
                KeyEventWrite.write(str);
                return -3;
            }

            CustomLog.i(TAG, "preview image format:" + imageFormat);

            int bpp = ImageFormat.getBitsPerPixel(imageFormat);
            int bufferSize = preViewWidth * preViewHeight * bpp / 8;

            CustomLog.i(TAG, "size heigth:" + preViewHeight + ",size width:"
                    + preViewWidth + ", bpp:" + bpp + ",bufferSize:" + bufferSize);

            byte[] callbackBuffer = new byte[bufferSize];
            mCamera.addCallbackBuffer(callbackBuffer); // 设置buffer
            // Size,在某些手机上bufferSize过大会导致异常

            mCamera.setPreviewCallbackWithBuffer(mPreviewCallBack);

            setCameraDisplayOrientation(mContext, mCameraIndex, mCamera);

            try {
                mCamera.startPreview();
            } catch (Exception ex) {
                CustomLog.e(TAG, "startPreview failed: " + ex.getMessage());
                String str = KeyEventConfig.START_PREVIEW + "_fail_" + mAccountId
                        + "_开启预览失败";
                KeyEventWrite.write(str);
                return -4;
            }

            String str = KeyEventConfig.START_PREVIEW + "_ok_" + mAccountId
                    + "_开启预览成功";
            KeyEventWrite.write(str);
        }

        return 0;
    }

    private void getCameraOrientation() {
        // mContext, mCameraIndex, mCamera
        android.hardware.Camera.CameraInfo info = new android.hardware.Camera.CameraInfo();
        android.hardware.Camera.getCameraInfo(mCameraIndex, info);
        int rotation = ((Activity) mContext).getWindowManager()
                .getDefaultDisplay().getRotation();
        int degrees = 0;
        switch (rotation) {
            case Surface.ROTATION_0:
                degrees = 0;
                break;
            case Surface.ROTATION_90:
                degrees = 90;
                break;
            case Surface.ROTATION_180:
                degrees = 180;
                break;
            case Surface.ROTATION_270:
                degrees = 270;
                break;
        }

        int cameraOrientation;
        if (info.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
            cameraOrientation = (info.orientation + degrees) % 360;
            cameraOrientation = (360 - cameraOrientation) % 360; // compensate
            // the
            // mirror
        } else { // back-facing
            cameraOrientation = (info.orientation - degrees + 360) % 360;
        }

        CustomLog.d(TAG, "getCameraOrientation cameraOrientation="
                + cameraOrientation + " mPushFrameOrientation="
                + mPushFrameOrientation + " info.orientation="
                + info.orientation + " degrees=" + degrees);
    }

    public void setCameraDisplayOrientation(Context context, int cameraId,
                                            android.hardware.Camera camera) {
        android.hardware.Camera.CameraInfo info = new android.hardware.Camera.CameraInfo();
        android.hardware.Camera.getCameraInfo(cameraId, info);
        int rotation = ((Activity) context).getWindowManager()
                .getDefaultDisplay().getRotation();
        int degrees = 0;
        switch (rotation) {
            case Surface.ROTATION_0:
                degrees = 0;
                break;
            case Surface.ROTATION_90:
                degrees = 90;
                break;
            case Surface.ROTATION_180:
                degrees = 180;
                break;
            case Surface.ROTATION_270:
                degrees = 270;
                break;
        }

        if (info.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
            mCameraOrientation = (info.orientation + degrees) % 360;
            mCameraOrientation = (360 - mCameraOrientation) % 360; // compensate
            // the
            // mirror
        } else { // back-facing
            mCameraOrientation = (info.orientation - degrees + 360) % 360;
        }

        CustomLog.d(TAG,
                "SMSDKControl::setCameraDisplayOrientation() orientation: "
                        + mCameraOrientation);
        camera.setDisplayOrientation(mCameraOrientation);

    }

    public int stopCameraPreview() {
        CustomLog.d(TAG, "SMSDKControl::stopCameraPreview() 停止摄像头预览");
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::stopCameraPreview() 未进行初始化，error. return -1");
            return -1;
        }

        isWatchCamera = false;
        recoverCameraHandler.removeMessages(0);

        if (mCamera == null) {
            CustomLog
                    .w(TAG,
                            "SMSDKControl::stopCameraPreview() camera == null warning!不用停止");
            return 0;
        }
        if (useUVCCamera) {
            useUVCCamera = false;
            try {
                mUVCCamera.stopPreview();
                if (mCodecType != 0) {
                    // gexy add in 2018-11-19 16:25:54
                    CustomLog.d(TAG, "mCamera.setPreviewTexture(mSurfTex)");
                    mCamera.setPreviewTexture(mSurfTex);
                }

            } catch (Exception e) {
                CustomLog.e(TAG, e.toString());
            }
        } else if (!mSelectUvcCamera) {
            try {
                mCamera.stopPreview();

                if (mCodecType != 0) {
                    // gexy add in 2018-11-19 16:25:54
                    CustomLog.d(TAG, "mCamera.setPreviewTexture(mSurfTex)");
                    mCamera.setPreviewTexture(mSurfTex);
                }

                mCamera.setPreviewCallback(null);
                mCamera.setPreviewDisplay(null);
                mCamera.addCallbackBuffer(null);
                mCamera.setPreviewCallbackWithBuffer(null);
            } catch (Exception e) {
                CustomLog.e(TAG, e.toString());
            }
        }

        return 0;
    }

    /**
     * 设置本地渲染窗口的背景图片。调用场景：关闭摄像头、切换到语音模式
     *
     * @param backgroudId
     * @return
     */
    public int setLocalRenderWindowBg(int backgroudId) {
        if (mSurfaceView == null) {
            return -1;
        }

        mSurfaceView.setBackgroundResource(backgroudId);
        return 0;
    }

    /**
     * 清除本地渲染窗口的背景图片
     *
     * @return
     */
    @SuppressWarnings("deprecation")
    public int removeLocalRenderWindowBg() {
        if (mSurfaceView == null) {
            return -1;
        }

        mSurfaceView.setBackgroundDrawable(null); // 为了4.0版本兼容性使用该api
        return 0;
    }

    /**
     * 返回可用的摄像头索引号列表,根据设备类型优先选择前后置，比如手机将优先选择前置摄像头
     *
     * @param deviceType
     */
    private List<Integer> getCameraIndexList(String deviceType) {
        CustomLog.d(TAG,
                "SMSDKControl::getCameraIndexList() 获取摄像头列表。 deviceType: "
                        + deviceType);
        List<Integer> result = new ArrayList<Integer>();
        int counts = 0;
        Camera.CameraInfo cameraInfo = new Camera.CameraInfo();
        counts = Camera.getNumberOfCameras();
        if (counts <= 0) {
            CustomLog.e(TAG,
                    "SMSDKControl:getCameraIndexList() error! 无可用摄像头设备");
            result.add(-1);
            return result;
        }
        for (int camIndex = 0; camIndex < counts; camIndex++) {
            Camera.getCameraInfo(camIndex, cameraInfo);
            if (DeviceType.DEVICE_PHONE.equalsIgnoreCase(deviceType)) {
                if (cameraInfo.facing == CameraInfo.CAMERA_FACING_FRONT) {
                    result.add(0, camIndex);
                    continue;
                }
            }

            result.add(camIndex);
        }
        for (int i = 0; i < result.size(); i++) {
            CustomLog.d(TAG, "camera index:" + result.get(i));
        }
        return result;
    }

    private void handleRemoteNetworkPoor() {
        CustomLog.d(TAG, "SMSDKControl::handleRemoteNetworkPoor() 远端网络差！");
        if (mSMSDKListener != null) {
            mSMSDKListener.onStatus(
                    MediaPlaySDKReturnCode.MEDIAPLAY_REMOTE_NETWORK_POOR, null);
        }
    }

    private void handleLocalNetworkPoor() {
        CustomLog.d(TAG, "SMSDKControl::handleLocalNetworkPoor() 本地网络差！");
        if (mSMSDKListener != null) {
            mSMSDKListener.onStatus(
                    MediaPlaySDKReturnCode.MEDIAPLAY_LOCAL_NETWORK_POOR, null);
        }
    }

    private void handleAllNetworkPoor() {
        CustomLog.d(TAG, "SMSDKControl::handleAllNetworkPoor() 双方网络差！");
        if (mSMSDKListener != null) {
            mSMSDKListener.onStatus(
                    MediaPlaySDKReturnCode.MEDIAPLAY_ALL_NETWORK_POOR, null);
        }
    }

    private void handleGetNetInfo() {
        CustomLog.d(TAG, "SMSDKControl::handleGetNetInfo() 获取网络信息！");
        if (mSMSDKListener != null) {
            mSMSDKListener.onStatus(
                    MediaPlaySDKReturnCode.MEDIAPLAY_GET_NET_INFO, null);
        }
    }

    private void handleNotifyAudioMode() {
        CustomLog.d(TAG, "SMSDKControl::handleNotifyAudioMode() 提醒切换到语音模式！");
        if (mSMSDKListener != null) {
            mSMSDKListener.onStatus(
                    MediaPlaySDKReturnCode.MEDIAPLAY_NOTIFY_AUDIO_TYPE, null);
        }
    }

    // {"Level":"网络等级宏所对应的值"}
    private void handleSignalStrengthUpdate(String signalMsg) {
        CustomLog.d(TAG,
                "SMSDKControl::handleSignalStrengthUpdate() 网络信号强度更新: "
                        + signalMsg);
        try {
            JSONObject json = new JSONObject(signalMsg);
            int level = Integer.valueOf(json.getString("Level"));
            if (mSMSDKListener != null) {
                mSMSDKListener.onStatus(
                        MediaPlaySDKReturnCode.SIGNAL_STRENGTH_UPDATE, level);
            }
        } catch (Exception e) {
            CustomLog.e(TAG,
                    "error! handleSignalStrengthUpdate()" + e.getMessage());
        }
    }

    // {"streamID":"1","description":"xxxxxxx"}
    private void handleDeviceLost(String deviceLostMsg) {
        CustomLog.d(TAG, "SMSDKControl::handleDeviceLost() 设备丢失: "
                + deviceLostMsg);
        try {
            JSONObject json = new JSONObject(deviceLostMsg);
            int streamId = Integer.valueOf(json.getString("streamID"));
            if (mSMSDKListener != null) {
                mSMSDKListener.onStatus(MediaPlaySDKReturnCode.DEVICES_LOST,
                        streamId);
            }
        } catch (Exception e) {
            CustomLog.e(TAG, "error! handleDeviceLost()" + e.getMessage());
        }
    }

    // 锁定画布，避免贴上背景图后，最后一帧的闪现
    private void lockCanvas(SurfaceView surfaceView) {
        if (surfaceView == null) {
            return;
        }
        Canvas canvas = surfaceView.getHolder().lockCanvas();
        try {
            if (canvas != null) {
                surfaceView.getHolder().unlockCanvasAndPost(canvas);
            }
        } catch (Exception e) {
            CustomLog.e(TAG, e.toString());
        }
    }

    // public int setVideoParam(int resourceId, VideoParameter param) {
    // CustomLog.d(TAG, "SMSDKControl::setVideoParam 设置视频采集模式.resourceId: " +
    // resourceId);
    // if (param == null) {
    // CustomLog.e(TAG, "SMSDKControl::setVideoParam() error! param 不能为null");
    // return -1;
    // }
    // param.logSelf();
    //
    // mVideoParamMap.put(resourceId, param);
    //
    // return 0;
    // }

    @SuppressWarnings("unchecked")
    public int setVideoParam(int resourceId, VideoParameter param) {
        CustomLog.d(TAG, "SMSDKControl::setVideoParam 设置视频采集模式.resourceId: "
                + resourceId);
        if (param == null) {
            CustomLog.e(TAG,
                    "SMSDKControl::setVideoParam() error! param 不能为null");
            return -1;
        }
        param.logSelf();
        // 针对苏州通服AR眼镜UVC摄像头特殊处理
        // if (resourceId == 2) {
        // mVideoParamMap.put(resourceId, param);
        // return 0;
        // }

        mCamera = Camera.open(mCameraIndex);
        Parameters parameter = null;
        int preViewWidth = param.getCapWidth();
        int preViewHeight = param.getCapHeight();
        try {
            parameter = mCamera.getParameters();
            CustomLog.d(TAG, "setVideoParam,parameter=" + parameter.toString());
            for (Size size : parameter.getSupportedPreviewSizes()) {
                CustomLog.d(TAG, "setVideoParam,SupportedPreviewSizes: "
                        + size.width + "x" + size.height);
            }
            List<Size> list = null;
            List<MySize> mList = new ArrayList<MySize>();
            if (parameter != null) {
                list = parameter.getSupportedPreviewSizes();
                if (list != null) {
                    for (int i = 0; i < list.size(); i++) {
                        mList.add(new MySize(list.get(i).width,
                                list.get(i).height));
                    }
                    Collections.sort(mList);
                    int width = getNearResolution(mList, preViewWidth,
                            preViewHeight).x;
                    int height = getNearResolution(mList, preViewWidth,
                            preViewHeight).y;
                    CustomLog.d(
                            TAG,
                            "setVideoParam,getNearResolution,width:"
                                    + String.valueOf(width) + " height:"
                                    + String.valueOf(height) + " resourceId:"
                                    + String.valueOf(resourceId));
                    param.setCapWidth(width);
                    param.setCapHeight(height);
                    mVideoParamMap.put(resourceId, param);
                }
            }
        } catch (Exception e) {
            CustomLog.e(TAG, "setVideoParam," + e.toString());
        }
        mCamera.release();
        mCamera = null;
        return 0;
    }

    private class MySize implements Comparable {
        int x;
        int y;

        MySize(int x, int y) {
            this.x = x;
            this.y = y;
        }

        @Override
        public int compareTo(Object o) {
            MySize obj = (MySize) o;
            if (x != obj.x) {
                return x - obj.x;
            }
            return y - obj.y;
        }

        public String toString() {
            return x + "X" + y;
        }
    }

    private MySize getNearResolution(List<MySize> list, int w, int h) {

        if (list == null)
            return null;
        int dif = 9999;
        int index = 0;
        for (int i = 0; i < list.size(); i++) {
            int n = (Math.abs((list.get(i).x) - w) + Math.abs((list.get(i).y)
                    - h));
            if (n < dif) {
                dif = n;
                index = i;
            }
        }
        CustomLog.d(
                TAG,
                "最接近 " + w + " X " + h + " is "
                        + String.valueOf(list.get(index).x) + " X "
                        + String.valueOf(list.get(index).y));

        return list.get(index);

    }

    public int setMediaAdaptiveParam(String param) {
        CustomLog
                .d(TAG, "SMSDKControl::setMediaAdaptiveParam 设置媒体自适应参数.param: "
                        + param);
        if (TextUtils.isEmpty(param)) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::setMediaAdaptiveParam() error! param 不能为null");
            return -1;
        }

        try {

            JSONObject originObject = new JSONObject(param);

            JSONObject object = originObject
                    .getJSONObject("adaptiveSettingPara");
            JSONObject upParameterObject = object
                    .getJSONObject("upParametersCfg");
            JSONObject downParameterObject = object
                    .getJSONObject("downParametersCfg");

            JSONObject upBandwidthObject = upParameterObject
                    .getJSONObject("upBandwidthCfg");
            JSONObject upVideoObject = upParameterObject
                    .getJSONObject("upVideoCfg");
            JSONObject upAudioObject = upParameterObject
                    .getJSONObject("upAudioCfg");
            JSONObject upDocVideoObject = upParameterObject
                    .getJSONObject("upDocVideoCfg");

            JSONObject downBandwidthObject = downParameterObject
                    .getJSONObject("downBandwidthcfg");
            JSONObject downVideoObject = downParameterObject
                    .getJSONObject("downVideocfg");
            JSONObject downAudioObject = downParameterObject
                    .getJSONObject("downAudioCfg");
            JSONObject downDocVideoObject = downParameterObject
                    .getJSONObject("downDocVideoCfg");

            upBandwidth = Integer.valueOf(upBandwidthObject
                    .getString(MediaAdaptiveParam.upBandwidth));
            upRetransBwRatio = Integer.valueOf(upBandwidthObject
                    .getString(MediaAdaptiveParam.upRetransBwRatio));
            downBandwidth = Integer.valueOf(downBandwidthObject
                    .getString(MediaAdaptiveParam.downBandwidth));
            downRetransBwRatio = Integer.valueOf(downBandwidthObject
                    .getString(MediaAdaptiveParam.downRetransBwRatio));

            // 视频 参数
            videoUpFecDataCount = Integer.valueOf(upVideoObject
                    .getString(MediaAdaptiveParam.videoUpFecDataCount));
            videoUpFecCheckCount = Integer.valueOf(upVideoObject
                    .getString(MediaAdaptiveParam.videoUpFecCheckCount));
            videoUpDataSendCount = Integer.valueOf(upVideoObject
                    .getString(MediaAdaptiveParam.videoUpDataSendCount));
            videoUpCheckSendCount = Integer.valueOf(upVideoObject
                    .getString(MediaAdaptiveParam.videoUpCheckSendCount));
            videoUpDataRetransSendCount = Integer.valueOf(upVideoObject
                    .getString(MediaAdaptiveParam.videoUpDataRetransSendCount));
            videoUpCheckRetransSendCount = Integer
                    .valueOf(upVideoObject
                            .getString(MediaAdaptiveParam.videoUpCheckRetransSendCount));
            videoUpDataResendCount = Integer.valueOf(upVideoObject
                    .getString(MediaAdaptiveParam.videoUpDataResendCount));
            videoDownFecDataCount = Integer.valueOf(downVideoObject
                    .getString(MediaAdaptiveParam.videoDownFecDataCount));
            videoDownFecCheckCount = Integer.valueOf(downVideoObject
                    .getString(MediaAdaptiveParam.videoDownFecCheckCount));
            videoDownDataSendCount = Integer.valueOf(downVideoObject
                    .getString(MediaAdaptiveParam.videoDownDataSendCount));
            videoDownCheckSendCount = Integer.valueOf(downVideoObject
                    .getString(MediaAdaptiveParam.videoDownCheckSendCount));
            videoDownDataRetransSendCount = Integer
                    .valueOf(downVideoObject
                            .getString(MediaAdaptiveParam.videoDownDataRetransSendCount));
            videoDownCheckRetransSendCount = Integer
                    .valueOf(downVideoObject
                            .getString(MediaAdaptiveParam.videoDownCheckRetransSendCount));
            videoDownDataResendCount = Integer.valueOf(downVideoObject
                    .getString(MediaAdaptiveParam.videoDownDataResendCount));
            videoDelayTime = Integer.valueOf(downVideoObject
                    .getString(MediaAdaptiveParam.videoDelayTime));

            // 音频 参数
            audioUpFecDataCount = Integer.valueOf(upAudioObject
                    .getString(MediaAdaptiveParam.audioUpFecDataCount));
            audioUpFecCheckCount = Integer.valueOf(upAudioObject
                    .getString(MediaAdaptiveParam.audioUpFecCheckCount));
            audioUpDataSendCount = Integer.valueOf(upAudioObject
                    .getString(MediaAdaptiveParam.audioUpDataSendCount));
            audioUpCheckSendCount = Integer.valueOf(upAudioObject
                    .getString(MediaAdaptiveParam.audioUpCheckSendCount));
            audioUpDataRetransSendCount = Integer.valueOf(upAudioObject
                    .getString(MediaAdaptiveParam.audioUpDataRetransSendCount));
            audioUpCheckRetransSendCount = Integer
                    .valueOf(upAudioObject
                            .getString(MediaAdaptiveParam.audioUpCheckRetransSendCount));
            audioUpDataResendCount = Integer.valueOf(upAudioObject
                    .getString(MediaAdaptiveParam.audioUpDataResendCount));
            audioDownFecDataCount = Integer.valueOf(downAudioObject
                    .getString(MediaAdaptiveParam.audioDownFecDataCount));
            audioDownFecCheckCount = Integer.valueOf(downAudioObject
                    .getString(MediaAdaptiveParam.audioDownFecCheckCount));
            audioDownDataSendCount = Integer.valueOf(downAudioObject
                    .getString(MediaAdaptiveParam.audioDownDataSendCount));
            audioDownCheckSendCount = Integer.valueOf(downAudioObject
                    .getString(MediaAdaptiveParam.audioDownCheckSendCount));
            audioDownDataRetransSendCount = Integer
                    .valueOf(downAudioObject
                            .getString(MediaAdaptiveParam.audioDownDataRetransSendCount));
            audioDownCheckRetransSendCount = Integer
                    .valueOf(downAudioObject
                            .getString(MediaAdaptiveParam.audioDownCheckRetransSendCount));
            audioDownDataResendCount = Integer.valueOf(downAudioObject
                    .getString(MediaAdaptiveParam.audioDownDataResendCount));
            audioDelayTime = Integer.valueOf(downAudioObject
                    .getString(MediaAdaptiveParam.audioDelayTime));

            // 文档 视频 参数
            DocVideoUpFecDataCount = Integer.valueOf(upDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoUpFecDataCount));
            DocVideoUpFecCheckCount = Integer.valueOf(upDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoUpFecCheckCount));
            DocVideoUpDataSendCount = Integer.valueOf(upDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoUpDataSendCount));
            DocVideoUpCheckSendCount = Integer.valueOf(upDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoUpCheckSendCount));
            DocVideoUpDataRetransSendCount = Integer
                    .valueOf(upDocVideoObject
                            .getString(MediaAdaptiveParam.DocVideoUpDataRetransSendCount));
            DocVideoUpCheckRetransSendCount = Integer
                    .valueOf(upDocVideoObject
                            .getString(MediaAdaptiveParam.DocVideoUpCheckRetransSendCount));
            DocVideoUpDataResendCount = Integer.valueOf(upDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoUpDataResendCount));
            DocVideoDownFecDataCount = Integer.valueOf(downDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoDownFecDataCount));
            DocVideoDownFecCheckCount = Integer.valueOf(downDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoDownFecCheckCount));
            DocVideoDownDataSendCount = Integer.valueOf(downDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoDownDataSendCount));
            DocVideoDownCheckSendCount = Integer.valueOf(downDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoDownCheckSendCount));
            DocVideoDownDataRetransSendCount = Integer
                    .valueOf(downDocVideoObject
                            .getString(MediaAdaptiveParam.DocVideoDownDataRetransSendCount));
            DocVideoDownCheckRetransSendCount = Integer
                    .valueOf(downDocVideoObject
                            .getString(MediaAdaptiveParam.DocVideoDownCheckRetransSendCount));
            DocVideoDownDataResendCount = Integer.valueOf(downDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoDownDataResendCount));
            DocVideoDelayTime = Integer.valueOf(downDocVideoObject
                    .getString(MediaAdaptiveParam.DocVideoDelayTime));

        } catch (JSONException e) {
            CustomLog.e(
                    TAG,
                    "error! SMSDKControl::setMediaAdaptiveParam "
                            + e.getMessage());
        }

        return 0;
    }

    public VideoParameter getVideoParam() {
        int resourceId = mCameraIndex;
        CustomLog.d(TAG, "SMSDKControl::getVideoParam() resourceId: "
                + resourceId);
        VideoParameter param = mVideoParamMap.get(mCameraIndex);
        if (param == null) {
            CustomLog.d(TAG, "SMSDKControl::getVideoParam resourceId "
                    + resourceId + "没有相关设置值，使用默认值 640x360,15,300");

            if (resourceId == 2) {
                param = new VideoParameter(DEFAULT_UVC_WEIGHT,
                        DEFAULT_UVC_HEIGHT, VideoParameter.DEFAULT_FPS,
                        STANDARD_EncBitrate);
            } else {
                param = new VideoParameter(STANDARD_CAPTURE_RATE_WEIGHT,
                        STANDARD_CAPTURE_RATE_HEIGHT,
                        VideoParameter.DEFAULT_FPS, STANDARD_EncBitrate);
            }

        }
        param.logSelf();
        return param;
    }

    public VideoParameter getVideoParam(int resourceId) {
        CustomLog.d(TAG, "SMSDKControl::getVideoParam() resourceId: "
                + resourceId);
        VideoParameter param = mVideoParamMap.get(resourceId);
        if (param != null) {
            param.logSelf();
        }
        if (param == null && resourceId == 999) {
            param = new VideoParameter();
            param.setCapWidth(640);
            param.setCapWidth(480);
            param.setCapFps(15);
            param.setEncBitrate(600);
        }
        return param;
    }

    private void setCameraOpenStatus(boolean isOpen) {
        CustomLog.d(TAG, "SMSDKControl::setCameraOpenStatus() isOpen: "
                + isOpen);
        isCameraOpen = isOpen;
    }

    public int adjustPreviewSize(int previewWidth, int previewHeight) {
        CustomLog.d(TAG, "SMSDKControl::adjustPreviewSize() previewWidth: "
                + previewWidth + "| previewHeight:" + previewHeight);
        adjustSurfaceFullScreenLandscape(mSurfaceView);
        return 0;
    }

    protected void adjustSurfaceFullScreenLandscape(
            final SurfaceView surfaceview) {
        surfaceview.postDelayed(new Runnable() {

            @Override
            public void run() {
                int w = ((ViewGroup) surfaceview.getParent()).getWidth();
                int h = ((ViewGroup) surfaceview.getParent()).getHeight();

                int capWidth = getVideoParam().getCapWidth();
                int capHeight = getVideoParam().getCapHeight();

                int localVideoWidth = w;
                int localVideoHeight = h;

                if ((float) w / h - (float) capWidth / capHeight > 0.0f) {
                    localVideoHeight = w * getVideoParam().getCapHeight()
                            / getVideoParam().getCapWidth();
                } else if ((float) w / h - (float) capWidth / capHeight < 0.0f) {
                    localVideoWidth = h * getVideoParam().getCapWidth()
                            / getVideoParam().getCapHeight();
                }

                CustomLog.d(
                        TAG,
                        "SMSDKControl::adjustSurfaceFullScreenLandscape() surfaceview width:"
                                + w
                                + " | surfaceview hight: "
                                + h
                                + " | localVideoWidth width: "
                                + localVideoWidth
                                + " | localVideoHeight height: "
                                + localVideoHeight
                                + " | parentWidth: "
                                + ((ViewGroup) surfaceview.getParent())
                                .getWidth()
                                + " |parentHeight: "
                                + ((ViewGroup) surfaceview.getParent())
                                .getHeight());

                if (surfaceview != null) {
                    FrameLayout.LayoutParams params = (FrameLayout.LayoutParams) surfaceview
                            .getLayoutParams();
                    params.width = localVideoWidth;
                    params.height = localVideoHeight;

                    params.gravity = Gravity.CENTER;

                    CustomLog.d(TAG,
                            "SMSDKControl::adjustSurfaceFullScreenLandscape() leftMargin: "
                                    + params.leftMargin + " | topMargin: "
                                    + params.topMargin + " | bottomMargin: "
                                    + params.bottomMargin + " | rightMargin: "
                                    + params.rightMargin);
                    surfaceview.setLayoutParams(params);
                    surfaceview.setBackgroundColor(Color.BLUE);
                }
            }
        }, 20);

    }

    private void setCpuInfo() {
        CustomLog.d(TAG, "SMSDKControl::setCpuInfo()");
        CpuUsageUtil.getInstance().init();
        setCpuTimer = new Timer();
        setCpuTimerTask = new TimerTask() {
            @Override
            public void run() {
                // TODO Auto-generated method stub
                int totalCpu = CpuUsageUtil.getInstance().getTotalCpuRate();
                int selfCpu = CpuUsageUtil.getInstance()
                        .getCurrentProcessCpuRate();
                if (totalCpu >= 85 || selfCpu >= 65) {
                    CustomLog.d(TAG, "totalCpu:" + String.valueOf(totalCpu)
                            + " selfCpu:" + String.valueOf(selfCpu));
                }
                if (totalCpu == -1 || selfCpu == -1) {

                } else {
                    // CustomLog.d(TAG,"SM_SDK_SetMeetingCpuAdaptionInfo");
                    SmSdkJNI.SM_SDK_SetMeetingCpuAdaptionInfo(totalCpu, selfCpu);
                }
            }
        };
        setCpuTimer.schedule(setCpuTimerTask, 0, 5000);
    }

    private void handleNotifyCpuOverLoad(String cpuOverloadMsg) {
        CustomLog.d(TAG, "SMSDKControl::handleNotifyCpuOverLoad() 占用cpu过载");
        try {
            JSONObject json = new JSONObject(cpuOverloadMsg);
            int cpuStatus = json.getInt("CPUStatus");
            if (mSMSDKListener != null) {
                mSMSDKListener.onStatus(MediaPlaySDKReturnCode.CPU_OVERLOAD,
                        cpuStatus);
            }
        } catch (Exception e) {
            CustomLog.e(TAG,
                    "error! handleNotifyCpuOverLoad()" + e.getMessage());
        }
    }

    private void handleNotifyFrameWorkMsg(String msg) {
        CustomLog.d(TAG, "handleNotifyFrameWorkMsg msg=" + msg);
        try {
            int isDocFlag = 0;
            JSONObject json = new JSONObject(msg);

            int streamId = json.getInt("streamID");
            String accountId = mButelopensdk
                    .getAccountIdByVideoStreamId(streamId);
            if (accountId == null) {
                isDocFlag = 1;
                accountId = mButelopensdk
                        .getAccountIdByDocVideoStreamId(streamId);
            }
            String info = json.getString("info");
            JSONObject infoObj = new JSONObject(info);
            int eventId = infoObj.getInt("eventId");

            JSONObject jsonObject = new JSONObject();
            jsonObject.put("accountId", accountId);
            jsonObject.put("isDocFlag", isDocFlag);
            if (eventId == 1) {
                if (mSMSDKListener != null) {
                    mSMSDKListener.onStatus(
                            MediaPlaySDKReturnCode.VIDEO_START_RENDER,
                            jsonObject.toString());
                }
            }

        } catch (Exception e) {
            CustomLog.e(TAG,
                    "error! handleNotifyFrameWorkMsg()" + e.getMessage());
        }
    }

    private int getVolumeLevel(double db) {
        int level = 0;
        if (db >= -60 && db <= -10) {
            level = (int) ((db + 60) / 50 * 90);
        } else if (db > -10) {
            level = (int) (100 + db);
        }
        if (level > 100) {
            level = 100;
        }
        return level;
    }

    private void handleNotifyVolumeUpdate(String volumeUpdateMsg) {
        // CustomLog.d(TAG, "SMSDKControl::handleNotifyVolumeUpdate()");

        try {
            JSONObject json = new JSONObject(volumeUpdateMsg);

            int streamId = json.getInt("streamID");
            String accountId = mButelopensdk
                    .getAccountIdByAudioStreamId(streamId);
//			int volume = (int) json.getDouble("left");
            int level = getVolumeLevel(json.getDouble("left"));
            int volumeLevel = 0;

            if (level <= 100 / 6) {
                volumeLevel = 0;
            } else if (100 / 6 < level && level <= 100 * 2 / 6) {
                volumeLevel = 1;
            } else if (100 * 2 / 6 < level && level <= 100 * 3 / 6) {
                volumeLevel = 2;
            } else if (100 * 3 / 6 < level && level <= 100 * 4 / 6) {
                volumeLevel = 3;
            } else if (100 * 4 / 6 < level && level <= 100 * 5 / 6) {
                volumeLevel = 4;
            } else if (100 * 5 / 6 < level && level < 100) {
                volumeLevel = 5;
            } else {
                volumeLevel = 6;
            }

//			if (volume <= -90) {
//				volumeLevel = 0;
//			} else if (volume > -90 && volume <= -75) {
//				volumeLevel = 1;
//			} else if (volume > -75 && volume <= -60) {
//				volumeLevel = 2;
//			} else if (volume > -60 && volume <= -45) {
//				volumeLevel = 3;
//			} else if (volume > -45 && volume <= -30) {
//				volumeLevel = 4;
//			} else if (volume > -30 && volume <= -15) {
//				volumeLevel = 5;
//			} else if (volume > -15 && volume <= 0) {
//				volumeLevel = 6;
//			}

            SpeakerInfo speakerInfo = new SpeakerInfo();

            if (accountId != null) {
                speakerInfo = mButelopensdk.getSpeakerByAccountId(accountId);
            } else {
                CustomLog.d(TAG, "accountId == null");
            }

            if (speakerInfo == null) {
                CustomLog.d(TAG, "handleNotifyVolumeUpdate,speakerInfo==null");
                return;
            }

            speakerInfo.setVolumeLevel(volumeLevel);

            // CustomLog.d(TAG, "handleNotifyVolumeUpdate,accountId:" +
            // accountId
            // + " volume:" + String.valueOf(volume) + " volumeLevel:"
            // + String.valueOf(volumeLevel));

            if (mSMSDKListener != null) {
                mSMSDKListener.onStatus(MediaPlaySDKReturnCode.VOLUME_UPDATE,
                        accountId);
            }

        } catch (Exception e) {
            CustomLog.e(TAG,
                    "error! handleNotifyVolumeUpdate()" + e.getMessage());
        }

    }

    private void handleNotifyNetStatus(String netStatusMsg) {
        CustomLog.d(TAG, "SMSDKControl::handleNotifyNetStatus() 网络信号");
        try {
            JSONObject json = new JSONObject(netStatusMsg);
            int netStatus = json.getInt("NetWorkStatus");
            if (mSMSDKListener != null) {
                mSMSDKListener.onStatus(MediaPlaySDKReturnCode.NET_STATUS,
                        netStatus);
            }
        } catch (Exception e) {
            CustomLog.e(TAG, "error! handleNotifyNetStatus()" + e.getMessage());
        }
    }

    private void handleNotifyVideoCloseAdapt(String videoCloseAdaptMsg) {
        CustomLog.d(TAG, "SMSDKControl::handleNotifyVideoCloseAdapt() 自适应关闭视频:"
                + videoCloseAdaptMsg);
        try {
            JSONObject json = new JSONObject(videoCloseAdaptMsg);
            int streamId = json.getInt("streamID");
            if (mSMSDKListener != null) {
                mSMSDKListener.onStatus(
                        MediaPlaySDKReturnCode.VIDEO_CLOSE_ADAPT, streamId);
            }
        } catch (Exception e) {
            CustomLog.e(TAG,
                    "error! handleNotifyVideoCloseAdapt()" + e.getMessage());
        }
    }

    /**
     * 设置是否自适应
     *
     * @param adaptState ：true：打开自适应； false：关闭自适应
     * @return
     */
    public int setAdaptState(boolean adaptState) {
        int result;
        if (adaptState) {
            result = SmSdkJNI.SM_SDK_AdjustAdapteState(0);
        } else {
            result = SmSdkJNI.SM_SDK_AdjustAdapteState(1);
        }
        return result;
    }

    /**
     * 设置是否开启大流模式
     *
     * @param isBigStream ：true：打开； false：关闭
     * @return
     */
    public int setBigStream(boolean isBigStream) {
        mIsBigStream = isBigStream;
        return 0;
    }

    private final OnDeviceConnectListener mOnDeviceConnectListener = new OnDeviceConnectListener() {

        @Override
        public void onAttach(final UsbDevice device) {
            // TODO Auto-generated method stub
            CustomLog.d(TAG, "onAttach");
            cameraHandler.sendEmptyMessage(ON_ATTACH);
        }

        @Override
        public void onCancel(UsbDevice arg0) {
            // TODO Auto-generated method stub
            CustomLog.d(TAG, "onCancel");

        }

        @Override
        public void onConnect(final UsbDevice device,
                              final UsbControlBlock ctrlBlock, final boolean createNew) {
            // TODO Auto-generated method stub
            CustomLog.d(TAG, "onConnect");
            Message msg = new Message();
            msg.what = ON_CONNECT;
            msg.obj = ctrlBlock;
            cameraHandler.sendMessage(msg);
        }

        @Override
        public void onDettach(UsbDevice arg0) {
            // TODO Auto-generated method stub
            CustomLog.d(TAG, "onDettach");
            cameraHandler.sendEmptyMessage(ON_DETTACH);
        }

        @Override
        public void onDisconnect(UsbDevice arg0, UsbControlBlock arg1) {
            // TODO Auto-generated method stub
            CustomLog.d(TAG, "onDisconnect");
            cameraHandler.sendEmptyMessage(ON_DISCONNECT);
        }

    };
    private final IFrameCallback mIFrameCallback = new IFrameCallback() {
        @Override
        public void onFrame(final ByteBuffer frame) {
            CustomLog.d(TAG, "onFrame");
            frame.clear();
            mDoRecoverCount = 0;
            recoverCameraHandler.removeMessages(0);
            byte[] cameraData = new byte[frame.capacity()];
            // transfer bytes from this buffer into the given destination array
            frame.get(cameraData, 0, cameraData.length);
            SmSdkJNI.SM_SDK_PushFrame(mVideoStremId,
                    SmSdkJNI.SMSDK_FRAME_TYPE_YUV, cameraData,
                    cameraData.length, mPushFrameOrientation);
        }
    };

    // 从activity中获取到filter
    public void setUseUVCCamera(List<DeviceFilter> filter) {
        this.mFilter = filter;
    }

    private void handleOnConnect(final UsbControlBlock ctrlBlock) {

        if (mUVCCamera != null) {
            mUVCCamera.destroy();
            mUVCCamera = null;
            CustomLog.d(TAG, "mUVCCamera != null");
        } else {
            CustomLog.d(TAG, "mUVCCamera == null");
        }
        // 打开uvc摄像头
        mUVCCamera = new UVCCamera();

        try {
            CustomLog.d(TAG, "mUVCCamera.open");
            mUVCCamera.open(ctrlBlock);
        } catch (Exception e) {
            CustomLog.d(TAG, "mUVCCamera.open Exception:" + e.toString());
        }

        try {
            if (getVideoParam().getCapWidth() == 1280
                    && getVideoParam().getCapHeight() == 720) {
                mUVCCamera.setPreviewSize(getVideoParam().getCapWidth(),
                        getVideoParam().getCapHeight(),
                        UVCCamera.FRAME_FORMAT_MJPEG);
            } else {
                mUVCCamera.setPreviewSize(getVideoParam().getCapWidth(),
                        getVideoParam().getCapHeight(),
                        UVCCamera.FRAME_FORMAT_YUYV);
            }

            CustomLog.d(TAG, "1st try camera.setPreviewSize");
        } catch (final IllegalArgumentException e) {
            CustomLog.d(TAG, "1st catch camera.setPreviewSize:" + e.toString());
            try {
                // fallback to YUV mode
                mUVCCamera.setPreviewSize(getVideoParam().getCapWidth(),
                        getVideoParam().getCapHeight(),
                        UVCCamera.DEFAULT_PREVIEW_MODE);
                CustomLog.d(TAG, "2nd try camera.setPreviewSize");
            } catch (final IllegalArgumentException e1) {
                CustomLog.d(TAG,
                        "2nd catch camera.setPreviewSize:" + e.toString());
                mUVCCamera.destroy();
                mUVCCamera = null;
                return;
            }
        }
        if (LocalHolder.getSurface() != null) {
            CustomLog.d(TAG, "LocalHolder.getSurface() != null");

            setCameraOpenStatus(true);
            // 摄像头资源已使用，如果mSurfaceView还在使用，说明现在是打開摄像头状态，需要把背景清除
            if (mSurfaceView != null) {
                mSurfaceView.setBackgroundDrawable(null);// 为了4.0版本兼容性使用该api
            }

            recoverCameraHandler.sendEmptyMessageDelayed(0,
                    RECOVER_CAMERA_DELAY_TIME);
            if (mCodecType == 0) {
                mUVCCamera.setPreviewDisplay(LocalHolder.getSurface());
            } else {
                // gexy modified in 2018-11-19 16:28:41
                mUVCCamera.setPreviewTexture(mSurfTex);
            }
            mUVCCamera.setFrameCallback(mIFrameCallback,
                    UVCCamera.PIXEL_FORMAT_NV12);
            mUVCCamera.startPreview();

        } else {
            CustomLog.d(TAG, "LocalHolder.getSurface() == null");
        }

        if (mUVCCamera != null) {
            CustomLog.d(TAG, "mUVCCamera != null");
        } else {
            CustomLog.d(TAG, "mUVCCamera == null");
        }

    }

    private void handleOnDisconnect() {
        if (mUVCCamera != null) {
            mUVCCamera.close();

            if (LocalHolder.getSurface() != null) {
                LocalHolder.getSurface().release();
            }

        }
    }

    public void setRotation(int rotation) {
        CustomLog.d(TAG, "setRotation，rotation:" + String.valueOf(rotation)
                + " mCameraIndex:" + String.valueOf(mCameraIndex));
        mCameraOrientation = rotation;
        switch (rotation) {
            case 0:
                mPushFrameOrientation = 0;
                break;
            case 90:
                if (mCameraIndex == mButelopensdk.BACK_CAMERA) {
                    mPushFrameOrientation = 270;
                } else {
                    mPushFrameOrientation = 90;
                }
                break;
            case 180:
                mPushFrameOrientation = 180;
                break;
            case 270:
                if (mCameraIndex == mButelopensdk.BACK_CAMERA) {// 后置摄像头
                    mPushFrameOrientation = 90;
                } else {
                    mPushFrameOrientation = 270;
                }
                break;
        }
    }

    public void setRotation(int rotation, boolean isFloatingView) {
        CustomLog.d(TAG, "setRotation，rotation:" + String.valueOf(rotation)
                + " mCameraIndex:" + String.valueOf(mCameraIndex));
        mCameraOrientation = rotation;

        switch (rotation) {
            case 0:
                mPushFrameOrientation = 0;
                break;
            case 90:
                if (mCameraIndex == mButelopensdk.BACK_CAMERA) {
                    mPushFrameOrientation = 270;
                } else {
                    mPushFrameOrientation = 90;
                }
                break;
            case 180:
                mPushFrameOrientation = 180;
                break;
            case 270:
                if (mCameraIndex == mButelopensdk.BACK_CAMERA) {// 后置摄像头
                    mPushFrameOrientation = 90;
                } else {
                    mPushFrameOrientation = 270;
                }
                break;
        }

        if (isFloatingView) {
            if (mCameraIndex == mButelopensdk.BACK_CAMERA) {// 后置摄像头
                mPushFrameOrientation += 3;
            } else {
                mPushFrameOrientation += 1;
            }
        } else {
            if (mCameraIndex == mButelopensdk.BACK_CAMERA) {// 后置摄像头
                mPushFrameOrientation += 2;
            } else {
                mPushFrameOrientation += 0;
            }
        }
        if (mCodecType != 0) {
            CustomLog.d(TAG,
                    "setRotation hardCodec setVideoOrientation mPushFrameOrientation="
                            + mPushFrameOrientation);
            SmSdkJNI.SM_SDK_SetVideoOrientation(mVideoStremId,
                    mPushFrameOrientation);
        }

        CustomLog.d(TAG, "setRotation，mPushFrameOrientation:"
                + mPushFrameOrientation);
    }

    public int getCurrentCameraIndex() {
        CustomLog.d(
                TAG,
                "getCurrentCameraIndex,mCameraIndex"
                        + String.valueOf(mCameraIndex));
        return mCameraIndex;
    }

    /**
     * 设置是否使用声音增益
     *
     * @param isAgc
     */
    public void setIsAgc(boolean isAgc) {
        CustomLog.d(TAG, "setIsAgc,isAgc:" + String.valueOf(isAgc));
        int result = SmSdkJNI.SM_SDK_SetIsAGC(isAgc ? 1 : 0);
        CustomLog.d(TAG, "setIsAgc,result:" + result);
    }

    /**
     * 获取是否在使用声音增益
     *
     * @return true：使用了声音增益 false：没有使用声音增益
     */
    public boolean getIsAgc() {
        CustomLog.d(TAG, "getIsAgc");
        boolean value = (SmSdkJNI.SM_SDK_GetIsAGC() == 0) ? false : true;
        CustomLog.d(TAG, "getIsAgc:" + String.valueOf(value));
        return value;
    }

    /**
     * @param area 服务器所在地域
     * @param isp  运营商
     * @return 0：成功；非零表示失败
     * @brief 设置客户端地域和运营商信息 进会时调用
     */
    public int setTerminalInfo(String area, String isp) {
        CustomLog.d(TAG, "setTerminalInfo,area:" + area + " isp:" + isp);
        int result = SmSdkJNI.SM_SDK_setTerminalInfo(area, isp);
        CustomLog.d(TAG, "setTerminalInfo,result:" + result);
        return result;
    }

    /**
     * 设置自己的surfaceview，音频会议或进会默认关闭摄像头时需要调用
     *
     * @param surfaceView
     */
    public void setMySurfaceView(SurfaceView surfaceView) {
        mSurfaceView = surfaceView;

        SurfaceHolder holder = mSurfaceView.getHolder();

        LocalHolder = holder;

        if (mLocalSurfaceCallBack != null) {
            CustomLog.d(TAG, "setMySurfaceView remove previous callback");
            holder.removeCallback(mLocalSurfaceCallBack);
        }
        mLocalSurfaceCallBack = new ExtendSurfaceHolderCallBack(
                new SurfaceViewCallBack() {
                    @Override
                    public void onSurfaceViewCreated(SurfaceHolder holder,
                                                     int streamId) {
                        CustomLog.d(TAG,
                                "setMySurfaceView::onSurfaceViewCreated() streamId: "
                                        + streamId);

                        CustomLog.d(TAG,
                                "setMySurfaceView onSurfaceViewCreated,isCameraOpen:"
                                        + String.valueOf(isCameraOpen));
                        if (isCameraOpen || mPollVideoStreamId > 0) {

                            // gexy add in 2018-11-19 16:21:41
                            CustomLog
                                    .d(TAG,
                                            "setMySurfaceView SM_SDK_SetDisplayWindow");
                            int result = SmSdkJNI.SM_SDK_SetDisplayWindow(
                                    streamId, (Object) holder.getSurface(),
                                    FILL_MODE_RAWRATIO_WITHBLACK,
                                    FILL_MODE_RAWRATIO_WITHBLACK);
                            if (result == 0) {
                                CustomLog
                                        .d(TAG,
                                                "setMySurfaceView TYPE_VIDEO,SM_SDK_SetDisplayWindow success");
                            } else {
                                CustomLog.e(TAG,
                                        "setMySurfaceView TYPE_VIDEO,SM_SDK_SetDisplayWindow fail, result:"
                                                + result);
                            }
                            mSurfTex = (SurfaceTexture) SmSdkJNI
                                    .SM_SDK_GetDisplayWindow(streamId);

                            CustomLog
                                    .d(TAG,
                                            "setMySurfaceView onSurfaceViewCreated() startCameraPreview");
                            startCameraPreview(holder);

                        } else {
                            CustomLog
                                    .d(TAG,
                                            "setMySurfaceView onSurfaceViewCreated()摄像头关闭状态，不做打开自预览操作");
                        }
                        changingCamera = false;

                    }

                    @Override
                    public void onSurfaceChanged(SurfaceHolder holder,
                                                 int format, int width, int height, int streamId) {
                        CustomLog.d(TAG,
                                "setMySurfaceView onSurfaceChanged() format: "
                                        + format + " | width: " + width
                                        + " | height:" + height);
                    }

                    @Override
                    public void onSurfaceDestroyed(SurfaceHolder holder,
                                                   int streamId) {
                        CustomLog.d(TAG,
                                "setMySurfaceView onSurfaceViewDestroyed() streamId: "
                                        + streamId);
                        SmSdkJNI.SM_SDK_StopLocalVideoRender(streamId);
                        if (mButelopensdk.getSpeakerInfoById(mAccountId) != null) {
                            changingCamera = true;
                        }
						releaseCamera();

                    }
                });
        if (mLocalSurfaceCallBack != null) {
            mLocalSurfaceCallBack.setValues(mVideoStremId);
        }
        holder.addCallback(mLocalSurfaceCallBack);
//		holder.setFormat(PixelFormat.TRANSLUCENT);
    }

    /**
     * @param ip      (String):录制服务器IP
     * @param port    (int):录制服务器port
     * @param pstrcmd (String):FileAddress
     * @return 0-成功，其他-失败 目前只支持一个录制服务器地址
     * @brief 远程录制相关接口, 设置的录制服务器信息在单次会议中有效（版本2）
     */
    public int setRecordServerAddr2(String ip, int port, String pstrcmd) {
        CustomLog.d(
                TAG,
                "setRecordServerAddr2,ip:" + ip + " port:"
                        + String.valueOf(port) + " pstrcmd:" + pstrcmd);
        int result = SmSdkJNI.SM_SDK_setRecordServerAddr2(ip, port, pstrcmd,
                pstrcmd.length());
        CustomLog.d(TAG,
                "setRecordServerAddr2,result:" + String.valueOf(result));
        return result;
    }

    /**
     * @return 0-成功，其他-失败
     * @brief 开始录制
     */
    public int startRecord() {
        CustomLog.d(TAG, "startRecord");
        int result = SmSdkJNI.SM_SDK_startRecord();
        CustomLog.d(TAG, "startRecord,result:" + String.valueOf(result));
        return result;
    }

    /**
     * @brief 切换到本地录制
     * @return 0-成功，其他-失败
     */
    // public int switchToLocalRecord() {
    // CustomLog.d(TAG, "switchToLocalRecord");
    // int result = SmSdkJNI.SM_SDK_switchToLocalRecord();
    // CustomLog
    // .d(TAG, "switchToLocalRecord,result:" + String.valueOf(result));
    // return result;
    // }

    /**
     * @return 0-成功，其他-失败
     * @brief 停止录制
     */
    public int stopRecord() {
        CustomLog.d(TAG, "stopRecord");
        int result = SmSdkJNI.SM_SDK_stopRecord();
        CustomLog.d(TAG, "stopRecord,result:" + String.valueOf(result));
        return result;
    }

    public void setConvertRecverIPMap(String mapStr) {
        CustomLog.d(TAG, "setConvertRecverIPMap");
        SmSdkJNI.SM_SDK_setConvertRecverIPMap(mapStr);
    }

    public void setUpConvertRelayIPMap(String mapStr) {
        CustomLog.d(TAG, "setUpConvertRelayIPMap");
        SmSdkJNI.SM_SDK_setUpConvertRelayIPMap(mapStr);
    }

    public void setDownConvertRelayIPMap(String mapStr) {
        CustomLog.d(TAG, "setDownConvertRelayIPMap");
        SmSdkJNI.SM_SDK_setDownConvertRelayIPMap(mapStr);
    }

    /**
     * 发布视频流
     *
     * @param streamType    媒体流类型(SMSDK_STREAM_TYPE_WHITEBOARD_SEND)
     * @param resourceId    资源标识
     * @param mediaType     媒体类型
     * @param displayWindow 本地预览窗口句柄(安卓传NULL)
     * @return 大于0 – streamId；其他 – 错误
     */
    public int publishVideo2(int streamType, int resourceId, int mediaType,
                             Object displayWindow) {
        CustomLog.d(TAG, "publishVideo2 streamType=" + streamType
                + " resourceId=" + resourceId + " mediaType=" + mediaType
                + " displayWindow=" + displayWindow);

        return SmSdkJNI.SM_SDK_PublishVideo2(streamType, resourceId, mediaType,
                displayWindow);
    }

    private int setCodecType(int codecType) {
        CustomLog.d(TAG, "setCodecType codecType=" + codecType);
        JSONObject json = new JSONObject();
        try {
            json.put("eventId", 1);
            json.put("isHard", codecType);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        String result = SmSdkJNI.SM_SDK_MsgNotify(-1, 1, json.toString(), json
                .toString().length());
        if (result == null || result.length() <= 0) {
            return -1;
        }
        try {
            JSONObject jsonResult = new JSONObject(result);
            String ret = jsonResult.optString("result");
            if (isNumeric(ret)) {
                return Integer.parseInt(ret);
            }
            return -3;
        } catch (JSONException e) {
            e.printStackTrace();
            return -2;
        }
    }

    /**
     * 利用正则表达式判断字符串是否是数字
     *
     * @param str
     * @return
     */
    private boolean isNumeric(String str) {
        Pattern pattern = Pattern.compile("[0-9]*");
        Matcher isNum = pattern.matcher(str);
        if (!isNum.matches()) {
            return false;
        }
        return true;
    }

    public void setVideoLoadingImage(int bigImage, int smallImage) {
        mBigImage = bigImage;
        mSmallImage = smallImage;
    }

    public void startRenderPollVideo(SurfaceView surfaceView) {
        CustomLog.d(TAG, "startRenderPollVideo mPollVideoStreamId="
                + mPollVideoStreamId);
        if (mPollVideoStreamId > 0) {
            startRenderLocalVedio(mPollVideoStreamId, MediaType.TYPE_VIDEO,
                    surfaceView);
        } else {
            CustomLog.e(TAG, "startRenderPollVideo 发布流失败或不在推流状态，不开启渲染!");
        }
    }

    public int stopRenderPollVideo() {
        CustomLog.d(TAG, "SMSDKControl::stopRenderPollVideo() ");
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::stopRenderLocalVedio() 未进行初始化，error. return -1");
            return -1;
        }

        mPollVideoStreamId = -1;
        return 0;
    }

    public int publishPollCameraVideo(int resources, int syncId,
                                      VideoParameter param) {
        CustomLog.d(TAG, "SMSDKControl::publishPollCameraVideo2() resources: "
                + resources + " | syncId: " + syncId);
        if (param != null) {
            param.logSelf();
        }
        if (mStatus != StatusCode.INIT) {
            CustomLog
                    .e(TAG,
                            "SMSDKControl::publishPollCameraVideo2() 未进行初始化，error. return -1");
            return -1;
        }
        mPollVideoParameter = getVideoParam();

//        int preViewWidth = getVideoParam().getCapWidth();
//        int preViewHeight = getVideoParam().getCapHeight();
//        int fps = getVideoParam().getCapFps();
//        int encBitrate = getVideoParam().getEncBitrate();

        int pollWidth = getVideoParam(999).getCapWidth();
        int pollHeight = getVideoParam(999).getCapHeight();
        int pollFps = getVideoParam(999).getCapFps();
        int pollBitrate = getVideoParam(999).getEncBitrate();

        if (mCodecType == 0) {
            mPollVideoStreamId = SmSdkJNI.SM_SDK_PublishCameraVideo(
                    SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SEND, resources,
                    SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_CAMRA,
                    SmSdkJNI.SMSDK_SOURCE_TYPE_DEVICE, null, null,
                    SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK, pollWidth,
                    pollHeight, pollFps, pollWidth, pollHeight, pollFps,
                    pollBitrate, SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
                    SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264,
                    SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_SOFT, videoUpFecDataCount,
                    videoUpFecCheckCount, videoUpDataSendCount,
                    videoUpCheckSendCount, videoUpDataRetransSendCount,
                    videoUpCheckRetransSendCount, videoUpDataResendCount);
        } else {
//            mPollVideoStreamId = SmSdkJNI.SM_SDK_publishMultiStreamCameraVideo(
//                    SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SENDMR, resources,
//                    SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_CAMRA,
//                    SmSdkJNI.SMSDK_SOURCE_TYPE_DEVICE, null, null,
//                    SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK,
//                    preViewWidth, preViewHeight, fps, preViewWidth,
//                    preViewHeight, fps, encBitrate,
//                    SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
//                    SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264,
//                    SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_HARD,
//                    SmSdkJNI.SMSDK_STREAM_IDLE, pollWidth, pollHeight,
//                    pollFps, pollBitrate,
//                    SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
//                    SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264,
//                    SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_HARD,
//                    SmSdkJNI.SMSDK_STREAM_RUNNING, videoUpFecDataCount,
//                    videoUpFecCheckCount, videoUpDataSendCount,
//                    videoUpCheckSendCount, videoUpDataRetransSendCount,
//                    videoUpCheckRetransSendCount, videoUpDataResendCount);

            mPollVideoStreamId = SmSdkJNI.SM_SDK_PublishCameraVideo(
                    SmSdkJNI.SMSDK_STREAM_TYPE_VIDEO_SEND, resources,
                    SmSdkJNI.SMSDK_MEDIA_TYPE_VIDEO_CAMRA,
                    SmSdkJNI.SMSDK_SOURCE_TYPE_DEVICE, null, null,
                    SmSdkJNI.SMSDK_FILL_MODE_RAWRATIO_WITHBLACK, pollWidth,
                    pollHeight, pollFps, pollWidth, pollHeight, pollFps,
                    pollBitrate, SmSdkJNI.SMSDK_CODEC_LEVEL_LOW,
                    SmSdkJNI.SMSDK_VIDEO_CODEC_ID_H264,
                    SmSdkJNI.SMSDK_VIDEO_CODEC_TYPE_HARD, videoUpFecDataCount,
                    videoUpFecCheckCount, videoUpDataSendCount,
                    videoUpCheckSendCount, videoUpDataRetransSendCount,
                    videoUpCheckRetransSendCount, videoUpDataResendCount);

//            if (mLocalSurfaceCallBack != null) {
//                mLocalSurfaceCallBack.setValues(mVideoStremId);
//            }
            // gexy add in 2018-11-19 16:20:18
//			if (mSurfaceView != null) {
//				CustomLog.d(TAG, "publishCameraVideoHard,mSurfaceView!=null");
//				SurfaceHolder holder = mSurfaceView.getHolder();
//
//				CustomLog.d(TAG, "publishCameraVideoHard:SM_SDK_SetDisplayWindow");
//				int result = SmSdkJNI.SM_SDK_SetDisplayWindow(mVideoStremId,
//						(Object) holder.getSurface(), FILL_MODE_RAWRATIO_WITHBLACK,
//						FILL_MODE_RAWRATIO_WITHBLACK);
//				if (result == 0) {
//					CustomLog
//							.d(TAG,
//									"publishCameraVideoHard:TYPE_VIDEO,SM_SDK_SetDisplayWindow success");
//				} else {
//					CustomLog.e(TAG,
//							"publishCameraVideoHard:TYPE_VIDEO,SM_SDK_SetDisplayWindow fail, result:"
//									+ result);
//				}
//				mSurfTex = (SurfaceTexture) SmSdkJNI
//						.SM_SDK_GetDisplayWindow(mVideoStremId);
//				CustomLog.d(TAG, "publishCameraVideoHard startCameraPreview");
//				startCameraPreview(holder);
//			} else {
//				CustomLog.d(TAG, "publishCameraVideoHard,mSurfaceView==null");
//			}
        }

        CustomLog.d(TAG, "publishPollCameraVideo mPollVideoStreamId=" + mPollVideoStreamId);
        return mPollVideoStreamId;
    }

    private boolean isInBigStreamList(String accountId) {
        for (int i = 0; i < mBigStreamList.size(); i++) {
            if (mBigStreamList.get(i).equals(accountId)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 设置为收大流
     *
     * @param list
     */
    public void setBigStream(List<String> list) {
        CustomLog.d(TAG, "setBigStream");
        mBigStreamList = list;
        List<String> allList = mMeetingInfo.getSpeakersAccountIdList();
        for (int i = 0; i < list.size(); i++) {
            CustomLog.d(TAG, "第" + String.valueOf(i) + "个视讯号：" +
                    list.get(i));
            SpeakerInfo speakerInfo = mMeetingInfo.getSpeakerByAccountId(list
                    .get(i));
            if (speakerInfo == null
                    || list.get(i).equals(mMeetingInfo.accountId)) {
                CustomLog.d(TAG, "speakerInfo为空或是自己，不做处理");
            } else {
                if (speakerInfo.getStreamInfo(StreamInfo.RESOURCE_TYPE_VIDEO) == null) {
                    CustomLog.d(TAG, speakerInfo.getAccountId() + "无视频流");
                } else {
                    int streamId = speakerInfo.getStreamInfo(
                            StreamInfo.RESOURCE_TYPE_VIDEO).getStreamId();
                    CustomLog.d(TAG,
                            "设置大流，accountId:" + speakerInfo.getAccountId()
                                    + " streamId:" + String.valueOf(streamId));
                    int result = SmSdkJNI.SM_SDK_SetMRDisplayMode(streamId,
                            SmSdkJNI.SM_SDK_displayWin_adaption_mode,
                            SmSdkJNI.SM_SDK_displayWin_big_wintype);
                    CustomLog.d(TAG, "result:" + String.valueOf(result));
                }
                allList.remove(list.get(i));
            }
        }
        for (int j = 0; j < allList.size(); j++) {
            SpeakerInfo speakerInfo = mMeetingInfo
                    .getSpeakerByAccountId(allList.get(j));
            if (speakerInfo == null || allList.get(j).equals(mAccountId)) {
                CustomLog.d(TAG, "speakerInfo为空或是自己，不做处理");
            } else {
                if (speakerInfo.getStreamInfo(StreamInfo.RESOURCE_TYPE_VIDEO) == null) {
                    CustomLog.d(TAG, speakerInfo.getAccountId() + "无视频流");
                } else {
                    int streamId = speakerInfo.getStreamInfo(
                            StreamInfo.RESOURCE_TYPE_VIDEO).getStreamId();
                    CustomLog.d(TAG,
                            "设置小流，accountId:" + speakerInfo.getAccountId()
                                    + " streamId:" + String.valueOf(streamId));
                    int result = SmSdkJNI.SM_SDK_SetMRDisplayMode(streamId,
                            SmSdkJNI.SM_SDK_displayWin_adaption_mode,
                            SmSdkJNI.SM_SDK_displayWin_small_wintype);
                    CustomLog.d(TAG, "result:" + String.valueOf(result));
                }
            }
        }
    }
}
