package cn.redcdn.butelopensdk.meetingcontrol;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;

import cn.redcdn.butelopensdk.constconfig.CmdId;
import cn.redcdn.butelopensdk.constconfig.CmdKey;
import cn.redcdn.butelopensdk.constconfig.RespModelStatusCode;
import cn.redcdn.butelopensdk.constconfig.SpeakerInfo;
import cn.redcdn.butelopensdk.util.ButelCommonUtil;
import cn.redcdn.butelopensdk.vo.Cmd;
import cn.redcdn.commonutil.NetConnectHelper;
import cn.redcdn.log.CustomLog;
import cn.redcdn.meeting.MeetingCtrlAgent;
import cn.redcdn.network.udp.UDPProcessor;
import cn.redcdn.network.udp.UDPReceiver;
import cn.redcdn.network.udp.WhiteBoardReceiver;

import com.redcdn.keyeventwrite.KeyEventConfig;
import com.redcdn.keyeventwrite.KeyEventWrite;

public class MeetingControl {
    private final String TAG = MeetingControl.this.getClass().getName();

    public interface MeetingControlListener {
        public void onReceiveCmd(Cmd cmd);

        public void onReceiveWhiteBoardData(byte[] data);
    }

    public class MeetingControlReturnCode {
        public static final int SUCCESS = 0;

        public static final int NOT_INIT = 1;

        public static final int REPEATED_CALL = 2;

        public static final int IN_PENDING = 3;
    }

    private class MeetingControlStatus {
        public static final int NOT_INIT = 0;

        public static final int INITED = 1;

        public static final int JOIN_MEETING = 2;

        public static final int EXCEPTION = 3;
    }

    private class MeetingControlOperation {
        public static final int NULL = 0;

        public static final int JOIN_MEETING = 1;

        public static final int GET_PARTICIPATOR_LIST = 2;

        public static final int ASK_FOR_SPEAK = 3;

        public static final int ASK_FOR_STOP_SPEAK = 4;

        public static final int LOCK_MEETING = 5;

        public static final int GIVE_MIC = 6;

        public static final int EXIT_MEETING = 7;

        public static final int ASK_FOR_START_EPISODE = 8;

        public static final int ASK_FOR_STOP_EPISODE = 9;

        public static final int ASK_FOR_OPEN_CAMERA = 10;

        public static final int ASK_FOR_CLOSE_CAMERA = 11;

        public static final int MASTER_CHANGE_MEETING_MODE = 12;

        public static final int MASTER_SET_USER_START_ON_MIC = 13;

        public static final int MASTER_SET_USER_STOP_ON_MIC = 14;

        public static final int USER_ASK_FOR_RAISE_HAND = 15;

        public static final int ASK_FOR_START_LIVE = 16;

        // v2.0
        public static final int APPLAY_FOR_START_SHARING = 17;

        public static final int APPLAY_FOR_STOP_SHARING = 18;

        public static final int ASK_FOR_SET_MAIN_SPEAKER = 19;

        public static final int ASK_PUBLISH_OR_UNPUBLISH_STREAMS = 20;

        public static final int ASK_UPDATE_STREAMS = 21;

        public static final int ASK_KICK_USER = 22;// 请出会议室

        @SuppressWarnings("unused")
        public static final int SEND_VIEWDATA_CONTROL = 23;// 发送反向控制命令

        public static final int ASK_FOR_OPEN_LOUDSPEAKER = 24;

        public static final int ASK_FOR_CLOSE_LOUDSPEAKER = 25;

        public static final int MASTER_OPERATE_USER_LOUDSPEAKER = 26; // 主持人指定某用户操作扬声器

        public static final int MASTER_OPERATE_USER_MEDIA_STATE = 27; // 主持人指定某用户操作媒体状态

        public static final int MASTER_ASK_FOR_END_MEETING = 28; // 主持人申请结束会议请求命令

        public static final int ASK_FOR_START_RECORDING = 29; // 开始录制请求

        public static final int ASK_FOR_STOP_RECORDING = 30; // 停止录制请求

        public static final int MASTER_TRANSFER_MASTERID = 31; // 主持人权限移交

        public static final int MASTER_CHANGE_CHAT_STATE = 32; // 主持人改变聊天状态

        public static final int GET_SERVER_BLACK_CACHE_CMD = 33; // 获取黑盒缓存命令
    }

    private int mStatus = MeetingControlStatus.NOT_INIT;

    private int mOperation = MeetingControlOperation.NULL;

    private Context mContext;

    private MeetingControlListener mMeetingControlListener;

    private int mMeetingId;

    private String mAccountId;

    /******** UDPProcessor相关 ********/
    private static final int MSG_NOTIFY_GET_CMD = 0;
    private static final int MSG_NOTIFY_GET_WHITEBOARD_DATA = 1;

    private static final int MSG_PEND_PUBLISH = 1;
    private static final int MSG_PEND_UPDATE_STREAMS = 2;

    private static final int DELAY_TIME_PEND_PUBLISH = 500;

    private Map<Integer, ArrayList<DevidedCmdInfo>> innerCmdMap;
    private String mAccountName;
    private String lastLayoutCmd = "";
    private String lastCaptionCmd = "";
    private String lastThemeCmd = "";

    private Map<String, Integer> cacheCmdMap = new HashMap<>();
    private boolean isPublishStream = false;

    private Handler notifyGetCmdHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case MSG_NOTIFY_GET_CMD:
                    filterCmd(analyzeCmd((String) msg.obj));
                    break;

                default:
                    break;
            }
            super.handleMessage(msg);
        }
    };

    private Handler pendingPublishHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case MSG_PEND_PUBLISH:
                    CustomLog.d(TAG,
                            "MeetingControl::pendPublishHandler 触发激活pending中的发布操作");
                    Bundle data = msg.getData();
                    CustomLog.d(
                            TAG,
                            "data.getString('jaStreams')="
                                    + data.getString("jaStreams"));
                    try {
                        askPublishOrUnPublishStreams(data.getString("token"),
                                msg.arg1,
                                (new JSONArray(data.getString("jaStreams"))));
                    } catch (JSONException e) {
                        CustomLog.e(TAG, e.toString());
                    }
                    break;
                case MSG_PEND_UPDATE_STREAMS:
                    CustomLog.d(TAG,
                            "MeetingControl::pendPublishHandler 触发激活pending中的发布操作");
                    Bundle updateData = msg.getData();
                    try {
                        askUpdateStreams(updateData.getString(CmdKey.TOKEN),
                                updateData.getString(CmdKey.RESOURCE_ID), msg.arg1,
                                updateData.getString(CmdKey.EXTRAINFO));
                    } catch (Exception e) {
                        CustomLog.e(TAG, e.toString());
                    }
                    break;
            }
        }
    };

    private UDPProcessor mUDPProcessor;

    private UDPReceiver udpReceiver = new UDPReceiver() {
        @Override
        public void process(String string) {
            // CustomLog.d(TAG, "udpReceiver=" + string);
            // Message msg = Message.obtain();
            // msg.what = MSG_NOTIFY_GET_CMD;
            // msg.obj = string;
            // notifyGetCmdHandler.sendMessage(msg);
            checkIsCmd3014(string);
        }
    };

    private Handler notifyGetWhiteBoardHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case MSG_NOTIFY_GET_WHITEBOARD_DATA:
                    mMeetingControlListener
                            .onReceiveWhiteBoardData((byte[]) msg.obj);
                    // analyzeData((byte[]) msg.obj);
                    break;
                default:
                    break;
            }
            super.handleMessage(msg);
        }
    };

    private UDPProcessor mWhiteBoardProcessor;

    private WhiteBoardReceiver whiteBoardReceiver = new WhiteBoardReceiver() {

        @Override
        public void process(byte[] bytes) {

//            CustomLog.d(TAG, "whiteBoardReceiver bytes.length=" + bytes.length);
            // mMeetingControlListener.onReceiveWhiteBoardData(bytes);

            Message msg = Message.obtain();
            msg.what = MSG_NOTIFY_GET_WHITEBOARD_DATA;
            msg.obj = bytes;
            notifyGetWhiteBoardHandler.sendMessage(msg);
        }
    };

    private boolean hasStartedTimer = false;
    private boolean hasSent4013 = false;
    private Handler timerHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case 0:
                    timeUp();
                    break;
            }
        }
    };

    @SuppressLint("UseSparseArrays")
    private void checkIsCmd3014(String string) {
        // CustomLog.d(TAG, "checkIsCmd3014");
        try {
            JSONObject jsonObject = new JSONObject(string);
            int cmdId = jsonObject.optInt("cmdID");

            if (cmdId == CmdId.DEVIDE_PACKAGE_RESP) {
                CustomLog.d(TAG, "is cmd3014,string:" + string);

                if (innerCmdMap == null) {
                    innerCmdMap = new HashMap<Integer, ArrayList<DevidedCmdInfo>>();
                }

                DevidedCmdInfo devidedCmdInfo = new DevidedCmdInfo();
                devidedCmdInfo.setIndex(jsonObject.optInt("index"));
                devidedCmdInfo.setTotalNum(jsonObject.optInt("totalNum"));
                devidedCmdInfo.setInnerCmdId(jsonObject.optInt("innerCmdID"));
                devidedCmdInfo.setContent(jsonObject.optString("content"));

                CustomLog
                        .d(TAG,
                                "index:"
                                        + String.valueOf(devidedCmdInfo
                                        .getIndex())
                                        + " totalNum:"
                                        + String.valueOf(devidedCmdInfo
                                        .getTotalNum())
                                        + " innerCmdID:"
                                        + String.valueOf(devidedCmdInfo
                                        .getInnerCmdId()));

                if (innerCmdMap.get(devidedCmdInfo.getInnerCmdId()) == null) {
                    ArrayList<DevidedCmdInfo> devidedCmdInfoList = new ArrayList<DevidedCmdInfo>();

                    devidedCmdInfoList.add(devidedCmdInfo);
                    innerCmdMap.put(devidedCmdInfo.getInnerCmdId(),
                            devidedCmdInfoList);

                } else {
                    innerCmdMap.get(devidedCmdInfo.getInnerCmdId()).add(
                            devidedCmdInfo);
                }

                if (innerCmdMap.get(devidedCmdInfo.getInnerCmdId()).size() == innerCmdMap
                        .get(devidedCmdInfo.getInnerCmdId()).get(0)
                        .getTotalNum()) {
                    if (devidedCmdInfo.getInnerCmdId() == CmdId.NEW_GET_PARTICIPATORS_RESP) {
                        CustomLog.d(TAG, "4014 end");
                        hasStartedTimer = false;
                        timerHandler.removeCallbacksAndMessages(null);
                    }
                    StringBuilder stringBuilder = new StringBuilder();

                    for (int i = 0; i < innerCmdMap.get(
                            devidedCmdInfo.getInnerCmdId()).size(); i++) {
                        stringBuilder.append(innerCmdMap
                                .get(devidedCmdInfo.getInnerCmdId()).get(i)
                                .getContent());
                    }

                    String combinedString = stringBuilder.toString();
                    innerCmdMap.remove(innerCmdMap.get(devidedCmdInfo
                            .getInnerCmdId()));

                    CustomLog.d(TAG, "combinedString:" + combinedString);
                    Message msg = Message.obtain();
                    msg.what = MSG_NOTIFY_GET_CMD;
                    msg.obj = combinedString;
                    notifyGetCmdHandler.sendMessage(msg);
                }

            } else {
                // CustomLog.d(TAG, "is not cmd3014");
                if (cmdId == CmdId.NEW_GET_PARTICIPATORS_RESP) {
                    CustomLog.d(TAG, "single 4014");
                    hasStartedTimer = false;
                    timerHandler.removeCallbacksAndMessages(null);
                }
                Message msg = Message.obtain();
                msg.what = MSG_NOTIFY_GET_CMD;
                msg.obj = string;
                notifyGetCmdHandler.sendMessage(msg);
            }
        } catch (JSONException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    private void timeUp() {
        CustomLog.d(TAG, "timeUp");
        try {
            CustomLog.d(TAG, "timeUp,4014 end");
            hasStartedTimer = false;
            timerHandler.removeCallbacksAndMessages(null);
            String combinedString = "{\"audience\":0,\"cmdID\":4014,\"meetingId\":"
                    + String.valueOf(mMeetingId)
                    + ",\"meetingName\":\""
                    + mAccountName + "\",\"paticipators\":[],\"status\":0}";

            CustomLog.d(TAG, "combinedString:" + combinedString);
            Message msg = Message.obtain();
            msg.what = MSG_NOTIFY_GET_CMD;
            msg.obj = combinedString;
            notifyGetCmdHandler.sendMessage(msg);
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * 构造方法
     *
     * @param meetingControlListener 通过此监听，接收命令
     * @param meetingId              会议号
     * @param accountId              帐号id
     */
    public MeetingControl(Context context) {
        mContext = context;
    }

    public int init(MeetingControlListener meetingControlListener,
                    int meetingId, String accountId, String accountName, String rcUrl,
                    String cfgPath, String logFileOutPath, boolean whiteBoardFlag) {

        int result = 0;

        mMeetingId = meetingId;
        mAccountId = accountId;
        mAccountName = accountName;
        mMeetingControlListener = meetingControlListener;
        int localPort = initUDPProcessor();
        int meetingControlAgentPort = initMeetingControlAgent(localPort, rcUrl,
                cfgPath, logFileOutPath);
        setUDPTargetPort(meetingControlAgentPort);
        CustomLog.d(TAG, "localPort = " + localPort + "; remotePort = "
                + meetingControlAgentPort);

        if (whiteBoardFlag) {
            int whiteBoardLocalPort = initWhiteBoradProcessor();
            int whiteBoardAgentPort = startBlackPipe("127.0.0.1",
                    whiteBoardLocalPort);
            setWhiteBoardTargetPort(whiteBoardAgentPort);
            CustomLog.d(TAG, "whiteBoardLocalPort = " + whiteBoardLocalPort
                    + "; whiteBoardAgentPort = " + whiteBoardAgentPort);
        }

        mStatus = MeetingControlStatus.INITED;
        if (localPort < 0 || meetingControlAgentPort <= 0) {
            result = -1;
        }
        return result;
    }

    public int SetChipInParam(String chipInIp, int chipInPort) {
        return MeetingCtrlAgent.SetChipInParam(chipInIp, chipInPort);
    }

    public int GetChipInPort() {
        return MeetingCtrlAgent.GetMCAgentChipInPort();
    }

    /**
     * 加入会议
     *
     * @param token
     * @param adminId
     * @param memberStatus
     * @param attendMode
     * @param userName
     * @param streamDesp   流类型描述：用一个字节表示，高4位置0，低4位表示四种流类型是否打开的状态，1为打开，0为关闭
     * @return
     */
    public int joinMeeting(String token, String adminId, int memberStatus,
                           int attendMode, String userName, int streamDesp,
                           String relayIdwhiteList, boolean isAutoSpek, boolean adminFlag, boolean pollFlag) {
        CustomLog.d(TAG,
                "MeetingControl::joinMeeting() 加入会议. relayIdwhiteList: "
                        + relayIdwhiteList);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mStatus == MeetingControlStatus.JOIN_MEETING) {
            CustomLog.d(TAG, "已经处于加入会议状态，不需要再次加入");
            return MeetingControlReturnCode.REPEATED_CALL;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以加入会议");
            mOperation = MeetingControlOperation.JOIN_MEETING;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.JOIN_MEETING);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                // jsonObject.put("adminId", adminId); 不需要再传递 adminId
                jsonObject.put("memberStatus", memberStatus);
                jsonObject.put("attendMode", attendMode);
                jsonObject.put("userName", userName);
                jsonObject.put(CmdKey.STREAM_TYPES, streamDesp);
                jsonObject.put("loudSpeakerStatus", 1); // loudSpeakerStatus:扬声器状态（类型int）
                // 1:打开 2:关闭
                if (adminFlag) {
                    jsonObject.put("isAutoSpeak", 2);
                    jsonObject.put("userType", 5);
                } else {
                    // isAutoSpeak:是否需要自动发言（类型int）1:自动发言2.不自动发言
                    jsonObject.put("isAutoSpeak", isAutoSpek ? 1 : 2);
                }
                if (pollFlag) {
                    jsonObject.put("userDeviceType", 1);
                }
                JSONArray relayIdArray = new JSONArray();

                if (!TextUtils.isEmpty(relayIdwhiteList)) {
                    String[] relayIds = relayIdwhiteList.split("\\|");
                    for (String s : relayIds) {
                        JSONObject realyJson = new JSONObject();
                        CustomLog.d(TAG, "relayIdwhiteList: " + s);
                        try {
                            int id = Integer.valueOf(s);
                            realyJson.put("id", id);
                            relayIdArray.put(realyJson);
                        } catch (Exception e) {
                            CustomLog.d(
                                    TAG,
                                    "Integer.valueOf(s) error! "
                                            + e.getMessage());
                        }
                    }
                }

                if (relayIdArray.length() != 0) {
                    jsonObject.put(CmdKey.WHITE_LIST, relayIdArray);
                }
            } catch (Exception e) {
                CustomLog.e(TAG, "joinMeeting error! " + e.getMessage());
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 加入会议
     *
     * @param token
     * @param adminId
     * @param memberStatus
     * @param attendMode
     * @param userName
     * @param streamDesp   流类型描述：用一个字节表示，高4位置0，低4位表示四种流类型是否打开的状态，1为打开，0为关闭
     * @return
     */
    public int joinMeeting(String token, String adminId, int memberStatus,
                           int attendMode, String userName, int streamDesp,
                           String relayIdwhiteList, boolean isAutoSpek, int userType,
                           String mpuId, String mpuIp, String mpuPort) {
        CustomLog.d(TAG,
                "MeetingControl::joinMeeting() 加入会议. relayIdwhiteList: "
                        + relayIdwhiteList);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mStatus == MeetingControlStatus.JOIN_MEETING) {
            CustomLog.d(TAG, "已经处于加入会议状态，不需要再次加入");
            return MeetingControlReturnCode.REPEATED_CALL;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以加入会议");
            mOperation = MeetingControlOperation.JOIN_MEETING;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.JOIN_MEETING);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                // jsonObject.put("adminId", adminId); 不需要再传递 adminId
                jsonObject.put("memberStatus", memberStatus);
                jsonObject.put("attendMode", attendMode);
                jsonObject.put("userName", userName);
                jsonObject.put(CmdKey.STREAM_TYPES, streamDesp);
                jsonObject.put("isAutoSpeak", isAutoSpek ? 1 : 2); // isAutoSpeak:是否需要自动发言（类型int）1:自动发言2.不自动发言
                jsonObject.put("loudSpeakerStatus", 1); // loudSpeakerStatus:扬声器状态（类型int）
                // 1:打开 2:关闭
                JSONArray relayIdArray = new JSONArray();

                if (!TextUtils.isEmpty(relayIdwhiteList)) {
                    String[] relayIds = relayIdwhiteList.split("\\|");
                    for (String s : relayIds) {
                        JSONObject realyJson = new JSONObject();
                        CustomLog.d(TAG, "relayIdwhiteList: " + s);
                        try {
                            int id = Integer.valueOf(s);
                            realyJson.put("id", id);
                            relayIdArray.put(realyJson);
                        } catch (Exception e) {
                            CustomLog.d(
                                    TAG,
                                    "Integer.valueOf(s) error! "
                                            + e.getMessage());
                        }
                    }
                }

                if (relayIdArray.length() != 0) {
                    jsonObject.put(CmdKey.WHITE_LIST, relayIdArray);
                }

                jsonObject.put("userType", userType);
                jsonObject.put("MPUId", mpuId);
                jsonObject.put("MPUIp", mpuIp);
                jsonObject.put("MPUPort", mpuPort);

            } catch (Exception e) {
                CustomLog.e(TAG, "joinMeeting error! " + e.getMessage());
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 退出会议
     *
     * @param token
     * @param exitMode
     * @return
     */
    public int exitMeeting(String token, int exitMode) {
        CustomLog.d(TAG, "发送退出会议命令");
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        }
//        else if (mOperation != MeetingControlOperation.NULL) {
//            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
//            return MeetingControlReturnCode.IN_PENDING;
//        }
        else {
            CustomLog.d(TAG, "状态合法，可以退出会议");
            mOperation = MeetingControlOperation.EXIT_MEETING;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.EXIT_MEETING);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put(CmdKey.EXIT_MODE, exitMode);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 获取参会者列表
     *
     * @param token
     * @return
     */
    public int getParticipatorList(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        CustomLog.d(TAG, "发送获取参会者列表命令");
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以获取参会者列表");
            mOperation = MeetingControlOperation.GET_PARTICIPATOR_LIST;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.NEW_GET_PARTICIPATORS);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);

                if (!hasStartedTimer) {
                    hasStartedTimer = true;
                    Message msg = new Message();
                    msg.what = 0;
                    timerHandler.sendMessageDelayed(msg, 5000);
                }

                hasSent4013 = true;
            } catch (JSONException e) {
                e.printStackTrace();
            }
            // if (MeetingControlCmdTest.isDebug) {
            // sendTestMsg(MeetingControlCmdTest.get_meeting_partner_list_response);
            // } else {
            // mUDPProcessor.send(jsonObject.toString());
            // }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 申请发言
     *
     * @param token
     * @param micId
     * @return
     */
    public int askForSpeak(String token, String SpeakerUserId) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        CustomLog.d(TAG, "发送申请发言命令");
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以申请发言");
            mOperation = MeetingControlOperation.ASK_FOR_SPEAK;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_SPEAK);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("SpeakerUserId", SpeakerUserId);
                // jsonObject.put("applyMode", 1);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 申请停止发言
     *
     * @param token
     * @return
     */
    public int askForStopSpeak(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        CustomLog.d(TAG, "发送申请停止发言命令");
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以申请停止发言");
            mOperation = MeetingControlOperation.ASK_FOR_STOP_SPEAK;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_STOP_SPEAK);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 加解锁会议
     *
     * @param token
     * @param lockInfo
     * @return
     */
    public int lockMeeting(String token, int lockInfo) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        CustomLog.d(TAG, "发送加|解锁会议命令");
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以发送加|解锁会议命令");
            mOperation = MeetingControlOperation.LOCK_MEETING;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.LOCK_MEETING);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("lockInfo", lockInfo);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 传麦
     *
     * @param token
     * @param beSpeakedUserId
     * @param micId
     * @return
     */
    public int giveMic(String token, String beSpeakedUserId) {
        // CustomLog.d(TAG, "发送传麦命令");
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "发送传麦命令");
            mOperation = MeetingControlOperation.GIVE_MIC;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.GIVE_MIC);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("beSpeakedUserId", beSpeakedUserId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * 请求插话
     */
    public int askForStartEpisode(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以申请开始插话");
            mOperation = MeetingControlOperation.ASK_FOR_START_EPISODE;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_START_EPISODE);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * 停止插话
     */
    public int askForStopEpisode(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以申请开始插话");
            mOperation = MeetingControlOperation.ASK_FOR_STOP_EPISODE;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_STOP_EPISODE);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    public int askForOpenCamera(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.e(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.e(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.e(TAG, "状态合法，可以申请打开摄像头");
            mOperation = MeetingControlOperation.ASK_FOR_OPEN_CAMERA;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_OPENORCLOSE_CAMERA);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("operatorId", 1);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    public int askForCloseCamera(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以申请打开摄像头");
            mOperation = MeetingControlOperation.ASK_FOR_CLOSE_CAMERA;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_OPENORCLOSE_CAMERA);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("operatorId", 2);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * 主持人改变会议模式
     *
     * @param token
     * @param meetingStyle
     * @return
     */
    public int masterChangeMeetingMode(String token, int meetingStyle) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以申请改变会议模式");
            mOperation = MeetingControlOperation.MASTER_CHANGE_MEETING_MODE;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.MASTER_CHANGE_MEETING_MODE);
                jsonObject.put("token", token);
                jsonObject.put("meetingStyle", meetingStyle);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * 主持人权限移交
     *
     * @param token
     * @param newMasterId
     * @return
     */
    public int masterTransferMasterId(String token, String newMasterId) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以申请主持人权限移交");
            mOperation = MeetingControlOperation.MASTER_TRANSFER_MASTERID;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.MASTER_TRANSFER_MASTERID);
                jsonObject.put("token", token);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("newMasterId", newMasterId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * 主持人申请结束会议
     *
     * @param token
     * @return
     */
    public int masterAskForEndMeeting(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        }
        // else if (mOperation != MeetingControlOperation.NULL) {
        // CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
        // return MeetingControlReturnCode.IN_PENDING;
        // }
        else {
            CustomLog.d(TAG, "状态合法，主持人可以申请结束会议");
            mOperation = MeetingControlOperation.MASTER_ASK_FOR_END_MEETING;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.MASTER_ASK_FOR_END_MEETING);
                jsonObject.put("token", token);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("meetingId", mMeetingId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 主持人指定用户发言 <Li>如果发言者账号不为空，则抢占该发言者；如果发言者账号为空，则视是否有空闲指标来决定是否发言
     *
     * @param token
     * @param beSpeakedUserId
     * @param speakerUserId
     * @return
     */
    public int masterSetUserStartSpeakOnMic(String token,
                                            String beSpeakedUserId, String speakerUserId) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，主持人指定用户在mic发言");
            mOperation = MeetingControlOperation.MASTER_SET_USER_START_ON_MIC;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID",
                        CmdId.MASTER_SET_USER_START_SPEAK_ON_MIC);
                jsonObject.put(CmdKey.BE_SPEAKED_USER_ID, beSpeakedUserId);
                jsonObject.put(CmdKey.SPEAKER_USER_ID, speakerUserId);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * 主持人指定用户在mic停止发言
     *
     * @param token
     * @param accountId
     * @param beSpeakedUserId
     * @param micId
     * @return
     */
    public int masterSetUserStopSpeakOnMic(String token, String beSpeakedUserId) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，主持人指定用户在mic停止发言");
            mOperation = MeetingControlOperation.MASTER_SET_USER_STOP_ON_MIC;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject
                        .put("cmdID", CmdId.MASTER_SET_USER_STOP_SPEAK_ON_MIC);
                jsonObject.put(CmdKey.BE_SPEAKED_USER_ID, beSpeakedUserId);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    public int masterKickUser(String token, String beKickedUserId) {
        CustomLog.d(TAG, "MeetingControl::masterKickUser() mOperation = "
                + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，主持人将参会方踢出会议: " + beKickedUserId);
            mOperation = MeetingControlOperation.ASK_KICK_USER;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.MASTER_KICK_USER);
                jsonObject.put(CmdKey.BE_KICKED_USERID, beKickedUserId);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * userAskForRaiseHand
     *
     * @param token
     * @return
     */
    public int userAskForRaiseHand(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，参会人申请举手");
            mOperation = MeetingControlOperation.USER_ASK_FOR_RAISE_HAND;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.USER_ASK_FOR_RAISE_HAND);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * askForStartLive
     *
     * @param token
     * @return
     */
    public int askForStartLive(String token, String liveName) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法,可以申请开启直播");
            mOperation = MeetingControlOperation.ASK_FOR_START_LIVE;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_START_LIVE);
                jsonObject.put("token", token);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("liveName", liveName);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /***
     * 申请屏幕分享
     *
     * @param token
     * @return
     */
    public int applyForStartScreenSharing(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            return optValidity;
        }

        mOperation = MeetingControlOperation.APPLAY_FOR_START_SHARING;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("cmdID", CmdId.APPLY_FOR_START_SCREEN_SHAREING);
            jsonObject.put("token", token);
            jsonObject.put("meetingId", mMeetingId);
            jsonObject.put("accountId", mAccountId);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    /***
     * 申请停止屏幕分享
     *
     * @param token
     * @return
     */
    public int applyForStopScreenShareing(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            return optValidity;
        }

        mOperation = MeetingControlOperation.APPLAY_FOR_STOP_SHARING;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("cmdID", CmdId.APPLY_FOR_STOP_SCREEN_SHAREING);
            jsonObject.put("token", token);
            jsonObject.put("meetingId", mMeetingId);
            jsonObject.put("accountId", mAccountId);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    /***
     * 主持人指定主講人
     *
     * @param token
     * @param speakerUserId
     * @return
     */
    public int masterSetSpeaker(String token, String speakerUserId) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            return optValidity;
        }

        mOperation = MeetingControlOperation.ASK_FOR_SET_MAIN_SPEAKER;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("cmdID", CmdId.MASTER_SET_MAIN_SPEAKER);
            jsonObject.put("token", token);
            jsonObject.put("meetingId", mMeetingId);
            jsonObject.put("accountId", mAccountId);
            jsonObject.put(CmdKey.SPEAKER_USER_ID, speakerUserId);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    /**
     * 申请打开扬声器
     *
     * @param token type: 1. 已打开；2.已关闭
     */
    public int askForOpenLoudspeaker(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            return optValidity;
        }

        mOperation = MeetingControlOperation.ASK_FOR_OPEN_LOUDSPEAKER;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("cmdID", CmdId.ASK_FOR_OPERATE_LOUDSPEAKER);
            jsonObject.put("token", token);
            jsonObject.put("meetingId", mMeetingId);
            jsonObject.put("accountId", mAccountId);
            jsonObject.put("type", 1);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    /**
     * 申请关闭扬声器
     *
     * @param token
     */
    public int askForCloseLoudspeaker(String token) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            return optValidity;
        }

        mOperation = MeetingControlOperation.ASK_FOR_CLOSE_LOUDSPEAKER;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("cmdID", CmdId.ASK_FOR_OPERATE_LOUDSPEAKER);
            jsonObject.put("token", token);
            jsonObject.put("meetingId", mMeetingId);
            jsonObject.put("accountId", mAccountId);
            jsonObject.put("type", 2);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    /***
     * 申请发布、取消发布流，如果当前正在做其它操作，将会记录此发布、取消发布状态，等到 DELAY_TIME_PEND_PUBLISH 之后再进行尝试.
     * 1. 取消发言时不用取消发布视频、音频流； 2. 取消屏幕分享时不用取消发布文档视频流；
     *
     * @param token
     * @param operatorType
     *            1: 发布； 2: 取消发布
     * @param jaStreams
     *            [{“resourceId”:”6789”,”mediaType”:2,”extraInfo”:”abcd”}]
     *
     * @return
     */
    public int askPublishOrUnPublishStreams(String token, int operatorType,
                                            JSONArray jaStreams) {
        CustomLog.d(TAG, "mOperation = " + mOperation);
        CustomLog.d(TAG,
                "MeetingControl::askPublishOrUnPublishStreams() 发布、取消发布流: token: "
                        + token + " operatorType: " + operatorType
                        + " | jaStreams: " + jaStreams);
        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            if (optValidity == MeetingControlReturnCode.IN_PENDING) {
                CustomLog.d(TAG, "处于 pending 状态，需要缓存，稍后进行发布操作");
                isPublishStream = true;
                Message msg = new Message();
                msg.what = MSG_PEND_PUBLISH;
                Bundle data = new Bundle();
                data.putString("token", token);
                data.putString("jaStreams", jaStreams.toString());
                msg.arg1 = operatorType;
                msg.setData(data);
                CustomLog.d(TAG, "data=" + data.toString());
                pendingPublishHandler.sendMessageDelayed(msg,
                        DELAY_TIME_PEND_PUBLISH);
            }
            return optValidity;
        }

        isPublishStream = false;
        mOperation = MeetingControlOperation.ASK_PUBLISH_OR_UNPUBLISH_STREAMS;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("cmdID", CmdId.ASK_PUBLISH_OR_UNPUBLISH_STREAMS);
            jsonObject.put("token", token);
            jsonObject.put("meetingId", mMeetingId);
            jsonObject.put("accountId", mAccountId);
            jsonObject.put(CmdKey.OPERATOR_TYPE, operatorType);
            jsonObject.put(CmdKey.STREAMS, jaStreams);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        CustomLog.d(TAG, "jsonObject.toString() = " + jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    public int askUpdateStreams(String token, String resourceId, int mediaType,
                                String extraInfo) {
        CustomLog.d(TAG, "MeetingControl::askUpdateStreams() 更新流: token: "
                + token + " | resourceId: " + resourceId + " | mediaType: "
                + mediaType + " | extraInfo: " + extraInfo);

        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            if (optValidity == MeetingControlReturnCode.IN_PENDING) {
                CustomLog.d(TAG, "处于 pending 状态，需要缓存，稍后进行更新操作");
                Message msg = new Message();
                msg.what = MSG_PEND_UPDATE_STREAMS;
                Bundle data = new Bundle();
                data.putString(CmdKey.TOKEN, token);
                data.putString(CmdKey.RESOURCE_ID, resourceId);
                data.putString(CmdKey.EXTRAINFO, extraInfo);
                msg.arg1 = mediaType;
                msg.setData(data);
                CustomLog.d(TAG, "data=" + data.toString());
                pendingPublishHandler.sendMessageDelayed(msg,
                        DELAY_TIME_PEND_PUBLISH);
            }
            return optValidity;
        }

        mOperation = MeetingControlOperation.ASK_UPDATE_STREAMS;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put(CmdKey.CMD_ID, CmdId.ASK_UPDATE_STREAMS);
            jsonObject.put(CmdKey.TOKEN, token);
            jsonObject.put(CmdKey.ACCOUNT_ID, mAccountId);
            jsonObject.put(CmdKey.MEETING_ID, mMeetingId);
            jsonObject.put(CmdKey.RESOURCE_ID, resourceId);
            jsonObject.put(CmdKey.MEDIA_TYPE, mediaType);
            jsonObject.put(CmdKey.EXTRAINFO, extraInfo);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        CustomLog.d(TAG, "jsonObject.toString() = " + jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    public int switchToLocalRecording(String targetId) {
        CustomLog.d(TAG, "sendswitchToLocalRecording,targetId:" + targetId);

        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            return optValidity;
        }

        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put(CmdKey.CMD_ID, CmdId.UI_SEND_CMD_ID);
            jsonObject.put(CmdKey.UI_CMD_TARGET_ID, targetId);
            jsonObject.put(CmdKey.UI_CMD_FROM_ID, mAccountId);
            jsonObject.put(CmdKey.TIMESTAMP,
                    (int) (System.currentTimeMillis() / 1000));
            StringBuilder dataString = new StringBuilder();
            dataString.append("#SWITCH_TO_LOCAL_RECORDING#");
            jsonObject.put(CmdKey.UI_CMD_DATA, dataString.toString());
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        CustomLog.d(TAG, "jsonObject.toString() = " + jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    public int sendDataViewControlPoint(String dataviewerId, float valueX,
                                        float valueY) {
        CustomLog.d(TAG,
                "MeetingControl::sendDataViewControlPoint() UI透传反向控制数据视图操作: dataviewerId: "
                        + dataviewerId + " | valueX: " + valueX + " | valueY: "
                        + valueY);

        int optValidity = juideOperateValidity();
        if (optValidity != 0) {
            return optValidity;
        }

        // mOperation = MeetingControlOperation.SEND_VIEWDATA_CONTROL;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put(CmdKey.CMD_ID, CmdId.UI_SEND_CMD_ID);
            jsonObject.put(CmdKey.UI_CMD_TARGET_ID, dataviewerId);
            jsonObject.put(CmdKey.UI_CMD_FROM_ID, mAccountId);
            jsonObject.put(CmdKey.TIMESTAMP,
                    (int) (System.currentTimeMillis() / 1000));
            // {"cmdID":100,"targetId":"69600006","data":"0#69600001#640.03#355.72"}
            StringBuilder dataString = new StringBuilder();
            dataString.append("0#");
            dataString.append(mAccountId);
            dataString.append("#");
            dataString.append(valueX);
            dataString.append("#");
            dataString.append(valueY);
            jsonObject.put(CmdKey.UI_CMD_DATA, dataString.toString());
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        CustomLog.d(TAG, "jsonObject.toString() = " + jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    public int sendSharingPicCmd(String targetId, String picUrl) {
        CustomLog.d(TAG, "MeetingControl::sendSharingPicCmd() targetId: "
                + targetId + " | picUrl:" + picUrl);

        // int optValidity = juideOperateValidity();
        // if (optValidity != 0) {
        // return optValidity;
        // }

        // mOperation = MeetingControlOperation.SEND_VIEWDATA_CONTROL;
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put(CmdKey.CMD_ID, CmdId.UI_SEND_CMD_ID);
            jsonObject.put(CmdKey.UI_CMD_TARGET_ID, targetId);
            jsonObject.put(CmdKey.UI_CMD_FROM_ID, mAccountId);
            jsonObject.put(CmdKey.TIMESTAMP,
                    (int) (System.currentTimeMillis() / 1000));
            // {"cmdID":100,"targetId":"69600006","data":"SharingPicCmd#69600001#http://img.mp.itc.cn/upload/20170727/447d808561d045fa81b265b4fc273c2f_th.jpg"}
            StringBuilder dataString = new StringBuilder();
            dataString.append("SharingPicCmd");
            dataString.append("#");
            dataString.append(mAccountId);
            dataString.append("#");
            dataString.append(picUrl);
            jsonObject.put(CmdKey.UI_CMD_DATA, dataString.toString());
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        CustomLog.d(TAG, "jsonObject.toString() = " + jsonObject.toString());
        return MeetingControlReturnCode.SUCCESS;
    }

    /**
     * 卸载
     */
    public void release() {
        CustomLog.d(TAG, "反初始化MeetingControl");
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，不需要反初始化");
        } else {
            mUDPProcessor.destroy();
            mUDPProcessor = null;
            mWhiteBoardProcessor.destroy();
            mWhiteBoardProcessor = null;
            releaseMeetingControlAgent();
            stopBlackPipe();
        }

        mStatus = MeetingControlStatus.NOT_INIT;
        mOperation = MeetingControlOperation.NULL;

        CustomLog.d(TAG, "反初始化MeetingControl结束");
    }

    private int initUDPProcessor() {
        CustomLog.d(TAG, "初始化UPDProcessor");
        int localPort;
        mUDPProcessor = new UDPProcessor(udpReceiver);
        localPort = mUDPProcessor.init();
        CustomLog.d(TAG, "初始化UPDProcessor结束");
        return localPort;
    }

    private int initWhiteBoradProcessor() {
        CustomLog.d(TAG, "初始化initWhiteBoradProcessor");
        int localPort;
        mWhiteBoardProcessor = new UDPProcessor(whiteBoardReceiver);
        localPort = mWhiteBoardProcessor.init();
        CustomLog.d(TAG, "初始化initWhiteBoradProcessor结束");
        return localPort;
    }

    private int initMeetingControlAgent(int localPort, String rcUrl,
                                        String cfgPath, String logFileOutPath) {
        // String rcUrl = "222.73.29.12:21006|222.73.29.12:21008";
        String agentServerIP = "127.0.0.1";
        int agentServerPort = 7000;
        String agentClientIP = "127.0.0.1";
        int agentClientPort = localPort;
        // String localIPAddress = CommonUtil.getLocalIpAddress();
        String localIPAddress = NetConnectHelper.getLocalIp(mContext);
        int newRcUrlPort = 9044;

        // MeetingCtrlAgent.LoadBreakpad();

        MeetingCtrlAgent.Stop();
        MeetingCtrlAgent.SetInitParam(cfgPath, logFileOutPath);
        CustomLog.d(TAG, "rcUrl=" + rcUrl + "agentServerIP=" + agentServerIP
                + "agentServerPort=" + agentServerPort + "agentClientIP="
                + agentClientIP + "agentClientPort=" + agentClientPort
                + "localIPAddress=" + localIPAddress + "newRcUrlPort"
                + newRcUrlPort);
        int result = MeetingCtrlAgent.Start(rcUrl, agentServerIP,
                agentServerPort, agentClientIP, agentClientPort,
                localIPAddress, newRcUrlPort);
        if (result > 0) {
            String str = KeyEventConfig.INIT_MEETING_AGENT + "_ok_"
                    + mAccountId + "_初始化会控agent成功";
            KeyEventWrite.write(str);
        } else {
            String str = KeyEventConfig.INIT_MEETING_AGENT + "_fail_"
                    + mAccountId + "_初始化会控agent失败";
            KeyEventWrite.write(str);
        }
        return result;
    }

    private void releaseMeetingControlAgent() {
        CustomLog.d(TAG, "卸载MeetingControlAgent");
        MeetingCtrlAgent.Stop();
        String str = KeyEventConfig.UNINSTALL_MEETINGCONTROL + "_ok_"
                + mAccountId + "_卸载会控agent成功";
        KeyEventWrite.write(str);
        CustomLog.d(TAG, "卸载MeetingControlAgent结束");
    }

    private int setUDPTargetPort(int targetPort) {
        mUDPProcessor.setDest(targetPort);
        return 0;
    }

    private int setWhiteBoardTargetPort(int targetPort) {
        mWhiteBoardProcessor.setDest(targetPort);
        return 0;
    }

    private int startBlackPipe(String targetIP, int targetPort) {
        int result = MeetingCtrlAgent.StartBlackPipe(targetIP, targetPort);
        if (result > 0) {
            CustomLog.d(TAG, "startBlackPipe成功");
        } else {
            CustomLog.d(TAG, "startBlackPipe失败");
        }
        return result;
    }

    private void stopBlackPipe() {
        CustomLog.d(TAG, "stopBlackPipe");
        MeetingCtrlAgent.StopBlackPipe();
    }

    /**
     * 判断操作合法性
     *
     * @return </p> 0: 操作合法 </p>!=0 : 操作不合法
     */
    private int juideOperateValidity() {
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作 ：" + mOperation);
            return MeetingControlReturnCode.IN_PENDING;
        }
        return 0;
    }

    private Cmd analyzeCmd(String jsonString) {
        Cmd cmd = new Cmd();
        try {
            JSONObject jsonObject = new JSONObject(jsonString);
            CustomLog.e(TAG,
                    "MeetingControl::analyzeCmd() UDP接收到的的原始数据为:jsonObject="
                            + jsonObject);
            cmd.setCmdId(jsonObject.optInt(CmdKey.CMD_ID));
            cmd.setStatus(jsonObject.optInt(CmdKey.STATUS));
            cmd.setMeetingState(jsonObject.optInt(CmdKey.MEETING_STATE));
            cmd.setShowMode(jsonObject.optInt(CmdKey.SHOW_MODE));
            cmd.setMeetingId(jsonObject.optInt(CmdKey.MEETING_ID));
            cmd.setUserId(jsonObject.optString(CmdKey.USER_ID));
            cmd.setAccountId(jsonObject.optString(CmdKey.ACCOUNT_ID));
            cmd.setMeetingPresenceId(jsonObject
                    .optInt(CmdKey.MEETING_PRESENCE_ID));
            cmd.setRole(jsonObject.optInt(CmdKey.ROLE));
            cmd.setToken(jsonObject.optString(CmdKey.TOKEN));
            cmd.setPartnerListStr(jsonObject.optString(CmdKey.PARTNER_LIST));
            cmd.setParticipators(jsonObject.optJSONArray(CmdKey.PARTICIPATORS));
            cmd.setUserName(jsonObject.optString(CmdKey.USERNAME));
            cmd.setServerStatus(jsonObject.optInt(CmdKey.SERVER_STATUS));
            cmd.setMic1UserId(jsonObject.optString(CmdKey.MIC_1_USERID));
            cmd.setMic2UserId(jsonObject.optString(CmdKey.MIC_2_USERID));
            cmd.setLockInfo(jsonObject.optInt(CmdKey.LOCK_INFO));
            cmd.setCmdFlag(jsonObject.optInt(CmdKey.CMD_FLAG));
            cmd.setOrigUserName(jsonObject.optString(CmdKey.ORIG_USER_NAME));
            cmd.setMic1CamStaus(jsonObject.optInt(CmdKey.MIC1_CAM_STATUS));
            cmd.setMic2CamStaus(jsonObject.optInt(CmdKey.MIC2_CAM_STATUS));
            cmd.setOperatorId(jsonObject.optInt(CmdKey.OPERATOR_ID));
            cmd.setMasterId(jsonObject.optString(CmdKey.MASTER_ID));
            cmd.setMasterName(jsonObject.optString(CmdKey.MASTER_NAME));
            cmd.setUserType(jsonObject.optInt(CmdKey.USER_TYPE));
            cmd.setMeetingStyle(jsonObject.optInt(CmdKey.MEETING_STYLE));
            cmd.setLiveStatus(jsonObject.optInt(CmdKey.LIVE_STATUS));
            cmd.setSpeakers(jsonObject.optJSONArray(CmdKey.SPEAKERS));
            cmd.setStreams(jsonObject.optJSONArray(CmdKey.STREAMS));
            cmd.setStreamOperatorType(jsonObject
                    .optInt(CmdKey.STREAM_OPERATORTYPE));
            cmd.setNewStreamOperatorType(jsonObject
                    .optInt(CmdKey.STREAM_NEWOPERATORTYPE));
            cmd.setResourceId(jsonObject.optInt(CmdKey.RESOURCE_ID));
            cmd.setMediaType(jsonObject.optInt(CmdKey.MEDIA_TYPE));
            cmd.setExtraInfo(jsonObject.optString(CmdKey.EXTRAINFO));
            cmd.setType(jsonObject.optInt(CmdKey.OPERATE_TYPE));
            cmd.setLoudSpeakerStatus(jsonObject
                    .optInt(CmdKey.LOUDSPEAKER_STATUS));
            cmd.setRecServerAddr(jsonObject
                    .optString(CmdKey.REC_SERVER_ADDRESS));
            cmd.setFileAddress(jsonObject.optString(CmdKey.FILE_ADDRESS));
            // cmd.setAudience(jsonObject.optInt(CmdKey.AUDIENCE));
            cmd.setData(jsonObject.optString(CmdKey.UI_CMD_DATA));
            cmd.setTargetId(jsonObject.optString(CmdKey.UI_CMD_TARGET_ID));
            cmd.setFromId(jsonObject.optString(CmdKey.UI_CMD_FROM_ID));
            cmd.setIsRecord(jsonObject.optInt(CmdKey.IS_RECORD));
            cmd.setOriMasterId(jsonObject.optString(CmdKey.ORI_MASTERID));
            cmd.setOriMasterName(jsonObject.optString(CmdKey.ORI_MASTER_NAME));
            cmd.setNewMasterId(jsonObject.optString(CmdKey.NEW_MASTERID));
            cmd.setNewMasterName(jsonObject.optString(CmdKey.NEW_MASTER_NAME));
            cmd.setResults(jsonObject.optJSONArray(CmdKey.RESULTS));

            cmd.setFromUserId(jsonObject.optString(CmdKey.UI_BLACK_CMD_FROM_ID));
            cmd.setTouserId(jsonObject.optString(CmdKey.UI_BLACK_CMD_TOUSER_ID));
            cmd.setMsgData(jsonObject.optString(CmdKey.UI_BLACK_CMD_MSG_DATA));
            cmd.setMessageId(jsonObject.optInt(CmdKey.UI_BLACK_CMD_MSG_ID));

            cmd.setSpeakersNum(jsonObject.optInt(CmdKey.SPEAKERS_NUM));

            cmd.setCmdKey(jsonObject.optString(CmdKey.CMDKEY));
            cmd.setCmdValue(jsonObject.optString(CmdKey.CMDVALUE));

            cmd.setMeetingChatState(jsonObject.optInt(CmdKey.CHAT_STATUS));

            cmd.setTimestamp(jsonObject.optLong(CmdKey.TIMESTAMP));

            cmd.setPollParam(jsonObject.optString(CmdKey.POLL_PARAM));
            cmd.setOperateType(jsonObject.optInt(CmdKey.POLL_OPERATOR_TYPE));
        } catch (JSONException e) {
            e.printStackTrace();
        }

        if ((cmd.getCmdId() == CmdId.SERVER_NOTICE_BLACK_CACHE_CMD && cmd
                .getCmdKey().equals("host_layout"))
                || (cmd.getCmdId() == CmdId.UI_GET_BLACK_CACHE_CMD_RESP && cmd
                .getCmdKey().equals("host_layout"))) {
            if (jsonString.equals(lastLayoutCmd)) {
                return null;
            } else {
                lastLayoutCmd = jsonString;
            }
        }

        if ((cmd.getCmdId() == CmdId.SERVER_NOTICE_BLACK_CACHE_CMD && cmd
                .getCmdKey().equals("meet_caption"))
                || (cmd.getCmdId() == CmdId.UI_GET_BLACK_CACHE_CMD_RESP && cmd
                .getCmdKey().equals("meet_caption"))) {
            if (jsonString.equals(lastCaptionCmd)) {
                return null;
            } else {
                lastCaptionCmd = jsonString;
            }
        }

        if ((cmd.getCmdId() == CmdId.SERVER_NOTICE_BLACK_CACHE_CMD && cmd
                .getCmdKey().equals("meet_theme"))
                || (cmd.getCmdId() == CmdId.UI_GET_BLACK_CACHE_CMD_RESP && cmd
                .getCmdKey().equals("meet_theme"))) {
            if (jsonString.equals(lastThemeCmd)) {
                return null;
            } else {
                lastThemeCmd = jsonString;
            }
        }

        return cmd;
    }

    private void filterCmd(Cmd respModel) {
//        CustomLog.d(TAG,
//                "MeetingControl::filterCmd() cmdId: " + respModel.getCmdId());
        // 未初始化，不处理任何命令
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            return;
        }

        if (respModel == null) {
            return;
        }

        // 已初始化，不处于加入会议过程中，不处理任何命令
        if (mStatus == MeetingControlStatus.INITED
                && mOperation != MeetingControlOperation.JOIN_MEETING) {
            return;
        }

        // 已初始化，处于加入会议过程中，收到非加入会议命令回应，不处理
        if (mStatus == MeetingControlStatus.INITED
                && mOperation == MeetingControlOperation.JOIN_MEETING
                && respModel.getCmdId() != CmdId.JOIN_MEETING_RESP) {
            CustomLog.e(TAG, "加入会议过程中收到非会议回应7002命令，直接丢弃");
            return;
        }

        // 异常状态下，收到任何非加入会议回应命令，直接丢弃
        if (mStatus == MeetingControlStatus.EXCEPTION
                && respModel.getCmdId() != CmdId.JOIN_MEETING_RESP) {
            CustomLog.d(TAG, "异常状态下，收到非加入会议命令回应，直接丢弃");
            return;
        }

        switch (respModel.getCmdId()) {
            case CmdId.JOIN_MEETING_RESP:
                if (mOperation != MeetingControlOperation.JOIN_MEETING) {
                    CustomLog.d(TAG, "不处于加入会议PENDING,收到加入会议回应命令直接丢弃");
                    return;
                }
                if (mStatus == MeetingControlStatus.EXCEPTION) {
                    if (respModel.getStatus() == RespModelStatusCode.TIME_OUT) {
                        CustomLog.d(TAG, "异常状态下，收到加入会议命令回应，并且超时，不更新状态");
                        mOperation = MeetingControlOperation.NULL;
                    } else {
                        CustomLog.d(TAG, "异常状态下，收到加入会议命令回应，更新状态");
                        mStatus = MeetingControlStatus.JOIN_MEETING;
                        mOperation = MeetingControlOperation.NULL;
                    }
                } else {
                    mStatus = MeetingControlStatus.JOIN_MEETING;
                    mOperation = MeetingControlOperation.NULL;
                }
                break;
            case CmdId.EXIT_MEETING_RESP:
                if (mOperation != MeetingControlOperation.EXIT_MEETING) {
                    CustomLog.d(TAG, "不处于退出会议PENDING,收到退出会议回应命令直接丢弃");
                    return;
                }
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.ASK_FOR_SPEAK_RESP:
                if (mOperation != MeetingControlOperation.ASK_FOR_SPEAK) {
                    CustomLog.d(TAG, "不处于申请发言PENDING,收到申请发言回应命令直接丢弃");
                    return;
                }
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.ASK_FOR_STOP_SPEAK_RESP:
                if (mOperation != MeetingControlOperation.ASK_FOR_STOP_SPEAK) {
                    CustomLog.d(TAG, "不处于停止发言PENDING,收到停止发言回应命令直接丢弃");
                    return;
                }
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.NEW_GET_PARTICIPATORS_RESP:
                if (mOperation == MeetingControlOperation.GET_PARTICIPATOR_LIST) {
                    mOperation = MeetingControlOperation.NULL;
                }

                break;
            case CmdId.GIVE_MIC_RESP:
                if (mOperation != MeetingControlOperation.GIVE_MIC) {
                    CustomLog.d(TAG, "不处于传麦PENDING,收到传麦回应命令直接丢弃");
                    return;
                }
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.LOCK_MEETING_RESP:
                if (mOperation != MeetingControlOperation.LOCK_MEETING) {
                    CustomLog.d(TAG, "不处于加锁会议PENDING,收到加锁会议回应命令直接丢弃");
                    return;
                }
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.START_SPEAK:
                CustomLog.d(TAG, "收到开始发言命令");
                // return;
                break;
            case CmdId.STOP_SPEAK:
                CustomLog.d(TAG, "收到停止发言命令");
                // return;
                break;
            case CmdId.EXCEPTION:
                if (mStatus == MeetingControlStatus.EXCEPTION) {
                    CustomLog.d(TAG, "出现981异常，发现已经在处理，直接丢弃");
                } else {
                    CustomLog.d(TAG, "出现981异常，记录当前MeetingControl状态");
                    mStatus = MeetingControlStatus.EXCEPTION;
                    mOperation = MeetingControlOperation.NULL;
                }
                break;
            case CmdId.ASK_FOR_START_EPISODE_RESP:
                CustomLog.d(TAG, "收到请求插话命令，回应");
                mOperation = MeetingControlOperation.NULL;

                break;
            case CmdId.ASK_FOR_OPENORCLOSE_CAMERA_RESP:
                CustomLog.d(TAG, "收到打开或者关闭摄像头命令，回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.START_RECEIVE_EPISODE:
                CustomLog.d(TAG, "收到服务器群发开始插话通知命令");
                break;
            case CmdId.ASK_FOR_STOP_EPISODE_RESP:
                CustomLog.d(TAG, "收到停止插话命令，回应");
                mOperation = MeetingControlOperation.NULL;

                break;
            case CmdId.STOP_RECEIVE_EPISODE:
                CustomLog.d(TAG, "收到服务器群发停止插话通知命令");
                break;
            case CmdId.MASTER_CHANGE_MEETING_MODE_RESP:
                CustomLog.d(TAG, "收到服务器群改变模式命令回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.MASTER_SET_USER_START_SPEAK_ON_MIC_RESP:
                CustomLog.d(TAG, "收到指定mic发言回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.ASK_FOR_OPERATE_LOUDSPEAKER_RESP:
                CustomLog.d(TAG, "收到用户申请操作扬声器回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.MASTER_SET_USER_STOP_SPEAK_ON_MIC_RESP:
                CustomLog.d(TAG, "收到指定mic停止发言提示");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.USER_ASK_FOR_RAISE_HAND_RESP:
                CustomLog.d(TAG, "收到申請舉手命令回應");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.ASK_FOR_START_LIVE_RESP:
                CustomLog.d(TAG, "收到开启直播回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.ASK_PUBLISH_OR_UNPUBLISH_STREAMS_RESP:
                CustomLog.d(TAG, "收到发布流或者取消发布流回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.SERVER_RETURN_APPLY_FOR_START_SCREEN_SHAREING:
                CustomLog.d(TAG, "收到屏幕分享回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.APPLY_FOR_STOP_SCREEN_SHAREING_RESP:
                CustomLog.d(TAG, "收到申请停止屏幕分享回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.SERVER_RETURN_STOP_SCREEN_SHAREING:
                CustomLog.d(TAG, "服务器通知屏幕分享用户断开<仅发给断开者自己>");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.ASK_UPDATE_STREAMS_RESP:
                CustomLog.d(TAG, "服务器回应更新流信息<仅发给更新者自己>");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.SERVER_NOTICE_UPDATE_STREAMS:
                CustomLog.d(TAG, "服务器群发流更新");
                break;
            case CmdId.MASTER_KICK_USER_RESP:
                CustomLog.d(TAG, "主持人踢人命令回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.SERVER_NOTICE_BE_KICKED:
                CustomLog.d(TAG, "服务器通知被踢命令");
                break;
            case CmdId.UI_SEND_CMD_ID:
                CustomLog.d(TAG, "UI反向控制命令回应");
                break;
            case CmdId.SERVER_NOTICE_MASTER_OPERATE_USER_LOUDSPEAKER:
                CustomLog.d(TAG, "服务器通知主持人要求某人操作扬声器命令");
                break;
            case CmdId.SERVER_NOTICE_MASTER_OPERATE_USER_STREAM:
                CustomLog.d(TAG, "服务器通知操作流（打开/关闭 摄像头/音频/文档）");
                break;
            case CmdId.SERVER_NOTICE_USER_CHANGE_LOUDSPEAKER_STATE:
                CustomLog.d(TAG, "用户更改扬声器状态");
                break;
            case CmdId.ASK_FOR_START_RECORDING_RESP:
                mOperation = MeetingControlOperation.NULL;
                CustomLog.d(TAG, "开始录制请求 回应");
                break;
            case CmdId.SERVER_NOTICE_START_RECORDING:
                mOperation = MeetingControlOperation.NULL;
                CustomLog.d(TAG, "服务器通知终端会议开始录制");
                break;
            case CmdId.ASK_FOR_STOP_RECORDING_RESP:
                mOperation = MeetingControlOperation.NULL;
                CustomLog.d(TAG, "停止录制请求回应");
                break;
            case CmdId.SERVER_NOTICE_STOP_RECORDING:
                CustomLog.d(TAG, "服务器通知终端会议停止录制");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.MASTER_TRANSFER_MASTERID_RESP:
                CustomLog.d(TAG, "主持人权限移交回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.SERVER_NOTICE_MASTERID_CHANGED:
                CustomLog.d(TAG, "服务器通知终端移交主持人权限");
                break;
            case CmdId.MASTER_ASK_FOR_END_MEETING_RESP:
                CustomLog.d(TAG, "主持人申请结束会议请求命令回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.SERVER_NOTICE_END_MEETING:
                CustomLog.d(TAG, "服务器通知终端结束会议命令");
                break;
            case CmdId.UI_BLACKBOX_CMD_ID:
                CustomLog.d(TAG, "收到黑盒命令");
                break;
            case CmdId.SERVER_NOTICE_BLACK_CACHE_CMD:
                CustomLog.d(TAG, "服务器广播黑盒缓存命令");
                break;
            case CmdId.UI_GET_BLACK_CACHE_CMD_RESP:
                CustomLog.d(TAG, "UI端主动获取黑盒缓存命令响应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.MASTER_CHANGE_MEETING_IM_STATE_RESP:
                CustomLog.d(TAG, "主持人改变会议聊天状态回应");
                mOperation = MeetingControlOperation.NULL;
                break;
            case CmdId.SERVER_NOTICE_MEETING_IM_STATE_CHANGE:
                CustomLog.d(TAG, "服务器广播会议聊天状态改变");
                break;
            case CmdId.SERVER_NOTICE_USER_POLL_CMD:
                CustomLog.d(TAG, "服务器通知某人被轮询命令");
                break;
            default:
                break;
        }
        if (respModel.getStatus() == RespModelStatusCode.TOKEN_TIME_OUT) {
            mStatus = MeetingControlStatus.EXCEPTION;
            mOperation = MeetingControlOperation.NULL;
            mMeetingControlListener.onReceiveCmd(respModel);
            return;
        }
        CustomLog.e(TAG, "respModel.getCmdId()" + respModel.getCmdId());
        mMeetingControlListener.onReceiveCmd(respModel);

        // 遍历查询是否有缓存命令没有发送
        if (cacheCmdMap != null && cacheCmdMap.size() > 0 && !isPublishStream) {
            CustomLog.d(TAG, "1 cacheCmdMap.size()=" + cacheCmdMap.size());
            if (mOperation == MeetingControlOperation.NULL) {
                Iterator<Map.Entry<String, Integer>> it = cacheCmdMap.entrySet().iterator();
                while (it.hasNext()) {
                    Map.Entry<String, Integer> entry = it.next();
                    String key = entry.getKey();
                    int value = entry.getValue();
                    CustomLog.d(TAG, "cache key=" + key + " value=" + value);
//                    mOperation = value;
                    mUDPProcessor.send(key);
                    it.remove();
                    break;
                }

            }
            CustomLog.d(TAG, "2 cacheCmdMap.size()=" + cacheCmdMap.size());
        }
    }

    /**
     * 开始录制请求
     *
     * @param token    (String):主持人认证口令
     * @param password (String):录制会议密码
     * @return 0:成功 非0:失败
     */
    public int askForStartRecording(String token, String password) {
        CustomLog.d(TAG, "askForStartRecording,mOperation = " + mOperation
                + " token:" + token + " password:" + password);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，开始录制请求 ");
            mOperation = MeetingControlOperation.ASK_FOR_START_RECORDING;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_START_RECORDING);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("password", password);
                jsonObject.put("token", token);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 停止录制请求
     *
     * @param token (String):主持人认证口令
     * @return 0:成功 非0:失败
     */
    public int askForStopRecording(String token) {
        CustomLog.d(TAG, "askForStopRecording,mOperation = " + mOperation
                + " token:" + token);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，停止录制请求");
            mOperation = MeetingControlOperation.ASK_FOR_STOP_RECORDING;
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.ASK_FOR_STOP_RECORDING);
                jsonObject.put("accountId", mAccountId);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("token", token);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    public UDPProcessor getWhiteBoardProcessor() {
        return mWhiteBoardProcessor;
    }

    public void setWhiteBoardProcessor(UDPProcessor mWhiteBoardProcessor) {
        this.mWhiteBoardProcessor = mWhiteBoardProcessor;
    }

    public void updateIPPortMap(String IPMapStr) {
        CustomLog.d(TAG, "updateIPPortMap");
        MeetingCtrlAgent.UpdateIPPortMap(IPMapStr);
    }

    /**
     * 通过100 申请操作数据共享
     *
     * @param
     * @return 0:成功 非0:失败
     */
    public int dataShareOperate(String operateAccountId, String msgData) {
        CustomLog.d(TAG, "masterSendInteractUsers,mOperation = " + mOperation
                + " operateAccountId=" + operateAccountId + " msgData="
                + msgData);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else {
            CustomLog.d(TAG, "状态合法，通过100 操作数据共享");
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put(CmdKey.CMD_ID, CmdId.UI_SEND_CMD_ID);
                jsonObject.put(CmdKey.UI_CMD_FROM_ID, mAccountId);
                jsonObject.put(CmdKey.UI_CMD_TARGET_ID, operateAccountId);
                jsonObject.put(CmdKey.TIMESTAMP,
                        (int) (System.currentTimeMillis() / 1000));
                jsonObject.put(CmdKey.UI_CMD_DATA, msgData);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 通过8052 回应
     *
     * @param
     * @return 0:成功 非0:失败
     */
    public int dataShareOperateResponse(String operateAccountId, String msgData) {
        CustomLog.d(TAG, "masterSendInteractUsers,mOperation = " + mOperation
                + " operateAccountId=" + operateAccountId + " msgData="
                + msgData);
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else {
            CustomLog.d(TAG, "状态合法，通过8052回应");
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put(CmdKey.CMD_ID, CmdId.UI_BLACKBOX_CMD_ID);
                jsonObject.put(CmdKey.UI_BLACK_CMD_FROM_ID, mAccountId);
                jsonObject.put(CmdKey.UI_BLACK_CMD_TOUSER_ID, operateAccountId);
                jsonObject.put(CmdKey.MEETING_ID, mMeetingId);
                jsonObject.put(CmdKey.UI_BLACK_CMD_MSG_ID, 517);
                jsonObject.put(CmdKey.UI_BLACK_CMD_MSG_DATA, msgData);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 获取黑盒缓存命令
     *
     * @param token
     * @return 0:成功 非0:失败
     */
    public int getServerBlackCacheCmd(String cmdKey, String token) {
        CustomLog.d(TAG, "getServerBlackCacheCmd,mOperation = " + mOperation
                + " token:" + token + " cmdKey=" + cmdKey);
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("cmdID", CmdId.UI_GET_BLACK_CACHE_CMD_REQ);
            jsonObject.put("token", token);
            jsonObject.put("accountId", mAccountId);
            jsonObject.put("meetingId", mMeetingId);
            jsonObject.put("cmdKey", cmdKey);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL || isPublishStream) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
            cacheCmdMap.put(jsonObject.toString(), MeetingControlOperation.GET_SERVER_BLACK_CACHE_CMD);
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，获取黑盒缓存命令");
            mOperation = MeetingControlOperation.GET_SERVER_BLACK_CACHE_CMD;
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 主持人改变会议文字聊天状态命令
     *
     * @param token
     * @param chatStatus
     * @return
     */
    public int masterChangeMeetingChatState(String token, int chatStatus) {
        CustomLog.d(TAG, "masterChangeMeetingIMState chatStatus = " + chatStatus);
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("cmdID", CmdId.MASTER_CHANGE_MEETING_IM_STATE);
            jsonObject.put("token", token);
            jsonObject.put("accountId", mAccountId);
            jsonObject.put("meetingId", mMeetingId);
            jsonObject.put("ChatStatus", chatStatus);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中，暂时无法操作");
//            cacheCmdMap.put(MeetingControlOperation.MASTER_CHANGE_CHAT_STATE, jsonObject.toString());
            return MeetingControlReturnCode.IN_PENDING;
        } else {
            CustomLog.d(TAG, "状态合法，可以发送改变会议文字聊天状态命令");
            mOperation = MeetingControlOperation.MASTER_CHANGE_CHAT_STATE;
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 公众消息
     *
     * @param msg 消息内容
     * @return 0:成功 非0:失败
     */
    public int sendMeetingPublicChatMessage(String msg) {
        CustomLog.d(TAG, "sendMeetingPublicChatMessage msg=" + msg);
        if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中");
//			return MeetingControlReturnCode.IN_PENDING;
        }
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else {
            CustomLog.d(TAG, "状态合法，通过8052发送公众消息");
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.UI_BLACKBOX_CMD_ID);
                jsonObject.put("fromuserid", mAccountId);
                jsonObject.put("meetingId", mMeetingId);
                jsonObject.put("messageid", 517);
                jsonObject.put("msgdata", msg);
                jsonObject.put("timestamp", ButelCommonUtil.getSystemMilliTime());
                jsonObject.put("touserid", "");
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString() + "\0");
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    /**
     * 私聊消息
     *
     * @param msg 消息内容
     * @return 0:成功 非0:失败
     */
    public int sendMeetingPrivateChatMessage(String msg, String targetId) {
        CustomLog.d(TAG, "sendMeetingPublicChatMessage msg=" + msg + " targetId=" + targetId);
        if (mOperation != MeetingControlOperation.NULL) {
            CustomLog.d(TAG, "处于PENDING中");
//			return MeetingControlReturnCode.IN_PENDING;
        }
        if (mStatus == MeetingControlStatus.NOT_INIT) {
            CustomLog.d(TAG, "未初始化状态，暂时无法操作");
            return MeetingControlReturnCode.NOT_INIT;
        } else {
            CustomLog.d(TAG, "状态合法，通过100发送私聊消息");
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("cmdID", CmdId.UI_SEND_CMD_ID);
                jsonObject.put("data", msg);
                jsonObject.put("fromId", mAccountId);
                jsonObject.put("targetId", targetId);
                jsonObject.put("timestamp", ButelCommonUtil.getSystemMilliTime());
            } catch (JSONException e) {
                e.printStackTrace();
            }
            mUDPProcessor.send(jsonObject.toString());
            return MeetingControlReturnCode.SUCCESS;
        }
    }

    public void sendPollAndPushStreamResult(int operateType, int result, String masterAccountId) {
        CustomLog.d(TAG,
                "sendPushStreamResult,result=" + result + " operateType=" + operateType);
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put(CmdKey.CMD_ID, CmdId.UI_SEND_CMD_ID);
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("#PUSH_STREAM_CTRL_RESP#");
            stringBuilder.append(result);
            stringBuilder.append("#PUSH_STREAM_CTRL#");
            stringBuilder.append(operateType + "#");
            stringBuilder.append(masterAccountId + "#");
            stringBuilder.append(mAccountId + "#");
            stringBuilder.append(System.currentTimeMillis() / 1000);
            jsonObject.put(CmdKey.UI_CMD_DATA, stringBuilder.toString());
            jsonObject.put(CmdKey.UI_CMD_FROM_ID, mAccountId);
            jsonObject.put(CmdKey.UI_CMD_TARGET_ID, masterAccountId);
            jsonObject.put(CmdKey.TIMESTAMP, System.currentTimeMillis() / 1000);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        mUDPProcessor.send(jsonObject.toString());
        CustomLog.d(TAG, "sendPushStreamResult,jsonObject.toString() = "
                + jsonObject.toString());
    }
}
