package cn.redcdn.butelopensdk.pushstream;

import android.content.Context;
import android.os.Build;

import cn.redcdn.butelopensdk.constconfig.StreamInfo;
import cn.redcdn.butelopensdk.media.SMSDKControl;
import cn.redcdn.butelopensdk.vo.VideoParameter;
import cn.redcdn.log.CustomLog;

public class PollAndPushControl {
	private final String TAG = PollAndPushControl.this.getClass().getName();
	private SMSDKControl mSMSDKControl;
	private Context mContext;
	private String mAccountId;

	private boolean mPollState = false; // 轮询状态

	private VideoParameter mVideoParameter; // 推流参数
	private int mVideoStremId;
	private String mDeviceMode = Build.MODEL;

	public PollAndPushControl(Context context, SMSDKControl smsdk,
                              String accountId) {
		CustomLog.d(TAG, "PollAndPushControl");
		mContext = context;
		mSMSDKControl = smsdk;
		mAccountId = accountId;
	}

	public void release() {
		stopPollAndPush();
		mSMSDKControl = null;
		mContext = null;
		// mFrameLayout = null;
	}

	public boolean getPollState() {
		CustomLog.d(TAG, "getPollState mPollState=" + mPollState);
		return mPollState;
	}

	public int startPollAndPush(String param) {
		CustomLog.d(TAG, "startPollAndPush");
		if (mPollState) {
			CustomLog.e(TAG, "当前正在轮询推流，不处理！");
			return 0;
		}

		mVideoStremId = mSMSDKControl.publishPollCameraVideo(
					StreamInfo.RESOURCE_TYPE_VIDEO, Integer.parseInt(mAccountId),
					mVideoParameter);

		CustomLog.d(TAG, "startPollAndPush mVideoStremId=" + mVideoStremId);

		if (mVideoStremId > 0) {
			mPollState = true;
		}
		return mVideoStremId;
	}

	public void stopPollAndPush() {
		CustomLog.d(TAG, "stopPollAndPush");
		if (!mPollState) {
			CustomLog.e(TAG, "当前不在轮询推流，不处理！");
			return;
		}
		mSMSDKControl.unPublish(mVideoStremId);
		mVideoStremId = -1;
		mPollState = false;
	}

}
