package cn.redcdn.butelopensdk.pushstream;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.graphics.ImageFormat;
import android.graphics.PixelFormat;
import android.hardware.Camera;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.PreviewCallback;
import android.view.Gravity;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.WindowManager;
import android.view.WindowManager.LayoutParams;
import cn.redcdn.butelopensdk.constconfig.DeviceRunEnvironment;
import cn.redcdn.log.CustomLog;

import com.channelsoft.framework.MediaFrameWork;

public class PushStreamControl {

	private final String TAG = PushStreamControl.this.getClass().getName();

	private Context mContext;

	private String mPushUrl;

	private boolean mPushStreamState = false;

	private Camera mCamera;

	private WindowManager mWindowManager;

	private LayoutParams mWindowParams;
	
	private SurfaceView mSurfaceView;
	
	private SurfaceHolder mHolder;
	
	SurfaceHolder.Callback mSurfaceHolderCallback = new SurfaceHolder.Callback() {

		@Override
		public void surfaceChanged(SurfaceHolder arg0, int arg1,
				int arg2, int arg3) {

		}

		@Override
		public void surfaceCreated(SurfaceHolder arg0) {
			CustomLog.d(TAG, "open camera");
			try {
				mCamera = Camera.open(0);
				mCamera.setPreviewDisplay(mHolder);
				Parameters parameter = mCamera.getParameters();
				
				if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_DW
						|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_YNDL) {
					parameter.setPreviewSize(640, 480);
					mCamera.setParameters(parameter);
					mCamera.addCallbackBuffer(new byte[640 * 480 * ImageFormat
					           						.getBitsPerPixel(parameter.getPreviewFormat()) / 8]);
				} else if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_DW
						|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_YNDL){
					parameter.setPreviewSize(640, 360);
					mCamera.setParameters(parameter);
					mCamera.addCallbackBuffer(new byte[640 * 360 * ImageFormat
					           						.getBitsPerPixel(parameter.getPreviewFormat()) / 8]);
				}
				
				mCamera.setPreviewCallbackWithBuffer(new PreviewCallback() {
					@Override
					public void onPreviewFrame(byte[] arg0, Camera arg1) {
//						CustomLog.d(TAG,"onPreviewFrame");
						if (mPushStreamState) {
							int result = MediaFrameWork
									.PushFrametoLiveRtmp(arg0,
											arg0.length);

//							CustomLog
//									.d(TAG,
//											"PushFrametoLiveRtmp result:"
//													+ String.valueOf(result)
//													+ " data.length:"
//													+ String.valueOf(arg0.length));
							
						}
						
					      if (mCamera != null) {
					          mCamera.addCallbackBuffer(arg0); // 需要调用，触发继续onPreviewFrame
					        }
						
					}
				});
				mCamera.setDisplayOrientation(0);
				mCamera.startPreview();
			} catch (Exception e) {
				CustomLog.e(TAG, "open camera error:" + e.toString());
			}
		}

		@Override
		public void surfaceDestroyed(SurfaceHolder arg0) {

		}

	};

	public PushStreamControl(Context context, String pushUrl) {
		CustomLog.d(TAG, "PushStreamControl,pushUrl:" + pushUrl);
		mContext = context;
		mPushUrl = pushUrl;
	}

	/**
	 * 开始推流
	 */
	@SuppressWarnings("static-access")
	public void startPushStream() {
		CustomLog.d(TAG, "startPushStream");

		if (mWindowManager == null) {
			CustomLog.d(TAG, "mWindowManager==null");
			mWindowManager = (WindowManager) mContext.getApplicationContext()
					.getSystemService(mContext.WINDOW_SERVICE);
			mWindowParams = new LayoutParams();
			mWindowParams.type = WindowManager.LayoutParams.TYPE_SYSTEM_ALERT;
			mWindowParams.flags = WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL
					| WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE
					| WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
					| WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED;
			mWindowParams.format = PixelFormat.RGBA_8888;
			mWindowParams.gravity = Gravity.LEFT | Gravity.TOP;
			mWindowParams.x = 0;
			mWindowParams.y = 0;
			mWindowParams.width = 1;
			mWindowParams.height = 1;
			mWindowParams.screenOrientation = ActivityInfo.SCREEN_ORIENTATION_NOSENSOR;
			mSurfaceView = new SurfaceView(mContext);
			mWindowManager.addView(mSurfaceView, mWindowParams);

			mHolder = mSurfaceView.getHolder();
			mHolder.addCallback(mSurfaceHolderCallback);
		}

		int result = -1;
		if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_DW
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_YNDL) {
			result = MediaFrameWork.CreateLiveRtmpStream(640, 480, 300000, 15,
					mPushUrl);
		} else if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_DW
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_YNDL){
			result = MediaFrameWork.CreateLiveRtmpStream(640, 360, 300000, 15,
					mPushUrl);
		}
		
		if (result == 0) {
			mPushStreamState = true;
		}
		CustomLog.d(TAG,
				"CreateLiveRtmpStream result:" + String.valueOf(result));

	}
	

	/**
	 * 暂停推流
	 */
	public void pausePushStream() {
		CustomLog.d(TAG, "pausePushStream");
		int result = MediaFrameWork.DestroyLiveRtmpStream();
		if (result == 0) {
			mPushStreamState = false;
		}
		CustomLog.d(TAG,
				"DestroyLiveRtmpStream result:" + String.valueOf(result));
	}

	/**
	 * 停止推流
	 */
	public void stopPushStream() {
		CustomLog.d(TAG, "stopPushStream");
		pausePushStream();
		CustomLog.d(TAG, "close camera");
		try {
			if(mWindowManager != null){
				mWindowManager.removeViewImmediate(mSurfaceView);
				mWindowManager = null;
			}
			if(mSurfaceView != null){
				mSurfaceView = null;
			}	
			if(mHolder != null && mSurfaceHolderCallback!=null){
				mHolder.removeCallback(mSurfaceHolderCallback);
			}
			if (mCamera != null) {
				mCamera.stopPreview();
				mCamera.setPreviewCallback(null);
				mCamera.setPreviewDisplay(null);
				mCamera.addCallbackBuffer(null);
				mCamera.setPreviewCallbackWithBuffer(null);
				mCamera.release();
				mCamera = null;
			} else {
				CustomLog.d(TAG, "mCamera==null");
			}
		} catch (Exception e) {
			CustomLog.e(TAG, "close camera error:" + e.toString());
		}
	}

}
