package cn.redcdn.butelopensdk.screenshare;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import cn.redcdn.butelopensdk.constconfig.DeviceRunEnvironment;
import cn.redcdn.dep.CBInterface;
import cn.redcdn.dep.ScreenSharingHostAgentJNI;
import cn.redcdn.log.CustomLog;

import com.channelsoft.sipsdk.SmSdkJNI;

public class ScreenShareControl {
	private final String TAG = ScreenShareControl.this.getClass().getName();

	/**
	 * 申请连接成功
	 */
	public static final int REQUEST_CONNECT_SUCCESS = 0;

	/**
	 * 申请连接失败，UI忙
	 */
	public static final int REFUSE_CONNECT_UI_BUSY = 1;

	/**
	 * 申请连接失败,未处于发言状态
	 */
	public static final int REFUSE_CONNECT_UNSPEAKING = 2;

	/**
	 * 申请连接失败，请求发送失败
	 */
	public static final int REFUSE_CONNECT_REQUEST_FAILED = 3;

	/**
	 * 申请连接失败，超时
	 */
	public static final int REFUSE_CONNECT_TIMEOUT = 4;

	/**
	 * 屏幕分享被抢占导致停止屏幕分享
	 */
	public static final int STOP_SHARING_BE_GRABED = 1;

	/**
	 * 停止发言导致停止屏幕分享
	 */
	public static final int STOP_SHARING_STOP_SPEAK = 2;

	/**
	 * 退出会议导致停止屏幕分享
	 */
	public static final int STOP_SHARING_EXIT_MEETING = 3;

	// private Context mContext;
	private final static int MSG_APPLY_START = 1;
	private final static int MSG_APPLY_STOT = 2;
	private final static int MSG_NAME_CHANGED = 3;
	private final static int MSG_START_SHARING = 4;
	private final static int MSG_FRAME_ARRIVE = 5;
	private final static int MSG_STOP_SHARING = 6;

	private ScreenShareListener mListener;

	private int mStreamId = -1;

	private INIT_STATE mInitState = INIT_STATE.NULL;

	private SHARE_STATE mShareState = SHARE_STATE.NULL;

	private boolean mSpeakState = false;

	private int port = -1; // 端口

	private enum INIT_STATE {
		NULL, INITED
	}

	private enum SHARE_STATE {
		NULL, APPLY_START, SHARING, APPLY_STOP
	}

	public interface ScreenShareListener {
		public void applyForStart();

		public void applyForStop();

		public void onNameChanged(String name);

		public void startShareing(int height, int width, int bitrate,
				int filmformat);

		public void frameDateArrive(byte[] fdata);

		public void stopShareing();
	}

	private CBInterface mCBInterface = new CBInterface() {

		@Override
		public int StopSharing() {
			Message msg = new Message();
			msg.what = MSG_STOP_SHARING;
			mHandler.sendMessage(msg);
			return 0;
		}

		@Override
		public int StartSharing(int height, int width, int bitrate,
				int filmformat) {
			Message msg = new Message();
			msg.what = MSG_START_SHARING;
			Bundle bundle = new Bundle();
			bundle.putInt("height", height);
			bundle.putInt("width", width);
			bundle.putInt("bitrate", bitrate);
			bundle.putInt("filmformat", filmformat);
			msg.setData(bundle);
			mHandler.sendMessage(msg);
			return 0;
		}

		@Override
		public void OnNameChanged(String newname) {
			Message msg = new Message();
			msg.what = MSG_NAME_CHANGED;
			msg.obj = newname;
			mHandler.sendMessage(msg);
		}

		@Override
		public int FrameDateArrive(byte[] fdata) {
			if (mInitState != INIT_STATE.INITED) {
				CustomLog.e(TAG, "FrameDateArrive() 状态不合法，未进行初始化  mInitState: "
						+ mInitState);
				return -1;
			}

			if (mShareState != SHARE_STATE.SHARING) {
				CustomLog.e(TAG, "FrameDateArrive() 状态不合法，未进行屏幕分享 mShareState"
						+ mShareState);
				return -2;
			}

			if (mStreamId == -1) {
				CustomLog.e(TAG, "FrameDateArrive() 没有获取到 streamId,无法push 数据");
				return -2;
			}
			// 将数据直接推送到SM_SDK端，减少中间环节。提升效率 (是否存在问题，需要测试)
			SmSdkJNI.SM_SDK_PushFrame(mStreamId, SmSdkJNI.SMSDK_FRAME_TYPE_RTP,
					fdata, fdata.length, 0);
			return 0;
		}

		@Override
		public int ApplyForStop() {
			Message msg = new Message();
			msg.what = MSG_APPLY_STOT;
			mHandler.sendMessage(msg);
			return 0;
		}

		@Override
		public int ApplyForStart() {
			Message msg = new Message();
			msg.what = MSG_APPLY_START;
			mHandler.sendMessage(msg);
			return 0;
		}
	};

	private Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MSG_APPLY_START:
				handlerApplyStart();
				break;
			case MSG_APPLY_STOT:
				handlerApplyStop();
				break;
			case MSG_NAME_CHANGED:
				handlerNameChanged((String) msg.obj);
				break;
			case MSG_START_SHARING:
				int height = msg.getData().getInt("height");
				int width = msg.getData().getInt("width");
				int bitrate = msg.getData().getInt("bitrate");
				int filmformat = msg.getData().getInt("filmformat");
				handlerStartSharing(height, width, bitrate, filmformat);
				break;
			case MSG_STOP_SHARING:
				handlerStopSharing();
				break;
			case MSG_FRAME_ARRIVE:
				break;
			default:
				break;
			}
			super.handleMessage(msg);
		}
	};

	/**
	 * 初始化 </p> 初始化成功后将返回屏幕分享端口号，该端口号需要发布到 HostAgent
	 * 端。当程序运行过程中ip发生变化需要release该模块，然后重新进行初始化。
	 * 
	 * @param context
	 *            context
	 * @param accountId
	 *            视讯号
	 * @param ip
	 *            本地ip地址
	 * @param cfgPath
	 *            配置文件路径
	 * @param logPath
	 *            日志路径
	 * @param listener
	 *            回调监听
	 * @return </p> >0：为屏幕分享返回的端口，需要设置到HostAgent中 </p> -1: 初始化失败，参数不合法 </p> -2:
	 *         初始化失败，已初始化 </p> <0: 初始化失败，Agent返回
	 */
	public int init(Context context, String accountId, String ip,
			String cfgPath, String logPath, ScreenShareListener listener) {
		CustomLog.d(TAG, "ScreenShareControl::init() accountId: " + accountId
				+ " | ip: " + ip + " | cfgPath:" + cfgPath + " |logPath: "
				+ logPath);
		if (TextUtils.isEmpty(accountId) || TextUtils.isEmpty(ip)
				|| listener == null) {
			CustomLog.d(TAG, "ScreenShareControl 初始化失败，参数不合法: -1");
			return -1;
		}
		if (mInitState == INIT_STATE.INITED) {
			CustomLog.d(TAG, "ScreenShareControl 初始化失败，已初始化");
			return -2;
		}
		int result = ScreenSharingHostAgentJNI.StartSsHostAgent(accountId, ip,
				cfgPath, logPath);
		if (result <= 0) {
			CustomLog.d(TAG, "ScreenShareControl 初始化失败,Agent返回: " + result);
			return result;
		}
		// mContext = context;
		ScreenSharingHostAgentJNI.setUserCallBack(mCBInterface);
		mListener = listener;
		mInitState = INIT_STATE.INITED;
		port = result;
		CustomLog.d(TAG, "ScreenShareControl 初始化成功，ret = " + result);
		return result;
	}

	/**
	 * 反初始化
	 * 
	 * @return </p> -1: 状态不合法，未初始化 </p> 0: 成功反初始化
	 */
	public int release() {
		CustomLog.d(TAG, "ScreenShareControl::release()");
		if (mInitState == INIT_STATE.NULL) {
			CustomLog.d(TAG, "ScreenShareControl::release() 状态不合法，未初始化");
			return -1;
		}
		// TODO 调用 reset 接口在某些机型上会引起崩溃
		if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_DW
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_YNDL
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_DW
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_YNDL) {
			ScreenSharingHostAgentJNI.resetUserCallBack(mCBInterface);
		}
		ScreenSharingHostAgentJNI.StopSsHostAgent();
		mStreamId = -1;
		port = -1;
		mInitState = INIT_STATE.NULL;
		mShareState = SHARE_STATE.NULL;
		// mContext = null;
		return 0;
	}

	/**
	 * 设置发言状态,当发言状态切换时需要调用该接口通知到 ScreenSharingHostAgent 端
	 * 
	 * @param state
	 *            </p> true 发言 </p> false 未发言
	 * @return </p> 0: 成功 </p> -1: 状态不合法，未初始化
	 */
	public int setSpeakState(boolean state) {
		if (mInitState == INIT_STATE.NULL) {
			CustomLog.d(TAG, "ScreenShareControl::setSpeakState() 状态不合法，未初始化");
			return -1;
		}
		CustomLog.d(TAG, "ScreenShareControl::setSpeakState() speakState: "
				+ state);
		mSpeakState = state;
		ScreenSharingHostAgentJNI.setSpeak(mSpeakState);
		return 0;
	}

	/**
	 * 申请开始屏幕分享回应。pc端小工具请求进行屏幕分享 -> ButelOpenSDK 收到请求后向STP服务器请求 ->
	 * STP服务器回复ButelOpenSDK -> ButelOpenSDK调用此接口回应申请结果
	 * 
	 * @param response
	 *            </p>{@link REQUEST_CONNECT_SUCCESS } 申请成功 </p>
	 *            {@link REFUSE_CONNECT_UI_BUSY} 申请失败，UI忙</p>
	 *            {@link REFUSE_CONNECT_UNSPEAKING} 申请失败，未处于发言状态</p>
	 *            {@link REFUSE_CONNECT_TIMEOUT} 申请失败，超时
	 * @return </p> -1: 状态不合法，未初始化 </p> 0: 成功
	 */
	public int responseApplyForStart(int response) {
		CustomLog.d(TAG,
				"ScreenShareControl::responseApplyForStart() response: "
						+ response);
		if (mInitState == INIT_STATE.NULL) {
			CustomLog.d(TAG,
					"ScreenShareControl::responseApplyForStart() 状态不合法，未初始化");
			return -1;
		}

		if (mShareState != SHARE_STATE.APPLY_START) {
			CustomLog.d(TAG, "当前未处于申请开始状态，拒绝回应请求! mShareState: " + mShareState);
			return -1;
		}

		ScreenSharingHostAgentJNI.ApplyStartResponse(response);
		mShareState = SHARE_STATE.NULL;
		return 0;
	}

	/**
	 * 强制停止屏幕分享。取消发言、退出会议、屏幕分享被抢占时调用
	 * 
	 * @param reason
	 *            停止屏幕分享原因 </p> {@link STOP_SHARING_BE_GRABED} 被抢占 </p>
	 *            {@link STOP_SHARING_STOP_SPEAK} 停止发言 </p>
	 *            {@link STOP_SHARING_EXIT_MEETING} 退出会议
	 * @param desp
	 *            描述，如果是被抢占，需要携带抢占者信息：{"accountId":"123","userName":"testName"}
	 * @return </p> 0：成功 </p> -1: 状态不合法，未初始化
	 */
	public int forceStopSharing(int reason, String desp) {
		CustomLog.d(TAG, "ScreenShareControl::forceStopSharing() reason: "
				+ reason + " |desp:" + desp);
		if (mInitState == INIT_STATE.NULL) {
			CustomLog.d(TAG,
					"ScreenShareControl::forceStopSharing() 状态不合法，未初始化");
			return -1;
		}

		ScreenSharingHostAgentJNI.NoticeForceStopSharing(reason, desp);

		return 0;
	}

	/**
	 * 设置streamId,用于屏幕分享数据push到媒体端
	 * 
	 * @param streamId
	 *            媒体流id
	 */
	public void setStreamId(int streamId) {
		CustomLog.d(TAG, "ScreenShareControl::setStreamId() streamId: "
				+ streamId);
		mStreamId = streamId;
	}

	/**
	 * 获取屏幕分享端口，该端口需要设置到HostAgent
	 * 
	 * @return port
	 */
	public int getPort() {
		return port;
	}

	private int handlerApplyStart() {
		CustomLog.d(TAG, "ScreenShareControl::handlerApplyStart()申请屏幕分享 ");
		if (mInitState != INIT_STATE.INITED) {
			CustomLog.e(TAG, "状态不合法，未进行初始化  mInitState: " + mInitState);
			responseApplyForStart(REFUSE_CONNECT_TIMEOUT);
			return -1;
		}

		if (mSpeakState == false) {
			CustomLog.e(TAG, "当前未处于发言状态，拒绝屏幕分享请求! mShareState: " + mShareState);
			responseApplyForStart(REFUSE_CONNECT_UNSPEAKING);
			return -2;
		}

		if (mShareState != SHARE_STATE.NULL) {
			CustomLog.e(TAG, "状态不合法! mShareState: " + mShareState);
			// 即使状态不合法，依然尝试连接,避免由于其它原因导致内部状态出错
		}

		mShareState = SHARE_STATE.APPLY_START;
		mListener.applyForStart();
		return 0;
	}

	private int handlerApplyStop() {
		CustomLog.d(TAG, "ScreenShareControl::handlerApplyStop() 申请停止屏幕分享 ");
		if (mInitState != INIT_STATE.INITED) {
			CustomLog.e(TAG, "handlerApplyStop() 状态不合法，未进行初始化  mInitState: "
					+ mInitState);
			return -1;
		}

		if (mShareState != SHARE_STATE.SHARING) {
			CustomLog.e(TAG, "handlerApplyStop() 状态不合法，未进行屏幕分享 mShareState"
					+ mShareState);
			return -2;
		}
		mShareState = SHARE_STATE.APPLY_STOP;
		mListener.applyForStop();
		return 0;
	}

	private int handlerNameChanged(String newname) {
		CustomLog.d(TAG, "ScreenShareControl::handlerNameChanged() name: "
				+ newname);
		if (mInitState != INIT_STATE.INITED) {
			CustomLog.e(TAG, "状态不合法，未进行初始化  mInitState: " + mInitState);
			return -1;
		}

		if (mShareState != SHARE_STATE.SHARING) {
			CustomLog.e(TAG, "状态不合法，未进行屏幕分享 mShareState" + mShareState);
			return -2;
		}
		mListener.onNameChanged(newname);
		return 0;
	}

	private int handlerStartSharing(int height, int width, int bitrate,
			int filmformat) {
		CustomLog.d(TAG, "ScreenShareControl::handlerStartSharing() height: "
				+ height + " | width: " + width + " | bitrate: " + bitrate
				+ " | filmformat: " + filmformat);
		if (mInitState != INIT_STATE.INITED) {
			CustomLog.e(TAG, "状态不合法，未进行初始化  mInitState: " + mInitState);
			return -1;
		}

		mListener.startShareing(height, width, bitrate, filmformat);
		mShareState = SHARE_STATE.SHARING;
		return 0;
	}

	private int handlerStopSharing() {
		CustomLog.d(TAG, "ScreenShareControl::handlerStopSharing()");
		if (mInitState != INIT_STATE.INITED) {
			CustomLog.e(TAG, "状态不合法，未进行初始化  mInitState: " + mInitState);
			return -1;
		}
		mShareState = SHARE_STATE.NULL;
		mListener.stopShareing();
		return 0;
	}
}
