package cn.redcdn.butelopensdk.util;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;

import cn.redcdn.log.CustomLog;

/**
 * 获取当前cpu总的使用率 和 当前应用的cpu使用率 Created by caiguo on 2017/6/18 0018.
 */

public class CpuUsageUtil {

	private String TAG = getClass().getName();
	private Boolean flag;// 是否cpu使用率
	private int currentProcessCpuRate = -1;// 当前应用cpu占用
											// （0-100）（-1未取得cpu占用情况，可以舍弃）
	private int totalCpuRate = -1;// cpu总共被占用的 （0-100）（-1未取得cpu占用情况，可以舍弃）
	private static CpuUsageUtil mInstance;

	/**
	 * 单例结构
	 * 
	 * @return
	 */
	public static CpuUsageUtil getInstance() {
		if (mInstance == null) {
			synchronized (CpuUsageUtil.class) {
				if (mInstance == null) {
					mInstance = new CpuUsageUtil();
				}
			}
		}
		return mInstance;
	}

	private void CpuUsageUtil() {
	}

	/**
	 * 初始化方法和释放方法配合使用
	 */
	public void init() {
		CustomLog.i(TAG, "init");
		flag = true;
//		Thread thread1 = new Thread(new MyRunnable1());
//		Thread thread2 = new Thread(new MyRunnable2());
//		thread1.start();
//		thread2.start();
	}

	/**
	 * 获取cpu总的使用率
	 * 
	 * @return -1 没有初始化
	 */
	private void totalCpuUsage() {
		try {
			RandomAccessFile reader = new RandomAccessFile("/proc/stat", "r");
			String load = reader.readLine();
			String[] toks = load.split(" ");
			long idle1 = Long.parseLong(toks[5]);
			long cpu1 = Long.parseLong(toks[2]) + Long.parseLong(toks[3])
					+ Long.parseLong(toks[4]) + Long.parseLong(toks[6])
					+ Long.parseLong(toks[7]) + Long.parseLong(toks[8]);
			// try {
			// Thread.sleep(360);
			// } catch (Exception e) {
			// }
			reader.seek(0);
			load = reader.readLine();
			reader.close();
			toks = load.split(" ");
			long idle2 = Long.parseLong(toks[5]);
			long cpu2 = Long.parseLong(toks[2]) + Long.parseLong(toks[3])
					+ Long.parseLong(toks[4]) + Long.parseLong(toks[6])
					+ Long.parseLong(toks[7]) + Long.parseLong(toks[8]);
			if (((cpu2 + idle2) - (cpu1 + idle1)) == 0) {
				return;
			}
			int tempRate = (int) (100 * (cpu2 - cpu1) / ((cpu2 + idle2) - (cpu1 + idle1)));
			if (tempRate < 0 || tempRate >= 100 || tempRate == 0) {
				return;
			}
			totalCpuRate = tempRate;
			return;
		} catch (IOException ex) {
			ex.printStackTrace();
		}
		totalCpuRate = -1;
	}

	/**
	 * 获取当前进程cpu占用率(0 到 100)
	 * 
	 * @return
	 */
	private void currentProcessCpuUsage() {
		float totalCpuTime1 = getTotalCpuTime();
		float processCpuTime1 = getAppCpuTime();
//		try {
//			Thread.sleep(360);
//		} catch (Exception e) {
//			e.toString();
//		}
		float totalCpuTime2 = getTotalCpuTime();
		float processCpuTime2 = getAppCpuTime();
		if ((processCpuTime2 - processCpuTime1)
				/ (totalCpuTime2 - totalCpuTime1) == 0) {
			return;
		}
		float cpuRate = 100 * (processCpuTime2 - processCpuTime1)
				/ (totalCpuTime2 - totalCpuTime1);
		if (cpuRate < 0 || cpuRate > 100 || cpuRate == 0) {
			return;
		}
		currentProcessCpuRate = (int) cpuRate;
	}

	private long getTotalCpuTime() { // 获取系统总CPU使用时间
		String[] cpuInfos = null;
		try {
			// CustomLog.d(TAG,"try1");
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream("/proc/stat")), 1000);
			String load = reader.readLine();
			// CustomLog.d(TAG, "load:"+load);
			reader.close();
			cpuInfos = load.split(" ");
			for (int a = 0; a < cpuInfos.length; a++) {
				// CustomLog.d(TAG,"cpuInfos.length:"+String.valueOf(cpuInfos.length)
				// +" a:"+
				// String.valueOf(a)+" cpuInfos:"+String.valueOf(cpuInfos[a]));
			}
		} catch (Exception ex) {
			// CustomLog.e(TAG,"catch1,"+ex.toString());
		}

		// catch (IOException ex)
		// {
		// ex.printStackTrace();
		// }

		try {
			// CustomLog.d(TAG,"try2");
			long totalCpu = Long.parseLong(cpuInfos[2])
					+ Long.parseLong(cpuInfos[3]) + Long.parseLong(cpuInfos[4])
					+ Long.parseLong(cpuInfos[6]) + Long.parseLong(cpuInfos[5])
					+ Long.parseLong(cpuInfos[7]) + Long.parseLong(cpuInfos[8]);
			return totalCpu;
		} catch (Exception e) {
			// CustomLog.e(TAG,"catch2,"+e.toString());
			return 0;
		}

	}

	private long getAppCpuTime() { // 获取应用占用的CPU时间
		String[] cpuInfos = null;
		try {
			// CustomLog.d(TAG,"try3");
			int pid = android.os.Process.myPid();
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					new FileInputStream("/proc/" + pid + "/stat")), 1000);
			String load = reader.readLine();
			reader.close();
			cpuInfos = load.split(" ");
		} catch (Exception ex2) {
			// CustomLog.e(TAG,"catch3,"+ex2.toString());
		}

		// catch (IOException ex)
		// {
		// ex.printStackTrace();
		// }

		try {
			// CustomLog.d(TAG,"try4");
			long appCpuTime = Long.parseLong(cpuInfos[13])
					+ Long.parseLong(cpuInfos[14])
					+ Long.parseLong(cpuInfos[15])
					+ Long.parseLong(cpuInfos[16]);
			return appCpuTime;
		} catch (Exception e) {
			// CustomLog.e(TAG,"catch4,"+e.toString());
			return 0;
		}

	}

	public int getCurrentProcessCpuRate() {
		// CustomLog.i(TAG,"currentProcessCpuRate=="+currentProcessCpuRate);
		currentProcessCpuUsage();
		return currentProcessCpuRate;
	}

	public int getTotalCpuRate() {
		// CustomLog.i(TAG,"totalCpuRate=="+totalCpuRate);
		totalCpuUsage();
		return totalCpuRate;
	}

	class MyRunnable1 implements Runnable {
		@Override
		public void run() {
			while (flag) {
				totalCpuUsage();
			}
		}
	}

	class MyRunnable2 implements Runnable {
		@Override
		public void run() {
			while (flag) {
				currentProcessCpuUsage();
			}
		}
	}

	public void relase() {
		CustomLog.i(TAG, "relase");
		flag = false;
		currentProcessCpuRate = -1;
		totalCpuRate = -1;
	}
}
