package cn.redcdn.butelopensdk.vo;

import cn.redcdn.butelopensdk.constconfig.DeviceRunEnvironment;
import cn.redcdn.log.CustomLog;

public class VideoParameter {
	private final static String TAG = VideoParameter.class.getName();
	private int capWidth = 640; // 采集分辨率-宽度
	private int capHeight = 480; // 采集分辨率-高度
	private int capFps = DEFAULT_FPS; // 采集帧率
	private int encBitrate = 300; // 编码码率

	public static int DEFAULT_FPS = 15;

	public VideoParameter() {
		if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_DW
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_YNDL) {
			capWidth = 1280;
			capHeight = 720;
			encBitrate = 2000;
			DEFAULT_FPS = 25;
		} else if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_DW
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_YNDL
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.Mobile_JHY
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.Mobile_HYYL) {
			capWidth = 640;
			capHeight = 480;
			encBitrate = 300;
			DEFAULT_FPS = 15;
		}
	}

	/**
	 * 构造方法
	 * 
	 * @param capWidth
	 *            采集分辨率-宽度
	 * @param capHeight
	 *            采集分辨率-高度
	 * @param capFps
	 *            采集帧率
	 * @param encBitrate
	 *            编码码率
	 */
	public VideoParameter(int capWidth, int capHeight, int capFps,
			int encBitrate) {
		this.capWidth = capWidth;
		this.capHeight = capHeight;
		this.capFps = capFps;
		this.encBitrate = encBitrate;
		if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_DW
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M3_YNDL) {
			DEFAULT_FPS = 25;
		} else if (DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_DW
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.M1_YNDL
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.Mobile_JHY
				|| DeviceRunEnvironment.mDeviceRunEnvironment == DeviceRunEnvironment.Mobile_HYYL) {
			DEFAULT_FPS = 15;
		}
	}

	public void logSelf() {
		CustomLog.d(TAG, "VideoParameter::logSelf() width: " + getCapWidth()
				+ " | height: " + getCapHeight() + " | fps: " + getCapFps()
				+ " | bitrate: " + getEncBitrate());
	}

	/**
	 * 设置采集分辨率-宽度
	 * 
	 * @param capWidth
	 *            宽度
	 * @return 0 成功，!=0 失败
	 */
	public int setCapWidth(int capWidth) {
		this.capWidth = capWidth;
		return 0;
	}

	/**
	 * 获取采集分辨率-高度
	 * 
	 * @return 采集分辨率-高度
	 */
	public int getCapWidth() {
		return this.capWidth;
	}

	/**
	 * 设置采集分辨率-高度
	 * 
	 * @param capHeight
	 *            采集分辨率-高度
	 * @return 0 成功，!=0 失败
	 */
	public int setCapHeight(int capHeight) {
		this.capHeight = capHeight;
		return 0;
	}

	/**
	 * 采集分辨率-高度
	 * 
	 * @return 采集分辨率-高度
	 */
	public int getCapHeight() {
		return this.capHeight;
	}

	/**
	 * 设置采集分辨率
	 * 
	 * @param capFps
	 *            采集分辨率
	 * @return 0 成功，!=0 失败
	 */
	public int setCapFps(int capFps) {
		this.capFps = capFps;
		return 0;
	}

	/**
	 * 获取采集分辨率
	 * 
	 * @return 采集分辨率
	 */
	public int getCapFps() {
		return this.capFps;
	}

	/**
	 * 设置编码码率
	 * 
	 * @param encBitrate
	 *            编码码率
	 * @return 0 成功，!=0 失败
	 */
	public int setEncBitrate(int encBitrate) {
		this.encBitrate = encBitrate;
		return 0;
	}

	/**
	 * 获取编码码率
	 * 
	 * @return 编码码率
	 */
	public int getEncBitrate() {
		return this.encBitrate;
	}

	public boolean isParamEqual(VideoParameter param) {
		if (param.getCapWidth() == getCapWidth()
				&& param.getCapHeight() == getCapHeight()
				&& param.getEncBitrate() == getEncBitrate()
				&& param.getCapFps() == getCapFps()) {
			return true;
		}
		return false;
	}

	/**
	 * 是否高于VGA，if (width > 640 || height > 480) 即认为高于VGA
	 * 
	 * @param width
	 *            分辨率-宽
	 * @param height
	 *            分辨率-高
	 * @return true: 高于VGA， false: 低于或等于VGA
	 */
	public static boolean isUpVGA(int width, int height) {
		boolean flag = false;
		if (width > 640 || height > 480) {
			flag = true;
		}
		CustomLog.d(TAG, "VideoParameter::isUpVGA() width:" + width
				+ " | height: " + height + " |result: " + flag);
		return flag;
	}

}