package cn.redcdn.butelopensdk.whiteboard;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import android.view.SurfaceView;
import cn.redcdn.butelopensdk.MeetingInfo;
import cn.redcdn.butelopensdk.constconfig.ShareType;
import cn.redcdn.log.CustomLog;
import cn.redcdn.network.udp.UDPProcessor;

import com.butel.MediaGeoMetry.MediaGeoMetryJni;
import com.butel.MediaGeoMetry.MediaGeoMetryJni.DrawCommandCallback;

public class GeoMetryControl {
	private final String TAG = GeoMetryControl.this.getClass().getSimpleName();
	private MediaGeoMetryJni mediaGeoMetryJni = null;
	private int mShareType;
	private int mShareUserId;
	private int mResId = 300;
	private int mSendType = 2;

	// 组包相关Map
	private Map<Integer, byte[]> dataMap = new HashMap();
	private Map<Integer, List<Integer>> dataRecive = new HashMap();
	
	private Map<Integer, Map<Integer, Integer>> msgIdMap = new HashMap<Integer, Map<Integer, Integer>>();

	private UDPProcessor mWhiteBoardProcessor;
	private MeetingInfo mMeetingInfo;

	private int sendCount = 0;
	private boolean mReOrder;
	private boolean mConsumeData;
	private DealReOrder dealReOrder = null;
	private DealConsumeData dealConsumeData = null;

	// 接收到的数据队列
	private Map<Integer, Map<Integer, UnSerializedCmdData>> mUserOrderData = new ConcurrentHashMap<Integer, Map<Integer, UnSerializedCmdData>>();
	private Map<Integer, Integer> mMinMessageId = new ConcurrentHashMap<Integer, Integer>();

	// 需要发送的数据队列
	private Map<Integer, ConsumeDataInfo> mConsumeDataMap = new ConcurrentHashMap<Integer, ConsumeDataInfo>();

	public void setWhiteBoardProcessor(UDPProcessor mWhiteBoardProcessor) {
		this.mWhiteBoardProcessor = mWhiteBoardProcessor;
	}

	public void setMeetingInfo(MeetingInfo mMeetingInfo) {
		this.mMeetingInfo = mMeetingInfo;
	}

	public GeoMetryControl() {
		mediaGeoMetryJni = new MediaGeoMetryJni();
	}

	private DrawCommandCallback drawCommandCallback = new DrawCommandCallback() {

		@Override
		public void onCommand(int flag, byte[] cmd, int useSecureSend,
				int timeout) {
			// if (cmd.length > 0) {
//			CustomLog.d(TAG, "DrawCommandCallback onCommand flag=" + flag
//					+ " length=" + cmd.length);
			createData(cmd, flag);
			// }
		}
	};

	public void analyzeData(byte[] bytes) {

		try {
			UnSerializedCmdData cmdData = new UnSerializedCmdData();
			int analyzeOffSet = 0;
			short sendType = (short) (bytes[analyzeOffSet] & 0xFF);
			analyzeOffSet += 1;
			cmdData.sendType = sendType;

			short destUserNum = (short) byte2ToUnsignedShort(bytes,
					analyzeOffSet);
			analyzeOffSet += 2;
			cmdData.destUserNum = destUserNum;

			List<Integer> destUserIDList = new ArrayList<Integer>();
			for (int i = 0; i < destUserNum; i++) {
				int destUserID = byte4ToInt(bytes, analyzeOffSet);
				destUserIDList.add(destUserID);
				analyzeOffSet += 4;
//				CustomLog.d(TAG, "analyzeData destUserID=" + destUserID);
			}
			cmdData.destUserIDList = destUserIDList;

			int accountId = byte4ToInt(bytes, analyzeOffSet);
			analyzeOffSet += 4;
			cmdData.accountId = accountId;

			short packetType = (short) (bytes[analyzeOffSet] & 0xFF);
			analyzeOffSet += 1;
			cmdData.packetType = packetType;

			int messageId = byte2ToUnsignedShort(bytes, analyzeOffSet);
			analyzeOffSet += 2;
			cmdData.messageId = messageId;

			short dataLen = (short) byte2ToUnsignedShort(bytes, analyzeOffSet);
			analyzeOffSet += 2;
			cmdData.dataLen = dataLen;

			byte[] data = new byte[dataLen];
			System.arraycopy(bytes, analyzeOffSet, data, 0, dataLen);
			analyzeOffSet += dataLen;
			cmdData.data = data;

			short messageLen = (short) byte2ToUnsignedShort(bytes,
					analyzeOffSet);
			analyzeOffSet += 2;
			cmdData.messageLen = messageLen;

			short offset = (short) byte2ToUnsignedShort(bytes, analyzeOffSet);
			analyzeOffSet += 2;
			cmdData.offset = offset;

			short packetNum = (short) byte2ToUnsignedShort(bytes, analyzeOffSet);
			cmdData.packetNum = packetNum;

			cmdData.recvtime = System.currentTimeMillis();

//			CustomLog.d(TAG, "analyzeData " + cmdData.toString());

			if (packetType == 1 || packetType == 33) {
				// 封装回应消息并发送
				transfer2RespsData(cmdData);

				if (packetNum == 1) {

					if (msgIdMap.containsKey(accountId)) {
						Map<Integer, Integer> idMap = msgIdMap.get(accountId);
						if (idMap != null && idMap.containsKey((int) messageId)
								&& idMap.get((int) messageId) == (int) dataLen) {
//							CustomLog.d(TAG, "analyzeData accountId:"
//									+ accountId + " messageId:" + messageId
//									+ " 重复 ");
							return;
						} else {
							if (idMap == null) {
								idMap = new HashMap<Integer, Integer>();
							}
							idMap.put((int) messageId, (int) dataLen);
						}
					} else {
						Map<Integer, Integer> idMap = new HashMap<Integer, Integer>();
						idMap.put((int) messageId, (int) dataLen);
						msgIdMap.put(accountId, idMap);
					}

				}

				if (messageId == 32767) {
					if (msgIdMap.containsKey(accountId)) {
						Map<Integer, Integer> idMap = msgIdMap.get(accountId);
						idMap.clear();
						msgIdMap.put(accountId, idMap);
					}
				}

				if (cmdData.packetNum > 1) {
					// 需要组包
					combineDataPacket((int) cmdData.messageId,
							(int) cmdData.offset, cmdData.data,
							(int) cmdData.messageLen, (int) cmdData.packetNum);
				}

				Map<Integer, UnSerializedCmdData> cmdDataMap = mUserOrderData
						.get(accountId);
				if (mUserOrderData.get(accountId) == null) {
					cmdDataMap = new ConcurrentHashMap<Integer, UnSerializedCmdData>();
					mMinMessageId.put(accountId, cmdData.messageId);
				}
				cmdDataMap.put((int) cmdData.messageId, cmdData);
				mUserOrderData.put(accountId, cmdDataMap);

			} else if (packetType == 17) {
//				CustomLog.d(TAG,
//						"analyzeData DealConsumeData 收到回应包 key="
//								+ (messageId + offset) + " size="
//								+ mConsumeDataMap.size());
				mConsumeDataMap.remove(messageId + offset);
			}

		} catch (Exception e) {
			CustomLog.e(TAG, "analyzeData error 数据异常! " + e.toString());
		}
	}

	// 处理回调数据并发送
	class DealConsumeData extends Thread {

		@Override
		public void run() {
			super.run();
			CustomLog.d(TAG, "DealConsumeData thread begin");
			while (mConsumeData) {
				try {
					Iterator<Map.Entry<Integer, ConsumeDataInfo>> it = mConsumeDataMap
							.entrySet().iterator();

					while (it.hasNext()) {
						Map.Entry<Integer, ConsumeDataInfo> consumeDataEntry = it
								.next();
						ConsumeDataInfo consumeDataInfo = consumeDataEntry
								.getValue();

//						CustomLog.d(TAG, "DealConsumeData send msgID="
//								+ consumeDataInfo.messageId + " length="
//								+ consumeDataInfo.data.length);
						// 发送数据
						mWhiteBoardProcessor.send(consumeDataInfo.data);

						int sendCount = consumeDataInfo.sendCount;
						if (sendCount == 0) {
							byte[] reSendData = transfer2ReSendData(consumeDataInfo.data);
							consumeDataInfo.data = reSendData;
						}
						// 发送次数+1
						consumeDataInfo.sendCount = ++sendCount;
						
						mConsumeDataMap.put(consumeDataEntry.getKey(),
								consumeDataInfo);

						// 如果发送次数超过2，则删除
						if (sendCount > 1) {
//							CustomLog.e(TAG, "DealConsumeData 未收到回应 补发次数:"
//									+ sendCount + " msgID="
//									+ consumeDataInfo.messageId);
							if (sendCount > 2) {
								it.remove();
							}
						}
					}

					sleep(100);
				} catch (Exception e) {
					CustomLog.e(TAG,
							"DealConsumeData Exception:" + e.toString());
				}
			}
			CustomLog.d(TAG, "DealConsumeData thread exit");
		}
	}

	// 处理接受数据
	class DealReOrder extends Thread {
		@Override
		public void run() {
			super.run();
			CustomLog.d(TAG, "DealReOrder thread begin");
			while (mReOrder) {
				try {
					if (mUserOrderData != null && mUserOrderData.size() > 0) {
						for (Map.Entry<Integer, Map<Integer, UnSerializedCmdData>> entry : mUserOrderData
								.entrySet()) {

							long now = System.currentTimeMillis();

							Map<Integer, UnSerializedCmdData> cmdDataMap = entry
									.getValue();

							if (cmdDataMap != null && cmdDataMap.size() > 0) {
								Iterator<Map.Entry<Integer, UnSerializedCmdData>> it = cmdDataMap
										.entrySet().iterator();
								while (it.hasNext()) {
									Map.Entry<Integer, UnSerializedCmdData> cmdEntry = it
											.next();

									int minv = mMinMessageId
											.get(entry.getKey());
									int elapse = (int) (now - cmdEntry
											.getValue().recvtime);
									int id = cmdEntry.getValue().messageId;

									if (elapse > 200) {
										if (id > minv || (minv - id) > 2000) // 丢包或
																				// 序号回环，如果序号落在最小序号之前的2000个之内，认为收到已经回调的之前的包，丢掉
										{

											if (cmdEntry.getValue().packetNum == 1) {
//												CustomLog
//														.d(TAG,
//																"dealReOrder PushCmdData nube="
//																		+ entry.getKey()
//																		+ " messageId="
//																		+ cmdEntry
//																				.getValue().messageId);
												// 直接抛给图元模块处理
												mediaGeoMetryJni
														.PushCmdData(cmdEntry
																.getValue().data);
											}

											mMinMessageId.put(entry.getKey(),
													(int) (id + 1));

											if ((minv - id) > 2000) {
												// 序号回环
												dataMap.clear();
												dataRecive.clear();
											}

										} else {
//											CustomLog
//													.e(TAG,
//															"dealReOrder 超时丢掉 nube="
//																	+ entry.getKey()
//																	+ " messageId="
//																	+ cmdEntry
//																			.getValue().messageId
//																	+ " elapse="
//																	+ elapse
//																	+ " size="
//																	+ cmdDataMap
//																			.size()
//																	+ " min="
//																	+ minv);
										}

										it.remove();

									} else {
										if (id == minv) {

											if (cmdEntry.getValue().packetNum == 1) {
//												CustomLog
//														.d(TAG,
//																"dealReOrder nube="
//																		+ entry.getKey()
//																		+ " PushCmdData messageId="
//																		+ cmdEntry
//																				.getValue().messageId);
												// 直接抛给图元模块处理
												mediaGeoMetryJni
														.PushCmdData(cmdEntry
																.getValue().data);
											}

											mMinMessageId.put(entry.getKey(),
													(int) (minv + 1));
											it.remove();
										}
									}
								}
							}

						}
					}

					sleep(10);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			CustomLog.d(TAG, "DealReOrder thread exit");
		}
	};

	// 组包
	public void combineDataPacket(int messageId, int offset, byte[] packetData,
			int messageLen, int packetNum) {
//		CustomLog.d(TAG, "combineDataPacket begin messageId=" + messageId
//				+ " offset=" + offset + " messageLen=" + messageLen
//				+ " packetNum=" + packetNum);
		List<Integer> dataOffSetNum = dataRecive.get(messageId);
		if (dataOffSetNum != null) {
			boolean findFlag = false;
			for (int i = 0; i < dataOffSetNum.size(); i++) {
				if (dataOffSetNum.get(i) == offset) {
					// 该数据已经处理，直接return
//					CustomLog.d(TAG, "combineDataPacket 该数据已处理");
					findFlag = true;
					return;
				}
			}
			if (!findFlag) {
				dataOffSetNum.add(offset);
				dataRecive.put(messageId, dataOffSetNum);
			}
		} else {
			dataOffSetNum = new ArrayList<Integer>();
			dataOffSetNum.add(offset);
			dataRecive.put(messageId, dataOffSetNum);
		}

		// 数据合并
		byte[] originData = dataMap.get(messageId);
		byte[] totalData;
		if (originData != null) {
			totalData = new byte[originData.length + packetData.length];

			if (offset == 0) {
				System.arraycopy(packetData, 0, totalData, 0, packetData.length);
				System.arraycopy(originData, 0, totalData, packetData.length,
						originData.length);
			} else if (offset > originData.length) {
				System.arraycopy(originData, 0, totalData, 0, originData.length);
				System.arraycopy(packetData, 0, totalData, originData.length,
						packetData.length);
			} else {
				byte[] left = new byte[offset];
				System.arraycopy(originData, 0, left, 0, left.length);
				byte[] right = new byte[originData.length - offset];
				System.arraycopy(originData, offset, right, 0,
						originData.length - offset);

				System.arraycopy(left, 0, totalData, 0, left.length);
				System.arraycopy(packetData, 0, totalData, left.length,
						packetData.length);
				System.arraycopy(right, 0, totalData, left.length
						+ packetData.length, right.length);
			}
		} else {
			totalData = new byte[packetData.length];

			System.arraycopy(packetData, 0, totalData, 0, packetData.length);
		}

		dataMap.put(messageId, totalData);

		if (totalData.length == messageLen) {
//			CustomLog.d(TAG, "combineDataPacket PushCmdData");
			// 组包完成
			// 将数据抛给图元管理模块处理
			mediaGeoMetryJni.PushCmdData(totalData);

			// 已处理的数据，从dataMap中删除
			dataMap.remove(messageId);
		}
	}

	private int byte2ToUnsignedShort(byte[] bytes, int off) {
		int high = bytes[off + 1];
		int low = bytes[off];
		return (high << 8 & 0xFF00) | (low & 0xFF);
	}

	private int byte4ToInt(byte[] bytes, int off) {
		int addr = 0;
		int b0 = bytes[off];
		int b1 = bytes[off + 1];
		int b2 = bytes[off + 2];
		int b3 = bytes[off + 3];

		addr = b0 & 0xFF;
		addr |= (((int) b1 << 8) & 0xFF00);
		addr |= (((int) b2 << 16) & 0xFF0000);
		addr |= (((int) b3 << 24) & 0xFF000000);
		return addr;
	}

	public void transfer2RespsData(UnSerializedCmdData cmdData) {
//		CustomLog.e(TAG, "transfer2RespsData begin");
		try {
			byte[] totalData = null;

			byte[] sendTypeResp = { charToByte((short) cmdData.sendType) };
			byte[] destUserNumResp = shortToByte((short) 1);

			byte[] destUserIDResp = intToByte(cmdData.accountId);

			byte[] accountIdResp = intToByte(Integer
					.parseInt(mMeetingInfo.accountId));
			byte[] packetTypeResp = { charToByte((short) 17) };
			byte[] messageIdResp = shortToByte((short) cmdData.messageId);
			byte[] dataLenResp = shortToByte((short) cmdData.dataLen);

			byte[] messageLenResp = shortToByte((short) cmdData.messageLen);
			byte[] offsetResp = shortToByte((short) cmdData.offset);
			byte[] packetNumResp = shortToByte((short) cmdData.packetNum);

			totalData = new byte[22 + cmdData.data.length];

			int dataOffSet = 0;

			System.arraycopy(sendTypeResp, 0, totalData, dataOffSet, 1);
			dataOffSet += 1;

			System.arraycopy(destUserNumResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(destUserIDResp, 0, totalData, dataOffSet, 4);
			dataOffSet += 4;

			System.arraycopy(accountIdResp, 0, totalData, dataOffSet, 4);
			dataOffSet += 4;

			System.arraycopy(packetTypeResp, 0, totalData, dataOffSet, 1);
			dataOffSet += 1;

			System.arraycopy(messageIdResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(dataLenResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(cmdData.data, 0, totalData, dataOffSet,
					cmdData.data.length);
			dataOffSet += cmdData.data.length;

			System.arraycopy(messageLenResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(offsetResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(packetNumResp, 0, totalData, dataOffSet, 2);

			mWhiteBoardProcessor.send(totalData);
//			CustomLog.d(TAG, "transfer2RespsData end totalData.length="
//					+ totalData.length + " messageId=" + cmdData.messageId);

		} catch (Exception e) {
			CustomLog.e(TAG, "transfer2RespsData error " + e.toString());
		}

	}

	private byte[] transfer2ReSendData(byte[] data) {
//		CustomLog.e(TAG, "transfer2ReSendData begin");
		UnSerializedCmdData cmdData = unSerializedCmdData(data);
		try {
			byte[] totalData = null;

			byte[] sendTypeResp = { charToByte((short) cmdData.sendType) };
			byte[] destUserNumResp = shortToByte((short) 1);

			byte[] destUserIDResp = intToByte(cmdData.accountId);

			byte[] accountIdResp = intToByte(Integer
					.parseInt(mMeetingInfo.accountId));
			byte[] packetTypeResp = { charToByte((short) 33) };
			byte[] messageIdResp = shortToByte((short) cmdData.messageId);
			byte[] dataLenResp = shortToByte((short) cmdData.dataLen);

			byte[] messageLenResp = shortToByte((short) cmdData.messageLen);
			byte[] offsetResp = shortToByte((short) cmdData.offset);
			byte[] packetNumResp = shortToByte((short) cmdData.packetNum);

			totalData = new byte[22 + cmdData.data.length];

			int dataOffSet = 0;

			System.arraycopy(sendTypeResp, 0, totalData, dataOffSet, 1);
			dataOffSet += 1;

			System.arraycopy(destUserNumResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(destUserIDResp, 0, totalData, dataOffSet, 4);
			dataOffSet += 4;

			System.arraycopy(accountIdResp, 0, totalData, dataOffSet, 4);
			dataOffSet += 4;

			System.arraycopy(packetTypeResp, 0, totalData, dataOffSet, 1);
			dataOffSet += 1;

			System.arraycopy(messageIdResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(dataLenResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(cmdData.data, 0, totalData, dataOffSet,
					cmdData.data.length);
			dataOffSet += cmdData.data.length;

			System.arraycopy(messageLenResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(offsetResp, 0, totalData, dataOffSet, 2);
			dataOffSet += 2;

			System.arraycopy(packetNumResp, 0, totalData, dataOffSet, 2);

//			CustomLog.d(TAG, "transfer2ReSendData end totalData.length="
//					+ totalData.length + " messageId=" + cmdData.messageId);

			return totalData;
		} catch (Exception e) {
			CustomLog.e(TAG, "transfer2ReSendData error " + e.toString());
		}
		return null;
	}

	private String byteToBit(byte b) {
		return "" + (byte) ((b >> 7) & 0x1) + (byte) ((b >> 6) & 0x1)
				+ (byte) ((b >> 5) & 0x1) + (byte) ((b >> 4) | 0x1)
				+ (byte) ((b >> 3) & 0x1) + (byte) ((b >> 2) & 0x1)
				+ (byte) ((b >> 1) & 0x1) + (byte) ((b >> 0) & 0x1);
	}

	private byte bitToByte(String byteStr) {
		int re, len;
		if (null == byteStr) {
			return 0;
		}
		len = byteStr.length();
		if (len != 4 && len != 8) {
			return 0;
		}
		if (len == 8) {// 8 bit处理
			if (byteStr.charAt(0) == '0') {// 正数
				re = Integer.parseInt(byteStr, 2);
			} else {// 负数
				re = Integer.parseInt(byteStr, 2) - 256;
			}
		} else {// 4 bit处理
			re = Integer.parseInt(byteStr, 2);
		}
		return (byte) re;
	}

	private void createData(byte[] data, int flag) {
//		CustomLog.e(TAG, "createData begin flag=" + flag + " data.length="
//				+ data.length);
		try {
			byte[] totalData = null;

			byte[] sendType = { charToByte((short) mSendType) };
			byte[] destUserNum;
			byte[] destUserID = null;
			if (flag > 0) {
				destUserNum = shortToByte((short) 1);
				destUserID = intToByte(flag);
			} else {
				destUserNum = shortToByte((short) 0);
			}
			byte[] accountId = intToByte(Integer
					.parseInt(mMeetingInfo.accountId));
			byte[] packetType = { charToByte((short) 1) };
			byte[] messageId = shortToByte((short) sendCount);
			// byte[] dataLen = shortToByte((short) data.length);

			byte[] messageLen = shortToByte((short) data.length);

			byte[] resId = shortToByte((short) mResId);
			byte[] shareType = shortToByte((short) mShareType);
			byte[] shareUserId = intToByte(mShareUserId);

			if (data.length > 1200) {
				// 拆包
				List<byte[]> dataList = new ArrayList<byte[]>();
				for (int i = 0; i < data.length / 1200 + 1; i++) {
					byte[] packetData;
					if (i == data.length / 1200) {
						packetData = new byte[data.length - 1200 * i];
					} else {
						packetData = new byte[1200];
					}
					System.arraycopy(data, 1200 * i, packetData, 0,
							packetData.length);

					dataList.add(packetData);
				}
				// 将拆包分开发送
				for (int i = 0; i < dataList.size(); i++) {
					byte[] offset = shortToByte((short) (1200 * i));

					if (flag > 0) {
						totalData = new byte[22 + 8 + dataList.get(i).length];
					} else {
						totalData = new byte[18 + 8 + dataList.get(i).length];
					}

					byte[] dataLen = shortToByte((short) dataList.get(i).length);
					byte[] packetNum = shortToByte((short) dataList.size());

					int dataOffSet = 0;

					System.arraycopy(sendType, 0, totalData, dataOffSet, 1);
					dataOffSet += 1;

					System.arraycopy(destUserNum, 0, totalData, dataOffSet, 2);
					dataOffSet += 2;

					if (flag > 0) {
						System.arraycopy(destUserID, 0, totalData, dataOffSet,
								4);
						dataOffSet += 4;
					}
					System.arraycopy(accountId, 0, totalData, dataOffSet, 4);
					dataOffSet += 4;

					System.arraycopy(packetType, 0, totalData, dataOffSet, 1);
					dataOffSet += 1;

					System.arraycopy(messageId, 0, totalData, dataOffSet, 2);
					dataOffSet += 2;

					System.arraycopy(dataLen, 0, totalData, dataOffSet, 2);
					dataOffSet += 2;

					System.arraycopy(dataList.get(i), 0, totalData, dataOffSet,
							dataList.get(i).length);
					dataOffSet += dataList.get(i).length;

					System.arraycopy(messageLen, 0, totalData, dataOffSet, 2);
					dataOffSet += 2;

					System.arraycopy(offset, 0, totalData, dataOffSet, 2);
					dataOffSet += 2;

					System.arraycopy(packetNum, 0, totalData, dataOffSet, 2);
					dataOffSet += 2;

					System.arraycopy(resId, 0, totalData, dataOffSet, 2);
					dataOffSet += 2;

					System.arraycopy(shareType, 0, totalData, dataOffSet, 2);
					dataOffSet += 2;

					System.arraycopy(shareUserId, 0, totalData, dataOffSet, 4);

					ConsumeDataInfo consumeDataInfo = new ConsumeDataInfo();
					consumeDataInfo.data = totalData;
					consumeDataInfo.sendCount = 0;
					consumeDataInfo.messageId = sendCount;

					mConsumeDataMap.put(sendCount + 1200 * i, consumeDataInfo);

//					CustomLog.d(TAG, "createData end（第" + (i + 1)
//							+ "个包） totalData.length=" + totalData.length
//							+ " dataLen=" + dataList.get(i).length + " offset="
//							+ 1200 * i);
				}
			} else {
				byte[] offset = shortToByte((short) 0);

				if (flag > 0) {
					totalData = new byte[22 + 8 + data.length];
				} else {
					totalData = new byte[18 + 8 + data.length];
				}

				byte[] dataLen = shortToByte((short) data.length);
				byte[] packetNum = shortToByte((short) 1);

				int dataOffSet = 0;

				System.arraycopy(sendType, 0, totalData, dataOffSet, 1);
				dataOffSet += 1;

				System.arraycopy(destUserNum, 0, totalData, dataOffSet, 2);
				dataOffSet += 2;

				if (flag > 0) {
					System.arraycopy(destUserID, 0, totalData, dataOffSet, 4);
					dataOffSet += 4;
				}
				System.arraycopy(accountId, 0, totalData, dataOffSet, 4);
				dataOffSet += 4;

				System.arraycopy(packetType, 0, totalData, dataOffSet, 1);
				dataOffSet += 1;

				System.arraycopy(messageId, 0, totalData, dataOffSet, 2);
				dataOffSet += 2;

				System.arraycopy(dataLen, 0, totalData, dataOffSet, 2);
				dataOffSet += 2;

				System.arraycopy(data, 0, totalData, dataOffSet, data.length);
				dataOffSet += data.length;

				System.arraycopy(messageLen, 0, totalData, dataOffSet, 2);
				dataOffSet += 2;

				System.arraycopy(offset, 0, totalData, dataOffSet, 2);
				dataOffSet += 2;

				System.arraycopy(packetNum, 0, totalData, dataOffSet, 2);
				dataOffSet += 2;

				System.arraycopy(resId, 0, totalData, dataOffSet, 2);
				dataOffSet += 2;

				System.arraycopy(shareType, 0, totalData, dataOffSet, 2);
				dataOffSet += 2;

				System.arraycopy(shareUserId, 0, totalData, dataOffSet, 4);

				ConsumeDataInfo consumeDataInfo = new ConsumeDataInfo();
				consumeDataInfo.data = totalData;
				consumeDataInfo.sendCount = 0;
				consumeDataInfo.messageId = sendCount;

				mConsumeDataMap.put((int) sendCount, consumeDataInfo);

//				CustomLog.d(TAG, "createData end totalData.length="
//						+ totalData.length);
			}

			if (sendCount == 32767) {
				sendCount = 0;
			} else {
				sendCount++;
			}

		} catch (Exception e) {
			CustomLog.e(TAG, "createData error " + e.toString());
		}
	}

	private byte charToByte(short num) {
		byte b = (byte) (num & 0xff);
		return b;
	}

	private byte[] shortToByte(short num) {
		int temp = num;
		byte[] b = new byte[2];
		for (int i = 0; i < b.length; i++) {
			b[i] = new Integer(temp & 0xff).byteValue();

			temp = temp >> 8; // 向右移8位
		}
		return b;
	}

	private byte[] intToByte(int num) {
		byte[] bytes = new byte[4];
		bytes[3] = (byte) ((num >> 24) & 0xff);
		bytes[2] = (byte) ((num >> 16) & 0xff);
		bytes[1] = (byte) ((num >> 8) & 0xff);
		bytes[0] = (byte) (num & 0xff);
		return bytes;
	}

	public int release() {
		CustomLog.d(TAG, "GeoMetryControl::release()");

		dataMap.clear();
		dataRecive.clear();

		msgIdMap.clear();
		mUserOrderData.clear();
		mMinMessageId.clear();
		mConsumeDataMap.clear();

		mediaGeometryUninit();

		return 0;
	}

	public int mediaGeometryInit(String logPath, String configPath,
			String rkPath, int userid, DrawCommandCallback cb) {
		CustomLog.d(TAG, "mediaGeometryInit logPath=" + logPath
				+ " configPath=" + configPath + " rkPath=" + rkPath
				+ " userid=" + userid);

		mReOrder = true;
		dealReOrder = new DealReOrder();
		dealReOrder.start();

		mConsumeData = true;
		dealConsumeData = new DealConsumeData();
		dealConsumeData.start();

		return mediaGeoMetryJni.MediaGeometryInit(logPath, configPath, rkPath,
				userid, drawCommandCallback);
	}

	private void mediaGeometryUninit() {
		CustomLog.d(TAG, "mediaGeometryUninit");
		mReOrder = false;
		try {
			dealReOrder.join();
		} catch (InterruptedException e) {
			CustomLog.d(TAG,
					"mediaGeometryUninit join dealReOrder thread failed");
		}
		dealReOrder = null;

		mConsumeData = false;
		try {
			dealConsumeData.join();
		} catch (InterruptedException e) {
			CustomLog.d(TAG,
					"mediaGeometryUninit join dealConsumeData thread failed");
		}
		dealConsumeData = null;

		mUserOrderData.clear();
		mMinMessageId.clear();
		dataMap.clear();
		dataRecive.clear();
		msgIdMap.clear();

		mediaGeoMetryJni.MediaGeometryUninit();
	}

	public int addStreamInfo(SurfaceView surface, int userid, int streamid) {
		CustomLog.d(TAG, "addStreamInfo surface=" + surface + " userid="
				+ userid + " streamid=" + streamid);
		mShareUserId = userid;
		return mediaGeoMetryJni.AddStreamInfo(surface, userid, streamid);
	}

	public int removeStreamInfo(int userid, int streamid) {
		CustomLog.d(TAG, "removeStreamInfo userid=" + userid + " streamid="
				+ streamid);
		return mediaGeoMetryJni.RemoveStreamInfo(userid, streamid);
	}

	public int setGraphicManagerActor(int type) {
		CustomLog.d(TAG, "setGraphicManagerActor type=" + type);
		mShareType = type;
		if (type == ShareType.SHARETYPE_WHITE_BOARD) {
			return mediaGeoMetryJni
					.SetGraphicManagerActor(MediaGeoMetryJni.WHITE_BOARD_MANAGER);
		} else if (type == ShareType.SHARETYPE_CAPTURE_CARD) {
			return mediaGeoMetryJni
					.SetGraphicManagerActor(MediaGeoMetryJni.GRAPHIC_LAYER_MANAGER);
		} else {
			return mediaGeoMetryJni
					.SetGraphicManagerActor(MediaGeoMetryJni.PROGRESS_CAPTURE_MANAGER);
		}
	}

	public int operateSelect(int sel, int color, int penwidth) {
		CustomLog.d(TAG, "operateSelect sel=" + sel + " color=" + color
				+ " penwidth=" + penwidth);

		return mediaGeoMetryJni.OperateSelect(sel, color, penwidth);
	}

	public int operateCancel() {
		CustomLog.d(TAG, "operateCancel()");

		return mediaGeoMetryJni.OperateCancel();
	}

	public int requestDraw(int userid) {
		CustomLog.d(TAG, "requestDraw userid=" + userid);

		return mediaGeoMetryJni.RequestDraw(userid);
	}

	public int leaveMeeting(int userid) {
		CustomLog.d(TAG, "leaveMeeting userid=" + userid);
		if (msgIdMap.containsKey(userid)) {
			CustomLog.d(TAG, "leaveMeeting remove userid=" + userid
					+ "from msgIdMap!");
			msgIdMap.remove(userid);
		}
		if (mUserOrderData.containsKey(userid)) {
			CustomLog.d(TAG, "leaveMeeting remove userid=" + userid
					+ "from mUserOrderData!");
			mUserOrderData.remove(userid);
		}
		if (mMinMessageId.containsKey(userid)) {
			CustomLog.d(TAG, "leaveMeeting remove userid=" + userid
					+ "from mMinMessageId!");
			mMinMessageId.remove(userid);
		}
		return mediaGeoMetryJni.LeaveMeeting(userid);
	}

	public void clearData(int userid) {
		CustomLog.d(TAG, "clearData userid=" + userid);
		if (userid == 0) {
			CustomLog.d(TAG, "白板共享结束 clearData AllMap");
			mConsumeDataMap.clear();
			msgIdMap.clear();
			mUserOrderData.clear();
			mMinMessageId.clear();
			dataMap.clear();
			dataRecive.clear();
		} else {
			if (msgIdMap.containsKey(userid)) {
				CustomLog.d(TAG, "clearData remove userid=" + userid
						+ "from msgIdMap!");
				msgIdMap.remove(userid);
			}
			if (mUserOrderData.containsKey(userid)) {
				CustomLog.d(TAG, "clearData remove userid=" + userid
						+ "from mUserOrderData!");
				mUserOrderData.remove(userid);
			}
			if (mMinMessageId.containsKey(userid)) {
				CustomLog.d(TAG, "clearData remove userid=" + userid
						+ "from mMinMessageId!");
				mMinMessageId.remove(userid);
			}
		}
	}

	public int drawText(String text, int font, int color, int fontSize, int x,
			int y) {
		CustomLog.d(TAG, "drawText text=" + text + " font=" + font + " color="
				+ color + " fontSize=" + fontSize + " x=" + x + " y=" + y);
		return mediaGeoMetryJni.DrawText(text, font, color, fontSize, x, y);
	}

	public int drawCommand(int cmd) {
		CustomLog.d(TAG, "drawCommand cmd=" + cmd);

		return mediaGeoMetryJni.DrawCommand(cmd);
	}

	public int imageOperate(int op, String path, int left, int top, int right,
			int bottom) {
		CustomLog.d(TAG, "imageOperate op=" + op + " path=" + path + " left="
				+ left + " top=" + top + " right=" + right + " bottom="
				+ bottom);

		return mediaGeoMetryJni
				.ImageOperate(op, path, left, top, right, bottom);
	}

	public int setWhiteBoardSendType(int sendType) {
		CustomLog.d(TAG, "setWhiteBoardSendType sendType=" + sendType);
		mSendType = sendType;
		return 0;
	}

	public UnSerializedCmdData unSerializedCmdData(byte[] bytes) {
		UnSerializedCmdData cmdData = new UnSerializedCmdData();
		int analyzeOffSet = 0;
		short sendType = (short) (bytes[analyzeOffSet] & 0xFF);
		analyzeOffSet += 1;
		cmdData.sendType = sendType;

		short destUserNum = (short) byte2ToUnsignedShort(bytes, analyzeOffSet);
		analyzeOffSet += 2;
		cmdData.destUserNum = destUserNum;

		List<Integer> destUserIDList = new ArrayList<Integer>();
		for (int i = 0; i < destUserNum; i++) {
			int destUserID = byte4ToInt(bytes, analyzeOffSet);
			destUserIDList.add(destUserID);
			analyzeOffSet += 4;
//			CustomLog.d(TAG, "analyzeData destUserID=" + destUserID);
		}
		cmdData.destUserIDList = destUserIDList;

		int accountId = byte4ToInt(bytes, analyzeOffSet);
		analyzeOffSet += 4;
		cmdData.accountId = accountId;

		short packetType = (short) (bytes[analyzeOffSet] & 0xFF);
		analyzeOffSet += 1;
		cmdData.packetType = packetType;

		int messageId = byte2ToUnsignedShort(bytes, analyzeOffSet);
		analyzeOffSet += 2;
		cmdData.messageId = messageId;

		short dataLen = (short) byte2ToUnsignedShort(bytes, analyzeOffSet);
		analyzeOffSet += 2;
		cmdData.dataLen = dataLen;

		byte[] data = new byte[dataLen];
		System.arraycopy(bytes, analyzeOffSet, data, 0, dataLen);
		analyzeOffSet += dataLen;
		cmdData.data = data;

		short messageLen = (short) byte2ToUnsignedShort(bytes, analyzeOffSet);
		analyzeOffSet += 2;
		cmdData.messageLen = messageLen;

		short offset = (short) byte2ToUnsignedShort(bytes, analyzeOffSet);
		analyzeOffSet += 2;
		cmdData.offset = offset;

		short packetNum = (short) byte2ToUnsignedShort(bytes, analyzeOffSet);
		cmdData.packetNum = packetNum;

		cmdData.recvtime = System.currentTimeMillis();

//		CustomLog.d(TAG, "analyzeData " + cmdData.toString());

		return cmdData;
	}

}
