package cn.redcdn.crash;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.Thread.UncaughtExceptionHandler;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import android.content.Context;
import android.os.Environment;
import android.util.Log;

/*
 * 使用规则：
 * 必须在manifest中添加android.permission.WRITE_EXTERNAL_STORAGE权限
 * 接口init中processName不能包含无效的字符
 */
public class Crash {
  private UncaughtExceptionHandler defaultHandler = null;
  private String crashDirPath = null;
  private int maxCrashFileCount = 10;
  private String mDir = "meeting";
  
  public void setDir(String dir) {
    mDir = dir;
  }
  
  public int init(Context context, String processName) {
    if (null == context || null == processName) {
      return -1;
    }
    
    if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
//      crashDirPath = Environment.getExternalStorageDirectory().getPath() +
//          "/meeting/crash/" + processName;
      crashDirPath = Environment.getExternalStorageDirectory().getPath() +"/"+mDir+
          "/crash/" + processName;
      Log.i("crash", "crashDirPath:" + crashDirPath);
    }

    defaultHandler = Thread.getDefaultUncaughtExceptionHandler();
    Thread.setDefaultUncaughtExceptionHandler(new UncaughtExceptionHandler() {
      @Override
      public void uncaughtException(Thread thread, Throwable ex) {
        if (null == crashDirPath) {
          defaultHandler.uncaughtException(thread, ex);
          return;
        }

        File crashFolder = new File(crashDirPath);
        if (!crashFolder.exists()) {
          crashFolder.mkdirs();
        }

        if (!crashFolder.isDirectory()) {
          defaultHandler.uncaughtException(thread, ex);
          return;
        }

        deleteOldFile(crashDirPath);

        Writer writer = new StringWriter();
        PrintWriter pinter = new PrintWriter(writer);
        ex.printStackTrace(pinter);
        String stackString = writer.toString();
        pinter.close();
        try {
          writer.close();
        } catch (IOException e) {
          e.printStackTrace();
        }

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
        writeCrashToFile(crashDirPath + "/" + dateFormat.format(new Date()) + ".crash",
            stackString);

        defaultHandler.uncaughtException(thread, ex);
        System.exit(0);
      }
    });
    
    return 0;
  }

  private void writeCrashToFile(String fileName, String fileContent) {
    try {
      FileOutputStream outStream  = new FileOutputStream(fileName);
      outStream.write(fileContent.getBytes());
      outStream.flush();
      outStream.close();
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }
  
  private void deleteOldFile(String crashDirPath) {
    File crashFileDir = new File(crashDirPath);
    if (!crashFileDir.isDirectory()) {
      return;
    }

    String[] fileNames = crashFileDir.list();
    if (fileNames.length < maxCrashFileCount) {
      return;
    }

    List<String> sortedListName = new ArrayList<String>();
    for (String name : fileNames) {
      sortedListName.add(name);
    }
    Collections.sort(sortedListName);

    int deleteCount = fileNames.length - maxCrashFileCount + 1;
    for (int i=0; i<deleteCount; i++) {
      File file = new File(crashDirPath + "/" + sortedListName.get(i));
      if (file.isFile()) {
        file.delete();
      }
    }
  }
}
