package cn.redcdn.datacenter;

import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.content.Context;
import android.text.TextUtils;

import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;
import cn.redcdn.network.httprequest.ConnectException;
import cn.redcdn.network.httprequest.PostTextHttp;

public class AbstractBusinessDataAuth<T> {


    /**
     * masterIp  主服务器IP地址
     * slaveIp   从服务器IP地址
     * slaveUrl  从服务器后台接口请求地址
     */
    public String masterIp = ConstConfig.getAuthorizeUrlPrefix();
    public String slaveIp = ConstConfig.getSlaveAuthorizeUrlPrefix();
    public String slaveUrl = slaveIp + "auth";

    private String tag = getClass().getName();
    private PostTextHttp httpPost;
    private boolean isReturn = false;

    /**
     * 定义master和slave服务器模式变量
     */
    private boolean bSlaveHttpStatus = false;

    // -1: 表示无效的操作，当前操作正在执行;
    // -2: 表示无效的编码;

    protected T execSync(String url, final String content) {
        httpPost = new PostTextHttp();
        httpPost.setURL(url);

        String responseContent = null;
        Parser parser = getParser();

        try {
            responseContent = httpPost.postUrlEncodeContentSync(content);
            CustomLog.d(tag, " ResponseContent = " + responseContent);
            if (parser != null) {
                parser.parse(responseContent);
                if (parser.isOk()) {
                    if (responseContent != null) {
                        isReturn = true;
                        return parseContentBody(parser.getBody());
                    }
                } else {
                    if (bSlaveHttpStatus == false && isReturn == false) {
                        if (TextUtils.isEmpty(slaveIp)) {
                            CustomLog.d(tag, "从服务器IP为空 0,onFail");

                            if (responseContent != null) {
                                isReturn = true;
                                return parseContentBody(parser.getBody());
                            }
                        } else {
                            CustomLog.d(tag, "请求从服务器 0: url = " + slaveUrl);
                            CustomLog.w(tag, "connect to slave server: " + content);
                            bSlaveHttpStatus = true;

                            httpPost = null;
                            return execSync(slaveUrl, content);
                        }
                    } else {
                        if (responseContent != null) {
                            isReturn = true;
                            return parseContentBody(parser.getBody());
                        }
                    }
                }
            } else {
                if (bSlaveHttpStatus == false && isReturn == false) {
                    if (TextUtils.isEmpty(slaveIp)) {
                        CustomLog.d(tag, "从服务器IP为空 0,onFail");

                        if (responseContent != null) {
                            isReturn = true;
                            return parseContentBody(parser.getBody());
                        }
                    } else {
                        CustomLog.d(tag, "请求从服务器 0: url = " + slaveUrl);
                        CustomLog.w(tag, "connect to slave server: " + content);
                        bSlaveHttpStatus = true;

                        httpPost = null;
                        return execSync(slaveUrl, content);
                    }
                } else {
                    if (responseContent != null) {
                        isReturn = true;
                        return parseContentBody(parser.getBody());
                    }
                }
            }
        } catch (ConnectException e) {
            CustomLog.e(tag, "ConnectException");
            if (bSlaveHttpStatus) {
                throw new ConnectException(e.getMessage());
            }
        } finally {
            if (bSlaveHttpStatus == false && isReturn == false) {
                if (TextUtils.isEmpty(slaveIp)) {
                    CustomLog.d(tag, "从服务器IP为空 1,onFail");
                    if (responseContent != null) {
                        isReturn = true;
                        return parseContentBody(parser.getBody());
                    }
                } else {
                    CustomLog.d(tag, "请求从服务器 1: url = " + slaveUrl);
                    CustomLog.w(tag, "connect to slave server: " + content);
                    bSlaveHttpStatus = true;

                    httpPost = null;
                    return execSync(slaveUrl, content);
                }
            }
        }


        return null;
    }

    protected T execSyncWithHttps(String url, final String content, String filePath) {
        httpPost = new PostTextHttp();
        httpPost.setURL(url);

        String responseContent = null;
        Parser parser = getParser();

        try {
            responseContent = httpPost.postUrlContentSyncWithHttps(filePath, content);
            CustomLog.d(tag, " ResponseContent = " + responseContent);
            if (parser != null) {
                parser.parse(responseContent);
                if (parser.isOk()) {
                    if (responseContent != null) {
                        isReturn = true;
                        return parseContentBody(parser.getBody());
                    }
                } else {
                    if (bSlaveHttpStatus == false && isReturn == false) {
                        if (TextUtils.isEmpty(slaveIp)) {
                            CustomLog.d(tag, "从服务器IP为空,onFail");
                            if (responseContent != null) {
                                isReturn = true;
                                return parseContentBody(parser.getBody());
                            }
                        } else {
                            CustomLog.d(tag, "请求从服务器: url = " + slaveUrl);
                            CustomLog.w(tag, "connect to slave server: " + content);
                            bSlaveHttpStatus = true;

                            httpPost = null;
                            return execSyncWithHttps(slaveUrl, filePath, content);
                        }
                    }
                }
            } else {
                if (bSlaveHttpStatus == false && isReturn == false) {
                    if (TextUtils.isEmpty(slaveIp)) {
                        CustomLog.d(tag, "从服务器IP为空,onFail");
                        if (responseContent != null) {
                            isReturn = true;
                            return parseContentBody(parser.getBody());
                        }
                    } else {
                        CustomLog.d(tag, "请求从服务器: url = " + slaveUrl);
                        CustomLog.w(tag, "connect to slave server: " + content);
                        bSlaveHttpStatus = true;

                        httpPost = null;
                        return execSyncWithHttps(slaveUrl, filePath, content);
                    }
                }
            }
        } catch (ConnectException e) {
            CustomLog.e(tag, "ConnectException");
            if (bSlaveHttpStatus) {
                throw new ConnectException(e.getMessage());
            }
        } finally {
            if (bSlaveHttpStatus == false && isReturn == false) {
                if (TextUtils.isEmpty(slaveIp)) {
                    CustomLog.d(tag, "从服务器IP为空,onFail");
                    if (responseContent != null) {
                        isReturn = true;
                        return parseContentBody(parser.getBody());
                    }
                } else {
                    CustomLog.d(tag, "请求从服务器: url = " + slaveUrl);
                    CustomLog.w(tag, "connect to slave server: " + content);
                    bSlaveHttpStatus = true;

                    httpPost = null;
                    return execSyncWithHttps(slaveUrl, filePath, content);
                }
            }
        }

        return null;
    }

    // -1: 表示无效的操作，当前操作正在执行;
    // -2: 表示无效的编码;
    protected int exec(String url, final String content) {
        if (null != httpPost) {

            return -1;
        }
        httpPost = new PostTextHttp() {

            @SuppressLint("NewApi")
            @Override
            protected void onRequestFailed(int statusCode) {
                CustomLog.e(tag, "onRequestFailed. statusCode =" + statusCode);
                if (statusCode != 0) {
                    if (bSlaveHttpStatus == false && !slaveIp.isEmpty()) {
                        CustomLog.w(tag, "connect to slave server" + content);

                        bSlaveHttpStatus = true;
                        httpPost = null;
                        exec(slaveUrl, content);
                    } else {
                        onFail(statusCode, null);
                    }
                } else {
                    onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                }
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {

                CustomLog.d("onRequestFinished:", responseContent);

                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }

                parser.parse(responseContent);
                if (parser.isOk()) {
                    try {
                        onSuccess(parseContentBody(parser.getBody()));
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                    }

                } else {
                    if (bSlaveHttpStatus == false && !slaveIp.isEmpty()) {
                        CustomLog.w(tag, "connect to slave server" + content);

                        bSlaveHttpStatus = true;
                        httpPost = null;
                        exec(slaveUrl, content);
                    } else {
                        onFail(parser.getHeaderValue(),parser.getBody().toString());
//                        onFailForAcquireAccountInfo(parser.getHeaderValue(), parser.getBody().toString());
                    }
                }
            }
        };
        httpPost.setURL(url);
        return httpPost.postUrlEncodeContent(content);
    }

    public void cancel() {
        if (null == httpPost) {
            return;
        }
        try {
            httpPost.cancel();
            httpPost = null;
        } catch (Exception e) {
            CustomLog.e(tag, "cancel:" + e.toString());
        }

    }

    protected T parseContentBody(JSONObject jo) {
        return null;
    }

    protected T parseContentBody(int headerValue, JSONObject jo)
            throws InvalidateResponseException {
        return parseContentBody(jo);
    }

    protected Parser getParser() {
        return null;
    }

    protected void onSuccess(T responseContent) {
    }

    ;


    // 返回HttpErrorCode、接口业务逻辑错误码、DataErrorCode
    protected void onFail(int statusCode, String statusInfo) {
        CustomLog.e(tag, "onFail, statusCode = " + statusCode + " statusInfo = " + statusInfo);
    }

    ;


}
