package cn.redcdn.datacenter;

import org.json.JSONObject;


import cn.redcdn.log.CustomLog;
import cn.redcdn.network.httprequest.PostTextHttp;

/**
 * AbstractBusinessDataBoth同时访问两台服务器，并对返回结果进行合并
 * 
 * @author Administrator
 *
 * @param <T>
 * 
 * GetNowMeetings(获取被邀请参加的所有会议)
 * 
 */

public class AbstractBusinessDataBoth<T> {
	
	private String tag = getClass().getName();
	private String FirstServerUrl;//1服务器后台接口请求地址
	private String SecondServerUrl;//2服务器后台接口请求地址
	private PostTextHttp FirstHttpPost;
	private PostTextHttp SecondHttpPost;
	private String FirstServerResponseContent=null;
	private String SecondServerResponseContent=null;
	public static final String Devide = "0_|_0";	
	private ServerStaus FirstServerStatus = ServerStaus.STATUS_NONE; //1服务器状态
	private ServerStaus SecondServerStatus = ServerStaus.STATUS_NONE; //2服务器状态
	private String requestUrl;
	
	private enum ServerStaus{
		STATUS_NONE, //初始状态
		STATUS_SUCCESS, //成功
		STATUS_FAILED //失败
	}
	
	public int exec(String FirstUrl,String SecondUrl, final String content){
		FirstServerUrl = FirstUrl;
		SecondServerUrl = SecondUrl;
		requestUrl = FirstServerUrl;
		
		int FirstServerReturn = FirstServerExec(FirstServerUrl,content);
		int SecondServerReturn = SecondServerExec(SecondServerUrl,content);
		
		if(FirstServerReturn<0&&SecondServerReturn<0){
			return -1000;
		}
		
		return 0;
		
	}
	
	/*
	 * 1服务器
	 */
	// -1: 表示无效的操作，当前操作正在执行;
	// -2: 表示无效的编码;
	protected int FirstServerExec(String url, final String content){
		
		if(null!=FirstHttpPost){
			return -1;
		}
		
		FirstHttpPost = new PostTextHttp() {
			
		@Override
		protected void onRequestFailed(int statusCode){
			FirstServerStatus = ServerStaus.STATUS_FAILED;
			
            if(SecondServerStatus == ServerStaus.STATUS_NONE){
            	return;
			}else if(SecondServerStatus == ServerStaus.STATUS_SUCCESS){
				if(null!=SecondServerResponseContent){
					ProcessSuccess(SecondServerResponseContent);
				}
				return;
			}else if(SecondServerStatus == ServerStaus.STATUS_FAILED){
			  ProcessFail(statusCode);
			}
			
		}
		
		@Override
		protected void onRequestFinished(int statusCode, String responseContent){
			CustomLog.d(tag,"onRequestFinished1: "+responseContent);
			FirstServerStatus = ServerStaus.STATUS_SUCCESS;
			FirstServerResponseContent = responseContent;
			if(SecondServerStatus == ServerStaus.STATUS_NONE){
				CustomLog.d(tag,"SecondServerStatus == ServerStaus.STATUS_NONE");
//				ProcessSuccess(responseContent);
			}else if(SecondServerStatus == ServerStaus.STATUS_SUCCESS){
				CustomLog.d(tag,"SecondServerStatus == ServerStaus.STATUS_SUCCESS");
				if(null!=SecondServerResponseContent){
				CustomLog.d(tag, "null!=SecondServerResponseContent");	
				ProcessSuccess(responseContent+Devide+SecondServerResponseContent);
				}else{
				CustomLog.d(tag, "null==SecondServerResponseContent");
				}
				
			}else if(SecondServerStatus == ServerStaus.STATUS_FAILED){
				CustomLog.d(tag,"SecondServerStatus == ServerStaus.STATUS_FAILED");
				ProcessSuccess(responseContent);
			}

		}
			
		};
		
		FirstHttpPost.setURL(FirstServerUrl);
		return FirstHttpPost.postUrlEncodeContent(content);
		
	}

	/*
	 * 2服务器
	 */
	// -1: 表示无效的操作，当前操作正在执行;
	// -2: 表示无效的编码;
	protected int SecondServerExec(String url, final String content){
		
		if(null!=SecondHttpPost){
			return -1;
		}
		
		SecondHttpPost = new PostTextHttp() {
			
		@Override
		protected void onRequestFailed(int statusCode){
			SecondServerStatus = ServerStaus.STATUS_FAILED;
			
			if(FirstServerStatus == ServerStaus.STATUS_NONE){
				return;
			}else if(FirstServerStatus == ServerStaus.STATUS_SUCCESS){
				if(null!=FirstServerResponseContent){
					ProcessSuccess(FirstServerResponseContent);
				}
				return;
			}else if(FirstServerStatus == ServerStaus.STATUS_FAILED){
			  ProcessFail(statusCode);
			}
			
		}
		
		@Override
		protected void onRequestFinished(int statusCode, String responseContent){
			CustomLog.d(tag,"onRequestFinished2: "+responseContent);
			SecondServerStatus = ServerStaus.STATUS_SUCCESS;
			SecondServerResponseContent = responseContent;
			if(FirstServerStatus == ServerStaus.STATUS_NONE){
				CustomLog.d(tag,"SecondServerStatus = ServerStaus.STATUS_SUCCESS"
			                +" & FirstServerStatus == ServerStaus.STATUS_NONE");
//				ProcessSuccess(responseContent);
			}else if(FirstServerStatus == ServerStaus.STATUS_SUCCESS){
				CustomLog.d(tag, ""
						    +" & FirstServerStatus == ServerStaus.STATUS_SUCCESS");
				if(null!=FirstServerResponseContent){
				CustomLog.d(tag,"null!=FirstServerResponseContent");	
				ProcessSuccess(FirstServerResponseContent+Devide+responseContent);
				}else{
					CustomLog.d(tag,"null==FirstServerResponseContent");	
				}
			}else if(FirstServerStatus == ServerStaus.STATUS_FAILED){
				CustomLog.d(tag, "FirstServerStatus == ServerStaus.STATUS_FAILED");
				ProcessSuccess(responseContent);
			}
			
		}
			
		};
		SecondHttpPost.setURL(SecondServerUrl);
		return SecondHttpPost.postUrlEncodeContent(content);
	}
	
	 public void cancel() {
		    if (null == FirstHttpPost) {
		      return;
		    }else{
		    try {
			    FirstHttpPost.cancel();
			    FirstHttpPost = null;
		    } catch(Exception e) {
		    	CustomLog.e(tag,"first cancel:" + e.toString());
		    }

		    }
		    
		    if (null == SecondHttpPost) {
			  return;
			}else{
			try {
				SecondHttpPost.cancel();
				SecondHttpPost = null;
			} catch (Exception e) {
				CustomLog.e(tag,"second cancel:" + e.toString());
			}

			}

		  }

		  protected T parseContentBody(JSONObject jo)
		      throws InvalidateResponseException {
		    return null;
		  }

		  protected T parseContentBody(int headerValue, JSONObject jo)
		      throws InvalidateResponseException { 
		    return parseContentBody(jo);
		  }

		  protected Parser getParser() {
		    return null;
		  }

		  protected void onSuccess(T responseContent) {
			  CustomLog.d(tag,"onSuccess");
		  };
		  
		  // 返回HttpErrorCode、接口业务逻辑错误码、DataErrorCode
		  protected void onFail(int statusCode, String statusInfo) {
		    CustomLog.e(tag,"onFail, statusCode = "+ statusCode+" statusInfo = "+statusInfo+"requestUrl = "+requestUrl);
		  };
          
		  //处理onRequestFinished
		  protected void ProcessSuccess(String responseContent) {
			  Parser parser = getParser();
			  
			  CustomLog.e(tag,"AbstractBusinessDataBoth responseContent:"+responseContent);
			  
		        if (null == parser) {
		          onFail(DataErrorCode.DATA_INVALIDATE_PARSER,  DataErrorInfo.SERVICE_FAILED);
		          return;
		        }
		     
		        parser.parse(responseContent);
		        if (parser.isOk()) {
		        	Object ob = null ;
		            try {
		            ob  = parseContentBody(parser.getHeaderValue(), parser.getBody());		            
		            } catch (InvalidateResponseException e) {
		              onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
		              return;
		            } catch (ClassCastException e) {
		              onFail(DataErrorCode.CLASS_CAST_EXCEPTION,DataErrorInfo.SERVICE_FAILED);
		              return;
		            } catch (NumberFormatException e) {
		              onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION,DataErrorInfo.SERVICE_FAILED);
		              return;
		            }
		            
		            onSuccess((T) ob);
		        } else {
		            onFail(parser.getHeaderValue(), parser.getHeaderText());
		        }
			}
		  
		  //处理onRequestFailed
		  protected void ProcessFail(int statusCode) {
                onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
			}
		  
}
