package cn.redcdn.datacenter;

import org.json.JSONObject;

import android.annotation.SuppressLint;

import cn.redcdn.log.CustomLog;
import cn.redcdn.network.httprequest.ConnectException;
import cn.redcdn.network.httprequest.PostTextHttp;

public class AbstractBusinessDataForAcquireAccountInfo<T> {
    private String tag = AbstractBusinessDataForAcquireAccountInfo.class.getName();

    private PostTextHttp httpPost;

    private String requestUrl;

    // 定义master和slvae服务器模式变量
    private boolean bSlaveHttpStatus = false;

    //正常情况返回类型为T的数据对象
    //异常情况下抛如下异常:
    //InvalidateOperateException: 无效的操作
    //ConnectException: 连接服务器失败;
    //InvalidateHttpResponse: 无效得http response,比如http response字段错误,以及http response io exception
    //UnhandledHttpStatus: 比如302,404等其他status code
    @SuppressLint("NewApi")
    protected T execSync(String url, final String content) {
        httpPost = new PostTextHttp();
        httpPost.setURL(url);
        requestUrl = url;
        String responseContent = null;
        try {
            responseContent = httpPost.postUrlEncodeContentSync(content);
            CustomLog.d(tag, " ResponseContent1 = " + responseContent);
        } catch (ConnectException e) {
            String slaveUrl = getSlaveUrl();
            if (slaveUrl.isEmpty()) {
                CustomLog.e(tag, " slaveUrl.isEmpty()");
                throw new ConnectException(e.getMessage());
            }
            CustomLog.w(tag, "connect to slave server");
            httpPost.setURL(slaveUrl);
            responseContent = httpPost.postUrlEncodeContentSync(content);
            CustomLog.d(tag, " ResponseContent2 = " + responseContent);
        }
        Parser parser = getParser();
        if (parser != null) {
            parser.parse(responseContent);
            try {
                return parseContentBody(parser.getBody());
            } catch (InvalidateResponseException e) {
            }
        }

        return null;
    }

    // -1: 表示无效的操作，当前操作正在执行;
    // -2: 表示无效的编码;
    protected int exec(String url, final String content) {
        if (null != httpPost) {

            return -1;
        }
        requestUrl = url;
        httpPost = new PostTextHttp() {

            @SuppressLint("NewApi")
            @Override
            protected void onRequestFailed(int statusCode) {
                CustomLog.e(tag, "onRequestFailed. statusCode =" + statusCode);
                if (statusCode != 0) {
                    if (bSlaveHttpStatus == false && !getSlaveUrl().isEmpty()) {
                        CustomLog.w(tag, "connect to slave server" + content);

                        bSlaveHttpStatus = true;
                        httpPost = null;
                        exec(getSlaveUrl(), content);
                    } else {
//            onFail(statusCode, null);
                        onFailForAcquireAccountInfo(statusCode, null);
                    }
                } else {
                    onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                }
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {

                CustomLog.d("onRequestFinished:", responseContent);

                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }

                parser.parse(responseContent);
                if (parser.isOk()) {
                    try {
                        onSuccess(parseContentBody(parser.getBody()));
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                    }

                } else {
                    if (bSlaveHttpStatus == false && !getSlaveUrl().isEmpty()) {
                        CustomLog.w(tag, "connect to slave server" + content);

                        bSlaveHttpStatus = true;
                        httpPost = null;
                        exec(getSlaveUrl(), content);
                    } else {
                        onFailForAcquireAccountInfo(parser.getHeaderValue(), parser.getBody().toString());
                    }
                }
            }
        };
        httpPost.setURL(url);
        return httpPost.postUrlEncodeContent(content);
    }

    protected String getSlaveUrl() {
        return "";
    }

    public void cancel() {
        if (null == httpPost) {
            return;
        }
        try {
            httpPost.cancel();
            httpPost = null;
        } catch (Exception e) {
            CustomLog.e(tag, "cancel:" + e.toString());
        }

    }

    protected T parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        return null;
    }

    protected Parser getParser() {
        return null;
    }

    protected void onSuccess(T responseContent) {
    }

    ;

    // 返回HttpErrorCode、接口业务逻辑错误码、DataErrorCode
    protected void onFail(int statusCode, String statusInfo) {
        CustomLog.e(tag, "onFail, statusCode = " + statusCode + " statusInfo = " + statusInfo + "requestUrl =" + requestUrl);
    }

    ;

    protected void onFailForAcquireAccountInfo(int statusCode, String body) {
    }

    ;

}
