package cn.redcdn.datacenter;

import org.json.JSONObject;

import cn.redcdn.log.CustomLog;
import cn.redcdn.network.httprequest.ConnectException;
import cn.redcdn.network.httprequest.PostTextHttp;

public class AbstractBusinessDataForPlayRefresher<T> {
  private String tag = getClass().getName();
  private PostTextHttp httpPost;
  private String requestUrl;

  // 正常情况返回类型为T的数据对象
  // 异常情况下抛如下异常
  // InvalidateOperateException: 无效的操作
  // ConnectException: 连接服务器失败;
  // InvalidateHttpResponse: 无效得http response,比如http response字段错误,以及http
  // response io exception
  // UnhandledHttpStatus: 比如302,404等其他status code
  
  protected T execSync(String url, final String content) {
    httpPost = new PostTextHttp();         
    
    httpPost.setURL(url);

    String responseContent = null;
    try {
      responseContent = httpPost.postUrlEncodeContentSync(content);
      CustomLog.d(tag, " ResponseContent = " + responseContent);
      Parser parser = getParser();
      if (parser != null) {
        parser.parse(responseContent);
        try {
          return parseContentBody(parser.getBody());
        } catch (InvalidateResponseException e) {
          CustomLog.e(tag, "同步接口parseContentBody异常: "+e.toString());
          return null;
        }
      }
    } catch (ConnectException e) {
        CustomLog.e(tag,"ConnectException");
        throw new ConnectException(e.getMessage());
    }
    return null;
  
  }

  // -1: 表示无效的操作，当前操作正在执行;
  // -2: 表示无效的编码;
  protected int exec(String url, final String content, String userAgent) {
    if (null != httpPost) {
      return -1;
    }
    requestUrl = url;
    httpPost = new PostTextHttp() {

      @Override
      protected void onRequestFailed(int statusCode) {
        onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
      }

      @Override
      protected void onRequestFinished(int statusCode, String responseContent) {
        CustomLog.d(tag,"onRequestFinished: "+responseContent);
        Parser parser = getParser();
        if (null == parser) {
          onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
          return;
        }
        parser.parse(responseContent);
        if (parser.isOk()) {
          Object ob = null ;
          try {
            ob  = parseContentBody(parser.getHeaderValue(), parser.getBody());
          } catch (InvalidateResponseException e) {
            onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
            return;
          } catch (ClassCastException e) {
            onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
            return;
          } catch (NumberFormatException e) {
            onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
            return;
          }
          onSuccess((T)ob);
        } else {
          onFail(parser.getHeaderValue(), parser.getHeaderText());
        }
      }
    };
    httpPost.setUserAgent(userAgent);
    httpPost.setURL(url);
    return httpPost.postUrlEncodeContent(content);
  }


  public void cancel() {
    if (null == httpPost) {
      return;
    }
    try {
        httpPost.cancel();
        httpPost = null;
    }catch(Exception e) {
    	CustomLog.e(tag,"cancel:" + e.toString());    	
    }

  }

  protected T parseContentBody(JSONObject jo)
      throws InvalidateResponseException {
    return null;
  }

  protected T parseContentBody(int headerValue, JSONObject jo)
      throws InvalidateResponseException { 
    return parseContentBody(jo);
  }

  protected Parser getParser() {
    return null;
  }

  protected void onSuccess(T responseContent) {
  };


  // 返回HttpErrorCode、接口业务逻辑错误码、DataErrorCode
  protected void onFail(int statusCode, String statusInfo) {
	  CustomLog.e(tag,"onFail, statusCode = "+ statusCode+" statusInfo = "+statusInfo+"requestUrl ="+requestUrl);
  };
}
