package cn.redcdn.datacenter;

import org.json.JSONObject;

import android.text.TextUtils;

import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;
import cn.redcdn.network.httprequest.PostTextHttp;

/**
 * AbstractBusinessDataSub比AbstractBusinessData类增加切换从服务器的过程,用于会议后台接口
 *
 * @param <T>
 * @author Administrator
 */

public class AbstractBusinessDataSub<T> {

    /**
     * masterIp  主服务器IP地址
     * slaveIp   从服务器IP地址
     * slaveUrl  从服务器后台接口请求地址
     */
    private String masterIp = ConstConfig.getMeetingManagementHttpUrlPrefix();
    private String slaveIp = ConstConfig.getSlaveMeetingManagementHttpUrlPrefix();
    private String slaveUrl = slaveIp + ConstConfig.bmsWebDomainSuffix;

    private String tag = getClass().getName();
    private PostTextHttp httpPost;

    private String requestUrl;

    /**
     * 定义master和slave服务器模式变量
     */
    private boolean bSlaveHttpStatus = false;

    // -1: 表示无效的操作，当前操作正在执行;
    // -2: 表示无效的编码;
    protected int exec(String url, final String content) {
        if (null != httpPost) {
            return -1;
        }
        requestUrl = url;
        httpPost = new PostTextHttp() {
            @Override
            protected void onRequestFailed(int statusCode) {

                if (bSlaveHttpStatus == false) {
                    if (TextUtils.isEmpty(slaveIp)) {
                        CustomLog.e(tag, "从服务器IP为空,onFail");
                        onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                    } else {
                        CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                        CustomLog.w(tag, "connect to slave server: " + content);
                        bSlaveHttpStatus = true;
                        httpPost = null;
                        exec(slaveUrl, content);
                    }
                } else {
                    onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                }

            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {

                CustomLog.d(tag, "onRequestFinished: " + responseContent);

                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }

                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
                    if (bSlaveHttpStatus) {
                        CustomLog.d(tag, "从服务器请求成功，交换ConstConfig内的主从服务器");
                        switchSlaveUrl();
                    }
                    onSuccess((T) ob);
                } else {
                    if (bSlaveHttpStatus == false) {
                        if (TextUtils.isEmpty(slaveIp)) {
                            CustomLog.e(tag, "从服务器IP为空,onFail");
                            onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                        } else {
                            CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                            CustomLog.w(tag, "connect to slave server: " + content);
                            bSlaveHttpStatus = true;
                            httpPost = null;
                            exec(slaveUrl, content);
                        }
                    } else {
                        onFail(parser.getHeaderValue(), parser.getHeaderText());
                    }
                }
            }
        };
        httpPost.setURL(url);
        return httpPost.postUrlEncodeContent(content);
    }

    protected void switchSlaveUrl() {
        CustomLog.d(tag, "尝试交换主、从服务器地址");
        if (TextUtils.isEmpty(slaveIp)) {
            CustomLog.d(
                    tag, "从服务器IP地址为空，不交换");
            return;
        }

        String oldSlave = ConstConfig.getSlaveMeetingManagementHttpUrlPrefix();
        String oldMaster = ConstConfig.getMeetingManagementHttpUrlPrefix();

        CustomLog.d(tag, "oldSlave: " + oldSlave + " | curSlave: " + slaveIp);
        CustomLog.d(tag, "oldMaster: " + oldMaster + " | curMaster: " + masterIp);

        if (!oldSlave.equals(masterIp)) {
            CustomLog.d(tag, "主、从服务器地址未被其他模块交换，正式交换地址。");
            ConstConfig.setMasterBmsWebDomain(slaveIp);
            ConstConfig.setSlaveBmsWebDomain(masterIp);
        }

        CustomLog.i(
                tag,
                "交换后，ConstConfig.getSlaveBmsWebDomain()="
                        + ConstConfig.getSlaveBmsWebDomain()
                        + ",  ConstConfig.getMasterBmsWebDomain()="
                        + ConstConfig.getMasterBmsWebDomain());
    }


    public void cancel() {
        if (null == httpPost) {
            return;
        }
        try {
            httpPost.cancel();
            httpPost = null;
        } catch (Exception e) {
            CustomLog.e(tag, "cancel:" + e.toString());
        }

    }

    protected T parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        return null;
    }

    protected T parseContentBody(int headerValue, JSONObject jo)
            throws InvalidateResponseException {
        return parseContentBody(jo);
    }

    protected Parser getParser() {
        return null;
    }

    protected void onSuccess(T responseContent) {
    }

    ;


    // 返回HttpErrorCode、接口业务逻辑错误码、DataErrorCode
    protected void onFail(int statusCode, String statusInfo) {
        CustomLog.e(tag, "onFail, statusCode = " + statusCode + " statusInfo = " + statusInfo + "requestUrl =" + requestUrl);
    }

    ;


}
