package cn.redcdn.datacenter.BaseClass;

import android.text.TextUtils;

import org.json.JSONObject;

import java.io.UnsupportedEncodingException;

import cn.redcdn.datacenter.DataErrorCode;
import cn.redcdn.datacenter.DataErrorInfo;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;
import cn.redcdn.network.httprequest.ConnectException;
import cn.redcdn.network.httprequest.PostTextHttp;

/**
 * AbstractBusinessDataBothSeq类 增加主服务器访问失败访问从服务器的过程
 * Created by gexy on 2019\10\24 0024.
 */

public class AbstractBusinessDataBothSeq<T> {

    private String tag = getClass().getName();
    private PostTextHttp httpPost;

    private String requestUrl;

    private boolean bSlaveHttpStatus = false;

    protected T execSync(String masterUrl, final String slaveUrl,final String content) {
        httpPost = new PostTextHttp();
        httpPost.setURL(masterUrl);

        if (!bSlaveHttpStatus) {
            requestUrl = masterUrl;
        } else {
            requestUrl = slaveUrl;
        }
        String responseContent = null;
        try {
            httpPost.setHttpHeader("Content-Type", "application/json");
            try {

                responseContent = httpPost.postCotentSync(content,"utf-8");
            } catch (UnsupportedEncodingException e1) {

                e1.printStackTrace();
            }
            CustomLog.d(tag, " ResponseContent = " + responseContent);
            Parser parser = getParser();
            if (parser != null) {
                if (parser.isOk()) {
                    parser.parse(responseContent);
                    try {
                        return parseContentBody(parser.getBody());
                    } catch (InvalidateResponseException e) {
                        CustomLog.e(tag, "同步接口parseContentBody异常: " + e.toString());
                        if (bSlaveHttpStatus == false) {
                            if (TextUtils.isEmpty(slaveUrl)) {
                                CustomLog.e(tag, "从服务器IP为空,onFail");
                                return null;
                            } else {
                                CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                                CustomLog.w(tag, "connect to slave server: " + content);
                                bSlaveHttpStatus = true;
                                httpPost = null;
                                execSync(masterUrl, slaveUrl, content);
                            }
                        } else {
                            return null;
                        }
                    }
                } else {
                    if (bSlaveHttpStatus == false) {
                        if (TextUtils.isEmpty(slaveUrl)) {
                            CustomLog.e(tag, "从服务器IP为空,onFail");
                            return null;
                        } else {
                            CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                            CustomLog.w(tag, "connect to slave server: " + content);
                            bSlaveHttpStatus = true;
                            httpPost = null;
                            execSync(masterUrl, slaveUrl, content);
                        }
                    } else {
                        return null;
                    }
                }
            } else {
                if (bSlaveHttpStatus == false) {
                    if (TextUtils.isEmpty(slaveUrl)) {
                        CustomLog.e(tag, "从服务器IP为空,onFail");
                        return null;
                    } else {
                        CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                        CustomLog.w(tag, "connect to slave server: " + content);
                        bSlaveHttpStatus = true;
                        httpPost = null;
                        execSync(masterUrl, slaveUrl, content);
                    }
                } else {
                    return null;
                }
            }
        } catch (ConnectException e) {
            CustomLog.e(tag,"ConnectException");
            if (bSlaveHttpStatus == false) {
                if (TextUtils.isEmpty(slaveUrl)) {
                    CustomLog.e(tag, "从服务器IP为空,onFail");
                    return null;
                } else {
                    CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                    CustomLog.w(tag, "connect to slave server: " + content);
                    bSlaveHttpStatus = true;
                    httpPost = null;
                    execSync(masterUrl,slaveUrl, content);
                }
            } else {
                return null;
            }
            throw new ConnectException(e.getMessage());
        }
        return null;

    }

    protected T execSyncEncode(String masterUrl, final String slaveUrl,final String content) {
        httpPost = new PostTextHttp();
        httpPost.setURL(masterUrl);

        if (!bSlaveHttpStatus) {
            requestUrl = masterUrl;
        } else {
            requestUrl = slaveUrl;
        }
        String responseContent = null;
        try {

            responseContent = httpPost.postUrlEncodeContentSync(content);

            CustomLog.d(tag, " ResponseContent = " + responseContent);
            Parser parser = getParser();
            if (parser != null) {
                if (parser.isOk()) {
                    parser.parse(responseContent);
                    try {
                        return parseContentBody(parser.getBody());
                    } catch (InvalidateResponseException e) {
                        CustomLog.e(tag, "同步接口parseContentBody异常: " + e.toString());
                        if (bSlaveHttpStatus == false) {
                            if (TextUtils.isEmpty(slaveUrl)) {
                                CustomLog.e(tag, "从服务器IP为空,onFail");
                                return null;
                            } else {
                                CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                                CustomLog.w(tag, "connect to slave server: " + content);
                                bSlaveHttpStatus = true;
                                httpPost = null;
                                execSyncEncode(masterUrl, slaveUrl, content);
                            }
                        } else {
                            return null;
                        }
                    }
                } else {
                    if (bSlaveHttpStatus == false) {
                        if (TextUtils.isEmpty(slaveUrl)) {
                            CustomLog.e(tag, "从服务器IP为空,onFail");
                            return null;
                        } else {
                            CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                            CustomLog.w(tag, "connect to slave server: " + content);
                            bSlaveHttpStatus = true;
                            httpPost = null;
                            execSyncEncode(masterUrl, slaveUrl, content);
                        }
                    } else {
                        return null;
                    }
                }
            } else {
                if (bSlaveHttpStatus == false) {
                    if (TextUtils.isEmpty(slaveUrl)) {
                        CustomLog.e(tag, "从服务器IP为空,onFail");
                        return null;
                    } else {
                        CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                        CustomLog.w(tag, "connect to slave server: " + content);
                        bSlaveHttpStatus = true;
                        httpPost = null;
                        execSyncEncode(masterUrl, slaveUrl, content);
                    }
                } else {
                    return null;
                }
            }
        } catch (ConnectException e) {
            CustomLog.e(tag,"ConnectException");
            if (bSlaveHttpStatus == false) {
                if (TextUtils.isEmpty(slaveUrl)) {
                    CustomLog.e(tag, "从服务器IP为空,onFail");
                    return null;
                } else {
                    CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                    CustomLog.w(tag, "connect to slave server: " + content);
                    bSlaveHttpStatus = true;
                    httpPost = null;
                    execSyncEncode(masterUrl,slaveUrl, content);
                }
            } else {
                return null;
            }
            throw new ConnectException(e.getMessage());
        }
        return null;

    }

    // -1: 表示无效的操作，当前操作正在执行;
    // -2: 表示无效的编码;
    protected int exec(final String masterUrl, final String slaveUrl, final String content) {
        if (null != httpPost) {
            return -1;
        }
        if (!bSlaveHttpStatus) {
            requestUrl = masterUrl;
        } else {
            requestUrl = slaveUrl;
        }
        httpPost = new PostTextHttp() {
            @Override
            protected void onRequestFailed(int statusCode) {

                if (bSlaveHttpStatus == false) {
                    if (TextUtils.isEmpty(slaveUrl)) {
                        CustomLog.e(tag, "从服务器IP为空,onFail");
                        onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                    } else {
                        CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                        CustomLog.w(tag, "connect to slave server: " + content);
                        bSlaveHttpStatus = true;
                        httpPost = null;
                        exec(masterUrl,slaveUrl, content);
                    }
                } else {
                    onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                }

            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {

                CustomLog.d(tag, "onRequestFinished: " + responseContent);

                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }

                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
//                    if (bSlaveHttpStatus) {
//                        CustomLog.d(tag, "从服务器请求成功，交换ConstConfig内的主从服务器");
//                        switchSlaveUrl();
//                    }
                    onSuccess((T) ob);
                } else {
                    if (bSlaveHttpStatus == false) {
                        if (TextUtils.isEmpty(slaveUrl)) {
                            CustomLog.e(tag, "从服务器IP为空,onFail");
                            onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                        } else {
                            CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                            CustomLog.w(tag, "connect to slave server: " + content);
                            bSlaveHttpStatus = true;
                            httpPost = null;
                            exec(masterUrl,slaveUrl, content);
                        }
                    } else {
                        onFail(parser.getHeaderValue(), parser.getHeaderText());
                    }
                }
            }
        };

        httpPost.setURL(requestUrl);

        try {

            return httpPost.postJsonContent(content);

        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            onFail(DataErrorCode.NOT_SUPPORT_ENCODING, DataErrorInfo.SERVICE_FAILED);
        }
        return -1;
    }

    protected int execEncode(final String masterUrl, final String slaveUrl, final String content) {
        if (null != httpPost) {
            return -1;
        }
        if (!bSlaveHttpStatus) {
            requestUrl = masterUrl;
        } else {
            requestUrl = slaveUrl;
        }
        httpPost = new PostTextHttp() {
            @Override
            protected void onRequestFailed(int statusCode) {

                if (bSlaveHttpStatus == false) {
                    if (TextUtils.isEmpty(slaveUrl)) {
                        CustomLog.e(tag, "从服务器IP为空,onFail");
                        onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                    } else {
                        CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                        CustomLog.w(tag, "connect to slave server: " + content);
                        bSlaveHttpStatus = true;
                        httpPost = null;
                        execEncode(masterUrl,slaveUrl, content);
                    }
                } else {
                    onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                }

            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {

                CustomLog.d(tag, "onRequestFinished: " + responseContent);

                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }

                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
//                    if (bSlaveHttpStatus) {
//                        CustomLog.d(tag, "从服务器请求成功，交换ConstConfig内的主从服务器");
//                        switchSlaveUrl();
//                    }
                    onSuccess((T) ob);
                } else {
                    if (bSlaveHttpStatus == false) {
                        if (TextUtils.isEmpty(slaveUrl)) {
                            CustomLog.e(tag, "从服务器IP为空,onFail");
                            onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
                        } else {
                            CustomLog.e(tag, "请求从服务器: url = " + slaveUrl);
                            CustomLog.w(tag, "connect to slave server: " + content);
                            bSlaveHttpStatus = true;
                            httpPost = null;
                            execEncode(masterUrl,slaveUrl, content);
                        }
                    } else {
                        onFail(parser.getHeaderValue(), parser.getHeaderText());
                    }
                }
            }
        };

        httpPost.setURL(requestUrl);

       return httpPost.postUrlEncodeContent(content);

    }

//    protected void switchSlaveUrl() {
//        CustomLog.d(tag, "尝试交换主、从服务器地址");
//        if (TextUtils.isEmpty(slaveIp)) {
//            CustomLog.d(
//                    tag, "从服务器IP地址为空，不交换");
//            return;
//        }
//
//        String oldSlave = ConstConfig.getSlaveMeetingManagementHttpUrlPrefix();
//        String oldMaster = ConstConfig.getMeetingManagementHttpUrlPrefix();
//
//        CustomLog.d(tag, "oldSlave: " + oldSlave + " | curSlave: " + slaveIp);
//        CustomLog.d(tag, "oldMaster: " + oldMaster + " | curMaster: " + masterIp);
//
//        if (!oldSlave.equals(masterIp)) {
//            CustomLog.d(tag, "主、从服务器地址未被其他模块交换，正式交换地址。");
//            ConstConfig.setMasterBmsWebDomain(slaveIp);
//            ConstConfig.setSlaveBmsWebDomain(masterIp);
//        }
//
//        CustomLog.i(
//                tag,
//                "交换后，ConstConfig.getSlaveBmsWebDomain()="
//                        + ConstConfig.getSlaveBmsWebDomain()
//                        + ",  ConstConfig.getMasterBmsWebDomain()="
//                        + ConstConfig.getMasterBmsWebDomain());
//    }


    public void cancel() {
        if (null == httpPost) {
            return;
        }
        try {
            httpPost.cancel();
            httpPost = null;
        } catch (Exception e) {
            CustomLog.e(tag, "cancel:" + e.toString());
        }

    }

    protected T parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        return null;
    }

    protected T parseContentBody(int headerValue, JSONObject jo)
            throws InvalidateResponseException {
        return parseContentBody(jo);
    }

    protected Parser getParser() {
        return null;
    }

    protected void onSuccess(T responseContent) {
    }


    // 返回HttpErrorCode、接口业务逻辑错误码、DataErrorCode
    protected void onFail(int statusCode, String statusInfo) {
        CustomLog.e(tag, "onFail, statusCode = " + statusCode + " statusInfo = " + statusInfo + "requestUrl =" + requestUrl);
    }
}
