package cn.redcdn.datacenter.JecCenter;

import android.text.TextUtils;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import cn.redcdn.datacenter.BaseClass.AbstractBusinessDataBothSeq;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.JecCenter.data.ContactInfo;
import cn.redcdn.datacenter.JecCenter.data.EnterpriseAllContacts;
import cn.redcdn.datacenter.JecCenter.data.Groups;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.medicalcenter.MDSAbstractBusinessData;
import cn.redcdn.datacenter.medicalcenter.MedicalParser;
import cn.redcdn.log.CustomLog;

/**
 * Created by gexy on 2019\2\15 0015.
 */

public class GetEnterpriseContactList extends AbstractBusinessDataBothSeq<EnterpriseAllContacts> {

    private String TAG = "GetEnterpriseContactList";

    public int getContactList(String token) {
        CustomLog.d(TAG, "getContactList token=" + token);
        JSONObject json = new JSONObject();
        try {
            json.put("token", token);
        } catch (Exception e) {
            // TODO: handle exception
            return -3;
        }
        if (TextUtils.isEmpty(ConstConfig.JEC_SLAVE_URL)) {
            exec(ConstConfig.getJecUrl() + ConstConfig.JEC_GET_ENTERPRISE_CONTACT, "", json.toString());
        }
        return exec(ConstConfig.getJecUrl() + ConstConfig.JEC_GET_ENTERPRISE_CONTACT, ConstConfig.getJecSlaveUrl() + ConstConfig.JEC_GET_ENTERPRISE_CONTACT, json.toString());
    }

    public int getContactListByGzip(String token) {
        CustomLog.d(TAG, "getContactList token=" + token);
        JSONObject json = new JSONObject();
        try {
            json.put("token", token);
        } catch (Exception e) {
            // TODO: handle exception
            return -3;
        }
        if (TextUtils.isEmpty(ConstConfig.JEC_SLAVE_URL)) {
            exec(ConstConfig.getJecUrl() + ConstConfig.JEC_GET_ENTERPRISE_CONTACT_BY_GZIP, "", json.toString());
        }
        return exec(ConstConfig.getJecUrl() + ConstConfig.JEC_GET_ENTERPRISE_CONTACT_BY_GZIP, ConstConfig.getJecSlaveUrl() + ConstConfig.JEC_GET_ENTERPRISE_CONTACT_BY_GZIP, json.toString());
    }

    protected EnterpriseAllContacts parseContentBody(JSONObject jo) throws InvalidateResponseException {
        EnterpriseAllContacts enterpriseAllContacts = new EnterpriseAllContacts();
        try {
            if (jo == null) {
                CustomLog.e(TAG, " getContactList jo = null");
                throw new InvalidateResponseException();
            }
            JSONObject allContactsObj = jo.optJSONObject("data");
            if (allContactsObj != null) {
                enterpriseAllContacts.limit = allContactsObj.optString("limit");
                enterpriseAllContacts.entName = allContactsObj.optString("entName");

                List<Groups> groupsAllList = new ArrayList<>();

                JSONArray groupsArr = allContactsObj.optJSONArray("allGroups");
                if (groupsArr != null) {
                    List<Groups> groupsList = parseGroups(groupsArr, true);
                    if (groupsList != null && groupsList.size() > 0) {
                        groupsAllList.addAll(groupsList);
                    }
                }

                JSONArray contactInfoArr = allContactsObj.optJSONArray("allContacts");
                if (contactInfoArr != null) {
                    List<Groups> contactList = parseGroups(contactInfoArr, false);
                    if (contactList != null && contactList.size() > 0) {
                        groupsAllList.addAll(contactList);
                    }
                }

                enterpriseAllContacts.allContacts = groupsAllList;
            }


        } catch (Exception e) {
            CustomLog.e(TAG, "getContactList invalidate reponse " + e.toString());
            throw new InvalidateResponseException();
        }

        return enterpriseAllContacts;
    }

//    protected JecParserInterface getJecParser() {
//        CustomLog.d(TAG,"get JecParser");
//        return new JecParser();
//    }

    protected Parser getParser() {
        return new MedicalParser();
    }

    private List<Groups> parseGroups(JSONArray groupArr, boolean isGroup) {
        if (groupArr == null) {
            return null;
        }
        List<Groups> groupsList = new ArrayList<>();
        try {
            if (groupArr != null) {
                for (int i = 0; i < groupArr.length(); i++) {
                    JSONObject object = groupArr.optJSONObject(i);
                    if (object != null) {
                        Groups groups = new Groups();
                        groups.id = object.optString("id");
                        groups.name = object.optString("name");
                        groups.isGroup = isGroup;

                        JSONArray contactsArr = object.optJSONArray("contacts");
                        if (contactsArr != null) {
                            List<ContactInfo> contactInfoList = new ArrayList<>();
                            for (int j = 0; j < contactsArr.length(); j++) {
                                JSONObject contactObj = contactsArr.optJSONObject(j);
                                if (contactObj != null) {
                                    ContactInfo contactInfo = new ContactInfo();
                                    contactInfo.id = contactObj.optString("id");
                                    contactInfo.name = contactObj.optString("name");
                                    contactInfo.nube = contactObj.optString("nube");
                                    contactInfo.mobile = contactObj.optString("mobile");

                                    if (!isGroup) {
                                        contactInfo.phone = contactObj.optString("phone");
                                        contactInfo.business = contactObj.optString("business");
                                        contactInfo.remarks = contactObj.optString("remarks");
                                    }

                                    contactInfoList.add(contactInfo);
                                }
                            }

                            groups.contacts = contactInfoList;
                        }

                        JSONArray group2Arr = object.optJSONArray("groups");
                        if (group2Arr != null) {
                            groups.groups = parseGroups(group2Arr, isGroup);
                        }
                        groupsList.add(groups);
                    }
                }
            }
        } catch (Exception e) {
            CustomLog.d(TAG, "Exception e:" + e.toString());
        }
        return groupsList;
    }
}
