package cn.redcdn.datacenter.JecCenter;

import android.util.Base64;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.protocol.HTTP;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URLDecoder;
import java.util.zip.GZIPInputStream;

import cn.redcdn.datacenter.MeetingManageParser;
import cn.redcdn.log.CustomLog;

/**
 * Created by gexy on 2019\2\17 0017.
 */

public class JecParser implements JecParserInterface {

    protected int rc;
    protected String rd;
    protected JSONObject body;

    @Override
    public int getHeaderValue() {
        return rc;
    }

    @Override
    public String getHeaderText() {
        return rd;
    }

    @Override
    public JSONObject getBody() {
        return body;
    }

    @Override
    public boolean isOk() {
        return rc==0;
    }

    @Override
    public void parse(HttpResponse response) {
        try {
            String jsonString = parseGzip(response.getEntity());
            CustomLog.d("JecParser", "jsonString=" + jsonString);
            JSONObject joContent = new JSONObject(jsonString);
            rc = joContent.getInt("code");
            rd = joContent.getString("msg");
            body = joContent;
        } catch (Exception e) {
            CustomLog.d("JecParser", "e:" + e.toString());
            rc = -1;
            rd = "数据异常,请重试!";
        }
    }

    private String parseGzip(HttpEntity entity) throws Exception {
//        InputStream is = new ByteArrayInputStream(str.getBytes());
        InputStream in = entity.getContent();
        GZIPInputStream gzipInputStream = new GZIPInputStream(in);
        BufferedReader reader = new BufferedReader(new InputStreamReader(
                gzipInputStream, HTTP.UTF_8));
        String line = null;
        StringBuffer sb = new StringBuffer();
        while ((line = reader.readLine()) != null) {
            sb.append(line).append("\n");
        }
        return sb.toString();
    }

    public static String decompress(String zipText) throws Exception {
//        byte[] zipBytes = zipText.getBytes();
        byte[] compressed = URLDecoder.decode(zipText).getBytes();
        if (compressed.length > 4)
        {
            GZIPInputStream gzipInputStream = new GZIPInputStream(
                    new ByteArrayInputStream(compressed, 4,
                            compressed.length - 4));

            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            for (int value = 0; value != -1;) {
                value = gzipInputStream.read();
                if (value != -1) {
                    baos.write(value);
                }
            }
            gzipInputStream.close();
            baos.close();
            String sReturn = new String(baos.toByteArray(), "UTF-8");
            return sReturn;
        }
        else
        {
            return "";
        }
    }

    public static String uncompressToString(byte[] bytes) {
        if (bytes == null || bytes.length == 0) {
            return null;
        }
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ByteArrayInputStream in = new ByteArrayInputStream(bytes);
        try {
            GZIPInputStream ungzip = new GZIPInputStream(in);
            byte[] buffer = new byte[256];
            int n;
            while ((n = ungzip.read(buffer)) >= 0) {
                out.write(buffer, 0, n);
            }
            return out.toString("UTF-8");
        } catch (IOException e) {
            CustomLog.e("gzip compress error.",e.getMessage());
        }
        return null;
    }
}
