package cn.redcdn.datacenter.cantacts;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.cantacts.data.ContanctsInfo;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;

public abstract class DownloadContactsData extends
		AbstractBusinessData<List<ContanctsInfo>> {
	private String tag = DownloadContactsData.class.getName();

	/**
	 * 获取联系人数据接口。服务器将返回联系人信息。 对应接口 《个人通讯录接口-批量操作》 4.2
	 * 
	 * @param timestamp
	 *            时间戳
	 * @param startLineNo
	 *            开始条目的行号，从1开始
	 * @param maxNum
	 *            一次可更新的最大条目数
	 * @param hasDelete
	 *            是否包含逻辑删除数据"，定义"0"标识不包含逻辑删除数据，"1"标识为包含逻辑删除数据
	 * @param accessToken
	 *            Token
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
	 */

	public int downloadContactsData(long timestamp, int startLineNo, int maxNum,
			int hasDelete, String accessToken) {
		CustomLog.i(tag, "downloadContactsData");

		JSONObject json = new JSONObject();
		try {
			json.put(ConstConfig.TIMESTAMP, timestamp);
			json.put(ConstConfig.STARTLINENO, startLineNo);
			json.put(ConstConfig.MAXNUM, maxNum);
			json.put(ConstConfig.HASDELETE, hasDelete);
		} catch (JSONException e) {
			return -3;
		}
		//转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
		return exec(
				ConstConfig.getDownloadContactsData(),
				ConstConfig.PARAM_USERCNTER_DOWNLOADCONTACTDATA
						+ jsonString + "&accessToken=" + accessToken);
	}

	/***
	 * 下载联系人同步接口
	 */
	public List<ContanctsInfo> downloadContactsDataSync(long timestamp, int startLineNo, int maxNum,
      int hasDelete, String accessToken) {
    CustomLog.i(tag, "downloadContactsData");

    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.TIMESTAMP, timestamp);
      json.put(ConstConfig.STARTLINENO, startLineNo);
      json.put(ConstConfig.MAXNUM, maxNum);
      json.put(ConstConfig.HASDELETE, hasDelete);
    } catch (JSONException e) {
      return null;
    }
  //转义特殊字符   % --> %25
    String jsonString = java.net.URLEncoder.encode(json.toString());
    return execSync(
        ConstConfig.getDownloadContactsData(),
        ConstConfig.PARAM_USERCNTER_DOWNLOADCONTACTDATA
            + jsonString + "&accessToken=" + accessToken);
  }
	protected List<ContanctsInfo> parseContentBody(JSONObject jo)
			throws InvalidateResponseException {
		try {
			if (jo == null) {
				CustomLog.e(tag, "downloadContactsData invalidate reponse");
				throw new InvalidateResponseException();
			}
			List<ContanctsInfo> mylist = new ArrayList<ContanctsInfo>();
		  CustomLog.d("DownLoadCantactsInfo：", jo.toString());
		
			ContanctsInfo coninfo = null;
			JSONArray jary = null;
			jary = (JSONArray) jo.get("contacts");
			  if(jary.length() > 0){
			    CustomLog.d("有这么多联系人","有"+jary.length()+"个联系人");
	         for (int j = 0; j < jary.length(); j++) {
	           CustomLog.d("第",j+"个联系人");
	          
	            JSONObject contactObj = jary.getJSONObject(j);
	            CustomLog.d("内容为：",contactObj.toString());
	            coninfo = new ContanctsInfo();
	            
	            coninfo.contactId = contactObj.getString("contactId");
	            
	            if(contactObj.has("isDeleted")){
                coninfo.isDeleted = contactObj.getInt("isDeleted");
            //    CustomLog.d("有isDeleted字段","有isDeleted字段");
              }else{
                coninfo.isDeleted = 0;
                CustomLog.d("没有isDeleted字段","没有isDeleted字段");
              }
	            
	            if(contactObj.has("name")){
	              coninfo.name = contactObj.getString("name");
	          //    CustomLog.d("有name字段","有name字段");
	            }else{
	              coninfo.name = "";
	              CustomLog.d("没有name字段","没有name字段");
	            }
	            
	            if(contactObj.has("nickName")){
	           //   CustomLog.d("有nickName字段","有nickName字段");
                coninfo.nickName = contactObj.getString("nickName");
              }else {
                coninfo.name = "";
                CustomLog.d("没有nickName字段","没有nickName字段");
              }
	            
	            coninfo.clientLineNumber = contactObj.getString("clientLineNumber");
	            coninfo.timestamp = contactObj.getLong("timestamp");
	            
	           if( contactObj.has("extendProperties")){
	             boolean hasUserFrom = false;
	             boolean hasServiceType= false;
	             boolean hasAppType= false;
	             boolean hasAccountType= false;
	             boolean hasworkUnitType= false;
	             boolean hasworkUnit= false;
	             boolean hasdepartment= false;
	             boolean hasprofessional= false;
	             boolean hasofficeTel= false;
	             boolean hassaveToContactsTime= false;
	             boolean hasuserType = false;
	             boolean hasmodel = false;
	            
	            
	             
	              JSONArray extendJarry = new JSONArray(contactObj.getString("extendProperties"));
	              for(int tmp= 0;tmp<extendJarry.length();tmp++){
	                  CustomLog.d("有属性", ((JSONObject)extendJarry.get(tmp)).keys().next().toString()+" 属性");
	                  if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("UserFrom")){
	                    hasUserFrom = true;
	                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("UserFrom");
	                    try {
	                    	coninfo.UserFrom = stJary.getInt(0);
	                    } catch(Exception ex) {
	                    	CustomLog.d(tag, "coninfo.UserFrom error: " + ex.getMessage());
	                    	hasUserFrom = false;
	                    }
	                    
	                  }else if (((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("ServiceType")){
	                    hasServiceType = true;
	                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("ServiceType");
	                    try {
	                    	coninfo.ServiceType = stJary.getInt(0);
	                    } catch(Exception ex) {
	                    	CustomLog.d(tag, "coninfo.ServiceType error: " + ex.getMessage());
	                    	hasServiceType = false;
	                    }
                      
	                  } else if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("AppType")){
	                    hasAppType = true;
	                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("AppType");
	                    try {
	                    	coninfo.AppType = stJary.getString(0);
	                    } catch(Exception ex) {
	                    	CustomLog.d(tag, "coninfo.AppType error: " + ex.getMessage());
	                    	hasAppType = false;
	                    }
                    }
	                  else if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("accountType")){
		                    hasAccountType = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("accountType");
		                    try {
		                    	coninfo.accountType = stJary.getInt(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.accountType error: " + ex.getMessage());
		                    	hasAccountType = false;
		                    }
	                    } 
	                  else if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("workUnitType")){
		                    hasworkUnitType = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("workUnitType");
		                    try {
		                    	coninfo.workUnitType = stJary.getInt(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.workUnitType error: " + ex.getMessage());
		                    	hasworkUnitType = false;
		                    }
	                    } 
	                  else if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("workUnit")){
		                    hasworkUnit = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("workUnit");
		                    try {
		                    	coninfo.workUnit  = stJary.getString(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.workUnit  error: " + ex.getMessage());
		                    	hasworkUnit = false;
		                    }
	                    } 
	                  else if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("department")){
		                    hasdepartment = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("department");
		                    try {
		                    	coninfo.department = stJary.getString(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.department error: " + ex.getMessage());
		                    	hasdepartment = false;
		                    }
	                    } 
	                  else if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("professional")){
	                	  hasprofessional = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("professional");
		                    try {
		                    	coninfo.professional = stJary.getString(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.professional error: " + ex.getMessage());
		                    	hasprofessional = false;
		                    }
	                    } 
	                  else if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("officeTel")){
	                	  hasofficeTel = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("officeTel");
		                    try {
		                    	coninfo.officeTel = stJary.getString(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.officeTel error: " + ex.getMessage());
		                    	hasofficeTel = false;
		                    }
	                    }  else if(((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("saveToContactsTime")){
	                    	hassaveToContactsTime = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("saveToContactsTime");
		                    try {
		                    	coninfo.saveToContactsTime = stJary.getString(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.saveToContactsTime error: " + ex.getMessage());
		                    	hassaveToContactsTime = false;
		                    }
	                    } else if (((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("userType")){
	                    	hasuserType = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("userType");
		                    try {
		                    	coninfo.userType = stJary.getInt(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.userType error: " + ex.getMessage());
		                    	hasuserType = false;
		                    }
						}else if (((JSONObject)extendJarry.get(tmp)).keys().next().toString().equals("model")){
	                    	hasmodel = true;
		                    JSONArray stJary = ((JSONObject)extendJarry.get(tmp)).getJSONArray("model");
		                    try {
		                    	coninfo.model = stJary.getString(0);
		                    } catch(Exception ex) {
		                    	CustomLog.e(tag, "coninfo.model error: " + ex.getMessage());
		                    	hasmodel = false;
		                    }
						}
	              }
	              if(hasUserFrom == false){
	                coninfo.UserFrom = 0;
	              }
	              if(hasServiceType == false){
                  coninfo.ServiceType = 1;
                }
	              if(hasAppType == false){
                  coninfo.AppType = "";
                }
	              if (hasAccountType == false) {
					coninfo.accountType = 0;
				}
	              if (hasworkUnitType == false) {
						coninfo.workUnitType = 0;
					}
	              if (hasworkUnit == false) {
						coninfo.workUnit = "";
					}
	              if (hasdepartment == false) {
						coninfo.department = "";
					}
	              if (hasprofessional == false) {
						coninfo.professional = "";
					}
	              if (hasofficeTel == false) {
						coninfo.officeTel = "";
					}
	              if (hassaveToContactsTime == false) {
						coninfo.saveToContactsTime = "";
					}
	              if (hasuserType == false) {
					coninfo.userType = 0;
				}
	              if (hasmodel == false) {
	            	  coninfo.model = "";
					
				}
	             
	              
	              
//	              if(((JSONObject)extendJarry.get(0)).has("ServiceType")){
//	                JSONArray stJary = ((JSONObject)extendJarry.get(0)).getJSONArray("ServiceType");
//	                coninfo.ServiceType = stJary.getInt(0);
//	              }else{
//	                CustomLog.e("没有ServiceType字段","没有ServiceType字段");
//	                coninfo.ServiceType = 1;
//	              }                                        
//	              if(((JSONObject)extendJarry.get(1)).has("AppType")){
//	                JSONArray atJary = ((JSONObject)extendJarry.get(1)).getJSONArray("AppType");
//	                coninfo.AppType = atJary.getString(0);
//	              }else{
//	                CustomLog.e("没有AppType字段","没有AppType字段");
//	                coninfo.AppType = "";
//	              }
//	              if(((JSONObject)extendJarry.get(2)).has("UserFrom")){
//	                JSONArray ufJary = ((JSONObject)extendJarry.get(2)).getJSONArray("UserFrom");
//	                coninfo.UserFrom = ufJary.getInt(0);
//	              }else{
//	                CustomLog.e("没有UserFrom字段","没有UserFrom字段");
//	                coninfo.UserFrom = 0;
//	              }
	           } else{
	             CustomLog.d("没有extendProperties字段","没有extendProperties字段");
	             coninfo.ServiceType = 1;
	             coninfo.AppType = "";
	             coninfo.UserFrom = 0;
	           }
	           
	           if( contactObj.has("nubeInfos")){
	          //   CustomLog.d("有nubeInfos字段","有nubeInfos字段");
	             JSONArray nubeJarry = new JSONArray(contactObj.getString("nubeInfos"));
	             JSONObject nubeinfoobj = new JSONObject(nubeJarry.get(0).toString());
	             if(nubeinfoobj.has("contactUserId")){
	               coninfo.contactUserId = nubeinfoobj.getString("contactUserId");
	             }else{
	               CustomLog.d("没有contactUserId字段","没有contactUserId字段");
	               coninfo.contactUserId = "";
	             }
	            if(nubeinfoobj.has("nubeNumber")){
	              coninfo.nubeNumber = nubeinfoobj.getString("nubeNumber");
	            }else{
	         //     CustomLog.e("没有nubeNumber字段","没有nubeNumber字段");
	              coninfo.nubeNumber = "";
	            }
	           }else {
	             CustomLog.d("没有nubeInfos字段","没有nubeInfos字段");
	             coninfo.contactUserId = "";
	             coninfo.nubeNumber = "";
	           }
	           
	           if( contactObj.has("phones")){
	       //      CustomLog.d("有phones字段","有phones字段");
	             JSONArray phoneJarry =  new JSONArray(contactObj.getString("phones"));
	             if(phoneJarry.getJSONObject(0).has("移动电话")){
	               JSONArray yddhJsrray =  new JSONArray(phoneJarry.getJSONObject(0).getString("移动电话"));
	               JSONObject tmpobj =  new JSONObject(yddhJsrray.getString(0));
	               if(tmpobj.has("contactUserId")){
	                   coninfo.contactUserId = tmpobj.getString("contactUserId");
	               }else{
	                 coninfo.contactUserId ="";
	               }
	               if(tmpobj.has("number")){
	                 coninfo.number = tmpobj.getString("number");
	               }else{
	                 coninfo.number ="";
	               }
	               if(tmpobj.has("trueNumber")){
	                 coninfo.trueNumber = tmpobj.getString("trueNumber");
	               }else{
	                 coninfo.trueNumber = "";
	               }
	             }else{
	               CustomLog.d("没有移动电话","没有移动电话字段");
	               coninfo.contactUserId ="";
	               coninfo.number ="";
	               coninfo.trueNumber = "";
	             }
	             
	           }  else{
	             CustomLog.d("没有phones","没有phones字段");
	             coninfo.contactUserId ="";
	             coninfo.number ="";
	             coninfo.trueNumber = "";
	           }
	           
	           if( contactObj.has("emails")){
	    	       //      CustomLog.d("有phones字段","有phones字段");
	    	             JSONArray emailJarry =  new JSONArray(contactObj.getString("emails"));
	    	             if(emailJarry.getJSONObject(0).has("工作")){
	    	               JSONArray yddhJsrray =  new JSONArray(emailJarry.getJSONObject(0).getString("工作"));
	    	               JSONObject tmpobj =  new JSONObject(yddhJsrray.getString(0));
	    	               if(tmpobj.has("contactUserId")){
	    	                   coninfo.contactUserId = tmpobj.getString("contactUserId");
	    	               }else{
	    	                 coninfo.contactUserId ="";
	    	               }
	    	               if(tmpobj.has("email")){
	    	                 coninfo.email = tmpobj.getString("email");
	    	               }else{
	    	                 coninfo.email ="";
	    	               }
	    	             }else{
	    	               CustomLog.d("没有工作","没有工作字段");
	    	               coninfo.contactUserId ="";
	    	               coninfo.email = "";
	    	             }
	    	             
	    	           }  else{
	    	             CustomLog.d("没有emails","没有emails字段");
	    	             coninfo.contactUserId ="";
	    	             coninfo.email = "";
	    	           }
	           
	           
	           
	           if( contactObj.has("urls")){
	         //    CustomLog.d("有urls字段","有urls字段");
	             JSONArray urlJarry =  new JSONArray(contactObj.getString("urls"));
	             JSONObject urlobj = new JSONObject(urlJarry.getString(0));
	             if(urlobj.has("headUrl")){
	         //      CustomLog.d("有headUrl字段","有headUrl字段");
	               JSONArray headarray = new JSONArray(urlobj.getString("headUrl"));
	               coninfo.urls = headarray.get(0).toString();
	             }else{
	               CustomLog.d("没有headUrl字段","没有headUrl字段");
	               coninfo.urls = "";
	             }
	           }  else {
	             CustomLog.d("没有urls字段","没有urls字段");
	             coninfo.urls = "";
	           }
	           
	            mylist.add(coninfo);
	          }
	      }
			
			return mylist;
		} catch (JSONException e) {
			CustomLog.e(tag, "DownLoadContacts  invalidate reponse"+e);
			
			throw new InvalidateResponseException();
		}
	}

	protected Parser getParser() {
		return new DownlaodContactsParser();
	}

	boolean objExist(String str, JSONObject contactObj) {
		return contactObj.has(str);// 判断是否存在该属性，若存在，则返回Value
	}

}
