package cn.redcdn.datacenter.enterprisecenter;

import org.json.JSONException;
import org.json.JSONObject;

import android.sax.TextElementListener;
import android.text.TextUtils;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.ExecResultCode;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.enterprisecenter.data.AccountInfo;
import cn.redcdn.log.CustomLog;

public abstract class EnterpriseMemberLogin extends AbstractBusinessData<AccountInfo> {
    private String tag = EnterpriseMemberLogin.class.getName();

    /**
     * @param imei       串号
     * @param deviceType 登录设备类型 详见cn.redcdn.datacenter.enterprisecenter.data.DeviceType
     * @return -1 表示无效的操作，当前操作正在执行
     * -2 表示无效的编码
     * -3 表示json操作异常
     */
    public int login(String imei, String deviceType) {
        CustomLog.i(tag, "enterprise login, imei:" + imei + " deviceType: " + deviceType);

        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.PARAM_IMEI, imei);
            json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
        } catch (JSONException e) {
            return -3;
        }

        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        return exec(ConstConfig.getAcquireAccountInfo(), ConstConfig.PARAM_ENTMEMBERLOGIN + jsonString);
    }

    public int login(String token) {
        CustomLog.i(tag, "enterprise login, token:" + token);

        if (TextUtils.isEmpty(token)) {
            CustomLog.e(tag, "token 不能为空");
            return ExecResultCode.PARAM_ERROR;
        }

        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.PARAM_TOKEN, token);
        } catch (JSONException e) {
            return -3;
        }

        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        return exec(ConstConfig.getAcquireAccountInfo(), ConstConfig.PARAM_ENTMEMBERLOGIN + jsonString);

    }

    public int otherLogin(String appKey, String uid, String nubeNumber, String deviceType) {
        CustomLog.i(tag, "enterprise otherLogin, appKey:" + appKey + " uid:" + uid
                + " nubeNumber:" + nubeNumber + " deviceType:" + deviceType);

//        if (TextUtils.isEmpty(token)) {
//            CustomLog.e(tag, "token 不能为空");
//            return ExecResultCode.PARAM_ERROR;
//        }

        JSONObject json = new JSONObject();
        try {
            json.put("appKey", appKey);
            json.put("uid", uid);
            json.put("nubeNumber", nubeNumber);
            json.put("deviceType", deviceType);
        } catch (JSONException e) {
            return -3;
        }

        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        return exec(ConstConfig.getAcquireAccountInfo(), ConstConfig.PARAM_OTHERENTMEMBERLOGIN + json.toString());
//        return exec(ConstConfig.getAcquireAccountInfo() + "?" + ConstConfig.PARAM_OTHERENTMEMBERLOGIN + json.toString(), json.toString());
    }

    protected AccountInfo parseContentBody(JSONObject jo) throws InvalidateResponseException {
        try {
            if (jo == null) {
                CustomLog.e(tag, "enterprise login invalidate reponse");
                throw new InvalidateResponseException();
            }

            JSONObject userInfo = jo.getJSONObject("userInfo");
            AccountInfo info = new AccountInfo();
            info.nickName = userInfo.optString(ConstConfig.NICK_NAME);
            info.accesstoken = userInfo.optString(ConstConfig.ACCESS_TOKEN);
            info.useEndTime = userInfo.optString(ConstConfig.USE_ENDTIME);
            info.nubeNumber = userInfo.optString(ConstConfig.NUBENUMBER);

            info.deviceType = userInfo.optString("deviceType");
            info.tradeIds = userInfo.optString("tradeIds");
            info.paidType = userInfo.optString("paidType");

            info.isEnableMeeting = userInfo.optInt("isEnableMeeting");
            info.isEnableQueue = userInfo.optInt("isEnableQueue");
            info.isEnableCharMessage = userInfo.optInt("isEnableCharMessage");
            info.isEnableShortVideo = userInfo.optInt("isEnableShortVideo");
            info.isEnablePicture = userInfo.optInt("isEnablePicture");
            info.isEnableCard = userInfo.optInt("isEnableCard");

            if (TextUtils.isEmpty(info.accesstoken) || TextUtils.isEmpty(info.useEndTime)) {
                CustomLog.e(tag, "enterprise login interface no token or useEndTime");
//                throw new InvalidateResponseException();
            }

            return info;
        } catch (JSONException e) {
            CustomLog.e(tag, "enterprise login invalidate reponse");
            throw new InvalidateResponseException();
        }
    }

    protected Parser getParser() {
        return new EnterpriseCenterParser();
    }
}
