package cn.redcdn.datacenter.enterprisecenter;

import org.json.JSONException;
import org.json.JSONObject;

import android.annotation.SuppressLint;
import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.enterprisecenter.data.AccountInfo;
import cn.redcdn.log.CustomLog;

public abstract class Login4Mobile extends AbstractBusinessData<AccountInfo> {
	private String tag = Login4Mobile.class.getName();

	/**
	 * 极会议登录接口。登录成功会返回成员详细信息（token、视讯号、参会名称、头像地址。。。）。 对应 《企业中心接口文档》1.4.9
	 * 
	 * @param account
	 *            登录帐号
	 * @param password
	 *            登录密码
	 * @param deviceType 
	 *            登录设备类型 详见cn.redcdn.datacenter.enterprisecenter.data.DeviceType       
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
	 */

	public int login(String account, String password , String deviceType) {
		CustomLog.i(tag, "Login4Mobile , account = " + account + "password = "
				+ password +" deviceType = "+ deviceType);

		JSONObject json = new JSONObject();
		try {
			json.put("account",account);
			json.put("password",password);
			json.put("deviceType",deviceType);
		} catch (JSONException e) {
			return -3;
		}

		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
		return exec(ConstConfig.getLogin4Mobile(),
				ConstConfig.PARAM_LOGIN4MOBILE + jsonString);
	}

	@SuppressLint("NewApi")
	protected AccountInfo parseContentBody(JSONObject jo)
			throws InvalidateResponseException {
		try {
			if (jo == null) {
				CustomLog.e(tag, "enterprise login invalidate reponse");
				throw new InvalidateResponseException();
			}

			JSONObject userInfo = jo.getJSONObject("userInfo");
			AccountInfo info = new AccountInfo();
			info.nickName = userInfo.optString(ConstConfig.NICK_NAME);
			info.accesstoken = userInfo.optString(ConstConfig.ACCESS_TOKEN);
			info.useEndTime = userInfo.optString(ConstConfig.USE_ENDTIME);
			info.nubeNumber = userInfo.optString(ConstConfig.NUBENUMBER);
			info.serviceType = userInfo.optInt(ConstConfig.SERVICETYPE);
			info.useStartTime = userInfo.optString(ConstConfig.USESTARTTIME);
			info.headUrl = userInfo.optString(ConstConfig.HEADURL);
			
			info.deviceType = userInfo.optString("deviceType");
	    info.isEnableMeeting = userInfo.optInt("isEnableMeeting");
	    info.isEnableQueue = userInfo.optInt("isEnableQueue");
	    info.isEnableCharMessage = userInfo.optInt("isEnableCharMessage");
	    info.isEnableShortVideo = userInfo.optInt("isEnableShortVideo");
	    info.isEnablePicture = userInfo.optInt("isEnablePicture");
	    info.isEnableCard = userInfo.optInt("isEnableCard");
	      
			if(userInfo.has("mobile")){
			  info.mobile = userInfo.optString(ConstConfig.MOBILE);
			}
			if (info.accesstoken.isEmpty()) {
				CustomLog.e(tag,
						"enterprise login interface no token");
				throw new InvalidateResponseException();
			}

			return info;
		} catch (JSONException e) {
			CustomLog.e(tag, "enterprise login invalidate reponse");
			throw new InvalidateResponseException();
		}
	}

	protected Parser getParser() {
		return new EnterpriseCenterParser();
	}
}
