package cn.redcdn.datacenter.enterprisecenter;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.enterprisecenter.data.ProductId;
import cn.redcdn.datacenter.enterprisecenter.data.Users;
import cn.redcdn.log.CustomLog;

public class SearchAccount extends AbstractBusinessData<List<Users>> {
	private String tag = SearchAccount.class.getName();

	 /**
   * 查询账号接口。查询成功会返回用户详细信息列表 对应 《企业中心接口文档》1.4.7
   * 
   * @param accounts
   *            视讯号列表
   * 
   * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常 ； -4： 输入为空
   */
  public int searchAccount(String[] accounts) {
    CustomLog.i(tag, "SearchAccount");
    if(accounts == null){
      CustomLog.e(tag, "accounts为空");
      return -4;
    }
    
    try {
      JSONObject json = new JSONObject();
      JSONArray ja = new JSONArray();
      for (int j = 0; j < accounts.length; j++) {
    	  if (ConstConfig.getNeedtoaddAppkey()) {
			ja.put(ConstConfig.getAccount(accounts[j], ConstConfig.getAppkey()));
		}else{
        ja.put(accounts[j]);
		}
      }
      json.put("accounts", ja);
      json.put(ConstConfig.PRODUCTID, ProductId.ALL);
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getSearchAccount(),
          ConstConfig.PARAM_SEARCHACCOUNT + jsonString);
    } catch (Exception e) {
      CustomLog.e(tag, e.getMessage());
      return -3;
    }
  }

  public List<Users> searchAccountSync(String[] accounts) {
    CustomLog.i(tag, "SearchAccountSync");

    if(accounts == null){
      CustomLog.e(tag, "accounts为空");
      return null;
    }

    String joNumbers = null;
    try {
      JSONObject json = new JSONObject();
      JSONArray ja = new JSONArray();
      for (int j = 0; j < accounts.length; j++) {
        ja.put(accounts[j]);
      }
      json.put("accounts", ja);
      json.put(ConstConfig.PRODUCTID, ProductId.ALL);
      joNumbers = json.toString();

    } catch (Exception e) {
      CustomLog.e(tag, e.getMessage());
      return null;
    }

	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(joNumbers);
    return execSync(ConstConfig.getSearchAccount(),
        ConstConfig.PARAM_SEARCHACCOUNT + jsonString);
  }
  
	/**
	 * 
	 * @param accounts 视讯号或者电话号码列表
	 * 
	 * @param productId 产品标示 详见 cn.redcdn.datacenter.enterprisecenter.data.ProductId
	 * 
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常 ； -4： 输入为空
	 */
	public int searchAccount(String[] accounts,String productId) {
		CustomLog.i(tag, "SearchAccount,productId = "+ productId);
		if(accounts == null){
		  CustomLog.e(tag, "accounts为空");
		  return -4;
		}
		
		try {
			JSONObject json = new JSONObject();
			JSONArray ja = new JSONArray();
			for (int j = 0; j < accounts.length; j++) {
				ja.put(accounts[j]);
			}
			json.put(ConstConfig.ACCOUNTS, ja);
			json.put(ConstConfig.PRODUCTID, productId);
			  //转义特殊字符   % --> %25
			String jsonString = java.net.URLEncoder.encode(json.toString());
			return exec(ConstConfig.getSearchAccount(),
					ConstConfig.PARAM_SEARCHACCOUNT + jsonString);
		} catch (Exception e) {
			CustomLog.e(tag, e.getMessage());
			return -3;
		}
	}

	public List<Users> searchAccountSync(String[] accounts,String productId) {
    CustomLog.i(tag, "SearchAccountSync");

    if(accounts == null){
      CustomLog.e(tag, "accounts为空");
      return null;
    }

    String joNumbers = null;
    try {
      JSONObject json = new JSONObject();
      JSONArray ja = new JSONArray();
      for (int j = 0; j < accounts.length; j++) {
        ja.put(accounts[j]);
      }
      json.put(ConstConfig.ACCOUNTS, ja);
      json.put(ConstConfig.PRODUCTID, productId);
      joNumbers = json.toString();

    } catch (Exception e) {
      CustomLog.e(tag, e.getMessage());
      return null;
    }
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(joNumbers);
    return execSync(ConstConfig.getSearchAccount(),
        ConstConfig.PARAM_SEARCHACCOUNT + jsonString);
	}

	protected List<Users> parseContentBody(JSONObject jo)
			throws InvalidateResponseException {
		try {

			if (jo == null) {
				CustomLog.e(tag, "SearchAccount JSONObject == null");
				throw new InvalidateResponseException();
			}
			List<Users> userList = new ArrayList<Users>();
			CustomLog.d(tag,"parseContentBody:"+jo.toString());
			Users us = null;

			JSONArray jary = (JSONArray) jo.get("users");
			if(jary.length()>0){
			   for (int j = 0; j < jary.length(); j++) {
				JSONObject userObj = jary.getJSONObject(j);
				us = new Users();
				if(objExist("name",userObj)){
				  us.name = userObj.optString("name");
				}
			  if(objExist("nickName",userObj)){
          us.nickName = userObj.optString("nickName");
        }
			  if(objExist("mobile",userObj)){
          us.mobile = userObj.optString("mobile");
        }
			  if(objExist("serviceType",userObj)){
			    if(userObj.optString("serviceType").toString().equals("0")){
			      us.serviceType =  0;
			    }
			    else{
			      us.serviceType = 1;
			    }
        }
			  if(objExist("appType",userObj)){
          us.appType = userObj.optString("appType");
        }
			  if(objExist("nubeNumber",userObj)){
          us.nubeNumber = userObj.optString("nubeNumber");
        }
			  if(objExist("uid",userObj)){
          us.uid = userObj.optString("uid");
        }
		     if(objExist("headUrl",userObj)){
	          us.headUrl = userObj.optString("headUrl");
	        }
		     
		     if(objExist("imei",userObj)){
           us.imei = userObj.optString("imei");
         }else us.imei = "";
		     
		     us.useStartTime = userObj.optString("useStartTime");
		     us.useEndTime = userObj.optString("useEndTime");
		     
		     us.deviceType = userObj.optString("deviceType");
         us.isEnableMeeting = userObj.optInt("isEnableMeeting");
         us.isEnableQueue = userObj.optInt("isEnableQueue");
         us.isEnableCharMessage = userObj.optInt("isEnableCharMessage");
         us.isEnableShortVideo = userObj.optInt("isEnableShortVideo");
         us.isEnablePicture = userObj.optInt("isEnablePicture");
         us.isEnableCard = userObj.optInt("isEnableCard");
         
				userList.add(us);
			}
			 }else{
			   CustomLog.e(tag,"未查到该视讯号");
			 }
			 CustomLog.d(tag,"userList.size = "+userList.size());
			return userList;
		} catch (JSONException e) {
			CustomLog.e(tag, "SearchAccount invalidate reponse");
			throw new InvalidateResponseException();
		}
	}

	protected Parser getParser() {
		return new EnterpriseCenterParser();
	}
	
  boolean objExist(String str, JSONObject contactObj) {
    return contactObj.has(str);// 判断是否存在该属性
  }
}
