package cn.redcdn.datacenter.hpucenter.triage;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.net.URLDecoder;
import java.net.URLEncoder;

import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.hpucenter.HpuParser;
import cn.redcdn.datacenter.hpucenter.data.CSLRoomDetailInfo;
import cn.redcdn.datacenter.hpucenter.data.EstimatesInfo;
import cn.redcdn.datacenter.hpucenter.data.ResponeseInfo;
import cn.redcdn.datacenter.hpucenter.data.TriageDoctorInfo;
import cn.redcdn.datacenter.hpucenter.data.TriageExpectResponseInfo;
import cn.redcdn.datacenter.medicalcenter.MDSAbstractBusinessData;
import cn.redcdn.log.CustomLog;

/**
 * Created by caizx on 2018/5/4.
 */

public class HPUGetTriageInfo extends MDSAbstractBusinessData<CSLRoomDetailInfo> {

    String TAG = HPUGetTriageInfo.class.getName();

    /**
     * 获取分诊诊疗室详情
     *
     * @param token
     * @param id    诊疗单id
     */
    public void getInfo(String token, String id) {
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("token", token);
            jsonObject.put("id", id);
        } catch (JSONException e) {
            CustomLog.e(TAG, e.getMessage());
        }
        exec(ConstConfig.getMedicalCenterUrl() + ConstConfig.HPU_TRIAGE_INFO, jsonObject.toString());
    }

    /**
     * 获取分诊诊疗室详情
     *
     * @param token
     * @param id    诊疗单id
     */
    public void getInfo(String token, String id, String versionNo) {
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("token", token);
            jsonObject.put("id", id);
            jsonObject.put("versionNo", versionNo);
        } catch (JSONException e) {
            CustomLog.e(TAG, e.getMessage());
        }
        exec(ConstConfig.getMedicalCenterUrl() + ConstConfig.HPU_TRIAGE_INFO, jsonObject.toString());
    }

    protected CSLRoomDetailInfo parseContentBody(JSONObject jo) {

        JSONObject json = jo.optJSONObject("data");
        CSLRoomDetailInfo info = new CSLRoomDetailInfo();
        if (json != null) {

            info.patientId = json.optString("patientId");
            info.guardId = json.optString("guardId");
            info.state = json.optString("state");
            info.curNum = json.optString("curNum");
            info.patientName = json.optString("patientName");
            info.patientCard = json.optString("patientCard");
            info.cardType = json.optString("cardType");
            ;
            info.patientMobile = json.optString("patientMobile");
            info.patientAge = json.optString("patientAge");
            info.patientSex = json.optString("patientSex");
            info.height = json.optString("height");
            info.weight = json.optString("weight");
            info.chief = URLDecoder.decode(json.optString("chief"));
            info.physical = URLDecoder.decode(json.optString("physical"));
            info.assCheckUrl = json.optString("assCheckUrl");
            info.problem = URLDecoder.decode(json.optString("problem"));
            info.guardCard = json.optString("guardCard");
            info.guardId = json.optString("guardId");
            info.guardMobile = json.optString("guardMobile");
            info.guardSex = json.optString("guardSex");
            info.requestName = json.optString("requestName");
            info.requestDep = json.optString("requestDep");
            info.requestHosp = json.optString("requestHosp");
            info.requestNubeNumber = json.optString("requestNubeNumber");
            info.responseNubeNumber = json.optString("responseNubeNumber");
            info.responseName = json.optString("responseName");
            info.responseHosp = json.optString("responseHosp");
            info.responseDep = json.optString("responseDep");
            info.advice = URLDecoder.decode(json.optString("advice"));
            info.requestHeadThumUrl = json.optString("requestHeadThumUrl");
            info.responseHeadThumUrl = json.optString("responseHeadThumUrl");
            info.schedulDate = json.optString("schedulDate");
            info.requestProfessional = json.optString("requestProfessional");
            info.responseProfessional = json.optString("responseProfessional");
            info.requestScore = json.optString("requestScore");
            info.requestReview = URLDecoder.decode(json.optString("requestReview"));
            info.responseReview = URLDecoder.decode(json.optString("responseReview"));
            info.responseScore = json.optString("responseScore");
            info.meetingNo = json.optString("meetingNo");
            info.groupId = json.optString("groupId");
            info.rangeNumber = json.optString("rangeNumber");
            info.guardName = json.optString("guardName");
            info.transferFlg = json.optString("transferFlg");
            info.transferNube = json.optString("transferNube");
            info.transferId = json.optString("transferId");
            info.transferAdvice = URLDecoder.decode(json.optString("transferAdvice"));
            info.transferHosp = json.optString("transferHosp");
            info.transferDept = json.optString("transferDept");
            info.sectionType = json.optString("sectionType");
            info.expertName = json.optString("expertName");
            info.transferSchedulDate = json.optString("transferSchedulDate");
            info.transferRangeFlg = json.optString("transferRangeFlg");
            info.logoUrL = json.optString("logoUrL");
            info.sectionName = json.optString("sectionName");
            info.printTransferUrl = json.optString("printTransferUrl");
            info.expectReason = json.optString("expectReason");
            info.expectRangeNumber = json.optString("expectRangeNumber");
            info.expectDate = json.optString("expectDate");
            info.triageType = json.optString("triageFlg");

            info.transferState = json.optString("transferState");
            info.confirmDate = json.optString("confirmDate");
            info.transferAuthority = json.optString("transferAuthority");
            info.expectUnionOrgId = json.optString("expectUnionOrgId");
            info.expectDeptId = json.optString("expectDeptId");
            info.expectUnionOrgName = json.optString("expectUnionOrgName");
            info.expectDeptName = json.optString("expectDeptName");
            info.comment = URLDecoder.decode(json.optString("comment"));

            info.cslNube = json.optString("cslNube");
            info.cslHosp = json.optString("cslHosp");
            info.cslDep = json.optString("cslDep");
            info.cslName = json.optString("cslName");
            info.cslHeadThumUrl = json.optString("cslHeadThumUrl");
            info.cslProfessional = json.optString("cslProfessional");


            JSONArray responseArr = json.optJSONArray("responses");
            if (responseArr != null) {
                for (int i = 0; i < responseArr.length(); i++) {
                    JSONObject tmp = responseArr.optJSONObject(i);
                    ResponeseInfo responeseInfo = new ResponeseInfo();
                    responeseInfo.responseNubeNumber = tmp.optString("responseNubeNumber");
                    responeseInfo.responseHosp = tmp.optString("responseHosp");
                    responeseInfo.responseDep = tmp.optString("responseDep");
                    responeseInfo.responseName = tmp.optString("responseName");
                    responeseInfo.responseHeadThumUrl = tmp.optString("responseHeadThumUrl");
                    responeseInfo.responseProfessional = tmp.optString("responseProfessional");
                    responeseInfo.transferAuthFlg = tmp.optString("transferAuthFlg");
                    info.responeseInfos.add(responeseInfo);
                }
            }

            JSONArray estimeArr = json.optJSONArray("estimates");
            if (estimeArr != null) {
                for (int i = 0; i < estimeArr.length(); i++) {
                    JSONObject tmp = estimeArr.optJSONObject(i);
                    EstimatesInfo estimatesInfo = new EstimatesInfo();
                    estimatesInfo.triageScope = tmp.optString("triageScope");
                    estimatesInfo.triageEstimes = URLDecoder.decode(tmp.optString("triageEstimate"));
                    estimatesInfo.triageName = tmp.optString("triageName");
                    estimatesInfo.triageEstimateFlag = tmp.optString("triageEstimateFlg");
                    estimatesInfo.responeseScope = tmp.optString("responseScope");
                    estimatesInfo.responeseEstimes = URLDecoder.decode(tmp.optString("responseEstimate"));
                    estimatesInfo.responeseName = tmp.optString("responseName");
                    estimatesInfo.responseEstimateFlg = tmp.optString("responseEstimateFlg");
                    estimatesInfo.advice = URLDecoder.decode(tmp.optString("advice"));
                    estimatesInfo.responseNube = tmp.optString("responseNube");
                    info.estimatesInfos.add(estimatesInfo);
                }
            }

            info.printTransferUrl = json.optString("printTriageUrl");

            JSONArray expectArr = json.optJSONArray("expectResponses");
            if (expectArr != null) {
                for (int i = 0; i < expectArr.length(); i++) {
                    JSONObject expect = expectArr.optJSONObject(i);
                    TriageExpectResponseInfo expectResponseInfo = new TriageExpectResponseInfo();
                    expectResponseInfo.hospId = expect.optString("hospId");
                    expectResponseInfo.hospName = expect.optString("hospName");
                    expectResponseInfo.depId = expect.optString("depId");
                    expectResponseInfo.depName = expect.optString("depName");

                    JSONArray doctorArr = expect.optJSONArray("doctors");
                    if (doctorArr != null) {
                        for (int j = 0; j < doctorArr.length(); j++) {
                            JSONObject doctor = doctorArr.optJSONObject(j);
                            TriageDoctorInfo doctorInfo = new TriageDoctorInfo();
                            doctorInfo.id = doctor.optString("id");
                            doctorInfo.name = doctor.optString("name");
                            doctorInfo.professional = doctor.optString("professional");
                            doctorInfo.headThumUrl = doctor.optString("headThumUrl");
                            doctorInfo.nubeNumber = doctor.optString("nubeNumber");

                            expectResponseInfo.doctors.add(doctorInfo);
                        }
                    }

                    info.expectResponseInfos.add(expectResponseInfo);
                }
            }

        }

        return info;
    }


    protected Parser getParser() {
        return new HpuParser();
    }


}
