package cn.redcdn.datacenter.medicalcenter;


import java.io.UnsupportedEncodingException;
import java.util.Map;

import org.apache.http.HttpResponse;
import org.apache.http.entity.mime.content.ContentBody;
//import org.apache.http.protocol.HTTP;
import org.json.JSONObject;

import cn.redcdn.datacenter.DataErrorCode;
import cn.redcdn.datacenter.DataErrorInfo;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.JecCenter.JecParserInterface;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.log.CustomLog;
import cn.redcdn.network.httprequest.ConnectException;
import cn.redcdn.network.httprequest.GetTextHttp;
import cn.redcdn.network.httprequest.PostContentProgress;
import cn.redcdn.network.httprequest.PostTextHttp;
//import cn.redcdn.network.httprequest.PostTextHttpByGzip;

public class MDSAbstractBusinessData<T> {
    private String tag = getClass().getName();
    private PostTextHttp httpPost;
    private String requestUrl;
//    private PostTextHttpByGzip httpPostByGzip;

    private GetTextHttp httpGet;

    // 正常情况返回类型为T的数据对象
    // 异常情况下抛如下异常
    // InvalidateOperateException: 无效的操作
    // ConnectException: 连接服务器失败;
    // InvalidateHttpResponse: 无效得http response,比如http response字段错误,以及http
    // response io exception
    // UnhandledHttpStatus: 比如302,404等其他status code

    protected T execSync(String url, final String content) {
        httpPost = new PostTextHttp();
        httpPost.setURL(url);
        requestUrl = url;
        String responseContent = null;
        try {
            httpPost.setHttpHeader("Content-Type", "application/json");
            try {

                responseContent = httpPost.postCotentSync(content, "utf-8");
            } catch (UnsupportedEncodingException e1) {

                e1.printStackTrace();
            }
//	      responseContent = httpPost.postUrlEncodeContentSync(content);
            CustomLog.d(tag, " ResponseContent = " + responseContent);
            Parser parser = getParser();
            if (parser != null) {
                parser.parse(responseContent);
                try {
                    return parseContentBody(parser.getBody());
                } catch (InvalidateResponseException e) {
                    CustomLog.e(tag, "同步接口parseContentBody异常: " + e.toString());
                    return null;
                }
            }
        } catch (ConnectException e) {
            CustomLog.e(tag, "ConnectException");
            throw new ConnectException(e.getMessage());
        }
        return null;

    }

    // -1: 表示无效的操作，当前操作正在执行;
    // -2: 表示无效的编码;
    protected int exec(String url, String content) {
        if (null != httpPost) {
            return -1;
        }
        requestUrl = url;
        httpPost = new PostTextHttp() {

            @Override
            protected void onRequestFailed(int statusCode) {
                onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {
                CustomLog.d(tag, "<<<<<<<onRequestFinished: " + responseContent);
                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }
                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
                    onSuccess((T) ob);
                } else {
                    onFail(parser.getHeaderValue(), parser.getHeaderText());
                }
            }
        };
        httpPost.setURL(url);
        //httpPost.setHttpHeader("Content-Type", "application/json");
        //httpPost.postJsonContent(content.toString());

        try {

            return httpPost.postJsonContent(content);

        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            onFail(DataErrorCode.NOT_SUPPORT_ENCODING, DataErrorInfo.SERVICE_FAILED);
        }
        return -1;
    }

    protected int execOther(String url, String content) {
        if (null != httpPost) {
            return -1;
        }
        requestUrl = url;
        httpPost = new PostTextHttp() {

            @Override
            protected void onRequestFailed(int statusCode) {
                onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {
                CustomLog.d(tag, "onRequestFinished: " + responseContent);
                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }
                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
                    onSuccess((T) ob);
                } else {
                    onFail(parser.getHeaderValue(), parser.getHeaderText());
                }
            }
        };
        httpPost.setURL(url);
        //httpPost.setHttpHeader("Content-Type", "application/json");
        //httpPost.postJsonContent(content.toString());

        try {

            return httpPost.postCotentWithContentType(content, "application/x-www-form-urlencoded; charset=UTF-8");

        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            onFail(DataErrorCode.NOT_SUPPORT_ENCODING, DataErrorInfo.SERVICE_FAILED);
        }
        return -1;
    }

    protected int execFaceCompare(String url, String content) {
        if (null != httpPost) {
            return -1;
        }
        requestUrl = url;
        httpPost = new PostTextHttp() {

            @Override
            protected void onRequestFailed(int statusCode) {
                onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {
                CustomLog.d(tag, "onRequestFinished: " + responseContent);
                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }
                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        JSONObject jsonObject = new JSONObject(responseContent);
                        ob = parseContentBody(parser.getHeaderValue(), jsonObject);
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (Exception e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
                    onSuccess((T) ob);
                } else {
                    onFail(parser.getHeaderValue(), parser.getHeaderText());
                }
            }
        };
        httpPost.setURL(url);
        //httpPost.setHttpHeader("Content-Type", "application/json");
        //httpPost.postJsonContent(content.toString());

        try {

            return httpPost.postCotentWithContentType(content, "application/x-www-form-urlencoded; charset=UTF-8");

        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            onFail(DataErrorCode.NOT_SUPPORT_ENCODING, DataErrorInfo.SERVICE_FAILED);
        }
        return -1;
    }

    protected int execByGzip(String url, String content) {
        if (null != httpPost) {
            return -1;
        }
        requestUrl = url;
        httpPost = new PostTextHttp() {

            @Override
            protected void onRequestFailed(int statusCode) {
                onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {
                super.onRequestFinished(statusCode, responseContent);
                CustomLog.d(tag, "onRequestFinished 0: " + responseContent);

                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }
                CustomLog.d(tag, "onRequestFinished 1 start parse");
                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
                    onSuccess((T) ob);
                } else {
                    onFail(parser.getHeaderValue(), parser.getHeaderText());
                }
            }
        };

//            @Override
//            protected void onRequestFinished(int statusCode, HttpResponse response) {
//                CustomLog.d(tag, "onRequestFinished 1: " + response);
//                JecParserInterface parser = getJecParser();
//                if (null == parser) {
//                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
//                    return;
//                }
//                CustomLog.d(tag, "onRequestFinished 1 start parse");
//                parser.parse(response);
//                if (parser.isOk()) {
//                    Object ob = null;
//                    try {
//                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
//                    } catch (InvalidateResponseException e) {
//                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
//                        return;
//                    } catch (ClassCastException e) {
//                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
//                        return;
//                    } catch (NumberFormatException e) {
//                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
//                        return;
//                    }
//                    onSuccess((T) ob);
//                } else {
//                    onFail(parser.getHeaderValue(), parser.getHeaderText());
//                }
//            }
//        };
        httpPost.setURL(url);

        try {

            return httpPost.postJsonContent(content);

        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            onFail(DataErrorCode.NOT_SUPPORT_ENCODING, DataErrorInfo.SERVICE_FAILED);
        }
        return -1;
    }

    protected int exeg(String url, String content) {
        if (null != httpGet) {
            return -1;
        }
        requestUrl = url;
        httpGet = new GetTextHttp() {

//            @Override
//            protected void onReadData(String s) {
//
//            }

            @Override
            protected void onRequestFailed(int statusCode) {
                onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {
                CustomLog.d(tag, "onRequestFinished: " + responseContent);
                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }
                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
                    onSuccess((T) ob);
                } else {
                    onFail(parser.getHeaderValue(), parser.getHeaderText());
                }
            }
        };
        httpGet.setURL(url);
        //httpPost.setHttpHeader("Content-Type", "application/json");
        //httpPost.postJsonContent(content.toString());

        return httpGet.get();
    }

    protected int exeu(String url, Map<String, ContentBody> bodys) {
        if (null != httpPost) {
            return -1;
        }
        requestUrl = url;
        httpPost = new PostTextHttp() {

            @Override
            protected void onRequestFailed(int statusCode) {
                onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {
                CustomLog.d(tag, "onRequestFinished: " + responseContent);
                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }
                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
                    onSuccess((T) ob);
                } else {
                    onFail(parser.getHeaderValue(), parser.getHeaderText());
                }
            }
        };
        httpPost.setURL(url);

        return httpPost.postMultipartContent(bodys, new PostContentProgress() {

            @Override
            public void onPostProgress(int percent) {
                // TODO Auto-generated method stub
                onUploadProgress(percent);
            }
        });
    }

    protected int exeu(String url, Map<String, ContentBody> bodys, int timeout) {
        if (null != httpPost) {
            return -1;
        }
        requestUrl = url;
        httpPost = new PostTextHttp() {

            @Override
            protected void onRequestFailed(int statusCode) {
                onFail(statusCode, DataErrorInfo.NETWORK_FAILED);
            }

            @Override
            protected void onRequestFinished(int statusCode, String responseContent) {
                CustomLog.d(tag, "onRequestFinished: " + responseContent);
                Parser parser = getParser();
                if (null == parser) {
                    onFail(DataErrorCode.DATA_INVALIDATE_PARSER, DataErrorInfo.SERVICE_FAILED);
                    return;
                }
                parser.parse(responseContent);
                if (parser.isOk()) {
                    Object ob = null;
                    try {
                        ob = parseContentBody(parser.getHeaderValue(), parser.getBody());
                    } catch (InvalidateResponseException e) {
                        onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (ClassCastException e) {
                        onFail(DataErrorCode.CLASS_CAST_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    } catch (NumberFormatException e) {
                        onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, DataErrorInfo.SERVICE_FAILED);
                        return;
                    }
                    onSuccess((T) ob);
                } else {
                    onFail(parser.getHeaderValue(), parser.getHeaderText());
                }
            }
        };
        httpPost.setURL(url);

        return httpPost.postMultipartContent(bodys, new PostContentProgress() {

            @Override
            public void onPostProgress(int percent) {
                // TODO Auto-generated method stub
                onUploadProgress(percent);
            }
        }, timeout);
    }


    public void cancel() {
        if (null == httpPost) {
            return;
        }
        try {
            httpPost.cancel();
            httpPost = null;
        } catch (Exception e) {
            CustomLog.e(tag, "cancel:" + e.toString());
        }

    }

    protected T parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        return null;
    }

    protected T parseContentBody(int headerValue, JSONObject jo)
            throws InvalidateResponseException {
        return parseContentBody(jo);
    }

    protected Parser getParser() {
        return null;
    }

    protected JecParserInterface getJecParser() {
        return null;
    }

    protected void onSuccess(T responseContent) {
    }

    ;

    protected void onUploadProgress(int percent) {
    }

    ;

    // 返回HttpErrorCode、接口业务逻辑错误码、DataErrorCode
    protected void onFail(int statusCode, String statusInfo) {

        CustomLog.e(tag, "onFail, statusCode = " + statusCode + " statusInfo = " + statusInfo + "requestUrl =" + requestUrl);
    }

    ;


}
