package cn.redcdn.datacenter.medicalcenter;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.hpucenter.data.DTInfo;
import cn.redcdn.datacenter.hpucenter.data.ModuleInfo;
import cn.redcdn.datacenter.hpucenter.data.SubModuleInfo;
import cn.redcdn.datacenter.medicalcenter.MDSAbstractBusinessData;
import cn.redcdn.datacenter.medicalcenter.data.MDSAccountInfo;
import cn.redcdn.datacenter.sptcenter.data.SPTBaseInfo;
import cn.redcdn.datacenter.sptcenter.data.SPTInfo;
import cn.redcdn.log.CustomLog;

public abstract class MDSappLogin extends MDSAbstractBusinessData<MDSAccountInfo> {

    private String tag = MDSappLogin.class.getName();

    public int applogin(String token, String nubeNumber) {
        CustomLog.d(tag, "MDSappLogin token=" + token + "nube =" + nubeNumber);
        JSONObject json = new JSONObject();
        try {
            json.put("token", token);
            json.put("nube", nubeNumber);
//			json.put("versionNo", ConstConfig.MDS_SERVICE_VERSIONNO);
        } catch (JSONException e) {
            // TODO: handle exception
            return -3;
        }
        return exec(ConstConfig.getMedicalCenterUrl() + ConstConfig.MDS_APP_LOGIN, json.toString());
    }

    public int applogin(String token, String nubeNumber, String versionNo) {
        CustomLog.d(tag, "MDSappLogin token=" + token + "nube =" + nubeNumber);
        JSONObject json = new JSONObject();
        try {
            json.put("token", token);
            json.put("nube", nubeNumber);
            json.put("versionNo", versionNo);
        } catch (JSONException e) {
            // TODO: handle exception
            return -3;
        }
        return exec(ConstConfig.getMedicalCenterUrl() + ConstConfig.MDS_APP_LOGIN, json.toString());
    }

    public int applogin(String token, String nubeNumber, String versionNo, String clientType) {
        CustomLog.d(tag, "MDSappLogin token=" + token + "nube =" + nubeNumber);
        JSONObject json = new JSONObject();
        try {
            json.put("token", token);
            json.put("nube", nubeNumber);
            json.put("versionNo", versionNo);
            json.put("clientType", clientType);
        } catch (JSONException e) {
            // TODO: handle exception
            return -3;
        }
        return exec(ConstConfig.getMedicalCenterUrl() + ConstConfig.MDS_APP_LOGIN, json.toString());
    }

    protected MDSAccountInfo parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        try {
            if (jo == null) {
                CustomLog.e(tag, "MDS APPlogin invalidate reponse");
                throw new InvalidateResponseException();
            }

            JSONObject userInfo = jo.getJSONObject("data");
            MDSAccountInfo info = new MDSAccountInfo();
            info.uid = userInfo.optString("id");
            info.headThumUrl = userInfo.optString("headThumUrl");
            info.headPreviewUrl = userInfo.optString("headPreviewUrl");
            info.nickName = userInfo.optString("nickName");
            info.mobile = userInfo.optString("mobile");
            info.mail = userInfo.optString("mail");
            info.accountType = userInfo.optString("accountType");
            info.nube = userInfo.optString("nube");
            info.qrCodeUrl = userInfo.optString("qrCodeUrl");
            info.workUnit = userInfo.optString("workUnit");
            info.workUnitType = userInfo.optString("workUnitType");
            info.department = userInfo.optString("department");
            info.professional = userInfo.optString("professional");
            info.creatTime = userInfo.optString("createTime");
            info.officTel = userInfo.optString("officeTel");
            info.certificateThum = userInfo.optString("certificateThumUrl");
            info.certificatePreview = userInfo.optString("certificatePreviewUrl");
            info.state = userInfo.optInt("state");
            info.reason = userInfo.optString("reason");
            info.doctubeState = userInfo.optInt("doctubeState");
            info.doctubeReason = userInfo.optString("doctubeReason");
            info.licenseThumUrl = userInfo.optString("licenseThumUrl");
            info.licensePreviewUrl = userInfo.optString("licensePreviewUrl");

            info.remarks = userInfo.optString("remarks");

            info.model = userInfo.optString("model");
            info.remarks2 = userInfo.optString("remarks2");
            info.remarks3 = userInfo.optString("remarks3");
            info.token = userInfo.optString("token");


            if (info.state == 1) {
                info.stateDes = "待审核";
            }
            if (info.state == 2) {
                info.stateDes = "审核通过";
            }
            if (info.state == 3) {
                info.stateDes = "未通过";
            }

            List<DTInfo> mList = new ArrayList<>();
            JSONArray array = userInfo.optJSONArray("dtList");
            if (array != null) {
                if (array.length() > 0) {
                    for (int i = 0; i < array.length(); i++) {
                        JSONObject tmpjson = array.optJSONObject(i);
                        List<ModuleInfo> moduleInfos = new ArrayList<>();
                        DTInfo info1 = new DTInfo();
                        if (tmpjson != null) {
                            info1.name = tmpjson.optString("dtName");
                            info1.id = tmpjson.optString("dtId");
                            info1.logoUrl = tmpjson.optString("logoUrl");
                            info1.mainLogoUrl = tmpjson.optString("mainLogoUrl");
                            info1.logoUrl = tmpjson.optString("logoUrl");
                            info1.qrCodeUrl = tmpjson.optString("qrCodeUrl");
                            info1.infoOffId = tmpjson.optString("infoOffId");
                            info1.intOffId = tmpjson.optString("intrOffId");
                            info1.intrOffName = tmpjson.optString("intrOffName");
                            info1.infoOffName = tmpjson.optString("infoOffName");
                            JSONArray mArray = tmpjson.optJSONArray("module");
                            if (mArray != null) {
                                for (int j = 0; j < mArray.length(); j++) {
                                    JSONObject mjson = mArray.optJSONObject(j);
                                    if (mjson != null) {
                                        ModuleInfo moduleInfo = new ModuleInfo();
                                        moduleInfo.id = mjson.optString("id");
                                        moduleInfo.name = mjson.optString("name");
                                        moduleInfo.showName = mjson.optString("showName");
                                        moduleInfo.status = mjson.optInt("status");
                                        moduleInfo.perInfo = mjson.optString("operInfo");
                                        moduleInfo.function = mjson.optString("function");

                                        JSONArray subMoudleArr = mjson.optJSONArray("subModules");
                                        if (subMoudleArr != null) {
                                            for (int sub = 0; sub < subMoudleArr.length(); sub++) {
                                                JSONObject subMoudleObj = subMoudleArr.optJSONObject(sub);
                                                SubModuleInfo subModuleInfo = new SubModuleInfo();
                                                subModuleInfo.id = subMoudleObj.optString("id");
                                                subModuleInfo.showName = subMoudleObj.optString("showName");
                                                subModuleInfo.showIcon = subMoudleObj.optString("showIcon");
                                                subModuleInfo.operInfo = subMoudleObj.optString("operInfo");
                                                subModuleInfo.subType = subMoudleObj.optString("subType");

                                                subModuleInfo.privilegeFlg = subMoudleObj.optInt("privilegeFlg");
                                                subModuleInfo.privilegeTip  = subMoudleObj.optString("privilegeTip");

                                                moduleInfo.subModuleInfoList.add(subModuleInfo);
                                            }
                                        }



                                        moduleInfos.add(moduleInfo);
                                    }
                                }
                            }
                            info1.moduleList = moduleInfos;
                        }
                        mList.add(info1);
                    }
                }
            }
            info.dtList = mList;

            JSONObject sptJson = userInfo.optJSONObject("profestraining");
            if (sptJson != null) {
                info.sptInfos.roleBid = sptJson.optString("roleBid");
                info.sptInfos.roleName = sptJson.optString("roleName");
                info.sptInfos.baseName = sptJson.optString("base");
                info.sptInfos.studentNo = sptJson.optString("num");
                info.sptInfos.trainTime = sptJson.optString("trainDate");
                info.sptInfos.professionBid = sptJson.optString("profession");
                JSONArray baseArr = sptJson.optJSONArray("bases");
                if (baseArr != null) {
                    for (int m = 0; m < baseArr.length(); m++) {
                        JSONObject baseJson = baseArr.optJSONObject(m);
                        SPTBaseInfo baseInfo = new SPTBaseInfo();
                        if (baseJson != null) {
                            baseInfo.baseId = baseJson.optString("id");
                            baseInfo.baseName = baseJson.optString("name");
                            baseInfo.showName = baseJson.optString("showName");
                            baseInfo.associationName = baseJson.optString("associationName");
                            baseInfo.associationId = baseJson.optString("association_id");
                        }
                        info.sptInfos.baseInfos.add(baseInfo);
                    }
                }
            }
            return info;
        } catch (JSONException e) {
            CustomLog.e(tag, "enterprise login invalidate reponse");
            throw new InvalidateResponseException();
        }
    }

    protected Parser getParser() {
        return new MedicalParser();
    }
}
