package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;

/**
 * 获取视频帐号
 * 
 * @author LeeDong
 * 
 */
public abstract class BindOnAccount extends AbstractBusinessData<JSONObject> {
  private static final String TAG = BindOnAccount.class.getName();
  
  /**
   * 
   * @param imei
   * @return
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法
   */
  public int bind(String imei) {
    CustomLog.d(TAG, "call bind imei = " + imei);
    
    if(imei == null  || imei.equals("")){
      CustomLog.e(TAG, "参数不合法");
      return -5;
    }
    
    if(ConstConfig.getBindOnAccountUrl() == null || ConstConfig.getBindOnAccountUrl().equals("")){
      CustomLog.e(TAG, "请求服务器地址为空");
      return -4;
    }
    
    try {
      JSONObject json = new JSONObject();
      json.put(ConstConfig.PARAM_IMEI, imei);
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getBindOnAccountUrl(), ConstConfig.PARAM_BINDONACCOUNT+jsonString);
    } catch (Exception e) {
      CustomLog.e(TAG, e.getMessage());
      return -3;
    }
    
  }
  
  protected Parser getParser() {
    return new MeetingManageParser();
  }
  
  protected JSONObject parseContentBody(JSONObject jo){
    if(jo == null){
      CustomLog.e(TAG,"jo == null");
    }
    return jo;
  }

}