package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessDataDouble;
import cn.redcdn.datacenter.AbstractBusinessDataForBindToken;
import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.ResponseEmpty;
import cn.redcdn.log.CustomLog;

public abstract class BindToken extends AbstractBusinessDataForBindToken/*AbstractBusinessDataDouble*/<ResponseEmpty> {
  private String tag = BindToken.class.getName();

  /**
   * Token视频账号绑定。返回结果码 对应《企业中心接口文档》1.4.8
   * 
   * @param phoneId
   *          nube号
   * @param token
   *          有效的token
   * @param name
   *          参会名称
   * @return 
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法
   */
  public int bindToken(String phoneId, String token, String name) {
    CustomLog.i(tag, " bindToken, phoneId:" + phoneId + "token:" + token
        + "name:" + name);

    if( token == null ||token.equals("")){
      CustomLog.e(tag, "参数token为空");
      return -5;
    }
    if(ConstConfig.getBindTokenUrl() == null ||ConstConfig.getBindTokenUrl().equals("")){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    
    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_PHONEID, phoneId);
      json.put(ConstConfig.PARAM_TOKEN, token);
      json.put(ConstConfig.PARAM_NAME, name);
    } catch (JSONException e) {
      return -3;
    }
    
    String FirstRequestUrl = ConstConfig.getBindTokenUrl();
    String SecondRequestUrl = ConstConfig.getSlaveMeetingManagementHttpUrlPrefix()	
			                 +ConstConfig.bmsWebDomainSuffix;
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
	
    return exec(FirstRequestUrl,SecondRequestUrl, ConstConfig.PARAM_BINDTOKEN
        + jsonString);
    
  }

  
  public int bindToken4jmeeting(String phoneId, String token, String name,String terminalType) {
    CustomLog.i(tag, " bindToken4jmeeting, phoneId:" + phoneId + "token:" + token
        + "name:" + name + " terminalType:"+terminalType);

    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_PHONEID, phoneId);
      json.put(ConstConfig.PARAM_TOKEN, token);
      json.put(ConstConfig.PARAM_NAME, name);
      json.put(ConstConfig.PARAM_TERMINALTYPE, terminalType);
    } catch (JSONException e) {
      return -3;
    }
    
    String FirstRequestUrl = ConstConfig.getBindTokenUrl();
    String SecondRequestUrl = ConstConfig.getSlaveMeetingManagementHttpUrlPrefix()	
			                 +ConstConfig.bmsWebDomainSuffix;
    
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
    return exec(FirstRequestUrl,SecondRequestUrl, ConstConfig.PARAM_BINDTOKEN
        + jsonString);
    
  }
  
  protected Parser getParser() {
    return new MeetingManageParser();
  }


}
