package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessDataDoublePwd;
import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.CheckHasMeetingPwdInfo;
import cn.redcdn.datacenter.meetingmanage.data.GetMeetingPwdInfo;
import cn.redcdn.log.CustomLog;

/**
 * CheckHasMeetingPwd 判断会议是否有密码
 * @author Administrator
 *
 */
public abstract class CheckHasMeetingPwd extends AbstractBusinessDataDoublePwd<CheckHasMeetingPwdInfo> {

	protected final String TAG = getClass().getName();
	
	/**
	 * 判断会议是否有密码接口
	 * 
	 * @param meetingId（类型：int | 说明：会议号）
	 * 
	 * @return
	 *  -1: 表示无效的操作，当前操作正在执行
	 *  -2: 表示无效的编码  
	 *  -3: 表示json操作异常
	 *  -4: 会议接口服务器地址不合法
	 *  -5: 输入参数不合法
	 *   0: 成功
	 *-999: 系统错误
	 *-900:传入的json数据不正确
	 *-901:参数不能为空
	 *-906:无效的会议No 
	 *-909:service参数不能为空
	 *-910:service参数不正确
	 */
	public int checkHasMeetingPwd(int meetingId) {
		
		CustomLog.i(TAG, "meetingId=" + meetingId);
		
		if(String.valueOf(meetingId) == null || String.valueOf(meetingId).equals("")){
			
			CustomLog.e(TAG, "参数meetingId为空");
			
			return -5;
			
		}
		
		if(ConstConfig.getCheckHasMeetingPwdUrl() == null || ConstConfig.getCheckHasMeetingPwdUrl().equals("")){
			
			CustomLog.e(TAG, "请求服务器地址为空");
			
			return -4;
			
		}
		
		JSONObject json = new JSONObject();
		
		try{
			
			json.put(ConstConfig.PARAM_MEETINGID, meetingId);
			
			  //转义特殊字符   % --> %25
			String jsonString = java.net.URLEncoder.encode(json.toString());
			return exec(ConstConfig.getCheckHasMeetingPwdUrl(),ConstConfig.getSlaveCheckHasMeetingPwdUrl(), ConstConfig.PARAM_CHECK_HAS_MEETING_PWD + jsonString);
			
		} catch (JSONException e) {
			
			return -3;
			
		}
		
	}
	
	protected CheckHasMeetingPwdInfo parseContentBody(JSONObject jo) 
			throws InvalidateResponseException {
		
		if(jo == null){
			
			CustomLog.e(TAG, "CHECKHASMEETINGPWD JSONObject == null");
			
			throw new InvalidateResponseException();
			
		}
		
		CheckHasMeetingPwdInfo info = new CheckHasMeetingPwdInfo();
		
		info.meetingHost = jo.optString("meetingHost");
		
		info.hasMeetingPwd = jo.optInt("hasMeetingPwd");
		
		return info;
	}
	
	protected Parser getParser() {
		
		return new MeetingManageParser();
		
	}
	
}
