package cn.redcdn.datacenter.meetingmanage;

import java.util.HashMap;

import java.util.Iterator;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;
import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.AbstractBusinessDataDouble;
import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.CheckMeetingPwdInfo;
import cn.redcdn.log.CustomLog;

public abstract class CheckMeetingPwd extends
        AbstractBusinessDataDouble<CheckMeetingPwdInfo> {
	protected int rc;
	protected String rd;
	private String tag = CheckMeetingPwd.class.getName();

	/**
	 * 校验会议密码接口
	 * @param token  接口调用凭证
	 * @param meetingId   需要校验密码的会议编号
	 * @param meetingPwd  待验证会议密码
	 * @return
	 *    0:成功
	 * -999:系统错误   
	 * -900:传入的json数据不正确
	 * -901:参数不能为空
	 * -902:Token不存在
	 * -903:Token已过期
	 * -906:无效的会议No
	 * -909:service参数不能为空
	 * -910:service参数不正确
	 * -928:会议密码不正确
	 */
	public int checkMeetingPwd(String token, int meetingId, String meetingPwd) {
		// TODO Auto-generated constructor stub
		CustomLog.i(tag, "call CheckMeetingPwd : token = " + token
				+ "meetingId = " + meetingId + "meetingPwd" + meetingPwd);

		if (token == null || token.equals("")) {
			CustomLog.e(tag, "token为空");
			return -5;
		}

		if ("null".equals(String.valueOf(meetingId))
				|| "0".equals(String.valueOf(meetingId))) {
			CustomLog.e(tag, "meetingId为空");
			return -4;
		}

		if (meetingPwd == null || meetingPwd.equals("")) {
			CustomLog.e(tag, "meetingPwd为空");
			return -3;
		}
		try {
			JSONObject json = new JSONObject();
			json.put(ConstConfig.PARAM_TOKEN, token);
			json.put(ConstConfig.PARAM_MEETINGID, meetingId);
			json.put(ConstConfig.PARAM_MEETINGPWD, meetingPwd);
			  //转义特殊字符   % --> %25
			String jsonString = java.net.URLEncoder.encode(json.toString());
			return exec(ConstConfig.getGetMeetingsUrl(),ConstConfig.getSlaveGetMeetingsUrl(),
					ConstConfig.PARAM_CHECKMEETINGPWD + jsonString);
		} catch (Exception e) {
			CustomLog.e(tag, e.getMessage());
		}
		return -2;
	}

	protected CheckMeetingPwdInfo parseContentBody(JSONObject bodyObject)
			throws InvalidateResponseException {
		if (bodyObject == null) {
			CustomLog.e(tag, "CheckMeetingPwd JSONObject == null");
			throw new InvalidateResponseException();
		}
		
		CheckMeetingPwdInfo info = new CheckMeetingPwdInfo();
		info.rc = bodyObject.optInt("rc");
		info.rd = bodyObject.optString("rd");
		return info;

	}
	protected Parser getParser() {
		return new CheckMeetingPwdParser();
	}
}
