package cn.redcdn.datacenter.meetingmanage;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.CheckROMVersionParser;
import cn.redcdn.log.CustomLog;

public abstract class CheckROMVersion  extends AbstractBusinessData<Map<String, Object>>{
  
  private String TAG = CheckROMVersion.class.getName();
  
  /**
   * 
   * @param productName
   * @param version
   * @return
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -4: 接口服务器地址不合法
   */
  public int checkVersion(String productName, String version) {
    
    CustomLog.d(TAG, "CALL checkVersion,productName = " + productName
        + ",version" + version);
    
    if(ConstConfig.getROMVersionCheckUrl() == null || ConstConfig.getROMVersionCheckUrl().equals("")){
      CustomLog.e(TAG, "请求服务器地址为空");
      return -4;
    }
    
    StringBuffer sb = new StringBuffer();
    sb.append(ConstConfig.PARAM_PRODUCT).append("=").append(productName).append("&")
    .append(ConstConfig.PARAM_VERSION).append("=").append(version);
    return exec(ConstConfig.getROMVersionCheckUrl()+"?"+sb.toString(), "");
    
  }
  
  protected Map<String, Object> parseContentBody(JSONObject bodyObject)
      throws InvalidateResponseException {
    if (bodyObject == null) {
      CustomLog.e(TAG, "CheckROMVersion JSONObject == null");
      throw new InvalidateResponseException();
    }
    CustomLog.d(TAG, "parseContentBody= "+ bodyObject );
    
    Iterator<String> keys = bodyObject.keys();
    Map<String, Object> map = null;
    if (keys != null && keys.hasNext()) {
      map = new HashMap<String, Object>();
      String key = null;
      try {
        do {
          key = keys.next();
          map.put(key, bodyObject.get(key));
        } while (keys.hasNext());
      } catch (JSONException e) {
        CustomLog.e(TAG, e.getMessage());
      }
    }
    return map;
    
  }
  
  protected Parser getParser(){
    return new CheckROMVersionParser();
  }
  
}
