package cn.redcdn.datacenter.meetingmanage;

import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.MeetingInfo;
import cn.redcdn.log.CustomLog;

public abstract class CreateMeeting extends AbstractBusinessDataSub<MeetingInfo> {

    private String tag = CreateMeeting.class.getName();

    /**
     * 即时会议
     * <p/>
     */
    public static final int MEETINGTYPE_INSTANT = 1;

    /**
     * 预约会议
     */
    public static final int MEETINGTYPE_RESERVE = 2;

    /**
     * 产品类型为极会议
     */
    public static final String MEETING_APP_JIHY = "JIHY";

    /**
     * 产品类型为可视极会议
     */
    public static final String MEETING_APP_KESHI_JIHY = "KESHI_JIHY";

    /**
     * 产品类型为红云会诊
     */
    public static final String MEETING_APP_BUTEL_CONSULTATION = "HVS";

    public static final String MEETING_APP_ZG_CONSULTATION = "zghy";

    public static final String MEETING_APP_JHY_CONSULTATION = "JHY_MOBILE";

    // **********************原始创建会议代码**************************************
    /**
     * 创建即时会议接口，主页点击召开会议时调用。服务器将返回会议id、会议管理员账号。
     * <p/>召开即时会议默认产品类型为"JIHY"，主题和开始时间为""
     *
     * @param token       有效的token
     * @param meetingType 固定为1：即时会议 。参见《会议接口 V2.13》 附录.会议类型枚举
     * @param phoneId     邀请人列表
    //     * @param extraInfo   扩展信息（json格式）
     * @return -1: 表示无效的操作，当前操作正在执行
     * -2: 表示无效的编码
     * -3: 表示json操作异常
     * -4: 会议接口服务器地址不合法
     * -5: 输入参数不合法
     */

    public int createMeeting(String token, int meetingType, List<String> phoneId) {

        String app = CreateMeeting.MEETING_APP_JIHY;
        String topic = "";
        String beginDateTime = "";
        return createMeeting(app, topic, phoneId, token,
                meetingType, beginDateTime);
    }

    /**
     * 手机预约会议接口，增加预约会议主题,预约会议需把自己添加到邀请人员列表中
     *
     * @param app           产品类型 默认空：极会议(极会议: JIHY, 可视极会议：KESHI_JIHY)
     * @param topic         手机预约会议主题
     * @param phoneId       邀请人员列表
     * @param token         有效的token
     * @param meetingType   会议类型
     * @param beginDateTime 预约会议开始时间与1970-1-1相差的秒数（UTC时区），即时会议不需要填写
     *                      //     * @param extraInfo   扩展信息（json格式）
     * @return -1: 表示无效的操作，当前操作正在执行
     * -2: 表示无效的编码
     * -3: 表示json操作异常
     * -4: 会议接口服务器地址不合法
     * -5: 输入参数不合法
     */
    public int createMeeting(String app, String topic, List<String> phoneId, String token,
                             int meetingType, String beginDateTime) {
        CustomLog.i(tag, "call createMeeting .token = " + token);
        if (token == null || token.equals("")) {
            CustomLog.e(tag, "参数token为空");
            return -5;
        }
        if (ConstConfig.getCreateMeetingUrl() == null || ConstConfig.getCreateMeetingUrl().equals("")) {
            CustomLog.e(tag, "请求服务器地址为空");
            return -4;
        }
        JSONObject json = new JSONObject();
        JSONArray ja1 = new JSONArray();
        JSONObject jo = null;
        try {
            for (int i = 0; i < phoneId.size(); i++) {
                jo = new JSONObject();
                jo.put("phoneId", phoneId.get(i));
                ja1.put(jo);
            }
            json.put(ConstConfig.PARAM_APP, app);
            json.put(ConstConfig.PARAM_TOPIC, topic);
            json.put(ConstConfig.PARAM_TOKEN, token);
            json.put(ConstConfig.PARAM_MEETINGTYPE, meetingType);
            json.put(ConstConfig.PARAM_INVOTEDUSERS, ja1);
            json.put(ConstConfig.PARAM_MEETINGTIME, beginDateTime);
//            json.put(ConstConfig.PARAM_MEETINGEXTRAINFO,extraInfo);
            //转义特殊字符   % --> %25
            String jsonString = java.net.URLEncoder.encode(json.toString());
            return exec(ConstConfig.getCreateMeetingUrl(),
                    ConstConfig.PARAM_CREATEMEETING + jsonString);
        } catch (Exception e) {
            CustomLog.e(tag, e.getMessage());
            return -3;
        }
    }

    /**
     * 手机预约会议接口,添加结束时间功能，增加预约会议主题,预约会议需把自己添加到邀请人员列表中
     *
     * @param app           产品类型 默认空：极会议(极会议: JIHY, 可视极会议：KESHI_JIHY)
     * @param topic         手机预约会议主题
     * @param phoneId       邀请人员列表
     * @param token         有效的token
     * @param meetingType   会议类型
     * @param beginDateTime 预约会议开始时间与1970-1-1相差的秒数（UTC时区），即时会议不需要填写
     *                      //     * @param extraInfo   扩展信息（json格式）
     * @return -1: 表示无效的操作，当前操作正在执行
     * -2: 表示无效的编码
     * -3: 表示json操作异常
     * -4: 会议接口服务器地址不合法
     * -5: 输入参数不合法
     */
    public int createMeeting(String app, String topic, List<String> phoneId, String token,
                             int meetingType, String beginDateTime, String effectiveHour) {
        CustomLog.i(tag, "call createMeeting .token = " + token);
        if (token == null || token.equals("")) {
            CustomLog.e(tag, "参数token为空");
            return -5;
        }
        if (ConstConfig.getCreateMeetingUrl() == null || ConstConfig.getCreateMeetingUrl().equals("")) {
            CustomLog.e(tag, "请求服务器地址为空");
            return -4;
        }
        JSONObject json = new JSONObject();
        JSONArray ja1 = new JSONArray();
        JSONObject jo = null;
        try {
            for (int i = 0; i < phoneId.size(); i++) {
                jo = new JSONObject();
                jo.put("phoneId", phoneId.get(i));
                ja1.put(jo);
            }
            json.put(ConstConfig.PARAM_APP, app);
            json.put(ConstConfig.PARAM_TOPIC, topic);
            json.put(ConstConfig.PARAM_TOKEN, token);
            json.put(ConstConfig.PARAM_MEETINGTYPE, meetingType);
            json.put(ConstConfig.PARAM_INVOTEDUSERS, ja1);
            json.put(ConstConfig.PARAM_MEETINGTIME, beginDateTime);
            json.put(ConstConfig.EFFECTIVE_TIME, effectiveHour);
//            json.put(ConstConfig.PARAM_MEETINGEXTRAINFO,extraInfo);
            //转义特殊字符   % --> %25
            String jsonString = java.net.URLEncoder.encode(json.toString());
            return exec(ConstConfig.getCreateMeetingUrl(),
                    ConstConfig.PARAM_CREATEMEETING + jsonString);
        } catch (Exception e) {
            CustomLog.e(tag, e.getMessage());
            return -3;
        }
    }


    /**
     * 创建加密会议接口
     *
     * @param token（类型：String         | 说明：有效的token）
     * @param meetingType（类型：int      | 说明：会议类型）
     * @param beginDateTime（类型：       String | 说明：会议开始时间与1970-1-1相差的秒数（UTC时区），即时会议不需要填写）
     * @param topic（类型：String         | 说明：手机会议主题）
     * @param meetingPwd（类型：String    | 说明：会议主持人创建会议携带的会议密码）
     * @param effectiveHour（类型：       String | 说明：会议的持续时长，单位小时）
     * @param phoneId（类型：List<String> | 说明：邀请人列表）
     *                                //     * @param extraInfo   扩展信息（json格式）
     * @return -1: 表示无效的操作，当前操作正在执行
     * -2: 表示无效的编码
     * -3: 表示json操作异常
     * -4: 会议接口服务器地址不合法
     * -5: 输入参数不合法
     * 0: 成功
     * -999:系统错误
     * -900:传入的json数据不正确
     * -901:参数不能为空
     * -902:Token不存在
     * -905:无效的会议类型
     * -909:service参数不能为空
     * -910:service参数不正确
     * -911:预约会议要设置会议开始时间
     * -912:没有添加参会邀请人
     * -926:无效的会议创建用户名或密码
     */
    public int createMeeting(String app, String topic, String meetingPwd, List<String> phoneId, String token,
                             int meetingType, String beginDateTime, String effectiveHour) {
        CustomLog.i(tag, "call createMeeting .token = " + token);
        if (token == null || token.equals("")) {
            CustomLog.e(tag, "参数token为空");
            return -5;
        }
        if (ConstConfig.getCreateMeetingUrl() == null || ConstConfig.getCreateMeetingUrl().equals("")) {
            CustomLog.e(tag, "请求服务器地址为空");
            return -4;
        }
        JSONObject json = new JSONObject();
        JSONArray ja1 = new JSONArray();
        JSONObject jo = null;
        try {
            for (int i = 0; i < phoneId.size(); i++) {
                jo = new JSONObject();
                jo.put("phoneId", phoneId.get(i));
                ja1.put(jo);
            }
            json.put(ConstConfig.PARAM_APP, app);
            json.put(ConstConfig.PARAM_TOPIC, topic);
            json.put(ConstConfig.PARAM_TOKEN, token);
            json.put(ConstConfig.PARAM_MEETINGTYPE, meetingType);
            json.put(ConstConfig.PARAM_INVOTEDUSERS, ja1);
            json.put(ConstConfig.PARAM_MEETINGTIME, beginDateTime);
            json.put(ConstConfig.EFFECTIVE_TIME, effectiveHour);
            json.put(ConstConfig.PARAM_MEETINGPWD, meetingPwd);
//            json.put(ConstConfig.PARAM_MEETINGEXTRAINFO,extraInfo);
            //转义特殊字符   % --> %25
            String jsonString = java.net.URLEncoder.encode(json.toString());
            return exec(ConstConfig.getCreateMeetingUrl(),
                    ConstConfig.PARAM_CREATEMEETING + jsonString);
        } catch (Exception e) {
            CustomLog.e(tag, e.getMessage());
            return -3;
        }
    }
    // ****************************************************************************

    // *********************盒端带qos分组功能代码**********************************

//    /**
//     * 创建即时会议接口，主页点击召开会议时调用。服务器将返回会议id、会议管理员账号。
//     * <p/>召开即时会议默认产品类型为"JIHY"，主题和开始时间为""
//     *
//     * @param token       有效的token
//     * @param meetingType 固定为1：即时会议 。参见《会议接口 V2.13》 附录.会议类型枚举
//     * @param phoneId     邀请人列表
//     * @param extraInfo   扩展信息（json格式）
//     * @return -1: 表示无效的操作，当前操作正在执行
//     * -2: 表示无效的编码
//     * -3: 表示json操作异常
//     * -4: 会议接口服务器地址不合法
//     * -5: 输入参数不合法
//     */
//
//    public int createMeeting(String token, int meetingType, List<String> phoneId, String extraInfo, int speakersNum) {
//
//        String app = CreateMeeting.MEETING_APP_JIHY;
//        String topic = "";
//        String beginDateTime = "";
//        return createMeeting(app, topic, phoneId, token,
//                meetingType, beginDateTime, extraInfo, speakersNum);
//    }
//
//    /**
//     * 手机预约会议接口，增加预约会议主题,预约会议需把自己添加到邀请人员列表中
//     *
//     * @param app           产品类型 默认空：极会议(极会议: JIHY, 可视极会议：KESHI_JIHY)
//     * @param topic         手机预约会议主题
//     * @param phoneId       邀请人员列表
//     * @param token         有效的token
//     * @param meetingType   会议类型
//     * @param beginDateTime 预约会议开始时间与1970-1-1相差的秒数（UTC时区），即时会议不需要填写
//     * @param extraInfo     扩展信息（json格式）
//     * @return -1: 表示无效的操作，当前操作正在执行
//     * -2: 表示无效的编码
//     * -3: 表示json操作异常
//     * -4: 会议接口服务器地址不合法
//     * -5: 输入参数不合法
//     */
//    public int createMeeting(String app, String topic, List<String> phoneId, String token,
//                             int meetingType, String beginDateTime, String extraInfo, int speakersNum) {
//        CustomLog.i(tag, "call createMeeting .token = " + token);
//        if (token == null || token.equals("")) {
//            CustomLog.e(tag, "参数token为空");
//            return -5;
//        }
//        if (ConstConfig.getCreateMeetingUrl() == null || ConstConfig.getCreateMeetingUrl().equals("")) {
//            CustomLog.e(tag, "请求服务器地址为空");
//            return -4;
//        }
//        JSONObject json = new JSONObject();
//        JSONArray ja1 = new JSONArray();
//        JSONObject jo = null;
//        try {
//            for (int i = 0; i < phoneId.size(); i++) {
//                jo = new JSONObject();
//                jo.put("phoneId", phoneId.get(i));
//                ja1.put(jo);
//            }
//            json.put(ConstConfig.PARAM_APP, app);
//            json.put(ConstConfig.PARAM_TOPIC, topic);
//            json.put(ConstConfig.PARAM_TOKEN, token);
//            json.put(ConstConfig.PARAM_MEETINGTYPE, meetingType);
//            json.put(ConstConfig.PARAM_INVOTEDUSERS, ja1);
//            json.put(ConstConfig.PARAM_MEETINGTIME, beginDateTime);
//            json.put(ConstConfig.PARAM_MEETINGEXTRAINFO, extraInfo);
//            json.put("speakersNum", speakersNum);
//            //转义特殊字符   % --> %25
//            String jsonString = java.net.URLEncoder.encode(json.toString());
//            return exec(ConstConfig.getCreateMeetingUrl(),
//                    ConstConfig.PARAM_CREATEMEETING + jsonString);
//        } catch (Exception e) {
//            CustomLog.e(tag, e.getMessage());
//            return -3;
//        }
//    }
//
//    /**
//     * 手机预约会议接口,添加结束时间功能，增加预约会议主题,预约会议需把自己添加到邀请人员列表中
//     *
//     * @param app           产品类型 默认空：极会议(极会议: JIHY, 可视极会议：KESHI_JIHY)
//     * @param topic         手机预约会议主题
//     * @param phoneId       邀请人员列表
//     * @param token         有效的token
//     * @param meetingType   会议类型
//     * @param beginDateTime 预约会议开始时间与1970-1-1相差的秒数（UTC时区），即时会议不需要填写
//     * @param extraInfo     扩展信息（json格式）
//     * @return -1: 表示无效的操作，当前操作正在执行
//     * -2: 表示无效的编码
//     * -3: 表示json操作异常
//     * -4: 会议接口服务器地址不合法
//     * -5: 输入参数不合法
//     */
//    public int createMeeting(String app, String topic, List<String> phoneId, String token,
//                             int meetingType, String beginDateTime, String effectiveHour, String extraInfo, int speakersNum) {
//        CustomLog.i(tag, "call createMeeting .token = " + token);
//        if (token == null || token.equals("")) {
//            CustomLog.e(tag, "参数token为空");
//            return -5;
//        }
//        if (ConstConfig.getCreateMeetingUrl() == null || ConstConfig.getCreateMeetingUrl().equals("")) {
//            CustomLog.e(tag, "请求服务器地址为空");
//            return -4;
//        }
//        JSONObject json = new JSONObject();
//        JSONArray ja1 = new JSONArray();
//        JSONObject jo = null;
//        try {
//            for (int i = 0; i < phoneId.size(); i++) {
//                jo = new JSONObject();
//                jo.put("phoneId", phoneId.get(i));
//                ja1.put(jo);
//            }
//            json.put(ConstConfig.PARAM_APP, app);
//            json.put(ConstConfig.PARAM_TOPIC, topic);
//            json.put(ConstConfig.PARAM_TOKEN, token);
//            json.put(ConstConfig.PARAM_MEETINGTYPE, meetingType);
//            json.put(ConstConfig.PARAM_INVOTEDUSERS, ja1);
//            json.put(ConstConfig.PARAM_MEETINGTIME, beginDateTime);
//            json.put(ConstConfig.EFFECTIVE_TIME, effectiveHour);
//            json.put(ConstConfig.PARAM_MEETINGEXTRAINFO, extraInfo);
//            json.put("speakersNum", speakersNum);
//            //转义特殊字符   % --> %25
//            String jsonString = java.net.URLEncoder.encode(json.toString());
//            return exec(ConstConfig.getCreateMeetingUrl(),
//                    ConstConfig.PARAM_CREATEMEETING + jsonString);
//        } catch (Exception e) {
//            CustomLog.e(tag, e.getMessage());
//            return -3;
//        }
//    }
//
//
//    /**
//     * 创建加密会议接口
//     *
//     * @param token（类型：String         | 说明：有效的token）
//     * @param meetingType（类型：int      | 说明：会议类型）
//     * @param beginDateTime（类型：       String | 说明：会议开始时间与1970-1-1相差的秒数（UTC时区），即时会议不需要填写）
//     * @param topic（类型：String         | 说明：手机会议主题）
//     * @param meetingPwd（类型：String    | 说明：会议主持人创建会议携带的会议密码）
//     * @param effectiveHour（类型：       String | 说明：会议的持续时长，单位小时）
//     * @param phoneId（类型：List<String> | 说明：邀请人列表）
//     * @param extraInfo               扩展信息（json格式）
//     * @return -1: 表示无效的操作，当前操作正在执行
//     * -2: 表示无效的编码
//     * -3: 表示json操作异常
//     * -4: 会议接口服务器地址不合法
//     * -5: 输入参数不合法
//     * 0: 成功
//     * -999:系统错误
//     * -900:传入的json数据不正确
//     * -901:参数不能为空
//     * -902:Token不存在
//     * -905:无效的会议类型
//     * -909:service参数不能为空
//     * -910:service参数不正确
//     * -911:预约会议要设置会议开始时间
//     * -912:没有添加参会邀请人
//     * -926:无效的会议创建用户名或密码
//     */
//    public int createMeeting(String app, String topic, String meetingPwd, List<String> phoneId, String token,
//                             int meetingType, String beginDateTime, String effectiveHour, String extraInfo, int speakersNum) {
//        CustomLog.i(tag, "call createMeeting .token = " + token);
//        if (token == null || token.equals("")) {
//            CustomLog.e(tag, "参数token为空");
//            return -5;
//        }
//        if (ConstConfig.getCreateMeetingUrl() == null || ConstConfig.getCreateMeetingUrl().equals("")) {
//            CustomLog.e(tag, "请求服务器地址为空");
//            return -4;
//        }
//        JSONObject json = new JSONObject();
//        JSONArray ja1 = new JSONArray();
//        JSONObject jo = null;
//        try {
//            for (int i = 0; i < phoneId.size(); i++) {
//                jo = new JSONObject();
//                jo.put("phoneId", phoneId.get(i));
//                ja1.put(jo);
//            }
//            json.put(ConstConfig.PARAM_APP, app);
//            json.put(ConstConfig.PARAM_TOPIC, topic);
//            json.put(ConstConfig.PARAM_TOKEN, token);
//            json.put(ConstConfig.PARAM_MEETINGTYPE, meetingType);
//            json.put(ConstConfig.PARAM_INVOTEDUSERS, ja1);
//            json.put(ConstConfig.PARAM_MEETINGTIME, beginDateTime);
//            json.put(ConstConfig.EFFECTIVE_TIME, effectiveHour);
//            json.put(ConstConfig.PARAM_MEETINGPWD, meetingPwd);
//            json.put(ConstConfig.PARAM_MEETINGEXTRAINFO, extraInfo);
//            json.put("speakersNum", speakersNum);
//            //转义特殊字符   % --> %25
//            String jsonString = java.net.URLEncoder.encode(json.toString());
//            return exec(ConstConfig.getCreateMeetingUrl(),
//                    ConstConfig.PARAM_CREATEMEETING + jsonString);
//        } catch (Exception e) {
//            CustomLog.e(tag, e.getMessage());
//            return -3;
//        }
//    }
    // ****************************************************************************

    protected MeetingInfo parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        if (jo == null) {
            CustomLog.e(tag, "CREATEMEETING JSONObject == null");
            throw new InvalidateResponseException();
        }
        MeetingInfo info = new MeetingInfo();

        info.meetingId = jo.optString("meetingId");
        info.adminPhoneId = jo.optString("adminPhoneId");

        return info;
    }

    protected Parser getParser() {
        return new MeetingManageParser();
    }


}
