package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.ResponseEmpty;
import cn.redcdn.log.CustomLog;

public abstract class EditMeeting extends AbstractBusinessData<ResponseEmpty>{
  private String tag = EditMeeting.class.getName();
  
  /**
   * EditMeeting 提交预约会议。
   * 
   * @param token
   *          
   * @param meetingId
   * 
   * @param userContact
   * 
   * @return 
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法
   */
  public int editMeeting(String token, int meetingId, String userContact) {
    CustomLog.i(tag, " editMeeting, meetingId:" + meetingId + "userContact:" + userContact
        + "token" + token);

    if (token == null ||token.equals("") ) {
      CustomLog.e(tag, "参数token为空");
      return -5;
    }
    if( ConstConfig.getEditMeetingUrl() == null ||ConstConfig.getEditMeetingUrl().equals("")){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    
    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_TOKEN, token);
      json.put(ConstConfig.PARAM_MEETINGID, meetingId);
      json.put(ConstConfig.PARAM_USERCONTACT, userContact);
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getEditMeetingUrl(), ConstConfig.PARAM_EDITMEETING
          + jsonString);
    } catch (JSONException e) {
      CustomLog.e(tag, e.getMessage());
      return -3;
    }

  }
  
  protected Parser getParser() {
    return new EditMeetingParser();
  }

}
