package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.MeetingInfomation;
import cn.redcdn.log.CustomLog;

public class GetMeetingInfomation extends AbstractBusinessDataSub<MeetingInfomation>{
  private String tag = GetMeetingInfomation.class.getName();

  /**
   * 获取会议信息    
   * @param meetingId     
   *        被查找的会议Id  hasMeetingPwd 会议是否有密码   meetingHost 会议主持人
   * @return
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法
   *   0: 成功
   *-999: 系统错误
   *-900: 传入的json数据不正确
   *-901: 参数不能为空
   *-909: service参数不能为空
   *-910: service参数不正确
   */
  public int getMeetingInfomation(int meetingId){
    CustomLog.i(tag, "call getMeetingInfomation ,meetingId = " + meetingId);
    
    if (meetingId == 0 ) {
      CustomLog.e(tag, "meetingId 为0");
      return -5;
    }
    if(ConstConfig.getMeetingInfomationUrl() == null ||ConstConfig.getMeetingInfomationUrl().equals("")){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    
    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_MEETINGID, meetingId);
      
	  //转义特殊字符   % --> %25
	  String jsonString = java.net.URLEncoder.encode(json.toString());
	
      return exec(ConstConfig.getMeetingInfomationUrl(),
          ConstConfig.PARAM_GETMEETINGINFO + jsonString);
    } catch (JSONException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
      return -3;
    }
    
  }
  
  protected Parser getParser() {
    return new MeetingManageParser();
  }
  
  protected MeetingInfomation parseContentBody(JSONObject jo)
      throws InvalidateResponseException{
    if (jo == null) {
      CustomLog.e(tag, "getMeetingInfomation JSONObject == null");
      throw new InvalidateResponseException();
    }
    MeetingInfomation mInfomation = new MeetingInfomation();
    mInfomation.enterpriseId = jo.optString("enterpriseId");
    mInfomation.terminalAccount = jo.optString("terminalAccount");
    mInfomation.terminalAccountName = jo.optString("terminalAccountName");
    mInfomation.expectStarttime = jo.optString("expectStarttime");
    mInfomation.expectEndtime = jo.optString("expectEndtime");
    mInfomation.realStarttime = jo.optString("realStarttime");
    mInfomation.realEndtime = jo.optString("realEndtime");
    mInfomation.meetingName = jo.optString("meetingName");
    mInfomation.topic = jo.optString("topic");
    mInfomation.meetingId = jo.optInt("meetingId");
    mInfomation.numberOfsquares = jo.optInt("numberOfsquares");
    mInfomation.meetingStatus = jo.optInt("meetingStatus");
    mInfomation.meetingType = jo.optInt("meetingType");
    mInfomation.yyBeginTime = jo.optString("yyBeginTime");
    mInfomation.meetingHost = jo.optString("meetingHost");
    mInfomation.hasMeetingPwd = jo.optInt("hasMeetingPwd");
    return mInfomation;
    
  }
}
