package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.MeetingInvitationSMSInfo;
import cn.redcdn.log.CustomLog;

public class GetMeetingInvitationSMS extends AbstractBusinessDataSub<MeetingInvitationSMSInfo>{
  private String tag = GetMeetingInvitationSMS.class.getName();

  /**
   *  即时会议
   *  <p/>
   */
  public static  final String MEETINGTYPE_INSTANT  = "1";
  
  /**
   *  预约会议
   */
  public static  final String MEETINGTYPE_RESERVE = "2";
  
  /**
   *  产品类型为极会议
   */
  public static  final String MEETING_APP_JIHY = "JIHY";
  
  /**
   *  产品类型为可视极会议
   */
  public static  final String MEETING_APP_KESHI_JIHY = "KESHI_JIHY";
  
      
  /**
   * 获取会议邀请的短信内容
   * @param int meetingId
   *              会议编号
   * @param String inviterPhoneId
   *              会议邀请者视讯号
   * @param String inviterName
   *              会议邀请者昵称
   * @return 
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法
   */
  public int getMeetingInvitationSMS(int meetingId,String inviterPhoneId,String inviterName){
    CustomLog.i(tag, "call getMeetingInvitationSMS  meetingId = " + meetingId +" inviterPhoneId = "
        +inviterPhoneId +" inviterName= "+"inviterName");
    if(inviterPhoneId.equals("") || inviterName.equals("")|| (meetingId ==0)){
      CustomLog.e(tag, "输入参数不合法");
      return -5;
    }
    if(ConstConfig.getEditMeetingUrl() == null||ConstConfig.getEditMeetingUrl().equals("")){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_MEETINGID, meetingId);
      json.put(ConstConfig.PARAM_INVITERPHONEID, inviterPhoneId);
      json.put(ConstConfig.PARAM_INVITERNAME, inviterName);
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getEditMeetingUrl(), ConstConfig.PARAM_GETMEETINGINVITATIONSMS
          + jsonString);
    } catch (JSONException e) {
      CustomLog.e(tag, e.getMessage());
      return -3;
    }
  }
  
  /**
   * 获取预约会议邀请的短信内容
   * @param String app
   *            产品类型 默认空：极会议 (极会议: JIHY, 可视极会议：KESHI_JIHY)
   * @param String meetingType
   *            会议类型 默认即时会议。预约会议：MEETINGTYPE_RESERVE
   * @param int meetingId
   *           会议编号
   * @param String inviterPhoneId
   *           会议邀请者视讯号
   * @param String inviterName
   *          会议邀请者昵称
   * @return
   *          -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
   */
  public int getMeetingInvitationSMS(String app,String meetingType ,int meetingId,String inviterPhoneId,String inviterName){
    CustomLog.i(tag, "call getMeetingInvitationSMS meetingType = "+meetingType+" meetingId = " + meetingId +" inviterPhoneId = "
        +inviterPhoneId +" inviterName= "+"inviterName");
    if(inviterPhoneId.equals("") || inviterName.equals("")|| (meetingId ==0)){
      CustomLog.e(tag, "输入参数不合法");
      return -1;
    }
    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_APP, app);
      json.put(ConstConfig.PARAM_MEETINGTYPE, meetingType);
      json.put(ConstConfig.PARAM_MEETINGID, meetingId);
      json.put(ConstConfig.PARAM_INVITERPHONEID, inviterPhoneId);
      json.put(ConstConfig.PARAM_INVITERNAME, inviterName);
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getEditMeetingUrl(), ConstConfig.PARAM_GETMEETINGINVITATIONSMS
          + jsonString);
    } catch (JSONException e) {
      CustomLog.e(tag, e.getMessage());
      return -3;
    }
  }
  
  protected Parser getParser() {
    return new MeetingManageParser();
  }
  
  protected MeetingInvitationSMSInfo parseContentBody(JSONObject jo){
    
    if (jo == null) {
      CustomLog.e(tag, "GetMeetingInvitationSMS JSONObject == null");
    }
    MeetingInvitationSMSInfo info = new MeetingInvitationSMSInfo();
    info.invitationSMS = jo.optString("invitationSMS");
    info.yyURL = jo.optString("yyURL");
    return info;
    
  }
  
  
}
