package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.MeetingZBInvitationInfo;
import cn.redcdn.log.CustomLog;

public class GetMeetingZBInvitationContent extends AbstractBusinessDataSub<MeetingZBInvitationInfo>{
  
  private String tag = GetMeetingZBInvitationContent.class.getName();
  
  /**
   *  MEETING_APP_JIHY 极会议
   */
  public static final String  MEETING_APP_JIHY = "JIHY";
  
  /**
   *  MEETING_APP_KESHI_JIHY 可视极会议
   */
  public static final String  MEETING_APP_KESHI_JIHY = "KESHI_JIHY";
  
  /**
   *   MEETING_APP_BUTEL_CONSULTATION 红云会诊
   */
  public static final String  MEETING_APP_BUTEL_CONSULTATION  = "HVS";
  
  /**
   * 获取会议直播邀请的短信内容和会议二维码图片URL
   * @param meetingId
   *               会议编号
   * @param inviterPhoneId 
   *               会议邀请者视讯号
   * @param inviterName  
   *               会议邀请者昵称       
   * @return 
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法
   */

  public int getMeetingZBInvitationContent(int meetingId,String inviterPhoneId, String inviterName){
    CustomLog.i(tag, "Call getMeetingZBInvitationContent,meetingId = " + meetingId+" inviterPhoneId="+inviterPhoneId
        +" inviterName="+ inviterName);
    
    if((meetingId == 0) || (inviterPhoneId == null||inviterPhoneId.equals("")) || (inviterName == null||inviterName.equals(""))){
      CustomLog.e(tag,"getMeetingZBInvitationContent参数不合法");
      return -5;
    }
    if(ConstConfig.getCreateMeetingUrl() == null || ConstConfig.getCreateMeetingUrl().equals("")){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    
    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_MEETINGID,meetingId);
      json.put(ConstConfig.PARAM_INVITERPHONEID,inviterPhoneId);
      json.put(ConstConfig.PARAM_INVITERNAME,inviterName);
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getCreateMeetingUrl(),
          ConstConfig.PARAM_GETMEETINGZBINVITATIONCONTENT + jsonString);
    } catch (JSONException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
      return -3;
    }
    
  }
  
  /**   用于可视极会议直播中，使用不同的短信内容。
   * 
   * @param app  产品类型，默认空：极会议   <p>
   *             MEETING_APP_JIHY = "JIHY"    MEETING_APP_KESHI_JIHY = "KESHI_JIHY"
   * @param meetingId
   *             会议编号
   * @param inviterPhoneId
   *             会议邀请者视讯号
   * @param inviterName
   *             会议邀请者昵称       
   * 
   * @return
   */
  public int getMeetingZBInvitationContent(String app,int meetingId,String inviterPhoneId, String inviterName){
    CustomLog.i(tag, "Call getMeetingZBInvitationContent,meetingId = " + meetingId+" inviterPhoneId="+inviterPhoneId
        +" inviterName="+ inviterName);
    
    if((meetingId == 0) || (inviterPhoneId == null || inviterPhoneId.equals("")) || (inviterName == null || inviterName.equals(""))){
      CustomLog.e(tag,"getMeetingZBInvitationContent参数不合法");
      return -5;
    }
    if(ConstConfig.getCreateMeetingUrl().equals("")|| ConstConfig.getCreateMeetingUrl() == null){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    
    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_APP,app);
      json.put(ConstConfig.PARAM_MEETINGID,meetingId);
      json.put(ConstConfig.PARAM_INVITERPHONEID,inviterPhoneId);
      json.put(ConstConfig.PARAM_INVITERNAME,inviterName);
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getCreateMeetingUrl(),
          ConstConfig.PARAM_GETMEETINGZBINVITATIONCONTENT + jsonString);
    } catch (JSONException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
      return -3;
    }
    
  }
  
  protected MeetingZBInvitationInfo parseContentBody(JSONObject bodyObject){
    if (bodyObject == null) {
      CustomLog.e(tag, "MeetingZBInvitationInfo JSONObject == null");
    }
    MeetingZBInvitationInfo info = new MeetingZBInvitationInfo();
    info.invitationSMS = bodyObject.optString("invitationSMS");
    info.meetingPictureURL = bodyObject.optString("meetingPictureURL");
    info.zbURL = bodyObject.optString("zbURL");
    CustomLog.e(tag,"Test info: "+ info.toString());
    return info;
  }
  
  protected Parser getParser() {
    return new MeetingManageParser();
  }
  
  
}
