package cn.redcdn.datacenter.meetingmanage;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessDataBoth;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;

public abstract class GetNowMeetings extends
		AbstractBusinessDataBoth<List<GetMeetingInfo>> {
	private String tag = GetNowMeetings.class.getName();

	/**
	 * 获取被邀请参加的所有会议接口。返回会议ID,邀请人帐号，邀请人昵称，邀请时间,会议主持人meetingHost,
	 * 会议预期结束时间expectEndtime,会议是否有密码hasMeetingPwd;
	 * 
	 * @param token
	 *            有效的token
	 * 
	 * @return -1: 表示无效的操作，当前操作正在执行 -2: 表示无效的编码 -3: 表示json操作异常 -4: 接口服务器地址不合法
	 *         -5: 输入参数不合法 0:成功 -999:系统错误 -900:传入的json数据不正确 -901:参数不能为空
	 *         -902:Token不存在 -903:Token已过期 -909:service参数不能为空 -910:service参数不正确
	 */
	public int getNowMeetings(String token) {

		CustomLog.i(tag, "call getNowMeetings : token = " + token);

		if (token == null || token.equals("")) {
			CustomLog.e(tag, "token为空");
			return -5;
		}
		if (ConstConfig.getGetMeetingsUrl() == null
				|| ConstConfig.getGetMeetingsUrl().equals("")) {
			CustomLog.e(tag, "请求服务器地址为空");
			return -4;
		}
		try {
			JSONObject json = new JSONObject();
			json.put(ConstConfig.PARAM_TOKEN, token);

			String FirstRequestUrl = ConstConfig.getGetMeetingsUrl();
			String SecondRequestUrl = ConstConfig
					.getSlaveMeetingManagementHttpUrlPrefix()
					+ ConstConfig.bmsWebDomainSuffix;
			// 转义特殊字符 % --> %25
			String jsonString = java.net.URLEncoder.encode(json.toString());
			return exec(FirstRequestUrl, SecondRequestUrl,
					ConstConfig.PARAM_GETNOWMEETINGS + jsonString);

		} catch (Exception e) {
			CustomLog.e(tag, e.getMessage());
		}
		return -3;
	}

	protected List<GetMeetingInfo> parseContentBody(JSONObject jo)
			throws InvalidateResponseException {
		try {
			if (jo == null) {
				CustomLog.e(tag, "GetNowMeetings jo == null");
				throw new InvalidateResponseException();
			}
			CustomLog.d(tag, jo.toString());
			List<GetMeetingInfo> getMeetingInfoList = new ArrayList<GetMeetingInfo>();

			List<GetMeetingInfo> getConveneMeetingInfoList = new ArrayList<GetMeetingInfo>();
			List<GetMeetingInfo> getReserveMmeetingInfoList = new ArrayList<GetMeetingInfo>();

			GetMeetingInfo info = null;

			JSONArray jary = jo.optJSONArray("data");
			for (int i = 0; i < jary.length(); i++) {
				JSONObject obj = jary.getJSONObject(i);
				info = new GetMeetingInfo();
				ProcessInfo(obj, info);
				getMeetingInfoList.add(info);
				if (info.meetingType == 1) {
					getConveneMeetingInfoList.add(info);
				} else if (info.meetingType == 2) {
					getReserveMmeetingInfoList.add(info);
				}

			}

			int SizeOfData1 = getMeetingInfoList.size();

			JSONArray jary2 = jo.optJSONArray("data2");
			if (null != jary2 && jary2.length() > 0) {

				CustomLog.e(tag, "GetNowMeetings data2!=null && length>0");

				for (int i = 0; i < jary2.length(); i++) {
					JSONObject obj = jary2.getJSONObject(i);
					info = new GetMeetingInfo();
					ProcessInfo(obj, info);

					boolean matchFlag = false;
					// 如果meetingId和phoneId都相同，则进行去重处理
					for (int s = 0; s < SizeOfData1; s++) {

						if (info.meetingId == getMeetingInfoList.get(s)
								.getMeetingId()
								&& info.phoneId.equals(getMeetingInfoList
										.get(s).getPhoneId())) {
							matchFlag = true;
							break;
						}
					}

					if (!matchFlag) {

						if (info.meetingType == 1) {
							getConveneMeetingInfoList.add(info);
						} else if (info.meetingType == 2) {
							getReserveMmeetingInfoList.add(info);
						}

					}
				}

				Collections.sort(getConveneMeetingInfoList);
				Collections.sort(getReserveMmeetingInfoList);
				getMeetingInfoList.clear();

				for (int a = 0; a < getConveneMeetingInfoList.size(); a++) {
					getMeetingInfoList.add(getConveneMeetingInfoList.get(a));
				}

				for (int b = 0; b < getReserveMmeetingInfoList.size(); b++) {
					getMeetingInfoList.add(getReserveMmeetingInfoList.get(b));
				}

			} else {
				CustomLog.e(tag, "GetNowMeetings data2==null || length=0");
			}

			return getMeetingInfoList;

		} catch (JSONException e) {
			CustomLog.e(tag, " GetNowMeetings invalidate reponse");
			throw new InvalidateResponseException();
		}
	}

	private GetMeetingInfo ProcessInfo(JSONObject obj, GetMeetingInfo info) {
		// TODO Auto-generated method stub
		info.createTime = obj.optString("createTime");
		info.creatorName = obj.optString("creatorName");
		info.meetingId = obj.optInt("meetingId");
		info.phoneId = obj.optString("phoneId");
		info.topic = obj.optString("topic");
		info.meetingType = obj.optInt("meetingType");
		info.hasMeetingPwd = obj.optInt("hasMeetingPwd");
		info.meetingHost = obj.optString("meetingHost");
		info.expectEndtime = obj.optString("expectEndtime");
		return info;
	}

	protected Parser getParser() {
		return new GetNowMeetingsParser();
	}

}
