package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.DeviceResolutionType;
import cn.redcdn.log.CustomLog;

public abstract class GetTerminalInfo extends
AbstractBusinessDataSub<DeviceResolutionType> {

  private String tag = GetTerminalInfo.class.getName();

  /**
   * 获取终端类型参数信息。对应会议接口V2.14 13
   * 
   * @param terminal
   *          终端类型
   * @return 
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   */

  public int getterminalinfo(String terminal) {
    CustomLog.i(tag, "terminal: " + terminal);
    
    if(ConstConfig.getGetTerminalInfoUrl() == null || ConstConfig.getGetTerminalInfoUrl().equals("")){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    JSONObject json = new JSONObject();
    try {
      json.put(ConstConfig.PARAM_TERMINAL, terminal);
    } catch (JSONException e) {
      return -3;
    }
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
    return exec(ConstConfig.getGetTerminalInfoUrl(),
        ConstConfig.PARAM_GET_TERMINAL_INFO_ + jsonString);
  }

  protected DeviceResolutionType parseContentBody(JSONObject jo)
      throws InvalidateResponseException {

    try {
      if (jo == null) {
        CustomLog.e(tag, "GetTerminalInfo JSONObject == null");
        throw new InvalidateResponseException();
      }

      if (jo.getString("resolution").equalsIgnoreCase("qvga")) {

        return DeviceResolutionType.qvga;
      } else if (jo.getString("resolution").equalsIgnoreCase("vga")) {

        return DeviceResolutionType.vga;
      } else if (jo.getString("resolution").equalsIgnoreCase("720p")) {

        return DeviceResolutionType.hdpi;
      } else if (jo.getString("resolution").equalsIgnoreCase("1080p")) {

        return DeviceResolutionType.xhdpi;
      }
    } catch (JSONException e) {
      CustomLog.e(tag, "GetTerminalInfo invalidate reponse");
      throw new InvalidateResponseException();

    }
    CustomLog.e(tag, "GetTerminalInfo失败，默认返回vga类型");
    return DeviceResolutionType.vga;

  }



  protected Parser getParser() {
    return new MeetingManageParser();
  }
}
