package cn.redcdn.datacenter.meetingmanage;

import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.ResponseEmpty;
import cn.redcdn.log.CustomLog;

public abstract class ModifyMeetingInviters extends
AbstractBusinessDataSub<ResponseEmpty> {
	private String tag = ModifyMeetingInviters.class.getName();
	 /**
   *  产品类型为极会议
   */
  public static  final String MEETING_APP_JIHY = "JIHY";
  
  /**
   *  产品类型为可视极会议
   */
  public static  final String MEETING_APP_KESHI_JIHY = "KESHI_JIHY";
  
  
	/**
	 * 4.增加参会人员接口。服务器将返回返回码描述。
	 * @param token
	 *            有效的token
	 * @param meetingId
	 *            要编辑的会议编号
	 * @param invotedUsers
	 *            新增的邀请人列表
	 * @param invotedPhones
	 *            新增的邀请人手机号
	 * 
	 * @return
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法	 
	 */
	public int modifymeetingInviter(String token, int meetingId,
			List<String> invotedUsers, List<String> invotedPhones) {
	  String app = ModifyMeetingInviters.MEETING_APP_JIHY;
	 return  modifymeetingInviter(app,token,meetingId,
	      invotedUsers, invotedPhones);
	}
	
	 /**
  * 4.增加参会人员接口。服务器将返回返回码描述。
  * @param app
  *            产品类型 默认空：极会议(极会议: JIHY, 可视极会议：KESHI_JIHY)
  * @param token
  *            有效的token
  * @param meetingId
  *            要编辑的会议编号
  * @param invotedUsers
  *            新增的邀请人列表
  * @param invotedPhones
  *            新增的邀请人手机号
  * 
  * @return 
  *  -1: 表示无效的操作，当前操作正在执行
  *  -2: 表示无效的编码
  *  -3: 表示json操作异常
  *  -4: 接口服务器地址不合法
  *  -5: 输入参数不合法  
  */
 public int modifymeetingInviter(String app,String token, int meetingId,
     List<String> invotedUsers, List<String> invotedPhones) {
   CustomLog.i(tag, "call modifymeetingInviter token : " + token);
   
   if(invotedUsers == null||invotedPhones == null || (token== null||token.equals(""))){
     CustomLog.e(tag, "参数不合法");
     return -5;
   }
   if(ConstConfig.getmodifyMeetingInvitersUrl() == null||ConstConfig.getmodifyMeetingInvitersUrl().equals("")){
     CustomLog.e(tag, "请求服务器地址为空");
     return -4;
   }
   
   JSONObject json = new JSONObject();
   JSONArray ja1 = new JSONArray();
   JSONArray ja2 = new JSONArray();
   JSONObject jo = null;
   try {
     for (int i = 0; i < invotedUsers.size(); i++) {
       jo = new JSONObject();
       jo.put("phoneId", invotedUsers.get(i));
       ja1.put(jo);
     }
     for (int j = 0; j < invotedPhones.size(); j++) {
       ja2.put(invotedPhones.get(j));
     }
     json.put(ConstConfig.PARAM_APP, app);
     json.put(ConstConfig.PARAM_TOKEN, token);
     json.put(ConstConfig.PARAM_MEETINGID, meetingId);
     json.put(ConstConfig.PARAM_INVOTEDUSERS, ja1);
     json.put(ConstConfig.PARAM_INVOTEDPHONES, ja2);
     CustomLog
         .d(tag,
             "modifymeetingInviter success"
                 + ConstConfig.PARAM_MODIFYMEETINGINVITERS
                 + json.toString());
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
     return exec(ConstConfig.getmodifyMeetingInvitersUrl(),
         ConstConfig.PARAM_MODIFYMEETINGINVITERS + jsonString);
   } catch (Exception e) {
     CustomLog.e(tag, e.getMessage());
   }
   return -3;

 }
	
	 /**
   * 4.增加参会人员接口。服务器将返回返回码描述。
   * @param smsType
   *           指定短信发送需求为即时会议还是预约会议  (1：即时会议 2： 预约会议  不传默认为即时会议 )
   * @param app
   *            产品类型 默认空：极会议(极会议: JIHY, 可视极会议：KESHI_JIHY)
   * @param token
   *            有效的token
   * @param meetingId
   *            要编辑的会议编号
   * @param invotedUsers
   *            新增的邀请人列表
   * @param invotedPhones
   *            新增的邀请人手机号
   * 
   * @return 
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法  
   */
  public int modifymeetingInviter(int smsType,String app,String token, int meetingId,
      List<String> invotedUsers, List<String> invotedPhones) {
    CustomLog.i(tag, "call modifymeetingInviter token : " + token);
    
    if(invotedUsers == null||invotedPhones == null || (token== null||token.equals(""))||(smsType!=1&&smsType!=2)){
      CustomLog.e(tag, "参数不合法");
      return -5;
    }
    if(ConstConfig.getmodifyMeetingInvitersUrl() == null||ConstConfig.getmodifyMeetingInvitersUrl().equals("")){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    
    JSONObject json = new JSONObject();
    JSONArray ja1 = new JSONArray();
    JSONArray ja2 = new JSONArray();
    JSONObject jo = null;
    try {
      for (int i = 0; i < invotedUsers.size(); i++) {
        jo = new JSONObject();
        jo.put("phoneId", invotedUsers.get(i));
        ja1.put(jo);
      }
      for (int j = 0; j < invotedPhones.size(); j++) {
        ja2.put(invotedPhones.get(j));
      }
      json.put(ConstConfig.PARAM_SYSTYPE, smsType);
      json.put(ConstConfig.PARAM_APP, app);
      json.put(ConstConfig.PARAM_TOKEN, token);
      json.put(ConstConfig.PARAM_MEETINGID, meetingId);
      json.put(ConstConfig.PARAM_INVOTEDUSERS, ja1);
      json.put(ConstConfig.PARAM_INVOTEDPHONES, ja2);
      CustomLog
          .d(tag,
              "modifymeetingInviter success"
                  + ConstConfig.PARAM_MODIFYMEETINGINVITERS
                  + json.toString());
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getmodifyMeetingInvitersUrl(),
          ConstConfig.PARAM_MODIFYMEETINGINVITERS + jsonString);
    } catch (Exception e) {
      CustomLog.e(tag, e.getMessage());
    }
    return -3;

  }

	protected Parser getParser() {
		return new MeetingManageParser();
	}

}
