package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONException;

import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessDataDouble;
import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.ResetMeetingPwdInfo;
import cn.redcdn.log.CustomLog;

/**
 * ResetMeetingPwd 修改会议密码
 * @author Administrator
 *
 */
public class ResetMeetingPwd extends AbstractBusinessDataDouble<ResetMeetingPwdInfo> {
	
	protected final String TAG = getClass().getName();
	
	/**
	 * 修改会议密码接口（只有会议主持人能修改会议密码）
	 * 
	 * @param token（类型：String | 说明：有效的token）
	 * 
	 * @param meetingId（类型： int | 说明：会议号）
	 * 
	 * @param meetingPwd（类型：String | 说明：新的会议密码）
	 * 
	 * @return
	 *  -1: 表示无效的操作，当前操作正在执行
	 *  -2: 表示无效的编码
	 *  -3: 表示json操作异常
	 *  -4: 会议接口服务器地址不合法
	 *  -5: 输入参数不合法
	 *   0: 成功
	 *-999: 系统错误 
	 *-900: 传入的json数据不正确
	 *-901: 参数不能为空
	 *-902: Token不存在
	 *-903: Token已过期
	 *-906: 无效的会议No
	 *-909: service参数不能为空
	 *-910: service参数不正确 
	 *-927: 接口调用者不是会议主持人 
	 * 
	 */
	public int resetMeetingPwd(String token, int meetingId, String meetingPwd){
		
		CustomLog.i(TAG, "token=" + token + ",meetingId=" + meetingId + ",meetingPwd=" + meetingPwd);
		
		if(token == null || token.equals("")){
			
			CustomLog.e(TAG, "参数token为空");
			
			return -5;
			
		}
		
		if(String.valueOf(meetingId) == null || String.valueOf(meetingId).equals("")){
			
			CustomLog.e(TAG, "参数meetingId为空");
			
			return -5;
			
		}
		
		if(String.valueOf(meetingPwd) == null){
			
			CustomLog.e(TAG, "参数meetingPwd为空");
			
			return -5;
			
		}
		
		if(ConstConfig.getResetMeetingPwdUrl() == null || ConstConfig.getResetMeetingPwdUrl().equals("")){
			
			CustomLog.e(TAG, "请求服务器地址为空");
			
			return -4;
			
		}
		
		JSONObject json = new JSONObject();
		
		try{
			
			json.put(ConstConfig.PARAM_TOKEN, token);
			
			json.put(ConstConfig.PARAM_MEETINGID, meetingId);
			
			json.put(ConstConfig.PARAM_MEETINGPWD, meetingPwd);
			
			  //转义特殊字符   % --> %25
			String jsonString = java.net.URLEncoder.encode(json.toString());
			return exec(ConstConfig.getResetMeetingPwdUrl(),ConstConfig.getSlaveResetMeetingPwdUrl(), ConstConfig.PARAM_RESET_MEETING_PWD + jsonString);
			
		} catch (JSONException e) {
			
			return -3;
			
		}
		
	}
	
	  protected ResetMeetingPwdInfo parseContentBody(JSONObject jo)
		      throws InvalidateResponseException {
		    if (jo == null) {
		      CustomLog.e(TAG, "RESETMEETINGPWD JSONObject == null");
		      throw new InvalidateResponseException();
		    }
		    
		    ResetMeetingPwdInfo info = new ResetMeetingPwdInfo();
			info.rc = jo.optInt("rc");
			info.rd = jo.optString("rd");
		    return info;
		    
		  }
	
	protected Parser getParser() {
		
		return new CheckMeetingPwdParser();
		
	}

}
