package cn.redcdn.datacenter.meetingmanage;

import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessDataSub;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.ResponseEmpty;
import cn.redcdn.log.CustomLog;

public abstract class VerifyMeetingNo extends
AbstractBusinessDataSub<ResponseEmpty> {
  private String tag = VerifyMeetingNo.class.getName();

  /**
   * 检查会议编号是否存在接口。服务器将返回返回码描述。
   * 
   * @param token
   *          有效的token
   * @param meetingId
   *          需要查询的会议编号
   * 
   * @return
   *  -1: 表示无效的操作，当前操作正在执行
   *  -2: 表示无效的编码
   *  -3: 表示json操作异常
   *  -4: 接口服务器地址不合法
   *  -5: 输入参数不合法  
   */
  public int verifymeetingNo(String token, int meetingId) {
    CustomLog.i(tag, "call verifymeetingNo token : " + token);
    
    if(token == null ||token.equals("")){
      CustomLog.e(tag, "token不合法");
      return -5;
    }
    if(ConstConfig.getVerifyMeetingNoUrl() == null || ConstConfig.getVerifyMeetingNoUrl().equals("")){
      CustomLog.e(tag, "请求服务器地址为空");
      return -4;
    }
    try {
      JSONObject json = new JSONObject();

      json.put(ConstConfig.PARAM_TOKEN, token);
      json.put(ConstConfig.PARAM_MEETINGID, meetingId);

      CustomLog.d(tag, "verifymeetingNo success"
          + ConstConfig.PARAM_VERFYMEETINGNO + json.toString());
      
	  //转义特殊字符   % --> %25
	  String jsonString = java.net.URLEncoder.encode(json.toString());
      
      return exec(ConstConfig.getVerifyMeetingNoUrl(),
          ConstConfig.PARAM_VERFYMEETINGNO + jsonString);
    } catch (Exception e) {
      CustomLog.e(tag, e.getMessage());
    }
    return -3;

  }

  protected Parser getParser() {
    return new MeetingManageParser();
  }


}
