package cn.redcdn.datacenter.usercenter;

import org.json.JSONException;
import org.json.JSONObject;

import android.text.TextUtils;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.AbstractBusinessDataForAcquireAccountInfo;
import cn.redcdn.datacenter.BaseClass.AbstractBusinessDataBothSeq;
import cn.redcdn.datacenter.ExecResultCode;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.enterprisecenter.EnterpriseCenterParser;
import cn.redcdn.datacenter.enterprisecenter.data.DeviceType;
import cn.redcdn.log.CustomLog;


public abstract class AcquireAccountInfo extends AbstractBusinessDataBothSeq<JSONObject> {
    private static final String TAG = AcquireAccountInfo.class.getName();

    /**
     * @param imei       串号
     * @param deviceType 登录设备类型 详见cn.redcdn.datacenter.enterprisecenter.data.DeviceType
     * @return -1 表示无效的操作，当前操作正在执行
     * -2 表示无效的编码
     * -3 表示json操作异常
     */

    public int acquireAccountInfo(String imei, String deviceType) {
        try {
            JSONObject json = new JSONObject();
            json.put(ConstConfig.PARAM_IMEI, imei);
            json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
            CustomLog.d(TAG, "imei: " + ConstConfig.PARAM_IMEI + " deviceType: " + deviceType);
            //转义特殊字符   % --> %25
            String jsonString = java.net.URLEncoder.encode(json.toString());
            if (TextUtils.isEmpty(ConstConfig.slaveEnterPriseUserCenterWebDomain)) {
                return execEncode(ConstConfig.getAcquireAccountInfoUrl(), "", ConstConfig.PARAM_ENTMEMBERLOGIN
                        + jsonString);
            }
            return execEncode(ConstConfig.getAcquireAccountInfoUrl(), ConstConfig.getAcquireAccountInfoSlaveUrl(), ConstConfig.PARAM_ENTMEMBERLOGIN
                    + jsonString);
        } catch (Exception e) {
            CustomLog.e(TAG, e.getMessage());
            return -3;
        }
    }

    public int acquireAccountInfo(String token) {
        CustomLog.i(TAG, "acquireAccountInfo, token:" + token);

        if (TextUtils.isEmpty(token)) {
            CustomLog.e(TAG, "token 不能为空");
            return ExecResultCode.PARAM_ERROR;
        }

        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.PARAM_TOKEN, token);
        } catch (JSONException e) {
            return -3;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        if (TextUtils.isEmpty(ConstConfig.slaveEnterPriseUserCenterWebDomain)) {
            return execEncode(ConstConfig.getAcquireAccountInfoUrl(), "",ConstConfig.PARAM_ENTMEMBERLOGIN + jsonString);
        }
        return execEncode(ConstConfig.getAcquireAccountInfoUrl(), ConstConfig.getAcquireAccountInfoSlaveUrl(),ConstConfig.PARAM_ENTMEMBERLOGIN + jsonString);

    }

    protected JSONObject parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        try {
            if (jo == null) {
                CustomLog.e(TAG, "AcquireAccountInfo  invalidate reponse");
                throw new InvalidateResponseException();
            }
            CustomLog.d(TAG, jo.toString());

            return jo;
        } catch (Exception e) {
            CustomLog.e(TAG, e.toString());
            throw new InvalidateResponseException();
        }
    }


    protected Parser getParser() {
        return new EnterpriseCenterParser();
    }

}
