package cn.redcdn.datacenter.usercenter;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.usercenter.data.UserInfo;
import cn.redcdn.log.CustomLog;

public abstract class ActivateAccount extends AbstractBusinessData<UserInfo> {
	private String tag = ActivateAccount.class.getName();

	/**
	 * 激活帐号接口。激活成功，返回用户信息。对应《个人用户中心接口文档》1.6.5
	 * 
	 * @param account
	 *            需要激活的帐号。手机号或者nube号
	 * @param activateCode
	 *            激活码。registerAccount调用后手机接收到激活码，
	 * @param PRODUCTID
	 *            客户端类型:极会议版为 prod002
	 * 
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
	 */
	public int activateAccount(String account, String activateCode,
			String productId) {
		CustomLog.i(tag, "usercenter RegisterAccount");

		JSONObject json = new JSONObject();
		try {
			if (ConstConfig.getNeedtoaddAppkey()) {
				json.put(ConstConfig.ACCOUNT,ConstConfig.getAccount(account, ConstConfig.getAppkey()));
			}else {
				json.put(ConstConfig.ACCOUNT, account);
			}
			json.put(ConstConfig.ACTIVATECODE, activateCode);
			json.put(ConstConfig.PRODUCTID, productId);
		} catch (JSONException e) {
			return -3;
		}
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
		return exec(ConstConfig.getActivateAccount(),
				ConstConfig.PARAM_USERCNTER_ACTIVATEACCOUNT + jsonString);
	}
	
	/**
	 * 激活帐号接口。激活成功，返回用户信息。对应《个人用户中心接口文档》1.6.5
	 * 增加参数appKey
	 * 
	 * @param account
	 *            需要激活的帐号。手机号或者nube号
	 * @param activateCode
	 *            激活码。registerAccount调用后手机接收到激活码，
	 * @param PRODUCTID
	 *            客户端类型:极会议版为 prod002
	 * @param appKey
	 *            唯一标识           
	 * 
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
	 */
	public int activateAccount(String account, String activateCode,
			String productId,String appKey) {
		CustomLog.i(tag, "usercenter RegisterAccount");

		JSONObject json = new JSONObject();
		try {
			if (ConstConfig.getNeedtoaddAppkey()) {
				json.put(ConstConfig.ACCOUNT,ConstConfig.getAccount(account, ConstConfig.getAppkey()));
			}else {
				json.put(ConstConfig.ACCOUNT, account);
			}
			json.put(ConstConfig.ACTIVATECODE, activateCode);
			json.put(ConstConfig.PRODUCTID, productId);
			json.put(ConstConfig.PARAM_APPKEY, appKey);
		} catch (JSONException e) {
			return -3;
		}
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
		return exec(ConstConfig.getActivateAccount(),
				ConstConfig.PARAM_USERCNTER_ACTIVATEACCOUNT + jsonString);
	}

	protected UserInfo parseContentBody(JSONObject jo)
			throws InvalidateResponseException {
		if (jo == null) {
    	CustomLog.e(tag, "enterprise login invalidate reponse");
    	throw new InvalidateResponseException();
    }

    UserInfo info = new UserInfo();
    JSONObject userInfo = jo.optJSONObject("user");
    info.appType = userInfo.optString("appType");
    info.nubeNumber = userInfo.optString("nubeNumber");
    info.mobile = userInfo.optString("mobile");
    info.uid = userInfo.optString("uid");

    return info;
	}

	protected Parser getParser() {
		return new UserCenterParse();
	}
}
