package cn.redcdn.datacenter.usercenter;

import org.json.JSONObject;

import android.text.TextUtils;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.AbstractBusinessDataAuth;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.enterprisecenter.data.Users;
import cn.redcdn.log.CustomLog;

public abstract class Authorize extends AbstractBusinessDataAuth<Users> {

    private String tag = Authorize.class.getName();

    /**
     * @param account    可选项有：手机号、Email、固话
     * @param password   密码
     * @param imei       客户端串号,必填项
     * @param productid  产品标示 详见 cn.redcdn.datacenter.enterprisecenter.data.ProductId
     * @param deviceType 登录设备类型 详见cn.redcdn.datacenter.enterprisecenter.data.DeviceType
     * @return
     */
    public Users authorizeSync(String account, String password, String imei,
                               String productid, String deviceType) {
        CustomLog.i(tag, "usercenter authorizeSync");
        if (imei.equals("") || imei == null) {
            CustomLog.e(tag, " authorizeSync 串号不能为空");
        }
        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.PARAM_IMEI, imei);
            if (!TextUtils.isEmpty(account)) {
                if (ConstConfig.getNeedtoaddAppkey()) {
                    json.put(ConstConfig.ACCOUNT, ConstConfig.getAccount(account, ConstConfig.getAppkey()));
                } else {
                    json.put(ConstConfig.ACCOUNT, account);
                }
            }
            if (!TextUtils.isEmpty(password)) {
                json.put(ConstConfig.PASSWORD, password);
            }
            if (!TextUtils.isEmpty(productid)) {
                json.put(ConstConfig.PRODUCTID, productid);
            }
            if (!TextUtils.isEmpty(deviceType)) {
                json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
            }
        } catch (Exception e) {
            CustomLog.e(tag, "异常： " + e.getMessage());
            return null;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        String authUrl = ConstConfig.getAuthorizeUrl();
        CustomLog.d(tag, "1 ConstConfig.getAuthorizeUrl() = " + authUrl);
        if (authUrl.contains("|")) {
            int index = authUrl.indexOf("|");
            String filePath = authUrl.substring(index + 1);
            String url = authUrl.substring(0, index);
            CustomLog.d(tag, "1 url = " + url + " filePath = " + filePath);
            return execSyncWithHttps(url, ConstConfig.PARAM_AUTHORIZE
                    + jsonString, filePath);
        } else {
            return execSync(authUrl, ConstConfig.PARAM_AUTHORIZE
                    + jsonString);
        }
    }

    public Users authorizeSync(String account, String password, String imei,
                               String productid, String deviceType, String filePath) {
        CustomLog.i(tag, "usercenter authorizeSync");
        if (imei.equals("") || imei == null) {
            CustomLog.e(tag, " authorizeSync 串号不能为空");
        }
        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.PARAM_IMEI, imei);
            if (!TextUtils.isEmpty(account)) {
                if (ConstConfig.getNeedtoaddAppkey()) {
                    json.put(ConstConfig.ACCOUNT, ConstConfig.getAccount(account, ConstConfig.getAppkey()));
                } else {
                    json.put(ConstConfig.ACCOUNT, account);
                }
            }
            if (!TextUtils.isEmpty(password)) {
                json.put(ConstConfig.PASSWORD, password);
            }
            if (!TextUtils.isEmpty(productid)) {
                json.put(ConstConfig.PRODUCTID, productid);
            }
            if (!TextUtils.isEmpty(deviceType)) {
                json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
            }
        } catch (Exception e) {
            CustomLog.e(tag, "异常： " + e.getMessage());
            return null;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        String authUrl = ConstConfig.getAuthorizeUrl();
        CustomLog.d(tag, "ConstConfig.getAuthorizeUrl() = " + authUrl);
        if (filePath == null || filePath.equals("")) {
            return execSync(authUrl, ConstConfig.PARAM_AUTHORIZE
                    + jsonString);
        } else {
            return execSyncWithHttps(authUrl, ConstConfig.PARAM_AUTHORIZE
                    + jsonString, filePath);
        }
    }

    //增加appKey appType参数
    public Users authorizeSync(String account, String password, String imei,
                               String productid, String deviceType, String appkey, String appType) {
        CustomLog.i(tag, "usercenter authorizeSync");
        if (imei.equals("") || imei == null) {
            CustomLog.e(tag, " authorizeSync 串号不能为空");
        }
        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.PARAM_IMEI, imei);
            if (!TextUtils.isEmpty(account)) {
                if (!TextUtils.isEmpty(appkey)) {
                    account = account + "|" + appkey;
                    json.put(ConstConfig.ACCOUNT, account);
                } else {
                    json.put(ConstConfig.ACCOUNT, account);
                }
            }
            if (!TextUtils.isEmpty(password)) {
                json.put(ConstConfig.PASSWORD, password);
            }
            if (!TextUtils.isEmpty(productid)) {
                json.put(ConstConfig.PRODUCTID, productid);
            }
            if (!TextUtils.isEmpty(deviceType)) {
                json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
            }
            if (!TextUtils.isEmpty(appkey)) {
                json.put(ConstConfig.PARAM_APPKEY, appkey);
            }
            if (!TextUtils.isEmpty(appType)) {
                json.put("appType", appType);
            }

        } catch (Exception e) {
            CustomLog.e(tag, "异常： " + e.getMessage());
            return null;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        String authUrl = ConstConfig.getAuthorizeUrl();
        CustomLog.d(tag, "2 ConstConfig.getAuthorizeUrl() = " + authUrl);
        if (authUrl.contains("|")) {
            int index = authUrl.indexOf("|");
            String filePath = authUrl.substring(index + 1);
            String url = authUrl.substring(0, index);
            CustomLog.d(tag, "2 url = " + url + " filePath = " + filePath);
            return execSyncWithHttps(url, ConstConfig.PARAM_AUTHORIZE
                    + jsonString, filePath);
        } else {
            return execSync(authUrl, ConstConfig.PARAM_AUTHORIZE
                    + jsonString);
        }
    }

    public Users authorizeSync(String account, String password, String imei,
                               String productid, String deviceType, String appkey, String appType, String captchaId, String imgCode) {
        CustomLog.i(tag, "usercenter authorizeSync");
        if (imei.equals("") || imei == null) {
            CustomLog.e(tag, " authorizeSync 串号不能为空");
        }
        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.PARAM_IMEI, imei);
            if (!TextUtils.isEmpty(account)) {
                if (!TextUtils.isEmpty(appkey)) {
                    account = account + "|" + appkey;
                    json.put(ConstConfig.ACCOUNT, account);
                } else {
                    json.put(ConstConfig.ACCOUNT, account);
                }
            }
            if (!TextUtils.isEmpty(password)) {
                json.put(ConstConfig.PASSWORD, password);
            }
            if (!TextUtils.isEmpty(productid)) {
                json.put(ConstConfig.PRODUCTID, productid);
            }
            if (!TextUtils.isEmpty(deviceType)) {
                json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
            }
            if (!TextUtils.isEmpty(appkey)) {
                json.put(ConstConfig.PARAM_APPKEY, appkey);
            }
            if (!TextUtils.isEmpty(appType)) {
                json.put("appType", appType);
            }
            if (!TextUtils.isEmpty(captchaId)) {
                json.put("captchaId", captchaId);
            }
            if (!TextUtils.isEmpty(imgCode)) {
                json.put("imgCode", imgCode);
            }

        } catch (Exception e) {
            CustomLog.e(tag, "异常： " + e.getMessage());
            return null;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        String authUrl = ConstConfig.getAuthorizeUrl();
        CustomLog.d(tag, "2 ConstConfig.getAuthorizeUrl() = " + authUrl);
        if (authUrl.contains("|")) {
            int index = authUrl.indexOf("|");
            String filePath = authUrl.substring(index + 1);
            String url = authUrl.substring(0, index);
            CustomLog.d(tag, "2 url = " + url + " filePath = " + filePath);
            return execSyncWithHttps(url, ConstConfig.PARAM_AUTHORIZE
                    + jsonString, filePath);
        } else {
            return execSync(authUrl, ConstConfig.PARAM_AUTHORIZE
                    + jsonString);
        }
    }

    public int authorizeAsyn(String account, String password, String imei,
                               String productid, String deviceType, String appkey, String appType, String captchaId, String imgCode) {
        CustomLog.i(tag, "usercenter authorizeAsyn");
        if (imei.equals("") || imei == null) {
            CustomLog.e(tag, " authorizeAsyn 串号不能为空");
        }
        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.PARAM_IMEI, imei);
            if (!TextUtils.isEmpty(account)) {
                if (!TextUtils.isEmpty(appkey)) {
                    account = account + "|" + appkey;
                    json.put(ConstConfig.ACCOUNT, account);
                } else {
                    json.put(ConstConfig.ACCOUNT, account);
                }
            }
            if (!TextUtils.isEmpty(password)) {
                json.put(ConstConfig.PASSWORD, password);
            }
            if (!TextUtils.isEmpty(productid)) {
                json.put(ConstConfig.PRODUCTID, productid);
            }
            if (!TextUtils.isEmpty(deviceType)) {
                json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
            }
            if (!TextUtils.isEmpty(appkey)) {
                json.put(ConstConfig.PARAM_APPKEY, appkey);
            }
            if (!TextUtils.isEmpty(appType)) {
                json.put("appType", appType);
            }
            if (!TextUtils.isEmpty(captchaId)) {
                json.put("captchaId", captchaId);
            }
            if (!TextUtils.isEmpty(imgCode)) {
                json.put("imgCode", imgCode);
            }

        } catch (Exception e) {
            CustomLog.e(tag, "异常： " + e.getMessage());
            return -1;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        String authUrl = ConstConfig.getAuthorizeUrl();
        CustomLog.d(tag, "2 ConstConfig.getAuthorizeUrl() = " + authUrl);
        if (authUrl.contains("|")) {
            int index = authUrl.indexOf("|");
            String filePath = authUrl.substring(index + 1);
            String url = authUrl.substring(0, index);
            CustomLog.d(tag, "2 url = " + url + " filePath = " + filePath);

            return exec(authUrl,ConstConfig.PARAM_AUTHORIZECAPTCHA
                    + jsonString);
        } else {

            return exec(authUrl,ConstConfig.PARAM_AUTHORIZECAPTCHA
                    + jsonString);
        }
    }

    protected Users parseContentBody(JSONObject jo) {

        if (jo == null) {
            CustomLog.e(tag, "usercenter authorizeSync invalidate reponse");
        }
        CustomLog.d(tag, jo.toString());
        Users us = new Users();
        us.status = Integer.parseInt(jo.optString("status"));
        us.message = jo.optString("message");

        if (jo.optString("status").equals("0")) {
            JSONObject userObj = jo.optJSONObject("user");
            us.appType = userObj.optString("appType");
            us.uid = userObj.optString("uid");
            us.imei = userObj.optString("imei");
            us.nubeNumber = userObj.optString("nubeNumber");
            us.mobile = userObj.optString("mobile");
            us.nickName = userObj.optString("nickname");
            us.mail = userObj.optString("mail");
            us.accessToken = userObj.optString("accessToken");
            if (userObj.optBoolean("allowedRecommend")) {
                us.allowedRecommend = "true";
            } else
                us.allowedRecommend = "false";

            us.headUrl = userObj.optString("headUrl");
            us.productId = userObj.optString("productId");
            us.blockStrangeCall = userObj.optString("blockStrangeCall");
            us.prod002 = userObj.optString("prod002");

            us.deviceType = userObj.optString("deviceType");
            us.isEnableMeeting = userObj.optInt("isEnableMeeting");
            us.isEnableQueue = userObj.optInt("isEnableQueue");
            us.isEnableCharMessage = userObj.optInt("isEnableCharMessage");
            us.isEnableShortVideo = userObj.optInt("isEnableShortVideo");
            us.isEnablePicture = userObj.optInt("isEnablePicture");
            us.isEnableCard = userObj.optInt("isEnableCard");
            us.changPwdFlg = userObj.optInt("changPwdFlg");
        } else {
            CustomLog.e(tag, "status不为0");
        }
        return us;

    }

    protected Parser getParser() {
        return new UserCenterParse();
    }

}
