package cn.redcdn.datacenter.usercenter;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.usercenter.data.UserInfo;
import cn.redcdn.log.CustomLog;

public abstract class ChangePassword extends AbstractBusinessData<UserInfo> {
    private String tag = ChangePassword.class.getName();

    /**
     * 修改密码接口。修改密码成功，返回用户信息。对应《个人用户中心接口文档》1.6.7
     *
     * @param newPassword 新密码，newPassword需要为MD5加密之后的。
     * @param accessToken 可用的accessToken
     * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
     */
    public int changePassword(String newPassword, String accessToken) {
        CustomLog.i(tag, "usercenter changePassword, newPassword:"
                + newPassword + "accessToken:" + accessToken);

        JSONObject json = new JSONObject();
        try {
            json.put(ConstConfig.NEWPASSWORD, newPassword);
            json.put(ConstConfig.ACCESS_TOKEN, accessToken);
        } catch (JSONException e) {
            return -3;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        return exec(ConstConfig.getChangePassword(),
                ConstConfig.PARAM_USERCNTER_CHANGEPASSWORD + jsonString
                        + "&accessToken=" + accessToken);
    }

    public int changePassword(String oldPassword, String newPassword, String accessToken) {
        CustomLog.i(tag, "usercenter changePassword, oldPassword:" + oldPassword + " newPassword:"
                + newPassword + " accessToken:" + accessToken);

        JSONObject json = new JSONObject();
        try {
            json.put("oldPassword", oldPassword);
            json.put(ConstConfig.NEWPASSWORD, newPassword);
//            json.put(ConstConfig.ACCESS_TOKEN, accessToken);
        } catch (JSONException e) {
            return -3;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        return exec(ConstConfig.getChangePassword(),
                ConstConfig.PARAM_USERCNTER_NEW_CHANGEPASSWORD
                        + "&accessToken=" + accessToken
                        + "&params=" + jsonString );
    }

    protected UserInfo parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        if (jo == null) {
            CustomLog
                    .e(tag, "usercenter changePassword invalidate reponse");
            throw new InvalidateResponseException();
        }
        UserInfo info = new UserInfo();
        JSONObject userInfo = jo.optJSONObject("user");

        info.appType = userInfo.optString("appType");
        info.nubeNumber = userInfo.optString("nubeNumber");
        info.mobile = userInfo.optString("mobile");
        info.uid = userInfo.optString("uid");

        return info;
    }

    protected Parser getParser() {
        return new UserCenterParse();
    }
}
