package cn.redcdn.datacenter.usercenter;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.usercenter.data.NubeNumberInfo;
import cn.redcdn.log.CustomLog;

public abstract class GetNubeNumberList extends
		AbstractBusinessData<NubeNumberInfo> {
	private String tag = GetNubeNumberList.class.getName();
		
	/**
	 * userType
	 * 1:外部用户
	 */
	public static final int USERTYPE_OUTER = 1;
	/**
	 * userType
	 * 2:内部用户
	 */
	public static final int USERTYPE_INNER = 2;
	/**
	 * userType
	 * 3:公免用户
	 */
	public static final int USERTYPE_PUBLIC = 3;
	/**
	 * userType
	 * 4:专业会议用户
	 */
	public static final int USERTYPE_PROFESSIONAL = 4;
	/**
	 * userType
	 * 5:第三方企业用户
	 */
	public static final int USERTYPE_THIRDPARTY = 5;
	/**
	 * userType
	 * 6:极会议用户
	 */
	public static final int USERTYPE_JIHUIYI = 6;
	/**
	 * userType
	 * 7:未知
	 */
	public static final int USERTYPE_UNKNOWN = 7;
	
	/**
	 * 获取视讯号接口，注册之前调用。服务器将返回一个待注册的视讯号。 对应接口 《个人用户中心》 1.6.12
	 * 
	 * @param imei
	 *            手机imei值，每个手机该值都是固定的
	 * @param appType
	 *            固定值
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
	 */
	public int getNubeNumberList(String imei, String appType) {
		CustomLog.i(tag, "usercenter getNubeNumberList, imei:" + imei);

		JSONObject json = new JSONObject();
		try {
			json.put(ConstConfig.APPTYPE, appType);
			json.put(ConstConfig.PARAM_IMEI, imei);
		  json.put(ConstConfig.USERTYPE, 6);
		} catch (JSONException e) {
			return -3;
		}
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
		return exec(ConstConfig.getGetNubeNumberList(),
				ConstConfig.PARAM_USERCNTER_GETNUBENUMBERLIST + jsonString);
	}
	
	public int getNubeNumberList(String imei, String appType, String appKey, int userType) {
		CustomLog.i(tag, "usercenter getNubeNumberList, imei:" + imei);
		int num = 1;
		return getNubeNumberList(imei,appType,appKey,num,userType);
	}
	
	public int getNubeNumberList(String imei, String appType, int num, int userType) {
		CustomLog.i(tag, "usercenter getNubeNumberList, imei:" + imei);
		String appKey = "";
		return getNubeNumberList(imei,appType,appKey,num,userType);
	}
	
	/**
	 * 获取视讯号接口，注册之前调用。服务器将返回一个待注册的视讯号。 对应接口 《个人用户中心》 1.6.12
	 * 
	 * @param imei
	 *            手机imei值，每个手机该值都是固定的
	 * @param appType
	 *            固定值
	 * @param appKey     
	 *            唯一标示 
	 * @param num
	 *           取值范围为1到10。如果该值缺省或大于10时，则为10。如果该值小于1，返回错误信息
	 * @param userType
	 *           用户类型 ： 1:外部用户                 GetNubeNumberList.USERTYPE_OUTER    
	 *                  2:内部用户                 GetNubeNumberList.USERTYPE_INNER
	 *                  3:公免用户                 GetNubeNumberList.USERTYPE_PUBLIC
	 *                  4:专业会议用户        GetNubeNumberList.USERTYPE_PROFESSIONAL
	 *                  5:第三方企业用户  GetNubeNumberList.USERTYPE_THIRDPARTY
	 *                  6:极会议用户            GetNubeNumberList.USERTYPE_JIHUIYI
	 *                  7:未知                         GetNubeNumberList.USERTYPE_UNKNOWN
	 *       
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
	 */
	public int getNubeNumberList(String imei, String appType, String appKey, int num, int userType) {
		CustomLog.i(tag, "usercenter getNubeNumberList, imei:" + imei);

		if(num<1){
			CustomLog.e(tag,"num不能小于1");
			return -5;
		}
		
		if(num >10){
			CustomLog.e(tag,"num大于10时,取num等于10");
			num = 10;
		}
		
		JSONObject json = new JSONObject();
		try {
			json.put(ConstConfig.APPTYPE, appType);
			json.put(ConstConfig.PARAM_IMEI, imei);
			json.put(ConstConfig.PARAM_APPKEY, appKey);
			json.put(ConstConfig.PARAM_NUM, num);
		  json.put(ConstConfig.USERTYPE, userType);
		} catch (JSONException e) {
			return -3;
		}
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
		return exec(ConstConfig.getGetNubeNumberList(),
				ConstConfig.PARAM_USERCNTER_GETNUBENUMBERLIST + jsonString);
	}

	protected NubeNumberInfo parseContentBody(JSONObject jo)
			throws InvalidateResponseException {
		try {
			if (jo == null) {
				CustomLog.e(tag, "enterprise login invalidate reponse");
				throw new InvalidateResponseException();
			}
			CustomLog.d(tag, jo.toString());
			NubeNumberInfo info = new NubeNumberInfo();

			JSONObject nubeNumberInfo = jo.optJSONObject("nubeNumberInfo");
			info.token = nubeNumberInfo.optString("transId");
			JSONArray ja = nubeNumberInfo.getJSONArray("nubeNumberlist");
			info.nubeNumber = ja.getString(0);

			return info;
		} catch (JSONException e) {
			CustomLog.e(tag, "usercenter getNubeNumberList invalidate reponse");
			throw new InvalidateResponseException();
		}
	}

	protected Parser getParser() {
		return new UserCenterParse();
	}
}
