package cn.redcdn.datacenter.usercenter;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.meetingmanage.data.ResponseEmpty;
import cn.redcdn.log.CustomLog;

public abstract class ReSendActivateCode extends
		AbstractBusinessData<ResponseEmpty> {
	private String tag = ReSendActivateCode.class.getName();
	
	/**
   * ProductType_SPSY: 视频兽医
   */
  public static final String ProductType_SPSY = "spsy";
  /**
   * ProductType_HVS: 红云医疗
   */
  public static final String ProductType_HVS = "hvs";
  /**
	 * ProductType_JHY: 极会议
	 * 
	 */
	public static final String  ProductType_JHY = "JHY_MOBILE";
	/**
	 * ProductType_ZGHY:紫光红云 
	 */
	public static final String  ProductType_ZGHY = "zghy";
	
  
  
	/**
	 * 重新发送激活码接口，结果码返回。对应《个人用户中心接口文档》1.6.6
	 * 
	 * @param account
	 *            注册的帐号
	 * @param productId
	 *            "prod002"
	 * 
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
	 */
	public int reSendActivateCode(String account, String productId) {
		CustomLog.i(tag, "usercenter reSendActivateCode, account:" + account
				+ "productId:" + productId);

		JSONObject json = new JSONObject();
		try {
			json.put(ConstConfig.ACCOUNT, account);
			json.put(ConstConfig.PRODUCTID, productId);
		} catch (JSONException e) {
			return -3;
		}
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
		return exec(
				ConstConfig.getReSendActivateCode(),
				ConstConfig.PARAM_USERCNTER_RESENDACTVIATECODE
						+ jsonString);
	}
	
	
	/**增加productType参数，提供给手机版hvs 项目
	 * 
	 * @param account
	 * @param productId
	 * @param productType
	 * @return
	 */
	 public int reSendActivateCode(String account, String productId,String productType) {
	    CustomLog.i(tag, "usercenter reSendActivateCode, account:" + account
	        + "productId:" + productId);

	    JSONObject json = new JSONObject();
	    try {
	      json.put(ConstConfig.ACCOUNT, account);
	      json.put(ConstConfig.PRODUCTID, productId);
	      json.put(ConstConfig.PARAM_PRODUCTYPE, productType);
	    } catch (JSONException e) {
	      return -3;
	    }
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
	    return exec(
	        ConstConfig.getReSendActivateCode(),
	        ConstConfig.PARAM_USERCNTER_RESENDACTVIATECODE
	            + jsonString);
	  }
	 
		/**增加appKey参数
		 * 
		 * @param account
		 * @param productId
		 * @param productType
		 * @param appKey
		 * @return
		 */
		 public int reSendActivateCode(String account, String productId,String productType,String appKey) {
		    CustomLog.i(tag, "usercenter reSendActivateCode, account:" + account
		        + "productId:" + productId);

		    JSONObject json = new JSONObject();
		    try {
		    	if (ConstConfig.getNeedtoaddAppkey()) {
					json.put(ConstConfig.ACCOUNT,ConstConfig.getAccount(account, ConstConfig.getAppkey()));
				}else {
					json.put(ConstConfig.ACCOUNT, account);
				}
		      json.put(ConstConfig.PRODUCTID, productId);
		      json.put(ConstConfig.PARAM_PRODUCTYPE, productType);
		      json.put(ConstConfig.PARAM_APPKEY, appKey);
		    } catch (JSONException e) {
		      return -3;
		    }
			  //转义特殊字符   % --> %25
			String jsonString = java.net.URLEncoder.encode(json.toString());
		    return exec(
		        ConstConfig.getReSendActivateCode(),
		        ConstConfig.PARAM_USERCNTER_RESENDACTVIATECODE
		            + jsonString);
		  }
		 
			/**增加appKey参数、dynamicUrl参数
			 * 
			 * @param account
			 * @param productId
			 * @param productType
			 * @param appKey
			 * @param dynamicUrl
			 * @return
			 */
			 public int reSendActivateCode(String account, String productId,String productType,String appKey,String dynamicUrl) {
			    CustomLog.i(tag, "usercenter reSendActivateCode, account:" + account
			        + "productId:" + productId);

			    JSONObject json = new JSONObject();
			    try {
			    	if (ConstConfig.getNeedtoaddAppkey()) {
						json.put(ConstConfig.ACCOUNT,ConstConfig.getAccount(account, ConstConfig.getAppkey()));
					}else {
						json.put(ConstConfig.ACCOUNT, account);
					}
			      json.put(ConstConfig.PRODUCTID, productId);
			      json.put(ConstConfig.PARAM_PRODUCTYPE, productType);
			      json.put(ConstConfig.PARAM_APPKEY, appKey);
			      json.put(ConstConfig.PARAM_DYNAMICURL, dynamicUrl);
			    } catch (JSONException e) {
			      return -3;
			    }
				  //转义特殊字符   % --> %25
				String jsonString = java.net.URLEncoder.encode(json.toString());
			    return exec(
			        ConstConfig.getReSendActivateCode(),
			        ConstConfig.PARAM_USERCNTER_RESENDACTVIATECODE
			            + jsonString);
			  }
		 	 
	protected Parser getParser() {
		return new UserCenterParse();
	}

}
