package cn.redcdn.datacenter.usercenter;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;
import cn.redcdn.datacenter.meetingmanage.data.ResponseEmpty;

public abstract class RegisterAccount extends
		AbstractBusinessData<ResponseEmpty> {
	private String tag = RegisterAccount.class.getName();
	
	/**
	 * ProductType_SPSY: 视频兽医
	 */
	public static final String ProductType_SPSY = "spsy";
	/**
	 * ProductType_HVS: 红云医疗
	 */
	public static final String ProductType_HVS = "hvs";
	/**
	 * ProductType_JHY: 极会议
	 * 
	 */
	public static final String  ProductType_JHY = "JHY_MOBILE";
	/**
	 * ProductType_ZGHY:紫光红云 
	 */
	public static final String  ProductType_ZGHY = "zghy";
	
	
	
	/**
	 * 注册接口。注册成功，返回用户信息，同时会收到激活码。对应《个人用户中心接口文档》1.6.4
	 * 
	 * @param appType
	 *            mobile
	 * @param account
	 *            注册的帐号
	 * @param password
	 *            密码
	 * @param nubeNumber
	 *            nube号
	 * @param imei
	 *            mobinp_JIHY
	 * @param productId
	 *            "prod002"
	 * @param deviceType
	 *           登录设备类型 详见cn.redcdn.datacenter.enterprisecenter.data.DeviceType
	 * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
	 */
	public int registerAccount(String appType, String account, String password,
			String nubeNumber, String imei, String productId ,String deviceType) {
		CustomLog.i(tag, "usercenter RegisterAccount, imei:" + imei+ " deviceType: "+deviceType);

		JSONObject json = new JSONObject();
		try {
			json.put(ConstConfig.APPTYPE, appType);
			if (ConstConfig.getNeedtoaddAppkey()) {
				json.put(ConstConfig.ACCOUNT,ConstConfig.getAccount(account, ConstConfig.getAppkey()));
			}else {
				json.put(ConstConfig.ACCOUNT, account);
			}
			json.put(ConstConfig.PASSWORD, password);
			json.put(ConstConfig.NUBENUMBER, nubeNumber);
			json.put(ConstConfig.PARAM_IMEI, imei);
			json.put(ConstConfig.PRODUCTID, productId);
			json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
		} catch (JSONException e) {
			return -3;
		}
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
		return exec(ConstConfig.getRegisterAccount(),
				ConstConfig.PARAM_USERCNTER_REGISTERACCOUNT + jsonString);
	}


	/**增加productType参数，提供给手机版hvs 项目
	 * 
	 * @param appType
	 * @param account
	 * @param password
	 * @param nubeNumber
	 * @param imei
	 * @param productId
	 * @param deviceType
	 * @param productType
	 * @return
	 */
	 public int registerAccount(String appType, String account, String password,
	      String nubeNumber, String imei, String productId ,String deviceType,String productType) {
	    CustomLog.i(tag, "usercenter RegisterAccount, imei:" + imei+ " deviceType: "+deviceType);

	    JSONObject json = new JSONObject();
	    try {
	      json.put(ConstConfig.APPTYPE, appType);
	      if (ConstConfig.getNeedtoaddAppkey()) {
				json.put(ConstConfig.ACCOUNT,ConstConfig.getAccount(account, ConstConfig.getAppkey()));
			}else {
				json.put(ConstConfig.ACCOUNT, account);
			}
	      json.put(ConstConfig.PASSWORD, password);
	      json.put(ConstConfig.NUBENUMBER, nubeNumber);
	      json.put(ConstConfig.PARAM_IMEI, imei);
	      json.put(ConstConfig.PRODUCTID, productId);
	      json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
	      json.put(ConstConfig.PARAM_PRODUCTYPE, productType);
	    } catch (JSONException e) {
	      return -3;
	    }
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(json.toString());
	    return exec(ConstConfig.getRegisterAccount(),
	        ConstConfig.PARAM_USERCNTER_REGISTERACCOUNT + jsonString);
	  }
	 
	 
		/**增加appKey参数
		 * 
		 * @param appType
		 * @param account
		 * @param password
		 * @param nubeNumber
		 * @param imei
		 * @param productId
		 * @param deviceType
		 * @param productType
		 * @param appKey
		 * @return
		 */
		 public int registerAccount(String appType, String account, String password,
		      String nubeNumber, String imei, String productId ,String deviceType,String productType,String appKey) {
		    CustomLog.i(tag, "usercenter RegisterAccount, imei:" + imei+ " deviceType: "+deviceType);

		    JSONObject json = new JSONObject();
		    try {
		      json.put(ConstConfig.APPTYPE, appType);
		      if (ConstConfig.getNeedtoaddAppkey()) {
					json.put(ConstConfig.ACCOUNT,ConstConfig.getAccount(account, ConstConfig.getAppkey()));
				}else {
					json.put(ConstConfig.ACCOUNT, account);
				}
		      json.put(ConstConfig.PASSWORD, password);
		      json.put(ConstConfig.NUBENUMBER, nubeNumber);
		      json.put(ConstConfig.PARAM_IMEI, imei);
		      json.put(ConstConfig.PRODUCTID, productId);
		      json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
		      json.put(ConstConfig.PARAM_PRODUCTYPE, productType);
		      json.put(ConstConfig.PARAM_APPKEY, appKey);
		    } catch (JSONException e) {
		      return -3;
		    }
			  //转义特殊字符   % --> %25
			String jsonString = java.net.URLEncoder.encode(json.toString());
		    return exec(ConstConfig.getRegisterAccount(),
		        ConstConfig.PARAM_USERCNTER_REGISTERACCOUNT + jsonString);
		  }
		 
			/**增加appKey参数、dynamicUrl参数
			 * 
			 * @param appType
			 * @param account
			 * @param password
			 * @param nubeNumber
			 * @param imei
			 * @param productId
			 * @param deviceType
			 * @param productType
			 * @param appKey
			 * @param dynamicUrl
			 * @return
			 */
			 public int registerAccount(String appType, String account, String password,
			      String nubeNumber, String imei, String productId ,String deviceType,String productType,
			      String appKey,String dynamicUrl) {
			    CustomLog.i(tag, "usercenter RegisterAccount, imei:" + imei+ " deviceType: "+deviceType);

			    JSONObject json = new JSONObject();
			    try {
			      json.put(ConstConfig.APPTYPE, appType);
			      if (ConstConfig.getNeedtoaddAppkey()) {
						json.put(ConstConfig.ACCOUNT,ConstConfig.getAccount(account, ConstConfig.getAppkey()));
					}else {
						json.put(ConstConfig.ACCOUNT, account);
					}
			      json.put(ConstConfig.PASSWORD, password);
			      json.put(ConstConfig.NUBENUMBER, nubeNumber);
			      json.put(ConstConfig.PARAM_IMEI, imei);
			      json.put(ConstConfig.PRODUCTID, productId);
			      json.put(ConstConfig.PARAM_DEVICETYPE, deviceType);
			      json.put(ConstConfig.PARAM_PRODUCTYPE, productType);
			      json.put(ConstConfig.PARAM_APPKEY, appKey);
			      json.put(ConstConfig.PARAM_DYNAMICURL, dynamicUrl);
			    } catch (JSONException e) {
			      return -3;
			    }
				  //转义特殊字符   % --> %25
				String jsonString = java.net.URLEncoder.encode(json.toString());
			    return exec(ConstConfig.getRegisterAccount(),
			        ConstConfig.PARAM_USERCNTER_REGISTERACCOUNT + jsonString);
			  }
	 
private String getAccount(String account,String appkey){
				 
return account+"|"+appkey;
				 
}
	protected Parser getParser() {
		return new UserCenterParse();
	}
}
