package cn.redcdn.datacenter.usercenter;

import org.json.JSONException;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.usercenter.data.UserInfo;
import cn.redcdn.log.CustomLog;

public abstract class ResetPassword extends AbstractBusinessData<UserInfo> {
    private String tag = ResetPassword.class.getName();

    /**
     * 重置密码接口。返回用户信息和结果码。对应《个人用户中心接口文档》1.6.9
     *
     * @param account          帐号
     * @param verificationCode 验证码, 由发送重置密码验证码后获得。
     * @param newPassword      新密码
     * @return -1 : 表示无效的操作，当前操作正在执行; -2: 表示无效的编码; -3: 表示json操作异常
     */
    public int resetPassword(String account, String verificationCode,
                             String newPassword) {
        CustomLog.i(tag, "usercenter resetPassword");

        JSONObject json = new JSONObject();
        try {
            if (ConstConfig.getNeedtoaddAppkey()) {
                json.put(ConstConfig.ACCOUNT, ConstConfig.getAccount(account, ConstConfig.getAppkey()));
            } else {
                json.put(ConstConfig.ACCOUNT, account);
            }
//			json.put(ConstConfig.ACCOUNT, account);
            json.put(ConstConfig.VERIFICATIONCODE, verificationCode);
            json.put(ConstConfig.NEWPASSWORD, newPassword);
            json.put(ConstConfig.PRODUCTID, "prod002");
        } catch (JSONException e) {
            return -3;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        return exec(ConstConfig.getResetPassword(),
                ConstConfig.PARAM_USERCNTER_RESETPASSWORD + jsonString);
    }

    public int resetPasswordUser(String account, String verificationCode,
                             String newPassword) {
        CustomLog.i(tag, "usercenter resetPassword");

        JSONObject json = new JSONObject();
        try {
            if (ConstConfig.getNeedtoaddAppkey()) {
                json.put(ConstConfig.ACCOUNT, ConstConfig.getAccount(account, ConstConfig.getAppkey()));
            } else {
                json.put(ConstConfig.ACCOUNT, account);
            }
//			json.put(ConstConfig.ACCOUNT, account);
            json.put(ConstConfig.VERIFICATIONCODE, verificationCode);
            json.put(ConstConfig.NEWPASSWORD, newPassword);
            json.put(ConstConfig.PRODUCTID, "prod002");
        } catch (JSONException e) {
            return -3;
        }
        //转义特殊字符   % --> %25
        String jsonString = java.net.URLEncoder.encode(json.toString());
        return exec(ConstConfig.getResetPassword(),
                ConstConfig.PARAM_USERCNTER_RESETPASSWORDUSER + jsonString);
    }

    protected UserInfo parseContentBody(JSONObject jo)
            throws InvalidateResponseException {
        if (jo == null) {
            CustomLog.e(tag, "usercenter resetPassword invalidate reponse");
            throw new InvalidateResponseException();
        }

        UserInfo info = new UserInfo();
        JSONObject userInfo = jo.optJSONObject("user");
        info.appType = userInfo.optString("appType");
        info.nubeNumber = userInfo.optString("nubeNumber");
        info.mobile = userInfo.optString("mobile");
        info.uid = userInfo.optString("uid");

        return info;
    }

    protected Parser getParser() {
        return new UserCenterParse();
    }

}
