package cn.redcdn.datacenter.usercenter;

import org.json.JSONObject;

import android.text.TextUtils;
import cn.redcdn.datacenter.AbstractBusinessData;
import cn.redcdn.datacenter.ExecResultCode;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.datacenter.usercenter.data.BasicInfo;
import cn.redcdn.log.CustomLog;

/**
 * 统一认证中心修改账号属性。 对应统一认证中心接口6.15
 * 
 * @author KevinZhang
 *
 */
public abstract class SetAccountAttr extends AbstractBusinessData<BasicInfo> {
  private String tag = SetAccountAttr.class.getName();

  /**
   * 异步修改参会名称接口
   * 
   * @param accessToken
   *          token
   * @param nickName
   *          参会名称 (如果传 null，则表示仅修改头像地址)
   * @param headUrl
   *          头像地址 (如果传 null，则表示仅修改参会名称)
   * @return {@link ExecResultCode.SUCCESS} = 0 成功
   *         <p>
   *         {@link ExecResultCode.DOING_ERROR} = -1 操作中
   *         <p>
   *         {@link ExecResultCode.INVALID_ENCODE_ERROR} = -2 无效的编码
   *         <p>
   *         {@link ExecResultCode.JSON_FORMAT_ERROR} = -3 JSON异常
   *         <p>
   *         {@link ExecResultCode.PARAM_ERROR} = -4 参数错误
   */
  public int setAccountAttr(String accessToken, String nickName, String headUrl) {
    CustomLog.i(tag, "异步接口修改参会名称 setAccountAttr  token: " + accessToken
        + " | nickName: " + nickName + " | headUrl: " + headUrl);
    if (TextUtils.isEmpty(accessToken)) {
      CustomLog
          .e(tag, "error：token不能为空! return :" + ExecResultCode.PARAM_ERROR);
      return ExecResultCode.PARAM_ERROR;
    }

    if (TextUtils.isEmpty(nickName) && TextUtils.isEmpty(headUrl)) {
      CustomLog.e(tag, "error：参会名称和头像地址不能都为空! return :"
          + ExecResultCode.PARAM_ERROR);
      return ExecResultCode.PARAM_ERROR;
    }

    try {
      JSONObject json = new JSONObject();
      if (!TextUtils.isEmpty(nickName)) {
        json.put("nickName", nickName);
      }

      if (!TextUtils.isEmpty(headUrl)) {
        json.put("headUrl", headUrl);
      }
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return exec(ConstConfig.getUserCenterSetAccountAttr(),
          "service=setAccountAttr&accessToken=" + accessToken + "&params="
              + jsonString);
    } catch (Exception e) {
      CustomLog.e(tag, e.getMessage());
      return ExecResultCode.JSON_FORMAT_ERROR;
    }
  }

  /**
   * 同步修改参会名称接口
   * 
   * @param accessToken
   *          token
   * @param nickName
   *          参会名称 (如果传 null，则表示仅修改头像地址)
   * @param headUrl
   *          头像地址 (如果传 null，则表示仅修改参会名称)
   * @return {@link BasicInfo} 通过getStatus() 和 getMessage() 方法获取返回值。
   * 
   */
  public BasicInfo setAccountAttrSync(String accessToken, String nickName,
      String headUrl) {
    CustomLog.i(tag, "同步接口修改参会名称 setAccountAttrSync  token: " + accessToken
        + " | nickName: " + nickName + " | headUrl: " + headUrl);

    if (TextUtils.isEmpty(accessToken)) {
      CustomLog
          .e(tag, "error：token不能为空! return :" + ExecResultCode.PARAM_ERROR);
      return new BasicInfo(ExecResultCode.PARAM_ERROR, "参数错误：token不能为空");
    }

    if (TextUtils.isEmpty(nickName) && TextUtils.isEmpty(headUrl)) {
      CustomLog.e(tag, "error：参会名称和头像地址不能都为空! return :"
          + ExecResultCode.PARAM_ERROR);
      return new BasicInfo(ExecResultCode.PARAM_ERROR, "参数错误：参会名称和头像地址不能都为空!");
    }

    try {
      JSONObject json = new JSONObject();
      if (!TextUtils.isEmpty(nickName)) {
        json.put("nickName", nickName);
      }

      if (!TextUtils.isEmpty(headUrl)) {
        json.put("headUrl", headUrl);
      }
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(json.toString());
      return execSync(ConstConfig.getUserCenterSetAccountAttr(),
          "service=setAccountAttr&accessToken=" + accessToken + "&params="
              + jsonString);
    } catch (Exception e) {
      CustomLog.e(tag, e.getMessage());
      return new BasicInfo(ExecResultCode.JSON_FORMAT_ERROR, "JSON格式错误!");
    }
  }

  protected BasicInfo parseContentBody(JSONObject jo)
      throws InvalidateResponseException {
    BasicInfo info = new BasicInfo();
    info.setStatus(Integer.parseInt(jo.optString("status")));
    info.setMessage(jo.optString("message"));
    return info;
  }

  protected Parser getParser() {
    return new UserCenterParse();
  }

}
