package cn.redcdn.datacenter.versionupdate;

import org.json.JSONObject;

import android.text.TextUtils;
import cn.redcdn.datacenter.DataErrorCode;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;
import cn.redcdn.network.httprequest.ConnectException;
import cn.redcdn.network.httprequest.PostTextHttp;

public class AbstractBusinessDataForCheckAppVersion<T> {
  /**
   * masterIp  应用升级主服务器IP地址
   * slaveIp   应用升级从服务器IP地址
   * slaveUrl  应用升级从服务器后台接口请求地址
   */
  private String masterIp = ConstConfig.getAppUpdateUrlPrefix();
  private String slaveIp  = ConstConfig.getSlaveAppUpdateUrlPrefix();
  private String slaveUrl  = ConstConfig.getSlaveAppUpdateUrl(); 
  
  private String tag = getClass().getName();
  private PostTextHttp httpPost;

  /**
   *  定义master和slave服务器模式变量
   */
  private boolean bSlaveHttpStatus = false;

  // 正常情况返回类型为T的数据对象
  // 异常情况下抛如下异常:
  // InvalidateOperateException: 无效的操作
  // ConnectException: 连接服务器失败;
  // InvalidateHttpResponse: 无效得http response,比如http response字段错误,以及http
  // response io exception
  // UnhandledHttpStatus: 比如302,404等其他status code
  
  protected T execSync(String url, final String content) {
    
    httpPost = new PostTextHttp();
    httpPost.setURL(url);

    String responseContent = null;
    try {
      responseContent = httpPost.postUrlEncodeContentSync(content);
      CustomLog.e(tag, " ResponseContent1 = " + responseContent);
    } catch (ConnectException e) {
      if(TextUtils.isEmpty(slaveIp)){
        CustomLog.e(tag, " 从服务器IP为空，抛异常");
        throw new ConnectException(e.getMessage());
      }
      
      CustomLog.w(tag, "connect to slave server");
      httpPost.setURL(slaveUrl);
      responseContent = httpPost.postUrlEncodeContentSync(content);
      CustomLog.e(tag, " ResponseContent2 = " + responseContent);
    }
    Parser parser = getParser();
    if (parser != null && responseContent != null) {
      CustomLog.e(tag,"同步接口从服务器请求成功，切换IP");
      switchSlaveUrl();
      parser.parse(responseContent);
      try {
        return parseContentBody(parser.getBody());
      } catch (InvalidateResponseException e) {
        CustomLog.e(tag,"InvalidateResponseException,"+ e.toString());
      }
    }

    return null;
  }

  // -1: 表示无效的操作，当前操作正在执行;
  // -2: 表示无效的编码;
  protected int exec(String url, final String content) {
    if (null != httpPost) {
      return -1;
    }
    
    httpPost = new PostTextHttp() {
      @Override
      protected void onRequestFailed(int statusCode) {

        if (statusCode != 0) {
          if (bSlaveHttpStatus == false) {
            if(TextUtils.isEmpty(slaveIp)){
              CustomLog.e(tag,"从服务器IP为空,onFail");
              onFail(statusCode, null);
            }
            CustomLog.e(tag,"请求从服务器: url = "+ slaveUrl);
            CustomLog.w(tag, "connect to slave server: " + content);
            bSlaveHttpStatus = true;
            httpPost = null;
            exec(slaveUrl, content);
          } else {
            onFail(statusCode, null);
          }
        } else {
          onFail(statusCode, null);
        }
      }

      @Override
      protected void onRequestFinished(int statusCode, String responseContent) {

        CustomLog.d(tag,"onRequestFinished: "+responseContent);
        if(bSlaveHttpStatus){
          CustomLog.d(tag,"从服务器请求成功，交换ConstConfig内的主从服务器");
          switchSlaveUrl();
        }
        
        Parser parser = getParser();
        if (null == parser) {
          onFail(DataErrorCode.DATA_INVALIDATE_PARSER, null);
          return;
        }

        parser.parse(responseContent);
        if (parser.isOk()) {
          Object ob = null ;
          try {
            ob  = parseContentBody(parser.getHeaderValue(), parser.getBody());
          } catch (InvalidateResponseException e) {
            onFail(DataErrorCode.DATA_INVALIDATE_RESPONSE_FORMAT, null);
            return;
          } catch (ClassCastException e) {
            onFail(DataErrorCode.CLASS_CAST_EXCEPTION, null);
            return;
          } catch (NumberFormatException e) {
            onFail(DataErrorCode.NUMBER_FORMAT_EXCEPTION, null);
            return;
          }
          onSuccess((T)ob);
        } else {
          onFail(parser.getHeaderValue(), parser.getHeaderText());
        }
      }
    };
    httpPost.setURL(url);
    return httpPost.postUrlEncodeContent(content);
  }

  protected void switchSlaveUrl() {
    if(TextUtils.isEmpty(slaveIp)){
      CustomLog.e(
          tag,"应用升级从服务器IP地址为空，不交换");
      return;
    }
    
    CustomLog.e(
        tag,
        "ConstConfig.getSlaveAppUpdateUrlPrefix()="
            + ConstConfig.getSlaveAppUpdateUrlPrefix()
            + ",  ConstConfig.getAppUpdateUrlPrefix()="
            + ConstConfig.getAppUpdateUrlPrefix());
    
    ConstConfig.setMasterAppUpdateServerWebDomain(slaveIp);
    ConstConfig.setSlaveAppUpdateServerWebDomain(masterIp);
    
    CustomLog.e(
        tag,
        "ConstConfig.getSlaveAppUpdateUrlPrefix()="
            + ConstConfig.getSlaveAppUpdateUrlPrefix()
            + ",  ConstConfig.getAppUpdateUrlPrefix()="
            + ConstConfig.getAppUpdateUrlPrefix());
  }

  
  public void cancel() {
    if (null == httpPost) {
      return;
    }

    httpPost.cancel();
    httpPost = null;
  }

  protected T parseContentBody(JSONObject jo)
      throws InvalidateResponseException {
    return null;
  }

  protected T parseContentBody(int headerValue, JSONObject jo)
      throws InvalidateResponseException { 
    return parseContentBody(jo);
  }

  protected Parser getParser() {
    return null;
  }

  protected void onSuccess(T responseContent) {
  };


  // 返回HttpErrorCode、接口业务逻辑错误码、DataErrorCode
  protected void onFail(int statusCode, String statusInfo) {
    CustomLog.e(tag,"onFail, statusCode = "+ statusCode+" statusInfo = "+statusInfo);
  };
}
