package cn.redcdn.datacenter.versionupdate;

import java.util.Date;
import org.json.JSONObject;

import cn.redcdn.datacenter.AbstractBusinessDataDouble;
import cn.redcdn.datacenter.InvalidateResponseException;
import cn.redcdn.datacenter.Parser;
import cn.redcdn.datacenter.config.ConstConfig;
import cn.redcdn.log.CustomLog;

public abstract class CheckAppVersion extends
       AbstractBusinessDataDouble<CheckAppVersionInfo> {
  private String TAG = CheckAppVersion.class.getName();

  public int checkVersion(String version, String pkgName, String terminal,
      String projectName) {
    CustomLog.d(TAG, "CheckAppVersion.checkVersion  version=" + version
        + ",pkgName=" + pkgName + ",terminal=" + terminal + ",projectName="
        + projectName);
    if ((version == null) || (pkgName == null) || (terminal == null)
        || (projectName == null)) {
      CustomLog.e(TAG, "参数为空");
      return -1;
    }
    JSONObject updateJsonObject = new JSONObject();
    try {
      updateJsonObject.put(ConstConfig.PARAM_TERMINAL, terminal);
      updateJsonObject.put(ConstConfig.PARAM_PROJECT, projectName);
      updateJsonObject.put(ConstConfig.PARAM_NAME, pkgName);
      updateJsonObject.put(ConstConfig.PARAM_VERSION, version);
      updateJsonObject.put(ConstConfig.PARAM_UTCTIME,
          String.valueOf((new Date().getTime()) / 1000));
      
      String FirstRequestUrl = ConstConfig.getAppUpdateUrl(); 
      String SecondRequestUrl = ConstConfig.getSlaveAppUpdateUrl();
	  //转义特殊字符   % --> %25
	String jsonString = java.net.URLEncoder.encode(updateJsonObject.toString());
      return exec(FirstRequestUrl,SecondRequestUrl,
          ConstConfig.PARAM_CHECK_APP_VERSION + jsonString);
      
    } catch (Exception e) {
      CustomLog.e(TAG, "调用接口出错 " + e.getMessage());
      return -2;
    }
  }
  
  /**
   * 增加新的参数   sn 设备串号
   * @param version
   * @param pkgName
   * @param sn
   * @param terminal
   * @param projectName
   * @return
   */
  
  
  public int checkVersion(String version, String pkgName,String sn,String terminal,
	      String projectName) {
	    CustomLog.d(TAG, "CheckAppVersion.checkVersion  version=" + version
	        + ",pkgName=" + pkgName +"sn = "+sn+ ",terminal=" + terminal + ",projectName="
	        + projectName);
	    if ((version == null) || (pkgName == null) || (terminal == null)
	        || (projectName == null)||(sn == null)) {
	      CustomLog.e(TAG, "参数为空");
	      return -1;
	    }
	    JSONObject updateJsonObject = new JSONObject();
	    try {
	      updateJsonObject.put(ConstConfig.PARAM_TERMINAL, terminal);
	      updateJsonObject.put(ConstConfig.PARAM_PROJECT, projectName);
	      updateJsonObject.put(ConstConfig.PARAM_SN, sn);
	      updateJsonObject.put(ConstConfig.PARAM_NAME, pkgName);
	      updateJsonObject.put(ConstConfig.PARAM_VERSION, version);
	      updateJsonObject.put(ConstConfig.PARAM_UTCTIME,
	          String.valueOf((new Date().getTime()) / 1000));
	      
	      String FirstRequestUrl = ConstConfig.getAppUpdateUrl(); 
	      String SecondRequestUrl = ConstConfig.getSlaveAppUpdateUrl();
		  //转义特殊字符   % --> %25
		String jsonString = java.net.URLEncoder.encode(updateJsonObject.toString());
	      return exec(FirstRequestUrl,SecondRequestUrl,
	          ConstConfig.PARAM_CHECK_APP_VERSION + jsonString);
	      
	    } catch (Exception e) {
	      CustomLog.e(TAG, "调用接口出错 " + e.getMessage());
	      return -2;
	    }
	  }

  protected CheckAppVersionInfo parseContentBody(int headerValue, JSONObject responseJsonObject) 
      throws InvalidateResponseException {
    try {   
      if (headerValue == 0){
        CheckAppVersionInfo info = new CheckAppVersionInfo();
        info.checkValue = headerValue;
        CustomLog.d(TAG, "CheckAppVersionInfo "+info);
        return info;
      }
      if (responseJsonObject == null) {
        CustomLog.e(TAG, "CheckAppVersionInfo JSONObject == null");
        throw new InvalidateResponseException();
      }
      CheckAppVersionInfo info = new CheckAppVersionInfo();
      info.checkValue = headerValue;
      info.interval = responseJsonObject.optLong("interval");
      info.updatePlugin = responseJsonObject.optString("updator");
      info.appInstallPackage = responseJsonObject.optString("installer");
      info.appHashCode = responseJsonObject.optString("installerCode");
      info.appInstallVersion = responseJsonObject.optString("version");
      info.isForcedInstall = responseJsonObject.optInt("compatible");
      info.descriptions = responseJsonObject.optString("descriptions");
      return info;
    } catch (Exception e) {
      CustomLog.e(TAG, "CheckAppVersion invalidate reponse");
      throw new InvalidateResponseException();
    }
  }

  protected Parser getParser() {
    return new CheckAppInstallParser();
  }

//  @Override
//  public String getSlaveUrl() {
//    if (ConstConfig.getSlaveAppUpdateUrl() == null
//        || ConstConfig.getSlaveAppUpdateUrl().isEmpty()) {
//      return "";
//    }
//    String SlaveUrl = ConstConfig.getAppUpdateUrl();
//    CustomLog.e(
//        TAG,
//        "ConstConfig.getSlaveAppUpdateUrl()="
//            + ConstConfig.getSlaveAppUpdateUrl()
//            + "ConstConfig.getAppUpdateUrl()=" + ConstConfig.getAppUpdateUrl());
//    ConstConfig.setMasterAppUpdateServerWebDomain(ConstConfig
//        .getSlaveAppUpdateUrl());
//    ConstConfig.setSlaveAppUpdateServerWebDomain(SlaveUrl);
//    CustomLog.e(
//        TAG,
//        "ConstConfig.getSlaveAppUpdateUrl()="
//            + ConstConfig.getSlaveAppUpdateUrl()
//            + "ConstConfig.getAppUpdateUrl()=" + ConstConfig.getAppUpdateUrl());
//
//    return ConstConfig.getAppUpdateUrl();
//  }
}
