package cn.redcdn.log;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import android.annotation.SuppressLint;
import android.os.Environment;
import android.util.Log;

public class LogcatFileManager {
  private static final String TAG = "LogcatFileManager";

  private static LogcatFileManager INSTANCE = null;
  private static String PATH_LOGCAT;
  private File logFile = null;
  private int limitFileCounts = 20; // 允许的生成的文件个数
  private static long singleFileSize = 1 * 1024 * 1024; // 单个文件的最大size
  private FileOutputStream out = null;
  @SuppressLint("SimpleDateFormat")
  private static SimpleDateFormat logfileFormat = new SimpleDateFormat(
      "yyyy-MM-dd-HH-mm-ss"); // 日志文件格式
  private boolean isFirstStart = false;
  private ExecutorService executorService;
  private static String LOG_PATH_SDCARD_DIR = Environment
      .getExternalStorageDirectory().getPath() + "/meeting/log/";
  private String mSaveDirectoy;
  private boolean isFirstStop = true;
  
@SuppressLint("SimpleDateFormat")
  private static SimpleDateFormat LogSdf = new SimpleDateFormat(
      "yyyy-MM-dd HH:mm:ss.SSSZ"); // 日志的输出格式
  private enum Operation {
    START, STOP, LOG
  }

  public synchronized static LogcatFileManager getInstance() {
    if (INSTANCE == null) {
      INSTANCE = new LogcatFileManager();
    }
    return INSTANCE;
  }

  private LogcatFileManager() {
	  
  }
  
  /**
   * 设置日志目录
   * @param dir 日志路径
   */
  public synchronized void setLogDir(String dir) {
    Log.d(TAG, "LogcatFileManager::setLogDir() dir:" + dir);
    LOG_PATH_SDCARD_DIR = Environment
        .getExternalStorageDirectory().getPath() + "/"+dir+ "/log/";
    Log.d(TAG, "LogcatFileManager::setLogDir() LOG_PATH_SDCARD_DIR:" + LOG_PATH_SDCARD_DIR);
  }

  public synchronized void start(String saveDirectoy) {
    Log.d(TAG, "初始化日志模块");
    if (!isFirstStart) {
      mSaveDirectoy = saveDirectoy;
      executorService = Executors.newSingleThreadExecutor();
      executorService.submit(new LogRunnable("", Operation.START));
    }
  }

  public synchronized void stop() {
    if (isFirstStop && isFirstStart && executorService != null) {
      isFirstStop = false;
      executorService.submit(new LogRunnable("", Operation.STOP));
      try {
		executorService.awaitTermination(2, TimeUnit.SECONDS); //最多等待2s使executorService执行队列中的剩余任务
	} catch (InterruptedException e) {
		e.printStackTrace();
	}
    }
  }

  public void setMessage(String message) {
	  if (isFirstStart && isFirstStop && executorService != null) {
      executorService.submit(new LogRunnable(message, Operation.LOG));
    }
  }

  private void setFolderPath(String folderPath) {
    Log.d(TAG, "LogcatFileManager::foderPath: " + folderPath);
    File folder = new File(folderPath);
    if (!folder.exists()) {
      folder.mkdirs();
    }

    if (!folder.isDirectory()) {
      CustomLog.e(TAG, "The logcat folder path is not a directory: "
          + folderPath);
    }

    PATH_LOGCAT = folderPath.endsWith("/") ? folderPath : folderPath + "/";

    checkFileCounts();

  }

  /**
   * 检查文件个数，是否超过限制
   */
  private void checkFileCounts() {
    CustomLog.d(TAG, "checkFileCounts");
    if (limitFileCounts == -1) {
      return;
    }
    File folder = new File(PATH_LOGCAT);
    File[] files = folder.listFiles();

    if (files.length < limitFileCounts) {
      return;
    }

    List<String> mPathString = new ArrayList<String>();
    for (File file : files) {
      mPathString.add(file.getAbsolutePath());
    }

    Collections.sort(mPathString);
    int removeNum = mPathString.size() - limitFileCounts + 1;
    for (int i = 0; i < mPathString.size(); i++) {
      if (i < removeNum) {
        File removeFile = new File(mPathString.get(i));
        removeFile.delete();
      }
    }

  }

  private boolean CreateNewFile(String dir) {
    Log.d(TAG, "创建日志文件:" + dir);
    File cfgFile = new File(dir);
    if (!cfgFile.exists()) {
      Log.e(TAG, "CreateNewFile 日志目录不存在!");
      return false;
    }
    boolean flag = false;
    try {
      Date nowtime = new Date();
      String needWriteFiel = logfileFormat.format(nowtime);
      logFile = new File(dir, needWriteFiel +"-"+android.os.Process.myPid()+"-logcat.log");
      out = new FileOutputStream(logFile, true);
      flag = true;
    } catch (FileNotFoundException e) {
      e.printStackTrace();
      flag = false;
    }
    return flag;
  }

  private boolean isBigLimit() {
    Boolean is = false;
    long size = logFile.length();
    if (size >= singleFileSize)
      is = true;
    return is;
  }

  private class LogRunnable implements Runnable {
    private String msg;
    private Operation flag;

    public LogRunnable(String msg, Operation flag) {
      this.msg = msg;
      this.flag = flag;
    }

    @Override
    public void run() {
      try {

        switch (this.flag) {
        case START:
          Log.d(TAG, "执行初始化日志模块");
          execStart(mSaveDirectoy);
          break;
        case STOP:
          Log.d(TAG, "执行关闭日志模块");
          execStop();
          break;
        case LOG:
          if (logFile != null && out != null) {
            if (!isBigLimit()) {
              logFile.getAbsolutePath();
              File file=new File(logFile.getAbsolutePath());   
              if(!file.exists())    
              {    
                    if (!CreateNewFile(PATH_LOGCAT)){
                      Log.d(TAG, "日志文件创建失败，关闭日志系统！");
                      stop();
                      return;
                    }
              }    
//              Date nowtime = new Date();
              out.write((msg + "\n").getBytes());
            } else {
              out.close();
              checkFileCounts();
              CreateNewFile(PATH_LOGCAT);
//              Date nowtime = new Date();
              out.write((msg + "\n").getBytes());
            }
            break;
          }
        }
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
  }

  private void execStart(String saveDirectoy) {
    setFolderPath(LOG_PATH_SDCARD_DIR + saveDirectoy);
    if (CreateNewFile(PATH_LOGCAT)) {
      isFirstStart = true;
      isFirstStop = true;
      Log.v(TAG, "新建日志文件成功");
    } else {
      Log.v(TAG, "新建日志文件失败");
      return;
    }
  }

  private void execStop() {
	  Log.d(TAG, "execStop!");
    try {
      out.close();
      logFile = null;
      out = null;
      isFirstStart = false;
    } catch (IOException e) {
      e.printStackTrace();
    }
    
    if(executorService != null) {
    	executorService.shutdown();
    }
  }
}