package cn.redcdn.network.httprequest;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.cert.CertificateFactory;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.GZIPInputStream;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;

import android.os.AsyncTask;

import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.conn.ConnectTimeoutException;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.ByteArrayBuffer;

import cn.redcdn.log.CustomLog;

public abstract class AsyncHttp {
	final static String tag = AsyncHttp.class.getName();

	public final static int BUFFER_SIZE = 8192;// 8k
	private final static int MAX_RETRY_COUNT = 0;
	private final static int SOCKET_TIMEOUT = 8 * 1000; // 8秒
	private final static int CONNECTION_TIMEOUT = 8 * 1000; // 8秒
	private final static String USER_AGENT = "RedcdnClient";

	enum ExcutingStatus {
		NONE, RUNNING
	}

	ExcutingStatus status = ExcutingStatus.NONE;

	// <HeaderName, HeaderValue>
	private Map<String, String> httpHeaders;
	private HttpParams httpParams;

	private AsyncTask<HttpUriRequest, byte[], ResponseEntry> task;
	private HttpUriRequest request;

	class ResponseEntry {
		private int code;
		private HttpResponse response;

		ResponseEntry(int code, HttpResponse response) {
			this.code = code;
			this.response = response;
		}
	};

	public AsyncHttp() {
		httpParams = new BasicHttpParams();
		setUserAgent(USER_AGENT);// TODO 直接调用该方法是否有问题
		setContentCharset(HTTP.UTF_8);
		setHttpElementCharset(HTTP.UTF_8);

		setSoTimeout(SOCKET_TIMEOUT);
		setConnectionTimeout(CONNECTION_TIMEOUT);

		httpHeaders = new HashMap<String, String>();
	}

	// http protocal param
	public void setUserAgent(String agent) {
		HttpProtocolParams.setUserAgent(httpParams, agent);
	}

	public void setContentCharset(String charset) {
		HttpProtocolParams.setContentCharset(httpParams, charset);
	}

	public String getContentCharset() {
		return HttpProtocolParams.getContentCharset(httpParams);
	}

	public void setHttpElementCharset(String charset) {
		HttpProtocolParams.setHttpElementCharset(httpParams, charset);
	}

	// http connect param
	public void setSoTimeout(int millisecondTimeout) {
		HttpConnectionParams.setSoTimeout(httpParams, millisecondTimeout);
	}

	public void setConnectionTimeout(int millisecondTimeout) {
		HttpConnectionParams.setConnectionTimeout(httpParams,
				millisecondTimeout);
	}

	public void setSocketBufferSize(int size) {
		HttpConnectionParams.setSocketBufferSize(httpParams, size);
	}

	public void setTcpNoDelay(boolean value) {
		HttpConnectionParams.setTcpNoDelay(httpParams, value);
	}

	// http header
	public void setHttpHeaders(Map<String, String> headerLists) {
		httpHeaders.putAll(headerLists);
	}

	public void setHttpHeader(String key, String value) {
		httpHeaders.put(key, value);
	}

	public String requestSync(final HttpUriRequest request) {
		CustomLog.i(tag, "async request");
		if (ExcutingStatus.RUNNING == status) {
			CustomLog.e(tag, "async request running...");
			throw new InvalidateOperateException("async request running...");
		}

		this.request = request;

		if (null != httpHeaders) {
			for (Map.Entry<String, String> entry : httpHeaders.entrySet()) {
				CustomLog.d(tag, "add request header, key:" + entry.getKey()
						+ ",value:" + entry.getValue());
				request.addHeader(entry.getKey(), entry.getValue());
			}
		}

		status = ExcutingStatus.RUNNING;
		int responseErrorCode = HttpErrorCode.HTTP_SERVER_ERROR;

		HttpResponse response = null;

		int retryCount = 0;
		do {
			CustomLog.i(tag, "requestSync::call http client execute");
			responseErrorCode = HttpErrorCode.HTTP_SERVER_ERROR;

			try {
				response = new DefaultHttpClient(httpParams).execute(request);
			} catch (ClientProtocolException e) {
				responseErrorCode = HttpErrorCode.HTTP_CLIENT_PROTOCOL_ERROR;
				CustomLog.e(tag,
						"http client execute fail, ClientProtocolException:"
								+ e.getMessage());
			} catch (IllegalStateException e) {
				responseErrorCode = HttpErrorCode.HTTP_ILLEGAL_STATE;
				CustomLog.e(
						tag,
						"http client execute fail, IllegalStateException:"
								+ e.getMessage());
			} catch (ConnectTimeoutException e) {
				responseErrorCode = HttpErrorCode.HTTP_CONNECT_TIMEOUT;
				CustomLog.e(tag,
						"http client execute fail, ConnectTimeoutException:"
								+ e.getMessage());
			} catch (SocketException e) {
				responseErrorCode = HttpErrorCode.HTTP_SOCKET_ERROR;
				CustomLog.e(tag, "http client execute fail, SocketException:"
						+ e.getMessage());
			} catch (SocketTimeoutException e) {
				responseErrorCode = HttpErrorCode.HTTP_SOCKET_RS_TIMEOUT;
				CustomLog.e(
						tag,
						"http client execute fail, SocketTimeoutException:"
								+ e.getMessage());
			} catch (IOException e) {
				responseErrorCode = HttpErrorCode.HTTP_IO_EXCEPTION;
				CustomLog.e(
						tag,
						"http client execute fail, IOException:"
								+ e.getMessage());
			} catch (IllegalArgumentException e) {
				responseErrorCode = HttpErrorCode.HTTP_ILLEGAL_ARGUMENT;
				CustomLog.e(tag,
						"http client execute fail, IllegalArgumentException:"
								+ e.getMessage());
			}
		} while ((responseErrorCode == HttpErrorCode.HTTP_CONNECT_TIMEOUT || responseErrorCode == HttpErrorCode.HTTP_SOCKET_RS_TIMEOUT)
				&& (++retryCount < MAX_RETRY_COUNT)
				&& (status == ExcutingStatus.RUNNING));

		StatusLine statusLine = null;
		if (null == response || null == (statusLine = response.getStatusLine())) {
			CustomLog.e(tag, "http response body invalidate");
			status = ExcutingStatus.NONE;
			throw new ConnectException(Integer.toString(responseErrorCode));
		}

		int statusCode = statusLine.getStatusCode();
		if (statusCode != 200 && statusCode != 206) {// TODO 重定向？
			CustomLog.e(tag,
					"http response statusCode invalidate, status code:"
							+ statusCode);
			status = ExcutingStatus.NONE;
			throw new UnhandledHttpStatus(Integer.toString(statusCode));
		}

		long contentLength = response.getEntity().getContentLength();
		if (-1 == contentLength && response.getEntity().isChunked()) {// transfer-codeing:
																		// chunk
			// do nothing
			// chunked
		} else if (contentLength > Integer.MAX_VALUE || contentLength < -1) {
			CustomLog.e(tag, "invalidate http response content length");
			status = ExcutingStatus.NONE;
			throw new InvalidateHttpResponse(
					Integer.toString(HttpErrorCode.HTTP_RESPONSE_INVALIDATE_LENGTH));
		}

		long arrayLen = contentLength;
		if (contentLength < 0) {
			arrayLen = BUFFER_SIZE;
		}

		int totalLen = 0;
		ByteArrayBuffer textBuffer = new ByteArrayBuffer((int) arrayLen);

		try {
			int readLen = 0;
			byte[] buffer = new byte[BUFFER_SIZE];
			InputStream input = response.getEntity().getContent();
			while ((readLen = input.read(buffer)) != -1) {
				textBuffer.append(buffer, 0, readLen);
				totalLen += readLen;
			}
			input.close();
		} catch (IOException e) {
			CustomLog.e(tag, "read data error:" + e.getMessage());
			status = ExcutingStatus.NONE;
			throw new InvalidateHttpResponse(
					Integer.toString(HttpErrorCode.HTTP_IO_EXCEPTION));
		}

		if (totalLen < contentLength) {
			CustomLog.e(tag, "read http response error");
			status = ExcutingStatus.NONE;
			throw new InvalidateHttpResponse(
					Integer.toString(HttpErrorCode.HTTP_IO_EXCEPTION));
		}

		status = ExcutingStatus.NONE;

		try {
			return new String(textBuffer.buffer(), getContentCharset());
		} catch (UnsupportedEncodingException e) {
			CustomLog.e(tag, "http response:" + e.getMessage());
		}

		return "";
	}

	public int request(final HttpUriRequest request) {
		CustomLog.i(tag, "async request");
		if (ExcutingStatus.RUNNING == status) {
			CustomLog.e(tag, "async request running...");
			return -1;
		}

		this.request = request;

		if (null != httpHeaders) {
			for (Map.Entry<String, String> entry : httpHeaders.entrySet()) {
				CustomLog.d(tag, "add request header, key:" + entry.getKey()
						+ ",value:" + entry.getValue());
				request.addHeader(entry.getKey(), entry.getValue());
			}
		}

		status = ExcutingStatus.RUNNING;

		task = new AsyncTask<HttpUriRequest, byte[], ResponseEntry>() {

			@Override
			protected void onPostExecute(ResponseEntry entry) {

				if (status == ExcutingStatus.NONE) {
					CustomLog.w(tag, "async task already cancel");
					return;
				}

				status = ExcutingStatus.NONE;

				if (0 == entry.code) {
					CustomLog.i(tag, "http request finish");

					onRequestFinished(entry.response);
				} else {
					CustomLog.e(tag, "http request error, errorcode:"
							+ entry.code);

					// entry.code less than 0, means http request error
					// ,else means http server statecode
					onRequestFailed(entry.code);
				}
			}

			// function onCancelled not be called in android 2.3.3,
			// but in android 4.1.1 will be. why?
			@Override
			protected void onCancelled(ResponseEntry entry) {
				CustomLog.w(tag, "async task onCancelled");
			}

			@Override
			protected ResponseEntry doInBackground(HttpUriRequest... params) {

				int responseErrorCode = HttpErrorCode.HTTP_SERVER_ERROR;
				HttpResponse response = null;

				int retryCount = 0;
				do {
					CustomLog.i(tag,
							"doInBackground:: call http client execute ");
					responseErrorCode = HttpErrorCode.HTTP_SERVER_ERROR;
					try {
						response = new DefaultHttpClient(httpParams)
								.execute(params[0]);
					} catch (ClientProtocolException e) {
						responseErrorCode = HttpErrorCode.HTTP_CLIENT_PROTOCOL_ERROR;
						CustomLog.e(tag,
								"http client execute fail, ClientProtocolException:"
										+ e.getMessage());
					} catch (IllegalStateException e) {
						responseErrorCode = HttpErrorCode.HTTP_ILLEGAL_STATE;
						CustomLog.e(tag,
								"http client execute fail, IllegalStateException:"
										+ e.getMessage());
					} catch (ConnectTimeoutException e) {
						responseErrorCode = HttpErrorCode.HTTP_CONNECT_TIMEOUT;
						CustomLog.e(tag,
								"http client execute fail, ConnectTimeoutException:"
										+ e.getMessage());
					} catch (SocketException e) {
						responseErrorCode = HttpErrorCode.HTTP_SOCKET_ERROR;
						CustomLog.e(tag,
								"http client execute fail, SocketException:"
										+ e.getMessage());
					} catch (SocketTimeoutException e) {
						responseErrorCode = HttpErrorCode.HTTP_SOCKET_RS_TIMEOUT;
						CustomLog.e(tag,
								"http client execute fail, SocketTimeoutException:"
										+ e.getMessage());
					} catch (IOException e) {
						responseErrorCode = HttpErrorCode.HTTP_IO_EXCEPTION;
						CustomLog.e(
								tag,
								"http client execute fail, IOException:"
										+ e.getMessage());
					} catch (IllegalArgumentException e) {
						responseErrorCode = HttpErrorCode.HTTP_ILLEGAL_ARGUMENT;
						CustomLog.e(tag,
								"http client execute fail, IllegalArgumentException:"
										+ e.getMessage());
					}
				} while ((responseErrorCode == HttpErrorCode.HTTP_CONNECT_TIMEOUT || responseErrorCode == HttpErrorCode.HTTP_SOCKET_RS_TIMEOUT)
						&& (++retryCount < MAX_RETRY_COUNT)
						&& (status == ExcutingStatus.RUNNING));

				StatusLine status = null;
				if (null == response
						|| null == (status = response.getStatusLine())) {
					CustomLog.e(tag, "http response body invalidate");
					return new ResponseEntry(responseErrorCode, null);
				}

				int statusCode = status.getStatusCode();
				if (statusCode != 200 && statusCode != 206) {// TODO 重定向？
					CustomLog.e(tag,
							"http response statusCode invalidate, status code:"
									+ statusCode);
					return new ResponseEntry(statusCode, null);
				}

				long contentLength = response.getEntity().getContentLength();
				if (-1 == contentLength && response.getEntity().isChunked()) {// transfer-codeing:
																				// chunk
					// do nothing
					// chunked
				} else if (contentLength > Integer.MAX_VALUE
						|| contentLength < -1) {
					CustomLog.e(tag, "invalidate http response content length");
					return new ResponseEntry(
							HttpErrorCode.HTTP_RESPONSE_INVALIDATE_LENGTH, null);
				}

				boolean isGzip = false;
				Header[] headers = response.getHeaders("Content-Encoding");
				for (Header header : headers) {
					if ("gzip".equals(header.getValue())) {
						isGzip = true;
						break;
					}
				}

				if (!isGzip) {
					onStart((int) contentLength);
				}

				CustomLog.d("tag", "http response isGzip=" + isGzip);

				byte[] buffer = new byte[BUFFER_SIZE];
				try {
					// 读数据前将读数据超时设置为10秒
					setSoTimeout(SOCKET_TIMEOUT);
					InputStream input = response.getEntity().getContent();
					if (isGzip) {
						GZIPInputStream gzipInputStream = new GZIPInputStream(
								input);
						BufferedReader reader = new BufferedReader(
								new InputStreamReader(gzipInputStream,
										HTTP.UTF_8));
						String line = null;
						// int readLen = 0, totalLen = 0;
						StringBuffer sb = new StringBuffer();
						while ((line = reader.readLine()) != null) {
							sb.append(line).append("\n");
						}
						CustomLog.d("tag",
								"http response content=" + sb.toString());
						// onStart(sb.toString().length() + 2);
						onReadData(sb.toString());
					} else {
						int readLen = 0, totalLen = 0;
						while ((readLen = input.read(buffer)) != -1) {
							onReadData(buffer, readLen);
							totalLen += readLen;
						}

						if (totalLen < contentLength) {
							CustomLog.e(tag, "read http response error");
							return new ResponseEntry(
									HttpErrorCode.HTTP_IO_EXCEPTION, null);
						}
					}

				} catch (IOException e) {
					CustomLog.e(tag, "read data error:" + e.getMessage());
					// 读数据异常，重新设置读数据超时为10秒
					setSoTimeout(SOCKET_TIMEOUT);
					return new ResponseEntry(HttpErrorCode.HTTP_IO_EXCEPTION,
							null);
				}
				// 读数据正常结束，重新设置读数据超时为10秒
				setSoTimeout(SOCKET_TIMEOUT);
				return new ResponseEntry(0, response);
			}
		};

		task.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, request);
		return 0;
	}
	
	public int request(final HttpUriRequest request,final int timeOut) {
		CustomLog.i(tag, "async request");
		if (ExcutingStatus.RUNNING == status) {
			CustomLog.e(tag, "async request running...");
			return -1;
		}

		this.request = request;

		setSoTimeout(timeOut);
		
		if (null != httpHeaders) {
			for (Map.Entry<String, String> entry : httpHeaders.entrySet()) {
				CustomLog.d(tag, "add request header, key:" + entry.getKey()
						+ ",value:" + entry.getValue());
				request.addHeader(entry.getKey(), entry.getValue());
			}
		}

		status = ExcutingStatus.RUNNING;

		task = new AsyncTask<HttpUriRequest, byte[], ResponseEntry>() {

			@Override
			protected void onPostExecute(ResponseEntry entry) {

				if (status == ExcutingStatus.NONE) {
					CustomLog.w(tag, "async task already cancel");
					return;
				}

				status = ExcutingStatus.NONE;

				if (0 == entry.code) {
					CustomLog.i(tag, "http request finish");

					onRequestFinished(entry.response);
				} else {
					CustomLog.e(tag, "http request error, errorcode:"
							+ entry.code);

					// entry.code less than 0, means http request error
					// ,else means http server statecode
					onRequestFailed(entry.code);
				}
			}

			// function onCancelled not be called in android 2.3.3,
			// but in android 4.1.1 will be. why?
			@Override
			protected void onCancelled(ResponseEntry entry) {
				CustomLog.w(tag, "async task onCancelled");
			}

			@Override
			protected ResponseEntry doInBackground(HttpUriRequest... params) {

				int responseErrorCode = HttpErrorCode.HTTP_SERVER_ERROR;
				HttpResponse response = null;

				int retryCount = 0;
				do {
					CustomLog.i(tag,
							"doInBackground:: call http client execute ");
					responseErrorCode = HttpErrorCode.HTTP_SERVER_ERROR;
					try {
						response = new DefaultHttpClient(httpParams)
								.execute(params[0]);
					} catch (ClientProtocolException e) {
						responseErrorCode = HttpErrorCode.HTTP_CLIENT_PROTOCOL_ERROR;
						CustomLog.e(tag,
								"http client execute fail, ClientProtocolException:"
										+ e.getMessage());
					} catch (IllegalStateException e) {
						responseErrorCode = HttpErrorCode.HTTP_ILLEGAL_STATE;
						CustomLog.e(tag,
								"http client execute fail, IllegalStateException:"
										+ e.getMessage());
					} catch (ConnectTimeoutException e) {
						responseErrorCode = HttpErrorCode.HTTP_CONNECT_TIMEOUT;
						CustomLog.e(tag,
								"http client execute fail, ConnectTimeoutException:"
										+ e.getMessage());
					} catch (SocketException e) {
						responseErrorCode = HttpErrorCode.HTTP_SOCKET_ERROR;
						CustomLog.e(tag,
								"http client execute fail, SocketException:"
										+ e.getMessage());
					} catch (SocketTimeoutException e) {
						responseErrorCode = HttpErrorCode.HTTP_SOCKET_RS_TIMEOUT;
						CustomLog.e(tag,
								"http client execute fail, SocketTimeoutException:"
										+ e.getMessage());
					} catch (IOException e) {
						responseErrorCode = HttpErrorCode.HTTP_IO_EXCEPTION;
						CustomLog.e(
								tag,
								"http client execute fail, IOException:"
										+ e.getMessage());
					} catch (IllegalArgumentException e) {
						responseErrorCode = HttpErrorCode.HTTP_ILLEGAL_ARGUMENT;
						CustomLog.e(tag,
								"http client execute fail, IllegalArgumentException:"
										+ e.getMessage());
					}
				} while ((responseErrorCode == HttpErrorCode.HTTP_CONNECT_TIMEOUT || responseErrorCode == HttpErrorCode.HTTP_SOCKET_RS_TIMEOUT)
						&& (++retryCount < MAX_RETRY_COUNT)
						&& (status == ExcutingStatus.RUNNING));

				StatusLine status = null;
				if (null == response
						|| null == (status = response.getStatusLine())) {
					CustomLog.e(tag, "http response body invalidate");
					return new ResponseEntry(responseErrorCode, null);
				}

				int statusCode = status.getStatusCode();
				if (statusCode != 200 && statusCode != 206) {// TODO 重定向？
					CustomLog.e(tag,
							"http response statusCode invalidate, status code:"
									+ statusCode);
					return new ResponseEntry(statusCode, null);
				}

				long contentLength = response.getEntity().getContentLength();
				if (-1 == contentLength && response.getEntity().isChunked()) {// transfer-codeing:
																				// chunk
					// do nothing
					// chunked
				} else if (contentLength > Integer.MAX_VALUE
						|| contentLength < -1) {
					CustomLog.e(tag, "invalidate http response content length");
					return new ResponseEntry(
							HttpErrorCode.HTTP_RESPONSE_INVALIDATE_LENGTH, null);
				}

				boolean isGzip = false;
				Header[] headers = response.getHeaders("Content-Encoding");
				for (Header header : headers) {
					if ("gzip".equals(header.getValue())) {
						isGzip = true;
						break;
					}
				}

				if (!isGzip) {
					onStart((int) contentLength);
				}

				CustomLog.d("tag", "http response isGzip=" + isGzip);

				byte[] buffer = new byte[BUFFER_SIZE];
				try {
					setSoTimeout(timeOut);
					InputStream input = response.getEntity().getContent();
					if (isGzip) {
						GZIPInputStream gzipInputStream = new GZIPInputStream(
								input);
						BufferedReader reader = new BufferedReader(
								new InputStreamReader(gzipInputStream,
										HTTP.UTF_8));
						String line = null;
						// int readLen = 0, totalLen = 0;
						StringBuffer sb = new StringBuffer();
						while ((line = reader.readLine()) != null) {
							sb.append(line).append("\n");
						}
						CustomLog.d("tag",
								"http response content=" + sb.toString());
						// onStart(sb.toString().length() + 2);
						onReadData(sb.toString());
					} else {
						int readLen = 0, totalLen = 0;
						while ((readLen = input.read(buffer)) != -1) {
							onReadData(buffer, readLen);
							totalLen += readLen;
						}

						if (totalLen < contentLength) {
							CustomLog.e(tag, "read http response error");
							return new ResponseEntry(
									HttpErrorCode.HTTP_IO_EXCEPTION, null);
						}
					}

				} catch (IOException e) {
					CustomLog.e(tag, "read data error:" + e.getMessage());
					setSoTimeout(timeOut);
					return new ResponseEntry(HttpErrorCode.HTTP_IO_EXCEPTION,
							null);
				}
				setSoTimeout(timeOut);
				return new ResponseEntry(0, response);
			}
		};

		task.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, request);
		return 0;
	}

	@SuppressWarnings("unused")
	public String requestSyncWithHttps(String requestUrl, String filePath,
			String requestMethod, String contentData) {

		URL url = null;
		try {
			url = new URL(requestUrl);
		} catch (MalformedURLException e) {
			CustomLog
					.d(tag, "requestSyncWithHttps URL error e=" + e.toString());
			throw new InvalidateOperateException(
					Integer.toString(HttpErrorCode.HTTP_ILLEGAL_ARGUMENT));
		}
		HttpsURLConnection conn = null;
		try {
			conn = (HttpsURLConnection) url.openConnection();
		} catch (IOException e) {
			CustomLog.d(
					tag,
					"requestSyncWithHttps openConnection error e="
							+ e.toString());
			throw new InvalidateOperateException(
					Integer.toString(HttpErrorCode.HTTP_IO_EXCEPTION));
		}

		try {
			if (filePath != null) {
				File file = new File(filePath);
				InputStream input;
				input = new FileInputStream(file);

				if (input != null) {
					conn.setSSLSocketFactory(getSSLSocketFactory(input));
					conn.setHostnameVerifier(ButelHostnameVerifier.INSTANCE);
				}
			}
		} catch (FileNotFoundException e) {
			CustomLog.d(tag,
					"requestSyncWithHttps open file error e=" + e.toString());
			throw new InvalidateOperateException(
					Integer.toString(HttpErrorCode.HTTP_ILLEGAL_ARGUMENT));
		}

		conn.setReadTimeout(5000);
		conn.setConnectTimeout(5000);
		conn.setDoOutput(true);
		conn.setDoInput(true);
		conn.setUseCaches(false);
		conn.setInstanceFollowRedirects(true);

		try {
			conn.setRequestMethod(requestMethod);
		} catch (ProtocolException e) {
			CustomLog.d(tag, "requestSyncWithHttps setRequestMethod error e="
					+ e.toString());
			throw new InvalidateOperateException(
					Integer.toString(HttpErrorCode.HTTP_CLIENT_PROTOCOL_ERROR));
		}
		try {
			conn.connect();
		} catch (IOException e) {
			CustomLog.d(tag,
					"requestSyncWithHttps connect error e=" + e.toString());
			throw new InvalidateOperateException(
					Integer.toString(HttpErrorCode.HTTP_IO_EXCEPTION));
		}

		DataOutputStream out = null;
		int code = 0;
		try {
			out = new DataOutputStream(conn.getOutputStream());
			out.writeBytes(contentData);

			out.flush();
			out.close();

			code = conn.getResponseCode();
		} catch (IOException e) {
			CustomLog.d(tag,
					"requestSyncWithHttps writeBytes error e=" + e.toString());
			throw new InvalidateOperateException(
					Integer.toString(HttpErrorCode.HTTP_IO_EXCEPTION));
		}

		CustomLog.d(tag, "requestSyncWithHttps code=" + code);

		if (code != 200 && code != 206) {
			CustomLog.e(tag, "requestSyncWithHttps 错误:" + code);
		} else {
			InputStream inputStream;
			try {
				inputStream = conn.getInputStream();
				StringBuilder sb = new StringBuilder();
				int flag;
				byte[] buf = new byte[1024];
				while ((flag = inputStream.read(buf)) != -1) {
					sb.append(new String(buf, 0, flag));
				}
				if (sb != null) {
					String s = sb.toString();
					CustomLog.d(tag, "s=" + s);

					return s;
				} else {
					CustomLog.d("requestSyncWithHttps", "空");
				}
				// 释放资源
				inputStream.close();
				inputStream = null;
				conn.disconnect();

			} catch (IOException e) {
				CustomLog.d(tag, "requestSyncWithHttps readResponse error e="
						+ e.toString());
				throw new InvalidateHttpResponse(
						Integer.toString(HttpErrorCode.HTTP_IO_EXCEPTION));
			}

		}
		return "";

	}

	/**
	 * 获取SSLSocketFactory
	 * 
	 * @param certificates
	 *            证书流文件
	 * @return
	 */
	private static SSLSocketFactory getSSLSocketFactory(
			InputStream... certificates) {
		try {
			CertificateFactory certificateFactory = CertificateFactory
					.getInstance("X.509");
			KeyStore keyStore = KeyStore.getInstance(KeyStore.getDefaultType());
			keyStore.load(null);
			int index = 0;
			for (InputStream certificate : certificates) {
				String certificateAlias = Integer.toString(index++);
				keyStore.setCertificateEntry(certificateAlias,
						certificateFactory.generateCertificate(certificate));

				try {
					if (certificate != null)
						certificate.close();
				} catch (IOException e) {
					CustomLog.d(tag,
							"getSSLSocketFactory error e=" + e.toString());
				}
			}

			SSLContext sslContext = SSLContext.getInstance("TLS");
			TrustManagerFactory trustManagerFactory = TrustManagerFactory
					.getInstance(TrustManagerFactory.getDefaultAlgorithm());
			trustManagerFactory.init(keyStore);
			sslContext.init(null, trustManagerFactory.getTrustManagers(),
					new SecureRandom());

			return sslContext.getSocketFactory();
		} catch (Exception e) {
			CustomLog.d(tag, "getSSLSocketFactory " + e.toString());
			e.printStackTrace();
		}
		return null;
	}

	public void cancel() {
		CustomLog.i(tag, "async request cancel");

		if (status == ExcutingStatus.NONE) {
			CustomLog.e(tag, "async request already cancel");
			return;
		}

		status = ExcutingStatus.NONE;

		try {
			request.abort();
		} catch (UnsupportedOperationException e) {
			CustomLog.w(tag, "abort http request exception:" + e.getMessage());
		}

		if (task != null) {
			task.cancel(true);
			task = null;
		}
	}

	// totalSize等于-1表示回应内容按chunked方式传输
	protected abstract void onStart(int totalSize);

	protected abstract void onReadData(byte[] buffer, int len);

	protected abstract void onReadData(String buffer);

	protected abstract void onRequestFinished(HttpResponse response);

	// 返回HttpErrorCode
	protected abstract void onRequestFailed(int statusCode);
}
