package cn.redcdn.network.httprequest;

import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.util.HashMap;
import java.util.Map;

import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.content.ContentBody;
import org.apache.http.protocol.HTTP;
import cn.redcdn.log.CustomLog;

public abstract class AsyncHttpPost extends AsyncHttp {
	private String tag = AsyncHttpPost.class.getName();

	private String mUrlString;

	public void setURL(String url) {
		mUrlString = url;
	}

	public int postMultipartContent(String name, ContentBody body,
			PostContentProgress progress) {
		Map<String, ContentBody> bodys = new HashMap<String, ContentBody>();
		bodys.put(name, body);
		return postMultipartContent(bodys, progress);
	}

	public int postMultipartContent(Map<String, ContentBody> bodys,
			PostContentProgress progress) {
		CustomLog.i(tag, "post content to http server:" + mUrlString);

		HttpPost httpPost = new HttpPost(mUrlString);
		MultipartProgressEntity reqEntity = new MultipartProgressEntity(
				progress);
		for (Map.Entry<String, ContentBody> entry : bodys.entrySet()) {
			reqEntity.addPart(entry.getKey(), entry.getValue());
		}
		httpPost.setEntity(reqEntity);

		return request(httpPost);
	}

	public int postMultipartContent(Map<String, ContentBody> bodys,
			PostContentProgress progress, int timeOut) {
		CustomLog.i(tag, "post content to http server:" + mUrlString);

		HttpPost httpPost = new HttpPost(mUrlString);
		MultipartProgressEntity reqEntity = new MultipartProgressEntity(
				progress);
		for (Map.Entry<String, ContentBody> entry : bodys.entrySet()) {
			reqEntity.addPart(entry.getKey(), entry.getValue());
		}
		httpPost.setEntity(reqEntity);

		return request(httpPost, timeOut * 1000);
	}

	public int postCotent(String content, String charset)
			throws UnsupportedEncodingException {
		CustomLog.i(tag, "post data:" + content + " to http server:"
				+ mUrlString);

		HttpPost httpPost = new HttpPost(mUrlString);
		httpPost.setEntity(new StringEntity(content, charset));

		return request(httpPost);
	}

	public String postCotentSync(String content, String charset)
			throws UnsupportedEncodingException {
		CustomLog.i(tag, "post data:" + content + " to http server:"
				+ mUrlString);

		HttpPost httpPost = new HttpPost(mUrlString);
		httpPost.setEntity(new StringEntity(content, charset));

		return requestSync(httpPost);
	}

	public int postCotent(String content) throws UnsupportedEncodingException {
		try {
			return postCotent(content, HTTP.UTF_8);
		} catch (UnsupportedEncodingException e) {
		}

		return -2;
	}

	public int postCotentWithContentType(String content, String contentType)
			throws UnsupportedEncodingException {
		try {
			setHttpHeader("Content-Type", contentType);
			setHttpHeader("User-Agent", "Android Butel");
			return postCotent(content, HTTP.UTF_8);
		} catch (UnsupportedEncodingException e) {
		}

		return -2;
	}

	public int postJsonContent(String content, String charset)
			throws UnsupportedEncodingException {
		setHttpHeader("Content-Type", "application/json");
		return postCotent(content, charset);
	}

	public int postJsonContent(String content)
			throws UnsupportedEncodingException {
		try {
			return postJsonContent(content, HTTP.UTF_8);
		} catch (UnsupportedEncodingException e) {
		}

		return -2;
	}

	public int postUrlEncodeContent(String content, String charset)
			throws UnsupportedEncodingException {
		setHttpHeader("Content-Type", "application/x-www-form-urlencoded");
		return postCotent(content, charset);
	}

	public int postUrlEncodeContent(String content) {
		try {
			return postUrlEncodeContent(content, HTTP.UTF_8);
		} catch (UnsupportedEncodingException e) {
		}

		return -2;
	}

	public String postUrlEncodeContentSync(String content, String charset)
			throws UnsupportedEncodingException {
		setHttpHeader("Content-Type", "application/x-www-form-urlencoded");
		return postCotentSync(content, charset);
	}

	public String postUrlEncodeContentSync(String content) {
		try {
			return postUrlEncodeContentSync(content, HTTP.UTF_8);
		} catch (UnsupportedEncodingException e) {
		}

		return "";
	}

	public int postXmlContent(String content, String charset)
			throws UnsupportedEncodingException {
		return postCotent(content, charset);
	}

	public int postXmlContent(String content) {
		try {
			return postUrlEncodeContent(content, HTTP.UTF_8);
		} catch (UnsupportedEncodingException e) {
		}

		return -2;
	}

	public String postUrlContentSyncWithHttps(String filePath,
			String contentData) {

		return requestSyncWithHttps(mUrlString, filePath, "POST", contentData);
	}
}
