package cn.redcdn.network.httprequest;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import org.apache.http.HttpResponse;

import android.os.Handler;
import android.os.Message;
import cn.redcdn.log.CustomLog;

public abstract class HttpDownloadFile {
	private final String tag = HttpDownloadFile.class.getName();

	public final int WRITE_FILE_FAILED = 0x1000;

	private AsyncHttpGet httpGet;
	private RandomAccessFile randomAccessFile;

	enum DownloadStatus {
		NONE, RUNNING
	}

	private DownloadStatus downloadStatus = DownloadStatus.NONE;

	private final int WRITE_FILE_FAILED_MESSAGE = 0x1;
	private final int WRITE_FILE_SIZE_MESSAGE = 0X2;

	private int fileSize = 0;
	private int writeFileSize = 0;
	// private int percent = -1;
	private final Handler messageHandler = new Handler() {
		@Override
		public void dispatchMessage(Message msg) {
			switch (msg.what) {
			case WRITE_FILE_SIZE_MESSAGE:
				if (DownloadStatus.RUNNING == downloadStatus) {
					onDownloadProgress(msg.arg1, msg.arg2);
				}
				break;

			case WRITE_FILE_FAILED_MESSAGE:
				if (DownloadStatus.RUNNING == downloadStatus) {
					CustomLog.e(tag, "write buffer to file fail");

					downloadStatus = DownloadStatus.NONE;
					httpGet.cancel();// 断开网络连接

					try {
						randomAccessFile.close();
					} catch (IOException e) {
						e.printStackTrace();
					}

					OnDownloadFail(WRITE_FILE_FAILED);
				}
				break;

			default:
				break;
			}
		}
	};

	public int downloadFile(String url, final String localFileName,
			final int compelete) throws FileNotFoundException {
		CustomLog.e(tag, "download file, url:" + url + ",localFileName:"
				+ localFileName + " compelete " + compelete);

		if (DownloadStatus.RUNNING == downloadStatus) {
			CustomLog.e(tag, "already downloading...");
			return -1;
		}
		writeFileSize = compelete;
		httpGet = new AsyncHttpGet() {

			// warning, running in background thread
			@Override
			protected void onStart(int totalSize) {
				fileSize = totalSize;
				File file = new File(localFileName);
				try {
					if (!file.exists()) {
						file.createNewFile();
					}
					if (compelete == 0) {
						RandomAccessFile accessFile = new RandomAccessFile(
								file, "rwd");
						// accessFile.setLength(fileSize);
						accessFile.close();
					}
					randomAccessFile = new RandomAccessFile(localFileName,
							"rwd");
					randomAccessFile.seek(compelete);
				} catch (IOException e) {
					CustomLog.e(tag, "onStart " + e);
				}
			}

			// warning, running in background thread
			@Override
			protected void onReadData(byte[] buffer, int len) {
				try {
					randomAccessFile.write(buffer, 0, len);
					writeFileSize += len;

					// CustomLog.d(tag, "download info, writeFileSize:" +
					// writeFileSize
					// + ",len:" + len + ",buffer length:" + buffer.length
					// + " fileSize " + fileSize);

					if (-1 == fileSize) {// transfer-coding:chunked
						return;
					}

					Message msg = Message.obtain();
					msg.what = WRITE_FILE_SIZE_MESSAGE;
					msg.arg1 = fileSize;
					msg.arg2 = writeFileSize;
					messageHandler.sendMessage(msg);

				} catch (IOException e) {
					messageHandler.sendEmptyMessage(WRITE_FILE_FAILED_MESSAGE);
				}
			}

			@Override
			protected void onReadData(String buffer) {

			}

			@Override
			protected void onRequestFinished(HttpResponse response) {
				try {
					randomAccessFile.close();
				} catch (IOException e) {
					OnDownloadFail(WRITE_FILE_FAILED);
					return;
				}

				CustomLog.i(tag, "download file ok");

				downloadStatus = DownloadStatus.NONE;
				OnDownloadFinish();
			}

			@Override
			protected void onRequestFailed(int statusCode) {
				CustomLog
						.e(tag, "download file fail, statusCode:" + statusCode);

				downloadStatus = DownloadStatus.NONE;
				OnDownloadFail(statusCode);
			}
		};

		downloadStatus = DownloadStatus.RUNNING;

		httpGet.setURL(url);
		// httpGet.setRange("bytes=" + compelete + "-");
		httpGet.setHttpHeader("Range", "bytes=" + compelete + "-");
		return httpGet.get();
	}

	public void cancel() {
		CustomLog.i(tag, "cancel download file");

		if (DownloadStatus.NONE == downloadStatus) {
			CustomLog.w(tag, "don't download file");
			return;
		}

		downloadStatus = DownloadStatus.NONE;
		httpGet.cancel();

		try {
			randomAccessFile.close();
		} catch (IOException e) {
			CustomLog.e(tag, "cancel " + e);
		}
	}

	protected abstract void OnDownloadFinish();

	protected abstract void OnDownloadFail(int statusCode);

	protected void onDownloadProgress(int fileSize, int complete) {
	};
}
