package cn.redcdn.network.httprequest;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

import org.apache.http.HttpResponse;
import org.apache.http.entity.mime.content.ContentBody;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.util.ByteArrayBuffer;
import org.json.JSONObject;
import cn.redcdn.log.CustomLog;

public abstract class HttpUpload {
	private String tag = HttpUpload.class.getName();

	private AsyncHttpPost httpPost;
	private ByteArrayBuffer textBuffer;

	enum UploadStatus {
		NONE, RUNNING
	}

	UploadStatus uploadStatus = UploadStatus.NONE;

	// Map<String, ContentBody> bodys = new HashMap<String, ContentBody>();
	// bodys.put("service", StringBody.create("Suggestion", "text/plain",
	// Charset.forName("UTF-8")));
	// bodys.put("params", StringBody.create(userInfo, "text/plain",
	// Charset.forName("UTF-8")));
	// bodys.put("file", new FileBody(new File(localFile)));
	//
	/*
	 * 录音文件上传使用接口
	 */
	public int upload(String url, String phoneId, String imei,
			String contactInfo, String filePath, String token) {

		Map<String, ContentBody> bodys = new HashMap<String, ContentBody>();
		bodys.put(
				"service",
				StringBody.create("Suggestion", "text/plain",
						Charset.forName("UTF-8")));
		JSONObject json = null;
		try {
			json = new JSONObject();
			json.put("token", token);
			json.put("contactInfo", contactInfo);
			json.put("imei", imei);
			json.put("phoneId", phoneId);
		} catch (Exception e) {
		}
		bodys.put(
				"params",
				StringBody.create(json.toString(), "text/plain",
						Charset.forName("UTF-8")));
		ContentBody body = new FileBody(new File(filePath));
		bodys.put("file", body);

		return upload(url, bodys);
	}

	public int upload(String url, String name, ContentBody body) {
		Map<String, ContentBody> bodys = new HashMap<String, ContentBody>();
		bodys.put(name, body);
		return upload(url, bodys);
	}

	public int upload(String url, Map<String, ContentBody> bodys) {
		if (url == null || null == bodys) {
			return -1;
		}

		CustomLog.i(tag, "http upload url:" + url + ", upload parameter count:"
				+ bodys.size());

		if (UploadStatus.RUNNING == uploadStatus) {
			CustomLog.e(tag, "already uploading...");
			return -1;
		}

		uploadStatus = UploadStatus.RUNNING;

		httpPost = new AsyncHttpPost() {
			@Override
			protected void onStart(int totalSize) {
				if (totalSize == -1) { // chunked
					totalSize = AsyncHttp.BUFFER_SIZE;
				}

				textBuffer = new ByteArrayBuffer(totalSize);
			}

			@Override
			protected void onReadData(byte[] buffer, int len) {
				textBuffer.append(buffer, 0, len);
			}

			@Override
			protected void onReadData(String buffer) {

			}

			@Override
			protected void onRequestFinished(HttpResponse response) {
				uploadStatus = UploadStatus.NONE;

				try {
					OnUploadFinish(
							response.getStatusLine().getStatusCode(),
							new String(textBuffer.buffer(), getContentCharset()));
				} catch (UnsupportedEncodingException e) {
					onRequestFailed(HttpErrorCode.HTTP_RESPONSE_INVALIDATE_ENCODING);
				}
			}

			@Override
			protected void onRequestFailed(int statusCode) {

				uploadStatus = UploadStatus.NONE;
				OnUploadFail(statusCode);
			}
		};

		httpPost.setURL(url);
		return httpPost.postMultipartContent(bodys, new PostContentProgress() {

			@Override
			public void onPostProgress(int percent) {
				CustomLog.d(tag,
						"upload percent:" + percent + httpPost.toString());

				onUploadProgress(percent);
			}

		});
	}

	public void cancel() {
		CustomLog.i(tag, "cancel upload");

		if (UploadStatus.NONE == uploadStatus) {
			CustomLog.w(tag, "don't upload");
			return;
		}

		uploadStatus = UploadStatus.NONE;
		httpPost.cancel();
	}

	public abstract void OnUploadFinish(int statusCode, String responseContent);

	public abstract void OnUploadFail(int statusCode);

	public void onUploadProgress(int percent) {
	};
}
