package cn.redcdn.network.httprequest;

import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.apache.http.entity.mime.MultipartEntity;

import android.os.Handler;
import android.os.Message;

import cn.redcdn.log.CustomLog;

public class MultipartProgressEntity extends MultipartEntity {
  private String tag = MultipartProgressEntity.class.getName();

  private class CalcOutputProcgress extends FilterOutputStream {

    private int totalLen = 0;
    private int percent = -1;
    
    public CalcOutputProcgress(OutputStream out) {
      super(out);
    }

    @Override
    public void write(byte[] buffer, int offset, int length) throws IOException {
      super.write(buffer, offset, length);

      totalLen += length;

      if (0 == contentLength) {
        CustomLog.w(tag, "total content length is zero");
        return;
      }

      int percentLocal = (int)(((float)totalLen/contentLength)*100);
      if (percentLocal != percent) {
        percent = percentLocal;

        Message msg = Message.obtain();
        msg.what = WRITE_MESSAGE;
        msg.arg1 = percent;
        messageHandler.sendMessage(msg);
      }
    }
  }

  private final int WRITE_MESSAGE = 0x1;
  
  private CalcOutputProcgress outputProgress;
  private PostContentProgress multipartProgress;

  private long contentLength = 0;

  private final Handler messageHandler = new Handler() {
    @Override
    public void dispatchMessage(Message msg) {
      switch (msg.what) {
        case WRITE_MESSAGE:
          multipartProgress.onPostProgress(msg.arg1);
          break;

        default:
          break;
      }
    }
  };

  public MultipartProgressEntity(){
    this(null);
  }

  public MultipartProgressEntity(PostContentProgress process) {
    multipartProgress = process;
  }

  //in background thread
  public void writeTo(final OutputStream outstream) throws IOException {
    if (null == outputProgress) {
      outputProgress = new CalcOutputProcgress(outstream);
      contentLength = getContentLength();
    }

    super.writeTo(outputProgress);
  }
}
