package cn.redcdn.network.httprequest;

import java.io.UnsupportedEncodingException;

import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.util.ByteArrayBuffer;

import cn.redcdn.log.CustomLog;

public class PostTextHttp extends AsyncHttpPost {

	private ByteArrayBuffer textBuffer;
	private String contentBuffer;

	@Override
	protected void onStart(int totalSize) {
		if (totalSize == -1) { // chunked
			totalSize = AsyncHttp.BUFFER_SIZE;
		}

		textBuffer = new ByteArrayBuffer(totalSize);
	}

	@Override
	protected void onReadData(byte[] buffer, int len) {
		textBuffer.append(buffer, 0, len);
	}

	@Override
	protected void onReadData(String buffer) {
		contentBuffer = buffer;
	}

	@Override
	protected void onRequestFinished(HttpResponse response) {
		try {
			boolean isGzip = false;
			Header[] headers = response.getHeaders("Content-Encoding");
			for (Header header : headers) {
				if ("gzip".equals(header.getValue())) {
					isGzip = true;
					break;
				}
			}

			if (isGzip) {
				onRequestFinished(response.getStatusLine().getStatusCode(),
						contentBuffer);
			} else {
				onRequestFinished(response.getStatusLine().getStatusCode(),
						new String(textBuffer.buffer(), getContentCharset()));
			}

		} catch (UnsupportedEncodingException e) {
			onRequestFailed(HttpErrorCode.HTTP_RESPONSE_INVALIDATE_ENCODING);
		}
	}

	protected void onRequestFailed(int statusCode) {
	};

	// 返回http status code
	protected void onRequestFinished(int statusCode, String responseContent) {
	};

	// 返回http status code
	protected void onRequestFinished(int statusCode, HttpResponse response) {
	};
}
