package cn.redcdn.network.udp;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.channels.DatagramChannel;

import android.os.NetworkOnMainThreadException;
import cn.redcdn.log.CustomLog;

public class UDPSocket {
	final static String tag = UDPSocket.class.getName();

	private final static int maxLength = 64 * 1024;
	private String localAddr = "";
	private DatagramChannel channel = null;
	private DatagramSocket datagramSocket = null;

	public int init() {
		release();

		try {
			channel = DatagramChannel.open();
			datagramSocket = channel.socket();
			datagramSocket.bind(null);
			if (android.os.Build.VERSION.SDK_INT >= 24) {
				datagramSocket.setSoTimeout(200); // 对于android 7.0 设备，receive
													// 线程会堵死使得send 发送不出去
			}

			// getLocalPort the local port of this socket
			// or -1 if this socket is closed and 0 if it is unbound
			return datagramSocket.getLocalPort();
		} catch (IllegalArgumentException e) {
			CustomLog.e(tag, "UDPSocket bind cause IllegalArgumentException:"
					+ e.getMessage());
		} catch (SocketException e) {
			CustomLog.e(tag,
					"UDPSocket bind cause SocketException:" + e.getMessage());
		} catch (IOException e) {
			CustomLog.e(tag,
					"UDPSocket bind cause IOException:" + e.getMessage());
		} catch (NetworkOnMainThreadException e) {
			CustomLog.e(
					tag,
					"UDPSocket bind cause NetworkOnMainThreadException:"
							+ e.getMessage());
		}

		return -1;
	}

	public boolean send(String remoteAddr, int remotePort, byte[] data) {
		CustomLog.d(tag, "UDPSocket send ip:" + remoteAddr + ", port:"
				+ remotePort);
		if (null == datagramSocket) {
			CustomLog.d(tag, "UDPSocket send null == datagramSocket");
			return false;
		}

		try {
			DatagramPacket dgmPacket = new DatagramPacket(data, data.length,
					InetAddress.getByName(remoteAddr), remotePort);
			datagramSocket.send(dgmPacket);
			return true;
		} catch (UnknownHostException e) {
			CustomLog.e(tag, "send data UnknownHostException:" + e.getMessage()
					+ ", remoteAddr:" + remoteAddr + ",remotePort:"
					+ remotePort);
		} catch (IOException e) {
			CustomLog.e(tag, "send data IOException:" + e.getMessage()
					+ ", remoteAddr:" + remoteAddr + ",remotePort:"
					+ remotePort);
		} catch (Exception e) {
			CustomLog.e(tag, "send data Exception:" + e.getMessage()
					+ ", remoteAddr:" + remoteAddr + ",remotePort:"
					+ remotePort);
		}
		return false;
	}

	public DatagramPacket receive() {
		if (null == datagramSocket) {
			return null;
		}

		byte receiver[] = new byte[maxLength];
		DatagramPacket pack = new DatagramPacket(receiver, receiver.length);
		try {
			datagramSocket.receive(pack);
			return pack;
		} catch (Exception e) {

		}
		return null;
	}

	/*
	 * receive以及send方法没有对datagramSocket做空指针异常处理，
	 * 因此release时不设置datagramSocket为空，多线程调用release方法以及receive或send时
	 * 不报空指针错误；如果这样datagramSocket是否有内存泄露？？
	 */
	public void release() {
		CustomLog.d(tag, "UDPSocket release");
		if (datagramSocket != null) {
			try {
				datagramSocket.close();
				channel.close();
				datagramSocket = null;
				channel = null;
			} catch (Exception e) {
				CustomLog.e(tag, "USocket Close Error:" + e);
			}
		}
	}

	/*
	 * 该方法没有被使用
	 */
	public String getLocalAddress() {
		if ((this.localAddr != null) && (this.localAddr.length() > 0)) {
			return this.localAddr;
		}

		if ((this.datagramSocket != null)
				&& (this.datagramSocket.getLocalAddress() != null)) {
			return this.datagramSocket.getLocalAddress().getHostAddress();
		}
		return "";
	}
}
