package com.butel.MediaGeoMetry;

import android.annotation.SuppressLint;
import android.view.MotionEvent;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnTouchListener;

public class MediaGeoMetryJni implements OnTouchListener
{
	private int mUserID = -1;
	private SurfaceView mView = null;
	private boolean	mDown = false;
	private MediaGeoMetryJni.DrawCommandCallback mCallback = null;
	private boolean mIsSetListener = false;
	
	//Graphic Manager Actor
	public static int GRAPHIC_LAYER_MANAGER			= 1;
	public static int WHITE_BOARD_MANAGER			= 2;
	public static int PROGRESS_CAPTURE_MANAGER		= 3;
	
	//Draw element and command
	public static int GEO_DRAW_LINE					= 0;
	public static int GEO_DRAW_CURVE				= 1;
	public static int GEO_DRAW_RECT					= 2;
	public static int GEO_DRAW_ELLIPS				= 3;
	public static int GEO_ERASE_RECT				= 4;
	public static int GEO_SELECT					= 5;
	public static int GEO_CLEAR_BOARD				= 6;
	public static int GEO_EDIT_TEXT					= 7;
	public static int GEO_ERROR_PIC					= 8;
	public static int GEO_CORRECT_PIC				= 9;
	public static int GEO_SHARE_PIC					= 10;
	public static int GEO_PLUS_PIC					= 11;
	public static int GEO_FLAG_PIC					= 12;
	public static int GEO_STAR_PIC					= 13;
	public static int GEO_THUMBTACK_PIC				= 14;
	public static int GEO_SINGLE_BRUSH				= 15;
	public static int GEO_MULTI_BRUSH				= 16;
	public static int GEO_RECTIN_TRAIANGLE			= 17;
	public static int GEO_LINE_SEGMENT				= 18;
	public static int GEO_LASER_PEN					= 19;
	public static int GEO_ERASE_TOOL				= 20;
	
	//Draw command
	public static int GEO_DRAW_CMD_REVOCATION		= 1;
	public static int GEO_DRAW_CMD_REPAINT			= 2;
	public static int GEO_DRAW_CMD_DELETE_MYSELF	= 3;
	public static int GEO_DRAW_CMD_DELETE_OTHERS	= 4;
	public static int GEO_DRAW_CMD_DELETE_ALL		= 5;
	
	//image operation
	public static int GEO_IMAGE_OP_SCREENSHOT		= 0;
	public static int GEO_IMAGE_OP_CHARLET			= 1;
	
	//font
	public static int GEO_FONT_SONGTYPE				= 0;
	public static int GEO_FONT_SOFTBLACK			= 1;
	public static int GEO_FONT_BLACKTYPE			= 2;
	public static int GEO_FONT_YOUYUANTYPE			= 3;
	public static int GEO_FONT_ARIAL				= 4;
	
	//mouse state
	public static int GEO_DOWN						= 1;
	public static int GEO_DOWNMOVE					= 2;
	public static int GEO_UP						= 3;

	public MediaGeoMetryJni()
	{
		mUserID = -1;
		mView = null;
		mDown = false;
		mCallback = null;
		mIsSetListener = false;
	}
	
	/*
	 * @brief local draw command callback interface
	 */
	public interface DrawCommandCallback
	{
		void onCommand(int flag, byte[] cmd, int useSecureSend, int timeout);
	}
	
	/*
	 * @brief GEO module init
	 * @param logPath log file path
	 * @param configPath config file path
	 * @param rkPath RDCodec path
	 * @param userid current user id
	 * @return 0 on success; othre failed
	 */
	public int MediaGeometryInit(String logPath, String configPath, String rkPath, int userid, DrawCommandCallback cb)
	{
		mUserID = userid;
		mCallback = cb;
		return MediaGeometryInitNative(logPath, configPath, rkPath, userid, this);
	}
	
	/*
	 * @brief GEO module uninit
	 */
	public native void MediaGeometryUninit();
	
	/*
	 * @brief Add stream to draw to the GEO module
	 * @param surface SurfaceView to draw into, if no need to draw, can be null, otherwise must be set
	 * @param userid the speaker's user id
	 * @param streamid stream id
	 * @return 0 on success; other failed
	 */
	public int AddStreamInfo(SurfaceView surface, int userid, int streamid)
	{
		int ret = AddStreamInfoNative(null, userid, streamid);
		if(ret == 0)
		{
			if(surface != null)
			{
				mView = surface;
			}
			
//			if(userid == mUserID)
//			{
//				if(mView != null)
//				{
//					mView.setClickable(true);
//					mView.setOnTouchListener(this);
//				}
//			}
		}
		return ret;
	}
	
	/*
	 * @brief Remove stream from GEO module
	 * @param userid speaker's user id
	 * @param streamid stream id
	 * @return 0 on success; other failed
	 */
	public native int RemoveStreamInfo(int userid, int streamid);
	
	/*
	 * @brief Set draw type
	 * @param type draw type, can be GRAPHIC_LAYER_MANAGER,WHITE_BOARD_MANAGER and PROGRESS_CAPTURE_MANAGER
	 * @return 0 on success; other failed
	 */
	public native int SetGraphicManagerActor(int type);
	
	/*
	 * @brief Select draw type or operation
	 * @param sel draw type or operation
	 * @param color color
	 * @param penwidth pen width
	 * @return 0 onsuccess; other failed
	 */
	public int OperateSelect(int sel, int color, int penwidth)
	{
		int ret = OperateSelectNative(sel, color, penwidth);
		if(ret == 0)
		{
			if(!mIsSetListener && mView != null)
			{
				mIsSetListener = true;
				mView.setClickable(true);
				mView.setOnTouchListener(this);
			}
		}
		return ret;
	}
	
	/*
	 * @brief stop drawing element and command
	 * @return 0 on success; other failed 
	 */
	public int OperateCancel()
	{
		if(mIsSetListener)
		{
			mView.setClickable(false);
			mView.setOnTouchListener(null);
			mIsSetListener = false;
		}
		
		return OperateCancelNative();
	}
	
	/*
	 * @brief request to draw
	 * @param userid local userid
	 * @return 0 on success; other failed
	 */
	public int RequestDraw(int userid)
	{
		if(userid != mUserID)
		{
			return -1;
		}
		int ret = RequestDrawNative(userid);
//		if(ret == 0)
//		{
//			if(userid == mUserID)
//			{
//				if(mView != null)
//				{
//					mView.setClickable(true);
//					mView.setOnTouchListener(this);
//				}
//			}
//		}
		return ret;
	}
	
	/*
	 * @brief notify GEO module someone leave meeting
	 * @param userid specify the one who leaves meeting
	 * @return 0 on success; other failed
	 */
	public native int LeaveMeeting(int userid);
	
	/*
	 * @brief edit text
	 * @param text text to edit
	 * @param color color
	 * @param fontSize font size
	 * @param x x position of text
	 * @param y y position of text
	 * @return 0 on success; other failed
	 */
	public native int DrawText(String text, int font, int color, int fontSize, int x, int y);
	
	/*
	 * @brief command operation
	 * @param cmd command type
	 * @return 0 on success; other failed
	 */
	public native int DrawCommand(int cmd);
	
	/*
	 * @brief image operation
	 * @param op screen shot or draw image
	 * @param path for screen shot, the path to save the image; for draw image, the path of the image to be drawed
	 * @param left left position
	 * @param top top position
	 * @param right right position
	 * @param bottom bottom position
	 * @return 0 on success; other failed
	 */
	public native int ImageOperate(int op, String path, int left, int top, int right, int bottom);
	
	/*
	 * @brief push command data to GEO module
	 * @param data command data
	 * @return 0 on success; other failed
	 */
	public native int PushCmdData(byte[] data);
	
	public void OnCommand(int flag, byte[] cmd, int useSecureSend, int timeout)
	{
		if(mCallback != null)
		{
			mCallback.onCommand(flag, cmd, useSecureSend, timeout);
		}
	}

	private native int PushLocalMouseAction(int action, int x, int y);
	private native int MediaGeometryInitNative(String logPath, String configPath, String rkPath, int userid, Object self);
	private native int AddStreamInfoNative(Object surface, int userid, int streamid);
	private native int RequestDrawNative(int userid);
	private native int OperateSelectNative(int sel, int color, int penwidth);
	private native int OperateCancelNative();
	
	static
	{
//		System.loadLibrary("gnustl_shared");
		System.loadLibrary("RDCodec");
		System.loadLibrary("MediaFrameWork");
		System.loadLibrary("MediaGeometry");
	}

	@SuppressLint("ClickableViewAccessibility") @Override
	public boolean onTouch(View v, MotionEvent event) {
		switch(event.getAction())
		{
		case MotionEvent.ACTION_DOWN:
			mDown = true;
			PushLocalMouseAction(GEO_DOWN, (int)event.getX(), (int)event.getY());
			break;
		case MotionEvent.ACTION_MOVE:
			if(mDown)
			{
				PushLocalMouseAction(GEO_DOWNMOVE, (int)event.getX(), (int)event.getY());
			}
			break;
		case MotionEvent.ACTION_UP:
			mDown = false;
			PushLocalMouseAction(GEO_UP, (int)event.getX(), (int)event.getY());
			break;
		}
		return false;
	}
}