package com.channelsoft.sipsdk;

public class SipSdkJni {
  public static MediaPlayListener mMediaPlayListener;

  public static void setMediaPlayListener(MediaPlayListener mediaPlayListener) {
    SipSdkJni.mMediaPlayListener = mediaPlayListener;
  }

  public static synchronized void removeMeidaPlayListener() {
    SipSdkJni.mMediaPlayListener = null;
  }

  public static final int MEDIAPLAY_INIT_SUC = 1;// mediaplay初始化调用成功
  public static final int MEDIAPLAY_INIT_FAILURE = 2;// mediaplay初始化调用失败

  public static final int MEDIAPLAY_RELEASE_SUC = 3;// mediaplayer释放成功
  public static final int MEDIAPLAY_RELEASE_FAILURE = 4;// mediaplayer释放失败

  public static final int MEDIAPLAY_MAC1_NEW_SPEAK_SUC = 5;// macphone1，发言成功
  public static final int MEDIAPLAY_MAC1_NEW_SPEAK_FAILURE = 6;// macphone1，发言失败

  public static final int MEDIAPLAY_MAC2_NEW_SPEAK_SUC = 7;// macphone2，发言成功
  public static final int MEDIAPLAY_MAC2_NEW_SPEAK_FAILURE = 8;// macphone2，发言失败

  //public static final int MEDIAPLAY_MAC1_STOP_SPEAK_SUC = 9;// //macphone1，停止发言成功
  //public static final int MEDIAPLAY_MAC1_STOP_SPEAK_FAILURE = 10;// //macphone1，停止发言失败

  //public static final int MEDIAPLAY_MAC2_STOP_SPEAK_SUC = 11;// //macphone2，停止发言成功
  //public static final int MEDIAPLAY_MAC2_STOP_SPEAK_FAILURE = 12;// //macphone2，停止发言失败

  public static final int MEDIAPLAY_MIC1_REQUEST_RENDER = 13; // mic接收窗口请求渲染
  public static final int MEDIAPLAY_MIC2_REQUEST_RENDER = 14; // mic2接收敞口请求渲染

  public static final int MEDIAPLAY_MIC1_SHARE_DOC_REQUEST_RENDER = 15; // mic1文档共享窗口请求渲染
  public static final int MEDIAPLAY_MIC2_SHARE_DOC_REQUEST_RENDER = 16; // mic2文档共享窗口请求渲染

  public static final int MEDIAPLAY_ADJUST_VIDEOSIZE_TO_VGA_CPU = 17; // 由于CPU原因调整分辨率到VGA
  public static final int MEDIAPLAY_ADJUST_VIDEOSIZE_TO_QVGA_CPU = 18; // 由于CPU原因调整分辨率到QVGA
  public static final int MEDIAPLAY_ADJUST_VIDEOSIZE_TO_VGA_BW = 19; // 由于带宽原因调整分辨率到VGA
  public static final int MEDIAPLAY_ADJUST_VIDEOSIZE_TO_QVGA_BW = 20; // 由于带宽原因调整分辨率到QVGA
  public static final int MEDIAPLAY_AUDIO_IN_DEVICE_INIT_FAILED = 21; //音频采集设备初始化失败
  public static final int MEDIAPLAY_AUDIO_OUT_DEVICE_INIT_FAILED = 22; //音频播放设备初始化失败

  //public static final int MEDIAPLAY_REMOTE_NETWORK_POOR = 21; // 对方网络差
  //public static final int MEDIAPLAY_LOCAL_NETWORK_POOR = 22; // 本地网络差
  //public static final int MEDIAPLAY_ALL_NETWORK_POOR = 23; // 双方网络都差

  private static synchronized void onMessageArrived(int statusCode) {
    if (mMediaPlayListener != null) {
      mMediaPlayListener.onMediaPlayStatusChanged(statusCode);
    }
  }

  /**
   * jni初始化接口，每次都要调用，而且要在创建GLSurfaceView之前调用
   */
  public final static native void initJni();

  /**
   * 向jni层传递视频YUV数据或文档共享rtp数据 函数参数context、id和len可以随便传，jni层没有使用这些值
   * 
   * @param context
   *          这个参数没用
   * @param data
   *          YUV数据指针
   * @param id
   *          发言的mic id，可以不传
   * @param bYUV
   *          指定data是YUV数据还是rtp数据，非零时为YUV数据；零时为rtp数据
   * @param len
   *          data长度，可以不传，下面可以获取长度
   */
  public final static native void pushFrame(long context, byte[] data, int id,
      int bYUV, int len);

  /**
   * 开始文档共享
   * 
   * @param micId
   *          进行文档共享的mic id
   * @param width
   *          文档共享图像的宽度
   * @param height
   *          文档共享图像的高度
   * @return 成功返回进行文档共享的mic id；失败返回其它值
   */
  public final static native int startShareDoc(int micId, int width, int height);

  /**
   * 停止文档共享
   * 
   * @param micId
   *          要停止文档共享的mic id
   * @return 成功返回0；失败返回其它值
   */
  public final static native int stopShareDoc(int micId);

  /**
   * 停止视频链路(保留接口，方便以后扩展使用，用做视频开关)
   * 
   * @param micId
   *          要停止视频链路的mic id
   * @return 成功返回0；失败返回其它值
   */
  public final static native int stopvideoview(int micId);

  /**
   * 启动视频链路(保留接口，方便以后扩展使用，用做视频开关)
   * 
   * @param micId
   *          要启动视频链路的mic id
   * @return 成功返回0；失败返回其它值
   */
  public final static native int startvideoview(int micId);

  /**
   * 初始化MediaPlay
   * 
   * @param jipaddress
   *          本地ip地址
   * @param port1
   *          mic1音频接收端口
   * @param port2
   *          mic1视频接收端口
   * @param port3
   *          mic2音频接收端口
   * @param port4
   *          mic2视频接收端口
   * @param jrc_url
   *          rc地址
   * @param jtoken
   *          token
   * @param jmeetingid
   *          会议id
   * @param juserid
   *          用户id
   * @param jfec_in
   *          FEC输入包个数
   * @param jfec_out
   *          FEC冗余包个数
   * @param agentPorr
   *          MediaServiceAgent session端口
   * @param agentSendReceivePort
   *          MediaServiceAgent收发媒体数据端口
   * @param width
   *          视频图像宽
   * @param height
   *          视频图像高
   * @return 成功返回0；失败返回其它值
   * @warrning 异步接口
   */
  public final static native int initmediaplay(String jipaddress, int port1,
      int port2, int port3, int port4, String jrc_url, String jtoken,
      String jmeetingid, String juserid, int jfec_in, int jfec_out,
      int agentPorr, int agentSendReceivePort, int width, int height);

  /**
   * 设置日志、编解码库路径
   * 
   * @param logcfgpath
   *          LogWriter配置文件路径
   * @param logfileoutpath
   *          日志路径
   * @param rk264path
   *          硬件编解码库路径
   * @return 成功返回0；失败返回其它值
   */
  public final static native int SetInitParam(String logcfgpath,
      String logfileoutpath, String rk264path);

  /**
   * 反初始化MediaPlay
   * @warrning 异步接口
   */
  public final static native void releasemediaplay();

  /**
   * 启动媒体接收链路
   * 
   * @param surf
   *          无用参数，传null即可
   * @param micId
   *          要启动的媒体接收链路mic id
   * @return 成功返回mic id；失败返回其它值
   */
  public final static native int startview(Object surf, int micId);

  /**
   * 停止媒体接收链路
   * 
   * @param micId
   *          要停止的媒体接收链路mic id
   * @return 成功返回0；失败返回其它值
   */
  public final static native int stopview(int micId);

  /**
   * 启动媒体发送链路
   * 
   * @param micId
   *          要启动的媒体发送两路mic id
   * @return 成功返回mic id；失败返回其它值
   * @warrning 异步接口
   */
  public final static native int startspeak(int micId);

  /**
   * 停止媒体发送链路
   * 
   * @param micId
   *          要停止的媒体发送链路mic id
   * @return 成功返回0；失败返回其它值
   */
  public final static native int stopspeak(int micId);

  /**
   * CLSurfaceView创建时调用
   * 
   * @param width
   *          要在该GLSurfaceView上渲染的视频宽度
   * @param height
   *          要在该GLSurfaceView上渲染的视频高度
   */
  public final static native void SurfaceCreated(int width, int height);

  /**
   * GLSurfaceView尺寸变化时调用
   * 
   * @param width
   *          变化后的GLSurfaceView的宽度
   * @param height
   *          变化后的GLSurfaceView的高度
   */
  public final static native void SurfaceChanged(int width, int height);

  /**
   * 向GLSurfaceView上画视频图像
   * 
   * @param micId
   *          将要绘制图像的GLSurfaceView对应的mic id
   */
  public final static native void DrawFrame(int micId);

  /**
   * OpenGL渲染初始化，请在GLSurfaceView创建时调用
   * 
   * @param w
   *          在该GLSurfaceView上渲染的图像宽度
   * @param h
   *          在该GLSurfaceView上渲染的图像高度
   * @param micId
   *          GLSurfaceView对应的mic id
   */
  public final static native void OpenGLRenderInit(int w, int h, int micId);

  /**
   * OpenGL渲染反初始化
   * 
   * @param micId
   *          GLSurfaceView对应的mic id
   */
  public final static native void OpenGLRenderRelease(int micId);

  /**
   * 获取媒体链路qos数据
   * 
   * @return 以json格式返回媒体链路qos数据
   */
  public final static native String GetStreamQosInfo();

  /**
   * 获取cpu和内存使用情况
   * 
   * @return 以json格式返回cpu和内存使用情况
   */
  public final static native String GetEquipmentLoadInfo();

  /**
   * 获取网络相关qos数据
   * 
   * @return 以json格式返回网络相关qos数据
   */
  public final static native String GetNetQosInfo();

  /**
   * 往jni层传特定的按钮键值组合，请在收到以下键值组合时调用此函数 1).121 2).131 3).141
   * 
   * @param key
   *          对应上述键值组合的一个值 2-->121 3-->131 4-->141
   */
  public final static native void OnKeyEvent(int key);

  /**
   * 获取自适应相关qos数据
   * 
   * @return 以json格式返回自适应qos数据
   */
  public final static native String GetAdaptQosInfo();

  /**
   * 重新开始发言
   * 
   * @param micID
   *          要重新发言的mic id
   * @return 成功返回midID；失败返回其它值
   */
  public final static native int reStartspeak(int micID);

  /**
   * 设置网络信息
   * 
   * @param localIP
   *          本地ip地址
   * @param netType
   *          网络类型；[0:有线],[1:wifi],[2:4G],[3:3G]
   * @warrning 请在成功调用initmediaplay后再调用此函数
   */
  public final static native void SetNetInfo(String localIP, int netType, int speed, int rssi);
  
  /*
   * 获取插话接收端口
   * @return 成功返回端口值；失败返回负值
   */
  public final static native int GetInterposeRecvPort();
  
  /*
   * 开始插话发送
   * @return 成功返回0；失败返回其它值
   */
  public final static native int StartInterposeSend();
  
  /*
   * 停止插话发送
   * @return 成功返回0；失败返回其它值
   */
  public final static native int StopInterposeSend();
  
  /*
   * 开始插话接收
   * @param userid 要开始接收的插话端用户id
   * @return 成功返回0；失败返回其它值
   */
  public final static native int StartInterposeRecv(String userid);
  
  /*
   * 停止插话接收
   * @param userid 要停止接收的插话端用户id
   * @return 成功返回0；失败返回其它值
   */
  public final static native int StopInterposeRecv(String userid);
  
    /*
   * 远端发言者关闭摄像头通知
   * @param micID 关闭视频的发言者所在的mic
   */
  public final static native void RemoteSpeakerCloseVideo(int micID);
  
  /*
   * 远端发言者打开摄像头通知
   * @param micID 打开视频的发言者所在的mic
   */
  public final static native void RemoteSpeakerOpenVideo(int micID);
  
  /*
   * 开启语音模式
   */
  public final static native void AudioModeOn();
  
  /*
   * 关闭语音模式
   */
  public final static native void AudioModeOff();
  
  /*
   * 对方停止发言
   * @param micID 停止发言者所在的mic
   */
  public final static native void RemoteStopSpeak(int micID);


  static {
    System.loadLibrary("logwriter");
    System.loadLibrary("MeetingQosInfoReportTools");
    System.loadLibrary("MediaServiceAgent"); 
    System.loadLibrary("agc"); 
//    System.loadLibrary("gnustl_shared"); 
//    System.loadLibrary("tbb"); 
//    System.loadLibrary("Part1Mask"); 
    System.loadLibrary("qnSipPhoneSdk");
    // initJni();
  }
}