package cn.redcdn.butelopensdk;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.PixelFormat;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.SurfaceView;

import cn.redcdn.butelopensdk.MeetingInfo.MemberStatus;
import cn.redcdn.butelopensdk.MeetingInfo.SpeakerType;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKMeetingControlOperationCode;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKOperationCode;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKOperationCode.CurrentRoleStatus;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKOperationCode.LiveStatus;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKOperationCode.MeetingStyleStatus;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKReturnCode;
import cn.redcdn.butelopensdk.constconfig.ButelOpenSDKStatusCode;
import cn.redcdn.butelopensdk.constconfig.CaptionAndThemeStyle;
import cn.redcdn.butelopensdk.constconfig.ChatMessageInfo;
import cn.redcdn.butelopensdk.constconfig.ChatType;
import cn.redcdn.butelopensdk.constconfig.CmdId;
import cn.redcdn.butelopensdk.constconfig.CmdKey;
import cn.redcdn.butelopensdk.constconfig.DataShareResult;
import cn.redcdn.butelopensdk.constconfig.DeviceType;
import cn.redcdn.butelopensdk.constconfig.LayoutInfo;
import cn.redcdn.butelopensdk.constconfig.MediaType;
import cn.redcdn.butelopensdk.constconfig.NetConfig;
import cn.redcdn.butelopensdk.constconfig.NotifyType;
import cn.redcdn.butelopensdk.constconfig.RespModelStatusCode;
import cn.redcdn.butelopensdk.constconfig.SpeakerInfo;
import cn.redcdn.butelopensdk.constconfig.StreamInfo;
import cn.redcdn.butelopensdk.constconfig.ViewInfo;
import cn.redcdn.butelopensdk.media.SMSDKControl;
import cn.redcdn.butelopensdk.media.SMSDKControl.SMSDKListener;
import cn.redcdn.butelopensdk.meetingcontrol.MeetingControl;
import cn.redcdn.butelopensdk.meetingcontrol.MeetingControl.MeetingControlListener;
import cn.redcdn.butelopensdk.meetingcontrol.MeetingControl.MeetingControlReturnCode;
import cn.redcdn.butelopensdk.pushstream.PollAndPushControl;
import cn.redcdn.butelopensdk.screenshare.ScreenShareControl;
import cn.redcdn.butelopensdk.screenshare.ScreenShareControl.ScreenShareListener;
import cn.redcdn.butelopensdk.util.ButelCommonUtil;
import cn.redcdn.butelopensdk.vo.Cmd;
import cn.redcdn.butelopensdk.vo.VideoParameter;
import cn.redcdn.butelopensdk.whiteboard.GeoMetryControl;
import cn.redcdn.commonutil.NetConnectHelper;
import cn.redcdn.commonutil.PhoneStateMonitor;
import cn.redcdn.dep.MeetingHostAgentJNI;
import cn.redcdn.dep.P2PConnCBInterface;
import cn.redcdn.log.CustomLog;

import com.channelsoft.sipsdk.SmSdkJNI;
import com.redcdn.keyeventwrite.KeyEventConfig;
import com.redcdn.keyeventwrite.KeyEventWrite;
import com.serenegiant.usb.DeviceFilter;

public class ButelOpenSDK {
    private final String TAG = ButelOpenSDK.this.getClass().getName();

    // 会议类型(int): 0-语音会议; 1-视频会议;
    public static final int MEETING_TYPE_AUDIO = 0;
    public static final int MEETING_TYPE_VIDEO = 1;
    private int mMeetingShowType = MEETING_TYPE_VIDEO;
    private boolean mFirstOpenCamera = true;

    public static int NO_CAMERA = -1;
    public static int FRONT_CAMERA = 1;
    public static int BACK_CAMERA = 0;
    public static int UVC_CAMERA = 2;

    private Timer renderVideoTimer;
    private TimerTask renderVideoTimerTask;

    private boolean mRecordStatus = false;

    private boolean mOpenMic = true;

    private Map<String, Long> mReceiveMsgMap;

    // true 打开 false 关闭
    private boolean mOuterMicState = true; // 会议外麦克风状态
    private boolean mOuterCameraState = true; // 会议外摄像头状态

    /**
     * 通知回调接口，当收到服务器端等下发的命令时，通过该回调接口通知到UI端
     */
    public interface ButelOpenSDKNotifyListener {
        public void onNotify(int notifyType, Object object);
    }

    /**
     * 操作回调接口，对于UI端的异步操作请求通过该回调接口通知到UI端
     */
    public interface ButelOpenSDKOperationListener {
        public void onOperationSuc(Object object);

        public void onOperationFail(Object object);
    }

    private InitStatus mInitStatus = InitStatus.NULL;

    private enum InitStatus {
        NULL, INITED
    }

    private Context mContext;

    private String mToken;

    // STP 操作码
    private int mMeetingControlOperation;

    private int mStatus;

    @SuppressWarnings("unused")
    @Deprecated
    private int mDeviceResolutionType;

    private String mCfgPath;

    private String mLogFileOutPath;

    private String mRcURL;

    private JSONArray mParticipatorList = new JSONArray();

    private String mDeviceType;

    private boolean mSelectUvcCamera = false;// 是否设置使用uvc摄像头

    private int mCloseCameraId;// 添加关闭摄像头背景图片Id

    private MeetingInfo mMeetingInfo;

    private SMSDKControl mSMSDKControl;

    private int mSignalStrengthLevel = 0; // 网络信号强度。0~4，其中0表示未获取到，1表示最强，4表示信号强度最弱

    private String mRelayIdWhiteList; // relayId 白名单

    private GeoMetryControl mGeoMetryControl;// 图元管理模块

    private boolean mWhiteBoardFlag = true; // 是否启用白板

    private SurfaceView mWhiteBoardView;// 白板view

    private String mWantMasterId;

    private int mCodecType = 0; // 编码类型

    private int mBigImage; // 大背景
    private int mSmallImage; // 小背景

    private boolean mAdminFlag = false; // 是否是超级管理员
    // private int operationCode = 0;
    //
    // private int mInviteOperateCode = -1; // 邀请PC的操作标识码
    //
    // private Map<String, Integer> mQueryRetryMap = new HashMap<String,
    // Integer>();
    //
    // private P2PConnCBInterface mP2PConnCBInterface;

    private PollAndPushControl mPollAndPushControl; // 轮询推流控制类
    private boolean mPollFlag = true; // 轮询开关(默认打开)
    private boolean mCaptionAndThemeState = true; // 主题字幕开关(默认打开)
    private Map<String, String> mUICmdMap;

    private SMSDKListener mSMSDKListener = new SMSDKListener() {
        @Override
        public void onStatus(int status, Object obj) {
            dispatchSMSDKResp(status, obj);
        }
    };

    private MeetingControl mMeetingControl;

    private MeetingControlListener mMeetingControlListener = new MeetingControlListener() {
        @Override
        public void onReceiveCmd(Cmd cmd) {
            CustomLog.i(TAG,
                    "ButelOpenSDK::MeetingControlListener onReceiveCmd.id = "
                            + cmd.getCmdId());
            // 在加入会议回应之前收到其它命令，是需要丢弃，不做处理的。
            if (cmd.getCmdId() != CmdId.JOIN_MEETING_RESP
                    && (mInitStatus != InitStatus.INITED || mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.JOIN_MEETING)) {
                CustomLog.i(TAG, "在加入会议回应之前收到其它命令，直接丢弃");
                return;
            }
            dispatchCmd(cmd);
        }

        @Override
        public void onReceiveWhiteBoardData(byte[] data) {
//            CustomLog
//                    .i(TAG,
//                            "ButelOpenSDK::MeetingControlListener onReceiveWhiteBoardData");

            if (mWhiteBoardFlag && mGeoMetryControl != null) {
                mGeoMetryControl.analyzeData(data);
            }
        }

    };

    private ScreenShareControl mScreenShareControl;

    private ScreenShareListener mScreenShareListener = new ScreenShareListener() {
        @Override
        public void stopShareing() {
            CustomLog.i(TAG, "ButelOpenSDK::stopShareing()");

            SpeakerInfo speaker = mMeetingInfo
                    .getSpeakerByAccountId(mMeetingInfo.accountId);
            if (speaker == null) {
                CustomLog.w(TAG, "ButelOpenSDK::stopShareing() 未发言，不用停止屏幕分享!");
                // mScreenShareControl.forceStopSharing(
                // ScreenShareControl.STOP_SHARING_STOP_SPEAK, "未发言");
                return;
            }

            if (speaker.getScreenShareStatus() != SpeakerInfo.SCREEN_SHARING) {
                CustomLog.w(TAG,
                        "warning! ButelOpenSDK::stopShareing()，已经未进行屏幕分享，无需停止");
                return;
            }

            StreamInfo docStream = speaker
                    .getStreamInfo(StreamInfo.RESOURCE_TYPE_DOC_VIDEO);
            if (docStream == null) {
                CustomLog.w(TAG,
                        "error! ButelOpenSDK::stopShareing() 无文档共享视频流信息");
                return;
            }

            int streamId = docStream.getStreamId();

            if (mSMSDKControl == null) {
                return;
            }

            // 取消媒体模块的文档视频流发布
            mSMSDKControl.unPublish(streamId);
            // PC端主动停止的屏幕分享，需要通知到stp服务器，以让其他终端收到
            mMeetingControl.applyForStopScreenShareing(mToken);
            // 判断mediaType是否是doc，add，streamId不能再此位置設置為-1，stoplocalrender還需要使用
            // mScreenShareControl.setStreamId(-1);

            // 此位置不能删除信息，需要在stoplocalRender里面实现
            // speaker.setScreenShareInfo(SpeakerInfo.SCREEN_NORMAL);

            handleNotifyListenerListOnNotify(NotifyType.STOP_SHARE_DOC, null);
        }

        @Override
        public void startShareing(int height, int width, int bitrate,
                                  int filmformat) {
            CustomLog.i(TAG, "ButelOpenSDK::startShareing() height: " + height
                    + " |width: " + width + " |bitrate: " + bitrate
                    + " filmformat: " + filmformat);
            SpeakerInfo speaker = mMeetingInfo
                    .getSpeakerByAccountId(mMeetingInfo.accountId);
            if (speaker == null) {
                CustomLog.e(TAG, "ButelOpenSDK::startShareing() 未发言，无法开始屏幕分享!");
                if (mScreenShareControl != null) {
                    mScreenShareControl.forceStopSharing(
                            ScreenShareControl.STOP_SHARING_STOP_SPEAK, "未发言");
                }
                return;
            }

            int resourceId = StreamInfo.RESOURCE_TYPE_DOC_VIDEO;
            int syncId = speaker.getSyncId();
            // 向媒体模块发布文档视频流
            int streamId = mSMSDKControl.publishDocVideo(resourceId, width,
                    height, bitrate, syncId, filmformat);
            speaker.setScreenShareInfo(SpeakerInfo.SCREEN_SHARING);
            speaker.setDocVideoParam(new VideoParameter(width, height,
                    VideoParameter.DEFAULT_FPS, bitrate));

            if (mScreenShareControl != null) {
                // 判断mediaType是否是doc，add
                mScreenShareControl.setStreamId(streamId);
            }

            CustomLog.i(TAG, "ButelOpenSDK::startShareing() streamId: "
                    + streamId);

            StreamInfo docStream = new StreamInfo(streamId, resourceId, syncId,
                    MediaType.TYPE_DOC_VIDEO);

            speaker.addStreamInfo(resourceId, docStream);
            CustomLog.d(TAG, "speaker = " + speaker.toString());
            JSONArray jsonArray = new JSONArray();
            JSONObject docStreamJson = new JSONObject();
            try {
                docStreamJson.put(CmdKey.RESOURCE_ID,
                        String.valueOf(resourceId));
                docStreamJson.put(CmdKey.MEDIA_TYPE, SpeakerInfo
                        .ParseSmMediaTypeToStp(MediaType.TYPE_DOC_VIDEO));
                docStreamJson.put(
                        CmdKey.EXTRAINFO,
                        Cmd.createExtraInfo(syncId,
                                speaker.getDocVideoResolution()[0],
                                speaker.getDocVideoResolution()[1]));
            } catch (JSONException e) {
                CustomLog.e(TAG, "error ! 构造json 出错! " + e.toString());
            }

            jsonArray.put(docStreamJson);

            // 向STP服务器发布文档视频流
            mMeetingControl.askPublishOrUnPublishStreams(mToken,
                    Cmd.STREAM_OPERATOR_TYPE_PUBLISH, jsonArray);

            handleNotifyListenerListOnNotify(NotifyType.START_SHARE_DOC, null);
        }

        @Override
        public void onNameChanged(String name) {
            CustomLog.i(TAG,
                    "ButelOpenSDK::ScreenShareListener onNameChanged name: "
                            + name);
            // handleNotifyListenerListOnNotify(NotifyType.SHARE_NAME_CHANGE,
            // name);
        }

        @Override
        public void frameDateArrive(byte[] fdata) {
            // todo nothing! 在 ScreenShareControl 内部进行数据推送
        }

        @Override
        public void applyForStop() {
            CustomLog.i(TAG, "ButelOpenSDK::ScreenShareListener applyForStop");
            mMeetingControl.applyForStopScreenShareing(mToken);
        }

        @Override
        public void applyForStart() {
            CustomLog.i(TAG, "ButelOpenSDK::ScreenShareListener applyForStart");
            int ret = mMeetingControl.applyForStartScreenSharing(mToken);
            if (ret != 0) {
                CustomLog.d(TAG,
                        "ButelOpenSDK::applyForStart() 申请开始屏幕分享失败，ret: 0");
                if (mScreenShareControl != null) {
                    mScreenShareControl
                            .responseApplyForStart(ScreenShareControl.REFUSE_CONNECT_UI_BUSY);
                }
            }
        }
    };

    private Handler mTimeoutCmdHandler = new Handler() {
        public void handleMessage(Message msg) {
            CustomLog.i(TAG, "mTimeoutCmdHandler 相关操作无返回，触发timeout 认为操作失败处理");
            if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_SPEAK) {
                callbackAskForSpeakOperation(false, null);
            } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_STOP_SPEAK) {
                callbackAskForStopSpeakOperation(false, null);
            } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.LOCK_MEETING) {
                mMeetingInfo.intentLockInfo = 1 - mMeetingInfo.intentLockInfo;
                callbackLockMeetingOperation(false, null);

            } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.GIVE_MIC) {
                callbackGiveMicOperation(false, null);
            }
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        }
    };

    /******** ButelSDK对外回调 ********/
    private List<ButelOpenSDKNotifyListener> mNotifyListenerList;

    private List<ButelOpenSDKOperationListener> mJoinMeetingListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mExitMeetingListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForSpeakListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForStopSpeakListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mGiveMicListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mLockMeetingListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForOpenCameraListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForCloseCameraListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForOpenMicPhoneListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForCloseMicPhoneListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForOpenLoudspeakerListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForCloseLoudspeakerListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    /*** 主持人模式功能 ***/
    private List<ButelOpenSDKOperationListener> mMasterChangeMeetingModeListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mMasterSetUserStartSpeakOnMicListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mMasterSetUserStopSpeakOnMicListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mUserAskForRaiseHandListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForStartLiveListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mMasterSetMainSpeakerListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mMasterKickUserListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForStartRecordingListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mAskForStopRecordingListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mQueryPCStateListenerList = new ArrayList<ButelOpenSDK.ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mInvitePCListenerList = new ArrayList<ButelOpenSDK.ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mMasterTransferMasterIdListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mMasterAskForEndMeetingListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    private List<ButelOpenSDKOperationListener> mChangeMeetingChatStateListenerList = new ArrayList<ButelOpenSDKOperationListener>();

    /***
     * 构造函数
     *
     * @param context
     *            Activity context
     */
    public ButelOpenSDK(Context context) {
        CustomLog.i(TAG, "构造ButelOpenSDK");
        mContext = context;
    }

    /**
     * 初始化方法。
     *
     * @param accountId            视讯号
     * @param accountName          参会名称
     * @param meetingId            会议id
     * @param deviceResolutionType 设备分辨率类型,不再使用，可传0
     * @param cfgPath              配置文件路径
     * @param logFileOutPath       日志输出路径
     * @param rcURL                路由地址，从nps获取
     * @param deviceType           设备类型
     * @param closecameraid        关闭摄像头时的贴图资源id，用户关闭摄像头自己发言窗口后贴背景图
     * @return 0 :初始化成功
     * <p>
     * -1 :初始化会控 失败
     * <p>
     * -2 :初始化SM_SDK 失败
     * <p>
     * -3 :初始化屏幕分享失败
     * <p>
     * <0 :其它失败
     * @see DeviceType
     */
    public int init(String accountId, String accountName, int meetingId,
                    int deviceResolutionType, String cfgPath, String logFileOutPath,
                    String rcURL, String deviceType, int closecameraid,
                    boolean selectSystemCamera) {
        CustomLog.i(TAG, "初始化ButelOpenSDK" + this + "accountId=" + accountId
                + "accountName=" + accountName + "meetingI" + meetingId
                + "deviceResolutionType=" + deviceResolutionType
                + "logFileOutPath=" + logFileOutPath + "cfgPath=" + cfgPath
                + "rcURL=" + rcURL + "deviceType=" + deviceType
                + "selectSystemCamera:" + String.valueOf(selectSystemCamera));

        if (mMeetingInfo == null) {
            mMeetingInfo = new MeetingInfo();
        }

        mMeetingInfo.meetingId = meetingId;
        mMeetingInfo.accountId = accountId;
        mMeetingInfo.accountName = accountName;
        mCloseCameraId = closecameraid;
        mDeviceResolutionType = deviceResolutionType;
        mCfgPath = cfgPath;
        mLogFileOutPath = logFileOutPath;
        mRcURL = rcURL;
        mDeviceType = deviceType;
        if (selectSystemCamera) {
            mSelectUvcCamera = false;
        } else {
            mSelectUvcCamera = true;
        }

        mReceiveMsgMap = new HashMap<>();
        mUICmdMap = new HashMap<>();

        // 初始化MeetingControl
        int result = 0;
        result = initMeetingControl();
        if (result != 0) {
            CustomLog.w(TAG, "初始化会控失败:" + result);
            return -1;
        } else {
            CustomLog.i(TAG, "初始化会控成功");
        }

        // 初始化SMSDK
        result = initSMSDKControl(mSelectUvcCamera);
        if (result != 0) {
            CustomLog.w(TAG, "初始化SM_SDK 失败:" + result);
            return -2;
        } else {
            CustomLog.i(TAG, "初始SM_SDK成功");
        }

        if (mWhiteBoardFlag) {
            result = initGeoMetryControl();
            if (result != 0) {
                CustomLog.w(TAG, "初始化图元管理模块失败:" + result);
                return -4;
            } else {
                CustomLog.i(TAG, "初始图元管理模块成功");
            }
        }

        // 初始化屏幕分享
//        result = initScreenSharingControl();
//        if (result <= 0) {
//            CustomLog.w(TAG, "初始化屏幕分享失败:" + result);
//            return -3;
//        } else {
//            CustomLog.i(TAG, "初始屏幕分享成功");
//        }

        // initHostAgent();
//        CustomLog.i(TAG, "设置HostAgent回调完成");

        if (mPollFlag) {
            initPollAndPushControl();
        }
        CustomLog.i(TAG, "初始化ButelOpenSDK完成" + this);
        mInitStatus = InitStatus.INITED;
        return 0;
    }

    private void initPollAndPushControl() {
        if (mPollAndPushControl == null && mPollFlag) {
            CustomLog.d(TAG, "initPollAndPushControl");
            mPollAndPushControl = new PollAndPushControl(mContext,
                    mSMSDKControl, mMeetingInfo.accountId);
        }
    }

    private void releasePollAndPushControl() {
        if (mPollAndPushControl != null && mPollFlag) {
            CustomLog.d(TAG, "releasePollAndPushControl");
            mPollAndPushControl.release();
            mPollAndPushControl = null;
        }
    }

    /**
     * 添加ButelOpenSDK通知listener
     *
     * @param notifyListener 需要添加的listener对象
     */
    public int addButelOpenSDKNotifyListener(
            ButelOpenSDKNotifyListener notifyListener) {
        if (mNotifyListenerList == null) {
            mNotifyListenerList = new ArrayList<ButelOpenSDKNotifyListener>();
        }
        if (!mNotifyListenerList.contains(notifyListener)) {
            mNotifyListenerList.add(notifyListener);
        }
        return 0;
    }

    /**
     * 移除ButelOpenSDK通知listener
     *
     * @param notifyListener 需要移除的listener对象
     * @return <p>
     * true: 移除成功
     * <p>
     * false: 移除失败
     */
    public boolean removeButelOpenSDKNotifyListener(
            ButelOpenSDKNotifyListener notifyListener) {
        if (mNotifyListenerList != null
                && mNotifyListenerList.contains(notifyListener)) {
            return mNotifyListenerList.remove(notifyListener);
        } else {
            return false;
        }
    }

    /**
     * 加入会议
     *
     * @param joinMeetingListener 加入会议监听回调
     * @param token               登录成功后返回的token
     * @param adminId             不再使用，可以传""
     * @return 同步返回操作结果，加入会议是否成功将通过 ButelOpenSDKOperationListener 异步返回
     * <p>
     * 0：操作成功
     * <p>
     * !=0：操作 失败
     */
    public int joinMeeting(ButelOpenSDKOperationListener joinMeetingListener,
                           String token, String adminId) {
        CustomLog.i(TAG, "发送加入会议命令  mMeetingControlOperation = "
                + mMeetingControlOperation);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "加入会议失败,存在正在操作事件: "
                    + ButelOpenSDKReturnCode.RETURN_PENDING_ERROR);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;// 所有正在操作中事件，同步结果都是返回-3
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.JOIN_MEETING;
        mToken = token;
        mMeetingInfo.adminId = adminId;
        // 如果有自动发言请求，需要添加
        int returnCode = mMeetingControl.joinMeeting(mToken,
                mMeetingInfo.adminId, isSpeaking() ? MemberStatus.IS_SPEAKING
                        : MemberStatus.IS_LISTENING, 0,
                mMeetingInfo.accountName, 0, mRelayIdWhiteList,
                mMeetingInfo.isAutoSpeak, mAdminFlag, mPollFlag);
        if (returnCode != MeetingControlReturnCode.SUCCESS) {
            CustomLog.w(TAG, "加入会议失败"
                    + ButelOpenSDKReturnCode.RETURN_OTHER_ERROR);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mJoinMeetingListenerList.add(joinMeetingListener);
        CustomLog.i(TAG, "加入会议操作请求成功，等待服务器返回！");
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 申请发言，若replaceId 不为空，表示抢占该发言者
     *
     * @param askForSpeakListener 操作结果回调
     * @param replaceId           <p>
     *                            "": 由服务器判断是否有空闲mic可供发言
     *                            <p>
     *                            !="": 被替换的视讯号id
     * @return <p>
     * 0: 操作成功发出，操作结果需要等待ButelOpenSDKOperationListener 回调返回
     * <p>
     * <0: 操作失败
     */
    public int askForSpeak(ButelOpenSDKOperationListener askForSpeakListener,
                           String replaceId) {
        if (isSpeaking()) {
            CustomLog.w(TAG, "正在发言，不能在申请发言");
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }
        CustomLog.i(TAG, "发送申请发言命令  mMeetingControlOperation = "
                + mMeetingControlOperation);
        mAskForSpeakListenerList.add(askForSpeakListener);
        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_SPEAK;

        int returnCode = mMeetingControl.askForSpeak(mToken, replaceId);
        if (returnCode != MeetingControlReturnCode.SUCCESS) {
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        CustomLog.i(TAG, "发送申请发言命令结束");
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 申请打开摄像头:向stp服务器申请发布视频流，申请成功后操作打开摄像头、向SMSDK发布视频流
     *
     * @param askForOpenCameraListener 操作结果回调listener
     * @return <p>
     * 0: 操作成功发出，操作结果需要等待ButelOpenSDKOperationListener 回调返回
     * <p>
     * !=0 : 操作失败
     */
    public int askForOpenCamera(
            ButelOpenSDKOperationListener askForOpenCameraListener) {
        CustomLog.d(TAG, "ButelOpenSDK::askForOpenCamera()申请打开摄像头");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
        if (speaker == null) {
            CustomLog.w(TAG,
                    "ButelOpenSDK::askForOpenCamera error! 当前未发言，无法打开摄像头");
            return -1;
        }

        if (speaker.getCamStatus() == SpeakerInfo.CAM_STATUS_OPEN) {
            CustomLog.w(TAG, "ButelOpenSDK::openCamera error! 当前已是打开状态。");
            return -2;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG,
                    "askForOpenCamera,正在执行其他操作,mMeetingControlOperation = "
                            + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        // 申请发布视频流
        JSONArray jsonArray = new JSONArray();
        JSONObject videoStreamJson = new JSONObject();
        try {
            videoStreamJson.put(CmdKey.RESOURCE_ID,
                    String.valueOf(StreamInfo.RESOURCE_TYPE_VIDEO));
            videoStreamJson.put(CmdKey.MEDIA_TYPE,
                    SpeakerInfo.ParseSmMediaTypeToStp(MediaType.TYPE_VIDEO));
            videoStreamJson.put(
                    CmdKey.EXTRAINFO,
                    Cmd.createExtraInfo(speaker.getSyncId(),
                            speaker.getVideoResolution()[0],
                            speaker.getVideoResolution()[1]));

            jsonArray.put(videoStreamJson);
            CustomLog.d(TAG, "jsonArray=" + jsonArray.toString());

            int result = mMeetingControl.askPublishOrUnPublishStreams(mToken,
                    Cmd.STREAM_OPERATOR_TYPE_PUBLISH, jsonArray);
            if (result != MeetingControlReturnCode.SUCCESS) {
                CustomLog.w(TAG, "向 STP 发布流失败 result: " + result);
                return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
            } else {
                mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_OPEN_CAMERA;
                mAskForOpenCameraListenerList.add(askForOpenCameraListener);
            }
        } catch (JSONException e) {
            CustomLog.e(TAG, "json 构造出错 : " + e.getMessage());
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 申请关闭摄像头
     *
     * @param askForCloseCameraListener 操作结果回调listener
     * @return <p>
     * =0: 操作成功发出，操作结果需要等待ButelOpenSDKOperationListener 回调返回
     * <p>
     * !=0: 操作失败
     */
    public int askForCloseCamera(
            ButelOpenSDKOperationListener askForCloseCameraListener) {
        CustomLog.i(TAG, "ButelOpenSDK::askForCloseCamera() 申请关闭摄像头");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
        if (speaker == null) {
            CustomLog.w(TAG,
                    "ButelOpenSDK::askForCloseCamera error! 当前未发言，无法关闭麦克风");
            return -1;
        }

        if (speaker.getCamStatus() == SpeakerInfo.CAM_STATUS_CLOSE) {
            CustomLog
                    .w(TAG, "ButelOpenSDK::askForCloseCamera error! 当前已是关闭状态。");
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        // 申请取消发布视频流

        JSONArray jsonArray = new JSONArray();
        JSONObject videoStreamJson = new JSONObject();
        try {
            videoStreamJson.put(CmdKey.RESOURCE_ID,
                    String.valueOf(StreamInfo.RESOURCE_TYPE_VIDEO));
            videoStreamJson.put(CmdKey.MEDIA_TYPE,
                    SpeakerInfo.ParseSmMediaTypeToStp(MediaType.TYPE_VIDEO));
            videoStreamJson.put(
                    CmdKey.EXTRAINFO,
                    Cmd.createExtraInfo(speaker.getSyncId(),
                            speaker.getVideoResolution()[0],
                            speaker.getVideoResolution()[1]));

            jsonArray.put(videoStreamJson);
            CustomLog.d(TAG, "jsonArray=" + jsonArray.toString());

            int result = mMeetingControl.askPublishOrUnPublishStreams(mToken,
                    Cmd.STREAM_OPERATOR_TYPE_UNPUBLISH, jsonArray);
            if (result != 0) {
                CustomLog.e(TAG, "向 STP 取消发布视频流失败 result: " + result);
                return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
            } else {
                mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_CLOSE_CAMERA;
                mAskForCloseCameraListenerList.add(askForCloseCameraListener);
            }
        } catch (JSONException e) {
            CustomLog.e(TAG, "json 构造出错 : " + e.getMessage());
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 打开其他参会方视频画面 本地操作，会触发恢复订阅该参会方的视频流
     *
     * @param accountId 参会方视讯号
     * @return <p>
     * 0：成功
     * <p>
     * !=0: 失败
     */
    public int openRemoteVideo(String accountId) {
        CustomLog.d(TAG, "ButelOpenSDK::openRemoteVideo accountId: "
                + accountId);

        SpeakerInfo speaker = getSpeakerInfoById(accountId);

        int VideoOffReason = SpeakerInfo.VIDEO_OFF_REASON_NULL;
        if (speaker != null) {
            if (speaker.getVideoStatus() == SpeakerInfo.VIDEO_OPEN) {
                CustomLog.d(TAG,
                        "ButelOpenSDK::openRemoteVideo 视频画面已是打开状态，无需再次操作");
                return 0;
            } else {
                VideoOffReason = speaker.getVideoOffReason();
                speaker.setVideoStatus(SpeakerInfo.VIDEO_OPEN, -1);
            }
        }

        int result = 0;

        if (mMeetingShowType == MEETING_TYPE_AUDIO) {
            CustomLog.d(TAG, "mMeetingShowType==MEETING_TYPE_AUDIO,不订阅视频流");
        } else {
            result = resumeSubscribe(accountId, MediaType.TYPE_VIDEO);
        }

        if (result == 0) {
            CustomLog.d(TAG, "ButelOpenSDK::openRemoteVideo 打开视频画面成功");
        } else {
            CustomLog.e(TAG, "ButelOpenSDK::openRemoteVideo 打开视频画面失败. result: "
                    + result);
            if (speaker != null) {
                speaker.setVideoStatus(SpeakerInfo.VIDEO_OFF, VideoOffReason);
            }
        }
        return result;
    }

    /**
     * 关闭其他参会方视频画面,本地操作，会触发暂停订阅该参会方的视频流
     *
     * @param accountId 参会方视讯号
     * @return <p>
     * 0：成功
     * <p>
     * !=0: 失败
     */
    public int closeRemoteVideo(String accountId) {
        CustomLog.d(TAG, "ButelOpenSDK::closeRemoteVideo accountId: "
                + accountId);
        int result = pauseSubscribe(accountId, MediaType.TYPE_VIDEO);

        if (result == 0 || result == -3) {
            CustomLog.d(TAG, "ButelOpenSDK::openRemoteVideo 关闭视频画面成功");
            getSpeakerInfoById(accountId).setVideoStatus(SpeakerInfo.VIDEO_OFF,
                    SpeakerInfo.VIDEO_OFF_REASON_MANUAL);
            result = 0;
        }

        return result;
    }

    /**
     * 打开其他参会方文档视频画面，本地操作，会触发恢复订阅该参会方文档视频流
     *
     * @param accountId 参会方视讯号
     * @return <p>
     * 0：成功
     * <p>
     * !=0: 失败
     */
    public int openRemoteDocVideo(String accountId) {
        CustomLog.d(TAG, "ButelOpenSDK::openRemoteDocVideo accountId: "
                + accountId);

        SpeakerInfo speaker = getSpeakerInfoById(accountId);

        if (speaker != null) {
            if (speaker.getDocVideoStatus() == SpeakerInfo.DOC_VIDEO_OPEN) {
                CustomLog.d(TAG,
                        "ButelOpenSDK::openRemoteDocVideo 文档视频画面已是打开状态，无需再次操作");
                return 0;
            } else {
                speaker.setDocVideoStatus(SpeakerInfo.DOC_VIDEO_OPEN);
            }
        }

        int result = 0;

        if (mMeetingShowType == MEETING_TYPE_AUDIO) {
            CustomLog.d(TAG, "mMeetingShowType==MEETING_TYPE_AUDIO,不订阅视频流");
        } else {
            result = resumeSubscribe(accountId, MediaType.TYPE_DOC_VIDEO);
        }

        if (result == 0) {
            CustomLog.d(TAG, "ButelOpenSDK::openRemoteVideo 打开文档视频画面成功");
        } else {
            if (speaker != null) {
                speaker.setDocVideoStatus(SpeakerInfo.DOC_VIDEO_OFF);
            }
        }

        return result;
    }

    /**
     * 关闭其他参会方文档视频画面,本地操作，会触发暂停订阅该参会方文档视频流
     *
     * @param accountId 参会方视讯号
     * @return <p>
     * 0：成功
     * <p>
     * !=0: 失败
     */
    public int closeRemoteDocVideo(String accountId) {
        CustomLog.d(TAG, "ButelOpenSDK::closeRemoteDocVideo accountId: "
                + accountId);
        int result = pauseSubscribe(accountId, MediaType.TYPE_DOC_VIDEO);

        if (result == 0) {
            CustomLog.d(TAG, "ButelOpenSDK::openRemoteVideo 关闭文档视频画面成功");
            getSpeakerInfoById(accountId).setDocVideoStatus(
                    SpeakerInfo.DOC_VIDEO_OFF);
        }

        return result;
    }

    @Deprecated
    /**
     * 主持人设置主讲人
     *
     * @param mainSpeakerId
     *          主讲人视讯号
     * @param masterSetMainSpeakerListener
     *          设置结果回调监听
     * @return
     * <p> 0: 操作成功发出，操作结果需要等待 masterSetMainSpeakerListener 回调返回
     * <p> !=0: 操作失败
     */
    public int masterSetMainSpeaker(String mainSpeakerId,
                                    ButelOpenSDKOperationListener masterSetMainSpeakerListener) {
        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有操作权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        SpeakerInfo speakerInfo = getSpeakerInfoById(mainSpeakerId);
        if (speakerInfo == null) {
            CustomLog.w(TAG, "指定的主持人非发言人，无法设置");
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int result = mMeetingControl.masterSetSpeaker(mToken, mainSpeakerId);
        if (result != 0) {
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        mMasterSetMainSpeakerListenerList.add(masterSetMainSpeakerListener);
        CustomLog.i(TAG, "mMasterChangeMeetingModeListenerList 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.MASTER_SET_MAIN_SPEAKER;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 主持人修改会议模式，当前只有指定发言、自由模式
     * <p>
     * 仅有主持人拥有操作权限
     *
     * @param meetingStyle                    会议模式 {@link ButelOpenSDKOperationCode.MeetingStyleStatus}
     * @param masterChangeMeetingModeListener 操作结果回调listener
     * @return <p>
     * 0: 操作成功发出，操作结果需要等待ButelOpenSDKOperationListener 回调返回
     * <p>
     * !=0: 操作失败
     */
    public int masterChangeMeetingMode(int meetingStyle,
                                       ButelOpenSDKOperationListener masterChangeMeetingModeListener) {

        CustomLog.d(TAG,
                "ButelOpenSDK::masterChangeMeetingMode() meetingStyle: "
                        + meetingStyle);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if ((mMeetingInfo.mMeetingStyle == meetingStyle)) {
            CustomLog.w(TAG, "模式相同，不能改变模式");
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有操作权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.masterChangeMeetingMode(mToken,
                meetingStyle);
        if (resultId != 0) {
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mMasterChangeMeetingModeListenerList
                .add(masterChangeMeetingModeListener);
        CustomLog.i(TAG, "mMasterChangeMeetingModeListenerList 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.MASTER_CHANGE_MEETING_MODE;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 主持人指定发言，只有主持人拥有操作权限
     *
     * @param beSpeakedUserId                      发言者视讯号
     * @param beSpeakedUserName                    发言者参会名称
     * @param replaceId                            被替换的发言者视讯号，如果为""，表示由服务器决定空闲mic进行发言
     * @param masterSetUserStartSpeakOnMicListener 操作结果回调 listener
     * @return <p>
     * 0: 操作成功，操作结果将通过 masterSetUserStartSpeakOnMicListener 异步返回
     * <p>
     * <0: 操作失败
     */
    public int masterSetUserStartSpeakOnMic(String beSpeakedUserId,
                                            String beSpeakedUserName, String replaceId,
                                            ButelOpenSDKOperationListener masterSetUserStartSpeakOnMicListener) {
        CustomLog.i(TAG,
                "ButelOpenSDK::masterSetUserStartSpeakOnMic() beSpeakedUserId "
                        + beSpeakedUserId + " | beSpeakedUserName: "
                        + beSpeakedUserName + " | replaceId:" + replaceId);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有此权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        SpeakerInfo speaker = mMeetingInfo
                .getSpeakerByAccountId(beSpeakedUserId);
        if (speaker != null) {
            CustomLog.w(TAG, "指定发言的参会方，已经在发言");
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.masterSetUserStartSpeakOnMic(mToken,
                beSpeakedUserId, replaceId);
        if (resultId != 0) {
            CustomLog.w(TAG, "指定发言失败 resultId: " + resultId);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mMasterSetUserStartSpeakOnMicListenerList
                .add(masterSetUserStartSpeakOnMicListener);
        CustomLog.i(TAG, "masterSetUserStartSpeakOnMic 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.MASTER_SET_USER_START_SPEAK_ON_MIC;
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 主持人取消发言者的发言，只有主持人拥有操作权限
     *
     * @param beSpeakedUserId                     发言者视讯号
     * @param masterSetUserStopSpeakOnMicListener 操作结果回调listener
     * @return </p> = 0: 操作成功，操作结果将通过 masterSetUserStartSpeakOnMicListener 异步返回
     * </p> < 0: 操作失败
     */
    public int masterSetUserStopSpeakOnMic(String beSpeakedUserId,
                                           ButelOpenSDKOperationListener masterSetUserStopSpeakOnMicListener) {
        CustomLog.i(TAG,
                "ButelOpenSDK::masterSetUserStopSpeakOnMic() beSpeakedUserId "
                        + beSpeakedUserId);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有此权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        SpeakerInfo speaker = getSpeakerInfoById(beSpeakedUserId);
        if (speaker == null) {
            CustomLog.w(TAG, "指定取消发言的参会方未做发言，无法取消");
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.masterSetUserStopSpeakOnMic(mToken,
                beSpeakedUserId);
        if (resultId != 0) {
            CustomLog.w(TAG, "取消指定发言失败 resultId: " + resultId);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mMasterSetUserStopSpeakOnMicListenerList
                .add(masterSetUserStopSpeakOnMicListener);
        CustomLog.i(TAG, "masterSetUserStopSpeakOnMic 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.MASTER_SET_USER_STOP_SPEAK_ON_MIC;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 主持人踢人功能，触发服务器将该参会方请出会议室，其他参会方会收到被踢者退出会议的通知
     *
     * @param beKickedUserId         被请出会议室参会方id
     * @param masterKickUserListener 操作结果回调listener
     * @return <p>
     * 0: 操作成功，操作结果将通过 masterKickUserListener 异步返回
     * <p>
     * <0: 操作失败
     */
    public int masterKickUser(String beKickedUserId,
                              ButelOpenSDKOperationListener masterKickUserListener) {
        CustomLog.i(TAG, "ButelOpenSDK::masterKickUser() beKickedUserId "
                + beKickedUserId);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有此权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.masterKickUser(mToken, beKickedUserId);
        if (resultId != 0) {
            CustomLog.w(TAG, "主持人踢人失败。 resultId: " + resultId);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mMasterKickUserListenerList.add(masterKickUserListener);
        CustomLog.i(TAG, "masterKickUser 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.MASTER_KICK_USER;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 主持人权限移交
     * <p>
     * 仅有主持人拥有操作权限
     *
     * @param newMasterId                    新主持人视讯号
     * @param masterTransferMasterIdListener 操作结果回调listener
     * @return <p>
     * 0: 操作成功发出，操作结果需要等待ButelOpenSDKOperationListener 回调返回
     * <p>
     * !=0: 操作失败
     */
    public int masterTransferMasterId(String newMasterId,
                                      ButelOpenSDKOperationListener masterTransferMasterIdListener) {

        CustomLog.d(TAG, "ButelOpenSDK::masterTransferMasterId() newMasterId: "
                + newMasterId);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if ((mMeetingInfo.mMeetingMasterAccountId.equals(newMasterId))) {
            CustomLog.w(TAG, "新旧主持人ID相同，不进行移交主持人操作");
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有操作权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.masterTransferMasterId(mToken,
                newMasterId);
        if (resultId != 0) {
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        mWantMasterId = newMasterId;
        mMasterTransferMasterIdListenerList.add(masterTransferMasterIdListener);
        CustomLog.i(TAG, "masterTransferMasterId 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.MASTER_TRANSFER_MASTERID;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 主持人申请结束会议请求命令
     * <p>
     * 仅有主持人拥有操作权限
     *
     * @param masterAskForEndMeetingListener 操作结果回调listener
     * @return <p>
     * 0: 操作成功发出，操作结果需要等待ButelOpenSDKOperationListener 回调返回
     * <p>
     * !=0: 操作失败
     */
    public int masterAskForEndMeeting(
            ButelOpenSDKOperationListener masterAskForEndMeetingListener) {

        CustomLog.d(TAG, "ButelOpenSDK::masterAskForEndMeeting()");

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有操作权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.masterAskForEndMeeting(mToken);
        if (resultId != 0) {
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mMasterAskForEndMeetingListenerList.add(masterAskForEndMeetingListener);
        CustomLog.i(TAG, "masterAskForEndMeeting 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.MASTER_ASK_FOR_END_MEETING;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 开始录制请求
     *
     * @param password                     (String):录制会议密码
     * @param askForStartRecordingListener (ButelOpenSDKOperationListener)
     * @return 0:成功 非0:失败
     */
    public int askForStartRecording(String password,
                                    ButelOpenSDKOperationListener askForStartRecordingListener) {
        CustomLog.i(TAG, "ButelOpenSDK::askForStartRecording(),password:"
                + password);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有此权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.askForStartRecording(mToken, password);
        if (resultId != 0) {
            CustomLog.w(TAG, "开始录制请求失败。 resultId: " + resultId);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mAskForStartRecordingListenerList.add(askForStartRecordingListener);
        CustomLog.i(TAG, "askForStartRecording 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_START_RECORDING;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 停止录制请求
     *
     * @param askForStopRecordingListener (ButelOpenSDKOperationListener)
     * @return 0:成功 非0:失败
     */
    public int askForStopRecording(
            ButelOpenSDKOperationListener askForStopRecordingListener) {
        CustomLog.i(TAG, "ButelOpenSDK::askForStopRecording()");

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有此权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.askForStopRecording(mToken);
        if (resultId != 0) {
            CustomLog.w(TAG, "停止录制请求。 resultId: " + resultId);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mAskForStopRecordingListenerList.add(askForStopRecordingListener);
        CustomLog.i(TAG, "askForStopRecording 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_STOP_RECORDING;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 举手。告知主持人有发言需求，将在主持人UI中做举手标识
     * <p>
     * 只有普通参会方才有操作权限
     *
     * @param userAskForRaiseHandListener 操作结果回调listener
     * @return <p>
     * = 0: 操作成功，操作结果将通过 userAskForRaiseHandListener 异步返回
     * <p>
     * < 0: 操作失败
     */
    public int userAskForRaiseHand(
            ButelOpenSDKOperationListener userAskForRaiseHandListener) {

        CustomLog.d(TAG, "ButelOpenSDK::userAskForRaiseHand()");

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.COMMON_ROLE) {
            CustomLog.w(TAG, "参会方才能举手");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.userAskForRaiseHand(mToken);
        if (resultId != 0) {
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mUserAskForRaiseHandListenerList.add(userAskForRaiseHandListener);
        CustomLog.i(TAG, "USER_ASK_FOR_RAISE_HAND 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.USER_ASK_FOR_RAISE_HAND;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 开启直播,只有主持人拥有操作权限
     *
     * @param liveName   直播名称
     * @param askForLive 操作结果回调listener
     * @return <p>
     * 0: 操作成功，操作结果将通过 userAskForRaiseHandListener 异步返回
     * <p>
     * <0: 操作失败
     */
    public int askForStartLive(String liveName,
                               ButelOpenSDKOperationListener askForLive) {
        CustomLog.d(TAG, "ButelOpenSDK::askForStartLive 开启直播 liveName: "
                + liveName);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mLiveStatus == LiveStatus.LIVING_NOW) {
            CustomLog.w(TAG, "已经是直播状态，不能开启直播");
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不能开启直播");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int resultId = mMeetingControl.askForStartLive(mToken, liveName);
        if (resultId != 0) {
            CustomLog.w(TAG, "开启直播失败 result: " + resultId);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mAskForStartLiveListenerList.add(askForLive);
        CustomLog.i(TAG, "askForStartLive 发送请求");
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_START_LIVE;

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    @Deprecated
    /**
     * 切换语音、视频模式。语音模式下只接收、传输音频流，且如果自己是发言者，将关闭视频画面传输，节约网络带宽
     *
     * @param modeType
     *          切换的目标模式 <Li>0: 视频模式 <Li>1: 语音模式
     * @param askForCloseOrOpenCameraListener
     *          操作结果回调 listener
     * @return </p> == 0: 操作成功，操作结果将通过 switchVideoOrAudioMode 异步返回 </p> < 0: 操作失败
     *         </p> == 1: 操作已经成功，不再进行异步回调返回
     */
    public int switchVideoOrAudioMode(int modeType,
                                      ButelOpenSDKOperationListener askForCloseOrOpenCameraListener) {
        CustomLog.i(TAG, "ButelOpenSDK::switchVideoOrAudioMode() modeType: "
                + modeType);

        // if (mInitStatus != InitStatus.INITED) {
        // CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
        // return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        // }
        //
        // if ((modeType == 0 && mMeetingInfo.mIsVideoMode)
        // || (modeType == 1 && !mMeetingInfo.mIsVideoMode)) {
        // CustomLog.w(TAG, "当前视频模式和准备切换到的视频模式一样，不需要切换");
        // return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        // }
        //
        // if (mMeetingControlOperation !=
        // ButelOpenSDKMeetingControlOperationCode.NULL) {
        // CustomLog.w(TAG,
        // "switchVideoOrAudioMode,正在执行其他操作,mMeetingControlOperation = "
        // + mMeetingControlOperation);
        // return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        // }
        // CustomLog.e(TAG, "modeType = " + modeType + "  mIsVideoMode= "
        // + mMeetingInfo.mIsVideoMode);
        //
        // // 切到视频模式
        // if (modeType == 0) {
        //
        // if (mMeetingInfo.mIsVideoMode) {
        // CustomLog.w(TAG, "视频模式下，切换视频模式，返回-1");
        // return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        // }
        // if (isSpeaking()) {
        // CustomLog.w(TAG, "语音模式下,在发言,切视频模式,操作askForOpenCamera");
        // mMeetingControlOperation =
        // ButelOpenSDKMeetingControlOperationCode.SWITCH_VIDEO_OR_AUDIO_MODE;
        // int returnCode = mMeetingControl.askForOpenCamera(mToken);
        // if (returnCode != MeetingControlReturnCode.SUCCESS) {
        // mMeetingControlOperation =
        // ButelOpenSDKMeetingControlOperationCode.NULL;
        // return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        // }
        // mSwitchTOVideoModeListenerList.add(askForCloseOrOpenCameraListener);
        // return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
        // }
        // CustomLog.i(TAG, "语音模式下,未发言,切视频模式,操作AudioModeOff");
        //
        // mMeetingInfo.mCameraIsOpen = true;
        // mSMSDKControl.audioModeOff();
        // mMeetingInfo.mIsVideoMode = !mMeetingInfo.mIsVideoMode;
        // mMeetingControlOperation =
        // ButelOpenSDKMeetingControlOperationCode.NULL;
        // return ButelOpenSDKReturnCode.RETURN_NONEED_CALLBACK_SUCCESS;
        // }
        // if (modeType == 1) {// 切到语音模式
        // if (!mMeetingInfo.mIsVideoMode) {
        // CustomLog.w(TAG, "语音模式下，切换语音模式，返回-1");
        // return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        // }
        // if (isSpeaking()) {
        // if (mMeetingInfo.mCameraIsOpen) {
        // CustomLog.i(TAG, "视频模式下，在发言,摄像头已打开,切到语音模式，操作askForCloseCamera");
        // mMeetingControlOperation =
        // ButelOpenSDKMeetingControlOperationCode.SWITCH_VIDEO_OR_AUDIO_MODE;
        // int returnCode = mMeetingControl.askForCloseCamera(mToken);
        // if (returnCode != MeetingControlReturnCode.SUCCESS) {
        // mMeetingControlOperation =
        // ButelOpenSDKMeetingControlOperationCode.NULL;
        // return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        // }
        // mSwitchToAudioModeListenerList.add(askForCloseOrOpenCameraListener);
        // return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
        // } else {
        // CustomLog.i(TAG, "视频模式，在发言，摄像头未打开,切到语音模式，操作AudioModeOn");
        // mMeetingControlOperation =
        // ButelOpenSDKMeetingControlOperationCode.NULL;
        // mSMSDKControl.audioModeOn();
        // mMeetingInfo.mIsVideoMode = !mMeetingInfo.mIsVideoMode;
        // mMeetingControlOperation =
        // ButelOpenSDKMeetingControlOperationCode.NULL;
        // return ButelOpenSDKReturnCode.RETURN_NONEED_CALLBACK_SUCCESS;
        // }
        // }
        //
        // CustomLog.i(TAG, "视频模式，未发言，切到语音模式，操作AudioModeOn");
        // mSMSDKControl.audioModeOn();
        // mMeetingInfo.mIsVideoMode = !mMeetingInfo.mIsVideoMode;
        // mMeetingControlOperation =
        // ButelOpenSDKMeetingControlOperationCode.NULL;
        // return ButelOpenSDKReturnCode.RETURN_NONEED_CALLBACK_SUCCESS;
        // }
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 申请停止发言
     *
     * @param askForStopSpeakListener 操作结果回调 listener
     * @return <p>
     * 0: 操作成功，操作结果将通过 askForStopSpeakListener 异步返回
     * <p>
     * <0: 操作失败
     */
    public int askForStopSpeak(
            ButelOpenSDKOperationListener askForStopSpeakListener) {
        CustomLog.i(TAG, "ButelOpenSDK::askForStopSpeak() 申请停止发言");

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (!isSpeaking()) {
            CustomLog.w(TAG, "当前没有在发言，不能停止发言");
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_STOP_SPEAK) {
            CustomLog.w(TAG, "当前正在操作申请停止发言");
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        CustomLog.i(TAG, "发送申请停止发言命令  mMeetingControlOperation = "
                + mMeetingControlOperation);
        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int returnCode = mMeetingControl.askForStopSpeak(mToken);
        if (returnCode != MeetingControlReturnCode.SUCCESS) {
            CustomLog.w(TAG, "申请停止发言失败 result: " + returnCode);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_STOP_SPEAK;
        mAskForStopSpeakListenerList.add(askForStopSpeakListener);
        CustomLog.i(TAG, "发送申请停止发言命令结束");
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 传麦：触发将自己的发言状态传递给指定非发言者
     *
     * @param giveMicListener 操作结果回调listener
     * @param beSpeakedUserId 传麦人员视讯号
     * @return </p> == 0: 操作成功，操作结果将通过 askForStopSpeakListener 异步返回 </p> < 0:
     * 操作失败
     */
    public int giveMic(ButelOpenSDKOperationListener giveMicListener,
                       String beSpeakedUserId) {
        CustomLog.i(TAG, "ButelOpenSDK::giveMic() 传麦给  " + beSpeakedUserId);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (!isSpeaking()) {
            CustomLog.w(TAG, "当前没有在发言，不能传麦");
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        CustomLog.i(TAG, "发送传麦命令  mMeetingControlOperation = "
                + mMeetingControlOperation);
        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.GIVE_MIC;

        int returnCode = mMeetingControl.giveMic(mToken, beSpeakedUserId);
        if (returnCode != MeetingControlReturnCode.SUCCESS) {
            CustomLog.w(TAG, "传麦失败 result: " + returnCode);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mGiveMicListenerList.add(giveMicListener);
        CustomLog.i(TAG, "发送传麦命令结束");
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 会议加解锁，只有主持人有权限操作：1代表加锁，0代表无锁
     *
     * @param lockMeetingListener 操作结果回调listener
     * @param lockstatus          目标状态：1代表加锁，0代表无锁
     * @return <p>
     * 0: 操作成功，操作结果将通过 lockMeetingListener 异步返回
     * <p>
     * <0: 操作失败
     */
    public int lockMeeting(ButelOpenSDKOperationListener lockMeetingListener,
                           int lockstatus) {
        CustomLog
                .i(TAG, "ButelOpenSDK::lockMeeting() curLockStatus: "
                        + mMeetingInfo.intentLockInfo + " desLockStatus: "
                        + lockstatus);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有此权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        if (lockstatus == mMeetingInfo.intentLockInfo) {
            CustomLog.w(TAG, "当前已经是加锁或者解锁状态，不需要加解锁 lockInfo=" + lockstatus
                    + "mCurMeetingInfo.intentLockInfo="
                    + mMeetingInfo.intentLockInfo);
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        CustomLog.i(TAG, "发送加|解锁会议命令  mMeetingControlOperation = "
                + mMeetingControlOperation);

        int returnCode = mMeetingControl.lockMeeting(mToken, lockstatus);
        if (returnCode != MeetingControlReturnCode.SUCCESS) {
            CustomLog.w(TAG, "加/解锁失败， result: " + returnCode);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.LOCK_MEETING;
        mLockMeetingListenerList.add(lockMeetingListener);
        mMeetingInfo.intentLockInfo = lockstatus; // 先将目标状态置为期待状态，等待操作结果回来再更新为真实状态
        CustomLog.i(TAG, "发送加|解锁会议命令结束" + "lockInfo=" + lockstatus);
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 退出会议
     *
     * @return <p>
     * 0: 操作成功
     * <p>
     * <0: 操作失败
     */
    public int exitMeeting() {
        CustomLog.i(TAG, "ButelOpenSDK::exitMeeting()");

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "处理退出会议失败  mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.EXIT_MEETING;
        mMeetingControl.exitMeeting(mToken, 1); // 强制退出
        CustomLog.i(TAG, "处理退出会议结束");
        return ButelOpenSDKReturnCode.RETURN_NONEED_CALLBACK_SUCCESS;
    }

    /**
     * 切换前、后置摄像头,手机版使用。
     *
     * @return <p>
     * 0: 操作成功
     * <p>
     * <0: 操作失败
     */
    public int changeCamera() {
        CustomLog.d(TAG, "ButelOpenSDK::changeCamera() 切换前后置摄像头");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        return mSMSDKControl.changeCamera();
    }

    @Deprecated
    public int stopWatchCamera() {
        CustomLog.i(TAG, "ButelOpenSDK::stopWatchCamera() 关闭摄像头监控");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (isSpeaking()) {
            // mMediaPlaySDK.stopWatchCamera();
        }
        return ButelOpenSDKReturnCode.RETURN_NONEED_CALLBACK_SUCCESS;
    }

    @Deprecated
    public int startWatchCamera() {
        CustomLog.i(TAG, "ButelOpenSDK::startWatchCamera() 重新开启摄像头监控");
        if (isSpeaking()) {
            // mMediaPlaySDK.startWatchCamera();
        }
        return ButelOpenSDKReturnCode.RETURN_NONEED_CALLBACK_SUCCESS;
    }

    /**
     * 反初始化
     */
    public void release() {
        CustomLog.i(TAG, "ButelOpenSDK::release()");
        if (renderVideoTimer != null) {
            renderVideoTimer.cancel();
            renderVideoTimer = null;
        }
        if (renderVideoTimerTask != null) {
            renderVideoTimerTask.cancel();
            renderVideoTimerTask = null;
        }
        if (mReceiveMsgMap != null) {
            mReceiveMsgMap.clear();
            mReceiveMsgMap = null;
        }

        if (mUICmdMap != null) {
            mUICmdMap.clear();
            mUICmdMap = null;
        }
        try {
            if (mContext != null) {
                PhoneStateMonitor.getInstance(mContext).release();
            }
        } catch (Exception e) {
            CustomLog.e(TAG, "release:" + e.toString());
        }
        releasePollAndPushControl();
        releaseMeetingControl();
        releaseGeoMetryControl();
        releaseSmsdkControl();
        releaseScreenShareControl();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
    }

    /**
     * 退会反初始化
     */
    public void exitMeetingRelease() {
        CustomLog.i(TAG, "ButelOpenSDK::exitMeetingRelease()");
        try {
            if (mContext != null) {
                PhoneStateMonitor.getInstance(mContext).release();
            }
        } catch (Exception e) {
            CustomLog.e(TAG, "exitMeetingRelease:" + e.toString());
        }
        Thread thread = new Thread(new ReleaseRunnable());
        thread.start();
    }

    class ReleaseRunnable implements Runnable {
        @Override
        public void run() {
            CustomLog.i(TAG, "ButelOpenSDK::release() ReleaseRunnable start");
            if (renderVideoTimer != null) {
                renderVideoTimer.cancel();
                renderVideoTimer = null;
            }
            if (renderVideoTimerTask != null) {
                renderVideoTimerTask.cancel();
                renderVideoTimerTask = null;
            }
            releasePollAndPushControl();
            releaseMeetingControl();
            releaseGeoMetryControl();
            releaseSmsdkControl();
            releaseScreenShareControl();
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            CustomLog.i(TAG, "ButelOpenSDK::release() ReleaseRunnable end");
        }
    }

    private void releaseMeetingControl() {
        CustomLog.i(TAG, "ButelOpenSDK::releaseMeetingControl()");
        if (mMeetingControl != null) {
            mMeetingControl.release();
            mMeetingControl = null;
        }
    }

    private void releaseSmsdkControl() {
        CustomLog.i(TAG, "ButelOpenSDK::releaseSmsdkControl()");
        if (mSMSDKControl != null) {
            mSMSDKControl.release();
            mSMSDKControl = null;
        }
    }

    private void releaseScreenShareControl() {
        CustomLog.i(TAG, "ButelOpenSDK::releaseScreenShareControl");
        if (mScreenShareControl != null) {
            mScreenShareControl.release();
            mScreenShareControl = null;
        }
    }

    private void releaseGeoMetryControl() {
        CustomLog.i(TAG, "ButelOpenSDK::releaseGeoMetryControl");
        if (mGeoMetryControl != null) {
            mGeoMetryControl.release();
            mGeoMetryControl = null;
        }
    }

    // 处理 STP 命令
    private void dispatchCmd(Cmd cmd) {
        CustomLog.i(TAG, "ButelOpenSDK::dispatchCmd 分发命令：" + cmd.getCmdId());

        if (cmd.getStatus() == RespModelStatusCode.ACCOUNT_CONTEXT_NOT_EXIST) {
            CustomLog.w(TAG, "发现致命错误ACCOUNT_CONTEXT_NOT_EXIST，直接汇报给上层");
            handleNotifyListenerListOnNotify(
                    NotifyType.ACCOUNT_CONTEXT_NOT_EXIST, cmd);
            return;
        }
        if (cmd.getStatus() == RespModelStatusCode.MANAGER_NOT_EXIST) {
            CustomLog.w(TAG, "发现致命错误MANAGER_NOT_EXIST，直接汇报给上层");
            handleNotifyListenerListOnNotify(NotifyType.MANAGER_NOT_EXIST, cmd);
            return;
        }
        if (cmd.getStatus() == RespModelStatusCode.MEETING_CONTEXT_INIT_FAILED) {
            CustomLog.w(TAG, "发现致命错误MEETING_CONTEXT_INIT_FAILED，直接汇报给上层");
            handleNotifyListenerListOnNotify(
                    NotifyType.MEETING_CONTEXT_INIT_FAILED, null);
            return;
        }
        if (cmd.getStatus() == RespModelStatusCode.OTHER_EXCEPTION) {
            CustomLog.w(TAG, "发现致命错误OTHER_EXCEPTION，直接汇报给上层");
            handleNotifyListenerListOnNotify(NotifyType.OTHER_EXCEPTION, null);
            return;
        }

        if (cmd.getStatus() == RespModelStatusCode.TOKEN_TIME_OUT) {

            if (cmd.getCmdId() == CmdId.JOIN_MEETING_RESP) {
                String str = KeyEventConfig.ADD_MEETING_BY_NUBE + "_fail_"
                        + mMeetingInfo.accountId + "_加入会议失败,错误码="
                        + cmd.getStatus();

                KeyEventWrite.write(str);
            }
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            mStatus = ButelOpenSDKStatusCode.EXCEPTION;
            handleNotifyListenerListOnNotify(NotifyType.TOKEN_TIME_OUT, null);
            return;
        }
        CustomLog.i(TAG, "此处分析命令返回状态，未发现致命错误，走正常流程");
        switch (cmd.getCmdId()) {
            case CmdId.JOIN_MEETING_RESP:
                handleJoinMeetingResp(cmd);
                break;
            case CmdId.EXIT_MEETING_RESP:
                handleExitMeetingResp(cmd);
                break;
            case CmdId.ASK_FOR_SPEAK_RESP:
                handleAskForSpeakResp(cmd);
                break;
            case CmdId.ASK_FOR_STOP_SPEAK_RESP:
                handleAskForStopSpeakResp(cmd);
                break;
            case CmdId.ASK_FOR_OPENORCLOSE_CAMERA_RESP:
                if (true) {
                    CustomLog
                            .d(TAG,
                                    "ButelOpenSDK::handleAskForOpenOrCloseCameraResp() 使用流操作处理，不再单独处理打开、关闭摄像头命令");
                    return;
                }
                // handleAskForOpenOrCloseCameraResp(cmd);
            case CmdId.REMOTE_USER_OPENORCLOSE_CAMERA:
                if (true) {
                    CustomLog
                            .d(TAG,
                                    "ButelOpenSDK::handleAskForOpenOrCloseCameraResp() 使用流操作处理，不再单独处理远端打开、关闭摄像头命令");
                    return;
                }
                // handleRemoteUserOpenOrCloseCamera(cmd);
            case CmdId.NEW_GET_PARTICIPATORS_RESP:
                handleGetParticipatorsResp(cmd);
                break;
            case CmdId.GIVE_MIC_RESP:
                handleGiveMicResp(cmd);
                break;
            case CmdId.LOCK_MEETING_RESP:
                handleLockMeetingResp(cmd);
                break;
            case CmdId.BE_GRABED:
                handleBeGrabed(cmd);
                break;
            case CmdId.EXCEPTION:
                handleException(cmd);
                break;
            case CmdId.SPEAKER_ON_LINE:
                handleSpeakerOnLine(cmd);
                break;
            case CmdId.SPEAKER_OFF_LINE:
                handleSpeakerOffLine(cmd);
                break;
            case CmdId.START_SPEAK:
                handleStartSpeak(cmd);
                break;
            case CmdId.STOP_SPEAK:
                handleStopSpeak(cmd);
                break;
            case CmdId.PERSON_JOIN_MEETING:
                handlePersonJoinMeeting(cmd);
                break;
            case CmdId.PERSON_EXIT_MEETING:
                handlePersonExitMeeting(cmd);
                break;
            case CmdId.NOTIFY_LOCK_INFO:
                handleServiceNotifyLockInfo(cmd); // 服务器通知 加/解 锁状态改变
                break;
            case CmdId.MASTER_CHANGE_MEETING_MODE_RESP:
                handleNotifyMasterMeetingModeResp(cmd);
                break;
            case CmdId.MASTER_SET_USER_START_SPEAK_ON_MIC_RESP:
                handleMasterSetUserStartSpeakOnMicResp(cmd);
                break;
            case CmdId.ASK_FOR_OPERATE_LOUDSPEAKER_RESP:
                handleUserAskForOperateLoudspeakerResp(cmd);
                break;
            case CmdId.MASTER_SET_USER_STOP_SPEAK_ON_MIC_RESP:
                handleMasterSetUserStopSpeakOnMicResp(cmd);
                break;
            case CmdId.MASTER_KICK_USER_RESP:
                handleMasterKickUserResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_BE_KICKED:
                handleServerNoticeBeKicked(cmd);
                break;
            case CmdId.USER_ASK_FOR_RAISE_HAND_RESP:
                handleUserAskForRaiseHandResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_MEETING_MODE_CHANGE:
                handleServerNoticeMeetingModeChange(cmd);
                break;
            case CmdId.SERVER_NOTICE_USER_ASK_FOR_RAISE_UP:
                handleServerNoticeUserAskForRaiseUp(cmd);
                break;
            case CmdId.ASK_FOR_START_LIVE_RESP:
                handleAskForStartLiveResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_START_LIVE:
                handleServerNoticeStartLive(cmd);
                break;
            case CmdId.APPLY_FOR_START_SCREEN_SHAREING_RESP: // 申请开始屏幕分享回应
                handleApplayForStartScreenSharingResp(cmd);
                break;
            case CmdId.APPLY_FOR_STOP_SCREEN_SHAREING_RESP: // 申请停止屏幕分享回应
                handleApplayForStopScreenSharingResp(cmd);
                break;
            case CmdId.SERVER_RETURN_APPLY_FOR_START_SCREEN_SHAREING:// 服务器返回屏幕分享申请结果：只发给申请方
                handleServerReturnApplayForSharingResult(cmd);
                break;
            case CmdId.SERVER_NOTICE_START_SCREEN_SHAREING: // 服务器通知新的屏幕分享用户产生
                handleServerNoticeStartScreenSharing(cmd);
                break;
            case CmdId.SERVER_RETURN_STOP_SCREEN_SHAREING: // 服务器返回停止屏幕分享结果：只发给申请方
                handleServerReturnSharingResult(cmd);
                break;
            case CmdId.SERVER_NOTICE_STOP_SCREEN_SHAREING: // 服务器通知屏幕分享用户失效
                handleServerNoticeStopScrrenSharing(cmd);
                break;
            case CmdId.MASTER_SET_MAIN_SPEAKER_RESP: // 主持人设置主讲人回应
                handleSetMainSpeakerResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_NEW_MAIN_SPEAKER: // 新主持人产生
                handleServerNoticeNewMainSpeaker(cmd);
                break;
            case CmdId.SERVER_NOTICE_MAIN_SPEAKER_INVALID: // 主持人失效
                handleServerNoticeMainSpeakerInvalid(cmd);
                break;
            case CmdId.ASK_PUBLISH_OR_UNPUBLISH_STREAMS_RESP: // 发布、取消发布流回应
                handleAskOpertionStreamResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_PUBLISH_OR_UNPUBLISH_STREAMS: // 收到服务器下发的发布、取消发布流
                handleServerNoticeStreamChanged(cmd);
                break;
            case CmdId.SERVER_NOTICE_UPDATE_STREAMS: // 收到服务器下发的流更新命令
                handleServerNoticeUpdateStream(cmd);
                break;
            case CmdId.SERVER_NOTICE_MASTER_OPERATE_USER_LOUDSPEAKER: // 服务器通知主持人要求某人操作扬声器命令
                handleServerNoticeMasterOperateUserLoudspeaker(cmd);
                break;
            case CmdId.SERVER_NOTICE_MASTER_OPERATE_USER_STREAM:// 服务器通知用户更改媒体状态
                handleServerNoticeMasterOperateUserStream(cmd);
                break;
            case CmdId.SERVER_NOTICE_USER_CHANGE_LOUDSPEAKER_STATE: // 用户更改扬声器状态
                handleServerNoticeUserChangeLoudspeakerState(cmd);
                break;
            case CmdId.ASK_FOR_START_RECORDING_RESP: // 开始录制请求 回应
                handleAskForStartRecordingResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_START_RECORDING: // 服务器通知终端会议开始录制
                handleServerNoticeStartRecording(cmd);
                break;
            case CmdId.ASK_FOR_STOP_RECORDING_RESP: // 停止录制请求回应
                handleAskForStopRecordingResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_STOP_RECORDING: // 服务器通知终端会议停止录制
                handleServerNoticeStopRecording(cmd);
                break;
            case CmdId.UI_SEND_CMD_ID:
                // handleReceiveLocalRecord(cmd);
                handleUICmd(cmd);
                break;
            case CmdId.MASTER_TRANSFER_MASTERID_RESP:
                handleMasterTransferMasterIdResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_MASTERID_CHANGED:
                handleServerNoticeMasterIdChanged(cmd);
                break;
            case CmdId.MASTER_ASK_FOR_END_MEETING_RESP:
                handleMasterAskForEndMeetingResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_END_MEETING:
                handleServerNoticeMasterEndMeeting(cmd);
                break;
            case CmdId.UI_BLACKBOX_CMD_ID:
                handleUIBlackCmd(cmd);
                break;
            case CmdId.SERVER_NOTICE_BLACK_CACHE_CMD:
                CustomLog.d(TAG, "服务器群发黑盒缓存命令");
                handleBlackCacheCmd(cmd);
                break;
            case CmdId.UI_GET_BLACK_CACHE_CMD_RESP:
                CustomLog.d(TAG, "UI端主动获取黑盒缓存命令响应");
                handleBlackCacheCmd(cmd);
                break;
            case CmdId.MASTER_CHANGE_MEETING_IM_STATE_RESP:
                CustomLog.d(TAG, "主持人改变会议聊天状态回应");
                handleMasterChangeMeetingChatStateResp(cmd);
                break;
            case CmdId.SERVER_NOTICE_MEETING_IM_STATE_CHANGE:
                CustomLog.d(TAG, "服务器广播会议聊天状态改变");
                handleServerNoticeMeetingChatStateChange(cmd);
                break;
            case CmdId.SERVER_NOTICE_USER_POLL_CMD:
                CustomLog.d(TAG, "服务器通知某人被轮询命令");
                handleServerPollCmd(cmd);
                break;
            default:
                handleInvalidCmd(cmd);
                break;
        }
    }

    private void handleServerNoticeMeetingChatStateChange(Cmd cmd) {
        int chatState = cmd.getMeetingChatState();
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::handleServerNoticeMeetingChatStateChange() 服务器通知会议聊天状态改变  chatState: "
                                + chatState);
        mMeetingInfo.meetingChatState = chatState;
        handleNotifyListenerListOnNotify(NotifyType.NOTIFY_MEETING_CHAT_STATE_CHANGE, cmd);
    }

    private void handleMasterChangeMeetingChatStateResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG, "ButelOpenSDK::handleMasterChangeMeetingChatStateResp() 处理改变会议聊天状态回应命令: "
                + status);
        if (status == RespModelStatusCode.SUCCESS) {
            callbackChangeMeetingChatStateOperation(true, cmd);
        } else {
            mMeetingInfo.intentLockInfo = 1 - mMeetingInfo.intentLockInfo;
            callbackChangeMeetingChatStateOperation(false, cmd);
        }
        CustomLog.i(TAG, "处理改变会议聊天状态回应命令结束");
    }

    private void handleBlackCacheCmd(Cmd cmd) {
        String key = cmd.getCmdKey();
        String value = cmd.getCmdValue();
        if ("host_layout".equals(key)) {
            if (mMeetingInfo.mMeetingMasterAccountId
                    .equals(mMeetingInfo.accountId)) {
                CustomLog.d(TAG, "自己是主持人，不响应服务器的黑盒缓存命令广播");
                return;
            }
            try {
                LayoutInfo layoutInfo = new LayoutInfo();
                JSONObject valueObj = new JSONObject(value);
                layoutInfo.accountId = valueObj.optString("accountId");
                layoutInfo.layoutType = valueObj.optInt("layoutType");
                layoutInfo.fullScreenViewId = valueObj
                        .optInt("fullScreenViewId");
                JSONArray viewsArray = valueObj.optJSONArray("views");
                if (viewsArray != null) {
                    for (int i = 0; i < viewsArray.length(); i++) {
                        JSONObject viewObj = viewsArray.optJSONObject(i);
                        if (viewObj != null) {
                            ViewInfo viewInfo = new ViewInfo();
                            viewInfo.id = viewObj.optInt("id");
                            viewInfo.nube = viewObj.optString("nube");
                            viewInfo.name = viewObj.optString("name");
                            viewInfo.type = viewObj.optInt("type");

                            layoutInfo.addView(viewInfo);
                        }
                    }
                }
                if (layoutInfo.getViews().size() > 0) {
                    handleNotifyListenerListOnNotify(
                            NotifyType.NOTIFY_LAYOUT_CHANGE, layoutInfo);
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }

        } else if ("meet_caption".equals(key)) {
            try {
                CaptionAndThemeStyle captionAndThemeStyle = new CaptionAndThemeStyle();
                JSONObject valueObj = new JSONObject(value);
                captionAndThemeStyle.state = valueObj.optInt("state");
                captionAndThemeStyle.content = valueObj.optString("content");
                captionAndThemeStyle.size = valueObj.optInt("size");
                captionAndThemeStyle.color = valueObj.optString("color");
                captionAndThemeStyle.align = valueObj.optInt("align");
                captionAndThemeStyle.bottomMargin = valueObj
                        .optDouble("bottom_margin");
                captionAndThemeStyle.speed = valueObj.optInt("speed");
                captionAndThemeStyle.bgcolor = valueObj.optString("bgcolor");
                captionAndThemeStyle.bgopacity = valueObj
                        .optDouble("bgopacity");

                handleNotifyListenerListOnNotify(
                        NotifyType.NOTIFY_MEET_CAPTION, captionAndThemeStyle);
            } catch (JSONException e) {
                e.printStackTrace();
            }
        } else if ("meet_theme".equals(key)) {
            try {
                CaptionAndThemeStyle captionAndThemeStyle = new CaptionAndThemeStyle();
                JSONObject valueObj = new JSONObject(value);
                captionAndThemeStyle.state = valueObj.optInt("state");
                captionAndThemeStyle.content = valueObj.optString("content");
                captionAndThemeStyle.size = valueObj.optInt("size");
                captionAndThemeStyle.color = valueObj.optString("color");
                captionAndThemeStyle.align = valueObj.optInt("align");
                captionAndThemeStyle.bgcolor = valueObj.optString("bgcolor");
                captionAndThemeStyle.bgopacity = valueObj
                        .optDouble("bgopacity");

                handleNotifyListenerListOnNotify(NotifyType.NOTIFY_MEET_THEME,
                        captionAndThemeStyle);
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    }

    private void handleUICmd(Cmd cmd) {
        CustomLog.d(TAG, "handleUICmd");
        String targetId = cmd.getTargetId();
        String fromId = cmd.getFromId();
        String data = cmd.getData();
        String timestamp = String.valueOf(cmd.getTimestamp());

        if (mUICmdMap != null && mUICmdMap.containsKey(timestamp)) {
            CustomLog.e(TAG, "handleUICmd 收到重复命令，不处理!");
            return;
        } else {
            mUICmdMap.put(timestamp, "");
        }

        if (targetId == null || (targetId.length() > 0 && !targetId.equals(mMeetingInfo.accountId))) {
            CustomLog.e(TAG, "接收者不为自己，不做处理！");
            return;
        }
        CustomLog.d(TAG, "handleUICmd data=" + data);
        if (data != null && data.length() > 0) {
            if (data.startsWith("#DPMSG#")) {
                String json = data.substring(7);
                try {
                    JSONObject msgJson = new JSONObject(json);
                    String accountId = msgJson.optString("accountId");
                    String cmdType = msgJson.optString("cmd");
                    String msg = msgJson.optString("msg");
                    if (mMeetingInfo.mMeetingMasterAccountId.equalsIgnoreCase(accountId)) {
                        if ("TOASTMSG".equalsIgnoreCase(cmdType) && mPollFlag) {
                            handleNotifyListenerListOnNotify(NotifyType.MASTER_WILL_START_POLL_AND_PUSH
                                    , msg);
                        }
                    } else {
                        CustomLog.d(TAG, "主持人视讯号不对，不处理。");
                    }
                } catch (Exception e) {
                    CustomLog.e(TAG, "#DPMSG# 解析异常 e=" + e.toString());
                }
            } else if (data.contains("#")) {
                String[] dataArray = data.split("#");
                if (dataArray.length == 3 && dataArray[0].equals("2")) {
                    CustomLog.d(TAG, "收到开始数据共享请求!");
                    String msgData2 = "3#1#" + mMeetingInfo.accountName + "#"
                            + dataArray[2];
                    mMeetingControl.dataShareOperateResponse(fromId, msgData2);
                    CustomLog.d(TAG, "回应手机端不支持!");
                }
            } else {
                try {
                    JSONObject msgJson = new JSONObject(data);
                    if (msgJson.has("t")) {
                        ChatMessageInfo chatMessageInfo = new ChatMessageInfo();
                        chatMessageInfo.messageOperation = 2;
                        chatMessageInfo.chatType = msgJson.optInt("t");
                        chatMessageInfo.IMMsgType = msgJson.optInt("IMType");
                        chatMessageInfo.messageID = ButelCommonUtil.getUUID32();
                        chatMessageInfo.senderNube = msgJson.optString("id");
                        chatMessageInfo.senderName = msgJson.optString("name");
                        chatMessageInfo.receive = mMeetingInfo.accountId;
                        chatMessageInfo.msgContent = msgJson.optString("msg");
                        chatMessageInfo.timestamp = cmd.getTimestamp();

                        boolean isRepeatMsg = checkIsRepeatMsg(chatMessageInfo);
                        if (!isRepeatMsg) {
                            String atIds = msgJson.optString("atIds");
                            if (atIds != null && atIds.length() > 0) {
                                String[] atIdArr = atIds.split(",");
                                List<String> atIdList = new ArrayList<>();
                                for (int i = 0; i < atIdArr.length; i++) {
                                    atIdList.add(atIdArr[i]);
                                }
                                chatMessageInfo.atNubes = atIdList;
                            }

                            handleNotifyListenerListOnNotify(NotifyType.NOTIFY_RECEIVE_MEETING_CHAT_MESSAGE
                                    , chatMessageInfo);
                        }
                    }
                } catch (Exception e) {
                    CustomLog.e(TAG, "数据异常或命令不支持，不做处理！e=" + e.toString());
                }
            }
        }
    }

    private void handleUIBlackCmd(Cmd cmd) {
        CustomLog.d(TAG, "handleUIBlackCmd");
        int meetingId = cmd.getMeetingId();
        String touserId = cmd.getTouserId();
        String fromUserId = cmd.getFromUserId();
        int messageId = cmd.getMessageId();

        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG, "会议号不一致，不做处理！");
            return;
        }

        if (touserId != null && !touserId.equals("") && !touserId.equals(mMeetingInfo.accountId)) {
            CustomLog.e(TAG, "接收者不为自己，不做处理！");
            return;
        }

        if (messageId == 517) {
            String msgData = cmd.getMsgData();
            if (msgData.contains("#")) {
                String[] data = msgData.split("#");
                if (data.length == 3 && data[0].equals("2")) {
                    CustomLog.d(TAG, "收到开始数据共享请求!");
                    String msgData2 = "3#1#" + mMeetingInfo.accountName + "#"
                            + data[2];
                    mMeetingControl.dataShareOperateResponse(fromUserId, msgData2);
                    CustomLog.d(TAG, "回应手机端不支持!");
                } else if (data.length == 4 && data[0].equals("3")) {
                    CustomLog.d(TAG, "收到请求开始数据共享回应!");
                    DataShareResult dataShareResult = new DataShareResult();
                    dataShareResult.operateAccountId = fromUserId;
                    dataShareResult.operateAccountName = data[2];
                    dataShareResult.operateResult = Integer.parseInt(data[1]);
                    handleNotifyListenerListOnNotify(
                            NotifyType.NOTIFY_DATA_SHARE_RESPONSE, dataShareResult);
                } else {
                    CustomLog.e(TAG, "数据异常或命令不支持，不做处理！");
                }
            } else {
                try {
                    JSONObject msgJson = new JSONObject(msgData);
                    if (msgJson.has("t")) {
                        ChatMessageInfo chatMessageInfo = new ChatMessageInfo();
                        chatMessageInfo.messageOperation = 2;
                        chatMessageInfo.chatType = msgJson.optInt("t");
                        chatMessageInfo.IMMsgType = msgJson.optInt("IMType");
                        chatMessageInfo.messageID = ButelCommonUtil.getUUID32();
                        chatMessageInfo.senderNube = msgJson.optString("id");
                        chatMessageInfo.senderName = msgJson.optString("name");
                        chatMessageInfo.receive = mMeetingInfo.accountId;
                        chatMessageInfo.msgContent = msgJson.optString("msg");
                        chatMessageInfo.timestamp = cmd.getTimestamp();

                        boolean isRepeatMsg = checkIsRepeatMsg(chatMessageInfo);
                        if (!isRepeatMsg) {
                            String atIds = msgJson.optString("atIds");
                            if (atIds != null && atIds.length() > 0) {
                                String[] atIdArr = atIds.split(",");
                                List<String> atIdList = new ArrayList<>();
                                for (int i = 0; i < atIdArr.length; i++) {
                                    atIdList.add(atIdArr[i]);
                                }
                                chatMessageInfo.atNubes = atIdList;
                            }

                            handleNotifyListenerListOnNotify(NotifyType.NOTIFY_RECEIVE_MEETING_CHAT_MESSAGE
                                    , chatMessageInfo);
                        }
                    }
                } catch (Exception e) {
                    CustomLog.e(TAG, "数据异常或命令不支持，不做处理！e=" + e.toString());
                }
            }
        } else if (messageId == 3700 && mPollFlag) {
            String msgData = cmd.getMsgData();
            String realData = "";
            // 协议转换
            if (msgData != null && msgData.length() > 0) {
                try {
                    JSONObject jsonObject = new JSONObject(msgData);
                    JSONObject realObj = new JSONObject();
                    realObj.put("vWidth", jsonObject.optInt("vWidth"));
                    realObj.put("vHeight", jsonObject.optInt("vHeight"));
                    realObj.put("bitrate", jsonObject.optInt("vBitrate"));

                    if (jsonObject.optInt("vWidth") < 0
                            || jsonObject.optInt("vHeight") < 0
                            || jsonObject.optInt("vBitrate") < 0) {
                        CustomLog.e(TAG, "轮询推流参数异常,不处理!");
                        return;
                    }
                    realData = realObj.toString();
                } catch (Exception e) {
                    CustomLog.e(TAG, "视频参数解析异常");
                    realData = "";
                }
            }
            if (mPollAndPushControl != null) {
                if (mPollAndPushControl.getPollState()) {
                    CustomLog.d(TAG, "当前正在轮询，修改参数重新发布流。");
                    // mPollAndPushControl.stopPollAndPush();
                    handleNotifyListenerListOnNotify(
                            NotifyType.SERVER_NOTICE_STOP_POLL_AND_PUSH, "");
                    int ret = mPollAndPushControl.startPollAndPush(realData);
                    if (ret > 0) {
                        handleNotifyListenerListOnNotify(
                                NotifyType.SERVER_NOTICE_START_POLL_AND_PUSH, "");
                    } else {
                        mMeetingControl.sendPollAndPushStreamResult(1, ret,
                                mMeetingInfo.mMeetingMasterAccountId);
                        if (ret < 0) {
                            CustomLog.d(TAG, "轮询过程中发布流失败，停止轮询。");
                            mPollAndPushControl.stopPollAndPush();
                        }
                    }
                }

            }
        }
    }

    private void handleServerPollCmd(Cmd cmd) {
        if (!mPollFlag) {
            CustomLog.e(TAG, "handleServerPollCmd 未打开轮询，不处理!");
            return;
        }
        String accountId = cmd.getAccountId();
        int meetingId = cmd.getMeetingId();
        int operatorType = cmd.getOperateType();
        String pollParam = cmd.getPollParam();

//        if (!mMeetingInfo.accountId.equals(accountId)) {
//            CustomLog.e(TAG, "被轮询视讯号不是自己,不处理!");
//            return;
//        }
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG, "会议号不一致,不处理!");
            return;
        }
        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
        if (speaker == null) {
            if (operatorType == 1) {
                // 开始推流
                CustomLog.d(TAG, "当前不在发言 开始推流");
                if (mPollAndPushControl != null) {
                    int ret = mPollAndPushControl.startPollAndPush(pollParam);
                    if (ret > 0) {
                        handleNotifyListenerListOnNotify(
                                NotifyType.SERVER_NOTICE_START_POLL_AND_PUSH, "");
                    } else {
                        mMeetingControl.sendPollAndPushStreamResult(1, ret,
                                mMeetingInfo.mMeetingMasterAccountId);
                        if (ret < 0) {
                            CustomLog.d(TAG, "轮询过程中发布流失败，停止轮询。");
                            mPollAndPushControl.stopPollAndPush();
                        }
                    }
                }
            } else if (operatorType == 2) {
                // 取消推流
                CustomLog.d(TAG, "当前不在发言 停止推流");
                if (mPollAndPushControl != null) {
                    handleNotifyListenerListOnNotify(
                            NotifyType.SERVER_NOTICE_STOP_POLL_AND_PUSH, "");
                }
            }
        } else {
            CustomLog.d(TAG, "当前正在发言");
            if (mPollAndPushControl != null && mPollAndPushControl.getPollState()) {
                CustomLog.d(TAG, "当前正在发言,且正在轮询，则取消推流!");
                handleNotifyListenerListOnNotify(
                        NotifyType.SERVER_NOTICE_STOP_POLL_AND_PUSH, "");
            }
        }
    }

    private boolean checkIsRepeatMsg(ChatMessageInfo chatMessageInfo) {
        if (mReceiveMsgMap != null) {
            long currentTime = System.currentTimeMillis() / 1000;
            String key = chatMessageInfo.getSenderNube() + chatMessageInfo.timestamp;
            if (mReceiveMsgMap.containsKey(key)) {
                long lastMsgTime = mReceiveMsgMap.get(key);
                if (currentTime - lastMsgTime <= 60) {
                    CustomLog.d(TAG, "checkIsRepeatMsg 60s内收到重复消息，不处理。");
                    return true;
                } else {
                    mReceiveMsgMap.put(key, currentTime);

                    // 遍历删除超过60s的聊天数据
                    Iterator<Entry<String, Long>> it = mReceiveMsgMap.entrySet().iterator();
                    while (it.hasNext()) {
                        Entry<String, Long> entry = it.next();
                        String mapKey = entry.getKey();
                        long mapValue = entry.getValue();
                        if (currentTime - mapValue > 60) {
//                            mReceiveMsgMap.remove(mapKey);
                            it.remove();
                        }
                    }
                    return false;
                }
            } else {
                mReceiveMsgMap.put(key, currentTime);

                // 遍历删除超过60s的聊天数据
                Iterator<Entry<String, Long>> it = mReceiveMsgMap.entrySet().iterator();
                while (it.hasNext()) {
                    Entry<String, Long> entry = it.next();
                    String mapKey = entry.getKey();
                    long mapValue = entry.getValue();
                    if (currentTime - mapValue > 60) {
//                        mReceiveMsgMap.remove(mapKey);
                        it.remove();
                    }
                }
                return false;
            }
        }
        return false;
    }

    private void handleAskForStartRecordingResp(Cmd cmd) {
        CustomLog.i(TAG, "handleAskForStartRecordingResp");
        int status = cmd.getStatus();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "handleAskForStartRecordingResp 开始录制请求 回应：" + status);
        if (status == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "开始录制请求成功");

            int result = startRecording(cmd.getRecServerAddr().split(":")[0],
                    Integer.valueOf(cmd.getRecServerAddr().split(":")[1]),
                    cmd.getFileAddress());

            if (result == 0) {
                CustomLog.i(TAG, "开始录制操作成功");
                mRecordStatus = true;
                callbackAskForStartRecordingOperation(true, cmd);
            } else {
                CustomLog.i(TAG, "开始录制操作失败");
                callbackAskForStartRecordingOperation(false, cmd);
            }

        } else {
            CustomLog.w(TAG, "开始录制请求失败");
            callbackAskForStartRecordingOperation(false, cmd);
        }
    }

    private void handleServerNoticeStartRecording(Cmd cmd) {
        CustomLog.i(TAG, "handleServerNoticeStartRecording");
        int meetingId = cmd.getMeetingId();
        String accountId = cmd.getAccountId();
        CustomLog.i(TAG,
                "handleServerNoticeStartRecording() 服务器通知终端会议开始录制。 meetingId: "
                        + meetingId + " | accountId: " + accountId);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG,
                    "handleServerNoticeStartRecording() error! 会议id不一致，当前会议： "
                            + mMeetingInfo.meetingId);
            return;
        }

        int result = startRecording(cmd.getRecServerAddr().split(":")[0],
                Integer.valueOf(cmd.getRecServerAddr().split(":")[1]),
                cmd.getFileAddress());

        if (result == 0) {
            CustomLog.i(TAG, "开始录制操作成功");
            mRecordStatus = true;
            handleNotifyListenerListOnNotify(
                    NotifyType.SERVER_NOTICE_START_RECORDING, cmd);
        } else {
            CustomLog.i(TAG, "开始录制操作失败,不通知给ui");
        }

    }

    private int startRecording(String ip, int port, String pstrcmd) {
        CustomLog.d(TAG, "startRecording");

        if (ip == null || String.valueOf(port) == null || pstrcmd == null) {
            CustomLog.e(TAG, "startRecording 失败，参数为空");
            return -1;
        }

        int setAddrResult = mSMSDKControl.setRecordServerAddr2(ip, port,
                pstrcmd);

        if (setAddrResult == 0) {
            CustomLog.i(TAG, "setAddrResult成功");

            int startRecord = mSMSDKControl.startRecord();

            if (startRecord == 0) {
                CustomLog.i(TAG, "startRecord成功");
                return 0;
            } else {
                CustomLog.i(
                        TAG,
                        "startRecord失败,startRecord:"
                                + String.valueOf(startRecord));
                return -3;
            }
        } else {
            CustomLog.i(
                    TAG,
                    "setAddrResult失败,setAddrResult:"
                            + String.valueOf(setAddrResult));
            return -2;
        }

    }

    private void handleAskForStopRecordingResp(Cmd cmd) {
        CustomLog.i(TAG, "handleAskForStopRecordingResp");
        int status = cmd.getStatus();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "handleAskForStopRecordingResp 停止录制请求回应：" + status);
        if (status == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "停止录制请求成功");

            int result = stopRecording();

            if (result == 0) {
                CustomLog.i(TAG, "停止录制操作成功");
                mRecordStatus = false;
                callbackAskForStopRecordingOperation(true, cmd);
            } else {
                CustomLog.i(TAG, "停止录制操作失败");
                callbackAskForStopRecordingOperation(false, cmd);
            }

        } else {
            CustomLog.w(TAG, "停止录制请求失败");
            callbackAskForStopRecordingOperation(false, cmd);
        }
    }

    private void handleServerNoticeStopRecording(Cmd cmd) {
        CustomLog.i(TAG, "handleServerNoticeStopRecording");
        int meetingId = cmd.getMeetingId();
        String accountId = cmd.getAccountId();
        CustomLog.i(TAG,
                "handleServerNoticeStopRecording() 服务器通知终端会议停止录制。 meetingId: "
                        + meetingId + " | accountId: " + accountId);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG,
                    "handleServerNoticeStopRecording() error! 会议id不一致，当前会议： "
                            + mMeetingInfo.meetingId);
            return;
        }

        int result = stopRecording();

        if (result == 0) {
            CustomLog.i(TAG, "停止录制操作成功");
            mRecordStatus = false;
            handleNotifyListenerListOnNotify(
                    NotifyType.SERVER_NOTICE_STOP_RECORDING, cmd);
        } else {
            CustomLog.i(TAG, "停止录制操作失败,不通知给ui");
        }

    }

    private int stopRecording() {
        CustomLog.d(TAG, "stopRecording");
        int result = mSMSDKControl.stopRecord();
        if (result == 0) {
            CustomLog.i(TAG, "stopRecording成功");
            return 0;
        } else {
            CustomLog
                    .i(TAG, "stopRecording失败,result:" + String.valueOf(result));
            return -1;
        }
    }

    private void handleApplayForStartScreenSharingResp(Cmd cmd) {
        CustomLog.i(TAG, "处理申请开始屏幕分享回应 status: " + cmd.getStatus());
        if (cmd.getStatus() != RespModelStatusCode.SUCCESS) {
            CustomLog.e(TAG, "申请开始屏幕分享请求发送失败");
            if (mScreenShareControl != null) {
                mScreenShareControl
                        .responseApplyForStart(ScreenShareControl.REFUSE_CONNECT_REQUEST_FAILED);
            }
            return;
        }

        CustomLog.i(TAG, "申请开始屏幕分享请求发送成功，等待服务器下发通知");
    }

    private void handleApplayForStopScreenSharingResp(Cmd cmd) {
        CustomLog.i(TAG, "处理申请停止屏幕分享回应 status: " + cmd.getStatus());
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
    }

    private void handleServerReturnApplayForSharingResult(Cmd cmd) {
        CustomLog.i(TAG, "处理申请开始屏幕分享结果  status: " + cmd.getStatus());
        // 申請屏幕分享結果操作>0表示成功
        if (cmd.getStatus() >= RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "申请开始屏幕分享成功");
            if (mScreenShareControl != null) {
                mScreenShareControl
                        .responseApplyForStart(ScreenShareControl.REQUEST_CONNECT_SUCCESS);
            }
            int meetingId = cmd.getMeetingId();
            String userId = cmd.getAccountId();
            String userName = cmd.getUserName();
            CustomLog.i(TAG, "处理申请开始屏幕分享结果 meetingId: " + meetingId
                    + " | userId: " + userId + " |userName: " + userName);
            if (meetingId != mMeetingInfo.meetingId) {
                CustomLog.w(TAG, "error! 会议id和当前会议id不一致。 current meetingId: "
                        + mMeetingInfo.meetingId);
                return;
            }

            SpeakerInfo speaker = getSpeakerInfoById(userId);

            if (speaker == null) {
                CustomLog.w(TAG, "error! 屏幕分享用户没有发言 userId: " + userId);
                return;
            }

            if (speaker.getScreenShareStatus() == SpeakerInfo.SCREEN_SHARING) {
                CustomLog.e(
                        TAG,
                        "error! 当前已经在屏幕分享! screen: "
                                + speaker.getScreenShareStatus());
                return;
            }
            speaker.setScreenShareInfo(SpeakerInfo.SCREEN_SHARING);
        } else {
            CustomLog.i(TAG, "申请开始屏幕分享失败");
            if (mScreenShareControl != null) {
                mScreenShareControl
                        .responseApplyForStart(ScreenShareControl.REFUSE_CONNECT_REQUEST_FAILED);
            }
        }
    }

    private void handleServerNoticeStartScreenSharing(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG, "处理服务器下发的新屏幕分享用户产生 meetingId: " + meetingId
                + " | userId: " + userId + " |userName: " + userName);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.w(TAG, "error! 会议id和当前会议id不一致。 current meetingId: "
                    + mMeetingInfo.meetingId);
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(userId);

        if (speaker == null) {
            CustomLog.e(TAG, "error! 服务器通知的屏幕分享用户没有发言 userId: " + userId);
            return;
        }

        if (speaker.getScreenShareStatus() == SpeakerInfo.SCREEN_SHARING) {
            CustomLog.e(TAG, "error! 当前用户已经在屏幕分享: " + speaker.getAccountId()
                    + " | screen " + speaker.getScreenShareStatus());
            return;
        }

        speaker.setScreenShareInfo(SpeakerInfo.SCREEN_SHARING);
        CustomLog.i(TAG, "通知UI端新屏幕分享产生。  meetingId: " + meetingId
                + " | userId: " + userId + " |userName: " + userName);
        handleNotifyListenerListOnNotify(
                NotifyType.SERVER_NOTICE_START_SCREEN_SHAREING, cmd);
    }

    private void handleServerReturnSharingResult(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        // 可能是自己申请停止，也可能是被抢占，cmd里面的userName和accountId是新用户的名称和视讯号，需要调用屏幕分享接口，告诉断开原因
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG, "服务器通知屏幕分享用户断开  meetingId: " + meetingId
                + " | userId: " + userId + " |userName: " + userName);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.w(TAG, "error! 会议id和当前会议id不一致。 current meetingId: "
                    + mMeetingInfo.meetingId);
            return;
        }

        if (!userId.equals(mMeetingInfo.accountId)) {
            CustomLog.e(TAG, "STP error! 服务器通知屏幕分享用户断开 应该只发给自己");
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(userId);

        if (speaker == null) {
            CustomLog.w(TAG, "error! 屏幕分享用户没有发言 userId: " + userId);
            return;
        }

        if (speaker.getScreenShareStatus() == SpeakerInfo.SCREEN_NORMAL) {
            CustomLog.e(
                    TAG,
                    "error! 当前没有进行屏幕分享! screen: "
                            + speaker.getScreenShareStatus());
            return;
        }

        speaker.setScreenShareInfo(SpeakerInfo.SCREEN_NORMAL);
        if (mScreenShareControl != null) {
            mScreenShareControl.forceStopSharing(
                    ScreenShareControl.STOP_SHARING_BE_GRABED, "未发言");
        }
        mSMSDKControl.unPublish(speaker
                .getStreamIdByResource(StreamInfo.RESOURCE_TYPE_DOC_VIDEO));
        handleNotifyListenerListOnNotify(NotifyType.STOP_SHARE_DOC, null);
    }

    private void handleServerNoticeStopScrrenSharing(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG, "处理服务器下发屏幕分享用户失效 meetingId: " + meetingId
                + " | userId: " + userId + " |userName: " + userName);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.w(TAG, "error! 会议id和当前会议id不一致。 current meetingId: "
                    + mMeetingInfo.meetingId);
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(userId);

        if (speaker == null) {
            CustomLog.w(TAG, "error! 屏幕分享用户没有发言 userId: " + userId);
            return;
        }

        if (speaker.getScreenShareStatus() != SpeakerInfo.SCREEN_SHARING) {
            CustomLog.e(TAG, "error! 当前该用户没有进行屏幕分享: " + speaker.getAccountId()
                    + " | screen " + speaker.getScreenShareStatus());
            return;
        }

        // 停止屏幕分享，清理数据
        if (mGeoMetryControl != null) {
            mGeoMetryControl.clearData(0);
        }

        // TODO 停止订阅，服务器不会单独下发停止发布流
        speaker.setScreenShareInfo(SpeakerInfo.SCREEN_NORMAL);
        handleNotifyListenerListOnNotify(
                NotifyType.SERVER_NOTICE_STOP_SCREEN_SHAREING, cmd);
    }

    private void handleSetMainSpeakerResp(Cmd cmd) {
        CustomLog.i(TAG, "处理收到主持人设置主讲人回应命令 status: " + cmd.getStatus());
        if (cmd.getStatus() == RespModelStatusCode.SUCCESS) {
            callbackMasterSetMainSpeakerOperation(true, cmd);
        } else {
            callbackMasterSetMainSpeakerOperation(false, cmd);
        }
    }

    private void handleServerNoticeNewMainSpeaker(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String speakerId = cmd.getAccountId();
        String speakerName = cmd.getUserName();
        CustomLog.i(TAG, "处理收到服务器下发的新主讲人产生 meetingId: " + meetingId
                + " | speakerId: " + speakerId + " |speakerName: "
                + speakerName);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.w(TAG, "error! 会议id和当前会议id不一致。 current meetingId: "
                    + mMeetingInfo.meetingId);
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(speakerId);

        if (speaker == null) {
            CustomLog.w(TAG, "error! 屏幕分享用户没有发言 userId: " + speakerId);
            return;
        }
        speaker.setSpeakerType(SpeakerInfo.SPEAKER_TYPE_MAIN);
        handleNotifyListenerListOnNotify(
                NotifyType.SERVER_NOTICE_NEW_MAIN_SPEAKER, cmd);
    }

    private void handleServerNoticeMainSpeakerInvalid(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String speakerId = cmd.getAccountId();
        String speakerName = cmd.getUserName();
        CustomLog.i(TAG, "处理收到服务器下发的新主讲人产生 meetingId: " + meetingId
                + " | speakerId: " + speakerId + " |speakerName: "
                + speakerName);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.w(TAG, "error! 会议id和当前会议id不一致。 current meetingId: "
                    + mMeetingInfo.meetingId);
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(speakerId);

        if (speaker == null) {
            CustomLog.w(TAG, "error! 屏幕分享用户没有发言 userId: " + speakerId);
            return;
        }
        // TODO so库未提交主讲相关
        // if (speakerId.equals(mMeetingInfo.accountId)) {
        // mSMSDKControl.setSpeakMode(SmSdkJNI.SM_SDK_SPEAKMODE_NORMAL);
        // }
        speaker.setSpeakerType(SpeakerInfo.SPEAKER_TYPE_NORMAL);
        handleNotifyListenerListOnNotify(
                NotifyType.SERVER_NOTICE_MAIN_SPEAKER_INVALID, cmd);
    }

    private void handleAskOpertionStreamResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG, "处理发布、取消发布流回应命令。 status: " + cmd.getStatus());

        boolean flag = (status == RespModelStatusCode.SUCCESS);

        // TODO 单独解析每一条发布是否成功
        if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_OPEN_MIC) {
            handleAskForOpenMicResp(flag, cmd);
        } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_CLOSE_MIC) {
            handleAskForCloseMicResp(flag, cmd);
        } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_OPEN_CAMERA) {
            handleAskForOpenCameraResp(flag, cmd);
        } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_CLOSE_CAMERA) {
            handleAskForCloseCameraResp(flag, cmd);
        } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.PUBLISH_OR_UNPUBLISH_STREAMS) {
            handlePublishOrUnpublishResp(flag, cmd);
        } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.PUBLISH_OR_UNPUBLISH_WHITEBOARD_STREAMS) {
            handlePublishOrUnpublishWhiteBoardResp(flag, cmd);
        } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_OPEN_LOUDSPEAKER) {
            handleAskForOpenLoudSpeakerResp(flag, cmd);
        } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_CLOSE_LOUDSPEAKER) {
            handleAskForCloseLoudSpeakerResp(flag, cmd);
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
    }

    private void handleServerNoticeStreamChanged(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG, "处理收到服务器下发言者发布、取消发布流  meetingId: " + meetingId
                + " | userId: " + userId + " |userName: " + userName);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG, "error! 会议id和当前会议id不一致。 current meetingId: "
                    + mMeetingInfo.meetingId);
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(userId);

        if (speaker == null) {
            CustomLog.e(TAG, "error! 不存在该发言者 userId: " + userId);
            return;
        }

        int operatorType = cmd.getStreamOperatorType();
        int resourceId = cmd.getResourceId();
        int mediaType = cmd.getMediaType();
        int syncId = Cmd.getSyncId(cmd.getExtraInfo());
        int shareType = Cmd.getShareType(cmd.getExtraInfo());

        CustomLog.d(TAG, "handleServerNoticeStreamChanged operatorType="
                + operatorType + " resourceId=" + resourceId + " mediaType="
                + mediaType + " syncId=" + syncId + " shareType=" + shareType);

        if (operatorType == Cmd.STREAM_OPERATOR_TYPE_PUBLISH) {
            CustomLog.i(TAG, "解析 operatorType 为发布流  resourceId: " + resourceId
                    + " |mediaType：" + mediaType + " syncId:" + syncId
                    + " shareType:" + shareType);

            int streamId = 0;
            if (mWhiteBoardFlag && (shareType != -1 && shareType != 0)) {
                streamId = mSMSDKControl.subscribe2(userId, mediaType,
                        resourceId, syncId, shareType);
                CustomLog.d(TAG, "subscribe2 streamId=" + streamId);
            } else {
                streamId = mSMSDKControl.subscribe(userId, mediaType,
                        resourceId, syncId);
            }

            speaker.addStreamInfo(resourceId, new StreamInfo(streamId,
                    resourceId, syncId, mediaType, shareType));

            if (mediaType == MediaType.TYPE_VIDEO) {
                CustomLog.d(TAG, "视频流发布");

                speaker.setCamStatus(SpeakerInfo.CAM_STATUS_OPEN);

                speaker.setVideoParam(Cmd.getVideoParam(cmd.getExtraInfo()));
                if (speaker.getVideoStatus() == SpeakerInfo.VIDEO_OFF) {
                    CustomLog
                            .d(TAG,
                                    "ButelOpenSDK::handleServerNoticeStreamChanged()，视频画面关闭状态，暂停订阅");
                    pauseSubscribe(userId, MediaType.TYPE_VIDEO);
                }
            } else if (mediaType == MediaType.TYPE_AUDIO) {
                CustomLog
                        .d(TAG,
                                "ButelOpenSDK::handleServerNoticeStreamChanged() 发布音频流，将麦克风置为打开状态");
                speaker.setMICStatus(SpeakerInfo.MIC_STATUS_ON);
            } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
                speaker.setDocVideoParam(Cmd.getVideoParam(cmd.getExtraInfo()));
            }

            handleNotifyListenerListOnNotify(
                    NotifyType.SERVER_NOTICE_STREAM_PUBLISH, cmd);
        } else if (operatorType == Cmd.STREAM_OPERATOR_TYPE_UNPUBLISH) {
            // 取消发布没有mediaType
            cmd.setMediaType(SpeakerInfo.ParseSmMediaTypeToStp(mediaType));
            CustomLog.i(TAG, "解析 operatorType 为取消发布流  resourceId: "
                    + resourceId + " |mediaType：" + mediaType + " syncId:"
                    + syncId);
            mSMSDKControl
                    .unSucscribe(speaker.getStreamIdByResource(resourceId));
            speaker.removeStreamInfo(resourceId);
            if (mediaType == MediaType.TYPE_VIDEO) {
                CustomLog
                        .d(TAG,
                                "ButelOpenSDK::handleServerNoticeStreamChanged() 取消发布视频流，将摄像头置为关闭状态");
                speaker.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
            } else if (mediaType == MediaType.TYPE_AUDIO) {
                CustomLog
                        .d(TAG,
                                "ButelOpenSDK::handleServerNoticeStreamChanged() 取消发布音频流，将麦克风置为关闭状态");
                speaker.setMICStatus(SpeakerInfo.MIC_STATUS_OFF);
            }
            handleNotifyListenerListOnNotify(
                    NotifyType.SERVER_NOTICE_STREAM_UNPUBLISH, cmd);
        }
    }

    private void handleServerNoticeUpdateStream(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG, "处理收到服务器流更新  meetingId: " + meetingId + " | userId: "
                + userId + " |userName: " + userName);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG, "error! 会议id和当前会议id不一致。 current meetingId: "
                    + mMeetingInfo.meetingId);
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(userId);

        if (speaker == null) {
            CustomLog.e(TAG, "error! 不存在该发言者 userId: " + userId);
            return;
        }

        int mediaType = cmd.getMediaType();

        if (mediaType == MediaType.TYPE_VIDEO) {
            CustomLog.d(TAG, "更新视频流");
            speaker.setVideoParam(Cmd.getVideoParam(cmd.getExtraInfo()));
        } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
            CustomLog.d(TAG, "更新文档视频流");
            speaker.setDocVideoParam(Cmd.getVideoParam(cmd.getExtraInfo()));
        }

        handleNotifyListenerListOnNotify(NotifyType.SPEAKER_VIDEO_PARAM_UPDATE,
                userId);
    }

    /**
     * 暂停订阅：暂停画面渲染，指暂停远端视频画面订阅，当视频画面不可见时可调用该接口，节约资源开销,该发言者的声音通道依然会保留
     *
     * @param accountId 暂定订阅的目标发言者视讯号
     * @param mediaType 暂停订阅的流类型文档视频或者人像视频{@link MediaType}
     * @return <p>
     * 0: 成功
     * <p>
     * !=0: 失败
     */
    public int pauseSubscribe(String accountId, int mediaType) {
        CustomLog.i(TAG, "ButelOpenSDK::pauseSubscribe() accountId: "
                + accountId + " | StreamType: " + mediaType);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (TextUtils.isEmpty(accountId)
                || accountId.equals(mMeetingInfo.accountId)) {
            CustomLog.e(TAG, "参数错误 accoutId不能为空，且不是自己: " + accountId);
            return -1;
        }

        SpeakerInfo speaker = mMeetingInfo.getSpeakerByAccountId(accountId);
        if (speaker == null) {
            CustomLog.e(TAG, "该视讯号没有发言，无法暂停订阅");
            return -2;
        }

        int streamId = speaker.getSreamIdByMediaType(mediaType);
        if (streamId == -1) {
            CustomLog.e(TAG, "该视讯号没有对应的流信息发布");
            return -3;
        }

        int ret = mSMSDKControl.pauseSubscribe(streamId);
        CustomLog.d(TAG, "ButelOpenSDK::pauseSubscribe() result: " + ret);
        return ret;
    }

    /**
     * 恢复订阅:恢复画面渲染，指恢复远端视频订阅
     * <p>
     * 如果对方关闭了摄像头，将无法进行恢复订阅视频画面
     *
     * @param accountId 恢复订阅的发言者视讯号
     * @param mediaType 媒体类型，文档视频或者人像视频 {@link MediaType}
     * @return <p>
     * 0: 成功
     * <p>
     * !=0: 失败
     */
    public int resumeSubscribe(String accountId, int mediaType) {
        CustomLog.i(TAG, "ButelOpenSDK::resumeSubscribe() accountId: "
                + accountId + " | StreamType: " + mediaType);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (TextUtils.isEmpty(accountId)
                || accountId.equals(mMeetingInfo.accountId)) {
            CustomLog.w(TAG, "参数错误 accoutId不能为空，且不是自己: " + accountId);
            return -1;
        }

        SpeakerInfo speaker = mMeetingInfo.getSpeakerByAccountId(accountId);
        if (speaker == null) {
            CustomLog.w(TAG, "该视讯号没有发言，无法暂停订阅");
            return -2;
        }

        if (mediaType == MediaType.TYPE_VIDEO
                && speaker.getCamStatus() == SpeakerInfo.CAM_STATUS_CLOSE) {
            CustomLog.d(TAG, "恢复订阅视频流，但对方摄像头是关闭状态，不予订阅");
            return -4;
        }

        if (mediaType == MediaType.TYPE_VIDEO
                && speaker.getVideoStatus() == SpeakerInfo.VIDEO_OFF) {
            CustomLog.d(TAG, "恢复订阅视频流，但视频画面是关闭状态，不予订阅");
            return -4;
        }

        if (mediaType == MediaType.TYPE_DOC_VIDEO
                && speaker.getDocVideoStatus() == SpeakerInfo.DOC_VIDEO_OFF) {
            CustomLog.d(TAG, "恢复订阅文档画面流，但文档画面是关闭状态，不予订阅");
            return -4;
        }

        int streamId = speaker.getSreamIdByMediaType(mediaType);
        if (streamId == -1) {
            CustomLog.w(TAG, "该视讯号没有对应的流信息发布");
            return -3;
        }

        int ret = mSMSDKControl.continueSubscribe(streamId, mediaType);
        CustomLog.d(TAG, "ButelOpenSDK::resumeSubscribe() result: " + ret);
        return ret;
    }

    /**
     * 开始本地渲染：开启本地的自预览摄像头画面或者文档共享画面
     *
     * @param mediaType   人像画面或文档共享画面 {@link MediaType}
     * @param surfaceView 显示窗口
     * @return <p>
     * 0: 成功
     * <p>
     * !=0: 失败
     */

    public int startLocalVideo(int mediaType, SurfaceView surfaceView) {
        CustomLog.i(TAG, "ButelOpenSDK::startLocalVideo() 开始本地渲染 mediaType: "
                + mediaType);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        SpeakerInfo speaker = mMeetingInfo
                .getSpeakerByAccountId(mMeetingInfo.accountId);
        if (speaker == null) {
            CustomLog.w(TAG, "error! 当前没有发言");
            return -2;
        }

        int streamId = speaker.getSreamIdByMediaType(mediaType);
        if (streamId == -1) {
            CustomLog.w(TAG, "该视讯号没有对应的流信息发布");
            return -3;
        }

        mSMSDKControl.startRenderLocalVedio(streamId, mediaType, surfaceView);
        return 0;
    }

    /**
     * 停止本地渲染：停止本地的自预览或者文档共享视频
     *
     * @param mediaType 人像视频或者文档视频 {@link MediaType}
     * @return <p>
     * 0: 成功
     * <p>
     * !=0: 失败
     */

    public int stopLocalVideo(int mediaType) {
        CustomLog.i(TAG, "ButelOpenSDK::stopLocalVideo() 停止本地渲染 mediaType: "
                + mediaType);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        SpeakerInfo speaker = mMeetingInfo
                .getSpeakerByAccountId(mMeetingInfo.accountId);
        if (speaker == null) {
            CustomLog.w(TAG, "error! 当前没有发言");
            return -2;
        }
        CustomLog.d(TAG, "speaker = " + speaker.toString());
        int streamId = speaker.getSreamIdByMediaType(mediaType);
        CustomLog.d(TAG, "stopLocalVideo streamType=" + mediaType + "streamId="
                + streamId);

        int ret = 0;

        if (streamId == -1) {
            CustomLog.w(TAG, "该视讯号没有对应的流信息发布");
        } else {
            int result = mSMSDKControl.unPublish(streamId);
            if (result == 0) {
                CustomLog.i(TAG, "ButelOpenSDK 取消发布流成功. streamId: " + streamId);
            } else {
                CustomLog.w(TAG, "ButelOpenSDK 取消发布流失败. streamId: " + streamId
                        + " | result: " + result);
            }
            ret = mSMSDKControl.stopRenderLocalVedio(streamId, mediaType);
        }

        if (mediaType == MediaType.TYPE_VIDEO) { // 停止本地视频渲染

            speaker.removeStreamInfo(StreamInfo.RESOURCE_TYPE_VIDEO);
            speaker.removeStreamInfo(StreamInfo.RESOURCE_TYPE_AUDIO);

            CustomLog.i(TAG, "stopLocalVideo 移除视频流，删除accountId信息");
        } else if (mediaType == MediaType.TYPE_DOC_VIDEO) { // 停止本地文档渲染
            speaker.setScreenShareInfo(SpeakerInfo.SCREEN_NORMAL);
            if (mScreenShareControl != null) {
                mScreenShareControl.setStreamId(-1);
            }
            speaker.removeStreamInfo(StreamInfo.RESOURCE_TYPE_DOC_VIDEO);
            speaker.removeStreamInfo(StreamInfo.RESOURCE_TYPE_DOC_AUDIO);
            CustomLog.i(TAG, "stopLocalVideo 移除文档流");
        }

        if (speaker.getmStreamsInfo().size() <= 0) {
            CustomLog.i(TAG, "stopLocalVideo 已无视频流信息,删除发言者信息");

            if (mMeetingInfo.getSpeakStatus() == false) {
                mMeetingInfo.removeSpeaker(mMeetingInfo.accountId);
            }

        }

        CustomLog.i(TAG, "mMeetingInfo..." + mMeetingInfo.getSpeakers().size());
        return ret;
    }

    /**
     * 开始远端渲染
     *
     * @param accountId   视讯号
     * @param mediaType   流类型{@link MediaType.TYPE_VIDEO} 视频流
     *                    {@link MediaType.TYPE_DOC_VIDEO} 文档视频流
     * @param surfaceView 渲染显示窗口
     * @return <p>
     * 0: 成功
     * <p>
     */
    public int startRemoteVideo(String accountId, int mediaType,
                                SurfaceView surfaceView) {
        CustomLog.i(TAG, "ButelOpenSDK::startRemoteVideo() accountId: "
                + accountId + " | StreamType: " + mediaType + " surfaceView: "
                + surfaceView.toString());
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (TextUtils.isEmpty(accountId)
                || accountId.equals(mMeetingInfo.accountId)) {
            CustomLog.w(TAG, "参数错误 accoutId不能为空，且不是自己: " + accountId);
            return -1;
        }

        SpeakerInfo speaker = mMeetingInfo.getSpeakerByAccountId(accountId);
        if (speaker == null) {
            CustomLog.w(TAG, "该视讯号没有发言，无法开始渲染");

            return -2;
        }

        if (mediaType != MediaType.TYPE_VIDEO
                && mediaType != MediaType.TYPE_DOC_VIDEO) {
            CustomLog.w(TAG, "Warning! mediaType不是视频或者文档视频，不支持开启远端渲染");
            return -4;
        }

        int streamId = speaker.getSreamIdByMediaType(mediaType);
        if (streamId == -1) {
            CustomLog.w(TAG, "该视讯号没有对应的流信息发布");
            return -3;
        }

        // if(mMeetingShowType==MEETING_TYPE_AUDIO){
        // CustomLog.d(TAG,"startRemoteVideo,是音频会议，不处理");
        // return -9;
        // }

        if (mediaType == MediaType.TYPE_DOC_VIDEO) {
            if (getRenderVideoInfo(accountId, mediaType) != null) {
                speaker.setDocVideoParam(getRenderVideoInfo(accountId,
                        mediaType));
            }
            setRenderVideoTimeTask(accountId, mediaType, speaker);

            // surfaceView.setZOrderMediaOverlay(true);
            // surfaceView.getHolder().setFormat(PixelFormat.TRANSLUCENT);
            // surfaceView.setZOrderOnTop(true);
            // surfaceView.getHolder().setFormat(PixelFormat.TRANSLUCENT);

            if (mWhiteBoardFlag) {
                StreamInfo streamInfo = speaker
                        .getStreamInfoByMediaType(mediaType);

                if (streamInfo != null && mGeoMetryControl != null) {
                    int shareType = streamInfo.getShareType();
                    CustomLog.d(TAG,
                            "ButelOpenSDK::startRemoteVideo() shareType="
                                    + shareType);

                    mGeoMetryControl.setGraphicManagerActor(streamInfo
                            .getShareType());

                    if (shareType != -1 && shareType != 0) {
                        int frameStreamID = mSMSDKControl
                                .getFrameWorkStreamID(streamId);
                        CustomLog.i(TAG,
                                "ButelOpenSDK::startRemoteVideo() streamId: "
                                        + streamId + " | frameStreamID: "
                                        + frameStreamID);
                        mGeoMetryControl.addStreamInfo(surfaceView,
                                Integer.parseInt(speaker.getAccountId()),
                                frameStreamID);
                    }
                }

            }

        }

        return mSMSDKControl.startRenderRemoteVedio(streamId, mediaType,
                surfaceView);
    }

    private static final int VIDEO_PARAM_UPDATE = 6666;

    private Handler videoParamUpdateHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);
            switch (msg.what) {
                case VIDEO_PARAM_UPDATE:
                    handleNotifyListenerListOnNotify(
                            NotifyType.SPEAKER_VIDEO_PARAM_UPDATE, (String) msg.obj);
                    break;
            }
        }
    };

    /**
     * 每两秒获取一次视频的分辨率信息
     *
     * @param accountId
     * @param mediaType
     * @param speaker
     */
    private void setRenderVideoTimeTask(final String accountId,
                                        final int mediaType, final SpeakerInfo speaker) {
        renderVideoTimer = new Timer();
        renderVideoTimerTask = new TimerTask() {
            @Override
            public void run() {
                // TODO Auto-generated method stub
                int a = -1;
                if (getRenderVideoInfo(accountId, mediaType) != null) {
                    a = speaker.setDocVideoParam(getRenderVideoInfo(accountId,
                            mediaType));
                }
                if (a == 1) {
                    Message msg = new Message();
                    msg.what = VIDEO_PARAM_UPDATE;
                    msg.obj = accountId;
                    videoParamUpdateHandler.sendMessage(msg);
                }
            }

        };
        renderVideoTimer.schedule(renderVideoTimerTask, 0, 2000);
    }

    /**
     * 停止远端白板view，仅白板调用
     *
     * @param accountId 发言者视讯号
     * @param mediaType 流类型
     *                  <p>
     *                  {@link MediaType.TYPE_VIDEO} 视频流
     *                  <p>
     *                  {@link MediaType.TYPE_DOC_VIDEO} 文档视频流
     * @return <p>
     * 0: 成功
     * <p>
     * !=0: 失败
     */
    public int stopWhiteBoardVideo(String accountId, int mediaType) {
        CustomLog.i(TAG, "ButelOpenSDK::stopWhiteBoardVideo() accountId: "
                + accountId + " | StreamType: " + mediaType);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG,
                    "ButelOpenSDK::stopWhiteBoardVideo() error ! 未进行初始化 initStatus: "
                            + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (TextUtils.isEmpty(accountId)
                || accountId.equals(mMeetingInfo.accountId)) {
            CustomLog.e(TAG,
                    "ButelOpenSDK::stopWhiteBoardVideo()参数错误 accoutId不能为空，且不是自己: "
                            + accountId);
            return -1;
        }

        SpeakerInfo speaker = mMeetingInfo.getSpeakerByAccountId(accountId);
        if (speaker == null) {
            CustomLog.e(TAG,
                    "ButelOpenSDK::stopWhiteBoardVideo()该视讯号没有发言，无法停止渲染");
            return -2;
        }

        int streamId = speaker.getSreamIdByMediaType(mediaType);
        if (streamId == -1) {
            CustomLog.w(TAG, "ButelOpenSDK::stopRemoteVideo()该视讯号没有对应的流信息发布");
        }

        if ((mediaType == MediaType.TYPE_VIDEO || mediaType == MediaType.TYPE_DOC_VIDEO)
                && streamId != -1) {
            mSMSDKControl.stopRenderRemoteVideo(streamId);
        }
        return 0;
    }

    /**
     * 开始远端白板view渲染，仅白板调用
     *
     * @param accountId   视讯号
     * @param mediaType   流类型{@link MediaType.TYPE_VIDEO} 视频流
     *                    {@link MediaType.TYPE_DOC_VIDEO} 文档视频流
     * @param surfaceView 渲染显示窗口
     * @return <p>
     * 0: 成功
     * <p>
     */
    public int startWhiteBoardVideo(String accountId, int mediaType,
                                    SurfaceView surfaceView) {
        CustomLog.i(TAG, "ButelOpenSDK::startWhiteBoardVideo() accountId: "
                + accountId + " | StreamType: " + mediaType + " surfaceView: "
                + surfaceView.toString());
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (TextUtils.isEmpty(accountId)
                || accountId.equals(mMeetingInfo.accountId)) {
            CustomLog.w(TAG, "参数错误 accoutId不能为空，且不是自己: " + accountId);
            return -1;
        }

        SpeakerInfo speaker = mMeetingInfo.getSpeakerByAccountId(accountId);
        if (speaker == null) {
            CustomLog.w(TAG, "该视讯号没有发言，无法开始渲染");

            return -2;
        }

        if (mediaType != MediaType.TYPE_VIDEO
                && mediaType != MediaType.TYPE_DOC_VIDEO) {
            CustomLog.w(TAG, "Warning! mediaType不是视频或者文档视频，不支持开启远端渲染");
            return -4;
        }

        int streamId = speaker.getSreamIdByMediaType(mediaType);
        if (streamId == -1) {
            CustomLog.w(TAG, "该视讯号没有对应的流信息发布");
            return -3;
        }

        // if(mMeetingShowType==MEETING_TYPE_AUDIO){
        // CustomLog.d(TAG,"startRemoteVideo,是音频会议，不处理");
        // return -9;
        // }

        // if (mediaType == MediaType.TYPE_DOC_VIDEO) {
        // if (getRenderVideoInfo(accountId, mediaType) != null) {
        // speaker.setDocVideoParam(getRenderVideoInfo(accountId,
        // mediaType));
        // }
        // setRenderVideoTimeTask(accountId, mediaType, speaker);
        //
        // // surfaceView.setZOrderMediaOverlay(true);
        // // surfaceView.getHolder().setFormat(PixelFormat.TRANSLUCENT);
        // // surfaceView.setZOrderOnTop(true);
        // // surfaceView.getHolder().setFormat(PixelFormat.TRANSLUCENT);
        //
        // if (mWhiteBoardFlag) {
        // StreamInfo streamInfo = speaker
        // .getStreamInfoByMediaType(mediaType);
        //
        // if (streamInfo != null) {
        // int shareType = streamInfo.getShareType();
        // CustomLog.d(TAG,
        // "ButelOpenSDK::startRemoteVideo() shareType="
        // + shareType);
        //
        // mGeoMetryControl.setGraphicManagerActor(streamInfo
        // .getShareType());
        //
        // if (shareType == ShareType.SHARETYPE_WHITE_BOARD
        // || shareType == ShareType.SHARETYPE_PROGRESS_CAPTURE) {
        // int frameStreamID = mSMSDKControl
        // .getFrameWorkStreamID(streamId);
        // CustomLog.i(TAG,
        // "ButelOpenSDK::startRemoteVideo() streamId: "
        // + streamId + " | frameStreamID: "
        // + frameStreamID);
        // mGeoMetryControl.addStreamInfo(surfaceView,
        // Integer.parseInt(speaker.getAccountId()),
        // frameStreamID);
        // }
        // }
        //
        // }
        //
        // }

        return mSMSDKControl.startRenderRemoteVedio(streamId, mediaType,
                surfaceView);
    }

    /**
     * 停止远端渲染：停止其他参会方发言画面渲染或文档分享画面渲染 如果在进行屏幕分享的发言者停止发言，需要先停止文档画面，再停止视频画面
     *
     * @param accountId 发言者视讯号
     * @param mediaType 流类型
     *                  <p>
     *                  {@link MediaType.TYPE_VIDEO} 视频流
     *                  <p>
     *                  {@link MediaType.TYPE_DOC_VIDEO} 文档视频流
     * @return <p>
     * 0: 成功
     * <p>
     * !=0: 失败
     */
    public int stopRemoteVideo(String accountId, int mediaType) {
        CustomLog.i(TAG, "ButelOpenSDK::stopRemoteVideo() accountId: "
                + accountId + " | StreamType: " + mediaType);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG,
                    "ButelOpenSDK::stopRemoteVideo() error ! 未进行初始化 initStatus: "
                            + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (TextUtils.isEmpty(accountId)
                || accountId.equals(mMeetingInfo.accountId)) {
            CustomLog.e(TAG,
                    "ButelOpenSDK::stopRemoteVideo()参数错误 accoutId不能为空，且不是自己: "
                            + accountId);
            return -1;
        }

        SpeakerInfo speaker = mMeetingInfo.getSpeakerByAccountId(accountId);
        if (speaker == null) {
            CustomLog.e(TAG, "ButelOpenSDK::stopRemoteVideo()该视讯号没有发言，无法停止渲染");
            return -2;
        }

        int streamId = speaker.getSreamIdByMediaType(mediaType);
        if (streamId == -1) {
            CustomLog.w(TAG, "ButelOpenSDK::stopRemoteVideo()该视讯号没有对应的流信息发布");
        }

        if ((mediaType == MediaType.TYPE_VIDEO || mediaType == MediaType.TYPE_DOC_VIDEO)
                && streamId != -1) {
            mSMSDKControl.stopRenderRemoteVideo(streamId);
        }

        if (mediaType == MediaType.TYPE_VIDEO) {
            clearSpeakerSubscribeStrems(speaker);
            CustomLog.i(TAG, "stopRemoteVideo 移除视频流");
            if (speaker.getmStreamsInfo().size() <= 0) {
                CustomLog.i(TAG, "stopRemoteVideo 已无视频流信息,删除发言者信息");
                mMeetingInfo.removeSpeaker(accountId);
            }
        } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
            if (mWhiteBoardFlag) {
                // 如果是白板共享，需要告知图元管理模块
                StreamInfo streamInfo = speaker
                        .getStreamInfoByMediaType(mediaType);
                if (streamInfo != null && mGeoMetryControl != null) {
                    int shareType = streamInfo.getShareType();
                    if (shareType != -1 && shareType != 0) {
                        CustomLog.i(TAG, "stopRemoteVideo 告知图元管理模块 shareType="
                                + shareType);
                        mGeoMetryControl.removeStreamInfo(
                                Integer.parseInt(mMeetingInfo.accountId),
                                mSMSDKControl.getFrameWorkStreamID(streamId));
                    }
                }
            }

            mSMSDKControl.unSucscribe(speaker
                    .getSreamIdByMediaType(MediaType.TYPE_DOC_VIDEO));
            speaker.removeStreamInfoByMediaType(mediaType);
            speaker.removeStreamInfoByMediaType(MediaType.TYPE_DOC_AUDIO);
            CustomLog.i(TAG, "stopRemoteVideo 移除文档流");
        }

        if (renderVideoTimer != null) {
            renderVideoTimer.cancel();
            renderVideoTimer = null;
        }
        if (renderVideoTimerTask != null) {
            renderVideoTimerTask.cancel();
            renderVideoTimerTask = null;
        }

        CustomLog.i(TAG, "ButelOpenSDK::stopRemoteVideo() speaker size: "
                + mMeetingInfo.getSpeakers().size());
        return 0;
    }

    /**
     * 打开本地麦克风: 先向信令服务器进行申请音频流发布，成功后打开声音采集
     *
     * @param listener 操作回调接口，信令接口的通知结果通过该回调返回
     * @return 0： 操作成功
     * <p>
     * -1: 当前未发言，无法打开
     * <p>
     * -2: 当前已是打开状态
     * <p>
     * -3: 发布音频流失败
     * <p>
     * -5：未进行初始化
     */
    public int openMicrophone(ButelOpenSDKOperationListener listener) {

        CustomLog.d(TAG, "ButelOpenSDK::openMicrophone 打开本地麦克风");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
        if (speaker == null) {
            CustomLog.w(TAG,
                    "ButelOpenSDK::openMicrophone error! 当前未发言，无法打开麦克风");
            return -1;
        }

        if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_ON) {
            CustomLog.w(TAG, "ButelOpenSDK::openMicrophone error! 当前已是打开状态。");
            return -2;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        JSONArray jsonArray = new JSONArray();
        JSONObject audeoStreamJson = new JSONObject();
        try {
            audeoStreamJson.put(CmdKey.RESOURCE_ID,
                    String.valueOf(StreamInfo.RESOURCE_TYPE_AUDIO));
            audeoStreamJson.put(CmdKey.MEDIA_TYPE,
                    SpeakerInfo.ParseSmMediaTypeToStp(MediaType.TYPE_AUDIO));
            audeoStreamJson.put(CmdKey.EXTRAINFO,
                    Cmd.createExtraInfo(speaker.getSyncId(), -1, -1));

            jsonArray.put(audeoStreamJson);
            CustomLog.d(TAG, "jsonArray=" + jsonArray.toString());

            int result = mMeetingControl.askPublishOrUnPublishStreams(mToken,
                    Cmd.STREAM_OPERATOR_TYPE_PUBLISH, jsonArray);
            if (result != 0) {
                CustomLog.w(TAG, "向 STP 发布流失败 result: " + result);
                return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
            } else {
                mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_OPEN_MIC;
                mAskForOpenMicPhoneListenerList.add(listener);
            }
        } catch (JSONException e) {
            CustomLog.e(TAG, "json 构造出错 : " + e.getMessage());
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 关闭本地麦克风: 先向信令服务器申请停止发布音频流，成功后停止声音采集。
     *
     * @param listener 操作回调接口，信令接口的通知结果通过该回调返回
     * @return 0： 操作成功
     * <p>
     * -1: 当前未发言，无法关闭
     * <p>
     * -2: 当前已是关闭状态
     * <p>
     * -4：关闭失败
     * <p>
     * -5：未进行初始化
     */
    public int closeMicrophone(ButelOpenSDKOperationListener listener) {
        CustomLog.d(TAG, "ButelOpenSDK::closeMicrophone 关闭本地麦克风");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
        if (speaker == null) {
            CustomLog.w(TAG,
                    "ButelOpenSDK::openMicrophone error! 当前未发言，无法关闭麦克风");
            return -1;
        }

        if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_OFF) {
            CustomLog.w(TAG, "ButelOpenSDK::openMicrophone error! 当前已是关闭状态。");
            return -2;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation = "
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        JSONArray jsonArray = new JSONArray();
        JSONObject audeoStreamJson = new JSONObject();
        try {
            audeoStreamJson.put(CmdKey.RESOURCE_ID,
                    String.valueOf(StreamInfo.RESOURCE_TYPE_AUDIO));
            audeoStreamJson.put(CmdKey.MEDIA_TYPE,
                    SpeakerInfo.ParseSmMediaTypeToStp(MediaType.TYPE_AUDIO));
            audeoStreamJson.put(CmdKey.EXTRAINFO,
                    Cmd.createExtraInfo(speaker.getSyncId(), -1, -1));

            jsonArray.put(audeoStreamJson);
            CustomLog.d(TAG, "jsonArray=" + jsonArray.toString());

            int result = mMeetingControl.askPublishOrUnPublishStreams(mToken,
                    Cmd.STREAM_OPERATOR_TYPE_UNPUBLISH, jsonArray);
            if (result != 0) {
                CustomLog.w(TAG, "向 STP 取消发布流失败 result: " + result);
                return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
            } else {
                mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_CLOSE_MIC;
                mAskForCloseMicPhoneListenerList.add(listener);
            }
        } catch (JSONException e) {
            CustomLog.e(TAG, "json 构造出错 : " + e.getMessage());
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 有新参会方加入时，如果自己的扬声器是关闭状态，则取消订阅新的音频流
     */
    public void personJoinMeetingCloseUserLoudSpeaker() {
        CustomLog.d(TAG, "personJoinMeetingCloseUserLoudSpeaker");

        for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {

            SpeakerInfo speaker = mMeetingInfo.getSpeakers().get(i);
            String userId = speaker.getAccountId();
            CustomLog.e(TAG, "userId=" + userId + "mMeetingInfo.accountId="
                    + mMeetingInfo.accountId);
            if (userId.equals(mMeetingInfo.accountId)) {
                CustomLog.d(TAG,
                        "ButelOpenSDK::unsubscribeStreams() 针对自己的流信息，不做处理。");
            } else {
                int syncId = speaker.getSyncId();
                ArrayList<Integer> removeStreamIdList = new ArrayList<Integer>();
                Iterator<Entry<Integer, StreamInfo>> it = speaker
                        .getmStreamsInfo().entrySet().iterator();

                while (it.hasNext()) {
                    Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
                            .next();
                    int key = (Integer) entry.getKey();
                    StreamInfo val = (StreamInfo) entry.getValue();
                    int mediaType = MediaType.TYPE_AUDIO;
                    CustomLog.i(
                            TAG,
                            "ButelOpenSDK::unsubscribeStreams() mediaType="
                                    + mediaType + "streamInfo ="
                                    + val.toString());

                    if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_OFF) {
                        CustomLog.i(TAG, "麦克风为关闭状态,不取消订阅音频流");

                        mMeetingInfo
                                .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_CLOSE);

                    } else if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_ON) {

                        if (val.getMediaType() == MediaType.TYPE_AUDIO) {
                            int result = mSMSDKControl.unSucscribe(val
                                    .getStreamId());
                            val.setSyncId(syncId);

                            if (result == 0) {
                                CustomLog.i(
                                        TAG,
                                        "ButelOpenSDK 取消订阅流成功。 mediaType： "
                                                + mediaType
                                                + " | streamId: "
                                                + String.valueOf(val
                                                .getStreamId()));
                                CustomLog
                                        .d(TAG,
                                                "ButelOpenSDK::unSucscribe() 取消音频流发布，将扬声器置为关闭状态");
                                removeStreamIdList.add(key);

                                mMeetingInfo
                                        .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_CLOSE);

                            } else {
                                CustomLog.w(
                                        TAG,
                                        "ButelOpenSDK::unsubscribeStreams() 取消订阅流失败. mediaType: "
                                                + mediaType
                                                + " | streamId: "
                                                + String.valueOf(val
                                                .getStreamId()));
                            }

                        }

                    }

                }

                for (int res : removeStreamIdList) {
                    speaker.setAudioResourceId(res);
                    speaker.removeStreamInfo(res);
                }

                CustomLog.d(TAG,
                        "ButelOpenSDK::personJoinMeetingCloseUserLoudSpeaker() 清理发言者的订阅信息结束 "
                                + speaker.toString());

            }

        }

    }

    /**
     * 参会者被主持人通知关闭扬声器后的操作
     */
    public void masterCloseUserLoudSpeaker() {
        CustomLog.d(TAG, "masterCloseUserLoudSpeaker");

        boolean notified = false;

        for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {

            SpeakerInfo speaker = mMeetingInfo.getSpeakers().get(i);
            String userId = speaker.getAccountId();
            CustomLog.e(TAG, "userId=" + userId + "mMeetingInfo.accountId="
                    + mMeetingInfo.accountId);
            if (userId.equals(mMeetingInfo.accountId)) {
                CustomLog.d(TAG,
                        "ButelOpenSDK::unsubscribeStreams() 针对自己的流信息，不做处理。");
            } else {
                int syncId = speaker.getSyncId();
                ArrayList<Integer> removeStreamIdList = new ArrayList<Integer>();
                Iterator<Entry<Integer, StreamInfo>> it = speaker
                        .getmStreamsInfo().entrySet().iterator();

                while (it.hasNext()) {
                    Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
                            .next();
                    int key = (Integer) entry.getKey();
                    StreamInfo val = (StreamInfo) entry.getValue();
                    int mediaType = MediaType.TYPE_AUDIO;
                    CustomLog.i(
                            TAG,
                            "ButelOpenSDK::unsubscribeStreams() mediaType="
                                    + mediaType + "streamInfo ="
                                    + val.toString());

                    if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_OFF) {
                        CustomLog.i(TAG, "麦克风为关闭状态,不取消订阅音频流");

                        mMeetingInfo
                                .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_CLOSE);

                        if (!notified) {
                            notifyUserClosedLoudspeaker();
                            handleNotifyListenerListOnNotify(
                                    NotifyType.MASTER_CLOSE_USERLOUDSPEAKER_SUCCESS,
                                    null);
                            notified = true;
                        }

                    } else if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_ON) {

                        if (val.getMediaType() == MediaType.TYPE_AUDIO) {
                            int result = mSMSDKControl.unSucscribe(val
                                    .getStreamId());
                            val.setSyncId(syncId);

                            if (result == 0) {
                                CustomLog.i(
                                        TAG,
                                        "ButelOpenSDK 取消订阅流成功。 mediaType： "
                                                + mediaType
                                                + " | streamId: "
                                                + String.valueOf(val
                                                .getStreamId()));
                                CustomLog
                                        .d(TAG,
                                                "ButelOpenSDK::unSucscribe() 取消音频流发布，将扬声器置为关闭状态");
                                removeStreamIdList.add(key);

                                mMeetingInfo
                                        .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_CLOSE);

                                if (!notified) {
                                    notifyUserClosedLoudspeaker();
                                    handleNotifyListenerListOnNotify(
                                            NotifyType.MASTER_CLOSE_USERLOUDSPEAKER_SUCCESS,
                                            null);
                                    notified = true;
                                }

                            } else {
                                CustomLog.w(
                                        TAG,
                                        "ButelOpenSDK::unsubscribeStreams() 取消订阅流失败. mediaType: "
                                                + mediaType
                                                + " | streamId: "
                                                + String.valueOf(val
                                                .getStreamId()));
                                if (!notified) {
                                    handleNotifyListenerListOnNotify(
                                            NotifyType.MASTER_CLOSE_USERLOUDSPEAKER_FAIL,
                                            null);
                                    notified = true;
                                }
                            }

                        }

                    }

                }

                for (int res : removeStreamIdList) {
                    speaker.setAudioResourceId(res);
                    speaker.removeStreamInfo(res);
                }

                CustomLog.d(TAG,
                        "ButelOpenSDK::masterCloseUserLoudSpeaker() 清理发言者的订阅信息结束 "
                                + speaker.toString());

            }

        }

    }

    /**
     * 参会者被主持人通知打开扬声器后的操作
     */
    public void masterOpenUserLoudSpeaker() {

        boolean notified = false;

        CustomLog.d(TAG, "masterOpenUserLoudSpeaker");

        for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {

            SpeakerInfo speaker = mMeetingInfo.getSpeakers().get(i);
            String userId = speaker.getAccountId();
            CustomLog.e(TAG, "userId=" + userId + "mMeetingInfo.accountId="
                    + mMeetingInfo.accountId);
            if (userId.equals(mMeetingInfo.accountId)) {
                CustomLog.d(TAG,
                        "ButelOpenSDK::subscribeStreams() 针对自己的流信息，不做处理。");
            } else {
                int syncId = speaker.getSyncId();
                Iterator<Entry<Integer, StreamInfo>> it = speaker
                        .getmStreamsInfo().entrySet().iterator();

                StreamInfo val = new StreamInfo();
                int mediaType = MediaType.TYPE_AUDIO;
                // val.setResourcesId(StreamInfo.RESOURCE_TYPE_AUDIO);
                if (speaker.getAudioResourceId() == -1) {
                    speaker.setAudioResourceId(StreamInfo.RESOURCE_TYPE_AUDIO);
                }
                val.setResourcesId(speaker.getAudioResourceId());
                speaker.setAudioResourceId(-1);
                val.setMediaType(mediaType);
                val.setSyncId(syncId);

                if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_OFF) {
                    CustomLog.i(TAG, "麦克风为关闭状态,不订阅音频流");

                    mMeetingInfo
                            .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_OPEN);

                    if (!notified) {
                        notifyUserClosedLoudspeaker();
                        handleNotifyListenerListOnNotify(
                                NotifyType.MASTER_OPEN_USERLOUDSPEAKER_SUCCESS,
                                null);

                        notified = true;
                    }

                } else if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_ON) {

                    int streamId = mSMSDKControl.subscribe(userId, mediaType,
                            val.getResourcesId(), syncId);
                    if (streamId > 0) {
                        CustomLog.i(
                                TAG,
                                "ButelOpenSDK 订阅流成功。 mediaType： " + mediaType
                                        + " | streamId: "
                                        + String.valueOf(streamId)
                                        + " | resourceId:"
                                        + String.valueOf(val.getResourcesId()));
                        CustomLog
                                .d(TAG,
                                        "ButelOpenSDK::subscribeStreams() 存在音频流发布，将扬声器置为打开状态");
                        val.setStreamId(streamId);

                        mMeetingInfo
                                .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_OPEN);

                        if (!notified) {
                            notifyUserOpenedLoudspeaker();
                            handleNotifyListenerListOnNotify(
                                    NotifyType.MASTER_OPEN_USERLOUDSPEAKER_SUCCESS,
                                    null);

                            notified = true;
                        }

                    } else {
                        CustomLog.w(
                                TAG,
                                "ButelOpenSDK::subscribeStreams() 订阅流失败. mediaType: "
                                        + mediaType + " | streamId: "
                                        + String.valueOf(val.getStreamId())
                                        + " | resourceId:"
                                        + String.valueOf(val.getResourcesId()));

                        if (!notified) {
                            handleNotifyListenerListOnNotify(
                                    NotifyType.MASTER_OPEN_USERLOUDSPEAKER_FAIL,
                                    null);

                            notified = true;
                        }

                    }

                    if (val.getStreamId() != -1) {
                        speaker.addStreamInfo(val.getResourcesId(), val);
                    }

                    CustomLog.d(TAG,
                            "ButelOpenSDK::addSpeakerSubscribeStrems() 添加发言者的订阅信息结束 "
                                    + speaker.toString());

                }
            }
        }

    }

    /**
     * 申请打开扬声器
     */
    public int askForOpenLoudspeaker(ButelOpenSDKOperationListener listener) {

        CustomLog.d(TAG, "ButelOpenSDK::askForOpenLoudspeaker 申请打开本地扬声器");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.getLoudspeakerStatus() == MeetingInfo.LOUNDSPEAKER_OPEN) {
            CustomLog.w(TAG,
                    "ButelOpenSDK::askForOpenLoudspeaker error! 当前已是打开状态。");
            return -2;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation="
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int ret = mMeetingControl.askForOpenLoudspeaker(mToken);

        if (ret != 0) {
            CustomLog.d(TAG,
                    "ButelOpenSDK::askForOpenLoudspeaker() 申请打开本地扬声器失败");
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        } else {
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_OPEN_LOUDSPEAKER;
            mAskForOpenLoudspeakerListenerList.add(listener);
        }

        mMeetingInfo.setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_OPEN);

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 申请关闭扬声器
     */
    public int askForCloseLoudspeaker(ButelOpenSDKOperationListener listener) {

        CustomLog.d(TAG, "ButelOpenSDK::askForCloseLoudspeaker 申请关闭本地扬声器");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.getLoudspeakerStatus() == MeetingInfo.LOUNDSPEAKER_CLOSE) {
            CustomLog.w(TAG,
                    "ButelOpenSDK::askForCloseLoudspeaker error! 当前已是关闭状态。");
            return -2;
        }

        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.w(TAG, "正在执行其他操作...mMeetingControlOperation="
                    + mMeetingControlOperation);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }

        int ret = mMeetingControl.askForCloseLoudspeaker(mToken);

        if (ret != 0) {
            CustomLog.d(TAG,
                    "ButelOpenSDK::askForCloseLoudspeaker() 申请关闭本地扬声器失败");
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        } else {
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.ASK_FOR_CLOSE_LOUDSPEAKER;
            mAskForCloseLoudspeakerListenerList.add(listener);
        }

        mMeetingInfo.setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_CLOSE);

        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;

    }

    /**
     * 通过请求打开扬声器来改变自己在其他终端显示的扬声器状态
     */
    public void notifyUserOpenedLoudspeaker() {
        CustomLog
                .d(TAG,
                        "ButelOpenSDK::notifyUserOpenedLoudspeaker 通过请求打开扬声器来改变自己在其他终端显示的扬声器状态");
        mMeetingControl.askForOpenLoudspeaker(mToken);
    }

    /**
     * 通过请求关闭扬声器来改变自己在其他终端显示的扬声器状态
     */
    public void notifyUserClosedLoudspeaker() {
        CustomLog
                .d(TAG,
                        "ButelOpenSDK::notifyUserClosedLoudspeaker 通过请求关闭扬声器来改变自己在其他终端显示的扬声器状态");
        mMeetingControl.askForCloseLoudspeaker(mToken);
    }

    /**
     * 设置进入会议是否自动发言。 仅对 调用 {@link joinMeeting()} 接口之前有效
     *
     * @param autoSpeak </p> true: 进入会议，如果有空闲mic，且会议模式是自由模式，服务器将允许其发言 </p> false:
     *                  进入会议，服务器不会自动让其发言
     * @return </p> == 0设置成功 </p> != 0设置失败
     */
    public int setAutoSpeak(boolean autoSpeak) {
        if (mMeetingInfo == null) {
            mMeetingInfo = new MeetingInfo();
        }
        mMeetingInfo.isAutoSpeak = autoSpeak;

        return 0;
    }

    @Deprecated
    /***
     * 设置发言时的摄像头采集模式
     *
     * @param rate
     *          <p>
     *          SMSDKCaptureRateCode.SUPER_CAPTURE_RATE = 1 : 超清
     *          <p>
     *          SMSDKCaptureRateCode.HIGH_CAPTURE_RATE = 2 : 高清
     *          <p>
     *          SMSDKCaptureRateCode.STANDARD_CAPTURE_RATE = 3 : 标清
     *          <p>
     *          SMSDKCaptureRateCode.SMOOTH_CAPTURE_RATE = 4 : 流畅
     * @return <p>
     *         0: 成功
     *         <p>
     *         !=0:失败
     */
    public int setVideoCaptureRate(int rate) {
        CustomLog.d(TAG, "ButelOpenSDK::setVideoCaptureRate 设置视频采集模式: " + rate);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        // return mSMSDKControl.setVideoCaptureRate(rate);
        return 0;
    }

    /**
     * 设置视频参数，包括采集参数、编码参数。如果设置的值不支持，将使用与设置值最匹配的值。
     *
     * @param resourceId 设备id/ 摄像头index索引
     * @param param      视频参数 {@link VideoParameter}
     * @return <p>
     * 0:成功
     * <p>
     * !=0:失败
     */
    public int setVideoParam(int resourceId, VideoParameter param) {
        CustomLog.d(TAG, "ButelOpenSDK::setVideoParam 设置视频采集模式.resourceId: "
                + resourceId);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        return mSMSDKControl.setVideoParam(resourceId, param);
    }

    /**
     * 设置媒体自适应配置，初始化之后加入会议之前设置
     *
     * @param param 自适应json格式值
     * @return <p>
     * 0: 表示成功
     * <p>
     * !=0: 表示失败
     */
    public int setMediaAdaptiveParam(String param) {
        CustomLog
                .d(TAG, "ButelOpenSDK::setMediaAdaptiveParam 设置媒体自适应配置.param: "
                        + param);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        return mSMSDKControl.setMediaAdaptiveParam(param);
    }

    /**
     * 设置Relay服务器白名单。
     * <p>
     * init()之后 加入会议之前进行设置
     * <p>
     * RelayRC 会保存relayId对应的服务器地址和端口，通过设置relayId白名单，流媒体在进行传输时将优先使用这些地址
     *
     * @param whitelist 白名单，格式为"id_1|id_2|id_3..."
     * @return <p>
     * 0：成功；
     * <p>
     * !=0: 失败
     */
    public int setRelayWhiteList(String whitelist) {
        CustomLog.d(TAG, "ButelOpenSDK::setRelayWhiteList() 设置relay白名单. "
                + whitelist);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        mRelayIdWhiteList = whitelist;
        return 0;
    }

    /**
     *
     * @return VideoParameter
     */
    /**
     * 获取渲染视频信息，如数据分享窗口的大小
     *
     * @param accountId 目标窗口视讯号
     * @param mediaType 目标媒体类型
     * @return
     */
    public VideoParameter getRenderVideoInfo(String accountId, int mediaType) {
        CustomLog.e(TAG, "ButelOpenSDK::getRenderVideoInfo() ");
        CustomLog.i(TAG, "ButelOpenSDK::getRenderVideoInfo() accountId: "
                + accountId + " | StreamType: " + mediaType);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return null;
        }

        if (TextUtils.isEmpty(accountId)) {
            CustomLog.w(TAG, "参数错误 accoutId不能为空");
            return null;
        }

        if (mediaType != MediaType.TYPE_VIDEO
                && mediaType != MediaType.TYPE_DOC_VIDEO) {
            CustomLog.w(TAG, "Warning! mediaType不是视频或者文档视频，不支持获取窗口渲染参数");
            return null;
        }

        SpeakerInfo speaker = mMeetingInfo.getSpeakerByAccountId(accountId);

        if (speaker == null) {
            return null;
        }

        int streamId = speaker.getSreamIdByMediaType(mediaType);
        if (streamId == -1) {
            CustomLog.w(TAG, "该视讯号没有对应的流信息发布");
            return null;
        }

        if (mSMSDKControl == null || String.valueOf(streamId) == null) {
            CustomLog.d(TAG,
                    "mSMSDKControl==null||String.valueOf(streamId)==null");
            return null;
        } else {
            CustomLog.d(TAG, "mSMSDKControl.getRenderVideoInfo");
            return mSMSDKControl.getRenderVideoInfo(streamId);
        }

    }

    /**
     * 获取网络信号强度
     *
     * @return 0~4，其中0表示没有获取到信号强度；1表示最强，4表示信号强度最弱
     */
    public int getSignalStrengthLevel() {
        CustomLog.d(TAG, "ButelOpenSDK::getSignalStrengthLevel() 获取网络信号强度： "
                + mSignalStrengthLevel);
        return mSignalStrengthLevel;
    }

    /**
     * 获取屏幕分享端口，该端口用于发布到HostAgent处。<bad design>
     *
     * @return port 端口值
     */
    public int getScreenSharingPort() {
        if (mScreenShareControl != null) {
            return mScreenShareControl.getPort();
        }
        return -1;
    }

    /**
     * 获取参会者列表信息，加入会议后调用，获取当前的参会者列表。会议过程中需要根据参会方加入、退出会议自行维护参会者列表。 <need
     * redesign:需要重新设计，返回参会者列表信息>
     *
     * @return 参会者jsonArray格式信息
     */
    public JSONArray getParticipatorList() {
        return mParticipatorList;
    }

    /**
     * 自己是否正在发言
     *
     * @return <p>
     * true: 发言
     * <p>
     * false: 未发言
     */
    public boolean isSpeaking() {
        return getSpeakerInfoById(mMeetingInfo.accountId) != null;
    }

    /**
     * 获取会议模式，当前模式分为指定发言模式和自由模式
     *
     * @return <p>
     * {@link MeetingStyleStatus.MASTER_MODE} ：指定发言（主持）模式
     * <p>
     * {@link MeetingStyleStatus.FREE_MODE} : 自由模式
     */
    public int getMeetingMode() {
        return mMeetingInfo.mMeetingStyle;
    }

    /**
     * 获取主持人视讯号
     */
    public String getMasterAccountId() {
        CustomLog.d(TAG, "getMasterAccountId id="
                + mMeetingInfo.mMeetingMasterAccountId);
        return mMeetingInfo.mMeetingMasterAccountId;
    }

    /**
     * 获取主持人名称
     */
    public String getMasterName() {
        return mMeetingInfo.mMeetingMasterName;
    }

    /**
     * 获取当前角色
     *
     * @return <p>
     * {@link CurrentRoleStatus.MASTER_ROLE} 主持人
     * <p>
     * {@link CurrentRoleStatus.COMMON_ROLE} 参会方
     */
    public int getCurrentRole() {
        return mMeetingInfo.mCurrentRole;
    }

    /**
     * 获取直播状态
     *
     * @return <p>
     * {@link LiveStatus.LIVING_NOW} 直播中
     * <p>
     * {@link LiveStatus.NO_START_LIVING} 未开启直播
     */
    public int getLiveStatus() {
        return mMeetingInfo.mLiveStatus;
    }

    @Deprecated
    /**
     * 是否是视频模式
     *
     * @return <p>
     *         true: 视频模式
     *         <p>
     *         false: 语音模式
     */
    public boolean isVideoMode() {
        return true;
    }

    /**
     * 获取会议加锁状态
     *
     * @return <p>
     * 1: 加锁
     * <p>
     * 0: 未加锁
     */
    public int getLockInfo() {
        return mMeetingInfo.intentLockInfo;
    }

    /**
     * 获取文档分享名称
     */
    public String getShareDocName() {
        return mMeetingInfo.shareDocName;
    }

    /**
     * 获取发言者列表
     */
    public List<SpeakerInfo> getSpeakers() {
        return mMeetingInfo.getSpeakers();
    }

    /**
     * 通过视讯号查询发言者
     *
     * @param accountId 视讯号
     * @return <p>
     * null: 如果该视讯号没有发言
     * <p>
     * SpeakerInfo: 该视讯号对应的发言者
     * @see SpeakerInfo
     */
    public SpeakerInfo getSpeakerInfoById(String accountId) {
        return mMeetingInfo.getSpeakerByAccountId(accountId);
    }

    /**
     * 获取自己的扬声器状态
     */
    public int getMyLoudspeakerStatus() {
        return mMeetingInfo.getLoudspeakerStatus();
    }

    /**
     * 发送数据视图控制点坐标，将触发数据视图提供者改变数据窗口显示。
     * <p>
     * 只有发言者有权限调用
     * <p>
     * 操作成功与否没有回应
     *
     * @param dataviewerId 数据视图提供者视讯号
     * @param valueX       x坐标，相对于数据视图原始大小，width >= x >= 0
     * @param valueY       y坐标，相对于数据视图原始大小，height >= y >= 0
     * @return 0 调用成功，!=0 调用失败
     */
    public int sendDataViewControlPoint(String dataviewerId, float valueX,
                                        float valueY) {
        CustomLog.i(TAG,
                "ButelOpenSDK::sendDataViewControlPoint() 反向操作数据视图画面 dataviewerId: "
                        + dataviewerId + " |valueX: " + valueX + " |valueY: "
                        + valueY);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
        if (speaker == null) {
            CustomLog
                    .w(TAG,
                            "ButelOpenSDK::sendDataViewControlPoint error! 当前未发言，没有权限进行反向操作数据视图");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        int result = mMeetingControl.sendDataViewControlPoint(dataviewerId,
                valueX, valueY);
        if (result != 0) {
            CustomLog.e(TAG, "向 STP 发送反向操作数据视图画面失败 result: " + result);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        return 0;
    }

    @Deprecated
    /**
     * 自预览窗口大小发生改变时，调用该接口，触发调整自预览画面：保持原始比例，居中显示
     * @param previewWidth 自预览窗口宽
     * @param previewHeight 自预览窗口高
     * @return ==0 成功， != 0 失败
     */
    public int adjustPreviewSize(int previewWidth, int previewHeight) {

        return mSMSDKControl.adjustPreviewSize(previewWidth, previewHeight);
    }

    /****************** private *******************************************************************************************/
    private int initMeetingControl() {
        CustomLog.i(TAG, "ButelOpenSDK::initMeetingControl()");
        mMeetingControl = new MeetingControl(mContext);
        int result = mMeetingControl.init(mMeetingControlListener,
                mMeetingInfo.meetingId, mMeetingInfo.accountId,
                mMeetingInfo.accountName, mRcURL, mCfgPath, mLogFileOutPath,
                mWhiteBoardFlag);
        CustomLog
                .i(TAG, "ButelOpenSDK::initMeetingControl() result: " + result);
        return result;
    }

    private int initSMSDKControl(boolean selectUvcCamera) {
        CustomLog.i(TAG, "ButelOpenSDK::initSMSDKControl()");

        mSMSDKControl = new SMSDKControl(mContext);
        int result = mSMSDKControl.initSMSDK(mMeetingInfo.meetingId,
                mMeetingInfo.accountId, mRcURL, mCfgPath, mLogFileOutPath,
                mDeviceType, selectUvcCamera, this, mSMSDKListener, mCodecType, mMeetingInfo);

        if (result == 0) {
            setRssiNetInfo();
            mSMSDKControl.setCloseCameraBackGroundId(mCloseCameraId);

            mSMSDKControl.setVideoLoadingImage(mBigImage, mSmallImage);
        }

        return result;
    }

    private int initScreenSharingControl() {
        CustomLog.i(TAG, "ButelOpenSDK::initScreenSharingControl()");
        mScreenShareControl = new ScreenShareControl();
        int result = mScreenShareControl.init(mContext, mMeetingInfo.accountId,
                NetConnectHelper.getLocalIp(mContext), mCfgPath,
                mLogFileOutPath, mScreenShareListener);
        return result;
    }

    private int initGeoMetryControl() {
        CustomLog.i(TAG, "ButelOpenSDK::initGeoMetryControl()");
        mGeoMetryControl = new GeoMetryControl();

        mGeoMetryControl.setWhiteBoardProcessor(mMeetingControl
                .getWhiteBoardProcessor());
        mGeoMetryControl.setMeetingInfo(mMeetingInfo);

        int result = mGeoMetryControl.mediaGeometryInit(mLogFileOutPath,
                mCfgPath, adjustLibPath(),
                Integer.parseInt(mMeetingInfo.accountId), null);
        return result;
    }

    private String adjustLibPath() {
        String path = mContext.getApplicationContext().getPackageResourcePath();
        String libRDCodecPath = "/data/data/" + mContext.getPackageName()
                + "/lib";
        CustomLog.d(TAG, "tttttttttt  libRDCodecPath = " + libRDCodecPath
                + "path=" + path);
        Log.e(TAG, "tttttttttt  libRDCodecPath = " + libRDCodecPath + "path="
                + path);
        return libRDCodecPath;
    }

    // 设置网络信息
    private void setRssiNetInfo() {
        CustomLog.i(TAG, "ButelOpenSDK::setNetInfo()");
        if (mSMSDKControl != null) {
            int mNetType = NetConfig.getNetType(mContext);
            String mActiveIp = NetConnectHelper.getLocalIp(mContext);
            int mSpeed = PhoneStateMonitor.getInstance(mContext).getLinkSpeed();
            int mRssi = PhoneStateMonitor.getInstance(mContext).getRSSI();
            mSMSDKControl.setNetInfo(mActiveIp, mNetType, mSpeed, mRssi, "");
            CustomLog.i(TAG, "mActiveIp=" + mActiveIp + "mNetType" + mNetType
                    + "mSpeed" + mSpeed + "mRssi=" + mRssi);
        }
    }

    private int reJoinMeeting(String newToken) {
        CustomLog.i(TAG, "发送重新加入会议命令  mMeetingControlOperation = "
                + mMeetingControlOperation);
        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            CustomLog.i(TAG, "重新加入会议失败,存在正在操作事件: "
                    + ButelOpenSDKReturnCode.RETURN_PENDING_ERROR);
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.RE_JOIN_MEETING;
        mToken = newToken;
        int returnCode = mMeetingControl.joinMeeting(mToken,
                mMeetingInfo.adminId, isSpeaking() ? MemberStatus.IS_SPEAKING
                        : MemberStatus.IS_LISTENING, 1,
                mMeetingInfo.accountName, mMeetingInfo
                        .getStreamDesp(mMeetingInfo.accountId),
                mRelayIdWhiteList, mMeetingInfo.isAutoSpeak, mAdminFlag, mPollFlag);
        if (returnCode != MeetingControlReturnCode.SUCCESS) {
            CustomLog.i(TAG, "重新加入会议失败 : "
                    + ButelOpenSDKReturnCode.RETURN_OTHER_ERROR);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        CustomLog.i(TAG, "重新加入会议操作请求成功，等待服务器返回！");
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    private int askForParticipatorList() {
        CustomLog.i(TAG, "发送获取参会者列表命令  mMeetingControlOperation = "
                + mMeetingControlOperation);
        if (mMeetingControlOperation != ButelOpenSDKMeetingControlOperationCode.NULL) {
            return ButelOpenSDKReturnCode.RETURN_PENDING_ERROR;
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.GET_PARTICIPATOR_LIST;
        int returnCode = mMeetingControl.getParticipatorList(mToken);
        if (returnCode != MeetingControlReturnCode.SUCCESS) {
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }
        CustomLog.i(TAG, "发送获取参会者列表命令结束");
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    private void handleAskForOpenCameraResp(boolean flag, Cmd cmd) {
        CustomLog.d(TAG,
                "ButelOpenSDK::handleAskForOpenCamera()处理申请打开摄像头回应，是否操作成功："
                        + flag);
        if (flag) {
            CustomLog
                    .d(TAG,
                            "打开摄像头操作,mMeetingType转为MEETING_TYPE_VIDEO,mFirstOpenCamera转为true");
            setMeetingShowType(MEETING_TYPE_VIDEO, true);
            if (mCodecType == 0) {
                CustomLog.d(TAG,
                        "handleAskForOpenCameraResp startCameraPreview 软编软解");
                int result = mSMSDKControl.startCameraPreview();
                // int result = 0;
                if (result != 0) {
                    CustomLog.e(TAG, "打开摄像头失败,result = " + result);
                    callbackAskForOpenCameraOperation(false, null);
                    return;
                }
            } else {
                // initCamera();
                CustomLog.d(TAG,
                        "handleAskForOpenCameraResp startCameraPreview 硬编硬解");
                // int result = mSMSDKControl.startCameraPreview();
                // int result = 0;
                // if (result != 0) {
                // CustomLog.e(TAG, "打开摄像头失败,result = " + result);
                // callbackAskForOpenCameraOperation(false, null);
                // return;
                // }
                // else {
                // mMeetingInfo.mCameraIsOpen = true;
                // CustomLog.e(TAG, "打开摄像头成功,result = " + result);
                // }
            }

            SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
            if (speaker == null) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::OpenCamera error! 当前未发言，无法打开摄像头");
                callbackAskForOpenCameraOperation(false, cmd);
                return;
            }

            if (speaker.getCamStatus() == SpeakerInfo.CAM_STATUS_OPEN) {
                CustomLog.e(TAG, "ButelOpenSDK::OpenCamera error! 当前已是打开状态。");
            }

            int syncId = speaker.getSyncId();

            StreamInfo videoStream = new StreamInfo(-1,
                    StreamInfo.RESOURCE_TYPE_VIDEO, syncId,
                    MediaType.TYPE_VIDEO);
            int videoStreamId = mSMSDKControl.publishCameraVideo(
                    StreamInfo.RESOURCE_TYPE_VIDEO, syncId);

            if (videoStreamId < 0) {
                CustomLog.e(TAG, "ButelOpenSDK::OpenCamera error! 发布视频流失败。");
                callbackAskForOpenCameraOperation(false, cmd);
                return;
            }
            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_OPEN);
            videoStream.setStreamId(videoStreamId);
            CustomLog.i(TAG, "发布音频流  streamId: " + videoStreamId);

            speaker.addStreamInfo(StreamInfo.RESOURCE_TYPE_VIDEO, videoStream);
        }

        callbackAskForOpenCameraOperation(flag, cmd);
    }

    private void handleAskForCloseLoudSpeakerResp(boolean flag, Cmd cmd) {

        CustomLog.d(TAG,
                "ButelOpenSDK::handleAskForCloseLoudspeakerResp()处理申请关闭扬声器回应，是否操作成功："
                        + flag);
        if (flag) {

            for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {

                SpeakerInfo speaker = mMeetingInfo.getSpeakers().get(i);
                String userId = speaker.getAccountId();
                CustomLog.e(TAG, "userId=" + userId + "mMeetingInfo.accountId="
                        + mMeetingInfo.accountId);
                if (userId.equals(mMeetingInfo.accountId)) {
                    CustomLog
                            .d(TAG,
                                    "ButelOpenSDK::unsubscribeStreams() 针对自己的流信息，不做处理。");
                } else {
                    int syncId = speaker.getSyncId();
                    ArrayList<Integer> removeStreamIdList = new ArrayList<Integer>();
                    Iterator<Entry<Integer, StreamInfo>> it = speaker
                            .getmStreamsInfo().entrySet().iterator();

                    while (it.hasNext()) {
                        Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
                                .next();
                        int key = (Integer) entry.getKey();
                        StreamInfo val = (StreamInfo) entry.getValue();
                        int mediaType = MediaType.TYPE_AUDIO;
                        CustomLog.i(
                                TAG,
                                "ButelOpenSDK::unsubscribeStreams() mediaType="
                                        + mediaType + "streamInfo ="
                                        + val.toString());

                        if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_OFF) {
                            CustomLog.i(TAG, "麦克风为关闭状态,不取消订阅音频流");
                            mMeetingInfo
                                    .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_CLOSE);
                        } else if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_ON) {

                            if (val.getMediaType() == MediaType.TYPE_AUDIO) {
                                int result = mSMSDKControl.unSucscribe(val
                                        .getStreamId());
                                val.setSyncId(syncId);

                                if (result == 0) {
                                    CustomLog.i(
                                            TAG,
                                            "ButelOpenSDK 取消订阅流成功。 mediaType： "
                                                    + mediaType
                                                    + " | streamId: "
                                                    + String.valueOf(val
                                                    .getStreamId()));
                                    CustomLog
                                            .d(TAG,
                                                    "ButelOpenSDK::unSucscribe() 取消音频流发布，将扬声器置为关闭状态");
                                    removeStreamIdList.add(key);
                                    mMeetingInfo
                                            .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_CLOSE);
                                } else {
                                    CustomLog.w(
                                            TAG,
                                            "ButelOpenSDK::unsubscribeStreams() 取消订阅流失败. mediaType: "
                                                    + mediaType
                                                    + " | streamId: "
                                                    + String.valueOf(val
                                                    .getStreamId()));
                                }

                            }

                        }

                    }

                    for (int res : removeStreamIdList) {
                        speaker.setAudioResourceId(res);
                        speaker.removeStreamInfo(res);
                    }

                    CustomLog.d(TAG,
                            "ButelOpenSDK::handleAskForCloseLoudSpeakerResp() 清理发言者的订阅信息结束 "
                                    + speaker.toString());

                }

            }

        }

        callbackAskForCloseLoudspeakerOperation(flag, cmd);

    }

    private void handleAskForOpenLoudSpeakerResp(boolean flag, Cmd cmd) {

        CustomLog.d(TAG,
                "ButelOpenSDK::handleAskForOpenLoudSpeakerResp()处理申请打开扬声器回应，是否操作成功："
                        + flag);
        if (flag) {

            for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {

                SpeakerInfo speaker = mMeetingInfo.getSpeakers().get(i);
                String userId = speaker.getAccountId();
                CustomLog.e(TAG, "userId=" + userId + "mMeetingInfo.accountId="
                        + mMeetingInfo.accountId);
                if (userId.equals(mMeetingInfo.accountId)) {
                    CustomLog.d(TAG,
                            "ButelOpenSDK::subscribeStreams() 针对自己的流信息，不做处理。");
                } else {
                    int syncId = speaker.getSyncId();
                    Iterator<Entry<Integer, StreamInfo>> it = speaker
                            .getmStreamsInfo().entrySet().iterator();

                    StreamInfo val = new StreamInfo();
                    int mediaType = MediaType.TYPE_AUDIO;
                    // val.setResourcesId(StreamInfo.RESOURCE_TYPE_AUDIO);
                    // CustomLog.d(TAG,
                    // "handleAskForOpenLoudSpeakerResp setResourcesId"+String.valueOf(val.getResourcesId()));
                    // val.setResourcesId(StreamInfo.RESOURCE_TYPE_AUDIO);
                    if (speaker.getAudioResourceId() == -1) {
                        speaker.setAudioResourceId(StreamInfo.RESOURCE_TYPE_AUDIO);
                    }
                    val.setResourcesId(speaker.getAudioResourceId());
                    speaker.setAudioResourceId(-1);
                    val.setMediaType(mediaType);
                    val.setSyncId(syncId);

                    if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_OFF) {
                        CustomLog.i(TAG, "麦克风为关闭状态,不订阅音频流");
                        mMeetingInfo
                                .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_OPEN);
                    } else if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_ON) {

                        int streamId = mSMSDKControl.subscribe(userId,
                                mediaType, val.getResourcesId(), syncId);
                        if (streamId > 0) {
                            CustomLog.i(
                                    TAG,
                                    "ButelOpenSDK 订阅流成功。 mediaType： "
                                            + mediaType
                                            + " | streamId: "
                                            + String.valueOf(streamId)
                                            + " | resourceId:"
                                            + String.valueOf(val
                                            .getResourcesId()));
                            CustomLog
                                    .d(TAG,
                                            "ButelOpenSDK::subscribeStreams() 存在音频流发布，将扬声器置为打开状态");
                            val.setStreamId(streamId);
                            mMeetingInfo
                                    .setLoudspeakerStatus(MeetingInfo.LOUNDSPEAKER_OPEN);
                        } else {
                            CustomLog.w(
                                    TAG,
                                    "ButelOpenSDK::subscribeStreams() 订阅流失败. mediaType: "
                                            + mediaType
                                            + " | streamId: "
                                            + String.valueOf(val.getStreamId())
                                            + " | resourceId:"
                                            + String.valueOf(val
                                            .getResourcesId()));
                        }

                        if (val.getStreamId() != -1) {
                            speaker.addStreamInfo(val.getResourcesId(), val);
                        }

                        CustomLog.d(TAG,
                                "ButelOpenSDK::addSpeakerSubscribeStrems() 添加发言者的订阅信息结束 "
                                        + speaker.toString());

                    }

                }
            }

        }

        callbackAskForOpenLoudspeakerOperation(flag, cmd);

    }

    private void handleAskForCloseCameraResp(boolean flag, Cmd cmd) {
        CustomLog.d(TAG,
                "ButelOpenSDK::handleAskForCloseCamera()处理申请关闭摄像头回应，是否操作成功："
                        + flag);
        if (flag) {
//            if (mCodecType == 0) {
            mSMSDKControl.closeCamera();
            int result = mSMSDKControl.releaseCamera();
            if (result != 0) {
                CustomLog.e(TAG, "关闭摄像头失败,result = " + result);
                callbackAskForCloseCameraOperation(false, cmd);
                return;
            }
//            }

            SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
            if (speaker == null) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::CloseCamera error! 当前未发言，无法关闭摄像头");
                callbackAskForCloseCameraOperation(false, cmd);
                return;
            }

            if (speaker.getCamStatus() == SpeakerInfo.CAM_STATUS_CLOSE) {
                CustomLog.e(TAG, "ButelOpenSDK::CloseCamera error! 当前已是关闭状态。");
            }

            int videoStreamId = speaker
                    .getStreamIdByResource(StreamInfo.RESOURCE_TYPE_VIDEO);
            CustomLog.d(TAG, "ButelOpenSDK::CloseCamera 停止发布的streamId: "
                    + videoStreamId);

            int ret = mSMSDKControl.unPublish(videoStreamId);
            if (ret != 0) {
                CustomLog.e(TAG, "ButelOpenSDK::CloseCamera 取消发布视频流失败! ret: "
                        + ret);
            }

            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
            speaker.removeStreamInfo(StreamInfo.RESOURCE_TYPE_VIDEO);
        }

        callbackAskForCloseCameraOperation(flag, cmd);
    }

    private void handleAskForOpenMicResp(boolean flag, Cmd cmd) {
        CustomLog.d(TAG,
                "ButelOpenSDK::handleAskForOpenMicResp()处理申请打开麦克风回应，是否操作成功："
                        + flag);
        if (flag) {
            mOpenMic = true;

            SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
            if (speaker == null) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::openMicrophone error! 当前未发言，无法打开麦克风");
                callbackAskForOpenMic(false, cmd);
                return;
            }

            if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_ON) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::openMicrophone error! 当前已是打开状态。");
            }

            int syncId = speaker.getSyncId();

            StreamInfo audioStream = new StreamInfo(-1,
                    StreamInfo.RESOURCE_TYPE_AUDIO, syncId,
                    MediaType.TYPE_AUDIO);
            int audioStreamId = mSMSDKControl.publishMicAudio(
                    StreamInfo.RESOURCE_TYPE_AUDIO, syncId);

            if (audioStreamId < 0) {
                CustomLog
                        .e(TAG, "ButelOpenSDK::openMicrophone error! 发布音频流失败。");
                callbackAskForOpenMic(false, cmd);

                if (audioStreamId == SMSDKControl.DEVICE_AUDIO_INIT_FAILED) {
                    CustomLog.d(TAG,
                            "ButelOpenSDK::publishStremsInfo() 通知UI音频采集失败");
                    handleNotifyListenerListOnNotify(
                            NotifyType.AUDIO_IN_DEVICE_INIT_FAILED, null);
                }
                return;
            }
            speaker.setMICStatus(SpeakerInfo.MIC_STATUS_ON);
            audioStream.setStreamId(audioStreamId);
            CustomLog.i(TAG, "发布音频流  streamId: " + audioStreamId);

            speaker.addStreamInfo(StreamInfo.RESOURCE_TYPE_AUDIO, audioStream);
        }
        callbackAskForOpenMic(flag, cmd);
    }

    private void handleAskForCloseMicResp(boolean flag, Cmd cmd) {
        CustomLog.d(TAG,
                "ButelOpenSDK::handleAskForCloseMicResp()处理申请关闭麦克风回应，是否操作成功："
                        + flag);
        if (flag) {
            SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
            if (speaker == null) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::closeMicrophone error! 当前未发言，无法关闭麦克风");
                callbackAskForCloseMic(flag, cmd);
                return;
            }

            if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_OFF) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::closeMicrophone error! 当前已是关闭状态。");
            }

            int audioStreamId = speaker
                    .getStreamIdByResource(StreamInfo.RESOURCE_TYPE_AUDIO);
            CustomLog.d(TAG, "ButelOpenSDK::closeMicrophone 停止发布的streamId: "
                    + audioStreamId);

            int ret = mSMSDKControl.unPublish(audioStreamId);
            if (ret != 0) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::closeMicrophone 取消发布音频流失败! ret: " + ret);
            }

            speaker.setMICStatus(SpeakerInfo.MIC_STATUS_OFF);
            speaker.removeStreamInfo(StreamInfo.RESOURCE_TYPE_AUDIO);
        }
        callbackAskForCloseMic(flag, cmd);
    }

    private void handleJoinMeetingResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG, "处理加入会议回应命令 status: " + status);
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        mMeetingInfo.intentLockInfo = cmd.getLockInfo();
        CustomLog.i(TAG, "mCurMeetingInfo.intentLockInfo="
                + mMeetingInfo.intentLockInfo);
        if (status == RespModelStatusCode.SUCCESS) {
            if (cmd.getMeetingState() == 0) {
                CustomLog.w(TAG, "收到7002命令缺少参数");
                callbackJoinMeetingOperation(false, cmd);
                return;
            }

            mMeetingInfo.mMeetingMasterName = cmd.getMasterName();
            mMeetingInfo.mMeetingMasterAccountId = cmd.getMasterId();
            mMeetingInfo.mLiveStatus = cmd.getLiveStatus();
            mMeetingInfo.speakersNum = cmd.getSpeakersNum();
            mMeetingInfo.meetingChatState = cmd.getMeetingChatState();
            if (mStatus == ButelOpenSDKStatusCode.EXCEPTION
                    && cmd.getUserType() == 2) {

                CustomLog.i(TAG, "-981重连参会，并且是主持人，不修改模式");

            } else {
                mMeetingInfo.mMeetingStyle = cmd.getMeetingStyle();
            }
            mMeetingInfo.mCurrentRole = cmd.getUserType();
            String str = KeyEventConfig.ADD_MEETING_BY_NUBE + "_ok_"
                    + mMeetingInfo.accountId + "_加入会议成功";
            KeyEventWrite.write(str);

            if (cmd.getIsRecord() == 1) {
                CustomLog.d(TAG, "handleJoinMeetingResp,是录制状态，开始录制");
                int result = startRecording(
                        cmd.getRecServerAddr().split(":")[0],
                        Integer.valueOf(cmd.getRecServerAddr().split(":")[1]),
                        cmd.getFileAddress());
                if (result == 0) {
                    CustomLog.i(TAG, "开始录制操作成功");
                    mRecordStatus = true;
                    handleNotifyListenerListOnNotify(
                            NotifyType.SERVER_NOTICE_START_RECORDING, cmd);
                } else {
                    CustomLog.i(TAG, "开始录制操作失败，不通知给ui");
                }
            } else {
                CustomLog.d(TAG, "handleJoinMeetingResp,不是录制状态");
            }

            if (cmd.getMeetingState() == 2) { // 2表示会议开始中，该值为历史遗留问题，目前产品不存在进入到会议会议还未开始的场景

                if (mStatus == ButelOpenSDKStatusCode.EXCEPTION) {
                    CustomLog.i(TAG, "加入会议，会议已经开始，发现正在处理981，恢复本地状态");
                    handleJoinMeetingWithExceptionResp(cmd);
                } else {
                    // 根据加入会议回应初始化speakerinfo 列表信息
                    initSpeakerInfo(cmd);

                    CustomLog.i(TAG, "mCurMeetingInfo.mSpeakerInfoList.size="
                            + mMeetingInfo.getSpeakers().size()
                            + mMeetingInfo.getSpeakers().toString());

                    subscribeStreams();
                    mStatus = ButelOpenSDKStatusCode.JOIN_MEETING_SUC;
                    // 合并获取参会者列表到加入会议接口中，不论接口获得成功还是失败，都给加入会议返回成功
                    int returnCode = askForParticipatorList();
                    if (returnCode < 0) {
                        callbackJoinMeetingOperation(true, cmd);
                    }
                }
            }

            // 设置主讲状态，必须是发言才能设置
            if (mMeetingInfo.isSpeaker == SpeakerType.SPEAKER) {
                mSMSDKControl.setSpeakMode(mMeetingInfo.isSpeaker);
            }

        } else if (cmd.getStatus() == RespModelStatusCode.TIME_OUT) {
            String str = KeyEventConfig.ADD_MEETING_BY_NUBE + "_fail_"
                    + mMeetingInfo.accountId + "_加入会议失败,错误码=" + cmd.getStatus();
            KeyEventWrite.write(str);
            if (mStatus == ButelOpenSDKStatusCode.EXCEPTION) {
                CustomLog.i(TAG, "处理981过程中，发现参会返回超时，继续重试");
                if (mMeetingInfo.exceptionRetryTime < 5) {
                    mMeetingInfo.exceptionRetryTime = mMeetingInfo.exceptionRetryTime + 1;
                    mMeetingControl.joinMeeting(mToken, mMeetingInfo.adminId,
                            isSpeaking() ? MemberStatus.IS_SPEAKING
                                    : MemberStatus.IS_LISTENING, 1,
                            mMeetingInfo.accountName, mMeetingInfo
                                    .getStreamDesp(mMeetingInfo.accountId),
                            mRelayIdWhiteList, mMeetingInfo.isAutoSpeak,
                            mAdminFlag, mPollFlag);
                } else {
                    callbackJoinMeetingOperation(false, cmd);
                }
            } else {
                CustomLog.i(TAG, "参会返回超时，通知上层退出会议");
                callbackJoinMeetingOperation(false, cmd);
            }
        }
        // 分支 除掉成功和token超时操作
        else {
            // String str = KeyEventConfig.ADD_MEETING_BY_NUBE + "_fail_"
            // + mCurMeetingInfo.accountId + "_加入会议失败";
            String str = KeyEventConfig.ADD_MEETING_BY_NUBE + "_fail_"
                    + mMeetingInfo.accountId + "_加入会议失败,错误码=" + cmd.getStatus();
            KeyEventWrite.write(str);
            // mStatus = ButelOpenSDKStatusCode.JOIN_MEETING_FAIL;
            callbackJoinMeetingOperation(false, cmd);
        }
        CustomLog.i(TAG, "处理加入会议回应命令结束");
    }

    private int initSpeakerInfo(Cmd cmd) {
        CustomLog.i(TAG, "ButelOpenSDK::initSpeakerInfo()");
        JSONArray speakerArray = cmd.getSpeakers();
        if (speakerArray != null) {
            for (int i = 0; i < speakerArray.length(); i++) {
                try {
                    JSONObject speakerJson = speakerArray.optJSONObject(i);
                    if (speakerJson != null) {
                        CustomLog.d(TAG, "开始构造发言者信息   " + i);
                        SpeakerInfo speaker = new SpeakerInfo("", "");
                        speaker.setAccountId(speakerJson
                                .optString(CmdKey.USER_ID));
                        speaker.setAccountName(speakerJson
                                .optString(CmdKey.USERNAME));
                        // speaker.setSpeakerType(speakerJson.optInt(CmdKey.SPEAKER_TYPE));
                        // TODO 暂时无主讲概念
                        speaker.setSpeakerType(SpeakerInfo.SPEAKER_TYPE_NORMAL);
                        speaker.setScreenShareInfo(speakerJson
                                .optInt(CmdKey.SCREEN_SHARING_INFO));
                        mMeetingInfo.addSpeaker(speaker);
                        // 构造媒体发布信息
                        CustomLog.d(TAG, "开始构造发言者流信息");
                        JSONArray streamArray = speakerJson
                                .optJSONArray(CmdKey.STREAMS);
                        if (streamArray == null) {
                            CustomLog.d(TAG, "该发言者无流信息");
                        } else {
                            for (int j = 0; j < streamArray.length(); j++) {
                                JSONObject streamJson = streamArray
                                        .getJSONObject(j);
                                cmd.setExtraInfo(streamJson
                                        .optString(CmdKey.EXTRAINFO));

                                StreamInfo streamInfo = new StreamInfo();
                                int resourceId = streamJson
                                        .optInt(CmdKey.RESOURCE_ID);

                                int mediaType = SpeakerInfo
                                        .ParseStpMediaTypeToSM(streamJson
                                                .optInt(CmdKey.MEDIA_TYPE));

                                int shareType = Cmd.getShareType(cmd
                                        .getExtraInfo());
                                streamInfo.setShareType(shareType);

                                if (mediaType == MediaType.TYPE_VIDEO) {

                                    if (mMeetingShowType == MEETING_TYPE_AUDIO) {
                                        CustomLog
                                                .d(TAG,
                                                        "initSpeakerInfo，mMeetingShowType==MEETING_TYPE_AUDIO,将摄像头置为关闭状态");
                                        speaker.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
                                    } else if (speaker.getAccountId().equals(
                                            mMeetingInfo.accountId)
                                            && mFirstOpenCamera == false) {
                                        CustomLog
                                                .d(TAG,
                                                        "initSpeakerInfo，mFirstOpenCamera==false,将摄像头置为关闭状态");
                                        speaker.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
                                    } else {
                                        CustomLog.d(TAG,
                                                "解析到视频流的发布信息,将摄像头置为打开状态");
                                        speaker.setCamStatus(SpeakerInfo.CAM_STATUS_OPEN);
                                        speaker.setVideoParam(Cmd
                                                .getVideoParam(cmd
                                                        .getExtraInfo()));
                                    }

                                } else if (mediaType == MediaType.TYPE_AUDIO) {
                                    CustomLog.d(TAG, "解析到音频流的发布信息,将麦克风置为打开状态");
                                    speaker.setMICStatus(SpeakerInfo.MIC_STATUS_ON);
                                } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
                                    speaker.setDocVideoParam(Cmd
                                            .getVideoParam(cmd.getExtraInfo()));
                                }
                                streamInfo.setResourcesId(resourceId);
                                try {
                                    streamInfo.setSyncId(Cmd.getSyncId(cmd
                                            .getExtraInfo()));
                                } catch (Exception e) {
                                    CustomLog
                                            .e(TAG,
                                                    "error! streamJson.optString(CmdKey.SYNC_ID)"
                                                            + streamJson
                                                            .optString(CmdKey.EXTRAINFO));
                                    streamInfo.setSyncId(-1);
                                }

                                streamInfo.setMediaType(mediaType);

                                CustomLog.d(TAG, "添加发言者 " + i + " 的流信息: "
                                        + streamInfo.toString());
                                speaker.addStreamInfo(resourceId, streamInfo);
                            }
                            CustomLog.d(TAG, "构造发言者及流信息结束  " + i + ":"
                                    + speaker.toString());

                        }
                    }
                } catch (JSONException e) {
                    CustomLog.e(TAG, "ButelOpenSDK::initSpeakerInfo() error! "
                            + e.toString());
                    return -1;
                }
            }
            CustomLog.i(TAG, "ButelOpenSDK::initSpeakerInfo() 完成 ");
            mMeetingInfo.LogSpeakersInfo();
        }

        return 0;
    }

    private void subscribeStreams() {
        CustomLog.i(TAG, "ButelOpenSDK::subscribeStreams() 解析所有流信息订阅流");
        for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {

            SpeakerInfo speaker = mMeetingInfo.getSpeakers().get(i);
            String userId = speaker.getAccountId();
            CustomLog.e(TAG, "userId=" + userId + "mMeetingInfo.accountId="
                    + mMeetingInfo.accountId);
            if (userId.equals(mMeetingInfo.accountId)) {
                CustomLog.d(TAG,
                        "ButelOpenSDK::subscribeStreams() 针对自己的流信息，不做处理。");
            } else {
                int syncId = speaker.getSyncId();

                Iterator<Entry<Integer, StreamInfo>> it = speaker
                        .getmStreamsInfo().entrySet().iterator();
                while (it.hasNext()) {
                    Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
                            .next();
                    StreamInfo val = (StreamInfo) entry.getValue();
                    int mediaType = val.getMediaType();
                    CustomLog.i(
                            TAG,
                            "ButelOpenSDK::subscribeStreams() mediaType="
                                    + mediaType + "streamInfo ="
                                    + val.toString());

                    /*
                     * if(mediaType==MediaType.TYPE_VIDEO&&(mMeetingShowType==
                     * MEETING_TYPE_AUDIO)){
                     * CustomLog.d(TAG,"subscribeStreams,是音频会议,不订阅视频流"); }else{
                     */

                    int shareType = val.getShareType();
                    int streamId = 0;
                    if (mWhiteBoardFlag && (shareType != -1 && shareType != 0)) {
                        streamId = mSMSDKControl.subscribe2(userId, mediaType,
                                val.getResourcesId(), syncId, shareType);
                        CustomLog.d(TAG, "subscribe2 streamId=" + streamId);
                    } else {
                        streamId = mSMSDKControl.subscribe(userId, mediaType,
                                val.getResourcesId(), syncId);
                    }

                    // int streamId = mSMSDKControl.subscribe(userId, mediaType,
                    // val.getResourcesId(), syncId);
                    if (streamId > 0) {
                        CustomLog.i(TAG, "ButelOpenSDK 订阅流成功. mediaType: "
                                + mediaType + " | streamId: " + streamId);
                        val.setStreamId(streamId);
                        if (mediaType == MediaType.TYPE_VIDEO) {
                            CustomLog
                                    .d(TAG,
                                            "ButelOpenSDK::subscribeStreams() 存在视频流，摄像头置为打开状态");
                            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_OPEN);

                            if (speaker.getVideoStatus() == SpeakerInfo.VIDEO_OFF) {
                                CustomLog
                                        .d(TAG,
                                                "ButelOpenSDK::subscribeStreams() 视频画面关闭状态，暂停订阅");
                                pauseSubscribe(userId, MediaType.TYPE_VIDEO);
                            } /*
                             * else if (VideoParameter.isUpVGA(
                             * speaker.getVideoResolution()[0],
                             * speaker.getVideoResolution()[1])) { CustomLog
                             * .d(TAG,
                             * "ButelOpenSDK::subscribeStreams() 视频画面高于VGA，暂停订阅"
                             * ); pauseSubscribe(userId, MediaType.TYPE_VIDEO);
                             * speaker.setVideoStatus(SpeakerInfo.VIDEO_OFF,
                             * SpeakerInfo.VIDEO_OFF_REASON_AUTO); }
                             */
                        }// end of mediaType == MediaType.TYPE_VIDEO

                        else if (mediaType == MediaType.TYPE_AUDIO) {
                            CustomLog
                                    .d(TAG,
                                            "ButelOpenSDK::subscribeStreams()  存在音频流发布，将麦克风置为打开状态");
                            speaker.setMICStatus(SpeakerInfo.MIC_STATUS_ON);
                        } // end of mediaType == MediaType.TYPE_AUDIO
                    } else {
                        CustomLog.w(TAG,
                                "ButelOpenSDK::subscribeStreams() 订阅流失败. mediaType: "
                                        + mediaType + " | streamId: "
                                        + streamId);
                    }
                    /* } */

                }
            }
        }
        CustomLog.i(TAG, "ButelOpenSDK::subscribeStreams() 订阅流结束 ");
    }

    private void handleNotifyMasterMeetingModeResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG,
                "ButelOpenSDK::handleNotifyMasterMeetingModeResp() status: "
                        + status);
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        if (status == RespModelStatusCode.SUCCESS) {
            if (mMeetingInfo.mMeetingStyle == MeetingStyleStatus.FREE_MODE) {
                mMeetingInfo.mMeetingStyle = MeetingStyleStatus.MASTER_MODE;
            } else {
                mMeetingInfo.mMeetingStyle = MeetingStyleStatus.FREE_MODE;
            }
            CustomLog.i(TAG, "设置会议模式成功 mCurMeetingInfo.mMeetingStyle: "
                    + mMeetingInfo.mMeetingStyle);
            callbackChangeMeetingModeOperation(true, cmd);
        } else {
            CustomLog.i(TAG, "设置会议模式失败！");
            callbackChangeMeetingModeOperation(false, cmd);
        }
    }

    private void handleMasterSetUserStartSpeakOnMicResp(Cmd cmd) {
        int status = cmd.getStatus();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG,
                "ButelOpenSDK::handleMasterSetUserStartSpeakOnMicResp() 主持人指定发言回应: "
                        + status);
        if (status == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "主持人指定发言成功");
            callbackMasterSetUserStartSpeakOperation(true, cmd);
        } else {
            CustomLog.w(TAG, "主持人指定发言失败");
            callbackMasterSetUserStartSpeakOperation(false, cmd);
        }
    }

    private void handleUserAskForOperateLoudspeakerResp(Cmd cmd) {
        int status = cmd.getStatus();

        CustomLog.i(TAG,
                "ButelOpenSDK::handleUserAskForOperateLoudspeakerResp() 用户申请操作扬声器回应: "
                        + status);
        if (status == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "用户操作扬声器成功");
            if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_OPEN_LOUDSPEAKER) {
                handleAskForOpenLoudSpeakerResp(true, cmd);
            } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_CLOSE_LOUDSPEAKER) {
                handleNotifyListenerListOnNotify(
                        NotifyType.NOTIFY_USER_CLOSE_LOUDSPEAKER, null);
                handleAskForCloseLoudSpeakerResp(true, cmd);
            }
        } else {
            CustomLog.w(TAG, "用户操作扬声器失败");
            if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_OPEN_LOUDSPEAKER) {
                handleAskForOpenLoudSpeakerResp(false, cmd);
            } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_CLOSE_LOUDSPEAKER) {
                handleAskForCloseLoudSpeakerResp(false, cmd);
            }
        }
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
    }

    private void handleMasterSetUserStopSpeakOnMicResp(Cmd cmd) {
        int status = cmd.getStatus();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "handleMasterSetUserStopSpeakOnMicResp 主持人指定停止发言回应："
                + status);
        if (status == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "主持人指定停止发言成功");
            callbackMasterSetUserStopSpeakOperation(true, cmd);
        } else {
            CustomLog.w(TAG, "主持人指定停止发言失败");
            callbackMasterSetUserStopSpeakOperation(false, cmd);
        }
    }

    private void handleMasterKickUserResp(Cmd cmd) {
        int status = cmd.getStatus();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "handleMasterKickUserResp 主持人踢人回应：" + status);
        if (status == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "主持人指定停止发言成功");
            callbackMasterKickUserOperation(true, cmd);
        } else {
            CustomLog.w(TAG, "主持人指定停止发言失败");
            callbackMasterKickUserOperation(false, cmd);
        }
    }

    private void handleMasterTransferMasterIdResp(Cmd cmd) {
        int status = cmd.getStatus();
        int meetingId = cmd.getMeetingId();
        String newMasterId = cmd.getNewMasterId();
        String newMasterName = cmd.getNewMasterName();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "handleMasterTransferMasterIdResp 主持人移交主持权限回应："
                + status);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG,
                    "handleMasterTransferMasterIdResp 会议号不一致 mMeetingInfo.meetingId="
                            + mMeetingInfo.meetingId + " meetingId="
                            + meetingId);
            return;
        }
        if (status == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "handleMasterTransferMasterIdResp 主持人移交主持权限成功");
            CustomLog.d(TAG, "handleMasterTransferMasterIdResp 切换参会角色为普通参会方");
            mMeetingInfo.mMeetingMasterAccountId = newMasterId;
            mMeetingInfo.mMeetingMasterName = newMasterName;
            mMeetingInfo.mCurrentRole = CurrentRoleStatus.COMMON_ROLE;
            callbackMasterTransferMasterIdOperation(true, cmd);
        } else {
            CustomLog.w(TAG, "主持人移交主持权限失败");
            mWantMasterId = "";
            callbackMasterTransferMasterIdOperation(false, cmd);
        }
    }

    private String getAccountNameByAccountId(String accountId) {
        List<SpeakerInfo> speakerInfoList = getSpeakers();
        for (int i = 0; i < speakerInfoList.size(); i++) {
            if (speakerInfoList.get(i).getAccountId().equals(accountId)) {
                return speakerInfoList.get(i).getAccountName();
            }
        }
        return "";
    }

    private void handleServerNoticeMasterIdChanged(Cmd cmd) {
        String newMasterId = cmd.getNewMasterId();
        String newMasterName = cmd.getNewMasterName();
        int meetingId = cmd.getMeetingId();
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::handleServerNoticeMasterIdChanged 收到服务器通知终端移交主持人权限命令  newMasterId: "
                                + newMasterId
                                + " |newMasterName:"
                                + newMasterName);

        if (mMeetingInfo.meetingId != meetingId) {
            CustomLog.e(TAG,
                    "handleServerNoticeMasterIdChanged 会议号不一致 mMeetingInfo.meetingId="
                            + mMeetingInfo.meetingId + " meetingId="
                            + meetingId);
            return;
        }

        mMeetingInfo.mMeetingMasterAccountId = newMasterId;
        mMeetingInfo.mMeetingMasterName = newMasterName;

        if (newMasterId.equals(mMeetingInfo.accountId)) {
            CustomLog.d(TAG, "handleServerNoticeMasterIdChanged 切换参会角色为主持人");
            mMeetingInfo.mCurrentRole = CurrentRoleStatus.MASTER_ROLE;
        } else {
            CustomLog.d(TAG, "handleServerNoticeMasterIdChanged 切换参会角色为普通参会方");
            mMeetingInfo.mCurrentRole = CurrentRoleStatus.COMMON_ROLE;
        }

        handleNotifyListenerListOnNotify(
                NotifyType.SERVER_NOTICE_MASTERID_CHANGED, cmd);

        if (mPollAndPushControl != null && mPollFlag) {
            handleNotifyListenerListOnNotify(
                    NotifyType.SERVER_NOTICE_STOP_POLL_AND_PUSH, "");
        }
    }

    private void handleMasterAskForEndMeetingResp(Cmd cmd) {
        int status = cmd.getStatus();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "handleMasterAskForEndMeetingResp 主持人申请结束会议请求命令回应："
                + status);

        if (status == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "handleMasterAskForEndMeetingResp 主持人申请结束会议成功");
            callbackMasterAskForEndMeetingOperation(true, cmd);
        } else {
            CustomLog.i(TAG, "handleMasterAskForEndMeetingResp 主持人申请结束会议失败");
            callbackMasterAskForEndMeetingOperation(false, cmd);
        }
    }

    private void handleServerNoticeMasterEndMeeting(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::handleServerNoticeMasterEndMeeting 收到服务器通知终端结束会议命令");

        if (mMeetingInfo.meetingId != meetingId) {
            CustomLog.e(TAG,
                    "handleServerNoticeMasterIdChanged 会议号不一致 mMeetingInfo.meetingId="
                            + mMeetingInfo.meetingId + " meetingId="
                            + meetingId);
            return;
        }

        handleNotifyListenerListOnNotify(NotifyType.SERVER_NOTICE_END_MEETING,
                cmd);
    }

    private void handleServerNoticeBeKicked(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String accountId = cmd.getAccountId();
        CustomLog.i(TAG, "handleServerNoticeBeKicked() 服务器通知被踢。 meetingId: "
                + meetingId + " | accountId: " + accountId);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG,
                    "handleServerNoticeBeKicked() error! 会议id不一致，当前会议： "
                            + mMeetingInfo.meetingId);
            return;
        }

        if (!accountId.equals(mMeetingInfo.accountId)) {
            CustomLog.e(TAG,
                    "handleServerNoticeBeKicked() error! 视讯号不一致，当前视讯号： "
                            + mMeetingInfo.accountId);
            return;
        }

        handleNotifyListenerListOnNotify(NotifyType.BE_KICKED, cmd);
    }

    private void handleUserAskForRaiseHandResp(Cmd cmd) {
        int status = cmd.getStatus();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "handleUserAskForRaiseHandResp 举手回应：" + status);
        if (cmd.getStatus() == RespModelStatusCode.SUCCESS) {
            CustomLog.i(TAG, "举手成功");
            callbackAskForRaiseHandOperation(true, cmd);
        } else {
            CustomLog.w(TAG, "举手失败");
            callbackAskForRaiseHandOperation(false, cmd);
        }
    }

    private void handleServerNoticeMeetingModeChange(Cmd cmd) {
        int meetingStyle = cmd.getMeetingStyle();
        CustomLog.d(TAG,
                "ButelOpenSDK::handleServerNoticeMeetingModeChange() 服务器通知模式改变, "
                        + "当前模式: " + mMeetingInfo.mMeetingStyle + " | 目标模式："
                        + meetingStyle);

        if (mMeetingInfo.mMeetingStyle == meetingStyle) {
            CustomLog.w(TAG, "模式相同，不需要改变");
            return;
        }

        mMeetingInfo.mMeetingStyle = cmd.getMeetingStyle();
        handleNotifyListenerListOnNotify(
                NotifyType.SERVER_NOTICE_MEETING_MODE_CHANGE, cmd);
    }

    private void handleServerNoticeUserAskForRaiseUp(Cmd cmd) {
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG,
                "ButelOpenSDK::handleServerNoticeUserAskForRaiseUp 收到服务器转发用户举手命令  userId: "
                        + userId + " |userName:" + userName);

        if (mMeetingInfo.mCurrentRole == CurrentRoleStatus.MASTER_ROLE
                || mAdminFlag) {
            handleNotifyListenerListOnNotify(
                    NotifyType.SERVER_NOTICE_USER_ASK_FOR_RAISE_HAND, cmd);
        } else {
            CustomLog.i(TAG, "当前角色为参会方，不应该收到举手命令");
        }
    }

    private void handleAskForStartLiveResp(Cmd cmd) {
        int status = cmd.getStatus();
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "ButelOpenSDK::handleAskForStartLiveResp 开启直播回应： "
                + status);
        if (status == RespModelStatusCode.SUCCESS) {
            mMeetingInfo.mLiveStatus = LiveStatus.LIVING_NOW;
            callbackAskForStartLiveOperation(true, cmd);
        } else {
            callbackAskForStartLiveOperation(false, cmd);
        }
    }

    private void handleServerNoticeStartLive(Cmd cmd) {
        String liveName = cmd.getLiveName();
        CustomLog.i(TAG,
                "ButelOpenSDK::handleServerNoticeStartLive 服务器通知开启直播  name: "
                        + liveName);
        mMeetingInfo.mLiveStatus = LiveStatus.LIVING_NOW;
        handleNotifyListenerListOnNotify(NotifyType.SERVER_NOTICE_LIVE, cmd);
    }

    private void handleExitMeetingResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG, "ButelOpenSDK::handleExitMeetingResp() 处理退出会议回应命令: "
                + status);
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        if (status == RespModelStatusCode.SUCCESS) {
            callbackExitMeetingOperation(true, cmd);
            exitMeetingRelease();
        } else {
            callbackExitMeetingOperation(false, cmd);
        }
        CustomLog.i(TAG, "处理退出会议回应命令结束");
    }

    private void handleAskForSpeakResp(Cmd cmd) {

        int status = cmd.getStatus();
        CustomLog.i(TAG, "ButelOpenSDK::handleAskForSpeakResp 处理申请发言回应命令: "
                + status);
        if (status == RespModelStatusCode.SUCCESS) {
            mOpenMic = true;
            mFirstOpenCamera = true;
            String str = KeyEventConfig.SPEARING + "_ok_"
                    + mMeetingInfo.accountId + "_申请发言成功";
            KeyEventWrite.write(str);
            // 不处理申请发言成功，等待新发言者产生
            mTimeoutCmdHandler.sendEmptyMessageDelayed(0, 10000);
        } else {
            String str = KeyEventConfig.SPEARING + "_fail_"
                    + mMeetingInfo.accountId + "_申请发言失败";
            KeyEventWrite.write(str);
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            callbackAskForSpeakOperation(false, null);
        }
    }

    private void handleAskForStopSpeakResp(Cmd cmd) {
        int status = cmd.getStatus();

        CustomLog.i(TAG,
                "ButelOpenSDK::handleAskForStopSpeakResp处理申请停止发言回应命令： "
                        + status);
        if (status == RespModelStatusCode.SUCCESS) {
            String str = KeyEventConfig.CANCEL_SPEARING + "_ok_"
                    + mMeetingInfo.accountId + "_申请停止发言成功";
            KeyEventWrite.write(str);
            mTimeoutCmdHandler.sendEmptyMessageDelayed(0, 10000);
        } else {
            String str = KeyEventConfig.CANCEL_SPEARING + "_fail_"
                    + mMeetingInfo.accountId + "_申请停止发言失败";
            KeyEventWrite.write(str);
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            callbackAskForStopSpeakOperation(false, cmd);
        }
    }

    private void handleGetParticipatorsResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG,
                "ButelOpenSDK::handleGetParticipatorsResp() 处理获取参会者列表回应命令: "
                        + status);
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        if (status == RespModelStatusCode.SUCCESS) {
            String str = KeyEventConfig.GET_PARTICIPANTS + "_ok_"
                    + mMeetingInfo.accountId + "_获取参会者列表成功";
            KeyEventWrite.write(str);
            mParticipatorList = cmd.getParticipators();
            CustomLog.i(TAG,
                    "mParticipatorList size = " + mParticipatorList.length());
            // TODO 解析参会人员列表，生成数据结构对象
            handleNotifyListenerListOnNotify(NotifyType.PARTICIPANT_CHANGED,
                    cmd);
        } else {
            String str = KeyEventConfig.GET_PARTICIPANTS + "_fail_"
                    + mMeetingInfo.accountId + "_获取参会者列表失败";
            KeyEventWrite.write(str);
            // handleGetParticipatorsListenerListOnOperationFail(cmd);
        }
        // 无论获取参会人员列表是否成功，均发送加入会议成功。
        callbackJoinMeetingOperation(true, cmd);

        // 加会成功后主动获取会议的主题和会议的字幕
        if (mCaptionAndThemeState) {
            getCacheMeetingCaption();
            getCacheMeetingTheme();
        }
    }

    private void handleJoinMeetingWithExceptionResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG,
                "ButelOpenSDK::handleJoinMeetingWithExceptionResp() 处理异常加入会议回应命令: "
                        + status);
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;

        if (status == RespModelStatusCode.SUCCESS) {

            mMeetingInfo.exceptionRetryTime = 0;
            String str = KeyEventConfig.DEAL_981_EVENT + "_ok_"
                    + mMeetingInfo.accountId + "_处理981成功";
            KeyEventWrite.write(str);
            boolean flag = updateSpeakerInfo(cmd);
            handleNotifyListenerListOnNotify(NotifyType.HANDLE_EXCEPTION_SUC,
                    flag);
            mStatus = ButelOpenSDKStatusCode.JOIN_MEETING_SUC;
            mMeetingInfo.isDealWith981 = false;

        } else {
            String str = KeyEventConfig.DEAL_981_EVENT + "_fail_"
                    + mMeetingInfo.accountId + "_处理981失败";
            KeyEventWrite.write(str);
            handleNotifyListenerListOnNotify(NotifyType.HANDLE_EXCEPTION_FAIL,
                    null);
        }
        CustomLog.i(TAG, "处理异常加入会议回应命令结束");
    }

    private void handleServerNoticeMasterOperateUserLoudspeaker(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String accountId = cmd.getAccountId();
        CustomLog
                .i(TAG,
                        "handleServerNoticeMasterOperateUserLoudspeaker() 服务器通知主持人要求某人操作扬声器。 meetingId: "
                                + meetingId + " | accountId: " + accountId);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG,
                    "handleServerNoticeMasterOperateUserLoudspeaker() error! 会议id不一致，当前会议："
                            + mMeetingInfo.meetingId);
            return;
        }

        handleNotifyListenerListOnNotify(
                NotifyType.MASTER_OPERATE_USER_LOUDSPEAKER, cmd);

    }

    private void handleServerNoticeMasterOperateUserStream(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String accountId = cmd.getAccountId();
        CustomLog.i(TAG,
                "handleServerNoticeMasterOperateUserStream() 服务器通知用户更改媒体状态。 meetingId: "
                        + meetingId + " | accountId: " + accountId);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG,
                    "handleServerNoticeMasterOperateUserStream() error! 会议id不一致，当前会议："
                            + mMeetingInfo.meetingId);
            return;
        }

        // 1.视频 2.音频 3.文档
        if (cmd.getMediaType() == 1 || cmd.getMediaType() == 3) {
            // 1.打开 2.关闭
            if (cmd.getNewStreamOperatorType() == 1) {
                handleNotifyListenerListOnNotify(
                        NotifyType.MASTER_OPEN_USER_CAMERA, null);
            } else if (cmd.getNewStreamOperatorType() == 2) {
                handleNotifyListenerListOnNotify(
                        NotifyType.MASTER_CLOSE_USER_CAMERA, null);
            } else {
                CustomLog
                        .e(TAG,
                                "video getNewStreamOperatorType:"
                                        + String.valueOf(cmd
                                        .getNewStreamOperatorType()));
            }
        } else if (cmd.getMediaType() == 2) {
            // 1.打开 2.关闭
            if (cmd.getNewStreamOperatorType() == 1) {
                handleNotifyListenerListOnNotify(
                        NotifyType.MASTER_OPEN_USER_MIC, null);
            } else if (cmd.getNewStreamOperatorType() == 2) {
                handleNotifyListenerListOnNotify(
                        NotifyType.MASTER_CLOSE_USER_MIC, null);
            } else {
                CustomLog
                        .e(TAG,
                                "audio getNewStreamOperatorType:"
                                        + String.valueOf(cmd
                                        .getNewStreamOperatorType()));
            }
        }

    }

    private void handleServerNoticeUserChangeLoudspeakerState(Cmd cmd) {
        int meetingId = cmd.getMeetingId();
        String accountId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG,
                "handleServerNoticeUserChangeLoudspeakerState() 服务器通知用户更改媒体状态。 meetingId: "
                        + meetingId + " | accountId: " + accountId
                        + " | userName" + userName);
        if (meetingId != mMeetingInfo.meetingId) {
            CustomLog.e(TAG,
                    "handleServerNoticeUserChangeLoudspeakerState() error! 会议id不一致，当前会议："
                            + mMeetingInfo.meetingId);
            return;
        }

        // type: 1. 已打开；2.已关闭
        if (cmd.getType() == 1) {
            handleNotifyListenerListOnNotify(NotifyType.USER_OPEN_LOUNDSPEAKER,
                    cmd);
        } else if (cmd.getType() == 2) {
            handleNotifyListenerListOnNotify(NotifyType.USER_CLOSE_LUNDSPEAKER,
                    cmd);
        }

    }

    /**
     * 更新发言者列表,会议过程中重新加入会议，需要检查发言者、流信息是否有变化
     *
     * @param cmd
     * @return </p>true: 发言者信息有变化 </p> false: 发言者信息无变化
     */
    private boolean updateSpeakerInfo(Cmd cmd) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::updateSpeakerInfo() 更新发言者信息。 更新前  mCurMeetingInfo.mSpeakerInfoList.size = "
                                + mMeetingInfo.getSpeakers().size()
                                + " | "
                                + mMeetingInfo.getSpeakers().toString());

        boolean changed = false;

        // 解析cmd
        JSONArray speakerArray = cmd.getSpeakers();
        ArrayList<String> speakerList = new ArrayList<String>(); // 构造发言者列表
        // 1.遍历新状态发言者，确认这些发言者之前是否存在
        if (speakerArray != null) {
            for (int i = 0; i < speakerArray.length(); i++) {
                try {
                    JSONObject speakerJson = speakerArray.getJSONObject(i);
                    if (speakerJson != null) {
                        String speakerAccountId = speakerJson
                                .optString(CmdKey.USER_ID);
                        String speakerAccountName = speakerJson
                                .optString(CmdKey.USERNAME);
                        CustomLog.d(TAG, "解析第 " + i + " 个发言者信息  accountId: "
                                + speakerAccountId + " name: "
                                + speakerAccountName);

                        int screenShareInfo = speakerJson
                                .optInt(CmdKey.SCREEN_SHARING_INFO);
                        int speakerType = speakerJson
                                .optInt(CmdKey.SPEAKER_TYPE);

                        speakerList.add(speakerAccountId);
                        SpeakerInfo speakerInfo = getSpeakerInfoById(speakerAccountId);

                        int changedCode = -1;// -1：默认； 0：新增；1：变动
                        // 1.1 发言者之前不存在
                        if (speakerInfo == null) {
                            changed = true;
                            CustomLog.d(TAG,
                                    "ButelOpenSDK::updateSpeakerInfo() 新增发言者: "
                                            + speakerAccountId);

                            speakerInfo = new SpeakerInfo("", "");

                            speakerInfo.setAccountId(speakerAccountId);
                            speakerInfo.setAccountName(speakerAccountName);
                            speakerInfo.setSpeakerType(speakerType);
                            speakerInfo.setScreenShareInfo(screenShareInfo);

                            mMeetingInfo.addSpeaker(speakerInfo);

                            // 1.1.1 发言者是自己，需要发布流
                            if (speakerAccountId
                                    .equalsIgnoreCase(mMeetingInfo.accountId)) {
                                CustomLog
                                        .d(TAG,
                                                "ButelOpenSDK::updateSpeakerInfo() 新增发言者是自己，需要发布自己音视频流");
                                publishStremsInfo();
                                // publishSelfStreamsInfoToStp();
                                continue;
                            } else { // 1.1.2 发言者是他人，需要订阅流
                                changedCode = 0;
                                CustomLog
                                        .d(TAG,
                                                "ButelOpenSDK::updateSpeakerInfo() 新增发言者不是自己，稍后解析流进行流订阅");
                            }
                        } // end of speaker == null

                        // 1.2 发言者之前已经存在
                        else {
                            CustomLog.d(TAG,
                                    "ButelOpenSDK::updateSpeakerInfo() 发言者已存在，稍后对比流信息是否有变更: "
                                            + speakerAccountId);
                            // 1.2.1 已存在的发言者是自己，只需要考虑屏幕分享状态
                            // 如果本地是断开状态，即时stp告知为连接状态，依然认为是断开；
                            // 如果本地是连接状态，stp却告知断开状态，认为是断开状态。
                            if (speakerAccountId
                                    .equalsIgnoreCase(mMeetingInfo.accountId)) {
                                if (speakerInfo.getScreenShareStatus() == SpeakerInfo.SCREEN_NORMAL
                                        && screenShareInfo == SpeakerInfo.SCREEN_SHARING) {
                                    CustomLog
                                            .d(TAG,
                                                    "ButelOpenSDK::updateSpeakerInfo() 本地为屏幕分享断开状态，stp告知为连接状态，依然认为是断开");
                                } else if (speakerInfo.getScreenShareStatus() == SpeakerInfo.SCREEN_SHARING
                                        && screenShareInfo == SpeakerInfo.SCREEN_NORMAL) {
                                    CustomLog
                                            .d(TAG,
                                                    "ButelOpenSDK::updateSpeakerInfo() 本地为屏幕分享连接状态，stp告知为断开状态，以stp状态为准");
                                    // 取消发布屏幕分享流
                                    speakerInfo
                                            .setScreenShareInfo(screenShareInfo);
                                    if (mScreenShareControl != null) {
                                        mScreenShareControl
                                                .forceStopSharing(
                                                        ScreenShareControl.STOP_SHARING_BE_GRABED,
                                                        "stp服务器告知断开");
                                    }
                                    mSMSDKControl
                                            .unPublish(speakerInfo
                                                    .getStreamIdByResource(StreamInfo.RESOURCE_TYPE_DOC_VIDEO));
                                    speakerInfo
                                            .removeStreamInfo(StreamInfo.RESOURCE_TYPE_DOC_VIDEO);
                                    speakerInfo
                                            .removeStreamInfo(StreamInfo.RESOURCE_TYPE_DOC_AUDIO);
                                }
                                continue;
                            }
                            changedCode = 1;
                            // 1.2.1 已存在的发言者是其他人
                            if (screenShareInfo != speakerInfo
                                    .getScreenShareStatus()) {
                                speakerInfo.setScreenShareInfo(screenShareInfo);
                                changed = true;
                            }
                        } // end of 1.2 发言者之前已经存在

                        JSONArray streamArray = speakerJson
                                .optJSONArray(CmdKey.STREAMS);

                        if (streamArray == null) {
                            CustomLog
                                    .d(TAG,
                                            "ButelOpenSDK::updateSpeakerInfo() 该发言者无流信息，清除流 ");
                            clearSpeakerSubscribeStrems(speakerInfo);
                            continue;
                        }

                        // 记录之前的摄像头和麦克风状态
                        int camStatus = speakerInfo.getCamStatus();
                        int micStatus = speakerInfo.getMICStatus();

                        speakerInfo.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
                        speakerInfo.setMICStatus(SpeakerInfo.MIC_STATUS_OFF);

                        boolean flag_videoParam_update = false;
                        boolean flag_docVideoParam_update = false;

                        ArrayList<Integer> streamList = new ArrayList<Integer>(); // 记录流信息，对比当前是否有发言者流被清楚

                        for (int j = 0; j < streamArray.length(); j++) {// 逐条流对比
                            JSONObject streamJson = streamArray
                                    .getJSONObject(j);

                            int resourcesId = streamJson
                                    .optInt(CmdKey.RESOURCE_ID);
                            cmd.setExtraInfo(streamJson
                                    .optString(CmdKey.EXTRAINFO));
                            int syncId = Cmd.getSyncId(cmd.getExtraInfo());
                            int mediaType = SpeakerInfo
                                    .ParseStpMediaTypeToSM(streamJson
                                            .optInt(CmdKey.MEDIA_TYPE));

                            streamList.add(resourcesId);

                            if (mediaType == MediaType.TYPE_VIDEO) {
                                CustomLog.d(TAG, "解析到视频流的发布信息,将摄像头置为打开状态");

                                if (mMeetingShowType == MEETING_TYPE_AUDIO) {
                                    CustomLog
                                            .d(TAG,
                                                    "updateSpeakerInfo,mMeetingShowType==MEETING_TYPE_AUDIO,将摄像头置为关闭状态");
                                    speakerInfo
                                            .setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
                                } else if (speakerInfo.getAccountId().equals(
                                        mMeetingInfo.accountId)
                                        && mFirstOpenCamera == false) {
                                    CustomLog
                                            .d(TAG,
                                                    "updateSpeakerInfo,mFirstOpenCamera==false,将摄像头置为关闭状态");
                                    speakerInfo
                                            .setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
                                } else {
                                    speakerInfo
                                            .setCamStatus(SpeakerInfo.CAM_STATUS_OPEN);
                                }

                                if (!speakerInfo.getVideoParam().isParamEqual(
                                        Cmd.getVideoParam(cmd.getExtraInfo()))) {
                                    CustomLog.d(TAG, "视频采集参数发生改变，需要更新");
                                    speakerInfo.setVideoParam(Cmd
                                            .getVideoParam(cmd.getExtraInfo()));
                                    flag_videoParam_update = true;
                                }

                            } else if (mediaType == MediaType.TYPE_AUDIO) {
                                CustomLog.d(TAG, "解析到音频流的发布信息,将麦克风置为打开状态");
                                speakerInfo
                                        .setMICStatus(SpeakerInfo.MIC_STATUS_ON);
                            } else if (mediaType == MediaType.TYPE_DOC_VIDEO) {
                                CustomLog.d(TAG, "解析到文档视频流的发布信息");
                                if (!speakerInfo.getDocVideoParam()
                                        .isParamEqual(
                                                Cmd.getVideoParam(cmd
                                                        .getExtraInfo()))) {
                                    CustomLog.d(TAG, "文档视频采集参数发生改变，需要更新");
                                    speakerInfo.setDocVideoParam(Cmd
                                            .getVideoParam(cmd.getExtraInfo()));
                                    flag_docVideoParam_update = true;
                                }
                            }

                            switch (changedCode) {
                                case 0: // 新增：对于新增发言者，增加所有流
                                    StreamInfo addstreamInfo = new StreamInfo();
                                    addstreamInfo.setResourcesId(resourcesId);
                                    addstreamInfo.setSyncId(syncId);
                                    addstreamInfo.setMediaType(mediaType);

                                    int addstreamId = 0;

                                    if (mediaType == MediaType.TYPE_VIDEO
                                            && (mMeetingShowType == MEETING_TYPE_AUDIO)) {
                                        CustomLog.d(TAG,
                                                "updateSpeakerInfo,是音频会议,不订阅视频流");
                                    } else {
                                        addstreamId = mSMSDKControl.subscribe(
                                                speakerAccountId, mediaType,
                                                resourcesId, syncId);
                                    }

                                    if (addstreamId > 0) {
                                        CustomLog.d(TAG, "新增发言者，流订阅成功,mediaType:"
                                                + mediaType + " streamId: "
                                                + addstreamId);
                                        if (mediaType == MediaType.TYPE_VIDEO) {
                                            if (speakerInfo.getVideoStatus() == SpeakerInfo.VIDEO_OFF) {
                                                CustomLog
                                                        .d(TAG,
                                                                "ButelOpenSDK::updateSpeakerInfo()，视频画面关闭状态，暂停订阅");
                                                pauseSubscribe(speakerAccountId,
                                                        MediaType.TYPE_VIDEO);
                                            }/*
                                             * else if (VideoParameter .isUpVGA(
                                             * speakerInfo .getVideoResolution()[0],
                                             * speakerInfo
                                             * .getVideoResolution()[1])) {
                                             * CustomLog .d(TAG,
                                             * "ButelOpenSDK::updateSpeakerInfo()，视频画面高于VGA，暂停订阅"
                                             * ); pauseSubscribe(speakerAccountId,
                                             * MediaType.TYPE_VIDEO); speakerInfo
                                             * .setVideoStatus(
                                             * SpeakerInfo.VIDEO_OFF,
                                             * SpeakerInfo.VIDEO_OFF_REASON_AUTO); }
                                             */
                                        }// end of MediaType.TYPE_VIDEO
                                    } else {
                                        CustomLog.e(TAG, "新增发言者，流订阅失败,mediaType: "
                                                + mediaType + " streamId: "
                                                + addstreamId);
                                    }

                                    addstreamInfo.setStreamId(addstreamId);
                                    speakerInfo.addStreamInfo(resourcesId,
                                            addstreamInfo);
                                    break;
                                case 1: // 更新：对于已存在的发言者，对比所有流，如果resourcesId或syncId不一致，则取消订阅后重新进行订阅
                                    StreamInfo streamInfo = speakerInfo
                                            .getStreamInfo(resourcesId);

                                    if (streamInfo == null) {
                                        CustomLog.d(TAG,
                                                "ButelOpenSDK::updateSpeakerInfo() 新增了流信息,mediaType: "
                                                        + mediaType);
                                        streamInfo = new StreamInfo();
                                        streamInfo.setResourcesId(resourcesId);
                                        streamInfo.setSyncId(syncId);
                                        streamInfo.setMediaType(mediaType);

                                        if (speakerAccountId != mMeetingInfo.accountId) {
                                            int updatestreamId = mSMSDKControl
                                                    .subscribe(speakerAccountId,
                                                            mediaType, resourcesId,
                                                            syncId);
                                            if (updatestreamId > 0) {
                                                CustomLog.d(TAG, "新增流订阅成功");
                                            } else {
                                                CustomLog.e(TAG, "新增流订阅失败! "
                                                        + updatestreamId);
                                            }
                                            if (mediaType == MediaType.TYPE_VIDEO) {
                                                CustomLog
                                                        .d(TAG, "新增视频流，判断是否需要暂停订阅");
                                                if (speakerInfo.getVideoStatus() == SpeakerInfo.VIDEO_OFF) {
                                                    CustomLog
                                                            .d(TAG,
                                                                    "ButelOpenSDK::updateSpeakerInfo()，视频画面关闭状态，暂停订阅");
                                                    pauseSubscribe(
                                                            speakerAccountId,
                                                            MediaType.TYPE_VIDEO);
                                                } /*
                                                 * else if (VideoParameter .isUpVGA(
                                                 * speakerInfo
                                                 * .getVideoResolution()[0],
                                                 * speakerInfo
                                                 * .getVideoResolution()[1])) {
                                                 * CustomLog .d(TAG,
                                                 * "ButelOpenSDK::updateSpeakerInfo()，视频画面高于VGA，暂停订阅"
                                                 * ); pauseSubscribe(
                                                 * speakerAccountId,
                                                 * MediaType.TYPE_VIDEO);
                                                 * speakerInfo .setVideoStatus(
                                                 * SpeakerInfo.VIDEO_OFF,
                                                 * SpeakerInfo
                                                 * .VIDEO_OFF_REASON_AUTO); }
                                                 */
                                            }// end of MediaType.TYPE_VIDEO
                                            streamInfo.setStreamId(updatestreamId);
                                        }// end of speakerAccountId !=
                                        // mMeetingInfo.accountId

                                        speakerInfo.addStreamInfo(resourcesId,
                                                streamInfo);
                                        changed = true;
                                    } // end of streamInfo == null
                                    else { // start of streamInfo != null
                                        if (resourcesId != streamInfo
                                                .getResourcesId()
                                                || syncId != streamInfo.getSyncId()) {
                                            CustomLog.d(TAG, "");
                                            mSMSDKControl.unSucscribe(streamInfo
                                                    .getStreamId());
                                            streamInfo.setResourcesId(resourcesId);
                                            streamInfo.setSyncId(syncId);
                                            int streamId = mSMSDKControl.subscribe(
                                                    speakerAccountId, mediaType,
                                                    resourcesId, syncId);
                                            if (streamId > 0) {
                                                CustomLog.d(TAG, "订阅成功");
                                                if (speakerInfo.getVideoStatus() == SpeakerInfo.VIDEO_OFF) {
                                                    CustomLog
                                                            .d(TAG,
                                                                    "ButelOpenSDK::updateSpeakerInfo()，视频画面关闭状态，暂停订阅");
                                                    pauseSubscribe(
                                                            speakerAccountId,
                                                            MediaType.TYPE_VIDEO);
                                                } /*
                                                 * else if (VideoParameter .isUpVGA(
                                                 * speakerInfo
                                                 * .getVideoResolution()[0],
                                                 * speakerInfo
                                                 * .getVideoResolution()[1])) {
                                                 * CustomLog .d(TAG,
                                                 * "ButelOpenSDK::updateSpeakerInfo()，视频画面高于VGA，暂停订阅"
                                                 * ); pauseSubscribe(
                                                 * speakerAccountId,
                                                 * MediaType.TYPE_VIDEO);
                                                 * speakerInfo .setVideoStatus(
                                                 * SpeakerInfo.VIDEO_OFF,
                                                 * SpeakerInfo
                                                 * .VIDEO_OFF_REASON_AUTO); }
                                                 */
                                            } else {
                                                CustomLog.e(TAG, "订阅失败! "
                                                        + streamId);
                                            }
                                            streamInfo.setStreamId(streamId);
                                            changed = true;
                                        }// end of 流发生更新
                                        else { // start of 流未发生更新,检查分辨率是否有更新
                                            if (flag_videoParam_update
                                                    || flag_docVideoParam_update) {
                                                CustomLog
                                                        .d(TAG,
                                                                "ButelOpenSDK::updateSpeakerInfo() 发言者视频采集分辨率发生改变，通知UI端 : ");
                                                handleNotifyListenerListOnNotify(
                                                        NotifyType.SPEAKER_VIDEO_PARAM_UPDATE,
                                                        speakerAccountId);
                                            }
                                        }// end of 流未发生更新,检查分辨率是否有更新

                                    }// end of streamInfo != null
                                    // TODO 对比流是否有减少，比如关闭了摄像头、关闭了麦克风
                                    break;
                            }
                        }// end of 逐条流对比查看是否有改动

                        speakerInfo.getmStreamsInfo();

                        Iterator<Entry<Integer, StreamInfo>> it = speakerInfo
                                .getmStreamsInfo().entrySet().iterator();
                        ArrayList<Integer> matchList = new ArrayList<Integer>();
                        while (it.hasNext()) {
                            Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
                                    .next();
                            int key = (Integer) entry.getKey();
                            boolean match = false;
                            for (int resourceId : streamList) {
                                if (resourceId == key) {
                                    match = true;
                                    break;
                                }
                            }
                            if (!match) {
                                CustomLog.d(TAG,
                                        "ButelOpenSDK::updateSpeakerInfo 有流被清除 ： "
                                                + key);
                                matchList.add(key);
                            }
                        }

                        for (int resourceId : matchList) {
                            mSMSDKControl.unSucscribe(speakerInfo
                                    .getStreamIdByResource(resourceId));

                            if (speakerInfo.getStreamInfo(resourceId)
                                    .getMediaType() == MediaType.TYPE_VIDEO) {
                                CustomLog
                                        .d(TAG,
                                                "ButelOpenSDK::updateSpeakerInfo() 取消发布视频流，将摄像头置为关闭状态");
                                speakerInfo
                                        .setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
                            } else if (speakerInfo.getStreamInfo(resourceId)
                                    .getMediaType() == MediaType.TYPE_AUDIO) {
                                CustomLog
                                        .d(TAG,
                                                "ButelOpenSDK::updateSpeakerInfo() 取消发布音频流，将麦克风置为关闭状态");
                                speakerInfo
                                        .setMICStatus(SpeakerInfo.MIC_STATUS_OFF);
                            }

                            speakerInfo.removeStreamInfo(resourceId);
                            handleNotifyListenerListOnNotify(
                                    NotifyType.SERVER_NOTICE_STREAM_UNPUBLISH,
                                    cmd);
                        }

                        if (camStatus != speakerInfo.getCamStatus()) {
                            CustomLog
                                    .d(TAG,
                                            "ButelOpenSDK::updateSpeakerInfo 该发言者摄像头状态发生变化");
                        }
                        if (micStatus != speakerInfo.getMICStatus()) {
                            CustomLog
                                    .d(TAG,
                                            "ButelOpenSDK::updateSpeakerInfo 该发言者麦克风状态发生变化");
                        }

                    }// end of speakerJson != null
                } catch (JSONException e) {
                    CustomLog.e(
                            TAG,
                            "ButelOpenSDK::updateSpeakerInfo() error! "
                                    + e.toString());
                }
            }// end of 解析获取到的发言者信息
        }// end of speakerArray != null

        // 3.解析是否有发言者删除
        for (int i = 0; i < getSpeakers().size(); i++) {
            SpeakerInfo info = getSpeakers().get(i);
            boolean matching = false;
            for (int j = 0; j < speakerList.size(); j++) {
                if (info.getAccountId().equalsIgnoreCase(speakerList.get(j))) {
                    matching = true;
                    break;
                }
            }

            if (!matching) {
                // 3.1 自己被删除，清除所有发布流
                if (info.getAccountId()
                        .equalsIgnoreCase(mMeetingInfo.accountId)) {
                    CustomLog.d(TAG,
                            "ButelOpenSDK::updateSpeakerInfo() 解析出自己被取消发言");
                    clearSpeakerPublishStreams();
                } else { // 3.2 其他人被删除，清除所有订阅流
                    CustomLog.d(TAG,
                            "ButelOpenSDK::updateSpeakerInfo() 解析出有发言者被取消发言: "
                                    + info.getAccountId());
                    clearSpeakerSubscribeStrems(info);
                }
                mMeetingInfo.removeSpeaker(info.getAccountId());

                changed = true;
            }
        }// end of 3.解析是否有发言者删除

        CustomLog
                .i(TAG,
                        "ButelOpenSDK::updateSpeakerInfo() 更新发言者信息。 更新后 mCurMeetingInfo.mSpeakerInfoList.size="
                                + mMeetingInfo.getSpeakers().size()
                                + mMeetingInfo.getSpeakers().toString());
        return changed;
    }

    // 清除发言者的所有订阅流信息
    private void clearSpeakerSubscribeStrems(SpeakerInfo speaker) {
        CustomLog.d(TAG,
                "ButelOpenSDK::clearSpeakerSubscribeStrems() 清理发言者的订阅信息 ： "
                        + speaker.getAccountId());
        List<Integer> removeResList = new ArrayList<Integer>();
        Iterator<Entry<Integer, StreamInfo>> it = speaker.getmStreamsInfo()
                .entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
                    .next();
            int key = (Integer) entry.getKey();
            StreamInfo val = (StreamInfo) entry.getValue();
            CustomLog.i(TAG, "key=" + key + "val=" + val.toString());
            int ret = mSMSDKControl.unSucscribe(val.getStreamId());
            if (ret == 0) {
                // val.setStreamId(-1);,stopRemoteVIdeo需要用到streamId，不能置为-1
                removeResList.add(key);
                CustomLog.i(TAG, "ButelOpenSDK 取消订阅流成功. resourceId: " + key
                        + " | mediaType: " + val.getMediaType());
            } else {
                CustomLog.w(TAG, "ButelOpenSDK 取消订阅流失败. resourceId: " + key
                        + " | mediaType: " + val.getMediaType() + " | result: "
                        + ret);
            }
        }

        for (int res : removeResList) {
            speaker.removeStreamInfo(res);
        }
        CustomLog.d(TAG,
                "ButelOpenSDK::clearSpeakerSubscribeStrems() 清理发言者的订阅信息结束 "
                        + speaker.toString());

    }

    private void clearSpeakerPublishStreams() {
        SpeakerInfo speaker = mMeetingInfo
                .getSpeakerByAccountId(mMeetingInfo.accountId);

        if (speaker == null) {
            CustomLog
                    .d(TAG,
                            "ButelOpenSDK::clearSpeakerSubscribeStrems() 清理发言者的发布信息, 自己不是发言者，不用清理 ");
            return;
        }
        CustomLog.d(TAG,
                "ButelOpenSDK::clearSpeakerSubscribeStrems() 清理发言者的发布信息 ： "
                        + speaker.getAccountId());
        Iterator<Entry<Integer, StreamInfo>> it = mMeetingInfo
                .getSpeakerByAccountId(mMeetingInfo.accountId)
                .getmStreamsInfo().entrySet().iterator();

        while (it.hasNext()) {
            Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
                    .next();
            int key = (Integer) entry.getKey();
            StreamInfo val = (StreamInfo) entry.getValue();
            CustomLog.i(TAG, "key=" + key + "val=" + val.toString());
            int result = mSMSDKControl.unPublish(val.getStreamId());
            if (result == 0) {
                CustomLog.i(TAG, "ButelOpenSDK 取消发布流成功. resourceId: " + key
                        + " |mediaType: " + val.getMediaType());
            } else {
                CustomLog.w(TAG, "ButelOpenSDK 取消发布流失败. resourceId: " + key
                        + " |mediaType: " + val.getMediaType() + " | result: "
                        + result);
            }
        }
    }

    private void handlePublishOrUnpublishWhiteBoardResp(boolean success, Cmd cmd) {
        int operatorType = cmd.getStreamOperatorType();
        CustomLog.d(TAG,
                "ButelOpenSDK::handlePublishOrUnpublishWhiteBoardResp() ");
        if (success && operatorType == Cmd.STREAM_OPERATOR_TYPE_PUBLISH) {
            CustomLog
                    .d(TAG,
                            "ButelOpenSDK::handlePublishOrUnpublishWhiteBoardResp() 解析到为发布白板流成功回应");

            // startDraw();

            // publishStremsInfo();
            // handleNotifyListenerListOnNotify(NotifyType.OPEN_CAMERA, null);
            // handleNotifyListenerListOnNotify(NotifyType.OPEN_MIC,null);
        }
    }

    private void handlePublishOrUnpublishResp(boolean success, Cmd cmd) {
        int operatorType = cmd.getStreamOperatorType();
        CustomLog.d(TAG, "ButelOpenSDK::handlePublishOrUnpublishResp() ");
        if (success && operatorType == Cmd.STREAM_OPERATOR_TYPE_PUBLISH) {
            CustomLog.d(TAG,
                    "ButelOpenSDK::handlePublishOrUnpublishResp() 解析到为发布流成功回应");
            // publishStremsInfo();
            // 解析results
            JSONArray resultsArray = cmd.getResults();
            if (resultsArray != null) {
                for (int i = 0; i < resultsArray.length(); i++) {
                    JSONObject object = resultsArray.optJSONObject(i);
                    if (object != null) {
                        String resourceId = object.optString("resourceId");
                        int status = object.optInt("status");
                        if (status == 0) {
                            if (Integer.parseInt(resourceId) == StreamInfo.RESOURCE_TYPE_AUDIO) {
                                handleNotifyListenerListOnNotify(
                                        NotifyType.OPEN_MIC, null);
                            } else if (Integer.parseInt(resourceId) == StreamInfo.RESOURCE_TYPE_VIDEO) {
                                handleNotifyListenerListOnNotify(
                                        NotifyType.OPEN_CAMERA, null);
                            }
                        }
                    }
                }
            }
        } else if (!success && operatorType == Cmd.STREAM_OPERATOR_TYPE_PUBLISH) {
            CustomLog
                    .w(TAG,
                            "ButelOpenSDK::handlePublishOrUnpublishResp() 发布流信息失败，将摄像头、麦克风置为关闭状态");
            // 关闭摄像头
            mSMSDKControl.closeCamera();
            int result = mSMSDKControl.releaseCamera();
            if (result != 0) {
                CustomLog.e(TAG, "关闭摄像头失败,result = " + result);
            }

            SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
            if (speaker == null) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::CloseCamera error! 当前未发言，无法关闭摄像头");
                return;
            }

            if (speaker.getCamStatus() == SpeakerInfo.CAM_STATUS_CLOSE) {
                CustomLog.e(TAG, "ButelOpenSDK::CloseCamera error! 当前已是关闭状态。");
            }

            int videoStreamId = speaker
                    .getStreamIdByResource(StreamInfo.RESOURCE_TYPE_VIDEO);
            CustomLog.d(TAG, "ButelOpenSDK::CloseCamera 停止发布的streamId: "
                    + videoStreamId);

            int ret = mSMSDKControl.unPublish(videoStreamId);
            if (ret != 0) {
                CustomLog.e(TAG, "ButelOpenSDK::CloseCamera 取消发布视频流失败! ret: "
                        + ret);
            }

            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
            speaker.removeStreamInfo(StreamInfo.RESOURCE_TYPE_VIDEO);

            // 关闭麦克风
            if (speaker.getMICStatus() == SpeakerInfo.MIC_STATUS_OFF) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::openMicrophone error! 当前已是关闭状态。");
            }

            int audioStreamId = speaker
                    .getStreamIdByResource(StreamInfo.RESOURCE_TYPE_AUDIO);
            CustomLog.d(TAG, "ButelOpenSDK::closeMicrophone 停止发布的streamId: "
                    + audioStreamId);

            int retAudio = mSMSDKControl.unPublish(audioStreamId);
            if (retAudio != 0) {
                CustomLog.e(TAG,
                        "ButelOpenSDK::closeMicrophone 取消发布音频流失败! ret: " + ret);
            }

            speaker.setMICStatus(SpeakerInfo.MIC_STATUS_OFF);
            speaker.removeStreamInfo(StreamInfo.RESOURCE_TYPE_AUDIO);

            handleNotifyListenerListOnNotify(NotifyType.CLOSE_CAMERA, null);
            handleNotifyListenerListOnNotify(NotifyType.CLOSE_MIC, null);
        }
    }

    // 发布自己的音视频流信息
    private void publishStremsInfo() {
        CustomLog.e(TAG, "ButelOpenSDK::publishStremsInfo() 发布自己作为发言者的流信息");
        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);

        if (speaker == null) {
            CustomLog.e(TAG, "ButelOpenSDK::publishStremsInfo() 自己未发言");
        }

        speaker.setVideoParam(mSMSDKControl.getVideoParam());

        // 向 SM_SDK 发布视频流、音频流信息
        int syncId = speaker.getSyncId();

        if (mMeetingShowType == MEETING_TYPE_AUDIO) {
            CustomLog
                    .d(TAG,
                            "publishStremsInfo,mMeetingShowType==MEETING_TYPE_AUDIO,不发布视频流");
            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
        } else if (!mFirstOpenCamera || !mOuterCameraState) {
            CustomLog
                    .d(TAG, "publishStremsInfo,mFirstOpenCamera==false,不发布视频流");
            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
        } else {
            CustomLog
                    .d(TAG,
                            "mMeetingShowType==MEETING_TYPE_VIDEO,mFirstOpenCamera==true,发布视频流");
            StreamInfo videoStream = new StreamInfo(-1,
                    StreamInfo.RESOURCE_TYPE_VIDEO, syncId,
                    MediaType.TYPE_VIDEO);
            int videoStreamId = mSMSDKControl.publishCameraVideo(
                    StreamInfo.RESOURCE_TYPE_VIDEO, syncId);

            videoStream.setStreamId(videoStreamId);
            CustomLog.i(TAG, "发布视频流  streamId: " + videoStreamId);

            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_OPEN);
            speaker.addStreamInfo(StreamInfo.RESOURCE_TYPE_VIDEO, videoStream);
        }

        int audioStreamId = 0;
        if (!mOpenMic || !mOuterMicState) {
            CustomLog.d(TAG, "publishStremsInfo,mOpenMic==false,不发布音频流");
            speaker.setMICStatus(SpeakerInfo.MIC_STATUS_OFF);
        } else {
            StreamInfo audioStream = new StreamInfo(-1,
                    StreamInfo.RESOURCE_TYPE_AUDIO, syncId,
                    MediaType.TYPE_AUDIO);
            audioStreamId = mSMSDKControl.publishMicAudio(
                    StreamInfo.RESOURCE_TYPE_AUDIO, syncId);
            if (audioStreamId < 0) {
                CustomLog.e(TAG, "发布音频流失败  streamId： " + audioStreamId);
            } else {
                audioStream.setStreamId(audioStreamId);
                CustomLog.i(TAG, "发布音频流  streamId: " + audioStreamId);
                speaker.setMICStatus(SpeakerInfo.MIC_STATUS_ON);
                speaker.addStreamInfo(StreamInfo.RESOURCE_TYPE_AUDIO,
                        audioStream);
            }
        }

        JSONArray jsonArray = new JSONArray();
        JSONObject videoStreamJson = new JSONObject();
        JSONObject audeoStreamJson = new JSONObject();
        try {
            if (mMeetingShowType == MEETING_TYPE_AUDIO
                    || mFirstOpenCamera == false || mOuterCameraState == false) {
                CustomLog
                        .d(TAG,
                                "mMeetingShowType==MEETING_TYPE_AUDIO,不添加videoStreamJson");
            } else {
                CustomLog
                        .d(TAG,
                                "mMeetingShowType==MEETING_MODE_VIDEO，添加videoStreamJson");
                videoStreamJson.put(CmdKey.RESOURCE_ID,
                        String.valueOf(StreamInfo.RESOURCE_TYPE_VIDEO));
                videoStreamJson
                        .put(CmdKey.MEDIA_TYPE, SpeakerInfo
                                .ParseSmMediaTypeToStp(MediaType.TYPE_VIDEO));
                videoStreamJson.put(
                        CmdKey.EXTRAINFO,
                        Cmd.createExtraInfo(syncId,
                                speaker.getVideoResolution()[0],
                                speaker.getVideoResolution()[1]));

                jsonArray.put(videoStreamJson);
            }

            if (mOpenMic && mOuterMicState && audioStreamId > 0) {
                audeoStreamJson.put(CmdKey.RESOURCE_ID,
                        String.valueOf(StreamInfo.RESOURCE_TYPE_AUDIO));
                audeoStreamJson
                        .put(CmdKey.MEDIA_TYPE, SpeakerInfo
                                .ParseSmMediaTypeToStp(MediaType.TYPE_AUDIO));
                audeoStreamJson.put(CmdKey.EXTRAINFO,
                        Cmd.createExtraInfo(syncId, -1, -1));

                jsonArray.put(audeoStreamJson);
            }

            CustomLog.d(TAG, "jsonArray=" + jsonArray.toString());

            if (jsonArray.length() > 0) {
                int result = mMeetingControl.askPublishOrUnPublishStreams(
                        mToken, Cmd.STREAM_OPERATOR_TYPE_PUBLISH, jsonArray);
                if (result != 0) {
                    CustomLog.w(TAG, "向 STP 发布流失败 result: " + result);
                } else {
                    mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.PUBLISH_OR_UNPUBLISH_STREAMS;
                }
            } else {
                CustomLog.d(TAG, "没有流信息，不进行发布操作。");
            }
        } catch (JSONException e) {
            CustomLog.e(TAG, "json 构造出错 : " + e.getMessage());
        }

        if (audioStreamId == SMSDKControl.DEVICE_AUDIO_INIT_FAILED) {
            CustomLog.d(TAG, "ButelOpenSDK::publishStremsInfo() 通知UI音频采集失败");
            handleNotifyListenerListOnNotify(
                    NotifyType.AUDIO_IN_DEVICE_INIT_FAILED, null);
        }
    }

    private void handleGiveMicResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG, "ButelOpenSDK::handleGiveMicResp() 处理传麦回应命令: "
                + status);

        if (status == RespModelStatusCode.SUCCESS) {
            // handleGiveMicListenerListOnOperationSuc(cmd);
            String str = KeyEventConfig.GIVE_MAIC + "_ok_"
                    + mMeetingInfo.accountId + "_传麦成功";
            KeyEventWrite.write(str);
            mTimeoutCmdHandler.sendEmptyMessageDelayed(0, 10000);
        } else {
            String str = KeyEventConfig.GIVE_MAIC + "_fail_"
                    + mMeetingInfo.accountId + "_传麦失败";
            KeyEventWrite.write(str);
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            callbackGiveMicOperation(false, cmd);
        }
        CustomLog.i(TAG, "处理传麦回应命令结束");
    }

    private void handleLockMeetingResp(Cmd cmd) {
        int status = cmd.getStatus();
        CustomLog.i(TAG, "ButelOpenSDK::handleLockMeetingResp() 处理加锁会议回应命令: "
                + status);
        if (status == RespModelStatusCode.SUCCESS) {
            String str = KeyEventConfig.LOCK_UNLOCK_IN_MEETINGROOM + "_ok_"
                    + mMeetingInfo.accountId + "_加|解锁成功";
            KeyEventWrite.write(str);
            mTimeoutCmdHandler.sendEmptyMessageDelayed(0, 10000);
        } else {
            String str = KeyEventConfig.LOCK_UNLOCK_IN_MEETINGROOM + "_fail_"
                    + mMeetingInfo.accountId + "_加|解锁失败";
            KeyEventWrite.write(str);
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            mMeetingInfo.intentLockInfo = 1 - mMeetingInfo.intentLockInfo;
            callbackLockMeetingOperation(false, cmd);

        }
        CustomLog.i(TAG, "处理加锁会议回应命令结束");
    }

    private void handleBeGrabed(Cmd cmd) {
        CustomLog.i(TAG, "ButelOpenSDK::handleBeGrabed() 处理账户被抢占命令");
        handleNotifyListenerListOnNotify(NotifyType.BE_GRABED, null);
    }

    private void handleException(Cmd cmd) {
        CustomLog.i(TAG, "ButelOpenSDK::handleException() 处理981异常命令");
        mMeetingInfo.isDealWith981 = true;
        mStatus = ButelOpenSDKStatusCode.EXCEPTION;
        mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        CustomLog.i(TAG, "通知上层出现981，开始重新加入会议");
        handleNotifyListenerListOnNotify(NotifyType.EXCEPTION, cmd);
        handleNotifyListenerListOnNotify(NotifyType.SERVICE_NOTICE_981, cmd);
        reJoinMeeting(mToken);
        CustomLog.i(TAG, "处理981异常命令结束");
    }

    private void handleServiceNotifyLockInfo(Cmd cmd) {
        int lockInfo = cmd.getLockInfo();
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::handleServiceNotifyLockInfo() 处理通知更新状态命令  mMeetingControlOperation: "
                                + mMeetingControlOperation
                                + " |lockInfo: "
                                + lockInfo);

        if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.LOCK_MEETING) {
            if (cmd.getLockInfo() == mMeetingInfo.intentLockInfo) {
                mTimeoutCmdHandler.removeMessages(0);
                callbackLockMeetingOperation(true, cmd);
                mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            } else {
                mTimeoutCmdHandler.removeMessages(0);
                callbackLockMeetingOperation(false, cmd);
                mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            }
        }

        handleNotifyListenerListOnNotify(NotifyType.NOTIFY_LOCK_INFO, cmd);
        CustomLog.i(TAG, "处理通知更新状态命令结束");
    }

    private void handleInvalidCmd(Cmd cmd) {
        CustomLog.i(TAG, "处理非法命令 cmdId: " + cmd.getCmdId());
    }

    // 该命令为服务器通知普通终端，发言者本身不会收到
    private void handleSpeakerOnLine(Cmd cmd) {
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG, "处理新发言者产生命令，我的ID = " + mMeetingInfo.accountId
                + " cmd的ID= " + userId);
        if (mMeetingInfo.accountId.equalsIgnoreCase(userId)) {
            CustomLog.e(TAG, "error! 根据接口协议自己不应该收到新发言者是自己的命令！");
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(userId);
        if (speaker != null) {
            CustomLog.w(TAG, "error! 该发言者已存在于发言列表，不应该再次收到其作为新发言者产生的命令！");
            return;
        }

        // 更新 SpeakerInfo List
        speaker = new SpeakerInfo();
        // 构造身份等信息
        speaker.setAccountId(userId);
        speaker.setAccountName(userName);
        // speaker.setSpeakerType(cmd.getSpeakerType());
        speaker.setSpeakerType(SpeakerInfo.SPEAKER_TYPE_NORMAL);
        // 流相关信息会有单独命令发布出来。
        mMeetingInfo.addSpeaker(speaker);

        handleNotifyListenerListOnNotify(NotifyType.SPEAKER_ON_LINE, cmd);

        CustomLog.i(TAG, "处理新发言者产生命令结束");

    }

    // 该命令为服务器通知普通终端
    private void handleSpeakerOffLine(Cmd cmd) {
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG, "处理新发言者失效命令，我的ID = " + mMeetingInfo.accountId
                + " cmd的ID= " + userId + " | userName: " + userName);

        if (mMeetingInfo.accountId.equalsIgnoreCase(userId)) {
            CustomLog.w(TAG, "error! 根据接口协议自己不应该收到发言者失效是自己的命令！");
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(userId);
        if (speaker == null) {
            CustomLog.w(TAG, "error! 该发言者不存在于发言列表，不应该再次收到其作为发言者失效的命令！");
            return;
        }

//        if (userId.equals(mMeetingInfo.mMeetingMasterAccountId)) {
//            CustomLog.d(TAG, "主持人退会，停止推流");
//            if (mPollAndPushControl != null) {
//                // mPollAndPushControl.stopPollAndPush();
//                handleNotifyListenerListOnNotify(
//                        NotifyType.SERVER_NOTICE_STOP_POLL_AND_PUSH, "");
//            }
//        }

        handleNotifyListenerListOnNotify(NotifyType.SPEAKER_OFF_LINE, cmd);

        CustomLog.i(TAG, "处理发言者被取消命令结束");
    }

    // 该命令为服务器发给发言者本身
    private void handleStartSpeak(Cmd cmd) {
        String accountId = cmd.getAccountId();
        CustomLog.i(TAG, "处理开始发言命令，我的ID = " + mMeetingInfo.accountId
                + " cmd的ID= " + accountId);
        if (!mMeetingInfo.accountId.equalsIgnoreCase(accountId)) {
            CustomLog.w(TAG, "error! 开始发言者不是自己！");
            return;
        }

        if (mPollAndPushControl != null && mPollFlag) {
            // mPollAndPushControl.stopPollAndPush();
            handleNotifyListenerListOnNotify(
                    NotifyType.SERVER_NOTICE_STOP_POLL_AND_PUSH, "");
        }

        if (mScreenShareControl != null) {
            mScreenShareControl.setSpeakState(true);
        }

        if (mSMSDKControl != null) {
            if (mMeetingShowType == MEETING_TYPE_AUDIO
                    || mFirstOpenCamera == false || mOuterCameraState == false) {
                mSMSDKControl.setSpeakStatus(true, false);
            } else {
                mSMSDKControl.setSpeakStatus(true, true);
            }

        }

        if (cmd.getCmdFlag() == 2) {
            CustomLog.d(TAG, "主持人指定我发言，将摄像头，麦克风置为打开状态!");
            mOpenMic = true;
            mFirstOpenCamera = true;
            mSMSDKControl.setSpeakStatus(true, true);
        }

        SpeakerInfo speaker = getSpeakerInfoById(accountId);
        if (speaker != null) {
            CustomLog.e(TAG, "error！ 已存在于发言者列表中。");
            return;
        }

        if (null != accountId && null != mMeetingInfo.accountId
                && accountId.equals(mMeetingInfo.accountId)) {
            mMeetingInfo.setSpeakStatus(true);
        }

        speaker = new SpeakerInfo();

        // 更新 SpeakerInfo List
        speaker.setAccountId(accountId);
        speaker.setAccountName(mMeetingInfo.accountName);
        // speaker.setSpeakerType(cmd.getSpeakerType());
        speaker.setSpeakerType(SpeakerInfo.SPEAKER_TYPE_NORMAL);

        mMeetingInfo.addSpeaker(speaker);

        if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_SPEAK) {
            mTimeoutCmdHandler.removeMessages(0);
            callbackAskForSpeakOperation(true, null);
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
        }

        publishStremsInfo();

        handleNotifyListenerListOnNotify(NotifyType.START_SPEAK, cmd);
    }

    // 该命令为服务器发给发言者本身
    private void handleStopSpeak(Cmd cmd) {
        String accountId = cmd.getAccountId();
        CustomLog.i(TAG, "处理停止发言命令，我的ID = " + mMeetingInfo.accountId
                + " cmd的ID= " + accountId);

        if (!mMeetingInfo.accountId.equalsIgnoreCase(accountId)) {
            CustomLog.w(TAG, "error! 停止发言者不是自己！");
            return;
        }

        SpeakerInfo speaker = getSpeakerInfoById(accountId);
        if (speaker == null) {
            CustomLog.w(TAG, "error！ 发言者列表中不存在自己。");
            return;
        }
        if (mScreenShareControl != null) {
            mScreenShareControl.setSpeakState(false);
            mScreenShareControl.forceStopSharing(
                    ScreenShareControl.STOP_SHARING_STOP_SPEAK, "停止发言");
        }

        if (mSMSDKControl != null) {
            mSMSDKControl.setSpeakStatus(false, false);
        }

        if (null != accountId && null != mMeetingInfo.accountId
                && accountId.equals(mMeetingInfo.accountId)) {
            mMeetingInfo.setSpeakStatus(false);
        }

        // 向 SM_SDK 停止发布所有相关流, STP 不用另行通知
        Iterator<Entry<Integer, StreamInfo>> it = speaker.getmStreamsInfo()
                .entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry<Integer, StreamInfo> entry = (Map.Entry<Integer, StreamInfo>) it
                    .next();
            int key = (Integer) entry.getKey();
            StreamInfo val = (StreamInfo) entry.getValue();
            CustomLog.i(TAG, "key=" + key + "val=" + val.toString());
            int result = mSMSDKControl.unPublish(val.getStreamId());
            if (result == 0) {
                CustomLog.i(TAG, "ButelOpenSDK 取消发布流成功. resourceId: " + key
                        + " mediaType: " + val.getMediaType());
            } else {
                CustomLog.w(TAG, "ButelOpenSDK 取消发布流失败. resourceId: " + key
                        + " mediaType: " + val.getMediaType() + " | result: "
                        + result);
            }
        }

        handleNotifyListenerListOnNotify(NotifyType.STOP_SPEAK, cmd);

        if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.ASK_FOR_STOP_SPEAK) {
            mTimeoutCmdHandler.removeMessages(0);
            callbackAskForStopSpeakOperation(true, cmd);
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            return;
        } else if (mMeetingControlOperation == ButelOpenSDKMeetingControlOperationCode.GIVE_MIC) {
            mTimeoutCmdHandler.removeMessages(0);
            callbackGiveMicOperation(true, cmd);
            mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.NULL;
            return;
        }
        CustomLog.i(TAG, "处理停止发言命令");
    }

    private void handlePersonJoinMeeting(Cmd cmd) {
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG,
                "ButelOpenSDK::handlePersonJoinMeeting() 处理参会者加入会议命令 userId: "
                        + userId + " | userName: " + userName);

        // 更新主持人名称
        if (userId.equalsIgnoreCase(mMeetingInfo.mMeetingMasterAccountId)
                && !userName.equalsIgnoreCase(mMeetingInfo.mMeetingMasterName)) {
            CustomLog.i(TAG,
                    "ButelOpenSDK::handlePersonJoinMeeting() 主持人名称发生改变，更新主持人名称为： "
                            + userName);
            mMeetingInfo.mMeetingMasterName = userName;
        }
        // 通知图元管理模块清理老数据，防止之前异常退会造成数据未及时清理
        if (mGeoMetryControl != null) {
            mGeoMetryControl.clearData(Integer.parseInt(userId));
        }
        // TODO 更新参会人员列表
        handleNotifyListenerListOnNotify(NotifyType.PERSON_JOIN_MEETING, cmd);
        CustomLog.i(TAG, "处理参会者加入会议命令结束");
    }

    private void handlePersonExitMeeting(Cmd cmd) {
        String userId = cmd.getAccountId();
        String userName = cmd.getUserName();
        CustomLog.i(TAG,
                "ButelOpenSDK::handlePersonExitMeeting() 处理参会者退出会议命令 userId: "
                        + userId + " | userName: " + userName);
        // 有人退会通知图元管理模块
        if (mGeoMetryControl != null) {
            mGeoMetryControl.leaveMeeting(Integer.parseInt(userId));
        }

        if (userId.equals(mMeetingInfo.mMeetingMasterAccountId)) {
            if (mPollAndPushControl != null && mPollFlag) {
                CustomLog.d(TAG, "主持人退会，停止推流");
                // mPollAndPushControl.stopPollAndPush();
                handleNotifyListenerListOnNotify(
                        NotifyType.SERVER_NOTICE_STOP_POLL_AND_PUSH, "");
            }
        }

        // TODO 更新参会人员列表
        handleNotifyListenerListOnNotify(NotifyType.PERSON_EXIT_MEETING, cmd);
        CustomLog.i(TAG, "处理参会者退出会议命令结束");
    }

    /******** MediaPlaySDK操作处理相关 ********/
    private void dispatchSMSDKResp(int status, Object obj) {
        // CustomLog.i(TAG, "处理MediaPlaySDK返回的命令");
        switch (status) {
            case SMSDKControl.MediaPlaySDKReturnCode.INIT_SUC:
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.INIT_FAILURE:
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.MIC_1_SPEAK_SUC:
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.RELEASE_SUC:
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.RELEASE_FAILURE:
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.CHANGE_TO_QVGA:
                handleNotifyListenerListOnNotify(NotifyType.CHANGE_TO_QVGA, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.CHANGE_TO_VGA:
                handleNotifyListenerListOnNotify(NotifyType.CHANGE_TO_VGA, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.MEDIAPLAY_REMOTE_NETWORK_POOR:
                handleNotifyListenerListOnNotify(
                        NotifyType.MEDIAPLAY_REMOTE_NETWORK_POOR, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.MEDIAPLAY_LOCAL_NETWORK_POOR:
                handleNotifyListenerListOnNotify(
                        NotifyType.MEDIAPLAY_LOCAL_NETWORK_POOR, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.MEDIAPLAY_ALL_NETWORK_POOR:
                handleNotifyListenerListOnNotify(
                        NotifyType.MEDIAPLAY_ALL_NETWORK_POOR, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.MEDIAPLAY_GET_NET_INFO:
                CustomLog.i(TAG, "MEDIAPLAY_GET_NET_INFO");
                setRssiNetInfo();
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.MEDIAPLAY_NOTIFY_AUDIO_TYPE:
                CustomLog.i(TAG, "网络差，自动提示切换到语音模式");
                handleNotifyListenerListOnNotify(NotifyType.AUTO_SWITCH_TO_AUDIO,
                        null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.MEDIAPLAY_AUDIO_IN_DEVICE_INIT_FAILED:
                CustomLog.i(TAG, "音频采集设备初始化失败");
                handleNotifyListenerListOnNotify(
                        NotifyType.AUDIO_IN_DEVICE_INIT_FAILED, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.MEDIAPLAY_AUDIO_OUT_DEVICE_INIT_FAILED:
                CustomLog.i(TAG, "音频播放设备初始化失败");
                handleNotifyListenerListOnNotify(
                        NotifyType.AUDIO_OUT_DEVICE_INIT_FAILED, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.CAMERA_OPEN_FAILED:
                CustomLog.d(TAG, "CAMERA_OPEN_FAILED");
                if (mPollAndPushControl != null && mPollFlag) {
                    if (mPollAndPushControl.getPollState()) {
                        CustomLog.d(TAG, "轮询过程中摄像头异常，停止轮询。");
                        mMeetingControl.sendPollAndPushStreamResult(1, -1,
                                mMeetingInfo.mMeetingMasterAccountId);
                        mPollAndPushControl.stopPollAndPush();
                    } else {
                        handleNotifyListenerListOnNotify(NotifyType.CAMERA_OPEN_FAILED,
                                null);
                    }
                } else {
                    handleNotifyListenerListOnNotify(NotifyType.CAMERA_OPEN_FAILED,
                            null);
                }
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.CAMERA_OPEN_SUCCESS:
                CustomLog.d(TAG, "CAMERA_OPEN_SUCCESS");
                if (mPollAndPushControl != null && mPollFlag) {
                    if (mPollAndPushControl.getPollState()) {
                        CustomLog.d(TAG, "轮询中,摄像头打开正常,通知轮询方");
                        mMeetingControl.sendPollAndPushStreamResult(1, 0,
                                mMeetingInfo.mMeetingMasterAccountId);
                    }
                }
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.START_SHARE_DOC:
                handleNotifyListenerListOnNotify(NotifyType.START_SHARE_DOC, null);
                CustomLog.i(TAG, "START_SHARE_DOC");
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.STOP_SHARE_DOC:
                handleNotifyListenerListOnNotify(NotifyType.STOP_SHARE_DOC, null);
                CustomLog.i(TAG, "STOP_SHARE_DOC");
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.SHARE_NAME_CHANGE:
                mMeetingInfo.shareDocName = (String) obj;
                CustomLog.i(TAG, "SHARE_NAME_CHANGE" + mMeetingInfo.shareDocName);
                handleNotifyListenerListOnNotify(NotifyType.SHARE_NAME_CHANGE, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.VIDEO_PARAM_UPDATE:
                CustomLog.i(TAG, "VIDEO_PARAM_UPDATE 自己的发言分辨率更新");
                getSpeakerInfoById(mMeetingInfo.accountId).setVideoParam(
                        mSMSDKControl.getVideoParam());

                handleNotifyListenerListOnNotify(
                        NotifyType.SPEAKER_VIDEO_PARAM_UPDATE,
                        mMeetingInfo.accountId);
                askUpdateStreams(StreamInfo.RESOURCE_TYPE_VIDEO);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.VIDEO_STREAMID_UPDATE:
                int streamId = (Integer) obj;
                CustomLog.i(TAG, "VIDEO_STREAMID_UPDATE 视频 streamId 更新: "
                        + streamId);
                getSpeakerInfoById(mMeetingInfo.accountId).getStreamInfo(
                        StreamInfo.RESOURCE_TYPE_VIDEO).setStreamId(streamId);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.SIGNAL_STRENGTH_UPDATE:
                int level = (Integer) obj;
                CustomLog.i(TAG, "mSignalStrengthLevel ：" + mSignalStrengthLevel
                        + " level: " + level);
                if (mSignalStrengthLevel != level) {
                    CustomLog.i(TAG, "mSignalStrengthLevel != level 更新网络信号强度");
                    mSignalStrengthLevel = level;
                    handleNotifyListenerListOnNotify(
                            NotifyType.SIGNAL_STRENGTH_UPDATE, mSignalStrengthLevel);
                }
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.DEVICES_LOST:
                // int deviceLostStreamId = (Integer) obj;
                handleNotifyListenerListOnNotify(
                        NotifyType.AUDIO_IN_DEVICE_INIT_FAILED, null);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.CPU_OVERLOAD:
                CustomLog.i(TAG,
                        "CPU_OVERLOAD,cpuStatus:" + String.valueOf((Integer) obj));
                int cpuStatus = (Integer) obj;
                handleNotifyListenerListOnNotify(NotifyType.NOTIFY_CPU_OVERLOAD,
                        cpuStatus);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.NET_STATUS:
                CustomLog.i(TAG,
                        "NET_STATUS,netStatus:" + String.valueOf((Integer) obj));
                int netStatus = (Integer) obj;
                handleNotifyListenerListOnNotify(NotifyType.NOTIFY_NET_STATUS,
                        netStatus);
                break;
            case SMSDKControl.MediaPlaySDKReturnCode.VIDEO_CLOSE_ADAPT:
                CustomLog.i(
                        TAG,
                        "VIDEO_CLOSE_ADAPT,videoCloseStreamId:"
                                + String.valueOf((Integer) obj));
                int videoCloseStreamId = (Integer) obj;

                List<SpeakerInfo> speakerList = mMeetingInfo.getSpeakers();

                String accountId = null;

                Cmd cmd = new Cmd();

                for (int i = 0; i < speakerList.size(); i++) {

                    SpeakerInfo speakerInfo = speakerList.get(i);

                    int videoStreamId = -1;
                    int docVideoStreamId = -1;

                    if (null != String.valueOf(speakerInfo
                            .getSreamIdByMediaType(MediaType.TYPE_VIDEO))) {
                        videoStreamId = speakerInfo
                                .getSreamIdByMediaType(MediaType.TYPE_VIDEO);
                        CustomLog.d(TAG, "VIDEO_CLOSE_ADAPT,videoStreamId:"
                                + String.valueOf(videoStreamId));
                        if (videoStreamId == videoCloseStreamId) {
                            accountId = speakerInfo.getAccountId();
                            cmd.setAccountId(accountId);
                            cmd.setMediaType(MediaType.TYPE_VIDEO);
                            CustomLog.d(
                                    TAG,
                                    "VIDEO_CLOSE_ADAPT,cmd.getAccountId():"
                                            + cmd.getAccountId()
                                            + " cmd.getMediaType():"
                                            + String.valueOf(cmd.getMediaType()));
                            return;
                        }
                    }

                    if (null != String.valueOf(speakerInfo
                            .getSreamIdByMediaType(MediaType.TYPE_DOC_VIDEO))) {
                        docVideoStreamId = speakerInfo
                                .getSreamIdByMediaType(MediaType.TYPE_DOC_VIDEO);
                        CustomLog.d(TAG, "VIDEO_CLOSE_ADAPT,docVideoStreamId:"
                                + String.valueOf(docVideoStreamId));
                        if (docVideoStreamId == videoCloseStreamId) {
                            accountId = speakerInfo.getAccountId();
                            cmd.setAccountId(accountId);
                            cmd.setMediaType(MediaType.TYPE_DOC_VIDEO);
                            CustomLog.d(
                                    TAG,
                                    "VIDEO_CLOSE_ADAPT,cmd.getAccountId():"
                                            + cmd.getAccountId()
                                            + " cmd.getMediaType():"
                                            + String.valueOf(cmd.getMediaType()));
                            return;
                        }
                    }

                }

                handleNotifyListenerListOnNotify(
                        NotifyType.NOTIFY_VIDEO_CLOSE_ADAPT, cmd);
                break;

            case SMSDKControl.MediaPlaySDKReturnCode.VOLUME_UPDATE:
                String vAccountId = (String) obj;
                // CustomLog.i(TAG, "VOLUME_UPDATE,accountId:" + vAccountId);
                handleNotifyListenerListOnNotify(NotifyType.NOTIFY_VOLUME_UPDATE,
                        vAccountId);
                break;

            case SMSDKControl.MediaPlaySDKReturnCode.VIDEO_START_RENDER:
                handleNotifyListenerListOnNotify(
                        NotifyType.NOTIFY_VIDEO_START_RENDER, obj);
                break;

            case SMSDKControl.MediaPlaySDKReturnCode.SURFACEVIEW_STATE_CHANGE:
                handleNotifyListenerListOnNotify(
                        NotifyType.NOTIFY_SURFACEVIEW_STATE_CHANGE, obj);
                break;

            default:
                CustomLog.w(TAG, "MediaPlaySDK返回非法命令");
                break;
        }
    }

    private void handleNotifyListenerListOnNotify(int notifyType, Object object) {
        // CustomLog.i(TAG,
        // "ButelOpenSDK::handleNotifyListenerListOnNotify() 通知回调。 "
        // + notifyType + " | listener size: "
        // + mNotifyListenerList.size());
        for (int i = 0; i < mNotifyListenerList.size(); i++) {
            mNotifyListenerList.get(i).onNotify(notifyType, object);
        }
    }

    private void callbackJoinMeetingOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackJoinMeetingOperation() 加入会议操作回调。 listener size: "
                        + mJoinMeetingListenerList.size() + " | status: "
                        + success);
        // 不用remove listener
        for (int i = 0; i < mJoinMeetingListenerList.size(); i++) {
            if (success) {
                mJoinMeetingListenerList.get(i).onOperationSuc(obj);
            } else {
                mJoinMeetingListenerList.get(i).onOperationFail(obj);
            }
        }
    }

    private void callbackAskForSpeakOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackAskForSpeakOperation() 请求发言操作回调。 listener size: "
                        + mAskForSpeakListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mAskForSpeakListenerList.size(); i++) {
            if (success) {
                mAskForSpeakListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForSpeakListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForSpeakListenerList.clear();
    }

    private void callbackAskForStopSpeakOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackAskForStopSpeakOperation() 申请停止发言操作回调。 listener size: "
                        + mAskForStopSpeakListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mAskForStopSpeakListenerList.size(); i++) {
            if (success) {
                mAskForStopSpeakListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForStopSpeakListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForStopSpeakListenerList.clear();
    }

    private void callbackChangeMeetingModeOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackChangeMeetingModeOperation() 改变会议模式回调。 listener size: "
                        + mMasterChangeMeetingModeListenerList.size()
                        + " | status: " + success);
        for (int i = 0; i < mMasterChangeMeetingModeListenerList.size(); i++) {
            if (success) {
                mMasterChangeMeetingModeListenerList.get(i).onOperationSuc(obj);
            } else {
                mMasterChangeMeetingModeListenerList.get(i)
                        .onOperationFail(obj);
            }
        }
        mMasterChangeMeetingModeListenerList.clear();
    }

    private void callbackMasterSetUserStartSpeakOperation(boolean success,
                                                          Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackMasterSetUserStartSpeakOperation() 主持人指定参会方发言回调。 listener size: "
                                + mMasterSetUserStartSpeakOnMicListenerList
                                .size() + " | status: " + success);
        for (int i = 0; i < mMasterSetUserStartSpeakOnMicListenerList.size(); i++) {
            if (success) {
                mMasterSetUserStartSpeakOnMicListenerList.get(i)
                        .onOperationSuc(obj);
            } else {
                mMasterSetUserStartSpeakOnMicListenerList.get(i)
                        .onOperationFail(obj);
            }
        }
        mMasterSetUserStartSpeakOnMicListenerList.clear();
    }

    private void callbackMasterSetUserStopSpeakOperation(boolean success,
                                                         Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackMasterSetUserStopSpeakOperation() 主持人指定参会方发言回调。 listener size: "
                                + mMasterSetUserStopSpeakOnMicListenerList
                                .size() + " | status: " + success);
        for (int i = 0; i < mMasterSetUserStopSpeakOnMicListenerList.size(); i++) {
            if (success) {
                mMasterSetUserStopSpeakOnMicListenerList.get(i).onOperationSuc(
                        obj);
            } else {
                mMasterSetUserStopSpeakOnMicListenerList.get(i)
                        .onOperationFail(obj);
            }
        }
        mMasterSetUserStopSpeakOnMicListenerList.clear();
    }

    private void callbackMasterSetMainSpeakerOperation(boolean success,
                                                       Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackMasterSetMainSpeakerOperation() 主持人设定主讲人操作回调。 listener size: "
                                + mMasterSetMainSpeakerListenerList.size()
                                + " | status: " + success);
        for (int i = 0; i < mMasterSetMainSpeakerListenerList.size(); i++) {
            if (success) {
                mMasterSetMainSpeakerListenerList.get(i).onOperationSuc(obj);
            } else {
                mMasterSetMainSpeakerListenerList.get(i).onOperationFail(obj);
            }
        }
        mMasterSetMainSpeakerListenerList.clear();
    }

    private void callbackMasterKickUserOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackMasterKickUserOperation() 主持人踢人操作回调。 listener size: "
                        + mMasterKickUserListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mMasterKickUserListenerList.size(); i++) {
            if (success) {
                mMasterKickUserListenerList.get(i).onOperationSuc(obj);
            } else {
                mMasterKickUserListenerList.get(i).onOperationFail(obj);
            }
        }
        mMasterKickUserListenerList.clear();
    }

    private void callbackMasterTransferMasterIdOperation(boolean success,
                                                         Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackMasterTransferMasterIdOperation() 主持人移交主持权限回应回调。 listener size: "
                                + mMasterTransferMasterIdListenerList.size()
                                + " | status: " + success);
        for (int i = 0; i < mMasterTransferMasterIdListenerList.size(); i++) {
            if (success) {
                mMasterTransferMasterIdListenerList.get(i).onOperationSuc(obj);
            } else {
                mMasterTransferMasterIdListenerList.get(i).onOperationFail(obj);
            }
        }
        mMasterTransferMasterIdListenerList.clear();
    }

    private void callbackMasterAskForEndMeetingOperation(boolean success,
                                                         Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackMasterAskForEndMeetingOperation() 主持人申请结束会议响应回调。 listener size: "
                                + mMasterAskForEndMeetingListenerList.size()
                                + " | status: " + success);
        for (int i = 0; i < mMasterAskForEndMeetingListenerList.size(); i++) {
            if (success) {
                mMasterAskForEndMeetingListenerList.get(i).onOperationSuc(obj);
            } else {
                mMasterAskForEndMeetingListenerList.get(i).onOperationFail(obj);
            }
        }
        mMasterAskForEndMeetingListenerList.clear();
    }

    private void callbackAskForRaiseHandOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackAskForRaiseHandOperation() 参会方举手操作回调。 listener size: "
                        + mUserAskForRaiseHandListenerList.size()
                        + " | status: " + success);
        for (int i = 0; i < mUserAskForRaiseHandListenerList.size(); i++) {
            if (success) {
                mUserAskForRaiseHandListenerList.get(i).onOperationSuc(obj);
            } else {
                mUserAskForRaiseHandListenerList.get(i).onOperationFail(obj);
            }
        }
        mUserAskForRaiseHandListenerList.clear();
    }

    private void callbackAskForStartLiveOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackAskForStartLiveOperation() 开启直播操作回调。 listener size: "
                        + mAskForStartLiveListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mAskForStartLiveListenerList.size(); i++) {
            if (success) {
                mAskForStartLiveListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForStartLiveListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForStartLiveListenerList.clear();
    }

    private void callbackAskForOpenCameraOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackAskForOpenCameraOperation() 打开摄像头操作回调。 listener size: "
                        + mAskForOpenCameraListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mAskForOpenCameraListenerList.size(); i++) {
            if (success) {
                mAskForOpenCameraListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForOpenCameraListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForOpenCameraListenerList.clear();
    }

    private void callbackAskForCloseCameraOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackAskForCloseCameraOperation() 关闭摄像头操作回调。 listener size: "
                        + mAskForCloseCameraListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mAskForCloseCameraListenerList.size(); i++) {
            if (success) {
                mAskForCloseCameraListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForCloseCameraListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForCloseCameraListenerList.clear();
    }

    private void callbackAskForOpenLoudspeakerOperation(boolean success,
                                                        Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackAskForOpenLoudspeakerOperation() 打开扬声器操作回调。 listener size: "
                                + mAskForOpenLoudspeakerListenerList.size()
                                + " | status: " + success);
        for (int i = 0; i < mAskForOpenLoudspeakerListenerList.size(); i++) {
            if (success) {
                mAskForOpenLoudspeakerListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForOpenLoudspeakerListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForOpenLoudspeakerListenerList.clear();
    }

    private void callbackAskForCloseLoudspeakerOperation(boolean success,
                                                         Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackAskForCloseLoudspeakerOperation() 关闭扬声器操作回调。 listener size: "
                                + mAskForCloseLoudspeakerListenerList.size()
                                + " | status: " + success);
        for (int i = 0; i < mAskForCloseLoudspeakerListenerList.size(); i++) {
            if (success) {
                mAskForCloseLoudspeakerListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForCloseLoudspeakerListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForCloseLoudspeakerListenerList.clear();
    }

    private void callbackAskForOpenMic(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackAskForOpenMic() 打开Mic操作回调。 listener size: "
                        + mAskForOpenMicPhoneListenerList.size()
                        + " | status: " + success);
        for (int i = 0; i < mAskForOpenMicPhoneListenerList.size(); i++) {
            if (success) {
                mAskForOpenMicPhoneListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForOpenMicPhoneListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForOpenMicPhoneListenerList.clear();
    }

    private void callbackAskForCloseMic(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackAskForCloseMic() 关闭Mic操作回调。 listener size: "
                        + mAskForCloseMicPhoneListenerList.size()
                        + " | status: " + success);
        for (int i = 0; i < mAskForCloseMicPhoneListenerList.size(); i++) {
            if (success) {
                mAskForCloseMicPhoneListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForCloseMicPhoneListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForCloseMicPhoneListenerList.clear();
    }

    private void callbackLockMeetingOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackLockMeetingOperation() 会议加/解锁操作回调。 listener size: "
                        + mLockMeetingListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mLockMeetingListenerList.size(); i++) {
            if (success) {
                mLockMeetingListenerList.get(i).onOperationSuc(obj);
            } else {
                mLockMeetingListenerList.get(i).onOperationFail(obj);
            }
            mLockMeetingListenerList.remove(i);
        }
        mLockMeetingListenerList.clear();
    }

    private void callbackChangeMeetingChatStateOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackChangeMeetingChatStateOperation() 改变会议聊天状态操作回调。 listener size: "
                        + mChangeMeetingChatStateListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mChangeMeetingChatStateListenerList.size(); i++) {
            if (success) {
                mChangeMeetingChatStateListenerList.get(i).onOperationSuc(obj);
            } else {
                mChangeMeetingChatStateListenerList.get(i).onOperationFail(obj);
            }
            mChangeMeetingChatStateListenerList.remove(i);
        }
        mChangeMeetingChatStateListenerList.clear();
    }

    private void callbackGiveMicOperation(boolean success, Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackGiveMicOperation() 传麦操作回调。 listener size: "
                                + mGiveMicListenerList.size() + " | status: "
                                + success);
        for (int i = 0; i < mGiveMicListenerList.size(); i++) {
            if (success) {
                mGiveMicListenerList.get(i).onOperationSuc(obj);
            } else {
                mGiveMicListenerList.get(i).onOperationFail(obj);
            }
        }
        mGiveMicListenerList.clear();
    }

    private void callbackExitMeetingOperation(boolean success, Object obj) {
        CustomLog.i(TAG,
                "ButelOpenSDK::callbackGiveMicOperation() 退出会议操作回调。 listener size: "
                        + mExitMeetingListenerList.size() + " | status: "
                        + success);
        for (int i = 0; i < mExitMeetingListenerList.size(); i++) {
            if (success) {
                mExitMeetingListenerList.get(i).onOperationSuc(obj);
            } else {
                mExitMeetingListenerList.get(i).onOperationFail(obj);
            }
        }
        mExitMeetingListenerList.clear();
    }

    private void askUpdateStreams(int resourceId) {
        CustomLog.d(TAG, "ButelOpenSDK::askUpdateStreams() 更新流信息. resourceId: "
                + resourceId);
        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);
        if (speaker == null) {
            CustomLog.d(TAG, "ButelOpenSDK::askUpdateStreams() 当前未发言，无法更新流信息！");
            return;
        }

        StreamInfo streamInfo = speaker.getStreamInfo(resourceId);
        if (streamInfo == null) {
            CustomLog.d(TAG, "ButelOpenSDK::askUpdateStreams() 没有对应流，无法更新流信息！");
            return;
        }

        mMeetingControl.askUpdateStreams(
                mToken,
                String.valueOf(streamInfo.getResourcesId()),
                streamInfo.getMediaType(),
                Cmd.createExtraInfo(speaker.getSyncId(),
                        speaker.getVideoResolution()[0],
                        speaker.getVideoResolution()[1]));
    }

    private void callbackAskForStartRecordingOperation(boolean success,
                                                       Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackAskForStartRecordingOperation() 开始录制请求回调。 listener size: "
                                + mAskForStartRecordingListenerList.size()
                                + " | status: " + success);
        for (int i = 0; i < mAskForStartRecordingListenerList.size(); i++) {
            if (success) {
                mAskForStartRecordingListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForStartRecordingListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForStartRecordingListenerList.clear();
    }

    private void callbackAskForStopRecordingOperation(boolean success,
                                                      Object obj) {
        CustomLog
                .i(TAG,
                        "ButelOpenSDK::callbackAskForStopRecordingOperation() 停止录制请求回调。 listener size: "
                                + mAskForStopRecordingListenerList.size()
                                + " | status: " + success);
        for (int i = 0; i < mAskForStopRecordingListenerList.size(); i++) {
            if (success) {
                mAskForStopRecordingListenerList.get(i).onOperationSuc(obj);
            } else {
                mAskForStopRecordingListenerList.get(i).onOperationFail(obj);
            }
        }
        mAskForStopRecordingListenerList.clear();
    }

    /**
     * 设置是否自适应
     *
     * @param adaptState ：true：打开自适应； false：关闭自适应
     * @return
     */
    public int setAdaptState(boolean adaptState) {
        int result = 0;
        result = mSMSDKControl.setAdaptState(adaptState);
        return result;
    }

    /**
     * 设置是否开启大流模式
     *
     * @param isBigStream ：true：打开； false：关闭
     * @return
     */
    public int setBigStream(boolean isBigStream) {
        CustomLog.d(TAG, "setBigStream isBigStream=" + isBigStream);
        mSMSDKControl.setBigStream(isBigStream);
        return 0;
    }

    public void setUseUVCCamera(List<DeviceFilter> filter) {
        mSMSDKControl.setUseUVCCamera(filter);
    }

    public void setRotation(int rotation) {
        CustomLog.d(TAG, "setRotation,rotation:" + String.valueOf(rotation));
        mSMSDKControl.setRotation(rotation);
    }

    public void setRotation(int rotation, boolean isFloatingView) {
        CustomLog.d(TAG, "setRotation,rotation:" + String.valueOf(rotation)
                + " isFloatingView=" + isFloatingView);
        mSMSDKControl.setRotation(rotation, isFloatingView);
    }

    public int getCurrentCameraIndex() {
        switch (mSMSDKControl.getCurrentCameraIndex()) {
            case -1:
                return NO_CAMERA;
            case 1:
                return FRONT_CAMERA;
            case 0:
                return BACK_CAMERA;
            case 2:
                return UVC_CAMERA;
            default:
                return NO_CAMERA;
        }
    }

    /**
     * 设置是否使用声音增益
     *
     * @param isAgc
     */
    public void setIsAgc(boolean isAgc) {
        mSMSDKControl.setIsAgc(isAgc);
    }

    /**
     * 获取是否在使用声音增益
     *
     * @return true：使用了声音增益 false：没有使用声音增益
     */
    public boolean getIsAgc() {
        return mSMSDKControl.getIsAgc();
    }

    /**
     * @param area 服务器所在地域
     * @param isp  运营商
     * @return 0：成功；非零表示失败
     * @brief 设置客户端地域和运营商信息 进会时调用
     */
    public int setTerminalInfo(String area, String isp) {
        if (TextUtils.isEmpty(area) || TextUtils.isEmpty(isp)) {
            CustomLog.d(TAG, "setTerminalInfo,参数为空");
            return -1;
        }
        int result = mSMSDKControl.setTerminalInfo(area, isp);
        return result;
    }

    /**
     * 设置会议模式（音频会议/视频会议）
     *
     * @param meetingShowType ：0-视频会议 1-音频会议
     * @param firstOpenCamera ： true-进会开启摄像头 false-进会关闭摄像头
     */
    public void setMeetingShowType(int meetingShowType, boolean firstOpenCamera) {
        CustomLog.d(
                TAG,
                "setMeetingShowType,mMeetingShowType:"
                        + String.valueOf(meetingShowType) + " firstOpenCamera:"
                        + String.valueOf(firstOpenCamera));

        mMeetingShowType = meetingShowType;
        mFirstOpenCamera = firstOpenCamera;

        if (mMeetingShowType == MEETING_TYPE_AUDIO) {

            mSMSDKControl.closeCamera();

            for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {
                closeRemoteVideo(mMeetingInfo.getSpeakers().get(i)
                        .getAccountId());
            }

        } else if (mFirstOpenCamera == false) {

            mSMSDKControl.closeCamera();

        }

    }

    /**
     * 设置会议模式（音频会议/视频会议）
     *
     * @param meetingShowType ：0-视频会议 1-音频会议
     * @param firstOpenCamera ： true-进会开启摄像头 false-进会关闭摄像头
     * @param openMic         : true-进会开启麦克风 false-进会关闭麦克风
     */
    public void setMeetingShowType(int meetingShowType,
                                   boolean firstOpenCamera, boolean openMic) {
        CustomLog.d(
                TAG,
                "setMeetingShowType,mMeetingShowType:"
                        + String.valueOf(meetingShowType) + " firstOpenCamera:"
                        + String.valueOf(firstOpenCamera));

        mMeetingShowType = meetingShowType;
        mFirstOpenCamera = firstOpenCamera;
        mOpenMic = openMic;

        if (mMeetingShowType == MEETING_TYPE_AUDIO) {

            mSMSDKControl.closeCamera();

            for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {
                closeRemoteVideo(mMeetingInfo.getSpeakers().get(i)
                        .getAccountId());
            }

        } else if (mFirstOpenCamera == false) {

            mSMSDKControl.closeCamera();

        }

    }

    /**
     * 会议参数设置
     *
     * @param openCamera ： true-默认开启摄像头 false-默认关闭摄像头
     * @param openMic    : true-默认开启麦克风 false-默认关闭麦克风
     */
    public void setOuterSetting(boolean openCamera, boolean openMic) {
        CustomLog.d(TAG, "setOuterSetting openCamera=" + openCamera + " openMic=" + openMic);

        mOuterCameraState = openCamera;
        mOuterMicState = openMic;
    }

    public int getMeetingShowType() {
        CustomLog.d(
                TAG,
                "getMeetingShowType,mMeetingShowType:"
                        + String.valueOf(mMeetingShowType));
        return mMeetingShowType;
    }

    public boolean getFirstOpenCamera() {
        CustomLog.d(
                TAG,
                "getFirstOpenCamera,mFirstOpenCamera:"
                        + String.valueOf(mFirstOpenCamera));
        return mFirstOpenCamera;
    }

    /**
     * 设置自己的surfaceview，音频会议或进会默认关闭摄像头时需要调用
     *
     * @param surfaceView
     */
    public void setMySurfaceView(SurfaceView surfaceView) {
        CustomLog.d(TAG, "setMySurfaceView");
        if (surfaceView == null) {
            CustomLog.d(TAG, "setMySurfaceView fail,surfaceView==null");
        }
        mSMSDKControl.setMySurfaceView(surfaceView);
    }

    /**
     * 发送分享图片，应用于陕西扶贫演示项目
     *
     * @param targetId 发送者目标视讯号
     * @param picUrl   图片url地址
     * @param bitmap   图片bitmap 内容
     * @return
     */
    public int sendSharingPicCmd(String targetId, String picUrl, Bitmap bitmap) {
        CustomLog.d(TAG, "ButelOpenSDK::sendSharingPicCmd()targetId: "
                + targetId + " | picUrl:" + picUrl);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (targetId.equals(mMeetingInfo.accountId)) {
            CustomLog
                    .d(TAG,
                            "sendSharingPicCmd() targetId equals curId, set background!");
            mSMSDKControl.setCloseCameraBackGroudBitmap(bitmap);
        } else {
            CustomLog
                    .d(TAG,
                            "sendSharingPicCmd() targetId non equals curId, notify others!");
            mMeetingControl.sendSharingPicCmd(targetId, picUrl);
        }
        return 0;
    }

    /**
     * 设置recver的转换map,可以实时调用
     *
     * @param mapStr json格式字符串
     */
    public int setConvertRecverIPMap(String mapStr) {
        CustomLog.d(TAG, "ButelOpenSDK::setConvertRecverIPMap() begin mapStr:"
                + mapStr);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        CustomLog.d(TAG, "mSMSDKControl setConvertRecverIPMap()");
        mSMSDKControl.setConvertRecverIPMap(mapStr);
        return 0;
    }

    /**
     * 设置relay的转换map,可以实时调用
     *
     * @param mapStr json格式字符串
     */
    public int setUpConvertRelayIPMap(String mapStr) {
        CustomLog.d(TAG, "ButelOpenSDK::setUpConvertRelayIPMap() begin mapStr:"
                + mapStr);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        CustomLog.d(TAG, "mSMSDKControl setUpConvertRelayIPMap()");
        mSMSDKControl.setUpConvertRelayIPMap(mapStr);
        return 0;
    }

    public int setDownConvertRelayIPMap(String mapStr) {
        CustomLog.d(TAG,
                "ButelOpenSDK::setDownConvertRelayIPMap() begin mapStr:"
                        + mapStr);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        CustomLog.d(TAG, "mSMSDKControl setDownConvertRelayIPMap()");
        mSMSDKControl.setDownConvertRelayIPMap(mapStr);
        return 0;
    }

    /**
     * 设置ip映射表 初始化成功，加入会议前调用
     *
     * @param IPMapStr json格式字符串
     */
    public int updateIPPortMap(String IPMapStr) {
        CustomLog.d(TAG, "ButelOpenSDK::updateIPPortMap() begin IPMapStr:"
                + IPMapStr);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        CustomLog.d(TAG, "mMeetingControl updateIPPortMap()");
        mMeetingControl.updateIPPortMap(IPMapStr);
        return 0;
    }

    /**
     * @param sel      draw type or operation
     * @param color    color
     * @param penwidth pen width
     * @return 0 onsuccess; other failed
     * @brief Select draw type or operation
     */
    public int operateSelect(int sel, int color, int penwidth) {
        CustomLog.d(TAG, "ButelOpenSDK::operateSelect()");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        return mGeoMetryControl.operateSelect(sel, color, penwidth);
    }

    /**
     * @return 0 on success; other failed
     * @brief stop drawing element and command
     */
    public int operateCancel() {
        CustomLog.d(TAG, "ButelOpenSDK::operateCancel()");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        return mGeoMetryControl.operateCancel();
    }

    /**
     * @param text     text to edit
     * @param color    color
     * @param fontSize font size
     * @param x        x position of text
     * @param y        y position of text
     * @return 0 on success; other failed
     * @brief edit text
     */
    public int drawText(String text, int font, int color, int fontSize, int x,
                        int y) {
        CustomLog.d(TAG, "ButelOpenSDK::drawText()");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        return mGeoMetryControl.drawText(text, font, color, fontSize, x, y);
    }

    /**
     * @param cmd command type
     * @return 0 on success; other failed
     * @brief command operation
     */
    public int drawCommand(int cmd) {
        CustomLog.d(TAG, "ButelOpenSDK::drawCommand()");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        return mGeoMetryControl.drawCommand(cmd);
    }

    /**
     * @param op     screen shot or draw image
     * @param path   for screen shot, the path to save the image; for draw image,
     *               the path of the image to be drawed
     * @param left   left position
     * @param top    top position
     * @param right  right position
     * @param bottom bottom position
     * @return 0 on success; other failed
     * @brief image operation
     */
    public int imageOperate(int op, String path, int left, int top, int right,
                            int bottom) {
        CustomLog.d(TAG, "ButelOpenSDK::imageOperate()");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        return mGeoMetryControl
                .imageOperate(op, path, left, top, right, bottom);
    }

    /**
     * @param userid local userid
     * @return 0 on success; other failed
     * @brief request to draw
     */
    public int requestDraw(int userid) {
        CustomLog.d(TAG, "ButelOpenSDK::requestDraw()");
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        return mGeoMetryControl.requestDraw(userid);
    }

    /**
     * @param 是否开启白板模式 调用init初始化前设置
     * @brief 设置白板开关
     */
    public void setWhiteBoardFlag(boolean flag) {
        CustomLog.d(TAG, "ButelOpenSDK::setWhiteBoardFlag flag=" + flag);
        mWhiteBoardFlag = flag;
    }

    /**
     * 获取分享者视讯号
     *
     * @return String 视讯号
     */
    public String getShareAccount() {
        CustomLog.d(TAG, "getShareAccount");
        List<SpeakerInfo> speakerInfoList = getSpeakers();
        for (int i = 0; i < speakerInfoList.size(); i++) {
            StreamInfo streamInfo = speakerInfoList.get(i)
                    .getStreamInfoByMediaType(MediaType.TYPE_DOC_VIDEO);
            if (streamInfo != null) {
                CustomLog.d(TAG, "getShareAccount 成功 视讯号为:"
                        + speakerInfoList.get(i).getAccountId());
                return speakerInfoList.get(i).getAccountId();
            }
        }
        return "";
    }

    /**
     * @param accountId
     * @return SpeakerInfo
     * @brief 根据accountId获取SpeakerInfo
     */
    public SpeakerInfo getSpeakerByAccountId(String accountId) {
        // CustomLog.d(TAG, "getSpeakerByAccountId,accountId:" + accountId);
        return mMeetingInfo.getSpeakerByAccountId(accountId);
    }

    /**
     * @param audioStreamId
     * @return accountId
     * @brief 根据音频流Id获取对应的视讯号
     */
    public String getAccountIdByAudioStreamId(int audioStreamId) {
        for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {
            if (mMeetingInfo.getSpeakers().get(i)
                    .getSreamIdByMediaType(MediaType.TYPE_AUDIO) == audioStreamId) {
                return mMeetingInfo.getSpeakers().get(i).getAccountId();
            }
        }
        return null;
    }

    /**
     * @param audioStreamId
     * @return accountId
     * @brief 根据视频流Id获取对应的视讯号
     */
    public String getAccountIdByVideoStreamId(int videoStreamId) {
        for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {
            if (mMeetingInfo.getSpeakers().get(i)
                    .getSreamIdByMediaType(MediaType.TYPE_VIDEO) == videoStreamId) {
                return mMeetingInfo.getSpeakers().get(i).getAccountId();
            }
        }
        return null;
    }

    /**
     * @param audioStreamId
     * @return accountId
     * @brief 根据视频流Id获取对应的视讯号
     */
    public String getAccountIdByDocVideoStreamId(int docVideoStreamId) {
        for (int i = 0; i < mMeetingInfo.getSpeakers().size(); i++) {
            if (mMeetingInfo.getSpeakers().get(i)
                    .getSreamIdByMediaType(MediaType.TYPE_DOC_VIDEO) == docVideoStreamId) {
                return mMeetingInfo.getSpeakers().get(i).getAccountId();
            }
        }
        return null;
    }

    /**
     * @param sendType tcp 1 udp 2
     * @brief 设置白板命令发送的方式
     */
    public int setWhiteBoardSendType(int sendType) {
        CustomLog.d(TAG, "ButelOpenSDK::setWhiteBoardSendType() sendType="
                + sendType);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        return mGeoMetryControl.setWhiteBoardSendType(sendType);
    }

    /**
     * @param type 0表示软解，非0表示硬解
     * @brief 设置编码方式
     * @note 调用时序，创建对象后，初始化前
     */
    public void setCodecType(int type) {
        CustomLog.d(TAG, "ButelOpenSDK::setCodecType() type=" + type);
        mCodecType = type;
    }

    /**
     * @param bigImage 大背景 smallImage 小背景
     * @brief 设置编图片加载背景图
     * @note 调用时序，创建对象后，初始化前
     */
    public void setVideoLoadingImage(int bigImage, int smallImage) {
        CustomLog.d(TAG, "ButelOpenSDK::setVideoLoadingImage() bigImage="
                + bigImage + " smallImage=" + smallImage);
        mBigImage = bigImage;
        mSmallImage = smallImage;
    }

    public int getShareType(String accountId) {
        SpeakerInfo speaker = mMeetingInfo.getSpeakerByAccountId(accountId);
        if (speaker == null) {
            CustomLog.w(TAG, "该视讯号没有发言，无法获取共享类型");
            return -2;
        }

        StreamInfo streamInfo = speaker
                .getStreamInfoByMediaType(MediaType.TYPE_DOC_VIDEO);

        if (streamInfo != null) {
            int shareType = streamInfo.getShareType();
            return shareType;
        } else {
            return -1;
        }
    }

    /**
     * @param isAdmin 是否是超级管机员
     * @brief设置超级管理员
     * @note 调用时序，加入会议之前调用
     */
    public void setAdminFlag(boolean isAdmin) {
        mAdminFlag = isAdmin;
    }

    /**
     * @brief 收到主持人指定发言命令，申请发布流 UI主动发布流
     */
    public void publishStreamsByMaster() {
        SpeakerInfo speaker = getSpeakerInfoById(mMeetingInfo.accountId);

        if (speaker == null) {
            CustomLog.e(TAG, "ButelOpenSDK::publishStremsInfo() 自己未发言");
        }

        speaker.setVideoParam(mSMSDKControl.getVideoParam());

        // 向 SM_SDK 发布视频流、音频流信息
        int syncId = speaker.getSyncId();

        if (mMeetingShowType == MEETING_TYPE_AUDIO) {
            CustomLog
                    .d(TAG,
                            "publishStremsInfo,mMeetingShowType==MEETING_TYPE_AUDIO,不发布视频流");
            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_CLOSE);
        } else {
            CustomLog
                    .d(TAG,
                            "mMeetingShowType==MEETING_TYPE_VIDEO,mFirstOpenCamera==true,发布视频流");
            StreamInfo videoStream = new StreamInfo(-1,
                    StreamInfo.RESOURCE_TYPE_VIDEO, syncId,
                    MediaType.TYPE_VIDEO);
            int videoStreamId = mSMSDKControl.publishCameraVideo(
                    StreamInfo.RESOURCE_TYPE_VIDEO, syncId);

            videoStream.setStreamId(videoStreamId);
            CustomLog.i(TAG, "发布视频流  streamId: " + videoStreamId);

            speaker.setCamStatus(SpeakerInfo.CAM_STATUS_OPEN);
            speaker.addStreamInfo(StreamInfo.RESOURCE_TYPE_VIDEO, videoStream);
        }

        int audioStreamId = 0;

        StreamInfo audioStream = new StreamInfo(-1,
                StreamInfo.RESOURCE_TYPE_AUDIO, syncId, MediaType.TYPE_AUDIO);
        audioStreamId = mSMSDKControl.publishMicAudio(
                StreamInfo.RESOURCE_TYPE_AUDIO, syncId);
        if (audioStreamId < 0) {
            CustomLog.e(TAG, "发布音频流失败  streamId： " + audioStreamId);
        } else {
            audioStream.setStreamId(audioStreamId);
            CustomLog.i(TAG, "发布音频流  streamId: " + audioStreamId);
            speaker.setMICStatus(SpeakerInfo.MIC_STATUS_ON);
            speaker.addStreamInfo(StreamInfo.RESOURCE_TYPE_AUDIO, audioStream);
        }

        JSONArray jsonArray = new JSONArray();
        JSONObject videoStreamJson = new JSONObject();
        JSONObject audeoStreamJson = new JSONObject();
        try {
            if (mMeetingShowType == MEETING_TYPE_AUDIO) {
                CustomLog
                        .d(TAG,
                                "mMeetingShowType==MEETING_TYPE_AUDIO,不添加videoStreamJson");
            } else {
                CustomLog
                        .d(TAG,
                                "mMeetingShowType==MEETING_MODE_VIDEO，添加videoStreamJson");
                videoStreamJson.put(CmdKey.RESOURCE_ID,
                        String.valueOf(StreamInfo.RESOURCE_TYPE_VIDEO));
                videoStreamJson
                        .put(CmdKey.MEDIA_TYPE, SpeakerInfo
                                .ParseSmMediaTypeToStp(MediaType.TYPE_VIDEO));
                videoStreamJson.put(
                        CmdKey.EXTRAINFO,
                        Cmd.createExtraInfo(syncId,
                                speaker.getVideoResolution()[0],
                                speaker.getVideoResolution()[1]));

                jsonArray.put(videoStreamJson);
            }

            if (audioStreamId > 0) {
                audeoStreamJson.put(CmdKey.RESOURCE_ID,
                        String.valueOf(StreamInfo.RESOURCE_TYPE_AUDIO));
                audeoStreamJson
                        .put(CmdKey.MEDIA_TYPE, SpeakerInfo
                                .ParseSmMediaTypeToStp(MediaType.TYPE_AUDIO));
                audeoStreamJson.put(CmdKey.EXTRAINFO,
                        Cmd.createExtraInfo(syncId, -1, -1));

                jsonArray.put(audeoStreamJson);
            }

            CustomLog.d(TAG, "jsonArray=" + jsonArray.toString());

            if (jsonArray.length() > 0) {
                int result = mMeetingControl.askPublishOrUnPublishStreams(
                        mToken, Cmd.STREAM_OPERATOR_TYPE_PUBLISH, jsonArray);
                if (result != 0) {
                    CustomLog.w(TAG, "向 STP 发布流失败 result: " + result);
                } else {
                    mMeetingControlOperation = ButelOpenSDKMeetingControlOperationCode.PUBLISH_OR_UNPUBLISH_STREAMS;
                }
            } else {
                CustomLog.d(TAG, "没有流信息，不进行发布操作。");
            }
        } catch (JSONException e) {
            CustomLog.e(TAG, "json 构造出错 : " + e.getMessage());
        }

        if (audioStreamId == SMSDKControl.DEVICE_AUDIO_INIT_FAILED) {
            CustomLog.d(TAG, "ButelOpenSDK::publishStremsInfo() 通知UI音频采集失败");
            handleNotifyListenerListOnNotify(
                    NotifyType.AUDIO_IN_DEVICE_INIT_FAILED, null);
        }
    }

    /**
     * 开启数据共享
     *
     * @param operateAccountId 被操作者视讯号
     */
    public int startDataShare(String operateAccountId) {
        CustomLog.d(TAG, "startDataShare operateAccountId=" + operateAccountId);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (operateAccountId == null || operateAccountId.equals("")) {
            return -1;
        }

        if (operateAccountId.equals(mMeetingInfo.accountId)) {
            CustomLog.d(TAG, "startDataShare 被操作者是手机本身，不支持!");
            DataShareResult dataShareResult = new DataShareResult();
            dataShareResult.operateAccountId = operateAccountId;
            dataShareResult.operateAccountName = mMeetingInfo.accountName;
            dataShareResult.operateResult = 1;
            handleNotifyListenerListOnNotify(
                    NotifyType.NOTIFY_DATA_SHARE_RESPONSE, dataShareResult);
        }

        String msgData = "2#" + mMeetingInfo.accountId + "#"
                + mMeetingInfo.accountName;
        int ret = mMeetingControl.dataShareOperate(operateAccountId, msgData);
        return ret;
    }

    /**
     * 关闭数据共享
     *
     * @param operateAccountId 被操作者视讯号
     */
    public int stopDataShare(String operateAccountId) {
        CustomLog.d(TAG, "stopDataShare operateAccountId=" + operateAccountId);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        if (operateAccountId == null || operateAccountId.equals("")) {
            return -1;
        }

        if (operateAccountId.equals(mMeetingInfo.accountId)) {
            CustomLog.d(TAG, "stopDataShare 被操作者是手机本身，不支持!");
            DataShareResult dataShareResult = new DataShareResult();
            dataShareResult.operateAccountId = operateAccountId;
            dataShareResult.operateAccountName = mMeetingInfo.accountName;
            dataShareResult.operateResult = 1;
            handleNotifyListenerListOnNotify(
                    NotifyType.NOTIFY_DATA_SHARE_RESPONSE, dataShareResult);
        }

        String msgData = "9#" + mMeetingInfo.accountId + "#"
                + mMeetingInfo.accountName;
        int ret = mMeetingControl.dataShareOperate(operateAccountId, msgData);
        return ret;
    }

    /**
     * @return int
     * @brief 获取会议最大发言方数
     * @note 加会成功后调用
     */
    public int getMeetingMaxSpeakerNum() {
        CustomLog.d(TAG, "ButelOpenSDK::getMeetingMaxSpeakerNum() num="
                + mMeetingInfo.speakersNum);
        if (mMeetingInfo != null) {
            return mMeetingInfo.speakersNum;
        }
        CustomLog.d(TAG,
                "ButelOpenSDK::getMeetingMaxSpeakerNum() 加会成功前调用，取默认值!");
        return 4;
    }

    private int getCacheMeetingCaption() {
        CustomLog.d(TAG, "ButelOpenSDK::getCacheMeetingCaption()");

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        mMeetingControl.getServerBlackCacheCmd("meet_caption", mToken);
        return 0;
    }

    private int getCacheMeetingTheme() {
        CustomLog.d(TAG, "ButelOpenSDK::getCacheMeetingTheme()");

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }
        mMeetingControl.getServerBlackCacheCmd("meet_theme", mToken);
        return 0;
    }

    /**
     * 主持人改变会议文字聊天状态，只有主持人有权限操作：1代表开启，0代表关闭
     *
     * @param changeMeetingChatStateListener 操作结果回调listener
     * @param state                          1代表开启，0代表关闭
     * @return <p>
     * 0: 操作成功，操作结果将通过 changeMeetingChatStateListener 异步返回
     * <p>
     * <0: 操作失败
     */
    public int masterOpenOrCloseMessage(ButelOpenSDKOperationListener changeMeetingChatStateListener,
                                        int state) {
        CustomLog
                .i(TAG, "ButelOpenSDK::masterChangeMeetingIMState() meetingIMState: "
                        + mMeetingInfo.meetingChatState + " state: "
                        + state);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (mMeetingInfo.mCurrentRole != CurrentRoleStatus.MASTER_ROLE
                && !mAdminFlag) {
            CustomLog.w(TAG, "不是主持人，不具有此权限");
            return ButelOpenSDKReturnCode.RETURN_NOAUTHORITY_ERROR;
        }

        if (state == mMeetingInfo.meetingChatState) {
            CustomLog.w(TAG, "当前已经是打开或者关闭聊天状态，state=" + state
                    + "mCurMeetingInfo.meetingIMState="
                    + mMeetingInfo.meetingChatState);
            return ButelOpenSDKReturnCode.RETURN_REDOING_ERROR;
        }

        int returnCode = mMeetingControl.masterChangeMeetingChatState(mToken, state);
        if (returnCode != MeetingControlReturnCode.SUCCESS) {
            CustomLog.w(TAG, "打开或者关闭聊天状态失败， result: " + returnCode);
            return ButelOpenSDKReturnCode.RETURN_OTHER_ERROR;
        }

        mChangeMeetingChatStateListenerList.add(changeMeetingChatStateListener);
        mMeetingInfo.meetingChatState = state; // 先将目标状态置为期待状态，等待操作结果回来再更新为真实状态
        CustomLog.i(TAG, "发送打开或者关闭聊天状态命令结束" + "state=" + state);
        return ButelOpenSDKReturnCode.RETURN_NEED_CALLBCAK_SUCCESS;
    }

    /**
     * 获取会议文字聊天状态
     *
     * @return 1代表开启 0代表关闭
     */
    public int getMeetingChatState() {
        CustomLog.d(TAG, "getMeetingChatState state=" + mMeetingInfo.meetingChatState);
        return mMeetingInfo.meetingChatState;
    }

    /**
     * 会议内发消息
     *
     * @param chatType   聊天类型
     *                   ChatType.PUBLIC_MSG 公众消息
     *                   ChatType.PRIVATE_MSG 私聊消息
     * @param content    消息内容
     * @param receiver   私聊消息接收者视讯号
     * @param atUserList 公众消息@者视讯号
     * @return
     */
    public int sendMeetingChatMessage(int chatType, String content, String receiver, List<String> atUserList) {
        CustomLog.d(TAG, "sendMeetingChatMessage chatType=" + chatType + " content=" + content);
        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return ButelOpenSDKReturnCode.RETURN_NOT_INIT_ERROR;
        }

        if (chatType == ChatType.PUBLIC_CHAT) {
            return mMeetingControl.sendMeetingPublicChatMessage(createMeetingChatMsg(chatType, content, receiver, atUserList));
        } else if (chatType == ChatType.PRIVATE_CHAT) {
            if (receiver != null && receiver.length() > 0) {
                mMeetingControl.sendMeetingPrivateChatMessage(createMeetingChatMsg(chatType, content, receiver, atUserList), receiver);
                return 0;
            }
        }
        return -1;
    }

    private String createMeetingChatMsg(int chatType, String content, String receiver, List<String> atUserList) {
        String atIds = "";
        if (atUserList != null) {
            for (int i = 0; i < atUserList.size(); i++) {
                if (i == atUserList.size() - 1) {
                    atIds = atIds + atUserList.get(i);
                } else {
                    atIds = atIds + atUserList.get(i) + ",";
                }
            }
        }
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("IMType", 0);
            jsonObject.put("atIds", atIds);
            jsonObject.put("id", mMeetingInfo.accountId);
            jsonObject.put("msg", content);
            jsonObject.put("name", mMeetingInfo.accountName);
            jsonObject.put("t", chatType);
        } catch (Exception e) {
            CustomLog.e(TAG, "createMeetingChatMsg error e=" + e.toString());
            return "";
        }
        CustomLog.d(TAG, "createMeetingChatMsg msg=" + jsonObject.toString());
        return jsonObject.toString();// + "\n" + "\0";
    }

    public void startRenderPollVideo(SurfaceView surfaceView) {
        CustomLog.d(TAG, "ButelOpenSDK::startRenderPollVideo() surfaceView="
                + surfaceView);

        if (mInitStatus != InitStatus.INITED) {
            CustomLog.e(TAG, "error ! 未进行初始化 initStatus: " + mInitStatus);
            return;
        }

        if (!mPollFlag) {
            CustomLog.e(TAG, "error ! 未打开轮询，不处理！");
            return;
        }

        mSMSDKControl.startRenderPollVideo(surfaceView);
    }

    public void stopRenderPollVideo() {
        if (mPollAndPushControl != null && mPollFlag) {
            CustomLog.d(TAG, "ButelOpenSDK::stopRenderPollVideo()");
            if (mPollAndPushControl.getPollState()) {
                mPollAndPushControl.stopPollAndPush();
                mSMSDKControl.stopRenderPollVideo();
            }
        }
    }

    /**
     * 设置为收大流
     *
     * @param list ：设置为大流的视讯号list
     */
    public void setBigStream(List<String> list) {
        mSMSDKControl.setBigStream(list);
    }

    /**
     * @param pollFlag true 打开轮询 false 关闭轮询
     * @brief 设置轮询开关 调用init初始化前设置
     */
    public void setPollFlag(boolean pollFlag) {
        CustomLog.d(TAG, "setPollFlag pollFlag=" + pollFlag);
        mPollFlag = pollFlag;
    }

    /**
     * @param state true 打开主题字幕 false 屏蔽主题字幕
     * @brief 主题字幕开关
     */
    public void setCaptionAndThemeState(boolean state) {
        CustomLog.d(TAG, "setCaptionAndThemeState state=" + state);
        mCaptionAndThemeState = state;
    }
}